package ag.ion.bion.officelayer.internal.draw;

import java.util.ArrayList;
import java.util.List;

import com.sun.star.container.XIndexAccess;
import com.sun.star.container.XNameContainer;
import com.sun.star.drawing.XDrawPage;
import com.sun.star.drawing.XShape;
import com.sun.star.lang.IndexOutOfBoundsException;
import com.sun.star.lang.WrappedTargetException;
import com.sun.star.lang.XComponent;
import com.sun.star.lang.XMultiServiceFactory;
import com.sun.star.uno.Any;
import com.sun.star.uno.AnyConverter;
import com.sun.star.uno.Exception;
import com.sun.star.uno.UnoRuntime;

import ag.ion.bion.officelayer.document.IDocument;
import ag.ion.bion.officelayer.draw.IDrawPage;
import ag.ion.bion.officelayer.draw.shapes.IShape;
import ag.ion.bion.officelayer.draw.shapes.IShapeFactory;
import ag.ion.bion.officelayer.internal.draw.shapes.ShapeFactory;

/**
 * @author Sebastian Patschorke
 */
public class DrawPage implements IDrawPage {

	private IShapeFactory shapeFactory = null;
	private XDrawPage xDrawPage = null;
	private IDocument document = null;
	private XMultiServiceFactory xFactory = null;

	public DrawPage(IDocument document, XDrawPage drawPage) {
		super();
		this.xDrawPage = drawPage;
		this.document = document;
		this.xFactory = getXMultiServiceFactory();
	}

	@Override
	public IShapeFactory getShapeFactory() {
		if (shapeFactory == null)
			shapeFactory = new ShapeFactory(this);
		return shapeFactory;
	}

	public boolean addShape(IShape shape) {
		if (xDrawPage == null)
			return false;
		xDrawPage.add(shape.getXShape());
		return true;
	}

	public boolean removeShape(IShape shape) {
		if (xDrawPage == null)
			return false;
		xDrawPage.remove(shape.getXShape());
		return true;
	}

	@Override
	public List<IShape> getShapes() {
		List<IShape> shapes = new ArrayList<IShape>();

		XIndexAccess indexAccess = (XIndexAccess) UnoRuntime.queryInterface(
				XIndexAccess.class, xDrawPage);

		for (int i = 0; i <= indexAccess.getCount(); i++) {
			Any shape = null;
			try {
				shape = (Any) indexAccess.getByIndex(i);
			} catch (IndexOutOfBoundsException e) {
				continue;
			} catch (WrappedTargetException e) {
				continue;
			}
			String type = shape.getType().getTypeName();
			if (!type.equals(XShape.class.getName()))
				continue;
			XShape xShape = (XShape) UnoRuntime.queryInterface(XShape.class,
					shape);

			IShape s = ((ShapeFactory) getShapeFactory())
					.constructShape(xShape);
			if (s == null)
				continue;
			shapes.add(s);

		}

		return shapes;
	}

	@Override
	public XDrawPage getXDrawPage() {
		return xDrawPage;
	}

	public String insertImage(String url) {
		String internalURL = null;
		try {
			Object xObj = xFactory
					.createInstance("com.sun.star.drawing.BitmapTable"); //$NON-NLS-1$
			XNameContainer xBitmapContainer = doCast(XNameContainer.class, xObj);

			String tempId = "tempImageId" + System.currentTimeMillis(); //$NON-NLS-1$
			xBitmapContainer.insertByName(tempId, url);
			internalURL = AnyConverter.toString(xBitmapContainer
					.getByName(tempId));
		} catch (Exception e) {
			// do nothing
		}
		return internalURL;
	}

	public XShape createShape(String sShapeType) {
		try {
			// get the given Shape service from the factory
			Object xObj = xFactory.createInstance(sShapeType);
			XShape xShape = doCast(XShape.class, xObj);

			return xShape;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	private XMultiServiceFactory getXMultiServiceFactory() {
		XComponent xComponent = document.getXComponent();
		XMultiServiceFactory xMultiServiceFactory = (XMultiServiceFactory) UnoRuntime
				.queryInterface(XMultiServiceFactory.class, xComponent);
		return xMultiServiceFactory;

	}

	@SuppressWarnings("unchecked")
	private <T> T doCast(Class<T> castTo, Object o) {
		T casted = (T) UnoRuntime.queryInterface(castTo, o);
		return casted;
	}

}
