package ag.ion.bion.officelayer.internal.draw.shapes;

import ag.ion.bion.officelayer.draw.shapes.IShape;
import ag.ion.bion.officelayer.draw.shapes.properties.IFillProperties;
import ag.ion.bion.officelayer.draw.shapes.properties.IFillable;
import ag.ion.bion.officelayer.draw.shapes.properties.ILineChangable;
import ag.ion.bion.officelayer.draw.shapes.properties.ILineProperties;
import ag.ion.bion.officelayer.draw.shapes.properties.IRotatable;
import ag.ion.bion.officelayer.draw.shapes.properties.IRotationProperties;
import ag.ion.bion.officelayer.draw.shapes.properties.IShadowProperties;
import ag.ion.bion.officelayer.draw.shapes.properties.IShadowSetable;
import ag.ion.bion.officelayer.draw.shapes.properties.IShapeText;
import ag.ion.bion.officelayer.draw.shapes.properties.ITextSetable;
import ag.ion.bion.officelayer.internal.draw.DrawPage;
import ag.ion.bion.officelayer.internal.draw.shapes.properties.FillProperties;
import ag.ion.bion.officelayer.internal.draw.shapes.properties.LineProperties;
import ag.ion.bion.officelayer.internal.draw.shapes.properties.RotationProperties;
import ag.ion.bion.officelayer.internal.draw.shapes.properties.ShadowProperties;
import ag.ion.bion.officelayer.internal.draw.shapes.properties.text.ShapeText;

import com.sun.star.awt.Point;
import com.sun.star.awt.Size;
import com.sun.star.beans.PropertyVetoException;
import com.sun.star.beans.UnknownPropertyException;
import com.sun.star.beans.XPropertySet;
import com.sun.star.drawing.XShape;
import com.sun.star.lang.IllegalArgumentException;
import com.sun.star.lang.WrappedTargetException;
import com.sun.star.text.XText;
import com.sun.star.uno.UnoRuntime;

/**
 * @author Sebastian Patschorke
 */
public abstract class Shape implements IShape, IFillable, IShadowSetable,
		IRotatable, ITextSetable, ILineChangable {

	private XShape xShape;
	private XPropertySet xPropertySet;
	private DrawPage drawPage;
	private ShapeText text = null;

	private IFillProperties fillProperties = null;
	private ILineProperties lineProperties = null;
	private IShadowProperties shadowProperties = null;
	private IRotationProperties rotationProperties = null;

	boolean added = false;

	protected Shape(XShape shape, DrawPage drawPage) {
		super();
		xShape = shape;
		this.drawPage = drawPage;
	}

	@Override
	public boolean addToPage() {
		if (added)
			return false;
		boolean result = drawPage.addShape(this);
		if (result) {
			update();
			added = true;
		}
		return result;
	}

	@Override
	public boolean removeFromPage() {
		if (!added)
			return false;
		boolean result = drawPage.removeShape(this);
		if (result)
			added = false;
		return result;
	}

	public boolean isAdded() {
		return added;
	}

	protected void setAdded(boolean added) {
		this.added = added;
	}

	public XShape getXShape() {
		return xShape;
	}

	public void setSize(Size size) {
		if (size == null)
			return;
		try {
			xShape.setSize(size);
		} catch (PropertyVetoException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void setPosition(Point position) {
		if (position == null)
			return;
		xShape.setPosition(position);
	}

	@Override
	public IFillProperties getFillProperties() {
		if (fillProperties == null)
			fillProperties = new FillProperties(this);
		return fillProperties;
	}

	@Override
	public ILineProperties getLineProperties() {
		if (lineProperties == null)
			lineProperties = new LineProperties(this);
		return lineProperties;
	}

	@Override
	public IShadowProperties getShadowProperties() {
		if (shadowProperties == null)
			shadowProperties = new ShadowProperties(this);
		return shadowProperties;
	}

	@Override
	public IRotationProperties getRotationProperties() {
		if (rotationProperties == null)
			rotationProperties = new RotationProperties(this);
		return rotationProperties;
	}

	public IShapeText getText() {
		if (text == null) {
			XText xText = (XText) UnoRuntime
					.queryInterface(XText.class, xShape);
			text = new ShapeText(xText);
		}
		return text;
	}

	@Override
	public void update() {
		if (text != null)
			text.update();
	}

	public void setProperty(String property, Object value) {
		if (xPropertySet == null)
			xPropertySet = (XPropertySet) UnoRuntime.queryInterface(
					XPropertySet.class, xShape);

		try {
			xPropertySet.setPropertyValue(property, value);
		} catch (UnknownPropertyException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (PropertyVetoException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (WrappedTargetException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	@SuppressWarnings("unchecked")
	protected <T> T doCast(Class<T> castTo, Object o) {
		T casted = (T) UnoRuntime.queryInterface(castTo, o);
		return casted;
	}
}
