package ag.ion.bion.officelayer.internal.draw.shapes;

import ag.ion.bion.officelayer.draw.shapes.IClosedBezierShape;
import ag.ion.bion.officelayer.draw.shapes.IConnector;
import ag.ion.bion.officelayer.draw.shapes.IEllipse;
import ag.ion.bion.officelayer.draw.shapes.IGraphicObjectShape;
import ag.ion.bion.officelayer.draw.shapes.IGroupShape;
import ag.ion.bion.officelayer.draw.shapes.ILine;
import ag.ion.bion.officelayer.draw.shapes.IRectangle;
import ag.ion.bion.officelayer.draw.shapes.IShape;
import ag.ion.bion.officelayer.draw.shapes.IShapeFactory;
import ag.ion.bion.officelayer.draw.shapes.ITextShape;
import ag.ion.bion.officelayer.internal.draw.DrawPage;
import ag.ion.noa.graphic.GraphicInfo;

import com.sun.star.awt.Size;
import com.sun.star.drawing.XShape;

/**
 * @author Sebastian Patschorke
 */
public class ShapeFactory implements IShapeFactory {

	private DrawPage drawPage;

	public ShapeFactory(DrawPage drawPage) {
		if (drawPage == null)
			// TODO externalize
			throw new IllegalArgumentException(
					"The submitted draw page is not valid."); //$NON-NLS-1$
		this.drawPage = drawPage;
	}

	@Override
	public IRectangle createRectangle(int width, int height) {
		IRectangle rectangle = new Rectangle(drawPage
				.createShape(IShape.RECTANGLE), drawPage);
		rectangle.setSize(new Size(width, height));
		return rectangle;
	}

	@Override
	public ITextShape createTextShape(int width, int height, String text) {
		ITextShape textShape = new TextShape(drawPage
				.createShape(IShape.TEXT_SHAPE), drawPage);
		textShape.setSize(new Size(width, height));
		textShape.getText().setText(text);
		return textShape;
	}

	@Override
	public IGraphicObjectShape createGraphicObjectShape(GraphicInfo graphicInfo) {
		IGraphicObjectShape graphicObjectShape = new GraphicObjectShape(
				drawPage.createShape(IShape.GRAPHIC_OBJECT_SHAPE), drawPage);

		String internalURL = drawPage.insertImage(graphicInfo.getUrl());
		if (internalURL == null)
			return null;

		graphicObjectShape.setGraphicURL(internalURL);
		Size size = new Size();
		size.Height = graphicInfo.getHeight();
		size.Width = graphicInfo.getWidth();
		graphicObjectShape.setSize(size);

		return graphicObjectShape;
	}

	@Override
	public IGroupShape createGroup() {
		IGroupShape group = new GroupShape(drawPage
				.createShape(IShape.GROUP_SHAPE), drawPage);
		return group;
	}

	@Override
	public IClosedBezierShape createClosedBezierShape() {
		IClosedBezierShape closedBezierShape = new ClosedBezierShape(drawPage
				.createShape(IShape.CLOSED_BEZIER_SHAPE), drawPage);
		return closedBezierShape;
	}

	@Override
	public IConnector createConnector() {
		IConnector connector = new Connector(drawPage
				.createShape(IShape.CONNECTOR), drawPage);
		return connector;
	}

	@Override
	public IEllipse createEllipse(int width, int height) {
		IEllipse ellipse = new Ellipse(drawPage.createShape(IShape.ELLIPSE),
				drawPage);
		ellipse.setSize(new Size(width, height));
		return ellipse;
	}

	@Override
	public ILine createLine(int width, int height) {
		ILine line = new Line(drawPage.createShape(IShape.LINE), drawPage);
		line.setSize(new Size(width, height));
		return line;
	}

	public IShape constructShape(XShape xShape) {
		String type = xShape.getShapeType();
		Shape shape = null;

		if (type.equals(IShape.CLOSED_BEZIER_SHAPE))
			shape = new ClosedBezierShape(xShape, drawPage);

		else if (type.equals(IShape.CONNECTOR))
			shape = new Connector(xShape, drawPage);

		else if (type.equals(IShape.ELLIPSE))
			shape = new Ellipse(xShape, drawPage);

		else if (type.equals(IShape.GRAPHIC_OBJECT_SHAPE))
			shape = new GraphicObjectShape(xShape, drawPage);

		else if (type.equals(IShape.GROUP_SHAPE))
			shape = new GroupShape(xShape, drawPage);

		else if (type.equals(IShape.LINE))
			shape = new Line(xShape, drawPage);

		else if (type.equals(IShape.RECTANGLE))
			shape = new Rectangle(xShape, drawPage);

		else if (type.equals(IShape.TEXT_SHAPE))
			shape = new TextShape(xShape, drawPage);

		else
			// TODO what do with an 'exotic' shape?
			return null;

		shape.setAdded(true);
		return shape;
	}
}
