package ag.ion.bion.officelayer.internal.draw.shapes.properties;

import java.awt.Color;

import ag.ion.bion.officelayer.draw.shapes.IShape;
import ag.ion.bion.officelayer.draw.shapes.data.Arrow;
import ag.ion.bion.officelayer.draw.shapes.properties.ILineProperties;

import com.sun.star.drawing.PolyPolygonBezierCoords;

/**
 * @author Sebastian Patschorke
 */
public class LineProperties implements ILineProperties {

	private IShape shape;

	public LineProperties(IShape shape) {
		super();
		this.shape = shape;
	}

	@Override
	public void setLineColor(Color color) {
		if (color == null)
			return;
		shape.setProperty(ILineProperties.COLOR, convertColor(color));
	}

	@Override
	public void setLineWidth(long width) {
		shape.setProperty(ILineProperties.WIDTH, new Long(width));
	}

	@Override
	public void setStartArrow(String name) {
		if (name == null)
			return;
		shape.setProperty(ILineProperties.START_NAME, name);
	}

	@Override
	public void setStartArrow(Arrow arrow) {
		if (arrow == null)
			return;
		PolyPolygonBezierCoords coords = arrow.getPolyPolygon().getCoords();
		if (coords == null)
			return;
		shape.setProperty(ILineProperties.START, coords);
	}

	@Override
	public void setEndArrow(String name) {
		if (name == null)
			return;
		shape.setProperty(ILineProperties.END_NAME, name);
	}

	@Override
	public void setEndArrow(Arrow arrow) {
		if (arrow == null)
			return;
		PolyPolygonBezierCoords coords = arrow.getPolyPolygon().getCoords();
		if (coords == null)
			return;
		shape.setProperty(ILineProperties.END, coords);
	}

	@Override
	public void setStartArrowWidth(int width) {
		shape.setProperty(ILineProperties.START_WIDTH, new Integer(width));
	}

	@Override
	public void setEndArrowWidth(int width) {
		shape.setProperty(ILineProperties.END_WIDTH, new Integer(width));
	}

	private Integer convertColor(Color color) {
		return new Integer(color.getRGB() & 0xFFFFFF);
	}
}
