/****************************************************************************
 * ubion.ORS - The Open Report Suite                                        *
 *                                                                          *
 * ------------------------------------------------------------------------ *
 *                                                                          *
 * Subproject: Office Editor UI                                             *
 *                                                                          *
 *                                                                          *
 * The Contents of this file are made available subject to                  *
 * the terms of GNU Lesser General Public License Version 2.1.              *
 *                                                                          * 
 * GNU Lesser General Public License Version 2.1                            *
 * ======================================================================== *
 * Copyright 2003-2005 by IOn AG                                            *
 *                                                                          *
 * This library is free software; you can redistribute it and/or            *
 * modify it under the terms of the GNU Lesser General Public               *
 * License version 2.1, as published by the Free Software Foundation.       *
 *                                                                          *
 * This library is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        *
 * Lesser General Public License for more details.                          *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public         *
 * License along with this library; if not, write to the Free Software      *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,                    *
 * MA  02111-1307  USA                                                      *
 *                                                                          *
 * Contact us:                                                              *
 *  http://www.ion.ag                                                       *
 *  info@ion.ag                                                             *
 *                                                                          *
 ****************************************************************************/
 
/*
 * Last changes made by $Author: markus $, $Date: 2006-09-11 15:38:29 +0200 (Mo, 11 Sep 2006) $
 */
package ag.ion.bion.workbench.office.editor.ui;

import ag.ion.bion.workbench.office.editor.core.EditorCorePlugin;

import ag.ion.bion.officelayer.application.IOfficeApplication;
import ag.ion.bion.officelayer.application.OfficeApplicationException;

import ag.ion.noa4e.ui.NOAUIPlugin;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.preference.IPreferenceStore;

import org.eclipse.swt.widgets.Shell;

import org.eclipse.ui.forms.widgets.FormToolkit;

import org.eclipse.ui.plugin.AbstractUIPlugin;

import org.osgi.framework.BundleContext;

import java.util.ResourceBundle;
import java.util.MissingResourceException;

/**
 * The main plugin class to be used in the desktop.
 * 
 * @author Andreas Brker
 * @author Markus Krger
 * @version $Revision: 9817 $
 */
public class EditorUIPlugin extends AbstractUIPlugin {
  
  /** ID of the plugin. */
  public static final String PLUGIN_ID = "ag.ion.bion.workbench.office.editor.ui"; //$NON-NLS-1$
  
  //The shared instance.
  private static EditorUIPlugin plugin;
  //Resource bundle.
  private ResourceBundle resourceBundle;
	
  //----------------------------------------------------------------------------
  /**
   * The constructor.
   * 
   * @author Andreas Br�ker
   */
  public EditorUIPlugin() {
		super();
		plugin = this;
		try {
		  resourceBundle = ResourceBundle.getBundle("ag.ion.bion.workbench.office.editor.ui.UiPluginResources"); //$NON-NLS-1$
		} 
	  catch (MissingResourceException missingResourceException) {
	      resourceBundle = null;
		}
  }
  //----------------------------------------------------------------------------
  /**
   * This method is called upon plug-in activation.
   * 
   * @param context bundle context
   * 
   * @throws Exception if the bundle can not be started
   * 
   * @author Andreas Brker
   * @author Markus Krger
   */
  public void start(BundleContext context) throws Exception {
  	super.start(context);  
  	
  	//only for compatibility
    IPreferenceStore noaUIPrefStore = NOAUIPlugin.getDefault().getPreferenceStore();
    String officeHome = noaUIPrefStore.getString(NOAUIPlugin.PREFERENCE_OFFICE_HOME);
    if(officeHome.length() == 0) {
      officeHome = getPreferenceStore().getString("ag.ion.bion.workbench.office.editor.ui.preferences.LocalOfficeApplicationPreferences.applicationPath");
      if(officeHome.length() != 0) {
        noaUIPrefStore.setValue(NOAUIPlugin.PREFERENCE_OFFICE_HOME,officeHome);
      }
    }  	
  }
  //----------------------------------------------------------------------------
  /**
   * This method is called when the plug-in is stopped.
   * 
   * @param context bundle context
   * 
   * @throws Exception if the bundle can not be stopped
   * 
   * @author Andreas Br�ker
   */
  public void stop(BundleContext context) throws Exception {
    try {
      EditorCorePlugin.getDefault().getManagedLocalOfficeApplication().getDesktopService().deactivateTerminationPrevention();
    } 
    catch (OfficeApplicationException officeApplicationException) {
      //no prevention
    }
    super.stop(context);    
  }
  //----------------------------------------------------------------------------
  /**
   * Returns the shared instance.
   * 
   * @return shared instance
   * 
   * @author Andreas Br�ker
   */
  public static EditorUIPlugin getDefault() {
    return plugin;
  }
  //----------------------------------------------------------------------------
  /**
   * Returns the string from the plugin's resource bundle,
   * or 'key' if not found.
   * 
   * @param key key to be used
   * 
   * @return string from the plugin's resource bundle,
   * or 'key' if not found
   * 
   * @author Andreas Br�ker
   */
  public static String getResourceString(String key) {
  	ResourceBundle bundle = EditorUIPlugin.getDefault().getResourceBundle();
		try {
		  return (bundle != null) ? bundle.getString(key) : key;
		} 
	  catch (MissingResourceException missingResourceException) {
	      return key;
		}
  }
  //----------------------------------------------------------------------------
  /**
   * Returns the plugin's resource bundle.
   * 
   * @return plugin's resource bundle
   * 
   * @author Andreas Br�ker
   */
  public ResourceBundle getResourceBundle() {
	return resourceBundle;
  }
  //----------------------------------------------------------------------------
  /**
   * Returns form toolkit.
   * 
   * @return form toolkit
   * 
   * @author Andreas Brker
   * 
   * @deprecated Use NOAUIPlugin.getFormToolkit() instead.
   */
  public static FormToolkit getFormToolkit() {
    return NOAUIPlugin.getFormToolkit();    
  }  
  //----------------------------------------------------------------------------
  /**
   * Starts local office application.
   * 
   * @param shell shell to be used
   * @param officeApplication office application to be started
   * 
   * @return information whether the office application was started or not - only 
   * if the status severity is <code>IStatus.OK</code> the application was started 
   * 
   * @author Andreas Brker
   * 
   * @deprecated Use startLocalOfficeApplication(Shell shell, IOfficeApplication officeApplication) instead.
   */
  public IStatus startLocalOfficeApplication(Shell shell, IOfficeApplication officeApplication) {        
  	return NOAUIPlugin.startLocalOfficeApplication(shell, officeApplication);
  }
  //----------------------------------------------------------------------------

}