/****************************************************************************
 * ubion.ORS - The Open Report Suite                                        *
 *                                                                          *
 * ------------------------------------------------------------------------ *
 *                                                                          *
 * Subproject: Office Editor UI                                             *
 *                                                                          *
 *                                                                          *
 * The Contents of this file are made available subject to                  *
 * the terms of GNU Lesser General Public License Version 2.1.              *
 *                                                                          * 
 * GNU Lesser General Public License Version 2.1                            *
 * ======================================================================== *
 * Copyright 2003-2005 by IOn AG                                            *
 *                                                                          *
 * This library is free software; you can redistribute it and/or            *
 * modify it under the terms of the GNU Lesser General Public               *
 * License version 2.1, as published by the Free Software Foundation.       *
 *                                                                          *
 * This library is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        *
 * Lesser General Public License for more details.                          *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public         *
 * License along with this library; if not, write to the Free Software      *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,                    *
 * MA  02111-1307  USA                                                      *
 *                                                                          *
 * Contact us:                                                              *
 *  http://www.ion.ag                                                       *
 *  info@ion.ag                                                             *
 *                                                                          *
 ****************************************************************************/
 
/*
 * Last changes made by $Author: markus $, $Date: 2006-09-04 17:29:10 +0200 (Mo, 04 Sep 2006) $
 */
package ag.ion.bion.workbench.office.editor.ui.editors;

import ag.ion.bion.officelayer.desktop.IFrame;
import ag.ion.bion.workbench.office.editor.ui.EditorUIPlugin;

import ag.ion.noa.document.IDocumentProvider;
import ag.ion.noa.frame.IFrameProvider;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.ResourcesPlugin;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;

import org.eclipse.jface.dialogs.ErrorDialog;

import org.eclipse.swt.widgets.Composite;

import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IPartListener2;
import org.eclipse.ui.IWorkbenchPartReference;
import org.eclipse.ui.PartInitException;

import org.eclipse.ui.part.EditorPart;

/**
 * Abstract editor for OpenOffice.org documents.
 * 
 * @author Andreas Brker
 * @author Markus Krger
 * @version $Revision: 9747 $
 */
public abstract class AbstractOfficeEditor extends EditorPart implements IDocumentProvider, IFrameProvider {  
  
  private static final String ERROR_STORING_DOCUMENT   = Messages.AbstractOfficeEditor_message_document_can_not_be_stored;
  
  private IResourceChangeListener resourceCloseProjectListener  = null;
  private IEditorFileSynchronizer officeEditorFileSynchronizer  = null; 
  
  //----------------------------------------------------------------------------  
  /**
   * Internal editor advisor state delegate.
   * 
   * @author Andreas Brker
   * @author Markus Krger
   */
  private class EditorAdvisorStateDelegate implements IEditorAdvisorStateDelegate {

    //----------------------------------------------------------------------------  
    /**
     * Sets dirty state.
     * 
     * @param dirtyState new dirty state to be used
     * 
     * @author Andreas Brker
     * @author Markus Krger
     */
    public void setDirtyState(boolean dirtyState) {
      EditorUIPlugin.getDefault().getWorkbench().getDisplay().asyncExec(new Runnable() {
        public void run() {          
          setDirty();          
        }        
      });
    }
    //----------------------------------------------------------------------------  
    
  }  
  //----------------------------------------------------------------------------  
  
  //----------------------------------------------------------------------------
  
  //----------------------------------------------------------------------------
  /**
   * Initializes this editor with the given editor site and input. 
   * 
   * @param editorSite the editor site to be used
   * @param editorInput the editor input to be used
   * 
   * @throws PartInitException if this editor was not initialized successfully
   * 
   * @author Andreas Brker
   * @author Markus Krger
   * @date 24.08.2006
   */
  public void init(IEditorSite editorSite, IEditorInput editorInput) throws PartInitException {
    setSite(editorSite);
    setInput(editorInput);
    
    if(editorInput instanceof IFileEditorInput) {
      //close editor if the related project is about to be closed
      IFile file = ((IFileEditorInput)getEditorInput()).getFile();
      IProject project = file.getProject();
      resourceCloseProjectListener = new EditorResourceProjectCloseListener(project,this);      
      ResourcesPlugin.getWorkspace().addResourceChangeListener(resourceCloseProjectListener, IResourceChangeEvent.PRE_CLOSE); 
      
      //update the editor if the resource was changed
      officeEditorFileSynchronizer = new OfficeEditorFileSynchronizer(file,this);
      officeEditorFileSynchronizer.install();
    }
  }
  //----------------------------------------------------------------------------
  /**
   * Creates the SWT controls for this workbench part. 
   * 
   * @param parent parent control
   * 
   * @author Andreas Brker
   * @author Markus Krger
   */  
  public void createPartControl(Composite parent) {  
    EditorUIPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getPartService().addPartListener(new IPartListener2(){
      public void partActivated(IWorkbenchPartReference partRef) {
      }
      public void partBroughtToTop(IWorkbenchPartReference partRef) {
      }
      public void partClosed(IWorkbenchPartReference partRef) {
        //do dispose here as otherwise causes a deadlock in openoffice.org
        if(partRef.getPart(false).equals(AbstractOfficeEditor.this)) {
          getOfficeEditorAdvisor().dispose();
        }
      }
      public void partDeactivated(IWorkbenchPartReference partRef) {
      }
      public void partOpened(IWorkbenchPartReference partRef) {
      }
      public void partHidden(IWorkbenchPartReference partRef) {
      }
      public void partVisible(IWorkbenchPartReference partRef) {
      }
      public void partInputChanged(IWorkbenchPartReference partRef) {
      }      
    });   
    
    try {
      getOfficeEditorAdvisor().constructEditorControl(parent, getEditorInput());
      if(getOfficeEditorAdvisor().getLoadedDocument() == null) 
        return;

      getOfficeEditorAdvisor().setStateDelegate(new EditorAdvisorStateDelegate());
      setPartName(getEditorInput().getName());
    }    
    catch(CoreException coreException) {
      ErrorDialog.openError(getSite().getShell(), Messages.AbstractOfficeEditor_dialog_title_error, coreException.getMessage(), coreException.getStatus());
    }
  }  
  //----------------------------------------------------------------------------  
  /**
   * Saves the contents of this part.
   * 
   * @param progressMonitor the progress monitor
   * 
   * @author Andreas Brker
   * @author Markus Krger
   */
  public void doSave(IProgressMonitor progressMonitor) {
    try {
      // The synchronizer is uninstalled to avoid the call of handle content change as a 
      // save is done by the editor as there will be nothing to be synchronized. It will be
      // reinstalled as save is done.
      if(officeEditorFileSynchronizer != null)
        officeEditorFileSynchronizer.uninstall();
      getOfficeEditorAdvisor().doSave(progressMonitor, false);
      if(officeEditorFileSynchronizer != null)
        officeEditorFileSynchronizer.install();
    }
    catch(CoreException coreException) {
      if(officeEditorFileSynchronizer != null)
        officeEditorFileSynchronizer.install();
      ErrorDialog.openError(getSite().getShell(), Messages.AbstractOfficeEditor_dialog_title_error, ERROR_STORING_DOCUMENT, coreException.getStatus());
    }
  }
  //----------------------------------------------------------------------------
  /**
   * Saves the contents of this part to another object. 
   * 
   * @author Andreas Brker 
   */
  public void doSaveAs() {
    try {
      getOfficeEditorAdvisor().doSaveAs(getSite().getShell());
    }
    catch(CoreException coreException) {
      ErrorDialog.openError(getSite().getShell(), Messages.AbstractOfficeEditor_dialog_title_error, ERROR_STORING_DOCUMENT, coreException.getStatus());
    }
  } 
  //----------------------------------------------------------------------------
  /**
   * Returns whether the contents of this part have changed since the last save operation.
   * 
   * @return true if the contents have been modified and need saving, and false if they have 
   * not changed since the last save
   * 
   * @author Andreas Brker
   */
  public boolean isDirty() {     
    return getOfficeEditorAdvisor().isDocumentDirty();
  }
  //----------------------------------------------------------------------------
  /**
   * Asks this part to take focus within the workbench.
   * 
   * @author Andreas Brker
   * @author Markus Krger
   */
  public void setFocus() {      
    if(getOfficeEditorAdvisor().getLoadedDocument() == null) {
      //no document is available - close the editor.
      EditorUIPlugin.getDefault().getWorkbench().getDisplay().asyncExec(new Runnable() {
        public void run() {          
          EditorUIPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getActivePage().closeEditor(AbstractOfficeEditor.this, false);
        }        
      });
    }
    else {
      getOfficeEditorAdvisor().getFrame().setFocus();
    }
  }   
  //----------------------------------------------------------------------------
  /**
   * Returns information whether the "Save As" operation is supported by this part.
   * 
   * @return information whether the "Save As" operation is supported by this part
   * 
   * @author Andreas Brker
   */
  public boolean isSaveAsAllowed() {
    return getOfficeEditorAdvisor().isSaveAsAllowed();  
  }
  //----------------------------------------------------------------------------
	/**
	 * Returns frame. Returns null
	 * if a frame is not available.
	 * 
	 * @return frame or null
	 * if a frame is not available
	 * 
	 * @author Andreas Brker
	 * @date 15.07.2006
	 */
	public IFrame getFrame() {
		return getOfficeEditorAdvisor().getFrame();
	}
  //----------------------------------------------------------------------------
  /**
   * Disposes of this workbench part
   * 
   * @author Andreas Brker
   * @author Markus Krger
   */
  public void dispose() { 
    if(resourceCloseProjectListener != null)
      ResourcesPlugin.getWorkspace().removeResourceChangeListener(resourceCloseProjectListener);
    if(officeEditorFileSynchronizer != null)
      officeEditorFileSynchronizer.uninstall();
    
    IOfficeEditorAdvisor officeEditorAdvisor = getOfficeEditorAdvisor();
    if(officeEditorAdvisor != null) {
      officeEditorAdvisor.dispose();
    }
    
    super.dispose();    
  }  
  //----------------------------------------------------------------------------
  /**
   * Sets editor dirty.
   * 
   * @author Andreas Brker
   */
  protected void setDirty() {
    firePropertyChange(PROP_DIRTY);   
    getEditorSite().getActionBars().updateActionBars();
    getEditorSite().getActionBars().getMenuManager().update();
    getEditorSite().getActionBars().getToolBarManager().update(true);
  }  
  //----------------------------------------------------------------------------
  /**
   * Returns office editor advisor. Sublasses must implement their own
   * office editor advisor.
   * 
   * @return office editor advisor
   * 
   * @author Andreas Brker
   */
  protected abstract IOfficeEditorAdvisor getOfficeEditorAdvisor();
  //----------------------------------------------------------------------------
  
}