/****************************************************************************
 * ubion.ORS - The Open Report Suite                                        *
 *                                                                          *
 * ------------------------------------------------------------------------ *
 *                                                                          *
 * Subproject: Office Editor UI                                             *
 *                                                                          *
 *                                                                          *
 * The Contents of this file are made available subject to                  *
 * the terms of GNU Lesser General Public License Version 2.1.              *
 *                                                                          * 
 * GNU Lesser General Public License Version 2.1                            *
 * ======================================================================== *
 * Copyright 2003-2005 by IOn AG                                            *
 *                                                                          *
 * This library is free software; you can redistribute it and/or            *
 * modify it under the terms of the GNU Lesser General Public               *
 * License version 2.1, as published by the Free Software Foundation.       *
 *                                                                          *
 * This library is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        *
 * Lesser General Public License for more details.                          *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public         *
 * License along with this library; if not, write to the Free Software      *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,                    *
 * MA  02111-1307  USA                                                      *
 *                                                                          *
 * Contact us:                                                              *
 *  http://www.ion.ag                                                       *
 *  info@ion.ag                                                             *
 *                                                                          *
 ****************************************************************************/
 
/*
 * Last changes made by $Author$, $Date$
 */
package ag.ion.bion.workbench.office.editor.ui.editors;

import ag.ion.bion.workbench.office.editor.ui.EditorUIPlugin;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;

import org.eclipse.ui.IEditorPart;

/**
 * Listen for editor resources that has to close the editor if the corresponding project closes.
 * 
 * @author Markus Krger
 * @version $Revision$
 */
public class EditorResourceProjectCloseListener implements IResourceChangeListener {

  private IProject    parentProject = null;
  private IEditorPart editorPart    = null;
  
  //----------------------------------------------------------------------------
  /**
   * Constructs new EditorResourceProjectCloseListener.
   * 
   * @param parentProject the parent project of the editor's resource to be used
   * @param editorPart the editor this listener is used for
   * 
   * @author Markus Krger
   * @date 24.08.2006
   */
  public EditorResourceProjectCloseListener(IProject parentProject, IEditorPart editorPart) {
    assert parentProject != null;
    assert editorPart != null;
    this.parentProject = parentProject;
    this.editorPart = editorPart;
  }
  //----------------------------------------------------------------------------
  /**
   * Notifies this listener that some resource changes 
   * are happening, or have already happened.
   *
   * @param event the resource change event
   * 
   * @author Markus Krger
   * @date 24.08.2006
   */
  public void resourceChanged(IResourceChangeEvent event) {
    IResource resource = event.getResource();
    if(resource.equals(parentProject)) {
      EditorUIPlugin.getDefault().getWorkbench().getDisplay().asyncExec(new Runnable() {
        public void run() {
          EditorUIPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getActivePage().closeEditor(editorPart, true);
        }              
      });            
    }
  }
  //----------------------------------------------------------------------------
  
}