/****************************************************************************
 * ubion.ORS - The Open Report Suite                                        *
 *                                                                          *
 * ------------------------------------------------------------------------ *
 *                                                                          *
 * Subproject: Office Editor UI                                             *
 *                                                                          *
 *                                                                          *
 * The Contents of this file are made available subject to                  *
 * the terms of GNU Lesser General Public License Version 2.1.              *
 *                                                                          * 
 * GNU Lesser General Public License Version 2.1                            *
 * ======================================================================== *
 * Copyright 2003-2005 by IOn AG                                            *
 *                                                                          *
 * This library is free software; you can redistribute it and/or            *
 * modify it under the terms of the GNU Lesser General Public               *
 * License version 2.1, as published by the Free Software Foundation.       *
 *                                                                          *
 * This library is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        *
 * Lesser General Public License for more details.                          *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public         *
 * License along with this library; if not, write to the Free Software      *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,                    *
 * MA  02111-1307  USA                                                      *
 *                                                                          *
 * Contact us:                                                              *
 *  http://www.ion.ag                                                       *
 *  info@ion.ag                                                             *
 *                                                                          *
 ****************************************************************************/
 
/*
 * Last changes made by $Author: andreas $, $Date: 2006-08-07 13:59:32 +0200 (Mo, 07 Aug 2006) $
 */
package ag.ion.bion.workbench.office.editor.ui.editors;

import ag.ion.bion.officelayer.desktop.IFrame;
import ag.ion.bion.officelayer.document.IDocument;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;

/**
 * Advisor for an office editor.
 * 
 * @author Andreas Brker
 * @author Markus Krger
 * @version $Revision: 9209 $
 */
public interface IOfficeEditorAdvisor {
  
  //----------------------------------------------------------------------------  
  /**
   * Constructs editor control.
   * 
   * @param parent composite to be used
   * @param editorInput editor input to be used
   * 
   * @return contructed editor control
   * 
   * @throws CoreException if the editor control can not be constructed
   * 
   * @author Andreas Brker
   */
  public Control constructEditorControl(Composite parent, IEditorInput editorInput) throws CoreException;  
  //----------------------------------------------------------------------------  
  /**
   * Sets editor advisor state delegate.
   * 
   * @param editorAdvisorStateDelegate editor advisor state delegate to be used
   * 
   * @author Andreas Brker 
   */
  public void setStateDelegate(IEditorAdvisorStateDelegate editorAdvisorStateDelegate);  
  //----------------------------------------------------------------------------  
  /**
   * Saves the document.
   * 
   * @param progressMonitor progress monitor to be used
   * @param isSubTask information whether this operation is only a sub task
   * 
   * @throws CoreException if the document can not be stored
   * 
   * @author Andreas Brker
   */
  public void doSave(IProgressMonitor progressMonitor, boolean isSubTask) throws CoreException;
  //----------------------------------------------------------------------------  
  /**
   * Saves the document as other document.
   *  
   * @param shell shell to be used
   * 
   * @throws CoreException if the document can not be stored
   * 
   * @author Andreas Brker
   */
  public void doSaveAs(Shell shell) throws CoreException;
  //----------------------------------------------------------------------------  
  /**
   * Returns information whether the document can saved to
   * another document.
   * 
   * @return information whether the document can saved to
   * another document
   * 
   * @author Andreas Brker
   * @date 13.07.2006
   */
  public boolean isSaveAsAllowed();  
  //----------------------------------------------------------------------------  
  /**
   * Returns document. Returns null if a document is not 
   * available.
   * 
   * @return document or null if a document is not 
   * available
   * 
   * @author Andreas Brker
   */
  public IDocument getLoadedDocument();
  //----------------------------------------------------------------------------  
  /**
   * Returns the editor part that the advisor uses.
   * 
   * @return the editor part that the advisor uses
   * 
   * @author Markus Krger
   */
  public IEditorPart getEditorPart();
  //----------------------------------------------------------------------------  
  /**
   * Returns information whether the document is dirty.
   * 
   * @return information whether the document is dirty
   * 
   * @author Andreas Brker
   */
  public boolean isDocumentDirty();
  //----------------------------------------------------------------------------  
  /**
   * Disposes the advisor.
   * 
   * @author Andreas Brker
   */
  public void dispose();
  //----------------------------------------------------------------------------   
  /**
   * Returns the frame, or null, if none was set.
   * 
   * @return the frame, or null, if none was set
   * 
   * @author Markus Krger
   */
  public IFrame getFrame();
  //----------------------------------------------------------------------------  
  
}