/****************************************************************************
 * ubion.ORS - The Open Report Suite                                        *
 *                                                                          *
 * ------------------------------------------------------------------------ *
 *                                                                          *
 * Subproject: Office Editor UI                                             *
 *                                                                          *
 *                                                                          *
 * The Contents of this file are made available subject to                  *
 * the terms of GNU Lesser General Public License Version 2.1.              *
 *                                                                          * 
 * GNU Lesser General Public License Version 2.1                            *
 * ======================================================================== *
 * Copyright 2003-2005 by IOn AG                                            *
 *                                                                          *
 * This library is free software; you can redistribute it and/or            *
 * modify it under the terms of the GNU Lesser General Public               *
 * License version 2.1, as published by the Free Software Foundation.       *
 *                                                                          *
 * This library is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        *
 * Lesser General Public License for more details.                          *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public         *
 * License along with this library; if not, write to the Free Software      *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,                    *
 * MA  02111-1307  USA                                                      *
 *                                                                          *
 * Contact us:                                                              *
 *  http://www.ion.ag                                                       *
 *  info@ion.ag                                                             *
 *                                                                          *
 ****************************************************************************/
 
/*
 * Last changes made by $Author$, $Date$
 */
package ag.ion.bion.workbench.office.editor.ui.editors;

import ag.ion.bion.officelayer.document.IDocument;

/**
 * Editor for OpenOffice.org documents.
 * 
 * @author Andreas Brker
 * @author Markus Krger
 * @version $Revision$
 */
public class OfficeEditor extends AbstractOfficeEditor {    
  
  private OfficeEditorAdvisor officeEditorAdvisor = null;
  
  //----------------------------------------------------------------------------
  /**
   * Returns document. Returns null if a document is not available.
   * 
   * @return loaded document or null if a document is not available
   * 
   * @author Andreas Brker
   */
  public IDocument getDocument() {
    if(officeEditorAdvisor != null)
      return officeEditorAdvisor.getLoadedDocument();
    return null;
  }  
  //----------------------------------------------------------------------------
  /**
   * Returns office editor advisor. Sublasses must implement their own
   * office editor advisor.
   * 
   * @return office editor advisor
   * 
   * @author Andreas Brker
   * @author Markus Krger
   */
  protected IOfficeEditorAdvisor getOfficeEditorAdvisor() {
    if(officeEditorAdvisor == null) 
      officeEditorAdvisor = new OfficeEditorAdvisor(this);
    return officeEditorAdvisor;
  }
  //----------------------------------------------------------------------------
  
}