/****************************************************************************
 * ubion.ORS - The Open Report Suite                                        *
 *                                                                          *
 * ------------------------------------------------------------------------ *
 *                                                                          *
 * Subproject: Office Editor UI                                             *
 *                                                                          *
 *                                                                          *
 * The Contents of this file are made available subject to                  *
 * the terms of GNU Lesser General Public License Version 2.1.              *
 *                                                                          * 
 * GNU Lesser General Public License Version 2.1                            *
 * ======================================================================== *
 * Copyright 2003-2005 by IOn AG                                            *
 *                                                                          *
 * This library is free software; you can redistribute it and/or            *
 * modify it under the terms of the GNU Lesser General Public               *
 * License version 2.1, as published by the Free Software Foundation.       *
 *                                                                          *
 * This library is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        *
 * Lesser General Public License for more details.                          *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public         *
 * License along with this library; if not, write to the Free Software      *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,                    *
 * MA  02111-1307  USA                                                      *
 *                                                                          *
 * Contact us:                                                              *
 *  http://www.ion.ag                                                       *
 *  info@ion.ag                                                             *
 *                                                                          *
 ****************************************************************************/
 
/*
 * Last changes made by $Author$, $Date$
 */
package ag.ion.bion.workbench.office.editor.ui.editors;

import ag.ion.bion.workbench.office.editor.ui.EditorUIPlugin;

import ag.ion.bion.officelayer.application.IOfficeApplication;

import ag.ion.bion.officelayer.desktop.GlobalCommands;
import ag.ion.bion.officelayer.desktop.IFrame;

import ag.ion.bion.officelayer.document.DocumentDescriptor;
import ag.ion.bion.officelayer.document.IDocument;
import ag.ion.bion.officelayer.document.IDocumentDescriptor;
import ag.ion.bion.officelayer.filter.IFilter;
import ag.ion.bion.officelayer.filter.TextFilter;
import ag.ion.bion.officelayer.filter.XMLDocBookFilter;

import ag.ion.noa.filter.MSOffice2003XMLFilter;
import ag.ion.noa.frame.IDispatchDelegate;

import ag.ion.noa4e.ui.operations.LoadDocumentOperation;

import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IStorageEditorInput;

import org.eclipse.ui.dialogs.ListDialog;
import org.eclipse.ui.part.EditorPart;

import org.eclipse.swt.SWT;

import org.eclipse.swt.awt.SWT_AWT;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IStorage;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.window.Window;

import java.awt.BorderLayout;
import java.awt.Frame;
import java.awt.Panel;

import java.io.InputStream;
import java.net.URL;

/**
 * Advisor for an OpenOffice.org text editor.
 * 
 * @author Andreas Brker
 * @author Markus Krger
 * @version $Revision$
 */
public class OfficeEditorAdvisor extends AbstractOfficeEditorAdvisor implements IOfficeEditorAdvisor {
  
  private static final IFilter[] XML_FILTERS_CALC = new IFilter[] {
    TextFilter.FILTER,
    MSOffice2003XMLFilter.FILTER
  };
  private static final IFilter[] XML_FILTERS_WRITER = new IFilter[] {
    TextFilter.FILTER,
    MSOffice2003XMLFilter.FILTER,
    XMLDocBookFilter.FILTER
  };
  private static final String[] XML_DOC_TYPES = new String[] {
    IDocument.WRITER,
    IDocument.CALC
  };

	private boolean isSaveAsAllowed = true;
	
  //----------------------------------------------------------------------------     
  /**
   * The constructor.
   * 
   * @param editorPart editor part used with this advisor.
   * 
   * @author Markus Krger
   */
  public OfficeEditorAdvisor(EditorPart editorPart) {
    super(editorPart);
  }
  //---------------------------------------------------------------------------- 
  /**
   * Constructs editor control.
   * 
   * @param parent composite to be used
   * @param editorInput editor input to be used
   * 
   * @return contructed editor control
   * 
   * @throws CoreException if the editor control can not be constructed
   * 
   * @author Andreas Brker
   * @author Markus Krger
   */
  public Control constructEditorControl(Composite parent, IEditorInput editorInput) throws CoreException {
    /**
     * XXX: This is only a hack. Content in the clipboard can cause
     * a deadlock during opening an OpenOffice.org document.
     */
    Clipboard clipboard = new Clipboard(Display.getCurrent());
    if(clipboard.getAvailableTypeNames().length != 0) {      
      clipboard.setContents(new Object[] {" "}, new Transfer[] {TextTransfer.getInstance()}); //$NON-NLS-1$
      clipboard.clearContents();
    }
    clipboard.dispose();
    /**
     * XXX: End.
     */
    
    Composite composite = new Composite(parent, SWT.EMBEDDED);
    composite.setVisible(true);

    try {      
      Frame frame = SWT_AWT.new_Frame(composite);      
      frame.setVisible(true);
      Panel panel = new Panel(new BorderLayout());
      frame.add(panel);
      IOfficeApplication officeApplication = getOfficeApplication(parent.getShell());
      if(officeApplication.isActive()) {
        IFrame officeFrame = officeApplication.getDesktopService().constructNewOfficeFrame(panel);
        setFrame(officeFrame);
        
        //resfresh file if possible
        if(editorInput instanceof IFileEditorInput) {
          IFile fileToRefresh = ((IFileEditorInput)editorInput).getFile();
          fileToRefresh.refreshLocal(0,null);
        }
        
        //get URL of document if possible
        URL documentURL = getURLFromEditorInput(editorInput);
        
        DocumentDescriptor documentDescriptor = new DocumentDescriptor();
        documentDescriptor.setTitle(editorInput.getName()); 
        if(documentURL != null) {
          documentDescriptor.setURL(documentURL.toString());      
          documentDescriptor.setBaseURL(documentURL.toString());
        }

        isSaveAsAllowed = true;
        InputStream inputStream = null;
        LoadDocumentOperation loadTextDocumentOperation = null;
        
        //Load document with input stream if it is not a base document
        if(!isBaseDocumentURL(documentURL) && editorInput instanceof IStorageEditorInput) {
        	IStorageEditorInput storageEditorInput = (IStorageEditorInput)editorInput;
          boolean readOnly = false;
          IStorage storage = storageEditorInput.getStorage();
          if(storage == null || storage.isReadOnly())
            readOnly = true;
          documentDescriptor.setReadOnly(readOnly);
          inputStream = storageEditorInput.getStorage().getContents();
          if(documentURL != null && documentURL.getFile().endsWith(".xml")) { //$NON-NLS-1$
            setXMLLoadFilter(composite.getShell(),documentDescriptor);         
          }
          loadTextDocumentOperation = new LoadDocumentOperation(null, officeApplication, officeFrame, inputStream, documentDescriptor);
        }
        else if(documentURL != null) {
          loadTextDocumentOperation = new LoadDocumentOperation(null,officeApplication, officeFrame, documentURL, documentDescriptor); 
        }
        
        try {
          if(loadTextDocumentOperation == null)
            throw new CoreException(
                new Status(
                    IStatus.ERROR, 
                    EditorUIPlugin.PLUGIN_ID, 
                    IStatus.ERROR, 
                    Messages.OfficeEditorAdvisor_error_invalid_editor_input + editorInput.getClass().getName(), 
                    null));
          EditorUIPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().run(true, true, loadTextDocumentOperation);
        }
        catch(InterruptedException interruptedException) {
          //do not consume   
          //TODO: clear the document in the office system
          officeFrame.close();
          frame.dispose();
          composite.dispose();
          composite = new Composite(parent, SWT.EMBEDDED);
          return composite;
        }
        Exception exception = loadTextDocumentOperation.getException();
        
        if(exception != null) 
          throw new CoreException(new Status(IStatus.ERROR, EditorUIPlugin.PLUGIN_ID, IStatus.OK, exception.getMessage(), exception));

        IDocument document = loadTextDocumentOperation.getDocument();    
        configureOfficeFrame(officeFrame, document);
        setDocument(document); 
        return composite;
      }
      return new Composite(parent, SWT.EMBEDDED);
    }
    catch(Throwable throwable) {
      String message = ""; //$NON-NLS-1$
      if(throwable.getMessage() != null)
        message = throwable.getMessage();
      throw new CoreException(new Status(IStatus.ERROR, EditorUIPlugin.PLUGIN_ID, IStatus.OK, message, throwable));
    }    
  }
  //----------------------------------------------------------------------------  
  /**
   * Returns information whether the document can saved to
   * another document.
   * 
   * @return information whether the document can saved to
   * another document
   * 
   * @author Andreas Brker
   * @date 13.07.2006
   */
  public boolean isSaveAsAllowed() {
  	return isSaveAsAllowed;
  }
  //----------------------------------------------------------------------------
  /**
   * Configures the office frame.
   * 
   * @param frame office frame to be configured
   * @param document the document to be used
   * 
   * @author Andreas Brker
   * @author Markus Krger
   * @date 15.07.2006
   */
  private void configureOfficeFrame(IFrame frame, IDocument document) {
    if(document == null)
      return;
    
  	frame.addDispatchDelegate(GlobalCommands.SAVE_AS, new IDispatchDelegate() {
      public void dispatch(Object[] arg0) {
				editorPart.getSite().getShell().getDisplay().asyncExec(new Runnable() {
					public void run() {
						editorPart.doSaveAs();
					}
				});
			}    	
    });
    
    frame.addDispatchDelegate(GlobalCommands.SAVE, new IDispatchDelegate() {
      public void dispatch(Object[] arg0) {
        editorPart.getSite().getShell().getDisplay().asyncExec(new Runnable() {
          public void run() {
            editorPart.doSave(new NullProgressMonitor());
          }
        });
      }     
    });
  	
  	frame.disableDispatch(GlobalCommands.CLOSE_DOCUMENT);
  	frame.disableDispatch(GlobalCommands.CLOSE_WINDOW);
  	frame.disableDispatch(GlobalCommands.QUIT_APPLICATION);
    frame.updateDispatches();
  }  
  //----------------------------------------------------------------------------   
  /**
   * Asks for the xml filter to be used and set it to the given document descriptor
   * 
   * @param shell the shell used for dialogs
   * @param documentDescriptor the document descriptor to be used
   * 
   * @author Markus Krger
   * @date 13.03.2008
   */
  public void setXMLLoadFilter(Shell shell, IDocumentDescriptor documentDescriptor) {
    ListDialog docTypeListDialog = new ListDialog(shell);
    docTypeListDialog.setMessage(Messages.OfficeEditorAdvisor_dialog_message_doc_type);
    docTypeListDialog.setTitle(Messages.OfficeEditorAdvisor_dialog_title_doc_type);
    docTypeListDialog.setInitialSelections(new Object[]{XML_DOC_TYPES[0]});
    docTypeListDialog.setContentProvider(new IStructuredContentProvider(){
      public Object[] getElements(Object inputElement) {
        return (String[])inputElement;
      }
      public void dispose() {
        //nothing to do
      }
      public void inputChanged(Viewer viewer, Object oldInput,
          Object newInput) {
        //nothing to do
      }              
    });
    docTypeListDialog.setLabelProvider(new LabelProvider());
    docTypeListDialog.setInput(XML_DOC_TYPES);
    if(docTypeListDialog.open() == Window.OK) {
      Object[] resultDocType = docTypeListDialog.getResult();
      if(resultDocType.length == 1) {
        final String docType = (String)resultDocType[0];
        ListDialog listFilterDialog = new ListDialog(shell);
        listFilterDialog.setMessage(Messages.OfficeEditorAdvisor_dialog_message_file_type);
        listFilterDialog.setTitle(Messages.OfficeEditorAdvisor_dialog_title_file_type);
        IFilter[] filtersToUse = XML_FILTERS_WRITER; 
        if(docType.equals(IDocument.CALC)) {
          filtersToUse = XML_FILTERS_CALC;
        }
        listFilterDialog.setInitialSelections(new Object[]{filtersToUse[0]});
        listFilterDialog.setContentProvider(new IStructuredContentProvider(){
          public Object[] getElements(Object inputElement) {
            return (IFilter[])inputElement;
          }
          public void dispose() {
            //nothing to do
          }
          public void inputChanged(Viewer viewer, Object oldInput,
              Object newInput) {
            //nothing to do
          }              
        });
        listFilterDialog.setLabelProvider(new LabelProvider(){
          public String getText(Object element) {
            if(element instanceof IFilter) {
              return ((IFilter)element).getName(docType);
            }
            return super.getText(element);
          }
        });
        listFilterDialog.setInput(filtersToUse);
        if(listFilterDialog.open() == Window.OK) {
          Object[] resultFilterType = listFilterDialog.getResult();
          if(resultFilterType.length == 1) {
            IFilter filterType = (IFilter)resultFilterType[0];
            documentDescriptor.setFilterDefinition(filterType.getFilterDefinition(docType));
          }
        }
      }
    }   
  }
  //----------------------------------------------------------------------------

}