/****************************************************************************
 * ubion.ORS - The Open Report Suite                                        *
 *                                                                          *
 * ------------------------------------------------------------------------ *
 *                                                                          *
 * Subproject: Office Editor UI                                             *
 *                                                                          *
 *                                                                          *
 * The Contents of this file are made available subject to                  *
 * the terms of GNU Lesser General Public License Version 2.1.              *
 *                                                                          * 
 * GNU Lesser General Public License Version 2.1                            *
 * ======================================================================== *
 * Copyright 2003-2005 by IOn AG                                            *
 *                                                                          *
 * This library is free software; you can redistribute it and/or            *
 * modify it under the terms of the GNU Lesser General Public               *
 * License version 2.1, as published by the Free Software Foundation.       *
 *                                                                          *
 * This library is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        *
 * Lesser General Public License for more details.                          *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public         *
 * License along with this library; if not, write to the Free Software      *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,                    *
 * MA  02111-1307  USA                                                      *
 *                                                                          *
 * Contact us:                                                              *
 *  http://www.ion.ag                                                       *
 *  info@ion.ag                                                             *
 *                                                                          *
 ****************************************************************************/
 
/*
 * Last changes made by $Author$, $Date$
 */
package ag.ion.bion.workbench.office.editor.ui.editors;

import ag.ion.bion.workbench.office.editor.ui.EditorUIPlugin;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;

import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.part.FileEditorInput;

/**
 * Syncronizer for office editor file resources.
 * 
 * @author Markus Krger
 * @version $Revision$
 */
public class OfficeEditorFileSynchronizer extends EditorFileSynchronizer 
  implements IResourceChangeListener, IResourceDeltaVisitor, IEditorFileSynchronizer {
  
  //----------------------------------------------------------------------------
  /**
   * Constructs new InteractionEditorFileSynchronizer.
   * 
   * @param file the editor's file resource to be used
   * @param editorPart the editor this listener is used for
   * 
   * @author Markus Krger
   * @date 25.08.2006
   */
  public OfficeEditorFileSynchronizer(IFile file, IEditorPart editorPart) {
    super(file,editorPart);
  }
  //----------------------------------------------------------------------------
  /**
   * The method is called if the resource is moved. 
   * Default implementation closes the belonging editor.
   * Clients may overide this method.
   * 
   * @param newFile the new file resource, can be null
   * 
   * @author Markus Krger
   * @date 25.08.2006
   */
  protected void handleMoved(IFile newFile) {
    if(newFile != null) {
      try {
        super.handleMoved(newFile);
        EditorUIPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getActivePage().openEditor(
            new FileEditorInput(newFile), editorPart.getEditorSite().getId(), true);
      }
      catch(Exception exception) {
        ErrorDialog.openError(editorPart.getSite().getShell(), Messages.OfficeEditorFileSynchronizer_internal_error, exception.getMessage(), 
          new Status(IStatus.ERROR, EditorUIPlugin.PLUGIN_ID, IStatus.ERROR, exception.getMessage(), exception));
      }
    }
    else {
      super.handleMoved(newFile);
    }
  }
  //----------------------------------------------------------------------------
  /**
   * The method is called whenever the resource's content changed. This
   * includes also a call of the "save" method of an editor.
   * This means, that if it is not intended to react when a "save" 
   * is done in an editor, then the editor has to un- and reinstall the synchronizer 
   * in its "save" method.
   * Default implementation closes the belonging editor.
   * Clients may overide this method.
   * 
   * @author Markus Krger
   * @date 25.08.2006
   */
  protected void handleContentChange() {
    try {
      super.handleContentChange();
      EditorUIPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getActivePage().openEditor(
          new FileEditorInput(file), editorPart.getEditorSite().getId(), true);
    }
    catch(Exception exception) {
      ErrorDialog.openError(editorPart.getSite().getShell(), Messages.OfficeEditorFileSynchronizer_internal_error, exception.getMessage(), 
          new Status(IStatus.ERROR, EditorUIPlugin.PLUGIN_ID, IStatus.ERROR, exception.getMessage(), exception));
    }
  }
  //----------------------------------------------------------------------------
  
}