/****************************************************************************
 * ubion.ORS - The Open Report Suite                                        *
 *                                                                          *
 * ------------------------------------------------------------------------ *
 *                                                                          *
 * Subproject: Office Editor UI                                             *
 *                                                                          *
 *                                                                          *
 * The Contents of this file are made available subject to                  *
 * the terms of GNU Lesser General Public License Version 2.1.              *
 *                                                                          * 
 * GNU Lesser General Public License Version 2.1                            *
 * ======================================================================== *
 * Copyright 2003-2005 by IOn AG                                            *
 *                                                                          *
 * This library is free software; you can redistribute it and/or            *
 * modify it under the terms of the GNU Lesser General Public               *
 * License version 2.1, as published by the Free Software Foundation.       *
 *                                                                          *
 * This library is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        *
 * Lesser General Public License for more details.                          *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public         *
 * License along with this library; if not, write to the Free Software      *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,                    *
 * MA  02111-1307  USA                                                      *
 *                                                                          *
 * Contact us:                                                              *
 *  http://www.ion.ag                                                       *
 *  info@ion.ag                                                             *
 *                                                                          *
 ****************************************************************************/
 
/*
 * Last changes made by $Author$, $Date$
 */
package ag.ion.bion.workbench.office.editor.ui.wizards.document;

import ag.ion.bion.workbench.office.editor.ui.EditorUIPlugin;

import ag.ion.bion.workbench.office.editor.core.EditorCorePlugin;

import ag.ion.bion.workbench.office.editor.internal.ui.operations.ConstructNewDocumentOperation;

import ag.ion.bion.officelayer.application.IOfficeApplication;
import ag.ion.noa4e.ui.NOAUIPlugin;

import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.wizard.Wizard;

import org.eclipse.ui.INewWizard;
import org.eclipse.ui.ide.IDE;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import java.lang.reflect.InvocationTargetException;

/**
 * Abstract wizard for new OpenOffice.org documents.
 * 
 * @author Andreas Brker
 * @version $Revision$
 */
public abstract class AbstractNewDocumentWizard extends Wizard implements INewWizard {
  
  //----------------------------------------------------------------------------
  /**
   * Constructs new AbstractNewDocumentWizard.
   * 
   * @author Andreas Brker
   */
  protected AbstractNewDocumentWizard() {
    setNeedsProgressMonitor(true);
  }
  //----------------------------------------------------------------------------
  /**
   * Performs any actions appropriate in response to the user 
   * having pressed the Finish button, or refuse if finishing
   * now is not permitted.
   *
   * @return <code>true</code> to indicate the finish request
   *   was accepted, and <code>false</code> to indicate
   *   that the finish request was refused
   * 
   * @author Andreas Brker
   */
  public boolean performFinish() {
    IOfficeApplication officeApplication = EditorCorePlugin.getDefault().getManagedLocalOfficeApplication();
    if(!officeApplication.isActive()) {      
      NOAUIPlugin.startLocalOfficeApplication(getShell(), officeApplication);
    }
    if(officeApplication.isActive()) {
      INewDocumentRequest newDocumentRequest = getNewDocumentRequest();
      ConstructNewDocumentOperation constructNewDocumentOperation = new ConstructNewDocumentOperation(officeApplication, 
          newDocumentRequest);
      try {
        getContainer().run(true, true, constructNewDocumentOperation);
      }
      catch(InterruptedException interruptedException) {
        //do not consume
        return false;
      }
      catch(InvocationTargetException invocationTargetException) {
        Throwable cause = invocationTargetException.getCause();
        ErrorDialog.openError(getShell(), Messages.AbstractNewDocumentWizard_dialog_error_title, cause.getMessage(), 
            new Status(IStatus.ERROR, EditorUIPlugin.PLUGIN_ID, IStatus.ERROR, cause.getMessage(), cause));
      }
      final IFile file = constructNewDocumentOperation.getDocumentFile();
      if(file != null) {       
        try {
        	IDE.openEditor(EditorUIPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getActivePage(), file);
        }
        catch(CoreException coreException) {
          ErrorDialog.openError(getShell(), Messages.AbstractNewDocumentWizard_dialog_error_title, coreException.getMessage(), coreException.getStatus());          
        }     
        return true;
      }   
      return false;
    }
    else
      return false;    
  }
  //----------------------------------------------------------------------------
  /**
   * Returns request for the new document. Returns null if a new document request
   * can not be provided.
   * 
   * @return request for the new document. Returns null if a new document request
   * can not be provided
   * 
   * @author Andreas Brker
   */
  protected INewDocumentRequest getNewDocumentRequest() {
  	if(getDefinePage() == null)
      return null;
    
    String documentType = getDocumentType();
    IContainer targetContainer = getDefinePage().getDocumentContainer();
    String documentName = getDefinePage().getDocumentName();
    if(documentType != null && targetContainer != null && documentName != null)
      return new NewDocumentRequest(documentType, targetContainer, documentName, getDefinePage().getTemplatePath());
    return null;
  }
  //----------------------------------------------------------------------------
  /**
   * Returns document wizards define page.
   * 
   * @return document wizards define page
   * 
   * @author Andreas Brker
   * @date 02.07.2006
   */
  protected abstract NewDocumentWizardDefinePage getDefinePage();
  //----------------------------------------------------------------------------
  /**
   * Returns document type of the wizard.
   * 
   * @return document type of the wizard
   * 
   * @author Andreas Brker
   * @date 02.07.2006
   */
  protected abstract String getDocumentType();  
  //----------------------------------------------------------------------------
  
}