/****************************************************************************
 *                                                                          *
 * NOA (Nice Office Access) for Eclipse                          						*
 * ------------------------------------------------------------------------ *
 *                                                                          *
 * The Contents of this file are made available subject to                  *
 * the terms of GNU Lesser General Public License Version 2.1.              *
 *                                                                          * 
 * GNU Lesser General Public License Version 2.1                            *
 * ======================================================================== *
 * Copyright 2003-2006 by IOn AG                                            *
 *                                                                          *
 * This library is free software; you can redistribute it and/or            *
 * modify it under the terms of the GNU Lesser General Public               *
 * License version 2.1, as published by the Free Software Foundation.       *
 *                                                                          *
 * This library is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        *
 * Lesser General Public License for more details.                          *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public         *
 * License along with this library; if not, write to the Free Software      *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,                    *
 * MA  02111-1307  USA                                                      *
 *                                                                          *
 * Contact us:                                                              *
 *  http://www.ion.ag																												*
 *  http://ubion.ion.ag                                                     *
 *  info@ion.ag                                                             *
 *                                                                          *
 ****************************************************************************/
 
/*
 * Last changes made by $Author: andreas $, $Date: 2006-08-07 13:59:32 +0200 (Mo, 07 Aug 2006) $
 */
package ag.ion.bion.workbench.office.editor.ui.wizards.document;

import ag.ion.bion.workbench.office.editor.ui.EditorUIPluginImages;

import ag.ion.bion.officelayer.document.IDocument;

import org.eclipse.jface.viewers.IStructuredSelection;

import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;

/**
 * Wizard for new OpenOffice.org base documents.
 * 
 * @author Andreas Brker
 * @date 16.03.2006
 * @version $Revision: 9209 $
 */
public class NewBaseDocumentWizard extends AbstractNewDocumentWizard implements INewWizard {

  private static final String DOCUMENT_FILE_EXTENSION = "odb"; //$NON-NLS-1$
  
  private NewDocumentWizardDefinePage newDocumentWizardDefinePage = null;
  
  private IStructuredSelection structuredSelection = null;
  
  //----------------------------------------------------------------------------
  /**
   * Constructs new NewBaseDocumentWizard.
   * 
   * @author Andreas Brker
   * @date 16.03.2006
   */
  public NewBaseDocumentWizard() {
    setWindowTitle(Messages.NewBaseDocumentWizard_title);
    setDefaultPageImageDescriptor(EditorUIPluginImages.getImageDescriptor(EditorUIPluginImages.IMG_WIZBAN_BASE_DOCUMENT));
  }
  //----------------------------------------------------------------------------
  /**
   * Initializes this creation wizard using the passed workbench and
   * object selection.
   *
   * @param workbench the current workbench
   * @param structuredSelection the current object selection
   * 
   * @author Andreas Brker
   * @date 16.03.2006
   */  
  public void init(IWorkbench workbench, IStructuredSelection structuredSelection) {
    this.structuredSelection = structuredSelection;    
  }
  //----------------------------------------------------------------------------
  /**
   * Adds any last-minute pages to this wizard.
   * 
   * @author Andreas Brker
   * @date 16.03.2006
   */
  public void addPages() {
    newDocumentWizardDefinePage = new NewDocumentWizardDefinePage(structuredSelection);
    newDocumentWizardDefinePage.setDocumentFileExtension(DOCUMENT_FILE_EXTENSION);
    newDocumentWizardDefinePage.setTitle(Messages.NewBaseDocumentWizard_define_page_title);
    newDocumentWizardDefinePage.setDescription(Messages.NewBaseDocumentWizard_define_page_description);
    newDocumentWizardDefinePage.setSupportTemplates(false);
    addPage(newDocumentWizardDefinePage);
  }
  //----------------------------------------------------------------------------
  /**
   * Returns document wizards define page.
   * 
   * @return document wizards define page
   * 
   * @author Andreas Brker
   * @date 02.07.2006
   */
  protected NewDocumentWizardDefinePage getDefinePage() {
  	return newDocumentWizardDefinePage;
  }
  //----------------------------------------------------------------------------
  /**
   * Returns document type of the wizard.
   * 
   * @return document type of the wizard
   * 
   * @author Andreas Brker
   * @date 02.07.2006
   */
  protected String getDocumentType() {
  	return IDocument.BASE;
  }
  //----------------------------------------------------------------------------
  
}