/****************************************************************************
 * ubion.ORS - The Open Report Suite                                        *
 *                                                                          *
 * ------------------------------------------------------------------------ *
 *                                                                          *
 * Subproject: Office Editor UI                                             *
 *                                                                          *
 *                                                                          *
 * The Contents of this file are made available subject to                  *
 * the terms of GNU Lesser General Public License Version 2.1.              *
 *                                                                          * 
 * GNU Lesser General Public License Version 2.1                            *
 * ======================================================================== *
 * Copyright 2003-2005 by IOn AG                                            *
 *                                                                          *
 * This library is free software; you can redistribute it and/or            *
 * modify it under the terms of the GNU Lesser General Public               *
 * License version 2.1, as published by the Free Software Foundation.       *
 *                                                                          *
 * This library is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        *
 * Lesser General Public License for more details.                          *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public         *
 * License along with this library; if not, write to the Free Software      *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,                    *
 * MA  02111-1307  USA                                                      *
 *                                                                          *
 * Contact us:                                                              *
 *  http://www.ion.ag                                                       *
 *  info@ion.ag                                                             *
 *                                                                          *
 ****************************************************************************/
 
/*
 * Last changes made by $Author$, $Date$
 */
package ag.ion.bion.workbench.office.editor.ui.wizards.document;

import org.eclipse.core.resources.IContainer;

/**
 * Request for a new document.
 * 
 * @author Andreas Brker
 * @author Markus Krger
 * @version $Revision$
 */
public class NewDocumentRequest implements INewDocumentRequest {

  private IContainer targetContainer = null;
  
  private String documentType = null;
  private String documentName = null;
  private String templatePath = null;
  
  //----------------------------------------------------------------------------
  /**
   * Constructs new NewDocumentRequest.
   * 
   * @param documentType document type to be used
   * @param targetContainer target container to be used
   * @param documentName document name to be used
   * @param templatePath template path to be used
   * 
   * @author Andreas Brker
   * @author Markus Krger
   */
  public NewDocumentRequest(String documentType, IContainer targetContainer, String documentName, String templatePath) {
    assert documentType != null;
    assert targetContainer != null;  
    assert documentName != null;
    
    this.documentType = documentType;          
    this.targetContainer = targetContainer;     	
    this.documentName = documentName;    
    this.templatePath = templatePath;
  }  
  //----------------------------------------------------------------------------
  /**
   * Returns type of the document.
   * 
   * @return type of the document
   * 
   * @author Andreas Brker
   */
  public String getDocumentType() {
    return documentType;
  }
  //----------------------------------------------------------------------------
  /**
   * Returns target container for the new document.
   * 
   * @return target container for the new document
   * 
   * @author Andreas Brker
   */
  public IContainer getTargetContainer() {
    return targetContainer;
  }
  //----------------------------------------------------------------------------
  /**
   * Returns name for the new document.
   * 
   * @return name for the new document
   * 
   * @author Andreas Brker
   */
  public String getDocumentName() {
    return documentName;
  }
  //----------------------------------------------------------------------------
  /**
   * Sets template path.
   * 
   * @param templatePath template path to be used
   * 
   * @author Andreas Brker
   * @date 02.07.2006
   */
  public void setTemplatePath(String templatePath) {
  	this.templatePath = templatePath;
  }  
  //----------------------------------------------------------------------------
  /**
   * Returns path of the template. Returns null
   * if a template path is not available.
   * 
   * @return path of the template or null
   * if a template path is not available
   * 
   * @author Andreas Brker
   * @date 02.07.2006
   */
  public String getTemplatePath() {
  	return templatePath;
  }
  //----------------------------------------------------------------------------
  
}