/****************************************************************************
 * ubion.ORS - The Open Report Suite                                        *
 *                                                                          *
 * ------------------------------------------------------------------------ *
 *                                                                          *
 * Subproject: Office Editor UI                                             *
 *                                                                          *
 *                                                                          *
 * The Contents of this file are made available subject to                  *
 * the terms of GNU Lesser General Public License Version 2.1.              *
 *                                                                          * 
 * GNU Lesser General Public License Version 2.1                            *
 * ======================================================================== *
 * Copyright 2003-2005 by IOn AG                                            *
 *                                                                          *
 * This library is free software; you can redistribute it and/or            *
 * modify it under the terms of the GNU Lesser General Public               *
 * License version 2.1, as published by the Free Software Foundation.       *
 *                                                                          *
 * This library is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        *
 * Lesser General Public License for more details.                          *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public         *
 * License along with this library; if not, write to the Free Software      *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,                    *
 * MA  02111-1307  USA                                                      *
 *                                                                          *
 * Contact us:                                                              *
 *  http://www.ion.ag                                                       *
 *  info@ion.ag                                                             *
 *                                                                          *
 ****************************************************************************/
 
/*
 * Last changes made by $Author$, $Date$
 */
package ag.ion.bion.workbench.office.editor.ui.wizards.document;

import ag.ion.bion.workbench.office.editor.ui.EditorUIPluginImages;

import ag.ion.bion.officelayer.document.IDocument;

import org.eclipse.jface.viewers.IStructuredSelection;

import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;

/**
 * Wizard for new OpenOffice.org text documents.
 * 
 * @author Andreas Brker
 * @version $Revision$
 */
public class NewTextDocumentWizard extends AbstractNewDocumentWizard implements INewWizard {

	/** Valid document extensions. */
  public static final String[] VALID_TEMPLATE_EXTENSIONS = new String[] {"ott", "odt", "dot", "doc", "sxw", "stw", "sdw", "vor"}; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$
	
  private static final String DOCUMENT_FILE_EXTENSION = "odt"; //$NON-NLS-1$
  private static final String TEMPLATE_FILE_EXTENSION = "ott"; //$NON-NLS-1$
  
  private NewDocumentWizardDefinePage newDocumentWizardDefinePage = null;
  
  private IStructuredSelection structuredSelection = null;
  
  //----------------------------------------------------------------------------
  /**
   * Constructs new NewTextDocumentWizard.
   * 
   * @author Andreas Brker
   */
  public NewTextDocumentWizard() {
    setWindowTitle(Messages.NewTextDocumentWizard_title);
    setDefaultPageImageDescriptor(EditorUIPluginImages.getImageDescriptor(EditorUIPluginImages.IMG_WIZBAN_TEXT_DOCUMENT));
  }
  //----------------------------------------------------------------------------
  /**
   * Initializes this creation wizard using the passed workbench and
   * object selection.
   *
   * @param workbench the current workbench
   * @param structuredSelection the current object selection
   * 
   * @author Andreas Brker
   */  
  public void init(IWorkbench workbench, IStructuredSelection structuredSelection) {
    this.structuredSelection = structuredSelection;    
  }
  //----------------------------------------------------------------------------
  /**
   * Adds any last-minute pages to this wizard.
   * 
   * @author Andreas Brker
   */
  public void addPages() {
    newDocumentWizardDefinePage = new NewDocumentWizardDefinePage(structuredSelection);
    newDocumentWizardDefinePage.setDocumentFileExtension(DOCUMENT_FILE_EXTENSION);
    newDocumentWizardDefinePage.setTemplateFileExtension(TEMPLATE_FILE_EXTENSION);
    newDocumentWizardDefinePage.setValidTemplateExtensions(VALID_TEMPLATE_EXTENSIONS);
    newDocumentWizardDefinePage.setTitle(Messages.NewTextDocumentWizard_define_page_title);
    newDocumentWizardDefinePage.setDescription(Messages.NewTextDocumentWizard_define_page_description);
    addPage(newDocumentWizardDefinePage);
  }
  //----------------------------------------------------------------------------
  /**
   * Returns document wizards define page.
   * 
   * @return document wizards define page
   * 
   * @author Andreas Brker
   * @date 02.07.2006
   */
  protected NewDocumentWizardDefinePage getDefinePage() {
  	return newDocumentWizardDefinePage;
  }
  //----------------------------------------------------------------------------
  /**
   * Returns document type of the wizard.
   * 
   * @return document type of the wizard
   * 
   * @author Andreas Brker
   * @date 02.07.2006
   */
  protected String getDocumentType() {
  	return IDocument.WRITER;
  }
  //----------------------------------------------------------------------------

}