/****************************************************************************
 *                                                                          *
 * NOA (Nice Office Access)                                     						*
 * ------------------------------------------------------------------------ *
 *                                                                          *
 * The Contents of this file are made available subject to                  *
 * the terms of GNU Lesser General Public License Version 2.1.              *
 *                                                                          * 
 * GNU Lesser General Public License Version 2.1                            *
 * ======================================================================== *
 * Copyright 2003-2006 by IOn AG                                            *
 *                                                                          *
 * This library is free software; you can redistribute it and/or            *
 * modify it under the terms of the GNU Lesser General Public               *
 * License version 2.1, as published by the Free Software Foundation.       *
 *                                                                          *
 * This library is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        *
 * Lesser General Public License for more details.                          *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public         *
 * License along with this library; if not, write to the Free Software      *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,                    *
 * MA  02111-1307  USA                                                      *
 *                                                                          *
 * Contact us:                                                              *
 *  http://www.ion.ag																												*
 *  http://ubion.ion.ag                                                     *
 *  info@ion.ag                                                             *
 *                                                                          *
 ****************************************************************************/
 
/*
 * Last changes made by $Author: andreas $, $Date: 2006-08-07 13:59:32 +0200 (Mo, 07 Aug 2006) $
 */
package ag.ion.noa4e.editor.ui.actions;

import ag.ion.bion.workbench.office.editor.ui.EditorUIPlugin;

import ag.ion.bion.officelayer.desktop.GlobalCommands;
import ag.ion.bion.officelayer.desktop.IFrame;

import ag.ion.noa.frame.IDispatch;
import ag.ion.noa.frame.IFrameProvider;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;

import org.eclipse.jface.dialogs.ErrorDialog;

import org.eclipse.jface.viewers.ISelection;

import org.eclipse.swt.widgets.Display;

import org.eclipse.ui.IEditorActionDelegate;
import org.eclipse.ui.IEditorPart;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

/**
 * Action in order to print a document.
 * 
 * @author Andreas Brker
 * @version $Revision: 9209 $
 * @date 15.07.2006
 */ 
public class PrintAction extends Action implements IEditorActionDelegate {

	/** ID of the action. */
	public static final String ACTION_ID = "ag.ion.noa4e.editor.ui.actions.PrintAction"; //$NON-NLS-1$
	
	private IFrameProvider frameProvider = null;
	
  //----------------------------------------------------------------------------
	/**
	 * Constructs new PrintAction.
	 *
	 * @author Andreas Brker
	 * @date 15.07.2006
	 */
	public PrintAction() {
		setText(Messages.PrintAction_text);
		setId(ACTION_ID);
		setEnabled(false);
	}
  //----------------------------------------------------------------------------
	/**
	 * Executes action logic.
	 * 
	 * @author Andreas Brker
	 * @date 15.07.2006
	 */
	public void run() {
		if(frameProvider != null) {
			IFrame frame = frameProvider.getFrame();
			if(frame != null) {
				try {
					IDispatch dispatch = frame.getDispatch(GlobalCommands.PRINT_DOCUMENT);
					if(dispatch != null)
						dispatch.dispatch();
				}
				catch(Throwable throwable) {
					ErrorDialog.openError(Display.getCurrent().getActiveShell(), Messages.PrintAction_dialog_title_internalError, 
							throwable.getMessage(), 
							new Status(IStatus.ERROR, EditorUIPlugin.PLUGIN_ID, IStatus.ERROR, throwable.getMessage(), 
							throwable));
				}
			}
		}
	}
  //----------------------------------------------------------------------------
  /**
   * Sets the active editor for the delegate.  
   * Implementors should disconnect from the old editor, connect to the 
   * new editor, and update the action to reflect the new editor.
   *
   * @param action the action proxy that handles presentation portion of the action
   * @param targetEditor the new editor target
   * 
   * @author Andreas Brker
	 * @date 15.07.2006
   */
	public void setActiveEditor(IAction action, IEditorPart targetEditor) {
		if(targetEditor instanceof IFrameProvider) {
			frameProvider = (IFrameProvider)targetEditor;
			setEnabled(true);
		}
		else
			setEnabled(false);
	}
  //----------------------------------------------------------------------------
  /**
   * Performs this action.
   * <p>
   * This method is called by the proxy action when the action has been
   * triggered. Implement this method to do the actual work.
   * </p><p>
   *
   * @param action the action proxy that handles the presentation portion of the
   *   action
   * 
   * @author Andreas Brker
	 * @date 15.07.2006
   */
	public void run(IAction action) {
		run();
	}
  //----------------------------------------------------------------------------
  /**
   * Notifies this action delegate that the selection in the workbench has changed.
   * <p>
   * Implementers can use this opportunity to change the availability of the
   * action or to modify other presentation properties.
   *
   * @param action the action proxy that handles presentation portion of 
   * 		the action
   * @param selection the current selection, or <code>null</code> if there
   * 		is no selection
   *  
   * @author Andreas Brker
	 * @date 15.07.2006
   */
	public void selectionChanged(IAction action, ISelection selection) {
	}
  //----------------------------------------------------------------------------
	
}