/****************************************************************************
 *                                                                          *
 * NOA (Nice Office Access)                                     						*
 * ------------------------------------------------------------------------ *
 *                                                                          *
 * The Contents of this file are made available subject to                  *
 * the terms of GNU Lesser General Public License Version 2.1.              *
 *                                                                          * 
 * GNU Lesser General Public License Version 2.1                            *
 * ======================================================================== *
 * Copyright 2003-2006 by IOn AG                                            *
 *                                                                          *
 * This library is free software; you can redistribute it and/or            *
 * modify it under the terms of the GNU Lesser General Public               *
 * License version 2.1, as published by the Free Software Foundation.       *
 *                                                                          *
 * This library is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        *
 * Lesser General Public License for more details.                          *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public         *
 * License along with this library; if not, write to the Free Software      *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,                    *
 * MA  02111-1307  USA                                                      *
 *                                                                          *
 * Contact us:                                                              *
 *  http://www.ion.ag																												*
 *  http://ubion.ion.ag                                                     *
 *  info@ion.ag                                                             *
 *                                                                          *
 ****************************************************************************/
 
/*
 * Last changes made by $Author: andreas $, $Date: 2006-08-07 13:59:32 +0200 (Mo, 07 Aug 2006) $
 */
package ag.ion.noa4e.editor.ui.dialogs;

import ag.ion.noa4e.ui.NOAUIPluginImages;

import ag.ion.bion.officelayer.document.IDocument;

import ag.ion.bion.officelayer.filter.IFilter;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.TitleAreaDialog;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;

import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.SWT;

import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;

import org.eclipse.swt.graphics.Image;

import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;

import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;

import org.eclipse.ui.model.BaseWorkbenchContentProvider;
import org.eclipse.ui.model.WorkbenchLabelProvider;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;

import org.eclipse.core.runtime.IPath;

/**
 * Save as dialog for office documents.
 * 
 * @author Andreas Brker
 * @version $Revision: 9209 $
 * @date 14.07.2006
 */ 
public class OfficeSaveAsDialog extends TitleAreaDialog {
	
	private static final String DEFAULT_MESSAGE = Messages.OfficeSaveAsDialog_dialog_default_message;
    
	private IPath originalFile 	= null;
  private IPath result				= null;

  private TreeViewer 	treeViewerWorkspace = null;    
  private Button 			okButton 						= null;    
  private Text 				textFolderName 			= null;    
  private Text 				textFileName 				= null;    
  private Combo 			comboFileTypes 			= null;
  private Image 			titleImage 					= null;
  
  private IContainer targetContainer = null;
  
  private IDocument document 			= null;
  private IFilter[] filters 			= null;
  private IFilter 	currentFilter = null;

  //----------------------------------------------------------------------------
  /**
   * Constructs new OfficeSaveAsDialog.
   * 
   * @param parentShell parent shell to be used
   * @param document document to be used
   * 
   * @author Andreas Brker
   * @date 14.07.2006
   */
  public OfficeSaveAsDialog(Shell parentShell, IDocument document) {
  	super(parentShell);
    setShellStyle(getShellStyle() | SWT.RESIZE);
    
    assert document != null;
    this.document = document;
    filters = document.getFilterProvider().getFilters();
  }
  //----------------------------------------------------------------------------
  /**
   * Returns filter for the document.
   * 
   * @return filter for the document
   * 
   * @author Andreas Brker
   * @date 14.07.2006
   */
  public IFilter getFilter() {
  	return currentFilter;
  }
  //----------------------------------------------------------------------------
  /**
   * Returns the selected target container. Returns null
   * if a target container is not available.
   * 
   * @return selected target container or null
   * if a target container is not available
   * 
   * @author Andreas Brker
   * @date 14.07.2006
   */
  public IContainer getTargetContainer() {
  	return targetContainer;
  }
  //----------------------------------------------------------------------------
  /**
   * Sets the original file to use.
   *
   * @param originalFile the original file
   * 
   * @author Andreas Brker
   * @date 14.07.2006
   */
  public void setOriginalFile(IPath path) {
  	this.originalFile = path;
  }
  //----------------------------------------------------------------------------
  /**
   * Returns the full path entered by the user.
   * <p>
   * Note that the file and container might not exist and would need to be created.
   * See the <code>IFile.create</code> method and the 
   * <code>ContainerGenerator</code> class.
   * </p>
   *
   * @return the path, or <code>null</code> if Cancel was pressed
   * 
   * @author Andreas Brker
   * @date 14.07.2006
   */
  public IPath getResult() {
  	return result;
  }
  //----------------------------------------------------------------------------
  /** 
   * The <code>SaveAsDialog</code> implementation of this <code>Window</code>
   * method disposes of the banner image when the dialog is closed.
   * 
   * @author Andreas Brker
   * @date 14.07.2006
   */
  public boolean close() {
    if (titleImage != null)
    	titleImage.dispose();
    return super.close();
  }
  //----------------------------------------------------------------------------
  /**
   * Creates dialog contents.
   * 
   * @param parent parent to be used
   * 
   * @author Andreas Brker
   * @date 14.07.2006
   */
  protected Control createContents(Composite parent) {
    Control contents = super.createContents(parent);
    initializeControls();

    setTitle(Messages.OfficeSaveAsDialog_dialog_title);
    titleImage = NOAUIPluginImages.getImageDescriptor(NOAUIPluginImages.IMG_WIZBAN_APPLICATION).createImage();
    setTitleImage(titleImage);
    setMessage(DEFAULT_MESSAGE);
    setDialogComplete(false);
    return contents;
  }
  //----------------------------------------------------------------------------
  /**
   * Creates buttons for button bar.
   * 
   * @param parent parent to be used
   * 
   * @author Andreas Brker
   * @date 14.07.2006
   */
  protected void createButtonsForButtonBar(Composite parent) {
    okButton = createButton(parent, IDialogConstants.OK_ID,
            IDialogConstants.OK_LABEL, true);
    createButton(parent, IDialogConstants.CANCEL_ID,
            IDialogConstants.CANCEL_LABEL, false);
  }
  //----------------------------------------------------------------------------
  /**
   * Creates dialog area.
   * 
   * @param parent parent to be used
   * 
   * @author Andreas Brker
   * @date 14.07.2006
   */
  protected Control createDialogArea(Composite parent) {
    Composite parentComposite = (Composite) super.createDialogArea(parent);
    Composite composite = new Composite(parentComposite, SWT.NONE);
    GridLayout layout = new GridLayout();
    layout.numColumns = 2;
    layout.marginHeight = convertVerticalDLUsToPixels(IDialogConstants.VERTICAL_MARGIN);
    layout.marginWidth = convertHorizontalDLUsToPixels(IDialogConstants.HORIZONTAL_MARGIN);
    layout.verticalSpacing = convertVerticalDLUsToPixels(IDialogConstants.VERTICAL_SPACING);
    layout.horizontalSpacing = convertHorizontalDLUsToPixels(IDialogConstants.HORIZONTAL_SPACING);
    composite.setLayout(layout);
    composite.setLayoutData(new GridData(GridData.FILL_BOTH));
    composite.setFont(parentComposite.getFont());

    buildWorkspaceViewer(composite);
    buildFileSection(composite);
    
    return parentComposite;
  }    
  //----------------------------------------------------------------------------
  /**
   * Configures the shell.
   * 
   * @param shell shell to be configured
   * 
   * @author Andreas Brker
   * @date 14.07.2006
   */
  protected void configureShell(Shell shell) {
    super.configureShell(shell);
    shell.setText(Messages.OfficeSaveAsDialog_shell_title);
  }	
  //----------------------------------------------------------------------------
	/**
	 * The OK button was pressed.
	 * 
	 * @author Andreas Brker
   * @date 14.07.2006
	 */
  protected void okPressed() {
  	IPath path = buildPath();    
    if (path.toFile().exists()) {
      String[] buttons = new String[] { IDialogConstants.YES_LABEL,
              IDialogConstants.NO_LABEL, IDialogConstants.CANCEL_LABEL };
      String question = NLS.bind(Messages.OfficeSaveAsDialog_dialog_text_overwrite, path.toString()); 
      MessageDialog messageDialog = new MessageDialog(getShell(),
          Messages.OfficeSaveAsDialog_dialog_title_overwrite,
          null, question, MessageDialog.QUESTION, buttons, 0);
      int overwrite = messageDialog.open();
      switch (overwrite) {
      case 0: // Yes
        break;
      case 1: // No
      	return;
      case 2: // Cancel
      default:
        cancelPressed();
        return;
      }
    }      
    result = path;
    close();
  }
  //---------------------------------------------------------------------------- 
  /**
   * Sets the completion state of this dialog and adjusts the enable state of
   * the Ok button accordingly.
   *
   * @param complete <code>true</code> if this dialog is compelete, and
   *  <code>false</code> otherwise
   * 
   * @author Andreas Brker
   * @date 14.07.2006
   */
  protected void setDialogComplete(boolean complete) {        
  	okButton.setEnabled(complete);
  }
  //----------------------------------------------------------------------------
  /**
   * Builds viewer for the workspace.
   * 
   * @param parent parent to be used
   * 
   * @author Andreas Brker
   * @date 14.07.2006
   */
  private void buildWorkspaceViewer(Composite parent) {
    Tree tree = new Tree(parent, SWT.BORDER);
    GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
    gridData.horizontalSpan = 2;
    gridData.heightHint = (int)(Display.getCurrent().getBounds().height * 0.25);
    tree.setLayoutData(gridData);
    treeViewerWorkspace = new TreeViewer(tree);
    treeViewerWorkspace.setContentProvider(new BaseWorkbenchContentProvider());
    treeViewerWorkspace.setLabelProvider(new WorkbenchLabelProvider());
    treeViewerWorkspace.addFilter(new ViewerFilter() {
			public boolean select(Viewer viewer, Object parentElement, Object element) {
				if(element instanceof IProject || element instanceof IFolder)
					return true;
				else {
					if(currentFilter != null) {
						if(element instanceof IFile) {
							String fileExtension = currentFilter.getFileExtension(document);
							if(fileExtension != null && ((IFile)element).getFileExtension() != null)
								if(fileExtension.equals(((IFile)element).getFileExtension()))
									return true;
						}
					}
					return false;
				}
			}        	
    });
    
    treeViewerWorkspace.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				ISelection selection =  event.getSelection();
				if(selection instanceof IStructuredSelection) {
					Object object = ((IStructuredSelection)selection).getFirstElement();
					if(object instanceof IContainer) {
						String selectedPath = ((IContainer)object).getFullPath().toString();
						if(!selectedPath.equals(textFolderName.getText()) && !textFolderName.getText().endsWith("/") && !textFolderName.getText().endsWith("\\")) { //$NON-NLS-1$ //$NON-NLS-2$
							textFolderName.setText(selectedPath);
							textFolderName.setSelection(textFolderName.getText().length());
							targetContainer = (IContainer)object;									
						}
					}
					else if(object instanceof IFile) {
						String name = ((IFile)object).getName();
						int index = name.indexOf(".");
						if(index != -1) {
							name = name.substring(0, index);
						}
						textFileName.setText(name);
					}
				}	
				checkDialogState();
			}        	
    });
  }
  //----------------------------------------------------------------------------
  /**
   * Builds file section.
   * 
   * @param parent parent to be used
   * 
   * @author Andreas Brker
   * @date 14.07.2006
   */
  private void buildFileSection(Composite parent) {
  	Label labelFolderName = new Label(parent, SWT.NONE);
  	labelFolderName.setText(Messages.OfficeSaveAsDialog_label_text_folder);
    labelFolderName.setBackground(parent.getBackground());
    
    textFolderName = new Text(parent, SWT.BORDER);
    GridData gridData = new GridData(SWT.FILL, SWT.NONE, true, false);
    textFolderName.setLayoutData(gridData);
    textFolderName.addModifyListener(new ModifyListener() {    	
			public void modifyText(ModifyEvent modifyEvent) {									
				String folderName = textFolderName.getText();
    		IResource resource = ResourcesPlugin.getWorkspace().getRoot().findMember(folderName);
    		if(resource != null && resource instanceof IContainer) {
    			targetContainer = (IContainer)resource;
    			treeViewerWorkspace.setSelection(new StructuredSelection(resource));	        			
    		}
    		else {
    			targetContainer = null;
    			checkDialogState();
    		}
			}
    });
    
    Label labelFileName = new Label(parent, SWT.NONE);
    labelFileName.setText(Messages.OfficeSaveAsDialog_label_text_fileName);
    labelFileName.setBackground(parent.getBackground());
    
    textFileName = new Text(parent, SWT.BORDER);
    gridData = new GridData(SWT.FILL, SWT.NONE, true, false);
    textFileName.setLayoutData(gridData);
    textFileName.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent modifyEvent) {
				checkDialogState();
			}    	
    });
    
    Label labelDocumentType = new Label(parent, SWT.NONE);
    labelDocumentType.setText(Messages.OfficeSaveAsDialog_label_text_fileType);
    labelDocumentType.setBackground(parent.getBackground());
    
    comboFileTypes = new Combo(parent, SWT.READ_ONLY);
    gridData = new GridData(SWT.FILL, SWT.NONE, true, false);
    comboFileTypes.setLayoutData(gridData);
    comboFileTypes.addSelectionListener(new SelectionAdapter() {
    	public void widgetSelected(SelectionEvent selectionEvent) {
    		int index = comboFileTypes.getSelectionIndex();
    		if(!currentFilter.equals(filters[index])) {
    			currentFilter = filters[index];
    			treeViewerWorkspace.refresh();
    			checkDialogState();
    		}
    	}
    });
  }
  //----------------------------------------------------------------------------
  /**
   * Initializes the controls. 
   * 
   * @author Andreas Brker
   * @date 14.07.2006
   */
  private void initializeControls() {
  	if(filters.length == 0)
  		return;
  	
  	int index = 0;
		for(int i=0, n=filters.length; i<n; i++) {
			String name = filters[i].getName(document) + " (." + filters[i].getFileExtension(document) + ")"; //$NON-NLS-1$ //$NON-NLS-2$
			comboFileTypes.add(name);
			if(originalFile != null && originalFile.getFileExtension() != null && filters[i].getFileExtension(document) != null)
				if(originalFile.getFileExtension().equals(filters[i].getFileExtension(document)))
					index = i;
		}
		comboFileTypes.select(index);
		currentFilter = filters[index];
		treeViewerWorkspace.setInput(ResourcesPlugin.getWorkspace().getRoot());
		
		if(originalFile != null) {
			IFile file = ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(originalFile);
			if(file != null) {
				IContainer container = file.getParent();
			 treeViewerWorkspace.setSelection(new StructuredSelection(container));
			}
		}		
    
		textFileName.setFocus();
	}
  //----------------------------------------------------------------------------
  /**
   * Checks state of the dialog. 
   * 
   * @author Andreas Brker
   * @date 14.07.2006
   */
  private void checkDialogState() {
  	if(currentFilter == null) {
  		setDialogComplete(false);
  		return;
  	}  		
  	
  	if(targetContainer == null) {
  		setMessage(Messages.OfficeSaveAsDialog_error_message_folderNotAvailable, IMessageProvider.ERROR);
  		setDialogComplete(false);
  		return;
  	}
  	
  	String fileName = textFileName.getText();
  	if(fileName.length() == 0) {
  		setMessage(DEFAULT_MESSAGE);
  		setDialogComplete(false);
  		return;
  	}
  		
  	char[] chars = fileName.toCharArray();
    for(int i=0; i< chars.length; i++) {
      if(chars[i] == '\\' || 
          chars[i] == '/' || 
          chars[i] == ':' ||
          chars[i] == '*' ||
          chars[i] == '?' ||
          chars[i] == '"' ||
          chars[i] == '<' ||
          chars[i] == '>' ||
          chars[i] == '|') {
      	setMessage(Messages.OfficeSaveAsDialog_error_message_invalidCharacter, IMessageProvider.ERROR);
    		setDialogComplete(false);
    		return;
      }
    }
    
    if(originalFile != null) {
    	if(originalFile.equals(buildPath())) {
    		setDialogComplete(false);
    		return;
    	}
    }
    
    setMessage(DEFAULT_MESSAGE);
    setDialogComplete(true);
  }
  //----------------------------------------------------------------------------
  /**
   * Builds path for the new file.
   * 
   * @return path for the new file
   * 
   * @author Andreas Brker
   * @date 14.07.2006
   */
  private IPath buildPath() {
  	IPath path = targetContainer.getLocation();
  	String fileName = textFileName.getText();
  	path = path.append(fileName);

    String fileExtension = path.getFileExtension();
    if(fileExtension == null)
    	path = path.addFileExtension(currentFilter.getFileExtension(document));
    else {
    	if(!fileExtension.equals(currentFilter.getFileExtension(document))) {
    		path = path.removeFileExtension();
    		path = path.addFileExtension(currentFilter.getFileExtension(document));
    	}
    }
    return path;
  }
  //----------------------------------------------------------------------------
  
}