/****************************************************************************
 *                                                                          *
 * NOA (Nice Office Access)                                     						*
 * ------------------------------------------------------------------------ *
 *                                                                          *
 * The Contents of this file are made available subject to                  *
 * the terms of GNU Lesser General Public License Version 2.1.              *
 *                                                                          * 
 * GNU Lesser General Public License Version 2.1                            *
 * ======================================================================== *
 * Copyright 2003-2006 by IOn AG                                            *
 *                                                                          *
 * This library is free software; you can redistribute it and/or            *
 * modify it under the terms of the GNU Lesser General Public               *
 * License version 2.1, as published by the Free Software Foundation.       *
 *                                                                          *
 * This library is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        *
 * Lesser General Public License for more details.                          *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public         *
 * License along with this library; if not, write to the Free Software      *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,                    *
 * MA  02111-1307  USA                                                      *
 *                                                                          *
 * Contact us:                                                              *
 *  http://www.ion.ag																												*
 *  http://ubion.ion.ag                                                     *
 *  info@ion.ag                                                             *
 *                                                                          *
 ****************************************************************************/
 
/*
 * Last changes made by $Author: andreas $, $Date: 2006-08-07 13:59:32 +0200 (Mo, 07 Aug 2006) $
 */
package ag.ion.noa4e.editor.ui.views;

import ag.ion.noa4e.ui.widgets.OfficePanel;

import ag.ion.bion.officelayer.document.DocumentDescriptor;

import ag.ion.bion.workbench.office.editor.ui.EditorUIPlugin;

import ag.ion.bion.workbench.office.editor.ui.wizards.document.NewDrawDocumentWizard;
import ag.ion.bion.workbench.office.editor.ui.wizards.document.NewFormulaDocumentWizard;
import ag.ion.bion.workbench.office.editor.ui.wizards.document.NewGlobalDocumentWizard;
import ag.ion.bion.workbench.office.editor.ui.wizards.document.NewPresentationDocumentWizard;
import ag.ion.bion.workbench.office.editor.ui.wizards.document.NewSpreadsheetDocumentWizard;
import ag.ion.bion.workbench.office.editor.ui.wizards.document.NewTextDocumentWizard;
import ag.ion.bion.workbench.office.editor.ui.wizards.document.NewWebDocumentWizard;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;

import org.eclipse.swt.SWT;

import org.eclipse.swt.widgets.Composite;

import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;

import org.eclipse.ui.part.ViewPart;

import org.eclipse.core.resources.IFile;

/**
 * View in order to show the preview
 * of a document.
 * 
 * @author Andreas Brker
 * @version $Revision: 9209 $
 * @date 28.06.2006
 */ 
public class PreviewView extends ViewPart implements ISelectionListener {

	private OfficePanel officePanel = null;
	
	private IWorkbenchPart workbenchPart = null;

  //----------------------------------------------------------------------------
	/**
	 * Internal customized office panel.
	 * 
	 * @author Andreas Brker
	 * @date 28.06.2006
	 */
	private class PreviewOfficePanel extends OfficePanel {

	  //----------------------------------------------------------------------------
		/**
		 * Constructs new PreviewOfficePanel.
		 * 
		 * @param parent parent to be used
		 * @param style style to be used
		 * 
		 * @author Andreas Brker
		 * @date 28.06.2006
		 */
		public PreviewOfficePanel(Composite parent, int style) {
			super(parent, style);
		}
	  //----------------------------------------------------------------------------
		/**
		 * Is called after a document loading operation was done. 
		 * 
		 * @author Andreas Brker
		 * @date 28.06.2006
		 */
		protected void documentLoadingOperationDone() {
			if(workbenchPart != null) {
				workbenchPart.getSite().getShell().getDisplay().asyncExec(new Runnable() {
					public void run() {
						try {
							Thread.sleep(50);
						}
						catch(Throwable throwable) {								
						}
						workbenchPart.setFocus();
					}
				});				
			}
			super.documentLoadingOperationDone();
		}
	  //----------------------------------------------------------------------------
		
	}
  //----------------------------------------------------------------------------
	
  //----------------------------------------------------------------------------
  /**
   * Creates the SWT controls for this workbench part.
   *
   * @param parent the parent control
   * 
   * @author Andreas Brker
	 * @date 28.06.2006
   */
	public void createPartControl(Composite parent) {
		officePanel = new PreviewOfficePanel(parent, SWT.NULL);
		EditorUIPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getSelectionService().addPostSelectionListener(this);
	}
  //----------------------------------------------------------------------------
  /**
   * Asks this part to take focus within the workbench.
   * 
   * @author Andreas Brker
	 * @date 28.06.2006
   */
	public void setFocus() {
		officePanel.setFocus();
	}
  //----------------------------------------------------------------------------
  /**
   * Notifies this listener that the selection has changed.
   * <p>
   * This method is called when the selection changes from one to a 
   * <code>non-null</code> value, but not when the selection changes to 
   * <code>null</code>. If there is a requirement to be notified in the latter 
   * scenario, implement <code>INullSelectionListener</code>. The event will
   * be posted through this method.
   * </p>
   *
   * @param part the workbench part containing the selection
   * @param selection the current selection. This may be <code>null</code> 
   * 		if <code>INullSelectionListener</code> is implemented
   * 
   * @author Andreas Brker
	 * @date 28.06.2006
   */
	public void selectionChanged(final IWorkbenchPart part, ISelection selection) {
		if(!selection.isEmpty()) {
			if(selection instanceof IStructuredSelection) {
				IStructuredSelection structuredSelection = (IStructuredSelection)selection;
				Object selectedObject = structuredSelection.getFirstElement();
				if(selectedObject instanceof IFile) {
					IFile file = (IFile)selectedObject;
					if(isValidExtension(file.getFileExtension())) {
						String filePath = file.getLocation().toOSString();
						DocumentDescriptor documentDescriptor = new DocumentDescriptor();
						documentDescriptor.setAsPreview(true);
						documentDescriptor.setReadOnly(true);
						workbenchPart = part;
						officePanel.loadDocument(true, filePath, documentDescriptor);	
					}
				}				
			}
		}
	}
  //----------------------------------------------------------------------------
	/**
	 * Disposes the preview view.
	 * 
	 * @author Andreas Brker
	 * @date 28.06.2006
	 */
	public void dispose() {
		officePanel.dispose();
		EditorUIPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getSelectionService().removePostSelectionListener(this);
		super.dispose();
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns information whether the submitted
	 * extension is valid.
	 * 
	 * @param extension extension to be checked
	 * 
	 * @return information whether the submitted
	 * extension is valid
	 * 
	 * @author Andreas Brker
	 * @date 16.07.2006
	 */
	private boolean isValidExtension(String extension) {
		if(extension == null)
			return false;
		for(int i=0, n=NewTextDocumentWizard.VALID_TEMPLATE_EXTENSIONS.length; i<n; i++) {
			if(extension.toLowerCase().equals(NewTextDocumentWizard.VALID_TEMPLATE_EXTENSIONS[i]))
				return true;					
		}
		for(int i=0, n=NewSpreadsheetDocumentWizard.VALID_TEMPLATE_EXTENSIONS.length; i<n; i++) {
			if(extension.toLowerCase().equals(NewSpreadsheetDocumentWizard.VALID_TEMPLATE_EXTENSIONS[i]))
				return true;					
		}
		for(int i=0, n=NewPresentationDocumentWizard.VALID_TEMPLATE_EXTENSIONS.length; i<n; i++) {
			if(extension.toLowerCase().equals(NewPresentationDocumentWizard.VALID_TEMPLATE_EXTENSIONS[i]))
				return true;					
		}
		for(int i=0, n=NewDrawDocumentWizard.VALID_TEMPLATE_EXTENSIONS.length; i<n; i++) {
			if(extension.toLowerCase().equals(NewDrawDocumentWizard.VALID_TEMPLATE_EXTENSIONS[i]))
				return true;					
		}
		for(int i=0, n=NewWebDocumentWizard.VALID_TEMPLATE_EXTENSIONS.length; i<n; i++) {
			if(extension.toLowerCase().equals(NewWebDocumentWizard.VALID_TEMPLATE_EXTENSIONS[i]))
				return true;					
		}
		for(int i=0, n=NewFormulaDocumentWizard.VALID_TEMPLATE_EXTENSIONS.length; i<n; i++) {
			if(extension.toLowerCase().equals(NewFormulaDocumentWizard.VALID_TEMPLATE_EXTENSIONS[i]))
				return true;					
		}
		for(int i=0, n=NewGlobalDocumentWizard.VALID_TEMPLATE_EXTENSIONS.length; i<n; i++) {
			if(extension.toLowerCase().equals(NewGlobalDocumentWizard.VALID_TEMPLATE_EXTENSIONS[i]))
				return true;					
		}
		return false;
	}
  //----------------------------------------------------------------------------

}