/****************************************************************************
 *                                                                          *
 * NOA (Nice Office Access)                                     						*
 * ------------------------------------------------------------------------ *
 *                                                                          *
 * The Contents of this file are made available subject to                  *
 * the terms of GNU Lesser General Public License Version 2.1.              *
 *                                                                          * 
 * GNU Lesser General Public License Version 2.1                            *
 * ======================================================================== *
 * Copyright 2003-2006 by IOn AG                                            *
 *                                                                          *
 * This library is free software; you can redistribute it and/or            *
 * modify it under the terms of the GNU Lesser General Public               *
 * License version 2.1, as published by the Free Software Foundation.       *
 *                                                                          *
 * This library is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        *
 * Lesser General Public License for more details.                          *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public         *
 * License along with this library; if not, write to the Free Software      *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,                    *
 * MA  02111-1307  USA                                                      *
 *                                                                          *
 * Contact us:                                                              *
 *  http://www.ion.ag																												*
 *  http://ubion.ion.ag                                                     *
 *  info@ion.ag                                                             *
 *                                                                          *
 ****************************************************************************/
 
/*
 * Last changes made by $Author: andreas $, $Date: 2006-08-07 13:09:58 +0200 (Mo, 07 Aug 2006) $
 */
package ag.ion.noa4e.internal.search.core;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceProxy;
import org.eclipse.core.resources.IResourceProxyVisitor;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.MultiStatus;

/**
 * Calcluator for the search effort. This class is nearly a mirror
 * of the class org.eclipse.search.internal.core.text.AmountOfWorkCalculator.
 * 
 * @author Andreas Brker
 * @version $Revision: 9195 $
 * @date 10.07.2006
 */ 
public class EffortCalculator implements IResourceProxyVisitor {
	
	private SearchScope 	searchScope = null;
	private MultiStatus 	status			= null;
	
	private int numberOfFilesToScan = 0;
	
  //----------------------------------------------------------------------------
	/**
	 * Constructs new EffortCalculator.
	 * 
	 * @param searchScope search scope to be used
	 * @param status status to be used
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	public EffortCalculator(SearchScope searchScope, MultiStatus status) {
		assert searchScope != null;
		assert status != null;
		
		this.status = status;
		this.searchScope = searchScope;
	}
  //----------------------------------------------------------------------------
	/** 
	 * Visits the given resource.
	 *
	 * @param proxy for requesting information about the resource being visited;
	 * this object is only valid for the duration of the invocation of this
	 * method, and must not be used after this method has completed
	 * 
	 * @return <code>true</code> if the resource's members should
	 *		be visited; <code>false</code> if they should be skipped
	 *
	 * @exception CoreException if the visit fails for some reason
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	public boolean visit(IResourceProxy proxy) {
		if(proxy.getType() != IResource.FILE) {
			return true;
		}
		
		if(searchScope.matchesFileName(proxy.getName())) {
			numberOfFilesToScan++;
		}
		return true;	
	}
  //----------------------------------------------------------------------------
	/**
	 * Calculates the number of file to be scan.
	 * 
	 * @return number of file to be scan
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	public int calculateEffort() {
		IResource[] roots = searchScope.getRootElements();
		for(int i= 0; i < roots.length; i++) {
			try {
				roots[i].accept(this, 0);
			} 
			catch (CoreException coreException) {
				status.add(coreException.getStatus());
			}
		}
		return numberOfFilesToScan;
	}
  //----------------------------------------------------------------------------
	
}