/****************************************************************************
 *                                                                          *
 * NOA (Nice Office Access)                                     						*
 * ------------------------------------------------------------------------ *
 *                                                                          *
 * The Contents of this file are made available subject to                  *
 * the terms of GNU Lesser General Public License Version 2.1.              *
 *                                                                          * 
 * GNU Lesser General Public License Version 2.1                            *
 * ======================================================================== *
 * Copyright 2003-2006 by IOn AG                                            *
 *                                                                          *
 * This library is free software; you can redistribute it and/or            *
 * modify it under the terms of the GNU Lesser General Public               *
 * License version 2.1, as published by the Free Software Foundation.       *
 *                                                                          *
 * This library is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        *
 * Lesser General Public License for more details.                          *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public         *
 * License along with this library; if not, write to the Free Software      *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,                    *
 * MA  02111-1307  USA                                                      *
 *                                                                          *
 * Contact us:                                                              *
 *  http://www.ion.ag																												*
 *  http://ubion.ion.ag                                                     *
 *  info@ion.ag                                                             *
 *                                                                          *
 ****************************************************************************/
 
/*
 * Last changes made by $Author: andreas $, $Date: 2006-08-07 13:09:58 +0200 (Mo, 07 Aug 2006) $
 */
package ag.ion.noa4e.internal.search.core;

import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

/**
 * Constructor for patterns. This class is nearly a mirror
 * of the class org.eclipse.search.internal.core.text.PatternConstructor.
 * 
 * @author Andreas Brker
 * @version $Revision: 9195 $
 * @date 10.07.2006
 */ 
public class PatternConstructor {
	
  //----------------------------------------------------------------------------
	/**
	 * Creates a pattern element from the pattern string which is either a reg-ex expression or in our old
	 * 'StringMatcher' format.
	 * 
	 * @param pattern search pattern to be used
	 * @param isCaseSensitive set to <code>true</code> to create a case insensitve pattern
	 * @param isRegexSearch <code>true</code> if the passed string already is a reg-ex pattern
	 * 
	 * @return the created pattern
	 * 
	 * @throws PatternSyntaxException if the pattern can not be created
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	public static Pattern createPattern(String pattern, boolean isCaseSensitive, boolean isRegexSearch) throws PatternSyntaxException {
		if (!isRegexSearch)
			pattern = asRegEx(pattern);
		
		if (!isCaseSensitive)
			return Pattern.compile(pattern, Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE | Pattern.MULTILINE);
		
		return Pattern.compile(pattern, Pattern.MULTILINE);
	}
  //----------------------------------------------------------------------------
	/**
	 * Converts '*' and '?' to regEx variables.
	 * 
	 * @return converted '*' and '?' as regEx variables
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	private static String asRegEx(String pattern) {		
		StringBuffer out = new StringBuffer(pattern.length());		
		boolean escaped = false;
		boolean quoting = false;
		
		int i= 0;
		while (i < pattern.length()) {
			char ch= pattern.charAt(i++);			
			if (ch == '*' && !escaped) {
				if (quoting) {
					out.append("\\E"); //$NON-NLS-1$
					quoting= false;
				}
				out.append(".*"); //$NON-NLS-1$
				escaped= false;
				continue;
			} 
			else if (ch == '?' && !escaped) {
				if (quoting) {
					out.append("\\E"); //$NON-NLS-1$
					quoting= false;
				}
				out.append("."); //$NON-NLS-1$
				escaped= false;
				continue;
			} 
			else if (ch == '\\' && !escaped) {
				escaped= true;
				continue;								
				
			} 
			else if (ch == '\\' && escaped) {
				escaped= false;
				if (quoting) {
					out.append("\\E"); //$NON-NLS-1$
					quoting= false;
				}
				out.append("\\\\"); //$NON-NLS-1$
				continue;								
			}
			
			if (!quoting) {
				out.append("\\Q"); //$NON-NLS-1$
				quoting= true;
			}
			if (escaped && ch != '*' && ch != '?' && ch != '\\')
				out.append('\\');
			out.append(ch);
			escaped= ch == '\\';
			
		}
		if (quoting)
			out.append("\\E"); //$NON-NLS-1$		
		return out.toString();
	}
  //----------------------------------------------------------------------------
	/**
	 * Prevents instatiation.
	 *
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	private PatternConstructor() {
	}
  //----------------------------------------------------------------------------
	
}