/****************************************************************************
 *                                                                          *
 * NOA (Nice Office Access)                                     						*
 * ------------------------------------------------------------------------ *
 *                                                                          *
 * The Contents of this file are made available subject to                  *
 * the terms of GNU Lesser General Public License Version 2.1.              *
 *                                                                          * 
 * GNU Lesser General Public License Version 2.1                            *
 * ======================================================================== *
 * Copyright 2003-2006 by IOn AG                                            *
 *                                                                          *
 * This library is free software; you can redistribute it and/or            *
 * modify it under the terms of the GNU Lesser General Public               *
 * License version 2.1, as published by the Free Software Foundation.       *
 *                                                                          *
 * This library is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        *
 * Lesser General Public License for more details.                          *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public         *
 * License along with this library; if not, write to the Free Software      *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,                    *
 * MA  02111-1307  USA                                                      *
 *                                                                          *
 * Contact us:                                                              *
 *  http://www.ion.ag																												*
 *  http://ubion.ion.ag                                                     *
 *  info@ion.ag                                                             *
 *                                                                          *
 ****************************************************************************/
 
/*
 * Last changes made by $Author: markus $, $Date: 2007-12-10 08:35:43 +0100 (Mo, 10 Dez 2007) $
 */
package ag.ion.noa4e.internal.search.ui;

import ag.ion.bion.officelayer.application.IOfficeApplication;

import ag.ion.bion.workbench.office.editor.core.EditorCorePlugin;

import ag.ion.noa.search.ISearchDescriptor;
import ag.ion.noa.search.SearchDescriptor;

import ag.ion.noa4e.internal.search.core.SearchScope;

import ag.ion.noa4e.search.NOASearchPlugin;

import ag.ion.noa4e.ui.NOAUIPlugin;

import org.eclipse.jface.dialogs.DialogPage;
import org.eclipse.jface.dialogs.IDialogSettings;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;

import org.eclipse.search.ui.ISearchPage;
import org.eclipse.search.ui.ISearchPageContainer;
import org.eclipse.search.ui.ISearchQuery;
import org.eclipse.search.ui.NewSearchUI;

import org.eclipse.swt.SWT;

import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;

import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;

import org.eclipse.ui.IWorkingSet;

import org.eclipse.core.resources.IResource;

import org.eclipse.core.runtime.IAdaptable;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

/**
 * Search page for office documents.
 * 
 * @author Andreas Brker
 * @version $Revision: 11608 $
 * @date 10.07.2006
 */ 
public class OfficeSearchPage extends DialogPage implements ISearchPage {
	
	private static final String SETTING_SECTION_NAME 				= "ag.ion.noa4e.internal.search.ui.OfficeSearchPage";
	private static final String SETTING_SEARCH_CONTENT 			= "searchContent";
	private static final String SETTING_FILE_PATTERNS 			= "filePatterns";
	private static final String SETTING_CASE_SENSITIVE 			= "caseSensitive";
	private static final String SETTING_REGULAR_EXPRESSION 	= "rugularExpression";
	private static final String SETTING_COMPLETE_WORDS 			= "completeWords";
	private static final String SETTING_SIMILARITY_SEARCH 	= "similaritySearch";
	private static final String SETTING_TYPE_OTT 						= "typeOTT";
	private static final String SETTING_TYPE_DOC 						= "typeDOC";
	private static final String SETTING_TYPE_WORD_XML 			= "typeWordXML";

	private ISearchPageContainer searchPageContainer = null;
	
	private Combo comboSearchContent = null;
	private Combo comboFilePatterns	 = null;
	
	private Button buttonCaseSensitive 			= null;
	private Button buttonRegularExpression 	= null;
	private Button buttonCompleteWords 			= null;
	private Button buttonSimilaritySearch		= null;
	
	private Button buttonTypeODT 			= null;
	private Button buttonTypeOTT 			= null;
	private Button buttonTypeDOC 			= null;
	private Button buttonTypeWordXML 	= null;
	
  //----------------------------------------------------------------------------
	/**
	 * Performs the action for this page.
	 * The search dialog calls this method when the Search
	 * button is pressed.
	 *
	 * @return <code>true</code> if the dialog can be closed after execution
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	public boolean performAction() {
		saveSettings();
		NewSearchUI.activateSearchResultView();
		NewSearchUI.runQueryInBackground(getSearchQuery());
		return true;
	}
  //----------------------------------------------------------------------------
	/**
	 * Sets the container of this page.
	 * The search dialog calls this method to initialize this page.
	 * Implementations may store the reference to the container.
	 *
	 * @param	container	the container for this page
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	public void setContainer(ISearchPageContainer searchPageContainer) {
		this.searchPageContainer = searchPageContainer;
	}
  //----------------------------------------------------------------------------
  /**
   * Creates the top level control for this dialog
   * page under the given parent composite.
   * 
   * @param parent the parent composite
   * 
   * @author Andreas Brker
	 * @date 10.07.2006
   */
	public void createControl(Composite parent) {
		Composite composite = new Composite(parent, SWT.NONE);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		composite.setLayout(gridLayout);
		
		Label labelSearchString = new Label(composite, SWT.NONE);
		labelSearchString.setText(Messages.OfficeSearchPage_label_text_searchContent);
		GridData gridData = new GridData(SWT.FILL, SWT.NONE, true, false);
		gridData.horizontalSpan = 2;
		labelSearchString.setLayoutData(gridData);
		
		comboSearchContent = new Combo(composite, SWT.NONE);
		comboSearchContent.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));
		comboSearchContent.addKeyListener(new KeyAdapter() {
			public void keyReleased(KeyEvent keyEvent) {
				checkPageState();
			}
		});
		
		buttonCaseSensitive = new Button(composite, SWT.CHECK);		
		buttonCaseSensitive.setText(Messages.OfficeSearchPage_button_text_caseSensitive);
		
		buildDetailsGroup(composite);
		
		Label labelFilePattern = new Label(composite, SWT.NONE);
		labelFilePattern.setText(Messages.OfficeSearchPage_label_text_documentNamePatterns);
		gridData = new GridData(SWT.FILL, SWT.NONE, true, false);
		gridData.horizontalSpan = 2;
		labelFilePattern.setLayoutData(gridData);
		
		comboFilePatterns = new Combo(composite, SWT.NONE);
		comboFilePatterns.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));
		
		Label labelFilePatternInfo = new Label(composite, SWT.NONE);
		labelFilePatternInfo.setText(Messages.OfficeSearchPage_label_text_patternInfo);
		gridData = new GridData(SWT.FILL, SWT.NONE, true, false);
		gridData.horizontalSpan = 2;
		labelFilePatternInfo.setLayoutData(gridData);
		
		buildDocumentTypeGroup(composite);
		
		restoreSettings();		
		setControl(composite);
	}
  //----------------------------------------------------------------------------
  /**
   * Sets the visibility of this dialog page.
   *
   * @param visible <code>true</code> to make this page visible,
   *  and <code>false</code> to hide it
   * 
   * @author Andreas Brker
	 * @date 10.07.2006
   */
	public void setVisible(boolean visible) {
		if(visible)
			checkPageState();
		super.setVisible(visible);
	}
  //----------------------------------------------------------------------------
	/**
	 * Builds search descriptor.
	 * 
	 * @return builded search descriptor
	 * 
	 * @author Andreas Brker
	 * @date 10.07.2006
	 */
	protected ISearchDescriptor buildSearchDescriptor() {
		String searchContent = comboSearchContent.getText();
		SearchDescriptor searchDescriptor = new SearchDescriptor(searchContent);
		searchDescriptor.setIsCaseSensitive(buttonCaseSensitive.getSelection());
		searchDescriptor.setUseCompleteWords(buttonCompleteWords.getSelection());
		searchDescriptor.setUseRegularExpression(buttonRegularExpression.getSelection());
		searchDescriptor.setUseSimilaritySearch(buttonSimilaritySearch.getSelection());
		return searchDescriptor;
	}
  //----------------------------------------------------------------------------
	/**
	 * Builds search scope.
	 * 
	 * @return builded search scope
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	protected SearchScope buildSearchScope() {
		SearchScope scope = null;
		switch(searchPageContainer.getSelectedScope()) {
			case ISearchPageContainer.SELECTION_SCOPE:
				scope = getSelectedResourcesScope();
				break;
			case ISearchPageContainer.WORKING_SET_SCOPE:
				IWorkingSet[] workingSets = searchPageContainer.getSelectedWorkingSets();
				String description = Messages.OfficeSearchPage_searchScopre_description_workingSets;
				scope = SearchScope.newSearchScope(description, workingSets);
        break;
      default:
          scope = SearchScope.newWorkspaceScope();
		}	

		scope.addFileNamePatterns(getFileNamePatterns());	
    scope.addFileExtensions(getFileExtensions());
		return scope;
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns office application.
	 * 
	 * @return office application
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	protected IOfficeApplication getOfficeApplication() {
		IOfficeApplication officeApplication = EditorCorePlugin.getDefault().getManagedLocalOfficeApplication();
		NOAUIPlugin.startLocalOfficeApplication(Display.getCurrent().getActiveShell(), officeApplication);
		return officeApplication;
	}
  //----------------------------------------------------------------------------
	/**
	 * Builds group for search details.
	 * 
	 * @param parent parent to be used
	 * 
	 * @author Andreas Brker
	 * @date 11.07.2006
	 */
	private void buildDetailsGroup(Composite parent) {
		Group groupDetails = new Group(parent, SWT.FLAT);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 3;
		groupDetails.setLayout(gridLayout);
		GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
		gridData.horizontalSpan = 2;
		groupDetails.setLayoutData(gridData);
		groupDetails.setText(Messages.OfficeSearchPage_group_text_details);
		
		buttonRegularExpression = new Button(groupDetails, SWT.CHECK);		
		buttonRegularExpression.setText(Messages.OfficeSearchPage_button_text_regularExpression);
		buttonRegularExpression.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent selectionEvent) {
				if(buttonRegularExpression.getSelection()) {
					buttonCompleteWords.setSelection(false);
					buttonCompleteWords.setEnabled(false);
					buttonSimilaritySearch.setSelection(false);
					buttonSimilaritySearch.setEnabled(false);
				}
				else {
					buttonCompleteWords.setEnabled(true);
					buttonSimilaritySearch.setEnabled(true);
				}
			}
		});
		
		buttonCompleteWords = new Button(groupDetails, SWT.CHECK);		
		buttonCompleteWords.setText(Messages.OfficeSearchPage_button_text_wholeWords);
		
		buttonSimilaritySearch = new Button(groupDetails, SWT.CHECK);		
		buttonSimilaritySearch.setText(Messages.OfficeSearchPage_button_text_similaritySearch);
		buttonSimilaritySearch.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent selectionEvent) {
				if(buttonSimilaritySearch.getSelection()) {
					buttonRegularExpression.setSelection(false);
					buttonRegularExpression.setEnabled(false);
				}
				else {
					buttonRegularExpression.setEnabled(true);
				}
			}
		});
	}
  //----------------------------------------------------------------------------
	/**
	 * Builds group for document types.
	 * 
	 * @param parent parent to be used
	 * 
	 * @author Andreas Brker
	 * @date 11.07.2006
	 */
	private void buildDocumentTypeGroup(Composite parent) {
		Group groupDocumentTypes = new Group(parent, SWT.FLAT);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		groupDocumentTypes.setLayout(gridLayout);
		GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
		gridData.horizontalSpan = 2;
		groupDocumentTypes.setLayoutData(gridData);
		groupDocumentTypes.setText(Messages.OfficeSearchPage_group_text_documentTypes);
		
		buttonTypeODT = new Button(groupDocumentTypes, SWT.CHECK);
		buttonTypeODT.setText("Open Document Text");
		buttonTypeODT.setSelection(true);
		buttonTypeODT.setEnabled(false);
		
		buttonTypeOTT = new Button(groupDocumentTypes, SWT.CHECK);
		buttonTypeOTT.setText("Open Template Text");
		buttonTypeOTT.setSelection(true);
		
		buttonTypeDOC = new Button(groupDocumentTypes, SWT.CHECK);
		buttonTypeDOC.setText("Microsoft Word");
		buttonTypeDOC.setSelection(true);
		
		buttonTypeWordXML = new Button(groupDocumentTypes, SWT.CHECK);
		buttonTypeWordXML.setText("Microsoft Word XML");
		buttonTypeWordXML.setSelection(true);
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns file name patterns.
	 * 
	 * @return file name patterns
	 * 
	 * @author Andreas Brker
	 * @date 11.07.2006
	 */
	private String[] getFileNamePatterns() {
		String fileNamePatternsComplete = comboFilePatterns.getText();
		if(fileNamePatternsComplete.length() == 0)
			return new String[0];
		
		List list = new ArrayList();
		String[] fileNamePatterns = fileNamePatternsComplete.split(","); //$NON-NLS-1$
		for(int i=0, n=fileNamePatterns.length; i<n; i++) {
			String fileNamePattern = fileNamePatterns[i];
			int index = fileNamePattern.indexOf("."); //$NON-NLS-1$
			if(index != -1) {
				fileNamePattern = fileNamePattern.substring(0, index);
			}
			if(!list.contains(fileNamePattern))
				list.add(fileNamePattern);
		}
		
		return (String[])list.toArray(new String[list.size()]);
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns file extensions.
	 * 
	 * @return file extensions
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	private String[] getFileExtensions() {
		List list = new ArrayList();
		if(buttonTypeODT.getSelection())
			list.add("odt"); //$NON-NLS-1$
		if(buttonTypeOTT.getSelection())
			list.add("ott");
		if(buttonTypeDOC.getSelection())
			list.add("doc"); //$NON-NLS-1$
		if(buttonTypeWordXML.getSelection())
			list.add("xml"); //$NON-NLS-1$
		return (String[])list.toArray(new String[list.size()]);
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns search query.
	 * 
	 * @return search query
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	private ISearchQuery getSearchQuery() {
		SearchScope searchScope = buildSearchScope();
		ISearchDescriptor searchDescriptor = buildSearchDescriptor();
		return new OfficeSearchQuery(getOfficeApplication(), searchScope, searchDescriptor);
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns selection.
	 *  
	 * @return selection
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	private ISelection getSelection() {
		return searchPageContainer.getSelection();
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns search scope for selected resources.
	 * 
	 * @return search scope for selected resources
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	private SearchScope getSelectedResourcesScope() {
		HashSet resources = new HashSet();
		ISelection selection = getSelection();
		if(selection instanceof IStructuredSelection && !selection.isEmpty()) {
			Iterator iterator = ((IStructuredSelection) selection).iterator();
			while(iterator.hasNext()) {
				Object object = iterator.next();
				IResource resource= null;			
				if(object instanceof IResource) {
					resource = (IResource)object;
				} 
				else if(object instanceof IAdaptable) {
					resource =(IResource)((IAdaptable)object).getAdapter(IResource.class);
				}
				
				if(resource != null) {
					resources.add(resource);
				}
			}
		}		
		IResource[] selectedResources = (IResource[]) resources.toArray(new IResource[resources.size()]);
		return SearchScope.newSearchScope(Messages.OfficeSearchPage_searchScope_description_resources, selectedResources);
	}
  //----------------------------------------------------------------------------
	/**
	 * Checks state of the page. 
	 * 
	 * @author Andreas Brker
	 * @date 14.07.2006
	 */
	private void checkPageState() {
		if(comboSearchContent.getText().length() == 0) {
			searchPageContainer.setPerformActionEnabled(false);
			return;
		}
		
		searchPageContainer.setPerformActionEnabled(true);			
	}	
  //----------------------------------------------------------------------------
	/**
	 * Saves settings of the dialog. 
	 * 
	 * @author Andreas Brker
	 * @date 14.07.2006
	 */
	private void saveSettings() {
		IDialogSettings dialogSettings = NOASearchPlugin.getDefault().getDialogSettings();
		IDialogSettings section = dialogSettings.getSection(SETTING_SECTION_NAME);
		if(section == null)
			section = dialogSettings.addNewSection(SETTING_SECTION_NAME);
		saveItems(section, SETTING_SEARCH_CONTENT, comboSearchContent);	
		saveItems(section, SETTING_FILE_PATTERNS, comboFilePatterns);	
		saveDetails(section);
		saveDocumentTypes(section);
	}
  //----------------------------------------------------------------------------
	/**
	 * Saves items.
	 * 
	 * @param dialogSettings dialog settings to be used
	 * @param key setting key to be used
	 * @param combo combo widget to be used
	 * 
	 * @author Andreas Brker
	 * @date 14.07.2006
	 */
	private void saveItems(IDialogSettings dialogSettings, String key, Combo combo) {
		String[] items = combo.getItems();
		List list = new ArrayList();
		for(int i=0, n=items.length; i<n; i++) {
			list.add(items[i]);
			if(i == 9)
				break;
		}
		String currentItem = combo.getText();
		if(!list.contains(currentItem))
			list.add(currentItem);
		dialogSettings.put(key, (String[])list.toArray(new String[list.size()]));
	}
  //----------------------------------------------------------------------------
	/**
	 * Saves search details.
	 * 
	 * @param dialogSettings dialog settings to be used
	 * 
	 * @author Andreas Brker
	 * @date 14.07.2006
	 */
	private void saveDetails(IDialogSettings dialogSettings) {
		dialogSettings.put(SETTING_CASE_SENSITIVE, buttonCaseSensitive.getSelection());
		dialogSettings.put(SETTING_COMPLETE_WORDS, buttonCompleteWords.getSelection());
		dialogSettings.put(SETTING_REGULAR_EXPRESSION, buttonRegularExpression.getSelection());
		dialogSettings.put(SETTING_SIMILARITY_SEARCH, buttonSimilaritySearch.getSelection());
	}
  //----------------------------------------------------------------------------
	/**
	 * Saves document types.
	 * 
	 * @param dialogSettings dialog settings to be used
	 * 
	 * @author Andreas Brker
	 * @date 14.07.2006
	 */
	private void saveDocumentTypes(IDialogSettings dialogSettings) {
		dialogSettings.put(SETTING_TYPE_OTT, buttonTypeOTT.getSelection());
		dialogSettings.put(SETTING_TYPE_DOC, buttonTypeDOC.getSelection());
		dialogSettings.put(SETTING_TYPE_WORD_XML, buttonTypeWordXML.getSelection());
	}
  //----------------------------------------------------------------------------
	/**
	 * Restores dialog settings. 
	 * 
	 * @author Andreas Brker
	 * @date 14.07.2006
	 */
	private void restoreSettings() {
		IDialogSettings dialogSettings = NOASearchPlugin.getDefault().getDialogSettings();
		IDialogSettings section = dialogSettings.getSection(SETTING_SECTION_NAME);
		if(section != null) {
			String[] searchContents = section.getArray(SETTING_SEARCH_CONTENT);
			if(searchContents != null)
				comboSearchContent.setItems(searchContents);
			String[] filePatterns = section.getArray(SETTING_FILE_PATTERNS);
			if(filePatterns != null)
				comboFilePatterns.setItems(filePatterns);
			buttonCaseSensitive.setSelection(section.getBoolean(SETTING_CASE_SENSITIVE));
			buttonCompleteWords.setSelection(section.getBoolean(SETTING_COMPLETE_WORDS));
			buttonRegularExpression.setSelection(section.getBoolean(SETTING_REGULAR_EXPRESSION));
			buttonSimilaritySearch.setSelection(section.getBoolean(SETTING_SIMILARITY_SEARCH));
			
			buttonTypeOTT.setSelection(section.getBoolean(SETTING_TYPE_OTT));
			buttonTypeDOC.setSelection(section.getBoolean(SETTING_TYPE_DOC));
			buttonTypeWordXML.setSelection(section.getBoolean(SETTING_TYPE_WORD_XML));
		}
	}	
  //----------------------------------------------------------------------------

}