/****************************************************************************
 *                                                                          *
 * NOA (Nice Office Access)                                     						*
 * ------------------------------------------------------------------------ *
 *                                                                          *
 * The Contents of this file are made available subject to                  *
 * the terms of GNU Lesser General Public License Version 2.1.              *
 *                                                                          * 
 * GNU Lesser General Public License Version 2.1                            *
 * ======================================================================== *
 * Copyright 2003-2006 by IOn AG                                            *
 *                                                                          *
 * This library is free software; you can redistribute it and/or            *
 * modify it under the terms of the GNU Lesser General Public               *
 * License version 2.1, as published by the Free Software Foundation.       *
 *                                                                          *
 * This library is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        *
 * Lesser General Public License for more details.                          *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public         *
 * License along with this library; if not, write to the Free Software      *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,                    *
 * MA  02111-1307  USA                                                      *
 *                                                                          *
 * Contact us:                                                              *
 *  http://www.ion.ag																												*
 *  http://ubion.ion.ag                                                     *
 *  info@ion.ag                                                             *
 *                                                                          *
 ****************************************************************************/
 
/*
 * Last changes made by $Author: andreas $, $Date: 2006-08-07 13:09:58 +0200 (Mo, 07 Aug 2006) $
 */
package ag.ion.noa4e.internal.search.ui;

import ag.ion.bion.officelayer.application.IOfficeApplication;

import ag.ion.noa.search.ISearchDescriptor;

import ag.ion.noa4e.internal.search.core.EffortCalculator;
import ag.ion.noa4e.internal.search.core.OfficeSearchVisitor;
import ag.ion.noa4e.internal.search.core.SearchScope;

import ag.ion.noa4e.search.NOASearchPlugin;

import org.eclipse.search.ui.ISearchQuery;
import org.eclipse.search.ui.ISearchResult;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.OperationCanceledException;

/**
 * Search query for office documents.
 * 
 * @author Andreas Brker
 * @version $Revision: 9195 $
 * @date 12.07.2006
 */ 
public class OfficeSearchQuery implements ISearchQuery {

	private SearchScope 				searchScope 			= null;
	private ISearchDescriptor 	searchDescriptor 	= null;
	private IOfficeApplication 	officeApplication = null;
	
	private OfficeSearchResult officeSearchResult = null;
	
  //----------------------------------------------------------------------------
	/**
	 * Constructs new OfficeSearchQuery.
	 * 
	 * @param officeApplication office application to be used
	 * @param searchScope search scope to be used
	 * @param searchDescriptor search descriptor to be used
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public OfficeSearchQuery(IOfficeApplication officeApplication, SearchScope searchScope, ISearchDescriptor searchDescriptor) {
		assert officeApplication != null;
		assert searchScope != null;
		assert searchDescriptor != null;
		
		this.officeApplication = officeApplication;
		this.searchScope = searchScope;
		this.searchDescriptor = searchDescriptor;
		
		officeSearchResult = new OfficeSearchResult(this);
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns search descriptor.
	 * 
	 * @return search descriptor
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public ISearchDescriptor getSearchDescriptor() {
		return searchDescriptor;
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns search scope.
	 * 
	 * @return search scope
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public SearchScope getSearchScope() {
		return searchScope;
	}
  //----------------------------------------------------------------------------
	/**
	 * This is the method that actually does the work, i.e. finds the results of
	 * the search query.
	 * 
	 * @param monitor the progress monitor to be used
	 * 
	 * @return the status after completion of the search job 
	 * 
	 * @throws OperationCanceledException Thrown when the search query has been canceled
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public IStatus run(IProgressMonitor progressMonitor) throws OperationCanceledException {
		MultiStatus status = new MultiStatus(NOASearchPlugin.PLUGIN_ID, IStatus.OK, Messages.OfficeSearchQuery_status_message_problemDuringSearch, null);
		EffortCalculator effortCalculator = new EffortCalculator(searchScope, status);
		int numberOfFilesToScan = effortCalculator.calculateEffort();
		
		OfficeSearchVisitor officeSearchVisitor = new OfficeSearchVisitor(officeApplication, searchScope, 
				searchDescriptor, officeSearchResult, status, numberOfFilesToScan);
		officeSearchVisitor.process(progressMonitor);
		
		return status;
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns a user readable label for this query. This will be used, for
	 * example to set the <code>Job</code> name if this query is executed in
	 * the background. Note that progress notification (for example, the number
	 * of matches found) should be done via the progress monitor passed into the
	 * <code>run(IProgressMonitor)</code> method
	 * 
	 * @return the user readable label of this query
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public String getLabel() {
		return Messages.OfficeSearchQuery_label_officeSearchQuery;
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns whether the query can be run more than once. Some queries may
	 * depend on transient information and return <code>false</code>.
	 * 
	 * @return whether this query can be run more than once
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public boolean canRerun() {
		return true;
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns whether this query can be run in the background. Note that
	 * queries must do proper locking when they are run in the background (e.g.
	 * get the appropriate workspace locks).
	 * 
	 * @return whether this query can be run in the background
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public boolean canRunInBackground() {
		return true;
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns the search result associated with this query. This method can be
	 * called before run is called.
	 * 
	 * @return this query's search result
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public ISearchResult getSearchResult() {
		return officeSearchResult;
	}	
  //----------------------------------------------------------------------------
	
}