/****************************************************************************
 *                                                                          *
 * NOA (Nice Office Access)                                     						*
 * ------------------------------------------------------------------------ *
 *                                                                          *
 * The Contents of this file are made available subject to                  *
 * the terms of GNU Lesser General Public License Version 2.1.              *
 *                                                                          * 
 * GNU Lesser General Public License Version 2.1                            *
 * ======================================================================== *
 * Copyright 2003-2006 by IOn AG                                            *
 *                                                                          *
 * This library is free software; you can redistribute it and/or            *
 * modify it under the terms of the GNU Lesser General Public               *
 * License version 2.1, as published by the Free Software Foundation.       *
 *                                                                          *
 * This library is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        *
 * Lesser General Public License for more details.                          *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public         *
 * License along with this library; if not, write to the Free Software      *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,                    *
 * MA  02111-1307  USA                                                      *
 *                                                                          *
 * Contact us:                                                              *
 *  http://www.ion.ag																												*
 *  http://ubion.ion.ag                                                     *
 *  info@ion.ag                                                             *
 *                                                                          *
 ****************************************************************************/
 
/*
 * Last changes made by $Author: andreas $, $Date: 2006-08-07 13:09:58 +0200 (Mo, 07 Aug 2006) $
 */
package ag.ion.noa4e.internal.search.ui;

import ag.ion.noa4e.internal.search.core.IOfficeSearchMatch;
import ag.ion.noa4e.search.SearchUIPluginImages;

import org.eclipse.jface.resource.ImageDescriptor;

import org.eclipse.osgi.util.NLS;

import org.eclipse.search.ui.ISearchQuery;
import org.eclipse.search.ui.ISearchResult;
import org.eclipse.search.ui.ISearchResultListener;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Search result for office documents.
 * 
 * @author Andreas Brker
 * @version $Revision: 9195 $
 * @date 12.07.2006
 */ 
public class OfficeSearchResult implements ISearchResult {

	private OfficeSearchQuery officeSearchQuery = null;
	
	private List listeners 		= null;
	private List matchesList 	= null;
	
  //----------------------------------------------------------------------------
	/**
	 * Constructs new OfficeSearchResult.
	 * 
	 * @param officeSearchQuery office search query to be used
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public OfficeSearchResult(OfficeSearchQuery officeSearchQuery) {
		assert officeSearchQuery != null;
		this.officeSearchQuery = officeSearchQuery;
	}
  //----------------------------------------------------------------------------
	/**
	 * Removes a <code>ISearchResultChangedListener</code>. Has no effect
	 * when the listener hasn't previously been added.
	 * 
	 * @param searchResultListener the listener to be removed
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public void addListener(ISearchResultListener searchResultListener) {
		if(searchResultListener == null)
			return;
		
		if(listeners == null)
			listeners = new ArrayList(3);
		
		if(!listeners.contains(searchResultListener))
			listeners.add(searchResultListener);
	}
  //----------------------------------------------------------------------------
	/**
	 * Removes a <code>ISearchResultChangedListener</code>. Has no effect
	 * when the listener hasn't previously been added.
	 * 
	 * @param searchResultListener the listener to be removed
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public void removeListener(ISearchResultListener searchResultListener) {
		if(searchResultListener != null && listeners != null)
			listeners.remove(searchResultListener);
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns a user readable label for this search result. The label is typically used in the result
	 * view and should contain the search query string and number of matches.
	 * 
	 * @return the label for this search result
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public String getLabel() {
		int numberOfMatches = getMatches().length;
		if(numberOfMatches == 1) {
			if (officeSearchQuery.getSearchDescriptor().getSearchContent().length() > 0) {
				Object[] args = {officeSearchQuery.getSearchDescriptor().getSearchContent(), officeSearchQuery.getSearchScope().getDescription() };
				return NLS.bind(Messages.OfficeSearchResult_monitor_singularLabel, args);
			}
			Object[] args = {officeSearchQuery.getSearchScope().getFileNamePatternDescription(), officeSearchQuery.getSearchScope().getDescription() };
			return NLS.bind(Messages.OfficeSearchResult_monitor_singularLabel_fileNameSearch, args);
		}
		if(numberOfMatches > 0) {
			Object[] args= {officeSearchQuery.getSearchDescriptor().getSearchContent(), new Integer(numberOfMatches), officeSearchQuery.getSearchScope().getDescription() }; 
			return NLS.bind(Messages.OfficeSearchResult_monitor_pluralPattern, args);
		}
		Object[] args= {officeSearchQuery.getSearchScope().getFileNamePatternDescription(), new Integer(numberOfMatches), officeSearchQuery.getSearchScope().getDescription()}; 
		return NLS.bind(Messages.OfficeSearchResult_monitor_pluralPattern_fileNameSearch, args); 
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns a user readable label for this search result. The label is typically used in the result
	 * view and should contain the search query string and number of matches.
	 * 
	 * @return the label for this search result
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public String getTooltip() {
		return getLabel();
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns an image descriptor for the given ISearchResult.
	 * 
	 * @return an image representing this search result or <code>null</code>
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public ImageDescriptor getImageDescriptor() {
		return SearchUIPluginImages.getImageDescriptor(SearchUIPluginImages.OBJECT_OFFICE_SEARCH_RESULT);
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns the query that produced this search result.
	 * 
	 * @return the query producing this result
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public ISearchQuery getQuery() {
		return officeSearchQuery;
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns all office search matches.
	 * 
	 * @return all office search matches
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public IOfficeSearchMatch[] getMatches() {
		if(matchesList == null)
			return IOfficeSearchMatch.EMPTY_ARRAY;
		
		return (IOfficeSearchMatch[])matchesList.toArray(new IOfficeSearchMatch[matchesList.size()]);
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns first office search match. Returns null
	 * if an office search match is not available.
	 * 
	 * @return first office search match or null
	 * if an office search match is not available
	 * 
	 * @author Andreas Brker
	 * @date 13.07.2006
	 */
	public IOfficeSearchMatch getFirstMatch() {
		if(matchesList == null || matchesList.size() == 0)
			return null;
		return (IOfficeSearchMatch)matchesList.get(0);
	}	
  //----------------------------------------------------------------------------
	/**
	 * Returns next office search match on the basis of the
	 * submitted office search match. Returns null
	 * if a next office search match is not available.
	 * 
	 * @param officeSearchMatch office search match to be used
	 * 
	 * @return next office search match on the basis of the
	 * submitted office search match or null
	 * if a next office search match is not available
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public IOfficeSearchMatch getNextMatch(IOfficeSearchMatch officeSearchMatch) {
		if(matchesList == null || officeSearchMatch == null)
			return null;
		int index = matchesList.indexOf(officeSearchMatch);
		if(index != -1) {
			if(index+1 < matchesList.size())
				return (IOfficeSearchMatch)matchesList.get(index+1);
		}
		return null;
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns previous office search match on the basis of the
	 * submitted office search match. Returns null
	 * if a previous office search match is not available.
	 * 
	 * @param officeSearchMatch office search match to be used
	 * 
	 * @return previous office search match on the basis of the
	 * submitted office search match or null
	 * if a previous office search match is not available
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public IOfficeSearchMatch getPreviousMatch(IOfficeSearchMatch officeSearchMatch) {
		if(matchesList == null || officeSearchMatch == null)
			return null;
		int index = matchesList.indexOf(officeSearchMatch);
		if(index != -1) {
			if(index-1 >= 0)
				return (IOfficeSearchMatch)matchesList.get(index-1);
		}
		return null;
	}	
  //----------------------------------------------------------------------------
	/**
	 * Adds new match.
	 * 
	 * @param officeSearchMatch office search match to be added
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public void addMatch(IOfficeSearchMatch officeSearchMatch) {
		if(officeSearchMatch == null)
			return;
		
		if(matchesList == null)
			matchesList = new ArrayList();
		matchesList.add(officeSearchMatch);
		fireMatchAdded();
	}
  //----------------------------------------------------------------------------
	/**
	 * Informs all listeners about the added match. 
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	private void fireMatchAdded() {
		if(listeners != null) {
			OfficeSearchResultEvent officeSearchResultEvent = new OfficeSearchResultEvent(this);
			Iterator iterator = listeners.iterator();
			while(iterator.hasNext()) {
				((ISearchResultListener)iterator.next()).searchResultChanged(officeSearchResultEvent);
			}
		}
	}
  //----------------------------------------------------------------------------
	
}