/****************************************************************************
 *                                                                          *
 * NOA (Nice Office Access)                                     						*
 * ------------------------------------------------------------------------ *
 *                                                                          *
 * The Contents of this file are made available subject to                  *
 * the terms of GNU Lesser General Public License Version 2.1.              *
 *                                                                          * 
 * GNU Lesser General Public License Version 2.1                            *
 * ======================================================================== *
 * Copyright 2003-2006 by IOn AG                                            *
 *                                                                          *
 * This library is free software; you can redistribute it and/or            *
 * modify it under the terms of the GNU Lesser General Public               *
 * License version 2.1, as published by the Free Software Foundation.       *
 *                                                                          *
 * This library is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        *
 * Lesser General Public License for more details.                          *
 *                                                                          *
 * You should have received a copy of the GNU Lesser General Public         *
 * License along with this library; if not, write to the Free Software      *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,                    *
 * MA  02111-1307  USA                                                      *
 *                                                                          *
 * Contact us:                                                              *
 *  http://www.ion.ag																												*
 *  http://ubion.ion.ag                                                     *
 *  info@ion.ag                                                             *
 *                                                                          *
 ****************************************************************************/
 
/*
 * Last changes made by $Author: andreas $, $Date: 2006-08-07 13:09:58 +0200 (Mo, 07 Aug 2006) $
 */
package ag.ion.noa4e.internal.search.ui;

import ag.ion.noa.document.IDocumentProvider;
import ag.ion.noa.document.ISearchableDocument;

import ag.ion.noa.search.ISearchDescriptor;
import ag.ion.noa.search.ResultMatchWalker;

import ag.ion.noa4e.internal.search.core.IOfficeSearchMatch;
import ag.ion.noa4e.search.NOASearchPlugin;
import ag.ion.noa4e.search.SearchUIPluginImages;

import ag.ion.bion.officelayer.document.IDocument;

import org.eclipse.core.resources.IFile;

import org.eclipse.core.runtime.CoreException;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;

import org.eclipse.jface.dialogs.ErrorDialog;

import org.eclipse.jface.resource.ImageDescriptor;

import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;

import org.eclipse.osgi.util.NLS;
import org.eclipse.search.ui.IContextMenuConstants;
import org.eclipse.search.ui.ISearchQuery;
import org.eclipse.search.ui.ISearchResult;
import org.eclipse.search.ui.ISearchResultListener;
import org.eclipse.search.ui.ISearchResultPage;
import org.eclipse.search.ui.ISearchResultViewPart;
import org.eclipse.search.ui.SearchResultEvent;

import org.eclipse.swt.SWT;

import org.eclipse.swt.graphics.Image;

import org.eclipse.swt.layout.GridData;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Table;

import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IMemento;

import org.eclipse.ui.IPartListener2;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchPartReference;

import org.eclipse.ui.ide.IDE;

import org.eclipse.ui.model.IWorkbenchAdapter;

import org.eclipse.ui.part.Page;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

/**
 * Search result page for office documents.
 * 
 * @author Andreas Brker
 * @version $Revision: 9195 $
 * @date 10.07.2006
 */ 
public class OfficeSearchResultPage extends Page implements ISearchResultPage {

	private Control 							control 							= null;	
	private TableViewer 					tableViewer 					= null;
	private ISearchResultViewPart searchResultViewPart 	= null;
	private IPartListener2 				partListener 					= null;
	
	private OfficeSearchResult 		currentSearchResult 	= null;
	private ISearchResultListener searchResultListener 	= null;
	
	private IAction nextMatchAction 		= null;
	private IAction previousMatchAction = null;
	
	private Map matchWalkerMap 	= null;
	private Map editorMap 			= null;
	
	
	private String id = null;
	
  //----------------------------------------------------------------------------
	/**
	 * Internal action in order to move to the next match.
	 * 
	 * @author Andreas Brker
	 * @version $Revision: 9195 $
	 * @date 12.07.2006
	 */
	private class NextMatchAction extends Action {
		
	  //----------------------------------------------------------------------------
		/**
		 * Constructs new PreviousMatchAction.
		 *
		 * @author Andreas Brker
     * @date 12.07.2006
		 */
		public NextMatchAction() {
			setImageDescriptor(SearchUIPluginImages.getImageDescriptor(SearchUIPluginImages.ELCL_NEXT_MATCH));
			setDisabledImageDescriptor(SearchUIPluginImages.getImageDescriptor(SearchUIPluginImages.DLCL_NEXT_MATCH));
			setToolTipText(Messages.OfficeSearchResultPage_action_toolTip_nextMatchAction);
		}
	  //----------------------------------------------------------------------------
		/**
		 * Executes action logic.
		 * 
		 * @author Andreas Brker
     * @date 12.07.2006
		 */
		public void run() {
			IOfficeSearchMatch officeSearchMatch = getOfficeSearchMatch(tableViewer.getSelection());
			if(officeSearchMatch == null && currentSearchResult != null) {
				officeSearchMatch = currentSearchResult.getFirstMatch();		
				tableViewer.setSelection(new StructuredSelection(officeSearchMatch));
			}
			if(officeSearchMatch != null)					
				showMextMatch(officeSearchMatch);
		}
	  //----------------------------------------------------------------------------
		
	}
  //----------------------------------------------------------------------------
	
  //----------------------------------------------------------------------------
	/**
	 * Internal action in order to move to the previous match.
	 * 
	 * @author Andreas Brker
	 * @version $Revision: 9195 $
	 * @date 12.07.2006
	 */
	private class PreviousMatchAction extends Action {
		
	  //----------------------------------------------------------------------------
		/**
		 * Constructs new PreviousMatchAction.
		 *
		 * @author Andreas Brker
     * @date 12.07.2006
		 */
		public PreviousMatchAction() {
			setImageDescriptor(SearchUIPluginImages.getImageDescriptor(SearchUIPluginImages.ELCL_PREVIOUS_MATCH));
			setDisabledImageDescriptor(SearchUIPluginImages.getImageDescriptor(SearchUIPluginImages.DLCL_PREVIOUS_MATCH));
			setToolTipText(Messages.OfficeSearchResultPage_action_toolTip_previousMatchAction);
		}
	  //----------------------------------------------------------------------------
		/**
		 * Executes action logic.
		 * 
		 * @author Andreas Brker
     * @date 12.07.2006
		 */
		public void run() {
			IOfficeSearchMatch officeSearchMatch = getOfficeSearchMatch(tableViewer.getSelection());			
			if(officeSearchMatch != null)
				showPreviousMatch(officeSearchMatch);
		}
	  //----------------------------------------------------------------------------
		
	}
  //----------------------------------------------------------------------------

  //----------------------------------------------------------------------------
	/**
	 * Internal content provider for the search result viewer.
	 * 
	 * @author Andreas Brker
	 * @version $Revision: 9195 $
	 * @date 12.07.2006
	 */
	private class ResultContentProvider implements IStructuredContentProvider {

	  //----------------------------------------------------------------------------
    /**
     * Disposes of this content provider.  
     * This is called by the viewer when it is disposed.
     * 
     * @author Andreas Brker
     * @date 12.07.2006
     */
		public void dispose() {
		}
	  //----------------------------------------------------------------------------
    /**
     * Notifies this content provider that the given viewer's input
     * has been switched to a different element.
     * <p>
     * A typical use for this method is registering the content provider as a listener
     * to changes on the new input (using model-specific means), and deregistering the viewer 
     * from the old input. In response to these change notifications, the content provider
     * should update the viewer (see the add, remove, update and refresh methods on the viewers).
     * </p>
     *
     * @param viewer the viewer
     * @param oldInput the old input element, or <code>null</code> if the viewer
     *   did not previously have an input
     * @param newInput the new input element, or <code>null</code> if the viewer
     *   does not have an input
     * 
     * @author Andreas Brker
     * @date 12.07.2006
     */
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}
	  //----------------------------------------------------------------------------
    /**
     * Returns the elements to display in the viewer 
     * when its input is set to the given element. 
     * These elements can be presented as rows in a table, items in a list, etc.
     * The result is not modified by the viewer.
     *
     * @param inputElement the input element
     * 
     * @return the array of elements to display in the viewer
     * 
     * @author Andreas Brker
     * @date 12.07.2006
     */
		public Object[] getElements(Object inputElement) {
			if(inputElement instanceof OfficeSearchResult) {
				return ((OfficeSearchResult)inputElement).getMatches();
			}
			else
				return new Object[0];
		}	
	  //----------------------------------------------------------------------------
		
	}
  //----------------------------------------------------------------------------
	
  //----------------------------------------------------------------------------
	/**
	 * Internal label provider for the search result viewer.
	 * 
	 * @author Andreas Brker
	 * @version $Revision: 9195 $
	 * @date 12.07.2006
	 */
	private class ResultLabelProvider implements ITableLabelProvider {

	  //----------------------------------------------------------------------------
    /**
     * Returns the label image for the given column of the given element.
     *
     * @param element the object representing the entire row, or 
     *    <code>null</code> indicating that no input object is set
     *    in the viewer
     * @param columnIndex the zero-based index of the column in which
     *   the label appears
     *   
     * @return Image or <code>null</code> if there is no image for the 
     *  given object at columnIndex
     * 
     * @author Andreas Brker
     * @date 12.07.2006
     */
		public Image getColumnImage(Object element, int columnIndex) {
			if(element instanceof IOfficeSearchMatch) {
				IOfficeSearchMatch officeSearchMatch = (IOfficeSearchMatch)element;
				IWorkbenchAdapter workbenchAdapter = (IWorkbenchAdapter)officeSearchMatch.getFile().getAdapter(IWorkbenchAdapter.class);
				if(workbenchAdapter != null) {
					ImageDescriptor descriptor = workbenchAdapter.getImageDescriptor(((IOfficeSearchMatch)element).getFile());
	        if (descriptor == null) {
	        	return null;
	        }
	        
	        Image image = SearchUIPluginImages.getImageRegistry().get(officeSearchMatch.getFile().getFileExtension());	       
	        if (image == null) {
            image = descriptor.createImage();
            SearchUIPluginImages.getImageRegistry().put(officeSearchMatch.getFile().getFileExtension(), image);
	        }
	        return image;
				}
			}
			return null;
		}
	  //----------------------------------------------------------------------------
    /**
     * Returns the label text for the given column of the given element.
     *
     * @param element the object representing the entire row, or
     *   <code>null</code> indicating that no input object is set
     *   in the viewer
     * @param columnIndex the zero-based index of the column in which the label appears
     * 
     * @return String or or <code>null</code> if there is no text for the 
     *  given object at columnIndex
     *  
     * @author Andreas Brker
     * @date 12.07.2006
     */
		public String getColumnText(Object element, int columnIndex) {
			if(element instanceof IOfficeSearchMatch) {
				return ((IOfficeSearchMatch)element).getFile().getName() + 
						NLS.bind(Messages.OfficeSearchResultPage_label_numberOfMatches, new Integer(((IOfficeSearchMatch)element).getMatchCount()));
			}
			return ""; //$NON-NLS-1$
		}
	  //----------------------------------------------------------------------------
    /**
     * Adds a listener to this label provider. 
     * Has no effect if an identical listener is already registered.
     *
     * @param listener a label provider listener
     * 
     * @author Andreas Brker
     * @date 12.07.2006
     */
		public void addListener(ILabelProviderListener listener) {
		}
	  //----------------------------------------------------------------------------
    /**
     * Disposes of this label provider.  When a label provider is
     * attached to a viewer, the viewer will automatically call
     * this method when the viewer is being closed.  When label providers
     * are used outside of the context of a viewer, it is the client's
     * responsibility to ensure that this method is called when the
     * provider is no longer needed.
     * 
     * @author Andreas Brker
     * @date 12.07.2006
     */
		public void dispose() {
		}
	  //----------------------------------------------------------------------------
    /**
     * Returns whether the label would be affected 
     * by a change to the given property of the given element.
     * This can be used to optimize a non-structural viewer update.
     * If the property mentioned in the update does not affect the label,
     * then the viewer need not update the label.
     *
     * @param element the element
     * @param property the property
     * 
     * @return <code>true</code> if the label would be affected,
     *    and <code>false</code> if it would be unaffected
     * 
     * @author Andreas Brker
     * @date 12.07.2006
     */
		public boolean isLabelProperty(Object element, String property) {
			return false;
		}
	  //----------------------------------------------------------------------------
    /**
     * Removes a listener to this label provider.
     * Has no affect if an identical listener is not registered.
     *
     * @param listener a label provider listener
     * 
     * @author Andreas Brker
     * @date 12.07.2006
     */
		public void removeListener(ILabelProviderListener listener) {
		}
	  //----------------------------------------------------------------------------
		
	}
  //----------------------------------------------------------------------------
	
  //----------------------------------------------------------------------------
	/**
	 * Returns an object representing the current user interface state of the
	 * page. For example, the current selection in a viewer. The UI state will
	 * be later passed into the <code>setInput()</code> method when the
	 * currently shown <code>ISearchResult</code> is shown again.
	 * 
	 * @return an object representing the UI state of this page
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public Object getUIState() {
		return tableViewer.getSelection();
	}
  //----------------------------------------------------------------------------
	/**
	 * Sets the search result to be shown in this search results page.
	 * Implementers should restore UI state (e.g. selection) from the previously
	 * saved <code>uiState</code> object.
	 * 
	 * @param searchResult the search result to be shown
	 * @param uiState the previously saved UI state
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public void setInput(ISearchResult searchResult, Object uiState) {
		if(currentSearchResult != null) {
			if(searchResultListener != null)
				currentSearchResult.removeListener(searchResultListener);
			matchWalkerMap = null;
			editorMap = null;
		}
		
		if(searchResult instanceof OfficeSearchResult) {
			currentSearchResult = (OfficeSearchResult)searchResult;
			if(searchResult != null) {
				if(searchResultListener == null)
					searchResultListener = new ISearchResultListener() {
						public void searchResultChanged(SearchResultEvent searchResultEvent) {
							getControl().getDisplay().asyncExec(new Runnable() {
								public void run() {
									tableViewer.refresh();
									searchResultViewPart.updateLabel();
									updateActionsEnableState();
								}
							});
							
						}				
					};			
				searchResult.addListener(searchResultListener);
				tableViewer.setInput(searchResult);
				if(uiState != null)
					tableViewer.setSelection((ISelection)uiState);
			}
			else {
				tableViewer.setInput(null);
			}
		}
	}
  //----------------------------------------------------------------------------
	/**
	 * Sets the search view this search results page is shown in. This method
	 * will be called before the page is shown for the first time (i.e. before
	 * the page control is created).
	 * 
	 * @param searchResultViewPart the parent search view
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public void setViewPart(ISearchResultViewPart searchResultViewPart) {
		this.searchResultViewPart = searchResultViewPart;
	}
  //----------------------------------------------------------------------------
	/**
	 * Restores the page state. Note that this applies only to state that is
	 * saved across sessions.
	 * 
	 * @param memento a memento to restore the page state from or <code>null</code>
	 *  if no previous state was saved
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public void restoreState(IMemento memento) {
		//not supported
	}
  //----------------------------------------------------------------------------
	/**
	 * Saves the page state in a memento. Note that this applies to state that
	 * should persist across sessions. 
	 * 
	 * @param memento a memento to receive the object state
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public void saveState(IMemento memento) {
		//not supported
	}
  //----------------------------------------------------------------------------
	/**
	 * Sets the id for this page. This method will be called before any other
	 * initialization is done.
	 * 
	 * @param id the id for this page
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public void setID(String id) {
		this.id = id;
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns the id set via <code>setID</code>.
	 * 
	 * @return the id of this page
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public String getID() {
		return id;
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns a user readable label for this search result page. The label will be
	 * used to describe the contents for the page to the user (for example it will be
	 * displayed in the search view title bar). To take an example from file search, 
	 * a label might read like this: 'test' - 896 matches in workspace.
	 * 
	 * @return the user readable label for this search result page
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public String getLabel() {
		if(currentSearchResult != null)
			return currentSearchResult.getLabel();
		else
			return ""; //$NON-NLS-1$
	}
  //----------------------------------------------------------------------------
  /**
   * Creates the SWT control for this page under the given parent 
   * control.
   *
   * @param parent the parent control
   * 
   * @author Andreas Brker
	 * @date 12.07.2006
   */
	public void createControl(Composite parent) {
		Table table = new Table(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION);
		table.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		tableViewer = new TableViewer(table);
		tableViewer.setContentProvider(new ResultContentProvider());
		tableViewer.setLabelProvider(new ResultLabelProvider());
		
		tableViewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent doubleClickEvent) {
				IOfficeSearchMatch officeSearchMatch = getOfficeSearchMatch(doubleClickEvent.getSelection());
				if(officeSearchMatch != null) {
					showInEditor(officeSearchMatch);
					updateActionsEnableState();
				}
			}			
		});
		
		tableViewer.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent selectionChangedEvent) {
				updateActionsEnableState();
			}			
		});
		
		buildActions();
		fillToolBar();
		
		partListener = new IPartListener2() {
			public void partActivated(IWorkbenchPartReference partRef) {
			}
			public void partBroughtToTop(IWorkbenchPartReference partRef) {
			}
			public void partClosed(IWorkbenchPartReference partRef) {
				IWorkbenchPart workbenchPart = partRef.getPart(true);
				if(workbenchPart instanceof IEditorPart) {
					releaseEditorPart((IEditorPart)workbenchPart);
				}
			}
			public void partDeactivated(IWorkbenchPartReference partRef) {
			}
			public void partOpened(IWorkbenchPartReference partRef) {
			}
			public void partHidden(IWorkbenchPartReference partRef) {
			}
			public void partVisible(IWorkbenchPartReference partRef) {
			}
			public void partInputChanged(IWorkbenchPartReference partRef) {
			}			
		};
		NOASearchPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getActivePage().addPartListener(partListener);
		updateActionsEnableState();
		this.control = table;
	}
  //----------------------------------------------------------------------------
  /**
   * Returns the SWT control for this page.
   *
   * @return the SWT control for this page, or <code>null</code> if this
   *   page does not have a control
   *   
   * @author Andreas Brker
	 * @date 12.07.2006
   */
	public Control getControl() {
		return control;
	}
  //----------------------------------------------------------------------------
	/**
	 * Disposes the page.
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	public void dispose() {
		if(partListener != null)
			NOASearchPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getActivePage().removePartListener(partListener);
		super.dispose();
	}
  //----------------------------------------------------------------------------
  /**
   * Sets focus to the page.
   * 
   * @author Andreas Brker
	 * @date 12.07.2006
   */
	public void setFocus() {
		if(control != null && !control.isDisposed()) {
			control.setFocus();
		}
	}
  //----------------------------------------------------------------------------
	/**
	 * Shows the submitted office search match in an editor.
	 * 
	 * @param officeSearchMatch office search match to be used
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	private void showInEditor(IOfficeSearchMatch officeSearchMatch) {		
		IEditorPart editorPart = getEditorPart(officeSearchMatch);
		if(editorPart == null) {
			IFile file = officeSearchMatch.getFile();
			try {
				editorPart = IDE.openEditor(NOASearchPlugin.getDefault().getWorkbench().
						getActiveWorkbenchWindow().getActivePage(), file);
				ResultMatchWalker resultMatchWalker = getNewMatchWalker(editorPart, officeSearchMatch);
				if(resultMatchWalker != null)
					resultMatchWalker.nextMatch();
			}
			catch (CoreException coreException) {
				ErrorDialog.openError(Display.getCurrent().getActiveShell(), 
						Messages.OfficeSearchResultPage_dialog_title_internalError, coreException.getMessage(), coreException.getStatus());
			}
		}
		else {
			NOASearchPlugin.getDefault().getWorkbench().
				getActiveWorkbenchWindow().getActivePage().activate(editorPart);
		}			
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns editor part of the submitted office search match. Returns null
	 * if an editor part is not available.
	 * 
	 * @param officeSearchMatch office search match to be used
	 * 
	 * @return editor part of the submitted office search match. Returns null
	 * if an editor part is not available
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	private IEditorPart getEditorPart(IOfficeSearchMatch officeSearchMatch) {
		if(editorMap == null)
			return null;
		
		return (IEditorPart)editorMap.get(officeSearchMatch);
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns new document match walker for the submitted office search match. Returns null
	 * if the editor part does not provide an office document.
	 * 
	 * @param editorPart editor part to be used
	 * @param officeSearchMatch office search match to be used
	 * 
	 * @return new document match walker for the submitted office search match or null
	 * if the editor part does not provide an office document
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	private ResultMatchWalker getNewMatchWalker(IEditorPart editorPart, IOfficeSearchMatch officeSearchMatch) {
		if(editorMap == null)
			editorMap = new HashMap();
		editorMap.put(officeSearchMatch, editorPart);
		
		if(editorPart instanceof IDocumentProvider) {			
			IDocument document = ((IDocumentProvider)editorPart).getDocument();
			ISearchDescriptor searchDescriptor = getCurrentSearchDescriptor();
			if(searchDescriptor != null) {
				if(document instanceof ISearchableDocument) {
					ISearchableDocument searchableDocument = (ISearchableDocument)document;
					ag.ion.noa.search.ISearchResult searchResult = searchableDocument.getSearchService().findAll(searchDescriptor);
					if(!searchResult.isEmpty()) {
						if(matchWalkerMap == null)
							matchWalkerMap = new HashMap();								
						ResultMatchWalker resultMatchWalker = new ResultMatchWalker(document, searchResult);
						matchWalkerMap.put(editorPart, resultMatchWalker);						
						return resultMatchWalker;						
					}
				}
			}
		}
		return null;
	}
  //----------------------------------------------------------------------------
	/**
	 * Releases the submitted editor part.
	 * 
	 * @param editorPart editor part to be released
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	private void releaseEditorPart(IEditorPart editorPart) {
		if(matchWalkerMap != null)
			matchWalkerMap.remove(editorPart);
		
		if(editorMap != null) {
			Set set = editorMap.entrySet();
			Iterator iterator = set.iterator();
			while(iterator.hasNext()) {
				Map.Entry entry = (Map.Entry)iterator.next();
				if(entry.getValue().equals(editorPart)) {
					editorMap.remove(entry.getKey());
					break;
				}
			}
		}
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns document match walker for the submitted editor part. Returns 
	 * null if a document match walker is not available.
	 * 
	 * @param editorPart editor part to be used
	 * 
	 * @return document match walker for the submitted editor part or 
	 * null if a document match walker is not available
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	private ResultMatchWalker getMatchWalker(IEditorPart editorPart) {
		if(matchWalkerMap == null || editorPart == null)
			return null;
		
		return (ResultMatchWalker)matchWalkerMap.get(editorPart);
	}
  //----------------------------------------------------------------------------
	/**
	 * Shows next match on the basis of the office search match.
	 * 
	 * @param officeSearchMatch office search match to be used
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	private void showMextMatch(IOfficeSearchMatch officeSearchMatch) {		
		IEditorPart editorPart = getEditorPart(officeSearchMatch);
		if(editorPart == null) {
			showInEditor(officeSearchMatch);
		}
		else {
			ResultMatchWalker resultMatchWalker = getMatchWalker(editorPart);
			if(resultMatchWalker != null) {
				if(resultMatchWalker.hasNextMatch()) {
					resultMatchWalker.nextMatch();
					if(!NOASearchPlugin.getDefault().getWorkbench().
							getActiveWorkbenchWindow().getActivePage().getActiveEditor().equals(editorPart))
						NOASearchPlugin.getDefault().getWorkbench().
								getActiveWorkbenchWindow().getActivePage().activate(editorPart);
				}
				else
					gotoNextDocument(officeSearchMatch);
			}
		}
		updateActionsEnableState();
	}
	
	/**
	 * Goes to the next document.
	 * 
	 * @param officeSearchMatch current office search match
	 * 
	 * @author Andreas Brker
	 * @date 13.07.2006
	 */
	private void gotoNextDocument(IOfficeSearchMatch officeSearchMatch) {
		officeSearchMatch = currentSearchResult.getNextMatch(officeSearchMatch);
		if(officeSearchMatch != null) {
			showMextMatch(officeSearchMatch);
			tableViewer.setSelection(new StructuredSelection(officeSearchMatch));
			IEditorPart editorPart = getEditorPart(officeSearchMatch);
			if(editorPart != null)
				NOASearchPlugin.getDefault().getWorkbench().
						getActiveWorkbenchWindow().getActivePage().activate(editorPart);
		}
	}
	
  //----------------------------------------------------------------------------
	/**
	 * Shows previous match on the basis of the submitted selection.
	 * 
	 * @param selection selection to be used
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */	
	private void showPreviousMatch(IOfficeSearchMatch officeSearchMatch) {		
		IEditorPart editorPart = getEditorPart(officeSearchMatch);
		if(editorPart == null) {
			showInEditor(officeSearchMatch);
		}
		else {
			ResultMatchWalker resultMatchWalker = getMatchWalker(editorPart);
			if(resultMatchWalker != null) {
				if(resultMatchWalker.hasPreviousMatch()) {
					resultMatchWalker.previousMatch();
					if(!NOASearchPlugin.getDefault().getWorkbench().
							getActiveWorkbenchWindow().getActivePage().getActiveEditor().equals(editorPart))
						NOASearchPlugin.getDefault().getWorkbench().
								getActiveWorkbenchWindow().getActivePage().activate(editorPart);
				}
				else
					gotoPreviousDocument(officeSearchMatch);
			}
		}
		updateActionsEnableState();
	}
	
	/**
	 * Goes to the previous document.
	 * 
	 * @param officeSearchMatch current office search match
	 * 
	 * @author Andreas Brker
	 * @date 13.07.2006
	 */
	private void gotoPreviousDocument(IOfficeSearchMatch officeSearchMatch) {
		officeSearchMatch = currentSearchResult.getPreviousMatch(officeSearchMatch);
		if(officeSearchMatch != null) {
			showPreviousMatch(officeSearchMatch);
			tableViewer.setSelection(new StructuredSelection(officeSearchMatch));
			IEditorPart editorPart = getEditorPart(officeSearchMatch);
			if(editorPart != null)
				NOASearchPlugin.getDefault().getWorkbench().
						getActiveWorkbenchWindow().getActivePage().activate(editorPart);
		}
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns office search match on the basis of the submitted selection.
	 * 
	 * @param selection selection to be used
	 * 
	 * @return office search match on the basis of the submitted selection
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	private IOfficeSearchMatch getOfficeSearchMatch(ISelection selection) {
		if(selection instanceof IStructuredSelection) {
			Object object = ((IStructuredSelection)selection).getFirstElement();
			if(object instanceof IOfficeSearchMatch) {
				return (IOfficeSearchMatch)object;
			}
		}
		return null;
	}
  //----------------------------------------------------------------------------
	/**
	 * Returns current search descriptor. Returns null
	 * if a search descriptor is not available.
	 * 
	 * @return current search descriptor or null
	 * if a search descriptor is not available
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	private ISearchDescriptor getCurrentSearchDescriptor() {
		if(currentSearchResult != null) {
			ISearchQuery searchQuery = currentSearchResult.getQuery();
			if(searchQuery instanceof OfficeSearchQuery) {
				return ((OfficeSearchQuery)searchQuery).getSearchDescriptor();
			}
		}
		return null;
	}
  //----------------------------------------------------------------------------
	/**
	 * Builds all actions. 
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	private void buildActions() {
		nextMatchAction = new NextMatchAction();
		previousMatchAction = new PreviousMatchAction();
	}
  //----------------------------------------------------------------------------
	/**
	 * Fills the tool bar. 
	 * 
	 * @author Andreas Brker
	 * @date 12.07.2006
	 */
	private void fillToolBar() {
		getSite().getActionBars().getToolBarManager().appendToGroup(IContextMenuConstants.GROUP_SHOW, nextMatchAction);
		getSite().getActionBars().getToolBarManager().appendToGroup(IContextMenuConstants.GROUP_SHOW, previousMatchAction);
	}
  //----------------------------------------------------------------------------
	/**
	 * Updates enable state of actions. 
	 * 
	 * @author Andreas Brker
	 * @date 13.07.2006
	 */
	private void updateActionsEnableState() {
		IOfficeSearchMatch officeSearchMatch = getOfficeSearchMatch(tableViewer.getSelection());
		if(officeSearchMatch == null && currentSearchResult != null)
			officeSearchMatch = currentSearchResult.getFirstMatch();
		if(officeSearchMatch == null) {
			nextMatchAction.setEnabled(false);	
			previousMatchAction.setEnabled(false);
			return;
		}
		IEditorPart editorPart = getEditorPart(officeSearchMatch);
		if(editorPart == null)
			nextMatchAction.setEnabled(true);
		else {
			ResultMatchWalker resultMatchWalker = getMatchWalker(editorPart);
			if(resultMatchWalker != null && resultMatchWalker.hasNextMatch())
				nextMatchAction.setEnabled(true);
			else if(currentSearchResult.getNextMatch(officeSearchMatch) != null)
				nextMatchAction.setEnabled(true);
			else
				nextMatchAction.setEnabled(false);				
		}
		
		ResultMatchWalker resultMatchWalker = getMatchWalker(editorPart);
		if(resultMatchWalker != null && resultMatchWalker.hasPreviousMatch())
			previousMatchAction.setEnabled(true);
		else if(currentSearchResult.getPreviousMatch(officeSearchMatch) != null)
			previousMatchAction.setEnabled(true);
		else
			previousMatchAction.setEnabled(false);		
	}	
  //----------------------------------------------------------------------------
	
}