package junit.requirementsontology.test;

import static org.junit.Assert.fail;

import java.util.Collection;
import java.util.Set;
import java.util.TreeSet;

import org.deft.requirementsontology.DirectedROModel;
import org.deft.requirementsontology.ObjectPropertyEdge;
import org.deft.requirementsontology.OntologyAccessor;
import org.junit.Assert;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

import edu.uci.ics.jung.graph.util.EdgeType;

public class DirectedROModelTest
{

	private static OntologyAccessor oA;
	private static DirectedROModel roModel;
	
	@BeforeClass
	public static void setUpBeforeClass() throws Exception
	{
		oA = new OntologyAccessor("RO_version_1_4_1_social_network_consistent.owl");
		roModel = new DirectedROModel(oA);
	}
	// end setUpBeforeClass

	@Before
	public void setUp() throws Exception
	{
		roModel.initialize();
	}
	// end setUp

	@Ignore
	public void testInitializeGoalModelGraph()
	{
		fail("Not yet implemented");
	}
	// end testInitializeGoalModelGraph

	@Test
	public void testInsertObjectProperty()
	{
		roModel.removeAll();
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertObjectProperty("isAuthoredBy");
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertIndividual("TomBeger");
		roModel.insertObjectProperty("isAuthoredBy");
		
		Assert.assertEquals(1,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.removeIndividual("TomBeger");
		roModel.insertIndividual("DataPrivacyProtection");
		roModel.insertObjectProperty("isAuthoredBy");
		
		Assert.assertEquals(1,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertIndividual("TomBeger");
		roModel.insertObjectProperty("isAuthoredBys");
		
		Assert.assertEquals(2,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertObjectProperty("isAuthoredBy");
		
		Assert.assertEquals(2,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(1,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals("isAuthoredBy",roModel.getGoalModelGraph().getEdges().toArray()[0].toString());
		Assert.assertEquals("DataPrivacyProtection",((ObjectPropertyEdge) roModel.getGoalModelGraph().getEdges().toArray()[0]).getSourceIndividual().getSimpleName());
		Assert.assertEquals("TomBeger",((ObjectPropertyEdge) roModel.getGoalModelGraph().getEdges().toArray()[0]).getTargetIndividual().getSimpleName());
		
		roModel.removeAll();
		roModel.insertIndividual("ExchangeInformation");
		roModel.insertIndividual("TomBeger");
		roModel.insertObjectProperty("isAuthoredBy");
		
		Assert.assertEquals(2,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(1,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertIndividual("ErikTittel");
		
		Assert.assertEquals(3,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(1,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertObjectProperty("isAuthoredBy");
		
		Assert.assertEquals(3,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(2,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.removeAll();
		roModel.insertIndividual("ExchangeInformation");
		roModel.insertIndividual("TomBeger");
		roModel.insertIndividual("ErikTittel");
		roModel.insertObjectProperty("isAuthoredBy");
		
		Assert.assertEquals(3,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(2,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertObjectProperty("isAuthoredBy");
		
		Assert.assertEquals(3,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(2,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertAll();
		
		Assert.assertEquals(67,roModel.getGoalModelGraph().getEdges(EdgeType.DIRECTED).size());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdges(EdgeType.UNDIRECTED).size());
	}
	// end testInsertObjectProperty

	@Test
	public void testInsertObjectPropertiesForIndividual()
	{
		roModel.removeAll();
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertObjectPropertiesForIndividual("DataControl");
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertIndividual("DataControl");
		roModel.insertObjectPropertiesForIndividual("DataControl");
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertIndividual("UserCanHideData");
		roModel.insertIndividual("UserHasProfilePage");
		roModel.insertObjectPropertiesForIndividual("DataControl");
		
		Assert.assertEquals(2,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertObjectPropertiesForIndividual("DataControl");
		
		Assert.assertEquals(2,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertIndividuals();
		roModel.insertObjectPropertiesForIndividual("DataControls");
		
		Assert.assertEquals(2,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertObjectPropertiesForIndividual("DataControl");
		
		Assert.assertEquals(3,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertIndividuals();
		
		Assert.assertEquals(3,roModel.getGoalModelGraph().getEdges(EdgeType.DIRECTED).size());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdges(EdgeType.UNDIRECTED).size());
		
		roModel.insertObjectPropertiesForIndividual("AccessibleToBlindUsers");
		
		Assert.assertEquals(6,roModel.getGoalModelGraph().getEdges(EdgeType.DIRECTED).size());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdges(EdgeType.UNDIRECTED).size());
		
		roModel.insertObjectPropertiesForIndividual("IntegratedInstantMessaging");
		
		Assert.assertEquals(9,roModel.getGoalModelGraph().getEdges(EdgeType.DIRECTED).size());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdges(EdgeType.UNDIRECTED).size());
	}
	// end testInsertObjectPropertiesForIndividual

	@Test
	public void testDirectedROModel()
	{
		DirectedROModel roModel = new DirectedROModel(oA);
		
		Assert.assertNotNull(roModel.getGoalModelGraph());
		Assert.assertNotNull(roModel.getDataProperties());
		Assert.assertNotNull(roModel.getOntologyClassNames());
		Assert.assertNotNull(roModel.getOntologyRootClassNames());
		Assert.assertNotNull(roModel.getOntologyAttributeClassNames());
		Assert.assertNotNull(roModel.getOntologyObjectPropertyNames());
		Assert.assertNotNull(roModel.getOntologyDataPropertyNames());
		Assert.assertNotNull(roModel.getOntologyIndividualNames());
		Assert.assertEquals(32,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(67,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(1,roModel.getDataProperties().size());
	}
	// end testDirectedROModel

	@Ignore
	public void testGetGoalModelForest()
	{
		fail("Not yet implemented");
	}
	// end testGetGoalModelForest

	@Test (expected=IllegalArgumentException.class)
	public void testGetObjectPropertiesStringString()
	{
		Collection<ObjectPropertyEdge> objectPropertyEdgeCollection1 = roModel.getObjectProperties("DataControl","DataControl");
		Collection<ObjectPropertyEdge> objectPropertyEdgeCollection2 = roModel.getObjectProperties("DataControls","DataControl");
		Collection<ObjectPropertyEdge> objectPropertyEdgeCollection3 = roModel.getObjectProperties("DataControl","DataControls");
		Collection<ObjectPropertyEdge> objectPropertyEdgeCollection4 = roModel.getObjectProperties("StoreInformation","DataPrivacyProtection");
		Collection<ObjectPropertyEdge> objectPropertyEdgeCollection5 = roModel.getObjectProperties("DataPrivacyProtection","StoreInformation");
		Collection<ObjectPropertyEdge> objectPropertyEdgeCollection6 = roModel.getObjectProperties("IntegratedInstantMessaging","UserHasPhotoAlbum");
		Collection<ObjectPropertyEdge> objectPropertyEdgeCollection7 = roModel.getObjectProperties("UserHasPhotoAlbum","IntegratedInstantMessaging");
		Collection<ObjectPropertyEdge> objectPropertyEdgeCollection8 = roModel.getObjectProperties("AccessibleToBlindUsers","UserHasPhotoAlbum");
		Collection<ObjectPropertyEdge> objectPropertyEdgeCollection9 = roModel.getObjectProperties("UserHasPhotoAlbum","AccessibleToBlindUsers");
		
		Assert.assertEquals(0,objectPropertyEdgeCollection1.size());
		Assert.assertNull(objectPropertyEdgeCollection2);
		Assert.assertNull(objectPropertyEdgeCollection3);
		Assert.assertEquals(1,objectPropertyEdgeCollection4.size());
		Assert.assertEquals(0,objectPropertyEdgeCollection5.size());
		Assert.assertEquals(1,objectPropertyEdgeCollection6.size());
		Assert.assertEquals(0,objectPropertyEdgeCollection7.size());
		Assert.assertEquals(1,objectPropertyEdgeCollection8.size());
		Assert.assertEquals(0,objectPropertyEdgeCollection9.size());
		
		Set<String> objectPropertyNameSet1 = new TreeSet<String>();
		Set<String> objectPropertyNameSet2 = new TreeSet<String>();
		Set<String> objectPropertyNameSet3 = new TreeSet<String>();
		
		for (ObjectPropertyEdge objectPropertyEdge : objectPropertyEdgeCollection4)
		{
			objectPropertyNameSet1.add(objectPropertyEdge.getSimpleName());
		}
		// end for
		
		for (ObjectPropertyEdge objectPropertyEdge : objectPropertyEdgeCollection6)
		{
			objectPropertyNameSet2.add(objectPropertyEdge.getSimpleName());
		}
		// end for
		
		for (ObjectPropertyEdge objectPropertyEdge : objectPropertyEdgeCollection8)
		{
			objectPropertyNameSet3.add(objectPropertyEdge.getSimpleName());
		}
		// end for
		
		Assert.assertTrue(objectPropertyNameSet1.contains("isNegativeContributionTo"));
		Assert.assertTrue(objectPropertyNameSet2.contains("isAlternativeTo"));
		Assert.assertTrue(objectPropertyNameSet3.contains("isInConflictWith"));
	}
	// end testGetObjectPropertiesStringString
	
	@Test
	public void testGetObjectPropertiesString()
	{
		Collection<ObjectPropertyEdge> objectPropertyEdgeCollection1 = roModel.getObjectProperties("UserHasPhotoAlbum");
		Collection<ObjectPropertyEdge> objectPropertyEdgeCollection2 = roModel.getObjectProperties("UserHasPhotoAlbums");
		Collection<ObjectPropertyEdge> objectPropertyEdgeCollection3 = roModel.getObjectProperties("ConnectPeople");
		
		Assert.assertEquals(3,objectPropertyEdgeCollection1.size());
		Assert.assertNull(objectPropertyEdgeCollection2);
		Assert.assertEquals(2,objectPropertyEdgeCollection3.size());
		
		Set<String> objectPropertyNameSet1 = new TreeSet<String>();
		Set<String> objectPropertyNameSet2 = new TreeSet<String>();
		
		for (ObjectPropertyEdge objectPropertyEdge : objectPropertyEdgeCollection1)
		{
			objectPropertyNameSet1.add(objectPropertyEdge.getSimpleName());
		}
		// end for
		
		for (ObjectPropertyEdge objectPropertyEdge : objectPropertyEdgeCollection3)
		{
			objectPropertyNameSet2.add(objectPropertyEdge.getSimpleName());
		}
		// end for
		
		Assert.assertFalse(objectPropertyNameSet1.contains("isAlternativeTo"));
		Assert.assertFalse(objectPropertyNameSet1.contains("isInConflictWith"));
		Assert.assertTrue(objectPropertyNameSet1.contains("isPositiveContributionTo"));
		Assert.assertTrue(objectPropertyNameSet1.contains("isAuthoredBy"));
		Assert.assertTrue(objectPropertyNameSet1.contains("isNegativeContributionTo"));
		Assert.assertTrue(objectPropertyNameSet2.contains("isAuthoredBy"));
		Assert.assertTrue(objectPropertyNameSet2.contains("refinesTo"));
	}
	// end testGetObjectPropertiesString
}
// end class DirectedROModelTest