package junit.requirementsontology.test;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.deft.requirementsontology.GeneralTraceabilityMatrix;
import org.deft.requirementsontology.OntologyAccessor;
import org.deft.requirementsontology.ROModel;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

public class GeneralTraceabilityMatrixTest
{
	private static OntologyAccessor oA;
	private static ROModel roModel;
	private static List<String> individualList1;
	private static List<String> individualList2;
	private static GeneralTraceabilityMatrix traceabilityMatrix;
	
	@BeforeClass
	public static void setUpBeforeClass() throws Exception
	{
		oA = new OntologyAccessor("RO_version_1_4_1_social_network_consistent.owl");
		roModel = new ROModel(oA);
		individualList1 = new ArrayList<String>(9);
		individualList1.add("DataControl");
		individualList1.add("ExchangeInformation");
		individualList1.add("TomBeger");
		individualList1.add("LoadPageTimeRestriction");
		individualList1.add("AccessibleToBlindUsers");
		individualList1.add("DevelopSocialNetworkingPlatform");
		individualList1.add("UserHasPhotoAlbum");
		individualList1.add("HighNumberOfUsers");
		individualList1.add("ErikTittel");
		individualList2 = new ArrayList<String>(7);
		individualList2.add("DataControl");
		individualList2.add("KatjaSiegemund");
		individualList2.add("ExchangeInformation");
		individualList2.add("UserHasPhotoAlbum");
		individualList2.add("TomBeger");
		individualList2.add("UserCanHideData");
		individualList2.add("ErikTittel");
		traceabilityMatrix = new GeneralTraceabilityMatrix(individualList1,individualList2,roModel);
	}
	// end setUpBeforeClass

	@Ignore
	public void testGetJTableMatrix()
	{
		Assert.fail("Not yet implemented");
	}
	// end testGetJTableMatrix

	@Test (expected=IllegalArgumentException.class)
	public void testGeneralTraceabilityMatrix()
	{
		List<String> list1 = new ArrayList<String>(3);
		List<String> list2 = new ArrayList<String>(3);
		list1.add("DataControl");
		list1.add("ExchangeInformation");
		list1.add("TomBeger");
		list2.add("UserHasProfilePage");
		list2.add("UserCanHideData");
		ROModel roModel1 = new ROModel(oA);
		ROModel roModel2 = new ROModel(oA);
		roModel2.removeAll();
		GeneralTraceabilityMatrix matrix1 = new GeneralTraceabilityMatrix(list1,list2,roModel1);
		
		Assert.assertNotNull(matrix1);
		Assert.assertEquals(3,matrix1.getSourceIndividuals().length);
		Assert.assertEquals(2,matrix1.getTargetIndividuals().length);
		
		GeneralTraceabilityMatrix matrix2 = new GeneralTraceabilityMatrix(list1,list2,roModel2);
		
		Assert.assertNull(matrix2);
	}
	// end testGeneralTraceabilityMatrix

	@Test
	public void testGetSourceIndividuals()
	{
		String[] sourceIndividuals = traceabilityMatrix.getSourceIndividuals();
		List<String> sourceIndividualList = Arrays.asList(sourceIndividuals);
		
		Assert.assertEquals(9,sourceIndividuals.length);
		Assert.assertTrue(sourceIndividualList.contains("DataControl"));
		Assert.assertTrue(sourceIndividualList.contains("ExchangeInformation"));
		Assert.assertTrue(sourceIndividualList.contains("TomBeger"));
		Assert.assertTrue(sourceIndividualList.contains("LoadPageTimeRestriction"));
		Assert.assertTrue(sourceIndividualList.contains("AccessibleToBlindUsers"));
		Assert.assertTrue(sourceIndividualList.contains("DevelopSocialNetworkingPlatform"));
		Assert.assertTrue(sourceIndividualList.contains("UserHasPhotoAlbum"));
		Assert.assertTrue(sourceIndividualList.contains("HighNumberOfUsers"));
		Assert.assertTrue(sourceIndividualList.contains("ErikTittel"));
	}
	// end testGetSourceIndividuals

	@Test
	public void testGetTargetIndividuals()
	{
		String[] targetIndividuals = traceabilityMatrix.getTargetIndividuals();
		List<String> targetIndividualList = Arrays.asList(targetIndividuals);
		
		Assert.assertEquals(7,targetIndividuals.length);
		Assert.assertTrue(targetIndividualList.contains("DataControl"));
		Assert.assertTrue(targetIndividualList.contains("KatjaSiegemund"));
		Assert.assertTrue(targetIndividualList.contains("ExchangeInformation"));
		Assert.assertTrue(targetIndividualList.contains("UserHasPhotoAlbum"));
		Assert.assertTrue(targetIndividualList.contains("TomBeger"));
		Assert.assertTrue(targetIndividualList.contains("UserCanHideData"));
		Assert.assertTrue(targetIndividualList.contains("ErikTittel"));
	}
	// end testGetTargetIndividuals

	@Test
	public void testSetCellContent()
	{
		String[][] matrixCells = traceabilityMatrix.getMatrixCells();
		
		Assert.assertEquals("",matrixCells[0][0]);
		
		traceabilityMatrix.setCellContent(0,0,"refinesTo");
		
		matrixCells = traceabilityMatrix.getMatrixCells();
		
		Assert.assertEquals("refinesTo",matrixCells[0][0]);
		
		// return to the original value
		traceabilityMatrix.setCellContent(0,0,"");
	}
	// end testSetCellContent

	@Test
	public void testGetMatrixCells()
	{
		String[][] matrixCells = traceabilityMatrix.getMatrixCells();
		Assert.assertEquals(9,matrixCells.length);
		Assert.assertEquals(7,matrixCells[0].length);
		Assert.assertEquals("",matrixCells[0][0]);
		Assert.assertEquals("",matrixCells[1][0]);
		Assert.assertEquals("",matrixCells[2][0]);
		Assert.assertEquals("",matrixCells[3][0]);
		Assert.assertEquals("",matrixCells[4][0]);
		Assert.assertEquals("",matrixCells[5][0]);
		Assert.assertEquals("",matrixCells[6][0]);
		Assert.assertEquals("",matrixCells[7][0]);
		Assert.assertEquals("",matrixCells[8][0]);
		Assert.assertEquals("isAuthoredBy",matrixCells[0][1]);
		Assert.assertEquals("",matrixCells[1][1]);
		Assert.assertEquals("",matrixCells[2][1]);
		Assert.assertEquals("",matrixCells[3][1]);
		Assert.assertEquals("isAuthoredBy",matrixCells[4][1]);
		Assert.assertEquals("",matrixCells[5][1]);
		Assert.assertEquals("",matrixCells[6][1]);
		Assert.assertEquals("",matrixCells[7][1]);
		Assert.assertEquals("",matrixCells[8][1]);
		Assert.assertEquals("",matrixCells[0][2]);
		Assert.assertEquals("",matrixCells[1][2]);
		Assert.assertEquals("",matrixCells[2][2]);
		Assert.assertEquals("",matrixCells[3][2]);
		Assert.assertEquals("",matrixCells[4][2]);
		Assert.assertEquals("refinesTo",matrixCells[5][2]);
		Assert.assertEquals("isPositiveContributionTo",matrixCells[6][2]);
		Assert.assertEquals("",matrixCells[7][2]);
		Assert.assertEquals("",matrixCells[8][2]);
		Assert.assertEquals("",matrixCells[0][3]);
		Assert.assertEquals("",matrixCells[1][3]);
		Assert.assertEquals("",matrixCells[2][3]);
		Assert.assertEquals("",matrixCells[3][3]);
		Assert.assertEquals("isInConflictWith",matrixCells[4][3]);
		Assert.assertEquals("",matrixCells[5][3]);
		Assert.assertEquals("",matrixCells[6][3]);
		Assert.assertEquals("",matrixCells[7][3]);
		Assert.assertEquals("",matrixCells[8][3]);
		Assert.assertEquals("",matrixCells[0][4]);
		Assert.assertEquals("isAuthoredBy",matrixCells[1][4]);
		Assert.assertEquals("",matrixCells[2][4]);
		Assert.assertEquals("",matrixCells[3][4]);
		Assert.assertEquals("",matrixCells[4][4]);
		Assert.assertEquals("isAuthoredBy",matrixCells[5][4]);
		Assert.assertEquals("",matrixCells[6][4]);
		Assert.assertEquals("isAuthoredBy",matrixCells[7][4]);
		Assert.assertEquals("",matrixCells[8][4]);
		Assert.assertEquals("hasRefinementTarget",matrixCells[0][5]);
		Assert.assertEquals("",matrixCells[1][5]);
		Assert.assertEquals("",matrixCells[2][5]);
		Assert.assertEquals("",matrixCells[3][5]);
		Assert.assertEquals("",matrixCells[4][5]);
		Assert.assertEquals("",matrixCells[5][5]);
		Assert.assertEquals("",matrixCells[6][5]);
		Assert.assertEquals("",matrixCells[7][5]);
		Assert.assertEquals("",matrixCells[8][5]);
		Assert.assertEquals("",matrixCells[0][6]);
		Assert.assertEquals("isAuthoredBy",matrixCells[1][6]);
		Assert.assertEquals("",matrixCells[2][6]);
		Assert.assertEquals("isAuthoredBy",matrixCells[3][6]);
		Assert.assertEquals("",matrixCells[4][6]);
		Assert.assertEquals("isAuthoredBy",matrixCells[5][6]);
		Assert.assertEquals("isAuthoredBy",matrixCells[6][6]);
		Assert.assertEquals("",matrixCells[7][6]);
		Assert.assertEquals("",matrixCells[8][6]);
	}
	// end testGetMatrixCells
}
// end class GeneralTraceabilityMatrixTest