package junit.requirementsontology.test;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.apache.commons.collections15.multimap.MultiHashMap;
import org.deft.requirementsontology.IndividualVertex;
import org.deft.requirementsontology.OntologyAccessor;
import org.deft.requirementsontology.ROModel;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

public class IndividualVertexTest
{
	private static OntologyAccessor oA;
	private static ROModel roModel;
	private static List<IndividualVertex> individualList;
	
	@BeforeClass
	public static void setUpBeforeClass() throws Exception
	{
		oA = new OntologyAccessor("RO_version_1_4_1_social_network_consistent.owl");
		roModel = new ROModel(oA);
		individualList = new ArrayList<IndividualVertex>(8);
		individualList.add(createIndividual("DataControl"));
		individualList.add(createIndividual("ExchangeInformation"));
		individualList.add(createIndividual("TomBeger"));
		individualList.add(createIndividual("LoadPageTimeRestriction"));
		individualList.add(createIndividual("AccessibleToBlindUsers"));
		individualList.add(createIndividual("DevelopSocialNetworkingPlatform"));
		individualList.add(createIndividual("UserHasPhotoAlbum"));
		individualList.add(createIndividual("HighNumberOfUsers"));
		individualList.add(createIndividual("TestCase1"));
	}
	// end setUpBeforeClass
	
	private static IndividualVertex createIndividual(String simpleIndividualName)
	{
		String individualEssetialType = roModel.getIndividualEssentialType(simpleIndividualName);
		String individualMainType = roModel.getIndividualMainType(simpleIndividualName);
		Set<String> individualComments = roModel.getIndividualComments(simpleIndividualName);
		Set<String> individualAttributes = roModel.getIndividualAttributes(simpleIndividualName);
		ArrayList<MultiHashMap<String,String>> individualData = roModel.getIndividualData(simpleIndividualName);
		return new IndividualVertex(simpleIndividualName,individualEssetialType,individualMainType,individualComments,individualAttributes,individualData.get(0),individualData.get(1),individualData.get(2),individualData.get(3),individualData.get(4),individualData.get(5),individualData.get(6),individualData.get(7));
	}
	// end createIndividual

	@Test
	public void testIndividualVertex()
	{
		IndividualVertex individual1 = createIndividual("DataControl");
		IndividualVertex individual2 = createIndividual("DataControls");

		Assert.assertEquals("DataControl",individual1.getSimpleName());
		Assert.assertEquals("Refinement",individual1.getEssentialType());
		Assert.assertEquals("Refinement",individual1.getMainType());
		Assert.assertEquals(0,individual1.getComments().size());
		Assert.assertEquals(0,individual1.getAttributes().size());
		Assert.assertEquals(2,individual1.getFormalAttributes().size());		
		Assert.assertEquals(2,individual1.getFormalAttributesWithContent().size());
		Assert.assertEquals(1,individual1.getAdditionalObjectProperties().size());
		Assert.assertEquals(1,individual1.getAdditionalObjectPropertiesWithContent().size());
		Assert.assertEquals(1,individual1.getAdditionalDataProperties().size());
		Assert.assertEquals(1,individual1.getAdditionalDataPropertiesWithContent().size());
		Assert.assertEquals(0,individual1.getIncomingObjectProperties().size());
		Assert.assertEquals(0,individual1.getIncomingObjectPropertiesWithContent().size());
		Assert.assertEquals("",individual2.getEssentialType());
		Assert.assertEquals("",individual2.getMainType());
		Assert.assertEquals(0,individual2.getComments().size());
		Assert.assertEquals(0,individual2.getAttributes().size());
		Assert.assertEquals(0,individual2.getFormalAttributes().size());
		Assert.assertEquals(0,individual2.getFormalAttributesWithContent().size());
		Assert.assertEquals(0,individual2.getAdditionalObjectProperties().size());
		Assert.assertEquals(0,individual2.getAdditionalObjectPropertiesWithContent().size());
		Assert.assertEquals(0,individual2.getAdditionalDataProperties().size());
		Assert.assertEquals(0,individual2.getAdditionalDataPropertiesWithContent().size());
		Assert.assertEquals(0,individual2.getIncomingObjectProperties().size());
		Assert.assertEquals(0,individual2.getIncomingObjectPropertiesWithContent().size());
		
		ArrayList<MultiHashMap<String,String>> individualData1 = roModel.getIndividualData("DataControl");
		
		Assert.assertEquals(individualData1.get(0),individual1.getFormalAttributes());
		Assert.assertEquals(individualData1.get(1),individual1.getFormalAttributesWithContent());
		Assert.assertEquals(individualData1.get(2),individual1.getAdditionalObjectProperties());
		Assert.assertEquals(individualData1.get(3),individual1.getAdditionalObjectPropertiesWithContent());
		Assert.assertEquals(individualData1.get(4),individual1.getAdditionalDataProperties());
		Assert.assertEquals(individualData1.get(5),individual1.getAdditionalDataPropertiesWithContent());
		Assert.assertEquals(individualData1.get(6),individual1.getIncomingObjectProperties());
		Assert.assertEquals(individualData1.get(7),individual1.getIncomingObjectPropertiesWithContent());
	}
	// end testIndividualVertex

	@Test
	public void testGetSimpleName()
	{
		Assert.assertEquals("DataControl",individualList.get(0).getSimpleName());
		Assert.assertEquals("ExchangeInformation",individualList.get(1).getSimpleName());
		Assert.assertEquals("TomBeger",individualList.get(2).getSimpleName());
		Assert.assertEquals("LoadPageTimeRestriction",individualList.get(3).getSimpleName());
		Assert.assertEquals("AccessibleToBlindUsers",individualList.get(4).getSimpleName());
		Assert.assertEquals("DevelopSocialNetworkingPlatform",individualList.get(5).getSimpleName());
		Assert.assertEquals("UserHasPhotoAlbum",individualList.get(6).getSimpleName());
		Assert.assertEquals("HighNumberOfUsers",individualList.get(7).getSimpleName());
		Assert.assertEquals("TestCase1",individualList.get(8).getSimpleName());
	}
	// end testGetSimpleName

	@Test
	public void testGetEssentialType()
	{
		Assert.assertEquals("Refinement",individualList.get(0).getEssentialType());
		Assert.assertEquals("Goal",individualList.get(1).getEssentialType());
		Assert.assertEquals("Stakeholder",individualList.get(2).getEssentialType());
		Assert.assertEquals("UseCase",individualList.get(3).getEssentialType());
		Assert.assertEquals("UserSatisfaction",individualList.get(4).getEssentialType());
		Assert.assertEquals("Goal",individualList.get(5).getEssentialType());
		Assert.assertEquals("AtomicFunctionalRequirement",individualList.get(6).getEssentialType());
		Assert.assertEquals("AtomicFunctionalRequirement",individualList.get(7).getEssentialType());
		Assert.assertEquals("TestCase",individualList.get(8).getEssentialType());
	}
	// end testGetEssentialType

	@Test
	public void testGetMainType()
	{
		Assert.assertEquals("Refinement",individualList.get(0).getMainType());
		Assert.assertEquals("Goal",individualList.get(1).getMainType());
		Assert.assertEquals("Stakeholder",individualList.get(2).getMainType());
		Assert.assertEquals("Story",individualList.get(3).getMainType());
		Assert.assertEquals("Requirement",individualList.get(4).getMainType());
		Assert.assertEquals("Goal",individualList.get(5).getMainType());
		Assert.assertEquals("Requirement",individualList.get(6).getMainType());
		Assert.assertEquals("Requirement",individualList.get(7).getMainType());
		Assert.assertEquals("TestCase",individualList.get(8).getMainType());
	}
	// end testGetMainType

	@Test
	public void testGetComments()
	{
		Assert.assertEquals(0,individualList.get(0).getComments().size());
		Assert.assertEquals(0,individualList.get(1).getComments().size());
		Assert.assertEquals(0,individualList.get(2).getComments().size());
		Assert.assertEquals(1,individualList.get(3).getComments().size());
		Assert.assertEquals(0,individualList.get(4).getComments().size());
		Assert.assertEquals(0,individualList.get(5).getComments().size());
		Assert.assertEquals(0,individualList.get(6).getComments().size());
		Assert.assertEquals(0,individualList.get(7).getComments().size());
		Assert.assertTrue(individualList.get(3).getComments().contains("Load Page in less than 2s"));
		Assert.assertTrue(individualList.get(8).getComments().contains("The user can view, edit, insert, delete information on profile page."));
	}
	// end testGetComments

	@Test
	public void testGetAttributes()
	{
		Set<String> attributeSet1 = individualList.get(6).getAttributes();
		Set<String> attributeSet2 = individualList.get(7).getAttributes();
		
		Assert.assertEquals(0,individualList.get(0).getAttributes().size());
		Assert.assertEquals(0,individualList.get(1).getAttributes().size());
		Assert.assertEquals(0,individualList.get(2).getAttributes().size());
		Assert.assertEquals(0,individualList.get(3).getAttributes().size());
		Assert.assertEquals(0,individualList.get(4).getAttributes().size());
		Assert.assertEquals(0,individualList.get(5).getAttributes().size());
		Assert.assertEquals(2,attributeSet1.size());
		Assert.assertEquals(1,attributeSet2.size());
		Assert.assertTrue(attributeSet1.contains("LowPriority"));
		Assert.assertTrue(attributeSet1.contains("Optional"));
		Assert.assertTrue(attributeSet2.contains("HighPriority"));
	}
	// end testGetAttributes

	@Test
	public void testGetFormalAttributeNames()
	{
		Set<String> attributeSet1 = individualList.get(0).getFormalAttributeNames();
		Set<String> attributeSet2 = individualList.get(1).getFormalAttributeNames();
		Set<String> attributeSet3 = individualList.get(2).getFormalAttributeNames();
		Set<String> attributeSet4 = individualList.get(3).getFormalAttributeNames();
		Set<String> attributeSet5 = individualList.get(4).getFormalAttributeNames();
		Set<String> attributeSet6 = individualList.get(5).getFormalAttributeNames();
		Set<String> attributeSet7 = individualList.get(6).getFormalAttributeNames();
		Set<String> attributeSet8 = individualList.get(7).getFormalAttributeNames();
		Set<String> attributeSet9 = individualList.get(8).getFormalAttributeNames();
		
		Assert.assertEquals(2,attributeSet1.size());
		Assert.assertEquals(5,attributeSet2.size());
		Assert.assertEquals(0,attributeSet3.size());
		Assert.assertEquals(6,attributeSet4.size());
		Assert.assertEquals(10,attributeSet5.size());
		Assert.assertEquals(5,attributeSet6.size());
		Assert.assertEquals(11,attributeSet7.size());
		Assert.assertEquals(11,attributeSet8.size());
		Assert.assertEquals(2,attributeSet9.size());
		Assert.assertTrue(attributeSet1.contains("hasRefinementSource"));
		Assert.assertTrue(attributeSet1.contains("hasRefinementTarget"));
		Assert.assertTrue(attributeSet2.contains("refinesTo"));
		Assert.assertTrue(attributeSet2.contains("hasRisk"));
		Assert.assertTrue(attributeSet2.contains("hasID"));
		Assert.assertTrue(attributeSet2.contains("hasObstacle"));
		Assert.assertTrue(attributeSet2.contains("isAuthoredBy"));
		Assert.assertTrue(attributeSet4.contains("describesRequirement"));
		Assert.assertTrue(attributeSet4.contains("hasScenario"));
		Assert.assertTrue(attributeSet4.contains("hasRisk"));
		Assert.assertTrue(attributeSet4.contains("hasID"));
		Assert.assertTrue(attributeSet4.contains("hasObstacle"));
		Assert.assertTrue(attributeSet4.contains("isAuthoredBy"));
		Assert.assertTrue(attributeSet9.contains("testsRequirement"));
		Assert.assertTrue(attributeSet9.contains("isAuthoredBy"));
	}
	// end testGetFormalAttributeNames

	@Test
	public void testGetFormalAttributeTypes()
	{
		Set<String> attributeSet = individualList.get(1).getFormalAttributeNames();
		
		Assert.assertTrue(attributeSet.contains("refinesTo"));
		Assert.assertTrue(attributeSet.contains("hasRisk"));
		Assert.assertTrue(attributeSet.contains("hasID"));
		Assert.assertTrue(attributeSet.contains("hasObstacle"));
		Assert.assertTrue(attributeSet.contains("isAuthoredBy"));
				
		for (String attributeName : attributeSet)
		{
			Set<String> attributeTypeSet = individualList.get(1).getFormalAttributeTypes(attributeName);
			
			if (attributeName.equals("refinesTo"))
			{
				Assert.assertEquals(2,attributeTypeSet.size());
				Assert.assertTrue(attributeTypeSet.contains("Goal"));
				Assert.assertTrue(attributeTypeSet.contains("Requirement"));
			}
			else if (attributeName.equals("hasRisk"))
			{
				Assert.assertEquals(1,attributeTypeSet.size());
				Assert.assertTrue(attributeTypeSet.contains("Risk"));
			}
			else if (attributeName.equals("hasID"))
			{
				Assert.assertEquals(1,attributeTypeSet.size());
				Assert.assertTrue(attributeTypeSet.contains("string"));
			}
			else if (attributeName.equals("hasObstacle"))
			{
				Assert.assertEquals(1,attributeTypeSet.size());
				Assert.assertTrue(attributeTypeSet.contains("Obstacle"));
			}
			else if (attributeName.equals("isAuthoredBy"))
			{
				Assert.assertEquals(1,attributeTypeSet.size());
				Assert.assertTrue(attributeTypeSet.contains("Stakeholder"));
			}
			// end if else if
		}
		// end for
	}
	// end testGetFormalAttributeTypes

	@Test
	public void testGetFormalAttributeIndividuals()
	{
		Set<String> attributeSet = individualList.get(1).getFormalAttributeNames();
		
		Assert.assertTrue(attributeSet.contains("refinesTo"));
		Assert.assertTrue(attributeSet.contains("hasRisk"));
		Assert.assertTrue(attributeSet.contains("hasID"));
		Assert.assertTrue(attributeSet.contains("hasObstacle"));
		Assert.assertTrue(attributeSet.contains("isAuthoredBy"));

		for (String attributeName : attributeSet)
		{
			Set<String> attributeTypeSet = individualList.get(1).getFormalAttributeTypes(attributeName);
			
			if (attributeName.equals("refinesTo"))
			{
				Assert.assertEquals(2,attributeTypeSet.size());
				Assert.assertTrue(attributeTypeSet.contains("Goal"));
				Assert.assertTrue(attributeTypeSet.contains("Requirement"));
			}
			else if (attributeName.equals("hasRisk"))
			{
				Assert.assertEquals(1,attributeTypeSet.size());
				Assert.assertTrue(attributeTypeSet.contains("Risk"));
			}
			else if (attributeName.equals("hasID"))
			{
				Assert.assertEquals(1,attributeTypeSet.size());
				Assert.assertTrue(attributeTypeSet.contains("string"));
			}
			else if (attributeName.equals("hasObstacle"))
			{
				Assert.assertEquals(1,attributeTypeSet.size());
				Assert.assertTrue(attributeTypeSet.contains("Obstacle"));
			}
			else if (attributeName.equals("isAuthoredBy"))
			{
				Assert.assertEquals(1,attributeTypeSet.size());
				Assert.assertTrue(attributeTypeSet.contains("Stakeholder"));
			}
			// end if else if
					
			for (String attributeType : attributeTypeSet)
			{
				Set<String> individualSet = individualList.get(1).getFormalAttributeIndividuals(attributeName,attributeType);
				
				if (attributeName.equals("refinesTo") && attributeType.equals("Goal"))
				{
					Assert.assertEquals(2,individualSet.size());
					Assert.assertTrue(individualSet.contains("StoreInformation"));
					Assert.assertTrue(individualSet.contains("CollectUserData"));
				}
				else if (attributeName.equals("refinesTo") && attributeType.equals("Requirement"))
				{
					Assert.assertEquals(1,individualSet.size());
					Assert.assertTrue(individualSet.contains(""));
				}
				else if (attributeName.equals("hasRisk") && attributeType.equals("Risk"))
				{
					Assert.assertEquals(1,attributeTypeSet.size());
					Assert.assertTrue(individualSet.contains(""));
				}
				else if (attributeName.equals("hasID") && attributeType.equals("string"))
				{
					Assert.assertEquals(1,attributeTypeSet.size());
					Assert.assertTrue(individualSet.contains(""));
				}
				else if (attributeName.equals("hasObstacle") && attributeType.equals("Obstacle"))
				{
					Assert.assertEquals(1,attributeTypeSet.size());
					Assert.assertTrue(individualSet.contains(""));
				}
				else if (attributeName.equals("isAuthoredBy") && attributeType.equals("Stakeholder"))
				{
					Assert.assertEquals(2,individualSet.size());
					Assert.assertTrue(individualSet.contains("TomBeger"));
					Assert.assertTrue(individualSet.contains("ErikTittel"));
				}
				// end if else if
			}
			// end for
		}
		// end for
	}
	// end testGetFormalAttributeIndividuals
	
	@Test
	public void testGetAdditionalObjectPropertyNames()
	{
		Set<String> objectPropertySet1 = individualList.get(0).getAdditionalObjectPropertyNames();
		Set<String> objectPropertySet2 = individualList.get(1).getAdditionalObjectPropertyNames();
		Set<String> objectPropertySet3 = individualList.get(2).getAdditionalObjectPropertyNames();
		Set<String> objectPropertySet4 = individualList.get(3).getAdditionalObjectPropertyNames();
		Set<String> objectPropertySet5 = individualList.get(4).getAdditionalObjectPropertyNames();
		Set<String> objectPropertySet6 = individualList.get(5).getAdditionalObjectPropertyNames();
		Set<String> objectPropertySet7 = individualList.get(6).getAdditionalObjectPropertyNames();
		Set<String> objectPropertySet8 = individualList.get(7).getAdditionalObjectPropertyNames();
		Set<String> objectPropertySet9 = individualList.get(8).getAdditionalObjectPropertyNames();

		Assert.assertEquals(1,objectPropertySet1.size());
		Assert.assertEquals(2,objectPropertySet2.size());
		Assert.assertEquals(0,objectPropertySet3.size());
		Assert.assertEquals(0,objectPropertySet4.size());
		Assert.assertEquals(1,objectPropertySet5.size());
		Assert.assertEquals(0,objectPropertySet6.size());
		Assert.assertEquals(2,objectPropertySet7.size());
		Assert.assertEquals(1,objectPropertySet8.size());
		Assert.assertEquals(0,objectPropertySet9.size());
		Assert.assertTrue(objectPropertySet1.contains("isAuthoredBy"));
		Assert.assertTrue(objectPropertySet2.contains("isConnectedToUseCase"));
		Assert.assertTrue(objectPropertySet2.contains("isNegativeContributionTo"));
		Assert.assertTrue(objectPropertySet5.contains("isInConflictWith"));
		Assert.assertTrue(objectPropertySet7.contains("isPositiveContributionTo"));
		Assert.assertTrue(objectPropertySet7.contains("isNegativeContributionTo"));
		Assert.assertTrue(objectPropertySet8.contains("isNegativeContributionTo"));
	}
	// end testGetAdditionalObjectPropertyNames

	@Test
	public void testGetAdditionalObjectPropertyType()
	{
		Set<String> objectPropertySet1 = individualList.get(0).getAdditionalObjectPropertyNames();
		Set<String> objectPropertySet2 = individualList.get(1).getAdditionalObjectPropertyNames();
		Set<String> objectPropertySet3 = individualList.get(4).getAdditionalObjectPropertyNames();
		String objectPropertyType1 = individualList.get(0).getAdditionalObjectPropertyType("isAuthoredBy");
		String objectPropertyType2 = individualList.get(1).getAdditionalObjectPropertyType("isConnectedToUseCase");
		String objectPropertyType3 = individualList.get(1).getAdditionalObjectPropertyType("isNegativeContributionTo");
		String objectPropertyType4 = individualList.get(5).getAdditionalObjectPropertyType("isInConflictWith");
		
		Assert.assertTrue(objectPropertySet1.contains("isAuthoredBy"));
		Assert.assertTrue(objectPropertySet2.contains("isConnectedToUseCase"));
		Assert.assertTrue(objectPropertySet2.contains("isNegativeContributionTo"));		
		Assert.assertTrue(objectPropertySet3.contains("isInConflictWith"));
		Assert.assertEquals("Stakeholder",objectPropertyType1);
		Assert.assertEquals("UseCase",objectPropertyType2);
		Assert.assertEquals("RequirementArtifact",objectPropertyType3);
		Assert.assertNull(objectPropertyType4);
	}
	// end testGetAdditionalObjectPropertyType

	@Test
	public void testGetAdditionalObjectPropertyIndividuals()
	{
		Set<String> objectPropertySet1 = individualList.get(0).getAdditionalObjectPropertyNames();
		Set<String> objectPropertySet2 = individualList.get(1).getAdditionalObjectPropertyNames();
		
		Assert.assertTrue(objectPropertySet1.contains("isAuthoredBy"));
		Assert.assertTrue(objectPropertySet2.contains("isConnectedToUseCase"));
		Assert.assertTrue(objectPropertySet2.contains("isNegativeContributionTo"));	
		
		String objectPropertyType1 = individualList.get(0).getAdditionalObjectPropertyType("isAuthoredBy");
		String objectPropertyType2 = individualList.get(1).getAdditionalObjectPropertyType("isConnectedToUseCase");
		String objectPropertyType3 = individualList.get(1).getAdditionalObjectPropertyType("isNegativeContributionTo");
		
		Assert.assertEquals("Stakeholder",objectPropertyType1);
		Assert.assertEquals("UseCase",objectPropertyType2);
		Assert.assertEquals("RequirementArtifact",objectPropertyType3);

		Set<String> individualSet1 = individualList.get(0).getAdditionalObjectPropertyIndividuals("isAuthoredBy",objectPropertyType1);
		Set<String> individualSet2 = individualList.get(1).getAdditionalObjectPropertyIndividuals("isConnectedToUseCase",objectPropertyType2);
		Set<String> individualSet3 = individualList.get(1).getAdditionalObjectPropertyIndividuals("isNegativeContributionTo",objectPropertyType3);
		
		Assert.assertEquals(1,individualSet1.size());
		Assert.assertEquals(1,individualSet2.size());
		Assert.assertEquals(1,individualSet3.size());
		Assert.assertTrue(individualSet1.contains("KatjaSiegemund"));
		Assert.assertTrue(individualSet2.contains("ProvidingUserInformation"));
		Assert.assertTrue(individualSet3.contains("ServerWithSufficientPerformance"));
	}
	// end testGetAdditionalObjectPropertyIndividuals

	@Test
	public void testGetAdditionalDataPropertyNames()
	{
		Set<String> dataPropertySet1 = individualList.get(0).getAdditionalDataPropertyNames();
		Set<String> dataPropertySet2 = individualList.get(1).getAdditionalDataPropertyNames();
		Set<String> dataPropertySet3 = individualList.get(2).getAdditionalDataPropertyNames();
		Set<String> dataPropertySet4 = individualList.get(3).getAdditionalDataPropertyNames();
		Set<String> dataPropertySet5 = individualList.get(4).getAdditionalDataPropertyNames();
		Set<String> dataPropertySet6 = individualList.get(5).getAdditionalDataPropertyNames();
		Set<String> dataPropertySet7 = individualList.get(6).getAdditionalDataPropertyNames();
		Set<String> dataPropertySet8 = individualList.get(7).getAdditionalDataPropertyNames();
		Set<String> dataPropertySet9 = individualList.get(8).getAdditionalDataPropertyNames();

		Assert.assertEquals(1,dataPropertySet1.size());
		Assert.assertEquals(0,dataPropertySet2.size());
		Assert.assertEquals(0,dataPropertySet3.size());
		Assert.assertEquals(0,dataPropertySet4.size());
		Assert.assertEquals(0,dataPropertySet5.size());
		Assert.assertEquals(0,dataPropertySet6.size());
		Assert.assertEquals(0,dataPropertySet7.size());
		Assert.assertEquals(0,dataPropertySet8.size());
		Assert.assertEquals(0,dataPropertySet9.size());
		Assert.assertTrue(dataPropertySet1.contains("hasRefinementReason"));
	}
	// end testGetAdditionalDataPropertyNames
	
	@Test
	public void testGetAdditionalDataPropertyType()
	{
		Set<String> dataPropertySet = individualList.get(0).getAdditionalDataPropertyNames();
		String dataPropertyType = individualList.get(0).getAdditionalDataPropertyType("hasRefinementReason");
		
		Assert.assertTrue(dataPropertySet.contains("hasRefinementReason"));
		Assert.assertEquals("string",dataPropertyType);
	}
	// end testGetAdditionalDataPropertyType

	@Test
	public void testGetAdditionalDataPropertyValues()
	{
		Set<String> dataPropertySet = individualList.get(0).getAdditionalDataPropertyNames();
		String dataPropertyType = individualList.get(0).getAdditionalDataPropertyType("hasRefinementReason");
		Set<String> dataPropertyValues = individualList.get(0).getAdditionalDataPropertyValues("hasRefinementReason","string");
		
		Assert.assertTrue(dataPropertySet.contains("hasRefinementReason"));
		Assert.assertEquals("string",dataPropertyType);
		Assert.assertEquals(1,dataPropertyValues.size());
		Assert.assertTrue(dataPropertyValues.contains("The user must be allowed to completely control which data shall be displayed and which not in order to satisfy data security plocies from government."));
	}
	// end testGetAdditionalDataPropertyValues

	@Test
	public void testGetIncomingObjectPropertyNames()
	{
		Set<String> incomingObjectPropertySet1 = individualList.get(0).getIncomingObjectPropertyNames();
		Set<String> incomingObjectPropertySet2 = individualList.get(1).getIncomingObjectPropertyNames();
		Set<String> incomingObjectPropertySet3 = individualList.get(2).getIncomingObjectPropertyNames();
		Set<String> incomingObjectPropertySet4 = individualList.get(3).getIncomingObjectPropertyNames();
		Set<String> incomingObjectPropertySet5 = individualList.get(4).getIncomingObjectPropertyNames();
		Set<String> incomingObjectPropertySet6 = individualList.get(5).getIncomingObjectPropertyNames();
		Set<String> incomingObjectPropertySet7 = individualList.get(6).getIncomingObjectPropertyNames();
		Set<String> incomingObjectPropertySet8 = individualList.get(7).getIncomingObjectPropertyNames();
		Set<String> incomingObjectPropertySet9 = individualList.get(8).getIncomingObjectPropertyNames();

		Assert.assertEquals(0,incomingObjectPropertySet1.size());
		Assert.assertEquals(2,incomingObjectPropertySet2.size());
		Assert.assertEquals(1,incomingObjectPropertySet3.size());
		Assert.assertEquals(1,incomingObjectPropertySet4.size());
		Assert.assertEquals(1,incomingObjectPropertySet5.size());
		Assert.assertEquals(1,incomingObjectPropertySet6.size());
		Assert.assertEquals(3,incomingObjectPropertySet7.size());
		Assert.assertEquals(0,incomingObjectPropertySet8.size());
		Assert.assertEquals(0,incomingObjectPropertySet9.size());
		Assert.assertTrue(incomingObjectPropertySet2.contains("refinesTo"));
		Assert.assertTrue(incomingObjectPropertySet2.contains("isPositiveContributionTo"));
		Assert.assertTrue(incomingObjectPropertySet3.contains("isAuthoredBy"));
		Assert.assertTrue(incomingObjectPropertySet4.contains("isDescribedByUseCase"));
		Assert.assertTrue(incomingObjectPropertySet5.contains("isNegativeContributionTo"));
		Assert.assertTrue(incomingObjectPropertySet6.contains("refinesTo"));
		Assert.assertTrue(incomingObjectPropertySet7.contains("isInConflictWith"));
		Assert.assertTrue(incomingObjectPropertySet7.contains("isAlternativeTo"));
		Assert.assertTrue(incomingObjectPropertySet7.contains("describesRequirement"));
	}
	// end testGetIncomingObjectPropertyNames

	@Test
	public void testGetIncomingObjectPropertySourceIndividualTypes()
	{
		Set<String> incomingObjectPropertySet1 = individualList.get(1).getIncomingObjectPropertyNames();
		Set<String> incomingObjectPropertySet2 = individualList.get(2).getIncomingObjectPropertyNames();
		Set<String> incomingObjectPropertySet3 = individualList.get(6).getIncomingObjectPropertyNames();
	
		Assert.assertTrue(incomingObjectPropertySet1.contains("refinesTo"));
		Assert.assertTrue(incomingObjectPropertySet1.contains("isPositiveContributionTo"));
		Assert.assertTrue(incomingObjectPropertySet2.contains("isAuthoredBy"));
		Assert.assertTrue(incomingObjectPropertySet3.contains("isInConflictWith"));
		Assert.assertTrue(incomingObjectPropertySet3.contains("isAlternativeTo"));
		Assert.assertTrue(incomingObjectPropertySet3.contains("describesRequirement"));
		
		Set<String> incomingObjectPropertyTypeSet1 = individualList.get(1).getIncomingObjectPropertySourceIndividualTypes("refinesTo");
		Set<String> incomingObjectPropertyTypeSet2 = individualList.get(1).getIncomingObjectPropertySourceIndividualTypes("isPositiveContributionTo");
		Set<String> incomingObjectPropertyTypeSet3 = individualList.get(2).getIncomingObjectPropertySourceIndividualTypes("isAuthoredBy");
		Set<String> incomingObjectPropertyTypeSet4 = individualList.get(6).getIncomingObjectPropertySourceIndividualTypes("isInConflictWith");
		Set<String> incomingObjectPropertyTypeSet5 = individualList.get(6).getIncomingObjectPropertySourceIndividualTypes("isAlternativeTo");
		Set<String> incomingObjectPropertyTypeSet6 = individualList.get(6).getIncomingObjectPropertySourceIndividualTypes("describesRequirement");
		
		Assert.assertEquals(1,incomingObjectPropertyTypeSet1.size());
		Assert.assertEquals(1,incomingObjectPropertyTypeSet2.size());
		Assert.assertEquals(6,incomingObjectPropertyTypeSet3.size()); 
		Assert.assertEquals(1,incomingObjectPropertyTypeSet4.size());
		Assert.assertEquals(1,incomingObjectPropertyTypeSet5.size());
		Assert.assertEquals(1,incomingObjectPropertyTypeSet6.size());
		Assert.assertTrue(incomingObjectPropertyTypeSet1.contains("Goal"));
		Assert.assertTrue(incomingObjectPropertyTypeSet2.contains("AtomicFunctionalRequirement"));
		Assert.assertTrue(incomingObjectPropertyTypeSet3.contains("AbstractFunctionalRequirement"));
		Assert.assertTrue(incomingObjectPropertyTypeSet3.contains("AtomicFunctionalRequirement"));
		Assert.assertTrue(incomingObjectPropertyTypeSet3.contains("Goal"));
		Assert.assertTrue(incomingObjectPropertyTypeSet3.contains("SystemSoftware"));
		Assert.assertTrue(incomingObjectPropertyTypeSet3.contains("Understandability"));
		Assert.assertTrue(incomingObjectPropertyTypeSet4.contains("UserSatisfaction"));
		Assert.assertTrue(incomingObjectPropertyTypeSet5.contains("AtomicFunctionalRequirement"));
		Assert.assertTrue(incomingObjectPropertyTypeSet6.contains("UseCase"));
	}
	// end testGetIncomingObjectPropertySourceIndividualTypes

	@Test
	public void testGetIncomingObjectPropertySourceIndividuals()
	{
		Set<String> incomingObjectPropertySet = individualList.get(2).getIncomingObjectPropertyNames();
		Set<String> incomingObjectPropertyTypeSet = individualList.get(2).getIncomingObjectPropertySourceIndividualTypes("isAuthoredBy");
		
		Assert.assertTrue(incomingObjectPropertySet.contains("isAuthoredBy"));
		Assert.assertTrue(incomingObjectPropertyTypeSet.contains("AbstractFunctionalRequirement"));
		Assert.assertTrue(incomingObjectPropertyTypeSet.contains("AtomicFunctionalRequirement"));
		Assert.assertTrue(incomingObjectPropertyTypeSet.contains("Goal"));
		Assert.assertTrue(incomingObjectPropertyTypeSet.contains("SystemSoftware"));
		Assert.assertTrue(incomingObjectPropertyTypeSet.contains("Understandability"));
		
		Set<String> incomingObjectPropertySourceIndividualSet1 = individualList.get(2).getIncomingObjectPropertySourceIndividuals("isAuthoredBy","AbstractFunctionalRequirement");
		Set<String> incomingObjectPropertySourceIndividualSet2 = individualList.get(2).getIncomingObjectPropertySourceIndividuals("isAuthoredBy","AtomicFunctionalRequirement");
		Set<String> incomingObjectPropertySourceIndividualSet3 = individualList.get(2).getIncomingObjectPropertySourceIndividuals("isAuthoredBy","Goal");
		Set<String> incomingObjectPropertySourceIndividualSet4 = individualList.get(2).getIncomingObjectPropertySourceIndividuals("isAuthoredBy","SystemSoftware");
		Set<String> incomingObjectPropertySourceIndividualSet5 = individualList.get(2).getIncomingObjectPropertySourceIndividuals("isAuthoredBy","Understandability");
		
		Assert.assertEquals(1,incomingObjectPropertySourceIndividualSet1.size());
		Assert.assertEquals(1,incomingObjectPropertySourceIndividualSet2.size());
		Assert.assertEquals(9,incomingObjectPropertySourceIndividualSet3.size()); 
		Assert.assertEquals(2,incomingObjectPropertySourceIndividualSet4.size());
		Assert.assertEquals(1,incomingObjectPropertySourceIndividualSet5.size());
		Assert.assertTrue(incomingObjectPropertySourceIndividualSet1.contains("AdvertisementOnProfilePage"));
		Assert.assertTrue(incomingObjectPropertySourceIndividualSet2.contains("HighNumberOfUsers"));
		Assert.assertTrue(incomingObjectPropertySourceIndividualSet3.contains("CollectUserData"));
		Assert.assertTrue(incomingObjectPropertySourceIndividualSet3.contains("CommercialSuccess"));
		Assert.assertTrue(incomingObjectPropertySourceIndividualSet3.contains("DataPrivacyProtection"));
		Assert.assertTrue(incomingObjectPropertySourceIndividualSet3.contains("DevelopSocialNetworkingPlatform"));
		Assert.assertTrue(incomingObjectPropertySourceIndividualSet3.contains("EarnMoney"));
		Assert.assertTrue(incomingObjectPropertySourceIndividualSet3.contains("ExchangeInformation"));
		Assert.assertTrue(incomingObjectPropertySourceIndividualSet3.contains("FastAccessToPlatform"));
		Assert.assertTrue(incomingObjectPropertySourceIndividualSet3.contains("PlatformIsEasyToUseForEveryone"));
		Assert.assertTrue(incomingObjectPropertySourceIndividualSet3.contains("StoreInformation"));
		Assert.assertTrue(incomingObjectPropertySourceIndividualSet4.contains("StoreUserDataInDatabase"));
		Assert.assertTrue(incomingObjectPropertySourceIndividualSet4.contains("UsingOracleDatabase"));
		Assert.assertTrue(incomingObjectPropertySourceIndividualSet5.contains("WellArrangedLayout"));
	}
	// end testGetIncomingObjectPropertySourceIndividuals
		
	@Test
	public void testGetFormalAttributes()
	{
		ArrayList<MultiHashMap<String,String>> individualData1 = roModel.getIndividualData("DataControl");
		ArrayList<MultiHashMap<String,String>> individualData2 = roModel.getIndividualData("ExchangeInformation");
		ArrayList<MultiHashMap<String,String>> individualData3 = roModel.getIndividualData("TomBeger");
		ArrayList<MultiHashMap<String,String>> individualData4 = roModel.getIndividualData("LoadPageTimeRestriction");
		ArrayList<MultiHashMap<String,String>> individualData5 = roModel.getIndividualData("AccessibleToBlindUsers");
		ArrayList<MultiHashMap<String,String>> individualData6 = roModel.getIndividualData("DevelopSocialNetworkingPlatform");
		ArrayList<MultiHashMap<String,String>> individualData7 = roModel.getIndividualData("UserHasPhotoAlbum");
		ArrayList<MultiHashMap<String,String>> individualData8 = roModel.getIndividualData("HighNumberOfUsers");
		ArrayList<MultiHashMap<String,String>> individualData9 = roModel.getIndividualData("TestCase1");
		
		Assert.assertEquals(individualData1.get(0),individualList.get(0).getFormalAttributes());
		Assert.assertEquals(individualData2.get(0),individualList.get(1).getFormalAttributes());
		Assert.assertEquals(individualData3.get(0),individualList.get(2).getFormalAttributes());
		Assert.assertEquals(individualData4.get(0),individualList.get(3).getFormalAttributes());
		Assert.assertEquals(individualData5.get(0),individualList.get(4).getFormalAttributes());
		Assert.assertEquals(individualData6.get(0),individualList.get(5).getFormalAttributes());
		Assert.assertEquals(individualData7.get(0),individualList.get(6).getFormalAttributes());
		Assert.assertEquals(individualData8.get(0),individualList.get(7).getFormalAttributes());
		Assert.assertEquals(individualData9.get(0),individualList.get(8).getFormalAttributes());
	}
	// end testGetFormalAttributes

	@Test
	public void testGetFormalAttributesWithContent()
	{
		ArrayList<MultiHashMap<String,String>> individualData1 = roModel.getIndividualData("DataControl");
		ArrayList<MultiHashMap<String,String>> individualData2 = roModel.getIndividualData("ExchangeInformation");
		ArrayList<MultiHashMap<String,String>> individualData3 = roModel.getIndividualData("TomBeger");
		ArrayList<MultiHashMap<String,String>> individualData4 = roModel.getIndividualData("LoadPageTimeRestriction");
		ArrayList<MultiHashMap<String,String>> individualData5 = roModel.getIndividualData("AccessibleToBlindUsers");
		ArrayList<MultiHashMap<String,String>> individualData6 = roModel.getIndividualData("DevelopSocialNetworkingPlatform");
		ArrayList<MultiHashMap<String,String>> individualData7 = roModel.getIndividualData("UserHasPhotoAlbum");
		ArrayList<MultiHashMap<String,String>> individualData8 = roModel.getIndividualData("HighNumberOfUsers");
		ArrayList<MultiHashMap<String,String>> individualData9 = roModel.getIndividualData("TestCase1");
		
		Assert.assertEquals(individualData1.get(1),individualList.get(0).getFormalAttributesWithContent());
		Assert.assertEquals(individualData2.get(1),individualList.get(1).getFormalAttributesWithContent());
		Assert.assertEquals(individualData3.get(1),individualList.get(2).getFormalAttributesWithContent());
		Assert.assertEquals(individualData4.get(1),individualList.get(3).getFormalAttributesWithContent());
		Assert.assertEquals(individualData5.get(1),individualList.get(4).getFormalAttributesWithContent());
		Assert.assertEquals(individualData6.get(1),individualList.get(5).getFormalAttributesWithContent());
		Assert.assertEquals(individualData7.get(1),individualList.get(6).getFormalAttributesWithContent());
		Assert.assertEquals(individualData8.get(1),individualList.get(7).getFormalAttributesWithContent());
		Assert.assertEquals(individualData9.get(1),individualList.get(8).getFormalAttributesWithContent());
	}
	// end testGetFormalAttributesWithContent

	@Test
	public void testGetAdditionalObjectProperties()
	{
		ArrayList<MultiHashMap<String,String>> individualData1 = roModel.getIndividualData("DataControl");
		ArrayList<MultiHashMap<String,String>> individualData2 = roModel.getIndividualData("ExchangeInformation");
		ArrayList<MultiHashMap<String,String>> individualData3 = roModel.getIndividualData("TomBeger");
		ArrayList<MultiHashMap<String,String>> individualData4 = roModel.getIndividualData("LoadPageTimeRestriction");
		ArrayList<MultiHashMap<String,String>> individualData5 = roModel.getIndividualData("AccessibleToBlindUsers");
		ArrayList<MultiHashMap<String,String>> individualData6 = roModel.getIndividualData("DevelopSocialNetworkingPlatform");
		ArrayList<MultiHashMap<String,String>> individualData7 = roModel.getIndividualData("UserHasPhotoAlbum");
		ArrayList<MultiHashMap<String,String>> individualData8 = roModel.getIndividualData("HighNumberOfUsers");
		ArrayList<MultiHashMap<String,String>> individualData9 = roModel.getIndividualData("TestCase1");
		
		Assert.assertEquals(individualData1.get(2),individualList.get(0).getAdditionalObjectProperties());
		Assert.assertEquals(individualData2.get(2),individualList.get(1).getAdditionalObjectProperties());
		Assert.assertEquals(individualData3.get(2),individualList.get(2).getAdditionalObjectProperties());
		Assert.assertEquals(individualData4.get(2),individualList.get(3).getAdditionalObjectProperties());
		Assert.assertEquals(individualData5.get(2),individualList.get(4).getAdditionalObjectProperties());
		Assert.assertEquals(individualData6.get(2),individualList.get(5).getAdditionalObjectProperties());
		Assert.assertEquals(individualData7.get(2),individualList.get(6).getAdditionalObjectProperties());
		Assert.assertEquals(individualData8.get(2),individualList.get(7).getAdditionalObjectProperties());
		Assert.assertEquals(individualData9.get(2),individualList.get(8).getAdditionalObjectProperties());
	}
	// end testGetAdditionalObjectProperties

	@Test
	public void testGetAdditionalObjectPropertiesWithContent()
	{
		ArrayList<MultiHashMap<String,String>> individualData1 = roModel.getIndividualData("DataControl");
		ArrayList<MultiHashMap<String,String>> individualData2 = roModel.getIndividualData("ExchangeInformation");
		ArrayList<MultiHashMap<String,String>> individualData3 = roModel.getIndividualData("TomBeger");
		ArrayList<MultiHashMap<String,String>> individualData4 = roModel.getIndividualData("LoadPageTimeRestriction");
		ArrayList<MultiHashMap<String,String>> individualData5 = roModel.getIndividualData("AccessibleToBlindUsers");
		ArrayList<MultiHashMap<String,String>> individualData6 = roModel.getIndividualData("DevelopSocialNetworkingPlatform");
		ArrayList<MultiHashMap<String,String>> individualData7 = roModel.getIndividualData("UserHasPhotoAlbum");
		ArrayList<MultiHashMap<String,String>> individualData8 = roModel.getIndividualData("HighNumberOfUsers");
		ArrayList<MultiHashMap<String,String>> individualData9 = roModel.getIndividualData("TestCase1");
		
		Assert.assertEquals(individualData1.get(3),individualList.get(0).getAdditionalObjectPropertiesWithContent());
		Assert.assertEquals(individualData2.get(3),individualList.get(1).getAdditionalObjectPropertiesWithContent());
		Assert.assertEquals(individualData3.get(3),individualList.get(2).getAdditionalObjectPropertiesWithContent());
		Assert.assertEquals(individualData4.get(3),individualList.get(3).getAdditionalObjectPropertiesWithContent());
		Assert.assertEquals(individualData5.get(3),individualList.get(4).getAdditionalObjectPropertiesWithContent());
		Assert.assertEquals(individualData6.get(3),individualList.get(5).getAdditionalObjectPropertiesWithContent());
		Assert.assertEquals(individualData7.get(3),individualList.get(6).getAdditionalObjectPropertiesWithContent());
		Assert.assertEquals(individualData8.get(3),individualList.get(7).getAdditionalObjectPropertiesWithContent());
		Assert.assertEquals(individualData9.get(3),individualList.get(8).getAdditionalObjectPropertiesWithContent());
	}
	// end testGetAdditionalObjectPropertiesWithContent

	@Test
	public void testGetAdditionalDataProperties()
	{
		ArrayList<MultiHashMap<String,String>> individualData1 = roModel.getIndividualData("DataControl");
		ArrayList<MultiHashMap<String,String>> individualData2 = roModel.getIndividualData("ExchangeInformation");
		ArrayList<MultiHashMap<String,String>> individualData3 = roModel.getIndividualData("TomBeger");
		ArrayList<MultiHashMap<String,String>> individualData4 = roModel.getIndividualData("LoadPageTimeRestriction");
		ArrayList<MultiHashMap<String,String>> individualData5 = roModel.getIndividualData("AccessibleToBlindUsers");
		ArrayList<MultiHashMap<String,String>> individualData6 = roModel.getIndividualData("DevelopSocialNetworkingPlatform");
		ArrayList<MultiHashMap<String,String>> individualData7 = roModel.getIndividualData("UserHasPhotoAlbum");
		ArrayList<MultiHashMap<String,String>> individualData8 = roModel.getIndividualData("HighNumberOfUsers");
		ArrayList<MultiHashMap<String,String>> individualData9 = roModel.getIndividualData("TestCase1");
		
		Assert.assertEquals(individualData1.get(4),individualList.get(0).getAdditionalDataProperties());
		Assert.assertEquals(individualData2.get(4),individualList.get(1).getAdditionalDataProperties());
		Assert.assertEquals(individualData3.get(4),individualList.get(2).getAdditionalDataProperties());
		Assert.assertEquals(individualData4.get(4),individualList.get(3).getAdditionalDataProperties());
		Assert.assertEquals(individualData5.get(4),individualList.get(4).getAdditionalDataProperties());
		Assert.assertEquals(individualData6.get(4),individualList.get(5).getAdditionalDataProperties());
		Assert.assertEquals(individualData7.get(4),individualList.get(6).getAdditionalDataProperties());
		Assert.assertEquals(individualData8.get(4),individualList.get(7).getAdditionalDataProperties());
		Assert.assertEquals(individualData9.get(4),individualList.get(8).getAdditionalDataProperties());
	}
	// end testGetAdditionalDataProperties

	@Test
	public void testGetAdditionalDataPropertiesWithContent()
	{
		ArrayList<MultiHashMap<String,String>> individualData1 = roModel.getIndividualData("DataControl");
		ArrayList<MultiHashMap<String,String>> individualData2 = roModel.getIndividualData("ExchangeInformation");
		ArrayList<MultiHashMap<String,String>> individualData3 = roModel.getIndividualData("TomBeger");
		ArrayList<MultiHashMap<String,String>> individualData4 = roModel.getIndividualData("LoadPageTimeRestriction");
		ArrayList<MultiHashMap<String,String>> individualData5 = roModel.getIndividualData("AccessibleToBlindUsers");
		ArrayList<MultiHashMap<String,String>> individualData6 = roModel.getIndividualData("DevelopSocialNetworkingPlatform");
		ArrayList<MultiHashMap<String,String>> individualData7 = roModel.getIndividualData("UserHasPhotoAlbum");
		ArrayList<MultiHashMap<String,String>> individualData8 = roModel.getIndividualData("HighNumberOfUsers");
		ArrayList<MultiHashMap<String,String>> individualData9 = roModel.getIndividualData("TestCase1");
		
		Assert.assertEquals(individualData1.get(5),individualList.get(0).getAdditionalDataPropertiesWithContent());
		Assert.assertEquals(individualData2.get(5),individualList.get(1).getAdditionalDataPropertiesWithContent());
		Assert.assertEquals(individualData3.get(5),individualList.get(2).getAdditionalDataPropertiesWithContent());
		Assert.assertEquals(individualData4.get(5),individualList.get(3).getAdditionalDataPropertiesWithContent());
		Assert.assertEquals(individualData5.get(5),individualList.get(4).getAdditionalDataPropertiesWithContent());
		Assert.assertEquals(individualData6.get(5),individualList.get(5).getAdditionalDataPropertiesWithContent());
		Assert.assertEquals(individualData7.get(5),individualList.get(6).getAdditionalDataPropertiesWithContent());
		Assert.assertEquals(individualData8.get(5),individualList.get(7).getAdditionalDataPropertiesWithContent());
		Assert.assertEquals(individualData9.get(5),individualList.get(8).getAdditionalDataPropertiesWithContent());
	}
	// end testGetAdditionalDataPropertiesWithContent

	@Test
	public void testGetIncomingObjectProperties()
	{
		ArrayList<MultiHashMap<String,String>> individualData1 = roModel.getIndividualData("DataControl");
		ArrayList<MultiHashMap<String,String>> individualData2 = roModel.getIndividualData("ExchangeInformation");
		ArrayList<MultiHashMap<String,String>> individualData3 = roModel.getIndividualData("TomBeger");
		ArrayList<MultiHashMap<String,String>> individualData4 = roModel.getIndividualData("LoadPageTimeRestriction");
		ArrayList<MultiHashMap<String,String>> individualData5 = roModel.getIndividualData("AccessibleToBlindUsers");
		ArrayList<MultiHashMap<String,String>> individualData6 = roModel.getIndividualData("DevelopSocialNetworkingPlatform");
		ArrayList<MultiHashMap<String,String>> individualData7 = roModel.getIndividualData("UserHasPhotoAlbum");
		ArrayList<MultiHashMap<String,String>> individualData8 = roModel.getIndividualData("HighNumberOfUsers");
		ArrayList<MultiHashMap<String,String>> individualData9 = roModel.getIndividualData("TestCase1");
		
		Assert.assertEquals(individualData1.get(6),individualList.get(0).getIncomingObjectProperties());
		Assert.assertEquals(individualData2.get(6),individualList.get(1).getIncomingObjectProperties());
		Assert.assertEquals(individualData3.get(6),individualList.get(2).getIncomingObjectProperties());
		Assert.assertEquals(individualData4.get(6),individualList.get(3).getIncomingObjectProperties());
		Assert.assertEquals(individualData5.get(6),individualList.get(4).getIncomingObjectProperties());
		Assert.assertEquals(individualData6.get(6),individualList.get(5).getIncomingObjectProperties());
		Assert.assertEquals(individualData7.get(6),individualList.get(6).getIncomingObjectProperties());
		Assert.assertEquals(individualData8.get(6),individualList.get(7).getIncomingObjectProperties());
		Assert.assertEquals(individualData9.get(6),individualList.get(8).getIncomingObjectProperties());
	}
	// end testGetIncomingObjectProperties

	@Test
	public void testGetIncomingObjectPropertiesWithContent()
	{
		ArrayList<MultiHashMap<String,String>> individualData1 = roModel.getIndividualData("DataControl");
		ArrayList<MultiHashMap<String,String>> individualData2 = roModel.getIndividualData("ExchangeInformation");
		ArrayList<MultiHashMap<String,String>> individualData3 = roModel.getIndividualData("TomBeger");
		ArrayList<MultiHashMap<String,String>> individualData4 = roModel.getIndividualData("LoadPageTimeRestriction");
		ArrayList<MultiHashMap<String,String>> individualData5 = roModel.getIndividualData("AccessibleToBlindUsers");
		ArrayList<MultiHashMap<String,String>> individualData6 = roModel.getIndividualData("DevelopSocialNetworkingPlatform");
		ArrayList<MultiHashMap<String,String>> individualData7 = roModel.getIndividualData("UserHasPhotoAlbum");
		ArrayList<MultiHashMap<String,String>> individualData8 = roModel.getIndividualData("HighNumberOfUsers");
		ArrayList<MultiHashMap<String,String>> individualData9 = roModel.getIndividualData("TestCase1");
		
		Assert.assertEquals(individualData1.get(7),individualList.get(0).getIncomingObjectPropertiesWithContent());
		Assert.assertEquals(individualData2.get(7),individualList.get(1).getIncomingObjectPropertiesWithContent());
		Assert.assertEquals(individualData3.get(7),individualList.get(2).getIncomingObjectPropertiesWithContent());
		Assert.assertEquals(individualData4.get(7),individualList.get(3).getIncomingObjectPropertiesWithContent());
		Assert.assertEquals(individualData5.get(7),individualList.get(4).getIncomingObjectPropertiesWithContent());
		Assert.assertEquals(individualData6.get(7),individualList.get(5).getIncomingObjectPropertiesWithContent());
		Assert.assertEquals(individualData7.get(7),individualList.get(6).getIncomingObjectPropertiesWithContent());
		Assert.assertEquals(individualData8.get(7),individualList.get(7).getIncomingObjectPropertiesWithContent());
		Assert.assertEquals(individualData9.get(7),individualList.get(8).getIncomingObjectPropertiesWithContent());
	}
	// end testGetIncomingObjectPropertiesWithContent

	@Test
	public void testToString()
	{
		Assert.assertEquals("DataControl",individualList.get(0).toString());
		Assert.assertEquals("ExchangeInformation",individualList.get(1).toString());
		Assert.assertEquals("TomBeger",individualList.get(2).toString());
		Assert.assertEquals("LoadPageTimeRestriction",individualList.get(3).toString());
		Assert.assertEquals("AccessibleToBlindUsers",individualList.get(4).toString());
		Assert.assertEquals("DevelopSocialNetworkingPlatform",individualList.get(5).toString());
		Assert.assertEquals("UserHasPhotoAlbum",individualList.get(6).toString());
		Assert.assertEquals("HighNumberOfUsers",individualList.get(7).toString());
		Assert.assertEquals("TestCase1",individualList.get(8).toString());
	}
	// end testToString 
}
// end class IndividualVertexTest