package junit.requirementsontology.test;

import java.util.ArrayList;
import java.util.Set;

import org.apache.commons.collections15.multimap.MultiHashMap;
import org.deft.requirementsontology.IndividualVertex;
import org.deft.requirementsontology.ObjectPropertyEdge;
import org.deft.requirementsontology.OntologyAccessor;
import org.deft.requirementsontology.ROModel;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

public class ObjectPropertyEdgeTest
{
	private static OntologyAccessor oA;
	private static ROModel roModel;
	private static IndividualVertex sourceIndividual;
	private static IndividualVertex targetIndividual;
	private static ObjectPropertyEdge objectProperty;

	@BeforeClass
	public static void setUpBeforeClass() throws Exception
	{
		oA = new OntologyAccessor("RO_version_1_4_1_social_network_consistent.owl");
		roModel = new ROModel(oA);
		sourceIndividual = createIndividual("DevelopSocialNetworkingPlatform");
		targetIndividual = createIndividual("ExchangeInformation");
		objectProperty = new ObjectPropertyEdge("refinesTo",sourceIndividual,targetIndividual);
	}
	// end setUpBeforeClass
	
	private static IndividualVertex createIndividual(String simpleIndividualName)
	{
		String individualEssetialType = roModel.getIndividualEssentialType(simpleIndividualName);
		String individualMainType = roModel.getIndividualMainType(simpleIndividualName);
		Set<String> individualComments = roModel.getIndividualComments(simpleIndividualName);
		Set<String> individualAttributes = roModel.getIndividualAttributes(simpleIndividualName);
		ArrayList<MultiHashMap<String,String>> individualData = roModel.getIndividualData(simpleIndividualName);
		return new IndividualVertex(simpleIndividualName,individualEssetialType,individualMainType,individualComments,individualAttributes,individualData.get(0),individualData.get(1),individualData.get(2),individualData.get(3),individualData.get(4),individualData.get(5),individualData.get(6),individualData.get(7));
	}
	// end createIndividual	

	@Test
	public void testObjectPropertyEdge()
	{
		Assert.assertEquals("refinesTo",objectProperty.getSimpleName());
		Assert.assertEquals("DevelopSocialNetworkingPlatform",objectProperty.getSourceIndividual().getSimpleName());
		Assert.assertEquals("ExchangeInformation",objectProperty.getTargetIndividual().getSimpleName());
	}
	// end testObjectPropertyEdge

	@Test
	public void testGetSourceIndividual()
	{
		IndividualVertex source = createIndividual("DevelopSocialNetworkingPlatform");
		IndividualVertex target = createIndividual("ExchangeInformation");
		ObjectPropertyEdge property = new ObjectPropertyEdge("refinesTo",source,target);
		
		Assert.assertEquals(source,property.getSourceIndividual());
	}
	// end testGetSourceIndividual

	@Test
	public void testGetTargetIndividual()
	{
		IndividualVertex source = createIndividual("DevelopSocialNetworkingPlatform");
		IndividualVertex target = createIndividual("ExchangeInformation");
		ObjectPropertyEdge property = new ObjectPropertyEdge("refinesTo",source,target);
		
		Assert.assertEquals(target,property.getTargetIndividual());
	}
	// end testGetTargetIndividual

	@Test
	public void testGetSimpleName()
	{
		Assert.assertEquals("refinesTo",objectProperty.getSimpleName());
	}
	// end testGetSimpleName

	@Test
	public void testToString()
	{
		Assert.assertEquals("refinesTo",objectProperty.toString());
	}
	// end testToString
}
// end class ObjectPropertyEdgeTest