package junit.requirementsontology.test;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.deft.requirementsontology.OntologyAccessor;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;
import org.semanticweb.owl.model.OWLClass;
import org.semanticweb.owl.model.OWLCommentAnnotation;
import org.semanticweb.owl.model.OWLConstant;
import org.semanticweb.owl.model.OWLDataProperty;
import org.semanticweb.owl.model.OWLDataPropertyExpression;
import org.semanticweb.owl.model.OWLDataRange;
import org.semanticweb.owl.model.OWLDescription;
import org.semanticweb.owl.model.OWLIndividual;
import org.semanticweb.owl.model.OWLObjectProperty;
import org.semanticweb.owl.model.OWLObjectPropertyExpression;

public class OntologyAccessorTest
{
	private static OntologyAccessor oA;
	
	@BeforeClass
	public static void setUpBeforeClass() throws Exception
	{
		oA = new OntologyAccessor("RO_version_1_4_1_social_network_consistent.owl");
	}
	// end setUpBeforeClass

	@Test
	public void testOntologyAccessorString()
	{
		OntologyAccessor oA2 = new OntologyAccessor("RO_version_1_4_1_social_network_consistent.owl");
		
		Assert.assertNotNull(oA2.getOwlOntology());
	}
	// end testOntologyAccessorString

	@Test
	public void testOntologyAccessorOWLOntology()
	{
		OntologyAccessor oA3 = new OntologyAccessor(oA.getOwlOntology());

		Assert.assertNotNull(oA3.getOwlOntology());
	}
	// end testOntologyAccessorOWLOntology

	@Ignore
	public void testOntologyAccessorInputStream()
	{
		Assert.fail("Not yet implemented");
	}
	// end testOntologyAccessorInputStream

	@Ignore
	public void testGetFilePath()
	{
		Assert.fail("Not yet implemented");
	}
	// end testGetFilePath

	@Test
	public void testGetFileName()
	{
		Assert.assertEquals("RO_version_1_4_1_social_network_consistent.owl",oA.getFileName());
	}
	// end testGetFileName

	@Test
	public void testGetOwlOntology()
	{
		Assert.assertNotNull(oA.getOwlOntology());
		Assert.assertEquals("http://purl.org/ro/ont",oA.getOwlOntology().getURI().toString());
	}
	// end testGetOwlOntology

	@Test
	public void testGetUri()
	{
		Assert.assertEquals("http://purl.org/ro/ont#Influencer",oA.getUri("Influencer").toString());
	}
	// end testGetUri

	@Test
	public void testGetUriString()
	{
		Assert.assertEquals("http://purl.org/ro/ont#Influencer",oA.getUriString("Influencer"));
	}
	// end testGetUriString
	
	@Test
	public void testGetSimpleName()
	{
		Assert.assertEquals("Influencer",oA.getSimpleName("http://purl.org/ro/ont#Influencer"));
	}
	// end testGetSimpleName

	@Test
	public void testGetOwlClass()
	{
		Assert.assertEquals("Influencer",oA.getOwlClass("Influencer").toString());
	}
	// end testGetOwlClass

	@Test
	public void testGetOwlIndividual()
	{
		Assert.assertEquals("ConnectPeople",oA.getOwlIndividual("ConnectPeople").toString());
	}
	// end testGetOwlIndividual

	@Test
	public void testGetOwlObjectProperty()
	{
		Assert.assertEquals("isAuthoredBy",oA.getOwlObjectProperty("isAuthoredBy").toString());
	}
	// end testGetOwlObjectProperty

	@Test
	public void testGetOwlDataProperty()
	{
		Assert.assertEquals("hasID",oA.getOwlDataProperty("hasID").toString());
	}
	// end testGetOwlDataProperty

	@Test
	public void testGetOwlIndividuals()
	{
		Set<OWLIndividual> individualSet1 = oA.getOwlIndividuals("Goals");
		Set<OWLIndividual> individualSet2 = oA.getOwlIndividuals("Goal");
		Set<String> individualSet3 = new TreeSet<String>();
		
		for (OWLIndividual individual : individualSet2)
		{
			individualSet3.add(individual.toString());
		}
		// end for
		
		Assert.assertEquals(0,individualSet1.size());
		Assert.assertEquals(9,individualSet2.size());
		Assert.assertTrue(individualSet3.contains("CollectUserData"));
		Assert.assertTrue(individualSet3.contains("CommercialSuccess"));
		Assert.assertTrue(individualSet3.contains("DataPrivacyProtection"));
		Assert.assertTrue(individualSet3.contains("DevelopSocialNetworkingPlatform"));
		Assert.assertTrue(individualSet3.contains("EarnMoney"));
		Assert.assertTrue(individualSet3.contains("ExchangeInformation"));
		Assert.assertTrue(individualSet3.contains("FastAccessToPlatform"));
		Assert.assertTrue(individualSet3.contains("PlatformIsEasyToUseForEveryone"));
		Assert.assertTrue(individualSet3.contains("StoreInformation"));
	}
	// end testGetOwlIndividual

	@Test
	public void testGetOwlObjectPropertiesString()
	{
		Map<OWLObjectPropertyExpression,Set<OWLIndividual>> objectPropertyMap1 = oA.getOwlObjectProperties("ExchangeInformations");
		Map<OWLObjectPropertyExpression,Set<OWLIndividual>> objectPropertyMap2 = oA.getOwlObjectProperties("ExchangeInformation");
		HashMap<String,Integer> objectPropertyMap3 = new HashMap<String,Integer>();
		
		for (OWLObjectPropertyExpression objectProperty : objectPropertyMap2.keySet())
		{
			Set<OWLIndividual> individualSet = objectPropertyMap2.get(objectProperty);
			objectPropertyMap3.put(objectProperty.toString(),individualSet.size());
		}
		// end for
		
		Assert.assertTrue(objectPropertyMap1.isEmpty());
		Assert.assertEquals(4,objectPropertyMap2.size());
		Assert.assertTrue(objectPropertyMap3.containsKey("refinesTo"));
		Assert.assertTrue(objectPropertyMap3.containsKey("isAuthoredBy"));
		Assert.assertTrue(objectPropertyMap3.containsKey("isNegativeContributionTo"));
		Assert.assertTrue(objectPropertyMap3.containsKey("isConnectedToUseCase"));
		Assert.assertEquals(2,objectPropertyMap3.get("refinesTo").intValue());
		Assert.assertEquals(2,objectPropertyMap3.get("isAuthoredBy").intValue());
		Assert.assertEquals(1,objectPropertyMap3.get("isNegativeContributionTo").intValue());
		Assert.assertEquals(1,objectPropertyMap3.get("isConnectedToUseCase").intValue());
	}
	// end testGetOwlObjectPropertiesString

	@Test
	public void testGetOwlDataPropertiesString()
	{
		Map<OWLDataPropertyExpression,Set<OWLConstant>> dataPropertyMap1 = oA.getOwlDataProperties("DataControls");
		Map<OWLDataPropertyExpression,Set<OWLConstant>> dataPropertyMap2 = oA.getOwlDataProperties("DataControl");
		HashMap<String,String> dataPropertyMap3 = new HashMap<String,String>();
		
		for (OWLDataPropertyExpression dataProperty : dataPropertyMap2.keySet())
		{
			Set<OWLConstant> constantSet = dataPropertyMap2.get(dataProperty);
			dataPropertyMap3.put(dataProperty.toString(),constantSet.toString());
		}
		// end for
		
		Assert.assertTrue(dataPropertyMap1.isEmpty());
		Assert.assertEquals(1,dataPropertyMap2.size());
		Assert.assertTrue(dataPropertyMap3.containsKey("hasRefinementReason"));
		Assert.assertTrue(dataPropertyMap3.containsValue("[The user must be allowed to completely control which data shall be displayed and which not in order to satisfy data security plocies from government.]"));
	}
	// end testGetOwlDataPropertiesString

	@Test
	public void testGetOwlTypes()
	{
		Set<OWLDescription> typeSet1 = oA.getOwlTypes("ServerWithSufficientPerformance");
		Set<String> typeSet2 = new TreeSet<String>();
		
		for (OWLDescription type : typeSet1)
		{
			typeSet2.add(type.toString());
		}
		// end for
		
		Assert.assertTrue(typeSet2.contains("Host"));
		Assert.assertTrue(typeSet2.contains("Mandatory"));
		Assert.assertTrue(typeSet2.contains("MediumPriority"));
		Assert.assertTrue(typeSet2.contains("Thing"));
	}
	// end testGetOwlTypes

	@Test
	public void testIsOwlType()
	{
		Assert.assertTrue(oA.isOwlType("ConnectPeople","Challenge"));
		Assert.assertFalse(oA.isOwlType("ConnectPeople","Goal"));
	}
	// end testIsOwlType

	@Test
	public void testIsSymmetricObjectProperty()
	{
		Assert.assertTrue(oA.isSymmetricObjectProperty("isAlternativeTo"));
		Assert.assertFalse(oA.isSymmetricObjectProperty("refinesTo"));
	}
	// end testIsSymmetricObjectProperty

	@Test
	public void testGetAllOwlSubclasses()
	{
		Set<OWLDescription> subclassSet1 = oA.getAllOwlSubclasses("Risk");
		Set<OWLDescription> subclassSet2 = oA.getAllOwlSubclasses("Story");
		Set<OWLDescription> subclassSet3 = oA.getAllOwlSubclasses("Requirement");
		Set<OWLDescription> subclassSet4 = oA.getAllOwlSubclasses("Artifact");
		Set<String> subclassSet5 = new TreeSet<String>();
		
		for (OWLDescription subclass : subclassSet2)
		{
			subclassSet5.add(subclass.toString());
		}
		// end for
		
		Assert.assertEquals(0,subclassSet1.size());
		Assert.assertEquals(2,subclassSet2.size());
		Assert.assertEquals(47,subclassSet3.size());
		Assert.assertEquals(60,subclassSet4.size());
		Assert.assertTrue(subclassSet5.contains("Scenario"));
		Assert.assertTrue(subclassSet5.contains("UseCase"));
	}
	// end testGetAllOwlSubclasses

	@Test
	public void testGetOwlSubclasses()
	{
		Set<OWLDescription> subclassSet1 = oA.getOwlSubclasses("Risk");
		Set<OWLDescription> subclassSet2 = oA.getOwlSubclasses("Story");
		Set<OWLDescription> subclassSet3 = oA.getOwlSubclasses("Requirement");
		Set<OWLDescription> subclassSet4 = oA.getOwlSubclasses("Artifact");
		Set<String> subclassSet5 = new TreeSet<String>();
		
		for (OWLDescription subclass : subclassSet2)
		{
			subclassSet5.add(subclass.toString());
		}
		// end for
		
		Assert.assertEquals(0,subclassSet1.size());
		Assert.assertEquals(2,subclassSet2.size());
		Assert.assertEquals(3,subclassSet3.size());
		Assert.assertEquals(3,subclassSet4.size());
		Assert.assertTrue(subclassSet5.contains("Scenario"));
		Assert.assertTrue(subclassSet5.contains("UseCase"));
	}
	// end testGetOwlSubclasses

	@Test
	public void testGetAllOwlSuperclasses()
	{
		Set<OWLDescription> superclassSet1 = oA.getAllOwlSuperclasses("Risk");
		Set<OWLDescription> superclassSet2 = oA.getAllOwlSuperclasses("Story");
		Set<OWLDescription> superclassSet3 = oA.getAllOwlSuperclasses("Requirement");
		Set<OWLDescription> superclassSet4 = oA.getAllOwlSuperclasses("Artifact");
		Set<OWLDescription> superclassSet5 = oA.getAllOwlSuperclasses("Artifacts");
		Set<String> superclassSet6 = new TreeSet<String>();
		
		for (OWLDescription subclass : superclassSet2)
		{
			superclassSet6.add(subclass.toString());
		}
		// end for
		
		Assert.assertEquals(4,superclassSet1.size());
		Assert.assertEquals(7,superclassSet2.size());
		Assert.assertEquals(13,superclassSet3.size());
		Assert.assertEquals(1,superclassSet4.size());
		Assert.assertEquals(0,superclassSet5.size());
		Assert.assertTrue(superclassSet6.contains("Artifact"));
		Assert.assertTrue(superclassSet6.contains("Influencer"));
		Assert.assertTrue(superclassSet6.contains("RequirementArtifact"));
		Assert.assertTrue(superclassSet6.contains("ObjectSomeValueFrom(hasObstacle Obstacle)"));
		Assert.assertTrue(superclassSet6.contains("ObjectSomeValueFrom(hasRisk Risk)"));
		Assert.assertTrue(superclassSet6.contains("ObjectSomeValueFrom(isAuthoredBy Stakeholder)"));
		Assert.assertTrue(superclassSet6.contains("DataSomeValuesFrom(hasID string)"));
	}
	// end testGetAllOwlSuperclasses

	@Test
	public void testGetOwlSuperclasses()
	{
		Set<OWLDescription> superclassSet1 = oA.getOwlSuperclasses("Risk");
		Set<OWLDescription> superclassSet2 = oA.getOwlSuperclasses("Story");
		Set<OWLDescription> superclassSet3 = oA.getOwlSuperclasses("Requirement");
		Set<OWLDescription> superclassSet4 = oA.getOwlSuperclasses("Artifact");
		Set<OWLDescription> superclassSet5 = oA.getOwlSuperclasses("Artifacts");
		Set<String> superclassSet6 = new TreeSet<String>();
		
		for (OWLDescription subclass : superclassSet3)
		{
			superclassSet6.add(subclass.toString());
		}
		// end for
		
		Assert.assertEquals(1,superclassSet1.size());
		Assert.assertEquals(1,superclassSet2.size());
		Assert.assertEquals(7,superclassSet3.size());
		Assert.assertEquals(1,superclassSet4.size());
		Assert.assertEquals(0,superclassSet5.size());
		Assert.assertTrue(superclassSet6.contains("RequirementArtifact"));
		Assert.assertTrue(superclassSet6.contains("ObjectSomeValueFrom(hasCost Cost)"));
		Assert.assertTrue(superclassSet6.contains("ObjectSomeValueFrom(hasFailurePostcondition FailurePostcondition)"));
		Assert.assertTrue(superclassSet6.contains("ObjectSomeValueFrom(hasTestCase TestCase)"));
		Assert.assertTrue(superclassSet6.contains("ObjectSomeValueFrom(hasTrigger Trigger)"));
		Assert.assertTrue(superclassSet6.contains("ObjectSomeValueFrom(hasVerificationMethod VerificationMethod)"));
		Assert.assertTrue(superclassSet6.contains("ObjectSomeValueFrom(isDescribedByUseCase UseCase)"));
	}
	// end testGetOwlSuperclasses

	@Test
	public void testGetOwlSomeRestrictions()
	{
		Set<OWLDescription> owlSomeRrestrictionSet1 = oA.getOwlSomeRestrictions("Risk");
		Set<OWLDescription> owlSomeRrestrictionSet2 = oA.getOwlSomeRestrictions("Story");
		Set<OWLDescription> owlSomeRrestrictionSet3 = oA.getOwlSomeRestrictions("Requirement");
		Set<OWLDescription> owlSomeRrestrictionSet4 = oA.getOwlSomeRestrictions("Artifact");
		Set<OWLDescription> owlSomeRrestrictionSet5 = oA.getOwlSomeRestrictions("Artifacts");
		Set<String> owlSomeRrestrictionSet6 = new TreeSet<String>();
		
		for (OWLDescription owlSomeRestriction : owlSomeRrestrictionSet3)
		{
			owlSomeRrestrictionSet6.add(owlSomeRestriction.toString());
		}
		// end for

		Assert.assertEquals(1,owlSomeRrestrictionSet1.size());
		Assert.assertEquals(4,owlSomeRrestrictionSet2.size());
		Assert.assertEquals(10,owlSomeRrestrictionSet3.size());
		Assert.assertEquals(1,owlSomeRrestrictionSet4.size());
		Assert.assertEquals(0,owlSomeRrestrictionSet5.size());
		Assert.assertTrue(owlSomeRrestrictionSet6.contains("ObjectSomeValueFrom(hasObstacle Obstacle)"));
		Assert.assertTrue(owlSomeRrestrictionSet6.contains("ObjectSomeValueFrom(hasRisk Risk)"));
		Assert.assertTrue(owlSomeRrestrictionSet6.contains("ObjectSomeValueFrom(isAuthoredBy Stakeholder)"));
		Assert.assertTrue(owlSomeRrestrictionSet6.contains("DataSomeValuesFrom(hasID string)"));
	}
	// end testGetOwlSomeRestriction

	@Test
	public void testGetAllOwlSomeRestrictions()
	{
		Set<OWLDescription> owlSomeRrestrictionSet1 = oA.getAllOwlSomeRestrictions("Risk");
		Set<OWLDescription> owlSomeRrestrictionSet2 = oA.getAllOwlSomeRestrictions("Story");
		Set<OWLDescription> owlSomeRrestrictionSet3 = oA.getAllOwlSomeRestrictions("Requirement");
		Set<OWLDescription> owlSomeRrestrictionSet4 = oA.getAllOwlSomeRestrictions("Artifact");
		Set<OWLDescription> owlSomeRrestrictionSet5 = oA.getAllOwlSomeRestrictions("Artifacts");
		Set<String> owlSomeRrestrictionSet6 = new TreeSet<String>();
		
		for (OWLDescription owlSomeRestriction : owlSomeRrestrictionSet2)
		{
			owlSomeRrestrictionSet6.add(owlSomeRestriction.toString());
		}
		// end for

		Assert.assertEquals(1,owlSomeRrestrictionSet1.size());
		Assert.assertEquals(7,owlSomeRrestrictionSet2.size());
		Assert.assertEquals(11,owlSomeRrestrictionSet3.size());
		Assert.assertEquals(18,owlSomeRrestrictionSet4.size());
		Assert.assertEquals(0,owlSomeRrestrictionSet5.size());
		Assert.assertTrue(owlSomeRrestrictionSet6.contains("ObjectSomeValueFrom(belongsToUseCase UseCase)"));
		Assert.assertTrue(owlSomeRrestrictionSet6.contains("ObjectSomeValueFrom(describesRequirement Requirement)"));
		Assert.assertTrue(owlSomeRrestrictionSet6.contains("ObjectSomeValueFrom(hasObstacle Obstacle)"));
		Assert.assertTrue(owlSomeRrestrictionSet6.contains("ObjectSomeValueFrom(hasRisk Risk)"));
		Assert.assertTrue(owlSomeRrestrictionSet6.contains("ObjectSomeValueFrom(hasScenario Scenario)"));
		Assert.assertTrue(owlSomeRrestrictionSet6.contains("ObjectSomeValueFrom(isAuthoredBy Stakeholder)"));
		Assert.assertTrue(owlSomeRrestrictionSet6.contains("DataSomeValuesFrom(hasID string)"));
	}
	// end testGetAllOwlSomeRestrictions

	@Test
	public void testGetAllOwlIndividuals()
	{
		Set<OWLIndividual> individualSet1 = oA.getAllOwlIndividuals("Risk");
		Set<OWLIndividual> individualSet2 = oA.getAllOwlIndividuals("Influencer");
		Set<OWLIndividual> individualSet3 = oA.getAllOwlIndividuals("Artifact");
		Set<OWLIndividual> individualSet4 = oA.getAllOwlIndividuals("Story");
		Set<OWLIndividual> individualSet5 = oA.getAllOwlIndividuals("HardwareComponent");
		Set<String> individualSet6 = new TreeSet<String>();
		
		for (OWLIndividual individual : individualSet4)
		{
			individualSet6.add(individual.toString());
		}
		// end for
		
		Assert.assertEquals(2,individualSet1.size());
		Assert.assertEquals(27,individualSet2.size());
		Assert.assertEquals(28,individualSet3.size());
		Assert.assertEquals(4,individualSet4.size());
		Assert.assertEquals(0,individualSet5.size());
		Assert.assertTrue(individualSet6.contains("LoadPageTimeRestriction"));
		Assert.assertTrue(individualSet6.contains("ProvidingUserInformation"));
		Assert.assertTrue(individualSet6.contains("FillProfilePage"));
		Assert.assertTrue(individualSet6.contains("UploadPhotos"));
	}
	// end testGetAllOwlIndividuals

	@Test
	public void testGetOwlObjectPropertyDomains()
	{
		Set<OWLDescription> objectPropertyDomainSet1 = oA.getOwlObjectPropertyDomains("isAuthoredBy");
		Set<OWLDescription> objectPropertyDomainSet2 = oA.getOwlObjectPropertyDomains("refinesTo");
		Set<OWLDescription> objectPropertyDomainSet3 = oA.getOwlObjectPropertyDomains("isRefinementOf");
		Set<OWLDescription> objectPropertyDomainSet4 = oA.getOwlObjectPropertyDomains("isRefinement");
		Set<OWLDescription> objectPropertyDomainSet5 = oA.getOwlObjectPropertyDomains("hasVerificationMethod");
		Set<String> objectPropertyDomainSet6 = new TreeSet<String>();
		
		for (OWLDescription objectPropertyDomain : objectPropertyDomainSet1)
		{
			objectPropertyDomainSet6.add(objectPropertyDomain.toString());
		}
		// end for
		
		Assert.assertEquals(1,objectPropertyDomainSet1.size());
		Assert.assertEquals(1,objectPropertyDomainSet2.size());
		Assert.assertEquals(0,objectPropertyDomainSet3.size());
		Assert.assertEquals(0,objectPropertyDomainSet4.size());
		Assert.assertEquals(0,objectPropertyDomainSet5.size());
		Assert.assertTrue(objectPropertyDomainSet6.contains("Artifact"));
	}
	// end testGetOwlObjectPropertyDomains

	@Test
	public void testGetOwlObjectPropertyDomain()
	{
		Assert.assertEquals("Artifact",oA.getOwlObjectPropertyDomain("isAuthoredBy").toString());
		Assert.assertEquals("RequirementArtifact",oA.getOwlObjectPropertyDomain("refinesTo").toString());
		Assert.assertNull(oA.getOwlObjectPropertyDomain("isRefinementOf"));
		Assert.assertNull(oA.getOwlObjectPropertyDomain("isRefinement"));
		Assert.assertNull(oA.getOwlObjectPropertyDomain("hasVerificationMethod"));
	}
	// end testGetOwlObjectPropertyDomain

	@Test
	public void testGetOwlObjectPropertyRanges()
	{
		Set<OWLDescription> objectPropertyRangeSet1 = oA.getOwlObjectPropertyRanges("isAuthoredBy");
		Set<OWLDescription> objectPropertyRangeSet2 = oA.getOwlObjectPropertyRanges("refinesTo");
		Set<OWLDescription> objectPropertyRangeSet3 = oA.getOwlObjectPropertyRanges("isRefinementOf");
		Set<OWLDescription> objectPropertyRangeSet4 = oA.getOwlObjectPropertyRanges("isRefinement");
		Set<OWLDescription> objectPropertyRangeSet5 = oA.getOwlObjectPropertyRanges("hasVerificationMethod");
		Set<String> objectPropertyRangeSet6 = new TreeSet<String>();
		
		for (OWLDescription objectPropertyRange : objectPropertyRangeSet1)
		{
			objectPropertyRangeSet6.add(objectPropertyRange.toString());
		}
		// end for
		
		Assert.assertEquals(1,objectPropertyRangeSet1.size());
		Assert.assertEquals(1,objectPropertyRangeSet2.size());
		Assert.assertEquals(0,objectPropertyRangeSet3.size());
		Assert.assertEquals(0,objectPropertyRangeSet4.size());
		Assert.assertEquals(0,objectPropertyRangeSet5.size());
		Assert.assertTrue(objectPropertyRangeSet6.contains("Stakeholder"));
	}
	// end testGetOwlObjectPropertyRanges

	@Test
	public void testGetOwlObjectPropertyRange()
	{
		Assert.assertEquals("Stakeholder",oA.getOwlObjectPropertyRange("isAuthoredBy").toString());
		Assert.assertEquals("RequirementArtifact",oA.getOwlObjectPropertyRange("refinesTo").toString());
		Assert.assertNull(oA.getOwlObjectPropertyRange("isRefinementOf"));
		Assert.assertNull(oA.getOwlObjectPropertyRange("isRefinement"));
		Assert.assertNull(oA.getOwlObjectPropertyRange("hasVerificationMethod"));
	}
	// end testGetOwlObjectPropertyRange

	@Test
	public void testGetOwlDataPropertyDomains()
	{
		Set<OWLDescription> dataPropertyDomainSet1 = oA.getOwlDataPropertyDomains("hasID");
		Set<OWLDescription> dataPropertyDomainSet2 = oA.getOwlDataPropertyDomains("hasCost");
		Set<OWLDescription> dataPropertyDomainSet3 = oA.getOwlDataPropertyDomains("hasRefinementReason");
		Set<OWLDescription> dataPropertyDomainSet4 = oA.getOwlDataPropertyDomains("hasRefinementReasons");
		Set<String> dataPropertyDomainSet5 = new TreeSet<String>();
		
		for (OWLDescription dataPropertyDomain : dataPropertyDomainSet1)
		{
			dataPropertyDomainSet5.add(dataPropertyDomain.toString());
		}
		// end for

		Assert.assertEquals(1,dataPropertyDomainSet1.size());
		Assert.assertEquals(0,dataPropertyDomainSet2.size());
		Assert.assertEquals(1,dataPropertyDomainSet3.size());
		Assert.assertEquals(0,dataPropertyDomainSet4.size());
		Assert.assertTrue(dataPropertyDomainSet5.contains("RequirementArtifact"));		
	}

	@Test
	public void testGetOwlDataPropertyDomain()
	{
		Assert.assertEquals("RequirementArtifact",oA.getOwlDataPropertyDomain("hasID").toString());
		Assert.assertNull(oA.getOwlDataPropertyDomain("hasCost"));
		Assert.assertEquals("Refinement",oA.getOwlDataPropertyDomain("hasRefinementReason").toString());
		Assert.assertNull(oA.getOwlDataPropertyDomain("hasRefinementReasons"));
	}
	// end testGetOwlDataPropertyDomain

	@Test
	public void testGetOwlDataPropertyRanges()
	{
		Set<OWLDataRange> dataPropertyRangeSet1 = oA.getOwlDataPropertyRanges("hasID");
		Set<OWLDataRange> dataPropertyRangeSet2 = oA.getOwlDataPropertyRanges("hasCost");
		Set<OWLDataRange> dataPropertyRangeSet3 = oA.getOwlDataPropertyRanges("hasRefinementReason");
		Set<OWLDataRange> dataPropertyRangeSet4 = oA.getOwlDataPropertyRanges("hasRefinementReasons");
		Set<String> dataPropertyRangeSet5 = new TreeSet<String>();
		
		for (OWLDataRange dataPropertyRange : dataPropertyRangeSet1)
		{
			dataPropertyRangeSet5.add(dataPropertyRange.toString());
		}
		// end for
		
		Assert.assertEquals(1,dataPropertyRangeSet1.size());
		Assert.assertEquals(1,dataPropertyRangeSet2.size());
		Assert.assertEquals(1,dataPropertyRangeSet3.size());
		Assert.assertEquals(0,dataPropertyRangeSet4.size());
		Assert.assertTrue(dataPropertyRangeSet5.contains("string"));		
	}

	@Test
	public void testGetOwlDataPropertyRange()
	{
		Assert.assertEquals("string",oA.getOwlDataPropertyRange("hasID").toString());
		Assert.assertEquals("integer",oA.getOwlDataPropertyRange("hasCost").toString());
		Assert.assertNull(oA.getOwlDataPropertyRange("hasRefinementReasons"));
	}
	// end testGetOwlDataPropertyRange

	@Test
	public void testGetOwlObjectProperties()
	{
		Set<OWLObjectProperty> objectPropertySet1 = oA.getOwlObjectProperties();
		Set<String> objectPropertySet2 = new TreeSet<String>();
		
		for (OWLObjectProperty objectProperty : objectPropertySet1)
		{
			objectPropertySet2.add(objectProperty.toString());
		}
		// end for
		
		Assert.assertEquals(33,objectPropertySet1.size());
		Assert.assertTrue(objectPropertySet2.contains("belongsToUseCase"));
		Assert.assertTrue(objectPropertySet2.contains("describesRequirement"));
		Assert.assertTrue(objectPropertySet2.contains("hasRelationship"));
		Assert.assertTrue(objectPropertySet2.contains("isAlternativeTo"));
		Assert.assertTrue(objectPropertySet2.contains("hasVerificationMethod"));
		Assert.assertTrue(objectPropertySet2.contains("isAuthoredBy"));
		Assert.assertFalse(objectPropertySet2.contains("isAuthored"));
		Assert.assertFalse(objectPropertySet2.contains("hasTriggers"));
	}
	// end testGetOwlObjectProperties

	@Test
	public void testGetOwlDataProperties()
	{
		Set<OWLDataProperty> dataPropertySet1 = oA.getOwlDataProperties();
		Set<String> dataPropertySet2 = new TreeSet<String>();
		
		for (OWLDataProperty dataProperty : dataPropertySet1)
		{
			dataPropertySet2.add(dataProperty.toString());
		}
		// end for
		
		Assert.assertEquals(7,dataPropertySet1.size());
		Assert.assertTrue(dataPropertySet2.contains("hasCost"));
		Assert.assertTrue(dataPropertySet2.contains("hasID"));
		Assert.assertTrue(dataPropertySet2.contains("hasRefinementReason"));
		Assert.assertTrue(dataPropertySet2.contains("hasTimingMetric"));
		Assert.assertTrue(dataPropertySet2.contains("hasResponseTime"));
		Assert.assertTrue(dataPropertySet2.contains("hasThroughput"));
		Assert.assertTrue(dataPropertySet2.contains("hasWaitingTime"));
		Assert.assertFalse(dataPropertySet2.contains("hasWaitingTimes"));
	}
	// end testGetOwlDataProperties

	@Test
	public void testGetOwlClasses()
	{
		Set<OWLClass> classSet1 = oA.getOwlClasses();
		Set<String> classSet2 = new TreeSet<String>();
		
		for (OWLClass owlClass : classSet1)
		{
			classSet2.add(owlClass.toString());
		}
		// end for
		
		Assert.assertEquals(99,classSet1.size());
		Assert.assertTrue(classSet2.contains("Artifact"));
		Assert.assertTrue(classSet2.contains("Problem"));
		Assert.assertTrue(classSet2.contains("Risk"));
		Assert.assertTrue(classSet2.contains("Adaptability"));
		Assert.assertTrue(classSet2.contains("Condition"));
		Assert.assertTrue(classSet2.contains("Priority"));
		Assert.assertTrue(classSet2.contains("Stakeholder"));
		Assert.assertTrue(classSet2.contains("Precondition"));
		Assert.assertTrue(classSet2.contains("Exception"));
		Assert.assertFalse(classSet2.contains("Thing"));
		Assert.assertFalse(classSet2.contains("Exceptions"));
	}
	// end testGetOwlClasses

	@Test
	public void testGetOwlRootClasses()
	{
		Set<OWLClass> classSet1 = oA.getOwlRootClasses();
		Set<String> classSet2 = new TreeSet<String>();
		
		for (OWLClass owlClass : classSet1)
		{
			classSet2.add(owlClass.toString());
		}
		// end for
		
		Assert.assertEquals(7,classSet1.size());
		Assert.assertTrue(classSet2.contains("Artifact"));
		Assert.assertTrue(classSet2.contains("Attribute"));
		Assert.assertTrue(classSet2.contains("Exception"));
		Assert.assertTrue(classSet2.contains("Refinement"));
		Assert.assertTrue(classSet2.contains("Source"));
		Assert.assertTrue(classSet2.contains("Stakeholder"));
		Assert.assertTrue(classSet2.contains("hasCost"));
		Assert.assertFalse(classSet2.contains("Influencer"));
		Assert.assertFalse(classSet2.contains("Requirement"));
		Assert.assertFalse(classSet2.contains("Risk"));
		Assert.assertFalse(classSet2.contains("Thing"));
		Assert.assertFalse(classSet2.contains("Exceptions"));
	}
	// end testGetOwlRootClasses

	@Test
	public void testGetOwlCommentAnnotations()
	{
		Set<OWLCommentAnnotation> annotationSet1 = oA.getOwlCommentAnnotations("LoadPageTimeRestriction");
		Set<OWLCommentAnnotation> annotationSet2 = oA.getOwlCommentAnnotations("LoadPageTimeRestrictions");
		Set<OWLCommentAnnotation> annotationSet3 = oA.getOwlCommentAnnotations("ConnectPeople");
		Set<String> annotationSet4 = new TreeSet<String>();
		
		for (OWLCommentAnnotation annotation : annotationSet1)
		{
			annotationSet4.add(annotation.toString());
		}
		// end for

		Assert.assertEquals(1,annotationSet1.size());
		Assert.assertEquals(0,annotationSet2.size());
		Assert.assertEquals(0,annotationSet3.size());
		Assert.assertTrue(annotationSet4.contains("Comment( Load Page in less than 2s)"));
		Assert.assertFalse(annotationSet4.contains("Comment( Load Page in less than 1s)"));
	}
	// end testGetOwlCommentAnnotations
}
// end class OntologyAccessorTest