package junit.requirementsontology.test;

import static org.junit.Assert.fail;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Set;
import java.util.TreeSet;

import org.apache.commons.collections15.multimap.MultiHashMap;
import org.deft.requirementsontology.DataProperty;
import org.deft.requirementsontology.IndividualVertex;
import org.deft.requirementsontology.ObjectPropertyEdge;
import org.deft.requirementsontology.OntologyAccessor;
import org.deft.requirementsontology.ROModel;
import org.junit.Assert;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

import edu.uci.ics.jung.graph.util.EdgeType;

public class ROModelTest
{
	private static OntologyAccessor oA;
	private static ROModel roModel;
	
	@BeforeClass
	public static void setUpBeforeClass() throws Exception
	{
		oA = new OntologyAccessor("RO_version_1_4_1_social_network_consistent.owl");
		roModel = new ROModel(oA);
	}
	// end setUpBeforeClass

	@Before
	public void setUp() throws Exception
	{
		roModel.initialize();
	}
	// end setUp

	@Ignore
	public void testInitializeGoalModelGraph()
	{
		fail("Not yet implemented");
	}
	// end testInitializeGoalModelGraph

	@Test
	public void testInsertObjectProperty()
	{
		roModel.removeAll();
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertObjectProperty("isAuthoredBy");
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertIndividual("TomBeger");
		roModel.insertObjectProperty("isAuthoredBy");
		
		Assert.assertEquals(1,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.removeIndividual("TomBeger");
		roModel.insertIndividual("DataPrivacyProtection");
		roModel.insertObjectProperty("isAuthoredBy");
		
		Assert.assertEquals(1,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertIndividual("TomBeger");
		roModel.insertObjectProperty("isAuthoredBys");
		
		Assert.assertEquals(2,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertObjectProperty("isAuthoredBy");
		
		Assert.assertEquals(2,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(1,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals("isAuthoredBy",roModel.getGoalModelGraph().getEdges().toArray()[0].toString());
		Assert.assertEquals("DataPrivacyProtection",((ObjectPropertyEdge) roModel.getGoalModelGraph().getEdges().toArray()[0]).getSourceIndividual().getSimpleName());
		Assert.assertEquals("TomBeger",((ObjectPropertyEdge) roModel.getGoalModelGraph().getEdges().toArray()[0]).getTargetIndividual().getSimpleName());
		
		roModel.removeAll();
		roModel.insertIndividual("ExchangeInformation");
		roModel.insertIndividual("TomBeger");
		roModel.insertObjectProperty("isAuthoredBy");
		
		Assert.assertEquals(2,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(1,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertIndividual("ErikTittel");
		
		Assert.assertEquals(3,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(1,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertObjectProperty("isAuthoredBy");
		
		Assert.assertEquals(3,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(2,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.removeAll();
		roModel.insertIndividual("ExchangeInformation");
		roModel.insertIndividual("TomBeger");
		roModel.insertIndividual("ErikTittel");
		roModel.insertObjectProperty("isAuthoredBy");
		
		Assert.assertEquals(3,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(2,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertObjectProperty("isAuthoredBy");
		
		Assert.assertEquals(3,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(2,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertAll();
		
		Assert.assertEquals(65,roModel.getGoalModelGraph().getEdges(EdgeType.DIRECTED).size());
		Assert.assertEquals(2,roModel.getGoalModelGraph().getEdges(EdgeType.UNDIRECTED).size());
		
		Collection<ObjectPropertyEdge> objectPropertyEdgeCollection = roModel.getGoalModelGraph().getEdges(EdgeType.UNDIRECTED);
		
		for (ObjectPropertyEdge objectPropertyEdge : objectPropertyEdgeCollection)
		{
			Assert.assertTrue((objectPropertyEdge.getSimpleName().equals("isAlternativeTo") || objectPropertyEdge.getSimpleName().equals("isInConflictWith")));

			if (objectPropertyEdge.getSimpleName().equals("isAlternativeTo"))
			{
				Assert.assertEquals("IntegratedInstantMessaging",objectPropertyEdge.getSourceIndividual().getSimpleName());
				Assert.assertEquals("UserHasPhotoAlbum",objectPropertyEdge.getTargetIndividual().getSimpleName());
			}
			// end if
			
			if (objectPropertyEdge.getSimpleName().equals("isInConflictWith"))
			{
				Assert.assertEquals("AccessibleToBlindUsers",objectPropertyEdge.getSourceIndividual().getSimpleName());
				Assert.assertEquals("UserHasPhotoAlbum",objectPropertyEdge.getTargetIndividual().getSimpleName());
			}
			// end if
		}
		// end for
	}
	// end testInsertObjectProperty

	@Test
	public void testInsertObjectPropertiesForIndividual()
	{
		roModel.removeAll();
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertObjectPropertiesForIndividual("DataControl");
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertIndividual("DataControl");
		roModel.insertObjectPropertiesForIndividual("DataControl");
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertIndividual("UserCanHideData");
		roModel.insertIndividual("UserHasProfilePage");
		roModel.insertObjectPropertiesForIndividual("DataControl");
		
		Assert.assertEquals(2,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertObjectPropertiesForIndividual("DataControl");
		
		Assert.assertEquals(2,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertIndividuals();
		roModel.insertObjectPropertiesForIndividual("DataControls");
		
		Assert.assertEquals(2,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertObjectPropertiesForIndividual("DataControl");
		
		Assert.assertEquals(3,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertIndividuals();
		
		Assert.assertEquals(3,roModel.getGoalModelGraph().getEdges(EdgeType.DIRECTED).size());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdges(EdgeType.UNDIRECTED).size());
		
		roModel.insertObjectPropertiesForIndividual("AccessibleToBlindUsers");
		
		Assert.assertEquals(5,roModel.getGoalModelGraph().getEdges(EdgeType.DIRECTED).size());
		Assert.assertEquals(1,roModel.getGoalModelGraph().getEdges(EdgeType.UNDIRECTED).size());
		
		roModel.insertObjectPropertiesForIndividual("IntegratedInstantMessaging");
		
		Assert.assertEquals(7,roModel.getGoalModelGraph().getEdges(EdgeType.DIRECTED).size());
		Assert.assertEquals(2,roModel.getGoalModelGraph().getEdges(EdgeType.UNDIRECTED).size());
	}
	// end testInsertObjectPropertiesForIndividual

	@Test
	public void testROModel()
	{
		ROModel roModel = new ROModel(oA);
		
		Assert.assertNotNull(roModel.getGoalModelGraph());
		Assert.assertNotNull(roModel.getDataProperties());
		Assert.assertNotNull(roModel.getOntologyClassNames());
		Assert.assertNotNull(roModel.getOntologyRootClassNames());
		Assert.assertNotNull(roModel.getOntologyAttributeClassNames());
		Assert.assertNotNull(roModel.getOntologyObjectPropertyNames());
		Assert.assertNotNull(roModel.getOntologyDataPropertyNames());
		Assert.assertNotNull(roModel.getOntologyIndividualNames());
		Assert.assertEquals(32,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(67,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(1,roModel.getDataProperties().size());
	}
	// end testROModel

	@Ignore
	public void testAbstractROModel()
	{
		fail("Not yet implemented");
	}
	// end testAbstractROModel

	@Test
	public void testInitialize()
	{
		Assert.assertEquals(32,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(67,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(1,roModel.getDataProperties().size());
		Assert.assertEquals(99,roModel.getOntologyClassNames().size());
		Assert.assertEquals(7,roModel.getOntologyRootClassNames().size());
		Assert.assertEquals(33,roModel.getOntologyAttributeClassNames().size());
		Assert.assertEquals(33,roModel.getOntologyObjectPropertyNames().size());
		Assert.assertEquals(7,roModel.getOntologyDataPropertyNames().size());
		Assert.assertEquals(32,roModel.getOntologyIndividualNames().size());
		
		roModel.removeAll();
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());

		roModel.initialize();

		Assert.assertEquals(32,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(67,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(1,roModel.getDataProperties().size());
		Assert.assertEquals(99,roModel.getOntologyClassNames().size());
		Assert.assertEquals(7,roModel.getOntologyRootClassNames().size());
		Assert.assertEquals(33,roModel.getOntologyAttributeClassNames().size());
		Assert.assertEquals(33,roModel.getOntologyObjectPropertyNames().size());
		Assert.assertEquals(7,roModel.getOntologyDataPropertyNames().size());
		Assert.assertEquals(32,roModel.getOntologyIndividualNames().size());
	}
	// end testInitialize

	@Ignore
	public void testInitializeStandardGoalModelGraph()
	{
		fail("Not yet implemented");
	}
	// end testInitializeStandardGoalModelGraph

	@Ignore
	public void testInitializeDirectedGoalModelGraph()
	{
		fail("Not yet implemented");
	}
	// end testInitializeDirectedGoalModelGraph

	@Test
	public void testInsertAll()
	{
		roModel.removeAll();
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		
		roModel.insertAll();
		
		Assert.assertEquals(32,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(67,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(1,roModel.getDataProperties().size());
		
		roModel.insertAll();
		
		Assert.assertEquals(32,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(67,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(1,roModel.getDataProperties().size());
	}
	// end testInsertAll

	@Test
	public void testInsertIndividuals()
	{
		roModel.removeAll();
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		
		roModel.insertIndividuals();
		
		Assert.assertEquals(32,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		
		roModel.insertIndividuals();
		
		Assert.assertEquals(32,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
	}
	// end testInsertIndividuals

	@Test
	public void testInsertObjectProperties()
	{
		roModel.removeAll();
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		
		roModel.insertObjectProperties();
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		
		roModel.insertIndividuals();
		roModel.insertObjectProperties();
		
		Assert.assertEquals(32,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(67,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		
		roModel.insertObjectProperties();
		
		Assert.assertEquals(32,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(67,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
	}
	// end testInsertObjectProperties

	@Test
	public void testInsertDataProperties()
	{
		roModel.removeAll();
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		
		roModel.insertDataProperties();
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		
		roModel.insertIndividual("DataControl");
		roModel.insertDataProperties();
		
		Assert.assertEquals(1,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(1,roModel.getDataProperties().size());
		
		roModel.insertDataProperties();
		
		Assert.assertEquals(1,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(1,roModel.getDataProperties().size());
	}
	// end testInsertDataProperties

	@Test (expected=IllegalArgumentException.class)
	public void testInsertIndividual()
	{
		roModel.removeAll();
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		
		roModel.insertIndividual("DataControl");
		
		Assert.assertEquals(1,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		Assert.assertEquals("DataControl",roModel.getGoalModelGraph().getVertices().toArray()[0].toString());
		Assert.assertTrue(roModel.containsIndividual("DataControl"));
		
		roModel.insertIndividual("DataControls");
		
		Assert.assertEquals(1,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		Assert.assertFalse(roModel.containsIndividual("DataControls"));
		
		roModel.insertIndividual("DataControl");
		
		Assert.assertEquals(1,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		Assert.assertTrue(roModel.containsIndividual("DataControl"));
		
		roModel.insertIndividual("DataControl");
		
		Assert.assertEquals(1,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
	}
	// end testInsertIndividual

	@Test
	public void testInsertDataProperty()
	{
		roModel.removeAll();
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		
		roModel.insertDataProperty("hasRefinementReason");
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		
		roModel.insertIndividual("DataControl");
		
		Assert.assertEquals(1,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		
		roModel.insertDataProperty("hasRefinementReasons");
		
		Assert.assertEquals(1,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		
		roModel.insertDataProperty("hasRefinementReason");
		
		Assert.assertEquals(1,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(1,roModel.getDataProperties().size());
		Assert.assertEquals("hasRefinementReason",((DataProperty) roModel.getDataProperties().values().toArray()[0]).getSimpleName());
		Assert.assertEquals("The user must be allowed to completely control which data shall be displayed and which not in order to satisfy data security plocies from government.",((DataProperty) roModel.getDataProperties().values().toArray()[0]).getDataValue());
		
		roModel.insertDataProperty("hasRefinementReason");
		
		Assert.assertEquals(1,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(1,roModel.getDataProperties().size());
	}
	// end testInsertDataProperty

	@Test
	public void testInsertIndividualWithProperties()
	{
		roModel.removeAll();
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		
		roModel.insertIndividualWithProperties("DataControl");
		
		Assert.assertEquals(1,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(1,roModel.getDataProperties().size());
		Assert.assertEquals("hasRefinementReason",((DataProperty) roModel.getDataProperties().values().toArray()[0]).getSimpleName());
		Assert.assertEquals("The user must be allowed to completely control which data shall be displayed and which not in order to satisfy data security plocies from government.",((DataProperty) roModel.getDataProperties().values().toArray()[0]).getDataValue());
		
		roModel.removeAll();
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		
		roModel.insertIndividualWithProperties("UserHasProfilePage");
		roModel.insertIndividualWithProperties("UserCanHideData");
		roModel.insertIndividualWithProperties("DataControl");
		
		Assert.assertEquals(3,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(3,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(1,roModel.getDataProperties().size());
		Assert.assertEquals(1,roModel.getObjectProperties("DataControl","UserHasProfilePage").size());
		Assert.assertEquals(1,roModel.getObjectProperties("DataControl","UserCanHideData").size());
		Assert.assertEquals(1,roModel.getObjectProperties("UserCanHideData","UserHasProfilePage").size());
		Assert.assertEquals("hasRefinementSource",((ObjectPropertyEdge) roModel.getObjectProperties("DataControl","UserHasProfilePage").toArray()[0]).getSimpleName());
		Assert.assertEquals("hasRefinementTarget",((ObjectPropertyEdge) roModel.getObjectProperties("DataControl","UserCanHideData").toArray()[0]).getSimpleName());
		Assert.assertEquals("isRefinementOf",((ObjectPropertyEdge) roModel.getObjectProperties("UserCanHideData","UserHasProfilePage").toArray()[0]).getSimpleName());
		Assert.assertEquals("hasRefinementReason",((DataProperty) roModel.getDataProperties().values().toArray()[0]).getSimpleName());
		Assert.assertEquals("The user must be allowed to completely control which data shall be displayed and which not in order to satisfy data security plocies from government.",((DataProperty) roModel.getDataProperties().values().toArray()[0]).getDataValue());

		roModel.insertIndividual("KatjaSiegemund");
		roModel.insertIndividualWithProperties("UserCanHideData");
		
		Assert.assertEquals(4,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(4,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(1,roModel.getDataProperties().size());
		Assert.assertEquals("isAuthoredBy",((ObjectPropertyEdge) roModel.getObjectProperties("UserCanHideData","KatjaSiegemund").toArray()[0]).getSimpleName());
		
		roModel.insertIndividualWithProperties("ErikTittel");

		Assert.assertEquals(5,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(4,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(1,roModel.getDataProperties().size());
		
		roModel.insertIndividualWithProperties("DataControl");

		Assert.assertEquals(5,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(5,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(1,roModel.getDataProperties().size());
		Assert.assertEquals(1,roModel.getObjectProperties("DataControl","KatjaSiegemund").size());
		Assert.assertEquals("isAuthoredBy",((ObjectPropertyEdge) roModel.getObjectProperties("DataControl","KatjaSiegemund").toArray()[0]).getSimpleName());
	}
	// end testInsertIndividualWithProperties

	@Test
	public void testInsertIndividualsString()
	{
		roModel.removeAll();
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getVertexCount());
		
		roModel.insertIndividuals("Artifacts");
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getVertexCount());
		
		roModel.insertIndividuals("Stakeholder");
		
		Assert.assertEquals(3,roModel.getGoalModelGraph().getVertexCount());
		
		roModel.insertIndividuals("Problem");
		
		Assert.assertEquals(5,roModel.getGoalModelGraph().getVertexCount());
		
		roModel.insertIndividuals("Artifact");
		
		Assert.assertEquals(31,roModel.getGoalModelGraph().getVertexCount());
		
		roModel.insertIndividuals("Artifact");
		
		Assert.assertEquals(31,roModel.getGoalModelGraph().getVertexCount());
	}
	// end testIndividualsString

	@Test
	public void testInsertIndividualsWithProperties()
	{
		roModel.removeAll();
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		
		roModel.insertIndividualsWithProperties("Stakeholder");
		
		Assert.assertEquals(3,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		
		roModel.insertIndividualsWithProperties("Artifact");
		
		Assert.assertEquals(31,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(64,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		
		roModel.insertIndividualsWithProperties("Refinement");

		Assert.assertEquals(32,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(67,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(1,roModel.getDataProperties().size());
		
		roModel.removeAll();
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		
		roModel.insertIndividualsWithProperties("Requirement");
		
		Assert.assertEquals(11,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(6,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		
		roModel.insertIndividualsWithProperties("Requirement");
		
		Assert.assertEquals(11,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(6,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		
		roModel.insertIndividualsWithProperties("Stakeholder");
		
		Assert.assertEquals(14,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(6,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		
		roModel.insertIndividualsWithProperties("Refinement");
		
		Assert.assertEquals(15,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(9,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(1,roModel.getDataProperties().size());
		
		roModel.insertIndividualsWithProperties("Artifact");
		
		Assert.assertEquals(32,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(67,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(1,roModel.getDataProperties().size());
	}
	// end testInsertIndividualsWithProperites

	@Test
	public void testInsertObjectPropertyWithIndividuals()
	{
		roModel.removeAll();
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertObjectPropertyWithIndividuals("refinesTos");
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertObjectPropertyWithIndividuals("refinesTo");
		
		Assert.assertEquals(13,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(12,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertObjectPropertyWithIndividuals("refinesTo");
		
		Assert.assertEquals(13,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(12,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.insertObjectPropertyWithIndividuals("isAuthoredBy");
		
		Assert.assertEquals(32,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(46,roModel.getGoalModelGraph().getEdgeCount());
	}
	// end testInsertObjectPropertyWithIndividuals

	@Test
	public void testInsertDataPropertyWithIndividuals()
	{
		roModel.removeAll();
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		
		roModel.insertDataPropertyWithIndividuals("hasRefinementReasons");
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		
		roModel.insertDataPropertyWithIndividuals("hasRefinementReason");
		
		Assert.assertEquals(1,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(1,roModel.getDataProperties().size());
		
		roModel.insertDataPropertyWithIndividuals("hasRefinementReason");
		
		Assert.assertEquals(1,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(1,roModel.getDataProperties().size());
	}
	// end testInsertDataPropertyWithIndividuals

	@Test
	public void testInsertDataPropertiesForIndividual()
	{
		roModel.removeAll();
		
		Assert.assertEquals(0,roModel.getDataProperties().size());
		
		roModel.insertDataPropertiesForIndividual("DataControl");
		
		Assert.assertEquals(0,roModel.getDataProperties().size());
		
		roModel.insertIndividual("DataControl");
		roModel.insertDataPropertiesForIndividual("DataControl");
		
		Assert.assertEquals(1,roModel.getDataProperties().size());
		
		roModel.insertDataPropertiesForIndividual("DataControl");
		
		Assert.assertEquals(1,roModel.getDataProperties().size());
	}
	// end testInsertDataPropertiesForIndividual

	@Test
	public void testRemoveIndividual()
	{
		Assert.assertEquals(32,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(67,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(1,roModel.getDataProperties().size());
		
		roModel.removeIndividual("DataControls");
		
		Assert.assertEquals(32,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(67,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(1,roModel.getDataProperties().size());
		
		roModel.removeIndividual("DataControl");
		
		Assert.assertEquals(31,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(64,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		
		roModel.removeIndividual("ExchangeInformation");
		
		Assert.assertEquals(30,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(55,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
	}
	// end testRemoveIndividual

	@Test
	public void testRemoveObjectProperty()
	{
		Assert.assertEquals(67,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.removeObjectProperty("hasRelationships");
		
		Assert.assertEquals(67,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.removeObjectProperty("hasRelationship");
		
		Assert.assertEquals(67,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.removeObjectProperty("isAuthoredBy");
		
		Assert.assertEquals(33,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.removeObjectProperty("hasGoal");
		
		Assert.assertEquals(33,roModel.getGoalModelGraph().getEdgeCount());
	}
	// end testRemoveObjectProperty

	@Test
	public void testRemoveObjectProperties()
	{
		Assert.assertEquals(67,roModel.getGoalModelGraph().getEdgeCount());
		
		roModel.removeObjectProperties();
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
	}
	// end testRemoveObjectProperties

	@Test
	public void testRemoveDataProperty()
	{
		Assert.assertEquals(1,roModel.getDataProperties().size());
		
		roModel.removeDataProperty("hasRefinementReasons");
		
		Assert.assertEquals(1,roModel.getDataProperties().size());
		
		roModel.removeDataProperty("hasRefinementReason");
		
		Assert.assertEquals(0,roModel.getDataProperties().size());
	}
	// end testRemoveDataProperty

	@Test
	public void testRemoveDataProperties()
	{
		Assert.assertEquals(1,roModel.getDataProperties().size());
		
		roModel.removeDataProperties();
		
		Assert.assertEquals(0,roModel.getDataProperties().size());
	}
	// end testRemoveDataProperties

	@Test
	public void testRemoveIndividuals()
	{
		Assert.assertEquals(32,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(67,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(1,roModel.getDataProperties().size());
		Assert.assertEquals(99,roModel.getOntologyClassNames().size());
		Assert.assertEquals(7,roModel.getOntologyRootClassNames().size());
		Assert.assertEquals(33,roModel.getOntologyAttributeClassNames().size());
		Assert.assertEquals(33,roModel.getOntologyObjectPropertyNames().size());
		Assert.assertEquals(7,roModel.getOntologyDataPropertyNames().size());
		Assert.assertEquals(32,roModel.getOntologyIndividualNames().size());
		
		roModel.removeIndividuals();
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		Assert.assertEquals(99,roModel.getOntologyClassNames().size());
		Assert.assertEquals(7,roModel.getOntologyRootClassNames().size());
		Assert.assertEquals(33,roModel.getOntologyAttributeClassNames().size());
		Assert.assertEquals(33,roModel.getOntologyObjectPropertyNames().size());
		Assert.assertEquals(7,roModel.getOntologyDataPropertyNames().size());
		Assert.assertEquals(32,roModel.getOntologyIndividualNames().size());
	}
	// end testRemoveIndividuals

	@Test
	public void testRemoveIndividualsString()
	{
		Assert.assertEquals(32,roModel.getGoalModelGraph().getVertexCount());
		
		roModel.removeIndividuals("Goals");
		
		Assert.assertEquals(32,roModel.getGoalModelGraph().getVertexCount());
		
		roModel.removeIndividuals("Goal");
		
		Assert.assertEquals(23,roModel.getGoalModelGraph().getVertexCount());
		
		roModel.removeIndividuals("Problem");
		
		Assert.assertEquals(21,roModel.getGoalModelGraph().getVertexCount());
		
		roModel.removeIndividuals("Artifact");
		
		Assert.assertEquals(4,roModel.getGoalModelGraph().getVertexCount());
	}
	// end testRemoveIndividualsString

	@Test
	public void testRemoveUnconnectedIndividuals()
	{
		Assert.assertEquals(32,roModel.getGoalModelGraph().getVertexCount());
		
		roModel.removeUnconnectedIndividuals();
		
		Assert.assertEquals(32,roModel.getGoalModelGraph().getVertexCount());
		
		roModel.removeObjectProperties();
		
		Assert.assertEquals(32,roModel.getGoalModelGraph().getVertexCount());
		
		roModel.removeUnconnectedIndividuals();
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getVertexCount());
		
		roModel.insertAll();
		
		Assert.assertEquals(32,roModel.getGoalModelGraph().getVertexCount());
		
		roModel.removeObjectProperty("isAuthoredBy");
		
		Assert.assertEquals(32,roModel.getGoalModelGraph().getVertexCount());
		
		roModel.removeUnconnectedIndividuals();
		
		Assert.assertEquals(29,roModel.getGoalModelGraph().getVertexCount());
	}
	// end testRemoveUnconnectedIndividuals

	@Test
	public void testRemoveAll()
	{
		Assert.assertEquals(32,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(67,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(1,roModel.getDataProperties().size());
		Assert.assertEquals(99,roModel.getOntologyClassNames().size());
		Assert.assertEquals(7,roModel.getOntologyRootClassNames().size());
		Assert.assertEquals(33,roModel.getOntologyAttributeClassNames().size());
		Assert.assertEquals(33,roModel.getOntologyObjectPropertyNames().size());
		Assert.assertEquals(7,roModel.getOntologyDataPropertyNames().size());
		Assert.assertEquals(32,roModel.getOntologyIndividualNames().size());
		
		roModel.removeAll();
		
		Assert.assertEquals(0,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getGoalModelGraph().getEdgeCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());
		Assert.assertEquals(99,roModel.getOntologyClassNames().size());
		Assert.assertEquals(7,roModel.getOntologyRootClassNames().size());
		Assert.assertEquals(33,roModel.getOntologyAttributeClassNames().size());
		Assert.assertEquals(33,roModel.getOntologyObjectPropertyNames().size());
		Assert.assertEquals(7,roModel.getOntologyDataPropertyNames().size());
		Assert.assertEquals(32,roModel.getOntologyIndividualNames().size());
	}
	// end testRemoveAll

	@Test
	public void testGetGoalModelGraph()
	{
		Assert.assertNotNull(roModel.getGoalModelGraph());
		Assert.assertEquals(32,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(67,roModel.getGoalModelGraph().getEdgeCount());
	}
	// end testGetGoalModelGraph

	@Test
	public void testGetIndividuals()
	{
		Assert.assertEquals(roModel.getGoalModelGraph().getVertices().size(),roModel.getIndividuals().size());
		Assert.assertEquals(roModel.getGoalModelGraph().getVertices().toString(),roModel.getIndividuals().toString());
	}
	// end testGetIndividuals

	@Test
	public void testGetDataProperties()
	{
		MultiHashMap<IndividualVertex,DataProperty> dataPropertyMap = roModel.getDataProperties();
		Set<IndividualVertex> individualSet = dataPropertyMap.keySet();
		IndividualVertex individual = (IndividualVertex) individualSet.toArray()[0];
		DataProperty dataProperty = (DataProperty) dataPropertyMap.get(individual).toArray()[0];
		
		Assert.assertEquals(1,dataPropertyMap.size());
		Assert.assertEquals(1,individualSet.size());
		Assert.assertEquals("DataControl",individual.getSimpleName());
		Assert.assertEquals("hasRefinementReason",dataProperty.getSimpleName());
		Assert.assertEquals("The user must be allowed to completely control which data shall be displayed and which not in order to satisfy data security plocies from government.",dataProperty.getDataValue());
	}
	// end testGetDataProperties

	@Test
	public void testGetIndividualsString()
	{
		Collection<IndividualVertex> individualCollection1 = roModel.getIndividuals("Artifact");
		Collection<IndividualVertex> individualCollection2 = roModel.getIndividuals("Stakeholder");
		Collection<IndividualVertex> individualCollection3 = roModel.getIndividuals("Refinement");
		Collection<IndividualVertex> individualCollection4 = roModel.getIndividuals("Attribute");
		Collection<IndividualVertex> individualCollection5 = roModel.getIndividuals("Problem");
		Collection<IndividualVertex> individualCollection6 = roModel.getIndividuals("Goal");
		Collection<IndividualVertex> individualCollection7 = roModel.getIndividuals("UserSatisfaction");
		Collection<IndividualVertex> individualCollection8 = roModel.getIndividuals("UserSatisfactions");
		
		Assert.assertEquals(28,individualCollection1.size());
		Assert.assertEquals(3,individualCollection2.size());
		Assert.assertEquals(1,individualCollection3.size());
		Assert.assertEquals(0,individualCollection4.size());
		Assert.assertEquals(2,individualCollection5.size());
		Assert.assertEquals(9,individualCollection6.size());
		Assert.assertEquals(1,individualCollection7.size());
		Assert.assertEquals(0,individualCollection8.size());
		
		Set<String> individualNameSet = new TreeSet<String>();
		
		for (IndividualVertex individual : individualCollection2)
		{
			individualNameSet.add(individual.getSimpleName());
		}
		// end for
		
		Assert.assertTrue(individualNameSet.contains("ErikTittel"));
		Assert.assertTrue(individualNameSet.contains("KatjaSiegemund"));
		Assert.assertTrue(individualNameSet.contains("TomBeger"));
	}
	// end testGetIndividualsString

	@Test
	public void testGetIndividual()
	{
		Assert.assertEquals("ConnectPeople",roModel.getIndividual("ConnectPeople").getSimpleName());
		Assert.assertNull(roModel.getIndividual("ConnectPeoples"));
	}
	// end testGetIndividual

	@Test
	public void testContainsIndividual()
	{
		Assert.assertTrue(roModel.containsIndividual("DataPrivacyProtection"));
		Assert.assertTrue(roModel.containsIndividual("CollectUserData"));
		Assert.assertTrue(roModel.containsIndividual("ServerCrash"));
		Assert.assertFalse(roModel.containsIndividual("DataPrivacyProtections"));
		Assert.assertFalse(roModel.containsIndividual("CollectUserDatas"));
		Assert.assertFalse(roModel.containsIndividual("ServerCrashs"));
		
		roModel.removeIndividual("DataPrivacyProtection");
		roModel.removeIndividual("CollectUserData");
		roModel.removeIndividual("ServerCrash");
		
		Assert.assertFalse(roModel.containsIndividual("DataPrivacyProtection"));
		Assert.assertFalse(roModel.containsIndividual("CollectUserData"));
		Assert.assertFalse(roModel.containsIndividual("ServerCrash"));
	}
	// end testContainsIndividual

	@Ignore
	public void testPrintInfo()
	{
		fail("Not yet implemented");
	}
	// end testPrintInfo

	@Test
	public void testGetObjectPropertiesString()
	{
		Collection<ObjectPropertyEdge> objectPropertyEdgeCollection1 = roModel.getObjectProperties("UserHasPhotoAlbum");
		Collection<ObjectPropertyEdge> objectPropertyEdgeCollection2 = roModel.getObjectProperties("UserHasPhotoAlbums");
		Collection<ObjectPropertyEdge> objectPropertyEdgeCollection3 = roModel.getObjectProperties("ConnectPeople");
		
		Assert.assertEquals(5,objectPropertyEdgeCollection1.size());
		Assert.assertNull(objectPropertyEdgeCollection2);
		Assert.assertEquals(2,objectPropertyEdgeCollection3.size());
		
		Set<String> objectPropertyNameSet1 = new TreeSet<String>();
		Set<String> objectPropertyNameSet2 = new TreeSet<String>();
		
		for (ObjectPropertyEdge objectPropertyEdge : objectPropertyEdgeCollection1)
		{
			objectPropertyNameSet1.add(objectPropertyEdge.getSimpleName());
		}
		// end for
		
		for (ObjectPropertyEdge objectPropertyEdge : objectPropertyEdgeCollection3)
		{
			objectPropertyNameSet2.add(objectPropertyEdge.getSimpleName());
		}
		// end for
		
		Assert.assertTrue(objectPropertyNameSet1.contains("isAlternativeTo"));
		Assert.assertTrue(objectPropertyNameSet1.contains("isInConflictWith"));
		Assert.assertTrue(objectPropertyNameSet1.contains("isPositiveContributionTo"));
		Assert.assertTrue(objectPropertyNameSet1.contains("isAuthoredBy"));
		Assert.assertTrue(objectPropertyNameSet1.contains("isNegativeContributionTo"));
		Assert.assertTrue(objectPropertyNameSet2.contains("isAuthoredBy"));
		Assert.assertTrue(objectPropertyNameSet2.contains("refinesTo"));
	}
	// end testGetObjectPropertiesString

	@Test
	public void testGetDataPropertiesString()
	{
		Collection<DataProperty> dataPropertyCollection1 = roModel.getDataProperties("DataControl");
		Collection<DataProperty> dataPropertyCollection2 = roModel.getDataProperties("DataControls");
		Collection<DataProperty> dataPropertyCollection3 = roModel.getDataProperties("ConnectPeople");
		
		Assert.assertEquals(1,dataPropertyCollection1.size());
		Assert.assertNull(dataPropertyCollection2);
		Assert.assertEquals(0,dataPropertyCollection3.size());
		
		Set<String> dataPropertyNameSet = new TreeSet<String>();
		
		for (DataProperty dataProperty : dataPropertyCollection1)
		{
			dataPropertyNameSet.add(dataProperty.getSimpleName());
		}
		// end for
		
		Assert.assertTrue(dataPropertyNameSet.contains("hasRefinementReason"));
	}
	// end testGetDataPropertiesString

	@Test (expected=IllegalArgumentException.class)
	public void testGetObjectPropertiesStringString()
	{
		Collection<ObjectPropertyEdge> objectPropertyEdgeCollection1 = roModel.getObjectProperties("DataControl","DataControl");
		Collection<ObjectPropertyEdge> objectPropertyEdgeCollection2 = roModel.getObjectProperties("DataControls","DataControl");
		Collection<ObjectPropertyEdge> objectPropertyEdgeCollection3 = roModel.getObjectProperties("DataControl","DataControls");
		Collection<ObjectPropertyEdge> objectPropertyEdgeCollection4 = roModel.getObjectProperties("StoreInformation","DataPrivacyProtection");
		Collection<ObjectPropertyEdge> objectPropertyEdgeCollection5 = roModel.getObjectProperties("DataPrivacyProtection","StoreInformation");
		Collection<ObjectPropertyEdge> objectPropertyEdgeCollection6 = roModel.getObjectProperties("IntegratedInstantMessaging","UserHasPhotoAlbum");
		Collection<ObjectPropertyEdge> objectPropertyEdgeCollection7 = roModel.getObjectProperties("UserHasPhotoAlbum","IntegratedInstantMessaging");
		Collection<ObjectPropertyEdge> objectPropertyEdgeCollection8 = roModel.getObjectProperties("AccessibleToBlindUsers","UserHasPhotoAlbum");
		Collection<ObjectPropertyEdge> objectPropertyEdgeCollection9 = roModel.getObjectProperties("UserHasPhotoAlbum","AccessibleToBlindUsers");
		
		Assert.assertEquals(0,objectPropertyEdgeCollection1.size());
		Assert.assertNull(objectPropertyEdgeCollection2);
		Assert.assertNull(objectPropertyEdgeCollection3);
		Assert.assertEquals(1,objectPropertyEdgeCollection4.size());
		Assert.assertEquals(0,objectPropertyEdgeCollection5.size());
		Assert.assertEquals(1,objectPropertyEdgeCollection6.size());
		Assert.assertEquals(1,objectPropertyEdgeCollection7.size());
		Assert.assertEquals(1,objectPropertyEdgeCollection8.size());
		Assert.assertEquals(1,objectPropertyEdgeCollection9.size());
		
		Set<String> objectPropertyNameSet1 = new TreeSet<String>();
		Set<String> objectPropertyNameSet2 = new TreeSet<String>();
		Set<String> objectPropertyNameSet3 = new TreeSet<String>();
		Set<String> objectPropertyNameSet4 = new TreeSet<String>();
		Set<String> objectPropertyNameSet5 = new TreeSet<String>();
		
		for (ObjectPropertyEdge objectPropertyEdge : objectPropertyEdgeCollection4)
		{
			objectPropertyNameSet1.add(objectPropertyEdge.getSimpleName());
		}
		// end for

		for (ObjectPropertyEdge objectPropertyEdge : objectPropertyEdgeCollection6)
		{
			objectPropertyNameSet2.add(objectPropertyEdge.getSimpleName());
		}
		// end for
		
		for (ObjectPropertyEdge objectPropertyEdge : objectPropertyEdgeCollection7)
		{
			objectPropertyNameSet3.add(objectPropertyEdge.getSimpleName());
		}
		// end for
		
		for (ObjectPropertyEdge objectPropertyEdge : objectPropertyEdgeCollection8)
		{
			objectPropertyNameSet4.add(objectPropertyEdge.getSimpleName());
		}
		// end for
		
		for (ObjectPropertyEdge objectPropertyEdge : objectPropertyEdgeCollection9)
		{
			objectPropertyNameSet5.add(objectPropertyEdge.getSimpleName());
		}
		// end for
		
		Assert.assertTrue(objectPropertyNameSet1.contains("isNegativeContributionTo"));
		Assert.assertTrue(objectPropertyNameSet2.contains("isAlternativeTo"));
		Assert.assertTrue(objectPropertyNameSet3.contains("isAlternativeTo"));
		Assert.assertTrue(objectPropertyNameSet4.contains("isInConflictWith"));
		Assert.assertTrue(objectPropertyNameSet5.contains("isInConflictWith"));
	}
	// end testGetObjectPropertiesStringString

	@Test
	public void testGetOntologyAccessor()
	{
		Assert.assertEquals(oA,roModel.getOntologyAccessor());
	}
	// end testGetOntologyAccessor

	@Test
	public void testGetOntologyClassNames()
	{
		Assert.assertEquals(99,roModel.getOntologyClassNames().size());
	}
	// end testGetOntologyClassNames

	@Test
	public void testGetOntologyObjectPropertyNames()
	{
		Assert.assertEquals(33,roModel.getOntologyObjectPropertyNames().size());
		Assert.assertTrue(roModel.getOntologyObjectPropertyNames().contains("belongsToUseCase"));
		Assert.assertTrue(roModel.getOntologyObjectPropertyNames().contains("hasCost"));
		Assert.assertTrue(roModel.getOntologyObjectPropertyNames().contains("hasRelationship"));
		Assert.assertTrue(roModel.getOntologyObjectPropertyNames().contains("isInConflictWith"));
		Assert.assertTrue(roModel.getOntologyObjectPropertyNames().contains("hasSoftMetric"));
		Assert.assertTrue(roModel.getOntologyObjectPropertyNames().contains("isAuthoredBy"));
		Assert.assertTrue(roModel.getOntologyObjectPropertyNames().contains("refinesTo"));
	}
	// end testGetOntologyObjectPropertyNames

	@Test
	public void testGetOntologyDataPropertyNames()
	{
		Assert.assertEquals(7,roModel.getOntologyDataPropertyNames().size());
		Assert.assertTrue(roModel.getOntologyDataPropertyNames().contains("hasCost"));
		Assert.assertTrue(roModel.getOntologyDataPropertyNames().contains("hasID"));
		Assert.assertTrue(roModel.getOntologyDataPropertyNames().contains("hasRefinementReason"));
		Assert.assertTrue(roModel.getOntologyDataPropertyNames().contains("hasTimingMetric"));
		Assert.assertTrue(roModel.getOntologyDataPropertyNames().contains("hasResponseTime"));
		Assert.assertTrue(roModel.getOntologyDataPropertyNames().contains("hasThroughput"));
		Assert.assertTrue(roModel.getOntologyDataPropertyNames().contains("hasWaitingTime"));
	}
	// end testGetOntologyDataPropertyNames

	@Test
	public void testGetOntologyIndividualNames()
	{
		Assert.assertEquals(32,roModel.getOntologyIndividualNames().size());
		Assert.assertTrue(roModel.getOntologyIndividualNames().contains("AccessibleToBlindUsers"));
		Assert.assertTrue(roModel.getOntologyIndividualNames().contains("AdvertisementOnProfilePage"));
		Assert.assertTrue(roModel.getOntologyIndividualNames().contains("CollectUserData"));
		Assert.assertTrue(roModel.getOntologyIndividualNames().contains("ConnectPeople"));
		Assert.assertTrue(roModel.getOntologyIndividualNames().contains("DataControl"));
		Assert.assertTrue(roModel.getOntologyIndividualNames().contains("DataPrivacyProtection"));
	}
	// end testGetOntologyIndividualNames

	@Test
	public void testGetOntologyRootClassNames()
	{
		Assert.assertEquals(7,roModel.getOntologyRootClassNames().size());
		Assert.assertTrue(roModel.getOntologyRootClassNames().contains("Artifact"));
		Assert.assertTrue(roModel.getOntologyRootClassNames().contains("Attribute"));
		Assert.assertTrue(roModel.getOntologyRootClassNames().contains("Exception"));
		Assert.assertTrue(roModel.getOntologyRootClassNames().contains("Refinement"));
		Assert.assertTrue(roModel.getOntologyRootClassNames().contains("Source"));
		Assert.assertTrue(roModel.getOntologyRootClassNames().contains("Stakeholder"));
		Assert.assertTrue(roModel.getOntologyRootClassNames().contains("hasCost"));
	}
	// end testGetOntologyRootClassNames

	@Test
	public void testGetOntologyAttributeClassNames()
	{
		Assert.assertEquals(33,roModel.getOntologyAttributeClassNames().size());
		Assert.assertTrue(roModel.getOntologyAttributeClassNames().contains("Attribute"));
		Assert.assertTrue(roModel.getOntologyAttributeClassNames().contains("Condition"));
		Assert.assertTrue(roModel.getOntologyAttributeClassNames().contains("FailurePostcondition"));
		Assert.assertTrue(roModel.getOntologyAttributeClassNames().contains("Complete"));
		Assert.assertTrue(roModel.getOntologyAttributeClassNames().contains("Important"));
		Assert.assertTrue(roModel.getOntologyAttributeClassNames().contains("HighPriority"));
	}
	// end testGetOntologyAttributeClassNames

	@Test
	public void testGetOntologyNoAttributeClassNames()
	{
		Assert.assertEquals(66,roModel.getOntologyNoAttributeClassNames().size());
		Assert.assertTrue(roModel.getOntologyNoAttributeClassNames().contains("Artifact"));
		Assert.assertTrue(roModel.getOntologyNoAttributeClassNames().contains("Exception"));
		Assert.assertTrue(roModel.getOntologyNoAttributeClassNames().contains("Refinement"));
		Assert.assertTrue(roModel.getOntologyNoAttributeClassNames().contains("Source"));
		Assert.assertTrue(roModel.getOntologyNoAttributeClassNames().contains("Stakeholder"));
		Assert.assertTrue(roModel.getOntologyNoAttributeClassNames().contains("hasCost"));
		Assert.assertTrue(roModel.getOntologyNoAttributeClassNames().contains("Goal"));
		Assert.assertTrue(roModel.getOntologyNoAttributeClassNames().contains("Challenge"));
		Assert.assertTrue(roModel.getOntologyNoAttributeClassNames().contains("Security"));
		Assert.assertTrue(roModel.getOntologyNoAttributeClassNames().contains("Portability"));
	}
	// end testGetOntologyNoAttributeClassNames

	@Test
	public void testGetIndividualComments()
	{
		Set<String> individualCommentSet1 = roModel.getIndividualComments("LoadPageTimeRestriction");
		Set<String> individualCommentSet2 = roModel.getIndividualComments("LoadPageTimeRestrictions");
		Set<String> individualCommentSet3 = roModel.getIndividualComments("ConnectPeople");
		
		Assert.assertEquals(1,individualCommentSet1.size());
		Assert.assertEquals(0,individualCommentSet2.size());
		Assert.assertEquals(0,individualCommentSet3.size());
		Assert.assertTrue(individualCommentSet1.contains("Load Page in less than 2s"));
	}
	// end testGetOntologyIndividualComments

	@Test
	public void testGetIndividualEssentialType()
	{
		Assert.assertEquals("UserSatisfaction",roModel.getIndividualEssentialType("AccessibleToBlindUsers"));
		Assert.assertEquals("",roModel.getIndividualEssentialType("AccessibleToBlindUserss"));
	}
	// end testGetIndividualEssentialType

	@Test
	public void testGetIndividualMainType()
	{
		Assert.assertEquals("Problem",roModel.getIndividualMainType("ServerCrash"));
		Assert.assertEquals("Challenge",roModel.getIndividualMainType("ConnectPeople"));
		Assert.assertEquals("Goal",roModel.getIndividualMainType("FastAccessToPlatform"));
		Assert.assertEquals("Requirement",roModel.getIndividualMainType("AccessibleToBlindUsers"));
		Assert.assertEquals("Story",roModel.getIndividualMainType("ProvidingUserInformation"));
		Assert.assertEquals("TestCase",roModel.getIndividualMainType("TestCase1"));
		Assert.assertEquals("Refinement",roModel.getIndividualMainType("DataControl"));
		Assert.assertEquals("Stakeholder",roModel.getIndividualMainType("TomBeger"));
		Assert.assertEquals("",roModel.getIndividualMainType("AccessibleToBlindUserss"));
	}
	// end testGetIndividualMainType

	@Test
	public void testBelongsToClassOrSubclass()
	{
		Assert.assertTrue(roModel.belongsToClassOrSubclass("ConnectPeople","Artifact"));
		Assert.assertTrue(roModel.belongsToClassOrSubclass("ConnectPeople","Challenge"));
		Assert.assertFalse(roModel.belongsToClassOrSubclass("ConnectPeople","Attribute"));
		Assert.assertFalse(roModel.belongsToClassOrSubclass("ConnectPeoples","Artifact"));
		Assert.assertFalse(roModel.belongsToClassOrSubclass("ConnectPeoples","Challenge"));
	}
	// end testBelongsToClassOrSubclass

	@Test
	public void testGetIndividualAttributes()
	{
		Set<String> attributeNameSet1 = roModel.getIndividualAttributes("LoadPageTimeRestriction");
		Set<String> attributeNameSet2 = roModel.getIndividualAttributes("LoadPageTimeRestrictions");
		Set<String> attributeNameSet3 = roModel.getIndividualAttributes("EarnMoney");
		
		Assert.assertEquals(0,attributeNameSet1.size());
		Assert.assertEquals(0,attributeNameSet2.size());
		Assert.assertEquals(2,attributeNameSet3.size());
		Assert.assertTrue(attributeNameSet3.contains("HighPriority"));
		Assert.assertTrue(attributeNameSet3.contains("Mandatory"));
	}
	// end testGetIndividualAttributes

	@Test
	public void testGetFormalAttributes()
	{
		MultiHashMap<String,String> formalAttributeMap1 = roModel.getFormalAttributes("Goal");
		MultiHashMap<String,String> formalAttributeMap2 = roModel.getFormalAttributes("Goals");
		MultiHashMap<String,String> formalAttributeMap3 = roModel.getFormalAttributes("Stakeholder");
		MultiHashMap<String,String> formalAttributeMap4 = roModel.getFormalAttributes("Optional");
		MultiHashMap<String,String> formalAttributeMap5 = roModel.getFormalAttributes("Artifact");
		MultiHashMap<String,String> formalAttributeMap6 = roModel.getFormalAttributes("Operability");
		
		Assert.assertEquals(5,formalAttributeMap1.size());
		Assert.assertEquals(0,formalAttributeMap2.size());
		Assert.assertEquals(0,formalAttributeMap3.size());
		Assert.assertEquals(0,formalAttributeMap4.size());
		Assert.assertEquals(1,formalAttributeMap5.size());
		Assert.assertEquals(10,formalAttributeMap6.size());
		Assert.assertTrue(formalAttributeMap1.keySet().contains("refinesTo"));
		Assert.assertTrue(formalAttributeMap1.keySet().contains("hasRisk"));
		Assert.assertTrue(formalAttributeMap1.keySet().contains("hasID"));
		Assert.assertTrue(formalAttributeMap1.keySet().contains("hasObstacle"));
		Assert.assertTrue(formalAttributeMap1.keySet().contains("isAuthoredBy"));
		Assert.assertTrue(formalAttributeMap5.keySet().contains("isAuthoredBy"));
		
		for (String formalAttributeName : formalAttributeMap1.keySet())
		{
			Collection<String> individualNameSet = formalAttributeMap1.get(formalAttributeName);
			
			if (individualNameSet.size() > 1)
			{
				Assert.assertEquals("refinesTo",formalAttributeName);
				Assert.assertTrue(individualNameSet.contains("Goal"));
				Assert.assertTrue(individualNameSet.contains("Requirement"));
			}
			// end if
		}
		// end for
	}
	// end testGetFormalAttributes

	@Test
	public void testGetIndividualData()
	{
		ArrayList<MultiHashMap<String,String>> individualDataMap1 = roModel.getIndividualData("DataControls");
		ArrayList<MultiHashMap<String,String>> individualDataMap2 = roModel.getIndividualData("DataControl");
		ArrayList<MultiHashMap<String,String>> individualDataMap3 = roModel.getIndividualData("KatjaSiegemund");
		
		Assert.assertEquals(0,individualDataMap1.get(0).size());
		Assert.assertEquals(0,individualDataMap1.get(1).size());
		Assert.assertEquals(0,individualDataMap1.get(2).size());
		Assert.assertEquals(0,individualDataMap1.get(3).size());
		Assert.assertEquals(0,individualDataMap1.get(4).size());
		Assert.assertEquals(0,individualDataMap1.get(5).size());
		Assert.assertEquals(0,individualDataMap1.get(6).size());
		Assert.assertEquals(0,individualDataMap1.get(7).size());
		Assert.assertEquals(2,individualDataMap2.get(0).size());
		Assert.assertEquals(2,individualDataMap2.get(1).size());
		Assert.assertEquals(1,individualDataMap2.get(2).size());
		Assert.assertEquals(1,individualDataMap2.get(3).size());
		Assert.assertEquals(1,individualDataMap2.get(4).size());
		Assert.assertEquals(1,individualDataMap2.get(5).size());
		Assert.assertEquals(0,individualDataMap2.get(6).size());
		Assert.assertEquals(0,individualDataMap2.get(7).size());
		Assert.assertEquals(0,individualDataMap3.get(0).size());
		Assert.assertEquals(0,individualDataMap3.get(1).size());
		Assert.assertEquals(0,individualDataMap3.get(2).size());
		Assert.assertEquals(0,individualDataMap3.get(3).size());
		Assert.assertEquals(0,individualDataMap3.get(4).size());
		Assert.assertEquals(0,individualDataMap3.get(5).size());
		Assert.assertEquals(1,individualDataMap3.get(6).size());
		Assert.assertEquals(5,individualDataMap3.get(7).size());
		Assert.assertTrue(individualDataMap2.get(0).containsKey("hasRefinementSource"));
		Assert.assertTrue(individualDataMap2.get(0).containsKey("hasRefinementTarget"));
		Assert.assertTrue(individualDataMap2.get(0).containsValue("RequirementArtifact"));
		Assert.assertTrue(individualDataMap2.get(1).containsKey("hasRefinementSource-RequirementArtifact"));
		Assert.assertTrue(individualDataMap2.get(1).containsKey("hasRefinementTarget-RequirementArtifact"));
		Assert.assertTrue(individualDataMap2.get(1).containsValue("UserHasProfilePage"));
		Assert.assertTrue(individualDataMap2.get(1).containsValue("UserCanHideData"));
		Assert.assertTrue(individualDataMap2.get(2).containsKey("isAuthoredBy"));
		Assert.assertTrue(individualDataMap2.get(2).containsValue("Stakeholder"));
		Assert.assertTrue(individualDataMap2.get(3).containsKey("isAuthoredBy-Stakeholder"));
		Assert.assertTrue(individualDataMap2.get(3).containsValue("KatjaSiegemund"));
		Assert.assertTrue(individualDataMap2.get(4).containsKey("hasRefinementReason"));
		Assert.assertTrue(individualDataMap2.get(4).containsValue("string"));
		Assert.assertTrue(individualDataMap2.get(5).containsKey("hasRefinementReason-string"));
		Assert.assertTrue(individualDataMap2.get(5).containsValue("The user must be allowed to completely control which data shall be displayed and which not in order to satisfy data security plocies from government."));
		Assert.assertTrue(individualDataMap3.get(6).containsKey("isAuthoredBy"));
		Assert.assertTrue(individualDataMap3.get(6).containsValue("UserSatisfaction"));
		Assert.assertTrue(individualDataMap3.get(6).containsValue("Refinement"));
		Assert.assertTrue(individualDataMap3.get(6).containsValue("Risk"));
		Assert.assertTrue(individualDataMap3.get(6).containsValue("AtomicFunctionalRequirement"));
		Assert.assertTrue(individualDataMap3.get(6).containsValue("Goal"));
		Assert.assertTrue(individualDataMap3.get(7).containsKey("isAuthoredBy-UserSatisfaction"));
		Assert.assertTrue(individualDataMap3.get(7).containsKey("isAuthoredBy-Refinement"));
		Assert.assertTrue(individualDataMap3.get(7).containsKey("isAuthoredBy-Risk"));
		Assert.assertTrue(individualDataMap3.get(7).containsKey("isAuthoredBy-AtomicFunctionalRequirement"));
		Assert.assertTrue(individualDataMap3.get(7).containsKey("isAuthoredBy-Goal"));
		Assert.assertTrue(individualDataMap3.get(7).containsValue("PlatformIsEasyToUseForEveryone"));
		Assert.assertTrue(individualDataMap3.get(7).containsValue("AccessibleToBlindUsers"));
		Assert.assertTrue(individualDataMap3.get(7).containsValue("InconvenientDesignForSeeingUsers"));
		Assert.assertTrue(individualDataMap3.get(7).containsValue("IntegratedInstantMessaging"));
		Assert.assertTrue(individualDataMap3.get(7).containsValue("UserCanHideData"));
		Assert.assertTrue(individualDataMap3.get(7).containsValue("DataControl"));
	}
	// end testGetIndividualData

	@Test
	public void testCreatesRefinementForest()
	{
		roModel.createRefinementForest();
	
		Collection<IndividualVertex> individualCollection = roModel.getGoalModelGraph().getVertices();
		Collection<ObjectPropertyEdge> objectPropertyEdgeCollection = roModel.getGoalModelGraph().getEdges();
		TreeSet<String> individualNameSet = new TreeSet<String>();
		
		for (IndividualVertex individual : individualCollection)
		{
			individualNameSet.add(individual.getSimpleName());
		}
		// end for
		
		for (ObjectPropertyEdge objectProperty : objectPropertyEdgeCollection)
		{
			Assert.assertTrue(objectProperty.getSimpleName().equals("refinesTo"));
		}
		// end for
		
		Assert.assertEquals(13,roModel.getGoalModelGraph().getVertexCount());
		Assert.assertEquals(0,roModel.getDataProperties().size());	
		Assert.assertTrue(individualNameSet.contains("CollectUserData"));
		Assert.assertTrue(individualNameSet.contains("UserHasProfilePage"));
		Assert.assertTrue(individualNameSet.contains("CommercialSuccess"));
		Assert.assertTrue(individualNameSet.contains("EarnMoney"));
		Assert.assertTrue(individualNameSet.contains("ConnectPeople"));
		Assert.assertTrue(individualNameSet.contains("DevelopSocialNetworkingPlatform"));
		Assert.assertTrue(individualNameSet.contains("PlatformIsEasyToUseForEveryone"));
		Assert.assertTrue(individualNameSet.contains("ExchangeInformation"));
		Assert.assertTrue(individualNameSet.contains("AdvertisementOnProfilePage"));
		Assert.assertTrue(individualNameSet.contains("StoreInformation"));
		Assert.assertTrue(individualNameSet.contains("WellArrangedLayout"));
		Assert.assertTrue(individualNameSet.contains("StoreUserDataInDatabase"));
		Assert.assertTrue(individualNameSet.contains("UsingOracleDatabase"));
	}
	// end testCreatesRefinementForest
}
// end class ROModelTest