package org.deft;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.equinox.app.IApplication;
import org.eclipse.equinox.app.IApplicationContext;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.application.WorkbenchAdvisor;

/**
 * This class is the startup point of the DEFT application.
 */
public class Deft implements IApplication {

	public Object start(IApplicationContext context) throws Exception {
		Map<String, String> arguments = getArguments(context);	
		// Create workbench advisor and display
		WorkbenchAdvisor workbenchAdvisor = new DeftWorkbenchAdvisor(arguments);
		Display display = PlatformUI.createDisplay();
		
		// Start the workbench
		try {
			int returnCode = PlatformUI.createAndRunWorkbench(display,
					workbenchAdvisor);
			if (returnCode == PlatformUI.RETURN_RESTART)			
				return IApplication.EXIT_RESTART;
			else
				return IApplication.EXIT_OK;
		} finally {
			display.dispose();
		}

	}

	public void stop() {
		final IWorkbench workbench = PlatformUI.getWorkbench();
		if (workbench == null)
			return;
		final Display display = workbench.getDisplay();
		display.syncExec(new Runnable() {
			public void run() {
				if (!display.isDisposed())
					workbench.close();
			}
		});
	}
	
	@SuppressWarnings("unchecked")
	public Map<String, String> getArguments(IApplicationContext context) {
		Map<String, String> map = new HashMap<String, String>();
		Map mContextArgs = context.getArguments();
		if (mContextArgs != null) {
			String[] args = (String[])mContextArgs.get("application.args");
			for (String s : args) {
				int posOfEqualsSign = s.indexOf("=");
				if (posOfEqualsSign != -1) {
					String parameterName = s.substring(0, posOfEqualsSign);
					String parameterValue = s.substring(posOfEqualsSign+1, s.length());
					if (!parameterName.equals("") && !parameterValue.equals("")) {
						map.put(parameterName, parameterValue);
					}
				}
			}
		}
		return map;
	}
	
}
