package org.deft;

import org.deft.actions.DeftActionFactory;
import org.deft.actions.contributionitem.DeftContributionItemFactory;
import org.deft.util.ImageHelper;
import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.ICoolBarManager;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.action.ToolBarContributionItem;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.actions.ContributionItemFactory;
import org.eclipse.ui.application.ActionBarAdvisor;
import org.eclipse.ui.application.IActionBarConfigurer;


/**
 * Adds actions to menu bars and toolbars in a workbench window. Also defines
 * GroupMarkers inside menus and toolbars. Those GroupMarkers can be used to
 * extend the menu with declarative code from plugin.xml.
 */
public final class DeftActionBarAdvisor extends ActionBarAdvisor implements
		IDeftActionConstants {


	public DeftActionBarAdvisor(IActionBarConfigurer actionBarConfigurer) {
		super(actionBarConfigurer);
	}
	
	@Override
	protected void makeActions(IWorkbenchWindow window) {
		register(ActionFactory.SAVE.create(window));
		register(ActionFactory.ABOUT.create(window));
		register(ActionFactory.INTRO.create(window));
		register(ActionFactory.SAVE_ALL.create(window));
		register(ActionFactory.UNDO.create(window));
		register(ActionFactory.REDO.create(window));
		register(ActionFactory.CUT.create(window));
		register(ActionFactory.COPY.create(window));
		register(ActionFactory.PASTE.create(window));
		register(ActionFactory.SELECT_ALL.create(window));
		register(ActionFactory.FIND.create(window));	
		register(ActionFactory.CLOSE.create(window));
		register(ActionFactory.CLOSE_ALL.create(window));
		register(ActionFactory.CLOSE_ALL_SAVED.create(window));
		register(ActionFactory.QUIT.create(window));
		register(ActionFactory.PREFERENCES.create(window));
		register(ActionFactory.HELP_CONTENTS.create(window));
		register(ActionFactory.HELP_SEARCH.create(window));

//		register(DeftActionFactory.EXPORT_DROP_DOWN.create(window));
		register(DeftActionFactory.IMPORT_DROP_DOWN.create(window));
//		register(DeftActionFactory.IMPORT_ARTIFACT.create(window));
		register(DeftActionFactory.DELETE.create(window));
		register(DeftActionFactory.RENAME.create(window));
		register(DeftActionFactory.NEW_DROP_DOWN.create(window));
		register(DeftActionFactory.SWITCH_REPOSITORY.create(window));
//		register(DeftActionFactory.UPDATE.create(window));
		
	}

	/**
	 * Creates the toolbar
	 */
	protected void fillCoolBar(ICoolBarManager cbManager) {

		// Add groups to the CoolBar
		cbManager.add(new GroupMarker(GROUP_FILE));
		cbManager.add(new GroupMarker(ADDITIONS));
		cbManager.add(new GroupMarker(GROUP_EDITOR));

		// Add groups to the File CoolBar
		IToolBarManager fileToolBar = new ToolBarManager(cbManager.getStyle());
		fileToolBar.add(new Separator(GROUP_NEW));
		fileToolBar.add(new Separator(GROUP_OPEN));
		fileToolBar.add(new Separator(GROUP_SAVE));
		fileToolBar.add(new Separator(GROUP_EDIT));
		fileToolBar.add(new Separator(GROUP_EXPORT));

		// Add Actions to File CoolBar
		IAction newDropDownAction = getAction(DeftActionFactory.NEW_DROP_DOWN.getId());
		fileToolBar.appendToGroup(GROUP_NEW, newDropDownAction);
				
//		fileToolBar.appendToGroup(GROUP_SAVE, getAction(ActionFactory.SAVE
//				.getId()));
//		fileToolBar.appendToGroup(GROUP_SAVE, getAction(ActionFactory.SAVE_ALL
//				.getId()));
//		fileToolBar.appendToGroup(GROUP_EXPORT,
//				getAction(DeftActionFactory.EXPORT_DROP_DOWN.getId()));
//
//		// Add to the cool bar manager
		cbManager.appendToGroup(GROUP_FILE, new ToolBarContributionItem(
				fileToolBar));

	}

	/**
	 * Adds menus to menubar
	 */
	protected void fillMenuBar(IMenuManager menubar) {
		menubar.add(createFileMenu());
		menubar.add(createEditMenu());
		menubar.add(new GroupMarker(ADDITIONS));
		menubar.add(createHelpMenu());
	}

	/**
	 * Creates and returns the File menu.
	 */
	private MenuManager createFileMenu() {

		MenuManager menu = new MenuManager("&File", MENU_FILE);

		menu.add(new Separator(GROUP_NEW));
		menu.add(new Separator(GROUP_CLOSE));
		menu.add(new Separator(GROUP_SAVE));
		menu.add(new Separator(GROUP_EDIT));
		menu.add(new Separator(GROUP_SWITCH));
		menu.add(new Separator(GROUP_VIEW));
		menu.add(new GroupMarker(GROUP_REOPEN));
		menu.add(new Separator(GROUP_QUIT));

		IWorkbenchWindow window = getActionBarConfigurer()
				.getWindowConfigurer().getWindow();

		menu.appendToGroup(GROUP_NEW,
				getAction(DeftActionFactory.NEW_DROP_DOWN.getId()));

		menu.appendToGroup(GROUP_NEW,
				getAction(DeftActionFactory.IMPORT_DROP_DOWN.getId()));

		menu.appendToGroup(GROUP_CLOSE,
				getAction(ActionFactory.CLOSE.getId()));
		
		menu.appendToGroup(GROUP_CLOSE,
				getAction(ActionFactory.CLOSE_ALL.getId()));

		menu.appendToGroup(GROUP_SAVE,
				getAction(ActionFactory.SAVE.getId()));
		
		menu.appendToGroup(GROUP_SAVE,
				getAction(ActionFactory.SAVE_ALL.getId()));

//		menu.appendToGroup(GROUP_EDIT,
//				getAction(DeftActionFactory.RENAME.getId()));

		menu.appendToGroup(GROUP_SWITCH,
				getAction(DeftActionFactory.SWITCH_REPOSITORY.getId()));


		menu.appendToGroup(GROUP_VIEW, DeftContributionItemFactory.VIEW_MENU
				.create(window));

		menu.appendToGroup(GROUP_REOPEN, ContributionItemFactory.REOPEN_EDITORS
				.create(window));

		IAction quitAction = getAction(ActionFactory.QUIT.getId());
//		quitAction.setImageDescriptor(ResourceHelper
//				.getImageDescriptor("enabled/exit.png"));
		menu.appendToGroup(GROUP_QUIT, quitAction);

		return menu;
	}

	/**
	 * Creates and returns the Edit menu.
	 */
	private MenuManager createEditMenu() {
		MenuManager menu = new MenuManager("&Edit", MENU_EDIT);

		menu.add(new Separator(GROUP_UNDO));
		menu.add(new Separator(GROUP_EDIT));		
		menu.add(new Separator(GROUP_SELECT));
		menu.add(new Separator(GROUP_FIND));
		menu.add(new Separator(GROUP_PREFERENCES));

		menu.appendToGroup(GROUP_UNDO, getAction(ActionFactory.UNDO.getId()));
		menu.appendToGroup(GROUP_UNDO, getAction(ActionFactory.REDO.getId()));

		menu.appendToGroup(GROUP_EDIT, getAction(ActionFactory.CUT.getId()));
		menu.appendToGroup(GROUP_EDIT, getAction(ActionFactory.COPY.getId()));
		menu.appendToGroup(GROUP_EDIT, getAction(ActionFactory.PASTE.getId()));		
		menu.appendToGroup(GROUP_EDIT, getAction(DeftActionFactory.DELETE.getId()));
//		menu.appendToGroup(GROUP_EDIT, getAction(DeftActionFactory.UPDATE.getId()));		
				
		IAction selectAllAction = getAction(ActionFactory.SELECT_ALL.getId());
//		selectAllAction.setImageDescriptor(ResourceHelper
//				.getImageDescriptor("enabled/edit-select-all.png"));
		menu.appendToGroup(GROUP_SELECT, selectAllAction);
		
		menu.appendToGroup(GROUP_FIND, getAction(ActionFactory.FIND.getId()));

		// Get preferences action, alter image and add to menu
		IAction preferencesAction = getAction(ActionFactory.PREFERENCES.getId());
//		preferencesAction.setImageDescriptor(ResourceHelper
//				.getImageDescriptor("enabled/preferences.png"));
		menu.appendToGroup(GROUP_PREFERENCES, preferencesAction);

		return menu;
	}

	/**
	 * Creates and returns the Help menu.
	 */
	private MenuManager createHelpMenu() {
		MenuManager menu = new MenuManager("&Help", MENU_HELP);

		menu.add(new Separator(GROUP_WELCOME));
		menu.add(new Separator(GROUP_HELP));
		menu.add(new Separator(GROUP_ABOUT));

		menu.appendToGroup(GROUP_WELCOME,
				getAction(ActionFactory.INTRO.getId()));

		/*
		menu.appendToGroup(GROUP_HELP, getAction(ActionFactory.HELP_CONTENTS
				.getId()));
		menu.appendToGroup(GROUP_HELP, getAction(ActionFactory.HELP_SEARCH
				.getId()));
		*/

		IAction aboutAction = getAction(ActionFactory.ABOUT.getId());
		aboutAction.setImageDescriptor(ImageHelper.getDeftIconDescriptor("enabled/about.png"));
		menu.appendToGroup(GROUP_ABOUT, aboutAction);

		return menu;
	}

	

}
