/*
 * This file is part of 'DEFT', a program for creating tutorials.
 * 
 * Copyright (C) 2007 Comarch Software AG [http://www.comarch.de]
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
package org.deft;


import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;

public class DeftErrorDialog extends ErrorDialog{
    
	private Button details;
	private boolean detailsOpen = false;
	private List list;
	
	public DeftErrorDialog(Shell parentShell, String dialogTitle, String message, IStatus status, int displayMask) {
		super(parentShell, dialogTitle, message, status, displayMask);
	}

	public static int open(Shell parentShell, String title,
            String message, IStatus status) {
		
		DeftErrorDialog dialog = new DeftErrorDialog(parentShell,title,message,status,IStatus.OK
                | IStatus.INFO | IStatus.WARNING | IStatus.ERROR);
        return dialog.open();
    }
	
    protected Control createDialogArea(Composite parent) {
        createMessageArea(parent);
        // create a composite with standard margins and spacing
        Composite composite = new Composite(parent, SWT.NONE);
        GridLayout layout = new GridLayout();
        layout.numColumns = 1;
        composite.setLayout(layout);
        //composite.setBackground(new Color(Display.getCurrent(),100,100,100));
        GridData childData = new GridData(GridData.FILL_BOTH);
        childData.horizontalSpan = 2;
        composite.setLayoutData(childData);
        composite.setFont(parent.getFont());
        return composite;
    }

    
    @Override
    protected void createButtonsForButtonBar(Composite parent) {
    	createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL,
                true);
        
		if (shouldShowDetailsButton()) {
            details = createButton(parent, IDialogConstants.DETAILS_ID,
                    IDialogConstants.HIDE_DETAILS_LABEL, false);
        }
    }
    
    @Override
    protected void buttonPressed(int id) {
        if (id == IDialogConstants.DETAILS_ID) {
            handleDetailsArea();
        } else {
            super.buttonPressed(id);
        }
    }

	private void handleDetailsArea() {
		if (detailsOpen){
			//hide details
			details.setText(IDialogConstants.HIDE_DETAILS_LABEL);
			list.dispose();
			detailsOpen = false;
		}else{
			//show details
			if (list == null || list.isDisposed()){
				list = createDropDownList((Composite)getDialogArea());
			}
			details.setText(IDialogConstants.SHOW_DETAILS_LABEL);
			detailsOpen = true;
		}
		
		getShell().pack();
	}
	
	

	
}
