/*
 * This file is part of 'DEFT', a program for creating tutorials.
 * 
 * Copyright (C) 2007 Comarch Software AG [http://www.comarch.de]
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
package org.deft;

import org.eclipse.core.runtime.Platform;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

/**
 * This class should be handled as a Singleton, the public constructor is only
 * because this class gets instantiated by the framework.
 * 
 * This class is needed to get the right preference store. Call
 * {@link DEFTPlugin#getPreferenceStore()} to get the right preference store.
 * 
 */
public class DeftPlugin extends AbstractUIPlugin {

	private static DeftPlugin instance;

	public final static String ID = "org.deft";

	/**
	 * Public constructor needed for framework instantiation.
	 */
	public DeftPlugin() {
		instance = this;
	}

	/**
	 * Returns the instance of this singleton.
	 * 
	 * @return
	 */
	public static DeftPlugin getInstance() {
		if (instance == null) {
			instance = new DeftPlugin();
		}
		return instance;
	}
	
	@Override
	public void start(BundleContext context) throws Exception {
		System.err.println("######### STARTING " + ID + " #########");
		super.start(context);
		Platform.addLogListener(new LogHandler());
	}

	@Override
	public void stop(BundleContext context) throws Exception {
		System.err.println("######### STOPPING " + ID + " #########");
		instance = null;
		super.stop(context);
	}

}
