package org.deft;

import java.util.LinkedList;
import java.util.List;

import org.deft.artifactviewer.ArtifactViewerConnector;
import org.deft.artifactviewer.ArtifactViewerConnectorRegistry;
import org.deft.configuration.ChainableGuiRegistry;
import org.deft.configuration.IntegratorGui;
import org.deft.configuration.OperationGui;
import org.deft.configuration.ProjectTypeGuiRegistry;
import org.deft.exporter.Exporter;
import org.deft.exporter.ExporterRegistry;
import org.deft.integrator.EditorIntegrator;
import org.deft.integrator.EditorIntegratorRegistry;
import org.deft.operation.Operation;
import org.deft.operation.OperationChain;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.chaptertype.ChapterType2IntegratorIdMappingRegistry;
import org.deft.repository.fragmentsource.FragmentSource;
import org.deft.repository.integrator.FileIntegrator;
import org.deft.repository.integrator.Integrator;
import org.deft.repository.projecttype.ProjectType;
import org.deft.repository.service.ExtensionService;
import org.deft.representationrenderer.RepresentationRenderer;
import org.deft.representationrenderer.RepresentationRendererRegistry;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.ui.IPerspectiveDescriptor;
import org.eclipse.ui.IPerspectiveRegistry;
import org.eclipse.ui.PlatformUI;

/**
 * ExtensionService interface which also contains 
 * methods that involve the GUI. 
 */
public class ExtensionServiceExtensionImpl implements ExtensionServiceExtension {

	private ExtensionService basicExtensionService;

	private ProjectTypeGuiRegistry projectTypeGuiReg;
	private ArtifactViewerConnectorRegistry artifactViewerConnectorReg;
	private RepresentationRendererRegistry rendererReg;
	private ChainableGuiRegistry guiReg;
	private EditorIntegratorRegistry editorIntegratorReg;
	private ChapterType2IntegratorIdMappingRegistry chapterType2IntegratorMappingReg;
	private ExporterRegistry exporterReg;
	
	public ExtensionServiceExtensionImpl(ExtensionService basicExtensionService,
			ProjectTypeGuiRegistry projectTypeGuiReg, 
			ArtifactViewerConnectorRegistry artifactViewerConnectorReg,
			RepresentationRendererRegistry rendererReg,
			ChainableGuiRegistry guiReg, 
			ChapterType2IntegratorIdMappingRegistry chapterType2IntegratorMappingReg,
			EditorIntegratorRegistry editorIntegratorReg, ExporterRegistry exporterReg) {

		this.basicExtensionService = basicExtensionService;
		this.projectTypeGuiReg = projectTypeGuiReg;
		this.artifactViewerConnectorReg = artifactViewerConnectorReg;
		this.rendererReg = rendererReg;
		this.guiReg = guiReg;
		this.chapterType2IntegratorMappingReg = chapterType2IntegratorMappingReg;
		this.editorIntegratorReg = editorIntegratorReg;
		this.exporterReg = exporterReg;
	}
	
	@Override
	public IPerspectiveDescriptor getPerspective(String projectTypeId) {
		String perspectiveId = projectTypeGuiReg.getPerspectiveId(projectTypeId);
		if (perspectiveId != null) {
			IPerspectiveRegistry perspectiveRegistry = PlatformUI.getWorkbench().getPerspectiveRegistry();
			IPerspectiveDescriptor perspective = perspectiveRegistry.findPerspectiveWithId(perspectiveId);
			return perspective;
		}
		return null;
	}
	
	@Override
	public String getChapterEditorId(String projectTypeId, String chapterTypeId) {
		return projectTypeGuiReg.getChapterEditorId(projectTypeId, chapterTypeId);
	}


	@Override
	public ArtifactViewerConnector getArtifactViewerConnectorForRendererId(String rendererId) {
		String viewerId = rendererReg.getArtifactViewerConnectorIdForRendererId(rendererId);
		ArtifactViewerConnector connector = artifactViewerConnectorReg.getConnector(viewerId);
		return connector;
	}
	
	
	
	
	@Override
	public FragmentSource getFragmentSource(String fragmentSourceId) {
		return basicExtensionService.getFragmentSource(fragmentSourceId);
	}

	@Override
	public List<String> getArtifactTypeIds() {
		return basicExtensionService.getArtifactTypeIds();
	}

	@Override
	public List<ArtifactType> getArtifactTypes() {
		return basicExtensionService.getArtifactTypes();
	}

	@Override
	public ArtifactType getArtifactType(String artifactTypeId) {
		return basicExtensionService.getArtifactType(artifactTypeId);
	}

	@Override
	public List<String> getProjectTypeIds() {
		return basicExtensionService.getProjectTypeIds();
	}

	@Override
	public List<ProjectType> getProjectTypes() {
		return basicExtensionService.getProjectTypes();
	}

	@Override
	public ProjectType getProjectType(String id) {
		return basicExtensionService.getProjectType(id);
	}


	@Override
	public List<String> getFileIntegratorIds() {
		return basicExtensionService.getFileIntegratorIds();
	}

	@Override
	public List<EditorIntegrator> getEditorIntegrators() {
		return editorIntegratorReg.getIntegrators();
	}

	@Override
	public EditorIntegrator getEditorIntegrator(String id) {
		return editorIntegratorReg.getIntegrator(id);
	}
	
	@Override
	public List<FileIntegrator> getFileIntegrators() {
		return basicExtensionService.getFileIntegrators();
	}

	@Override
	public FileIntegrator getFileIntegrator(String id) {
		return basicExtensionService.getFileIntegrator(id);
	}

	@Override
	public List<String> getFileExtensionsForArtifactTypeId(String artifactTypeId) {
		return basicExtensionService.getFileExtensionsForArtifactTypeId(artifactTypeId);
	}

	@Override
	public List<String> getArtifactTypeIdsForFileExtension(String fileExtension) {
		return basicExtensionService.getArtifactTypeIdsForFileExtension(fileExtension);
	}

	@Override
	public List<FileIntegrator> getFileIntegratorsForChapterType(String chapterTypeId) {
		return basicExtensionService.getFileIntegratorsForChapterType(chapterTypeId);
	}

	@Override
	public List<String> getChapterTypeIdsForFileIntegrator(String integratorId) {
		return basicExtensionService.getChapterTypeIdsForFileIntegrator(integratorId);
	}

	@Override
	public List<String> getOperationChainIds() {
		return basicExtensionService.getOperationChainIds();
	}

	@Override
	public List<OperationChain> getOperationChains() {
		return basicExtensionService.getOperationChains();
	}
	
	@Override
	public List<OperationChain> getCompatibleOperationChainsForFileIntegration(String artifactTypeId, String chapterTypeId) {
		return basicExtensionService.getCompatibleOperationChainsForFileIntegration(artifactTypeId, chapterTypeId);
	}

	@Override
	public OperationChain getOperationChain(String id) {
		return basicExtensionService.getOperationChain(id);
	}

	@Override
	public List<OperationChain> getCompatibleOperationChainsForEditorIntegration(
			String artifactTypeId, String chapterTypeId) {
		List<OperationChain> chains = new LinkedList<OperationChain>();
		for (OperationChain chain : basicExtensionService.getOperationChains()) {
			if (areCompatible(artifactTypeId, chain, chapterTypeId)) {
				chains.add(chain);
			}
		}
		return chains;
	}
	
	private boolean areCompatible(String artifactTypeId, OperationChain chain,
			String chapterTypeId) {
		ArtifactType artifactType = getArtifactType(artifactTypeId);
		boolean compatible = chain.isValidInputArtifactType(artifactType)
				&& doCompatibleEditorIntegratorsExist(chain, chapterTypeId);
		return compatible;
	}
	
	private boolean doCompatibleEditorIntegratorsExist(OperationChain chain, String chapterTypeId) {
		List<EditorIntegrator> integratorsForChapterType = getEditorIntegratorsForChapterType(chapterTypeId);
		for (Integrator integrator : integratorsForChapterType) {
			String integratorInputType = integrator.getInputRepresentationType();
			String chainOutputType = chain.getFinalRepresentationType();
			if (integratorInputType.equals(chainOutputType)) {
				return true;
			}
		}
		return false;	
	}

	

	@Override
	public List<String> getOperationChainIdsForArtifactType(
			String artifactTypeId) {
		return basicExtensionService.getOperationChainIdsForArtifactType(artifactTypeId);
	}

	@Override
	public List<String> getFileIntegratorIdsForOperationChain(String operationChainId) {
		return basicExtensionService.getFileIntegratorIdsForOperationChain(operationChainId);
	}

	@Override
	public List<RepresentationRenderer> getRenderersForRepresentationType(String representationType) {
		List<RepresentationRenderer> renderers = new LinkedList<RepresentationRenderer>();
		for (RepresentationRenderer renderer : rendererReg.getRenderers()) {
			if (renderer.canRender(representationType)) {
				renderers.add(renderer);
			}
		}
		return renderers;
	}

	@Override
	public RepresentationRenderer getRepresentationRenderer(String rendererId) {
		RepresentationRenderer renderer = rendererReg.getRenderer(rendererId);
		return renderer;
	}

	@Override
	public OperationGui getOperationGui(Operation operation) {
		return guiReg.getOperationGui(operation);
	}

	@Override
	public String getOperationGuiName(Operation operation) {
		return guiReg.getOperationGuiName(operation);
	}
	

	@Override
	public IntegratorGui getIntegratorGui(Integrator integrator) {
		return guiReg.getIntegratorGui(integrator);
	}
	
	@Override
	public String getIntegratorGuiName(Integrator integrator) {
		return guiReg.getIntegratorGuiName(integrator);

	}


	@Override
	public List<EditorIntegrator> getEditorIntegratorsForOperationChain(String operationChainId) {
		List<EditorIntegrator> integrators = new LinkedList<EditorIntegrator>();
		OperationChain chain = getOperationChain(operationChainId);
		String chainRepTypeId = chain.getFinalRepresentationType();
		for (EditorIntegrator integrator : getEditorIntegrators()) {
			String integratorRepTypeId = integrator.getInputRepresentationType();
			if (integratorRepTypeId.equals(chainRepTypeId)) {
				integrators.add(integrator);
			}
		}
		return integrators;
	}

	@Override
	public List<String> getEditorIntegratorIds() {
		return editorIntegratorReg.getIntegratorIds();
	}

	@Override
	public List<EditorIntegrator> getEditorIntegratorsForChapterType(String chapterTypeId) {
		List<EditorIntegrator> integrators = new LinkedList<EditorIntegrator>();
		List<String> ids = chapterType2IntegratorMappingReg.getIntegratorIds(chapterTypeId);
		for (String integratorId : ids) {
			Integrator integrator = getEditorIntegrator(integratorId);
			if (integrator != null) {
				integrators.add((EditorIntegrator)integrator);
			}
		}
		return integrators;
	}


	@Override
	public List<String> getExporterIds() {
		List<String> exporterIds = exporterReg.getExporterIds();
		return exporterIds;
	}
	
	@Override
	public List<String> getExporterIds(String chapterTypeId) {
		List<String> exporterIds = exporterReg.getExporterIds(chapterTypeId);
		return exporterIds;
	}
	
	@Override
	public Exporter getExporter(String exporterId) {
		Exporter exporter = exporterReg.getExporter(exporterId);
		return exporter;
	}
	
	@Override
	public List<WizardPage> getWizardPages(String exporterId) {
		List<WizardPage> pages = exporterReg.getWizardPages(exporterId);
		return pages;
	}

	
}
