/*
 * This file is part of 'DEFT', a program for creating tutorials.
 * 
 * Copyright (C) 2007 Comarch Software AG [http://www.comarch.de]
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
package org.deft;

import org.eclipse.core.runtime.ILogListener;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.widgets.Display;

public class LogHandler implements ILogListener {

	public void logging(IStatus status, String plugin) {
		String pluginName = status.getPlugin();
		if (!pluginName.startsWith("org.eclipse"))
			if (status.getSeverity() == Status.ERROR || status.getCode() == 1)
				openErrorDialog(status);
		
	}

	private void openErrorDialog(IStatus status) {
		final IStatus resultStatus;
		
		Throwable e = status.getException();
		if (e != null) {
			resultStatus = createStatus(e, status.getMessage(), status);
		} else {
			resultStatus = status;
		}
		
		// Open dialog via syncExec so that we can call this method out of non-display threads
		Display.getDefault().syncExec(new Runnable() {
			public void run() {
				DeftErrorDialog.open(Display.getCurrent().getActiveShell(), null , null , resultStatus);
			}
		});
	}
	
	/**
	 * Builds a MultiStatus that includes the whole stacktrace.
	 * @param exception
	 * @param message
	 * @param status
	 * @return
	 */
	private IStatus createStatus(Throwable exception, String message, IStatus status){
		if (exception != null){
			MultiStatus ms = new MultiStatus(status.getPlugin(),status.getCode(), message, exception);
			for (StackTraceElement ste : exception.getStackTrace()) {
				ms.add(new Status(status.getSeverity(), status.getPlugin(), status.getCode(), ste.toString(), null));
			}
			
			IStatus cause = createStatus(exception.getCause(), "Caused by:  ", status);
			if (cause != null){
				ms.add(cause);
			}
			return ms;
		}
		return null;
	}

}
