package org.deft.actions;

import org.deft.gui.EditorHelper;
import org.deft.gui.SelectionHelper;
import org.deft.gui.view.project.ProjectExplorerView;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;

public abstract class AbstractDeftAction extends Action implements IWorkbenchAction, IPartListener {
	
	private IWorkbenchWindow window;
	private SelectionHelper selectionHelper;
	private EditorHelper editorHelper;
	private SelectionListener selectionListener; 
	
	public AbstractDeftAction(IWorkbenchWindow window) {
		this.window = window;
		this.selectionListener = new SelectionListener();
		this.selectionHelper = new SelectionHelper(window);
		this.editorHelper = new EditorHelper(window);
		window.getSelectionService().addSelectionListener(ProjectExplorerView.ID, selectionListener);
		window.getPartService().addPartListener(this);
		setEnabledState();
	}
	
	protected IWorkbenchWindow getWindow() {
		//window can be set to null during some internal update events
		//(e.g. when opening a view via the ViewMenu)
		if (window == null) {
			IWorkbench wb = PlatformUI.getWorkbench();
			window = wb.getActiveWorkbenchWindow();
		}
		return window;
	}
	
	protected abstract void setEnabledState();
	
	protected SelectionHelper getSelectionHelper() {
		if (selectionHelper == null) {
			IWorkbenchWindow window = getWindow();
			selectionHelper = new SelectionHelper(window);
		}
		return selectionHelper;
	}
	
	protected EditorHelper getEditorHelper() {
		return editorHelper;
	}
	
	
	@Override
	public void dispose() {	
		this.selectionHelper = null;
		if (window != null) {
			this.window.getSelectionService().removeSelectionListener(selectionListener);
			this.window = null;
		}
	}
	
	
	protected boolean isProjectViewActive() {
		IWorkbenchPart activePart = getActivePart();
		boolean isProjectViewActive = activePart instanceof ProjectExplorerView;
		return isProjectViewActive;
	}
	
	protected IWorkbenchPart getActivePart() {
		IWorkbenchPart activePart = null;
		IWorkbenchWindow window = getWindow();
		if (window.getActivePage() != null) {
			activePart = window.getActivePage().getActivePart();
		}
		return activePart;
	}
	
	
	private class SelectionListener implements ISelectionListener {

		@Override
		public void selectionChanged(IWorkbenchPart part, ISelection selection) {
			setEnabledState();
		}
		
	}

	
	/**
	 * This method runs whenever some part of the workbench gets activated and
	 * then enables or disables this Action. 
	 * 
	 * This is necessary, for example, to avoid the remove dialog popping up 
	 * while deleting some text in the text editor with the Delete key.
	 */
	@Override
	public void partActivated(IWorkbenchPart part) {
		setEnabledState();
	}
	
	@Override
	public void partDeactivated(IWorkbenchPart part) {
	}

	@Override
	public void partBroughtToTop(IWorkbenchPart part) {
	}

	@Override
	public void partClosed(IWorkbenchPart part) {
	}

	@Override
	public void partOpened(IWorkbenchPart part) {
	}
	
	

}
