package org.deft.actions;

import org.deft.RepositoryFacade;
import org.deft.gui.SelectionHelper;
import org.deft.gui.view.project.nodes.CategoryNode;
import org.deft.gui.view.project.nodes.ProjectNode;
import org.deft.repository.datamodel.Folder;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.FragmentType;
import org.deft.repository.datamodel.Project;
import org.eclipse.ui.IWorkbenchWindow;

public abstract class AbstractNewFragmentAction extends AbstractDeftAction {

	
	protected RepositoryFacade repository = RepositoryFacade.instance();
	
	private Fragment createdFragment;
	
	public AbstractNewFragmentAction(IWorkbenchWindow window) {
		super(window);
	}
	
	
	protected void setEnabledState() {
		SelectionHelper selection = getSelectionHelper();
		if (selection.isOnlySingleCategoryNodeSelected()) {
			setEnabledStateForSelectedCategory();
		} else if (selection.isOnlySingleFolderNodeSelected()) {
			setEnabledStateForSelectedFolder();
		} else {
			setEnabled(false);
		}
	}
	

	
	public void run() {
		SelectionHelper selection = getSelectionHelper();
		if (selection.isOnlySingleCategoryNodeSelected()) {
			CategoryNode category = getSelectedCategoryNode();
			ProjectNode projectNode = (ProjectNode)category.getParent();
			Project project = projectNode.getProject();
			FragmentType fragType = category.getType();
			createdFragment = createFragment(project, fragType);
		} else if (selection.isOnlySingleFolderNodeSelected()) {
			Folder parentFolder = selection.getSelectedFolder();
			createdFragment = createFragment(parentFolder);
		}
	}

	public Fragment getCreatedFragment() {
		return createdFragment;
	}
	

	
	protected CategoryNode getSelectedCategoryNode() {
		SelectionHelper selection = getSelectionHelper();
		CategoryNode cat = (CategoryNode)selection.getFirstSelectedProjectViewNode();
		return cat;
	}
	
	
	

	protected abstract Fragment createFragment(Project project, FragmentType fragType);
	
	protected abstract Fragment createFragment(Folder parentFolder);

	protected abstract void setEnabledStateForSelectedCategory();
	
	protected abstract void setEnabledStateForSelectedFolder();



}
