package org.deft.actions;

import java.util.List;

import org.deft.exporter.ExportWizard;
import org.deft.exporter.Exporter;
import org.deft.gui.SelectionHelper;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Tutorial;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchWindow;

public class ExportAction extends AbstractDeftAction {
	
	private Exporter exporter;
	private List<WizardPage> wizardPages;
	
	public ExportAction(IWorkbenchWindow window, String caption, Exporter exporter, List<WizardPage> wizardPages) {
		super(window);
		this.exporter = exporter;
		this.wizardPages = wizardPages;
		setText(caption);
	}
	
		
	@Override
	protected void setEnabledState() {
		SelectionHelper selection = getSelectionHelper();
		if (selection.isExactlyOneProjectViewNodeSelected()) {
			boolean isValidTarget = isValidTarget();
			setEnabled(isValidTarget);
		} else {
			setEnabled(false);
		}
	}
	
	private boolean isValidTarget() {
		SelectionHelper selection = getSelectionHelper();
		boolean isValidTarget = selection.isOnlySingleChapterNodeSelected()
				|| selection.isOnlySingleTutorialNodeSelected();
		return isValidTarget;
	}
	
	
	@Override
	public void run() {
		SelectionHelper selection = getSelectionHelper();
		if (selection.isOnlySingleChapterNodeSelected()) {
			Chapter chapter = selection.getSelectedChapter();
			exportChapter(chapter);
		} else if (selection.isOnlySingleTutorialNodeSelected()) {
			Tutorial tutorial = selection.getSelectedTutorial();
			exportTutorial(tutorial);
		}
	}
	

	public void exportChapter(Chapter chapter) {
		//TODO ask for saving of chapters
		ExportWizard wizard = new ExportWizard(chapter, exporter, wizardPages);
		Shell shell = Display.getCurrent().getActiveShell();
		WizardDialog dia = new WizardDialog(shell, wizard);
		dia.open();
	}
	



	public void exportTutorial(Tutorial tutorial) {
	}


	

}
