package org.deft.actions;

import org.deft.chaptereditor.ChapterEditorPart;
import org.deft.gui.EditorHelper;
import org.deft.gui.SelectionHelper;
import org.deft.integrator.EditorChapterDocument;
import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.Chapter;
import org.eclipse.ui.IWorkbenchWindow;

/**
 * Action for jumping to artifact references in documents, or the 
 * corresponding artifact representations. This comprises
 * 
 * - opening the document (chapter), if it is not already open
 * - making the editor with the document the active editor
 * - going to the location in the editor where the reference or 
 *   representation is
 */
public class GoToReferencedRepresentationAction extends AbstractDeftAction {

	public GoToReferencedRepresentationAction(IWorkbenchWindow window) {
		super(window);
		setText("Go to Reference");
	}

	@Override
	protected void setEnabledState() {
		SelectionHelper selectionHelper = getSelectionHelper();
		boolean enabled = selectionHelper.isOnlySingleArtifactReferenceNodeSelected();
		setEnabled(enabled);
	}
	
	
	
	@Override
	public void run() {
		SelectionHelper selectionHelper = getSelectionHelper();
		if (selectionHelper.isOnlySingleArtifactReferenceNodeSelected()) {
			ArtifactReference reference = selectionHelper.getSelectedArtifactReference();
			goToReference(reference);
		}

	}
	
	public void goToReference(ArtifactReference reference) {
		EditorHelper editorHelper = getEditorHelper();
		Chapter chapter = reference.getChapter();
//		ChapterEditorPart editor = editorHelper.getOrOpenEditorFor(chapter);
		//we need openEditor instead of getOrOpenEditorFor, because otherwise we only jump
		//the reference if the editor in question is already active
		ChapterEditorPart editor = editorHelper.openEditor(chapter);
		EditorChapterDocument chapterDoc = editor.getChapterDocument();
		chapterDoc.scrollToRepresentation(reference);
	}
}
