package org.deft.actions;

import java.util.List;

import org.deft.RepositoryFacade;
import org.deft.fragmentsourcegui.FragmentSourceGui;
import org.deft.gui.SelectionHelper;
import org.deft.gui.UIArtifactTypeResolver;
import org.deft.gui.view.project.nodes.CategoryNode;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.Folder;
import org.deft.repository.datamodel.FragmentType;
import org.deft.repository.datamodel.Project;
import org.deft.repository.exception.DeftArtifactTypeNotFoundException;
import org.deft.repository.fragmentsource.FragmentSource;
import org.deft.repository.fragmentsource.OriginalFragmentLocation;
import org.deft.repository.repositorymanager.data.ArtifactTypeResolver;
import org.deft.repository.service.ExtensionService;
import org.eclipse.ui.IWorkbenchWindow;

public class ImportArtifactAction extends AbstractDeftAction {

	private FragmentSourceGui fragmentSourceGui;
	
	public ImportArtifactAction(IWorkbenchWindow window, FragmentSourceGui fragmentSourceGui) {
		super(window);
		this.fragmentSourceGui = fragmentSourceGui;
		this.setText(fragmentSourceGui.getName());
	}

	
	@Override
	protected void setEnabledState() {
		SelectionHelper selection = getSelectionHelper();
		if (selection.isExactlyOneProjectViewNodeSelected()) {
			boolean isValidTarget = isValidTarget();
			setEnabled(isValidTarget);
		} else {
			setEnabled(false);
		}
	}
	
	private boolean isValidTarget() {
		boolean isValidTarget = isOnlySingleArtifactCategoryNodeSelected()
				|| isOnlySingleArtifactFolderNodeSelected();
		return isValidTarget;
	}
	
	@Override
	public void run() {
		SelectionHelper selection = getSelectionHelper();
		if (isOnlySingleArtifactCategoryNodeSelected()) {
			Project project = getProjectOfSelectedCategoryNode();
			importArtifacts(project);
		} else if (isOnlySingleArtifactFolderNodeSelected()) {
			Folder artifactFolder = selection.getSelectedFolder();
			importArtifacts(artifactFolder);
		}
	}
	

	
	private boolean isOnlySingleArtifactCategoryNodeSelected() {
		SelectionHelper selection = getSelectionHelper();
		if (selection.isOnlySingleCategoryNodeSelected()) {
			CategoryNode cat = (CategoryNode)selection.getFirstSelectedProjectViewNode();
			boolean isArtifactCategory = cat.getType() == FragmentType.ARTIFACT;
			return isArtifactCategory;
		}
		return false;
	}
	
	
	private Project getProjectOfSelectedCategoryNode() {
		SelectionHelper selection = getSelectionHelper();
		CategoryNode cat = (CategoryNode)selection.getFirstSelectedProjectViewNode();
		Project project = cat.getParent().getProject();
		return project;
	}

	
	private boolean isOnlySingleArtifactFolderNodeSelected() {
		SelectionHelper selection = getSelectionHelper();
		if (selection.isOnlySingleFolderNodeSelected()) {
			Folder folder = selection.getSelectedFolder();
			boolean isArtifactFolder = folder.getFragmentTypeAffiliation() == FragmentType.ARTIFACT;
			return isArtifactFolder;
		}
		return false;
	}
	

	
	
	

	
	
	public void importArtifacts(Project project) {
		importArtifacts(project, null);
	}
	

	public void importArtifacts(Folder folder) {
		Project project = folder.getProject();
		importArtifacts(project, folder);
	}
	
	
	private void importArtifacts(Project project, Folder folder) {
		List<OriginalFragmentLocation> userSelectedLocations = fragmentSourceGui.getLocationsForImport();
		FragmentSource fragmentSource = fragmentSourceGui.getFragmentSource();
		
		ExtensionService es = RepositoryFacade.instance().getExtensionService();
		ArtifactTypeResolver resolver = new UIArtifactTypeResolver(getWindow(), es);
		try {
			for (OriginalFragmentLocation location : userSelectedLocations) {
				ArtifactType type = resolver.getArtifactType(fragmentSource, location);
				String name = location.getName();
				int latestRevision = project.getLatestRevision();
				int newRevision = latestRevision + 1;
				
				RepositoryFacade.instance().importArtifact(project, folder, type.getTypeId(), 
						fragmentSource, location, name, newRevision);
			}
		} catch (DeftArtifactTypeNotFoundException e) {
			System.out.println(e.getMessage());
		}
		
	}
	
}
