package org.deft.actions;

import org.deft.gui.dialog.NewChapterDialog;
import org.deft.gui.view.project.nodes.CategoryNode;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Folder;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.FragmentType;
import org.deft.repository.datamodel.Project;
import org.deft.util.ImageHelper;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbenchWindow;

public class NewChapterAction extends AbstractNewFragmentAction {

	public NewChapterAction(IWorkbenchWindow window) {
		super(window);
		setText("Chapter");
		setImageDescriptor(ImageHelper.getDeftIconDescriptor("enabled/fragment-chapter.png"));
		setDisabledImageDescriptor(ImageHelper.getDeftIconDescriptor("disabled/fragment-chapter.png"));
	}
	
	@Override
	public void run() {
		super.run();
		Fragment createdFragment = getCreatedFragment();
		if (createdFragment != null) {
			Chapter chapter = (Chapter)createdFragment;
			IWorkbenchWindow window = getWindow();
			new OpenChapterAction(window).openChapter(chapter);
		}
	}
	
	
	/**
	 * If the tutorial category is selected, this action will be enabled.
	 */
	@Override
	protected void setEnabledStateForSelectedCategory() {
		CategoryNode cat = getSelectedCategoryNode();
		FragmentType fragType = cat.getType();
		boolean isChapterCategory = fragType == FragmentType.CHAPTER;
		setEnabled(isChapterCategory);
	}
	
	/**
	 * If a folder within the tutorial category is selected, this action
	 * will be enabled.
	 */
	@Override
	protected void setEnabledStateForSelectedFolder() {
		Folder folder = getSelectionHelper().getSelectedFolder();
		FragmentType fragType = folder.getFragmentTypeAffiliation();
		boolean isChapterCategory = fragType == FragmentType.CHAPTER;
		setEnabled(isChapterCategory);
	}


	@Override
	public Chapter createFragment(Project project, FragmentType fragType) {
		String title = "Please enter the name of the new chapter.";
		NewChapterDialog dialog = new NewChapterDialog(title, project, Display.getDefault().getActiveShell());

		if (okPressed(dialog)) {
			String chapterName = dialog.getChapterName();
			String chapterTypeId = dialog.getChapterType().getId();
			Chapter chapter = repository.createChapter(project, chapterName, chapterTypeId);
			return chapter;
		}
		return null;
	}
	
	
	public Chapter createFragment(Project project) {
		Chapter chapter = createFragment(project, FragmentType.CHAPTER);
		return chapter;
	}


	@Override
	public Chapter createFragment(Folder parentFolder) {
		Project project = parentFolder.getProject();
		String title = "Please enter the name of the new chapter.";
		NewChapterDialog dialog = new NewChapterDialog(title, project, Display.getDefault().getActiveShell());
		if (okPressed(dialog)) {
			String chapterName = dialog.getChapterName();
			String chapterTypeId = dialog.getChapterType().getId();
			Chapter chapter = repository.createChapter(parentFolder, chapterName, chapterTypeId);
			return chapter;
		}
		return null;
	}
	

	
	private boolean okPressed(NewChapterDialog dialog) {
		return dialog.open() == Dialog.OK;
	}

	
	
}
