package org.deft.actions;

import org.deft.gui.input.FragmentNameInputValidator;
import org.deft.repository.datamodel.Folder;
import org.deft.repository.datamodel.FragmentType;
import org.deft.repository.datamodel.Project;
import org.deft.util.ImageHelper;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbenchWindow;

public class NewFolderAction extends AbstractNewFragmentAction {

	public NewFolderAction(IWorkbenchWindow window) {
		super(window);
		setText("Folder");
		setImageDescriptor(ImageHelper.getDeftIconDescriptor("enabled/fragment-folder.png"));
		setDisabledImageDescriptor(ImageHelper.getDeftIconDescriptor("disabled/fragment-folder.png"));
	}

	/**
	 * If the category is selected, this action will be enabled. 
	 */
	@Override
	protected void setEnabledStateForSelectedCategory() {
		setEnabled(true);
	}


	/**
	 * If a folder is selected, this action will be enabled.
	 */
	@Override
	protected void setEnabledStateForSelectedFolder() {
		setEnabled(true);
	}

	@Override
	public Folder createFragment(Project project, FragmentType fragType) {
		FragmentNameInputValidator validator = new FragmentNameInputValidator(project, fragType);
		InputDialog dialog = openAndGetInputDialog(validator);
		if (okPressed(dialog)) {
			String text = dialog.getValue();
			Folder folder = repository.createFolder(project, fragType, text);
			return folder;
		}
		return null;
	}



	@Override
	public Folder createFragment(Folder parentFolder) {
		FragmentNameInputValidator validator = new FragmentNameInputValidator(parentFolder);
		InputDialog dialog = openAndGetInputDialog(validator);
		if (okPressed(dialog)) {
			String text = dialog.getValue();
			Folder folder = repository.createFolder(parentFolder, text);
			return folder;
		}
		return null;
	}
	
	private InputDialog openAndGetInputDialog(FragmentNameInputValidator validator) {

		String dialogText = "Please enter the name of the new folder.";
		String dialogTitle = "Enter name";

		InputDialog dialog = new InputDialog(Display.getDefault()
				.getActiveShell(), dialogTitle, dialogText, null, validator);
		return dialog;
	}
	
	private boolean okPressed(InputDialog dialog) {
		return dialog.open() == Dialog.OK;
	}
}
