package org.deft.actions;

import org.deft.RepositoryFacade;
import org.deft.gui.dialog.NewProjectDialog;
import org.deft.repository.exception.DeftFragmentAlreadyExistsException;
import org.deft.util.ImageHelper;
import org.eclipse.jface.action.Action;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;

/**
 * A GUI action for creating a new project. A dialog is opened, into which the project
 * name can be entered.
 * 
 * This action is managed internally via the DeftActionFactory.
 */
public class NewProjectAction extends Action implements IWorkbenchAction {

	private RepositoryFacade repository = RepositoryFacade.instance();
	
	public NewProjectAction() {
		setText("Project");
		setImageDescriptor(ImageHelper.getDeftIconDescriptor("enabled/fragment-project.png"));
		setDisabledImageDescriptor(ImageHelper.getDeftIconDescriptor("disabled/fragment-project.png"));
	}

	@Override
	public void run() {
		openInputDialog("Enter project name");
	}

	private void openInputDialog(String message) {
		NewProjectDialog dialog = new NewProjectDialog(message, Display.getDefault().getActiveShell());
		if (dialog.open() != NewProjectDialog.OK)
			return;

		try {
			String projectTypeId = dialog.getProjectType().getProjectTypeId();
			String projectName = dialog.getProjectName();
			repository.createNewProject(projectName, projectTypeId);
		} catch (DeftFragmentAlreadyExistsException e) {
			openInputDialog("Retype project name (" + e.getMessage() + ")");
		}
	}

	@Override
	public void dispose() {
		// do nothing
	}

}
