package org.deft.actions;

import org.deft.gui.input.FragmentNameInputValidator;
import org.deft.gui.view.project.nodes.CategoryNode;
import org.deft.repository.datamodel.Folder;
import org.deft.repository.datamodel.FragmentType;
import org.deft.repository.datamodel.Project;
import org.deft.repository.datamodel.Tutorial;
import org.deft.util.ImageHelper;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbenchWindow;

public class NewTutorialAction extends AbstractNewFragmentAction {

	public NewTutorialAction(IWorkbenchWindow window) {
		super(window);
		setText("Tutorial");
		setImageDescriptor(ImageHelper.getDeftIconDescriptor("enabled/fragment-tutorial.png"));
		setDisabledImageDescriptor(ImageHelper.getDeftIconDescriptor("disabled/fragment-tutorial.png"));
	}
	
	
	/**
	 * If the tutorial category is selected, this action will be enabled.
	 */
	@Override
	protected void setEnabledStateForSelectedCategory() {
		CategoryNode cat = getSelectedCategoryNode();
		FragmentType fragType = cat.getType();
		boolean isTutorialCategory = fragType == FragmentType.TUTORIAL;
		setEnabled(isTutorialCategory);
	}
	
	/**
	 * If a folder within the tutorial category is selected, this action
	 * will be enabled.
	 */
	@Override
	protected void setEnabledStateForSelectedFolder() {
		Folder folder = getSelectionHelper().getSelectedFolder();
		FragmentType fragType = folder.getFragmentTypeAffiliation();
		boolean isTutorialCategory = fragType == FragmentType.TUTORIAL;
		setEnabled(isTutorialCategory);
	}


	@Override
	public Tutorial createFragment(Project project, FragmentType fragType) {
		FragmentNameInputValidator validator = new FragmentNameInputValidator(project, fragType);
		InputDialog dialog = openAndGetInputDialog(validator);
		if (okPressed(dialog)) {
			String text = dialog.getValue();
			Tutorial tutorial = repository.createTutorial(project, text);
			return tutorial;
		}
		return null;
	}



	@Override
	public Tutorial createFragment(Folder parentFolder) {
		FragmentNameInputValidator validator = new FragmentNameInputValidator(parentFolder);
		InputDialog dialog = openAndGetInputDialog(validator);
		if (okPressed(dialog)) {
			String text = dialog.getValue();
			Tutorial tutorial = repository.createTutorial(parentFolder, text);
			return tutorial;
		}
		return null;
	}
	
	private InputDialog openAndGetInputDialog(FragmentNameInputValidator validator) {

		String dialogText = "Please enter the name of the new tutorial.";
		String dialogTitle = "Enter name";

		InputDialog dialog = new InputDialog(Display.getDefault()
				.getActiveShell(), dialogTitle, dialogText, null, validator);
		return dialog;
	}
	
	private boolean okPressed(InputDialog dialog) {
		return dialog.open() == Dialog.OK;
	}
}
