package org.deft.actions;

import org.deft.gui.SelectionHelper;
import org.eclipse.ui.IWorkbenchWindow;

/**
 * Combined Action for opening chapters and artifact references. 
 *
 * Opening chapters and opening artifact references are 2 different things.
 * However, the user should see the same context menu item, regardless of
 * whether he right-clicked a chapter or an artifact reference.
 * 
 * This action keeps an instance of OpenChapterAction and GoToReferencedArtifactAction,
 * and forwards calls to the correct action, depending on the selected node in 
 * the ProjectView.
 */
public class OpenAction extends AbstractDeftAction {

	private OpenChapterAction openChapterAction;
	private GoToReferencedRepresentationAction goToReferencedRepresentationAction;
	
	public OpenAction(IWorkbenchWindow window) {
		super(window);
		setText("Open");
		openChapterAction = new OpenChapterAction(window);
		goToReferencedRepresentationAction = new GoToReferencedRepresentationAction(window);
	}

	@Override
	protected void setEnabledState() {
		AbstractDeftAction action = getConcreteOpenAction();
		if (action != null) {
			action.setEnabledState();
			boolean enabled = action.isEnabled();
			this.setEnabled(enabled);
		} else {
			this.setEnabled(false);
		}
	}
	
	
	@Override
	public void run() {
		AbstractDeftAction action = getConcreteOpenAction();
		if (action != null) {
			action.run();
		}
	}
	
	private AbstractDeftAction getConcreteOpenAction() {
		
		if (selectionIsChapter()) {
			return openChapterAction;
		} else if (selectionIsReference()) {
			return goToReferencedRepresentationAction;
		}
		return null;
	}
	
	
	private boolean selectionIsChapter() {
		SelectionHelper selection = getSelectionHelper();
		boolean isChapter = selection.isOnlySingleChapterNodeSelected();
		return isChapter;
	}
	
	
	private boolean selectionIsReference() {
		boolean isReference = getSelectionHelper().isOnlySingleArtifactReferenceNodeSelected();
		return isReference;
	}

}
