package org.deft.actions;

import org.deft.DeftPlugin;
import org.deft.RepositoryFacade;
import org.deft.gui.dialog.SelectRepositoryDialog;
import org.deft.preferences.PreferenceConstants;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchWindow;


public class SwitchRepositoryAction extends AbstractDeftAction {

	private RepositoryFacade repository = RepositoryFacade.instance();

	public SwitchRepositoryAction(IWorkbenchWindow window) {
		super(window);
		setText("Switch Repository...");
	}

	@Override
	public void run() {
		IPath newPath = openDialogAndGetNewRepositoryPath();
		if (isValidPath(newPath)) {
			getEditorHelper().closeAllEditors();
			saveLocationInPreferenceStore(newPath);
			repository.getRepositoryOptions().setRepositoryLocation(newPath.toOSString());
		}
	}
	
	
	private IPath openDialogAndGetNewRepositoryPath() {
		Shell shell = Display.getDefault().getActiveShell();
		SelectRepositoryDialog dia = new SelectRepositoryDialog(shell);
		dia.create();
		dia.setTitle("Change repository?");
		dia.setMessage("If you change the repository all open files will be saved and closed.");
		if (dia.open() == Dialog.OK) {
			IPath path = dia.getDirectory();
			return path;
		}
		return null;
	}
	
	private boolean isValidPath(IPath path) {
		return path != null && path.toFile().isDirectory();
	}
	
	private void saveLocationInPreferenceStore(IPath path) {
		DeftPlugin.getInstance().getPreferenceStore().setValue(
				PreferenceConstants.REPOSITORY_PATH, path.toOSString());
	}
	
	
	@Override
	protected void setEnabledState() {
		setEnabled(true);
	}
}
