package org.deft.actions;

import java.util.List;

import org.deft.artifacttypesupport.codefile.AstTransformerRegistry;
import org.deft.artifacttypesupport.codefile.LanguageSpecifics;
import org.deft.artifacttypesupport.codefile.Parser;
import org.deft.artifacttypesupport.codefile.PlainCodeFileArtifactType;
import org.deft.artifacttypesupport.codefile.XPathGenerator;
import org.deft.artifacttypesupport.codefile.transform.OutlineNode;
import org.deft.artifacttypesupport.codefile.transform.OutlineNodeTransformer;
import org.deft.gui.SelectionHelper;
import org.deft.repository.ast.TreeNode;
import org.deft.repository.ast.jxpath.XPath;
import org.deft.repository.datamodel.Artifact;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.IWorkbenchWindow;

public class TestAction extends AbstractDeftAction {

	private TreeViewer viewer;
	
	public TestAction(IWorkbenchWindow window) {
		super(window);
		this.setText("AST");
	}


	@Override
	protected void setEnabledState() {
		SelectionHelper selection = getSelectionHelper();
		if (selection.isOnlySingleArtifactNodeSelected()) {
			Artifact artifact = selection.getSelectedArtifact();
			boolean isJava = artifact.getArtifactTypeId().equals("org.deft.repository.artifacttype.codefile.java");
			setEnabled(isJava);
		} else {
			setEnabled(false);
		}
	}
	
	
	@Override
	public void run() {
		SelectionHelper selection = getSelectionHelper();
		Artifact artifact = selection.getSelectedArtifact();
		String content = new String(artifact.getArtifactContent());
		PlainCodeFileArtifactType type = (PlainCodeFileArtifactType)artifact.getArtifactType();
		Parser parser = (Parser)type.getValidator();
		TreeNode ast = parser.getAst(content);
		
		LanguageSpecifics ls = type.getLanguageSpecifics();
		AstTransformerRegistry transReg = ls.getTransformerRegistry();
		OutlineNodeTransformer transformer = transReg.getSimpleOutlineTransformer();
		OutlineNode newRoot = transformer.transform(ast);

		XPathGenerator xpathGenerator = ls.getXPathGenerator();
		
		
		
	    final Display display = Display.getCurrent();
	    Shell shell = new Shell(display);

	    shell.setText("Widget");
	    shell.setBounds(10, 10, 600, 600);
	    
	    //SWT.MULTI for multiple selections in tree
	    Tree tree = new Tree(shell, SWT.MULTI | SWT.BORDER);
	    tree.setSize(450, 450);
		viewer = new TreeViewer(tree);
		viewer.setContentProvider(new AstContentProvider());
		viewer.setLabelProvider(new AstLabelProvider());
		viewer.setAutoExpandLevel(3);
		viewer.setInput(newRoot);
		
		Button button = new Button(shell, SWT.NONE);
		button.setText("XPaths");
		button.setBounds(480, 0, 50, 50);
		button.addSelectionListener(new PrintXPathsListener(xpathGenerator));
		
		shell.open();
	    while (!shell.isDisposed()) {
		   if (!display.readAndDispatch()) {
		      display.sleep();
		   }
		}
	}
	
	private class PrintXPathsListener implements org.eclipse.swt.events.SelectionListener {

		private XPathGenerator xpathGenerator;
		
		private PrintXPathsListener(XPathGenerator xpathGenerator) {
			this.xpathGenerator = xpathGenerator;
		}
		
		@Override
		public void widgetSelected(SelectionEvent e) {
			IStructuredSelection selection = (IStructuredSelection)viewer.getSelection();
			List<?> selectedObjects = selection.toList();
			for (Object o : selectedObjects) {
				OutlineNode outlineNode = (OutlineNode)o;
				//the xpathGenerator does some XPath evaluations, which only work
				//if the tree has a TreeNodeRoot. The Outline-tree doesn't have a
				//TreeNodeRoot, so we work on the underlying TreeNode-tree instead.
				TreeNode treeNode = outlineNode.getTreeNode();
				XPath xpath = xpathGenerator.generateXPath(treeNode);
				System.out.println(outlineNode + ": " + xpath);
			}
		}

		@Override
		public void widgetDefaultSelected(SelectionEvent e) {
		}

		
	}

}
