package org.deft.actions;

import org.deft.actions.contributionitem.DeftContributionItemFactory;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.IMenuCreator;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;

/**
 * Action for the dropdown menu with contains the menu items for the Update With 
 * actions
 */
public class UpdateWithDropDownAction extends Action implements IWorkbenchAction {

	private IWorkbenchWindow window;
	

	public UpdateWithDropDownAction(IWorkbenchWindow window) {
		this.window = window;
		this.setMenuCreator(menuCreator);
		this.setText("Update With");
		this.setEnabled(true);
	}
	
	/**
	 * Creates the menu manager for the drop-down.
	 */
	private IMenuCreator menuCreator = new IMenuCreator() {
		
		private MenuManager dropDownMenuMgr;
		
		private void createDropDownMenuMgr() {

			if (dropDownMenuMgr == null) {
				IContributionItem menu = DeftContributionItemFactory.UPDATE_WITH_MENU.create(window);
				dropDownMenuMgr = new MenuManager();
				dropDownMenuMgr.add(menu);
			}
		}
		
		public Menu getMenu(Menu parent) {
			createDropDownMenuMgr();
			Menu menu = new Menu(parent);
			IContributionItem[] items = dropDownMenuMgr.getItems();
			for (int i = 0; i < items.length; i++) {
				IContributionItem item = items[i];
				IContributionItem newItem = item;
				if (item instanceof ActionContributionItem) {
					newItem = new ActionContributionItem(
							((ActionContributionItem) item).getAction());
				}
				newItem.fill(menu, -1);
			}
			return menu;
		}
	
		public Menu getMenu(Control parent) {
			createDropDownMenuMgr();
			return dropDownMenuMgr.createContextMenu(parent);
		}
	
		public void dispose() {
			if (dropDownMenuMgr != null) {
				dropDownMenuMgr.dispose();
				dropDownMenuMgr = null;
			}
		}
	
	};

	

	public void dispose() {
		if (menuCreator != null) {
			menuCreator.dispose();
		}
		menuCreator = null;
		window = null;
	}
	
}
