package org.deft.actions.contributionitem;

import java.util.List;

import org.deft.ExtensionServiceExtension;
import org.deft.RepositoryFacade;
import org.deft.actions.ExportAction;
import org.deft.exporter.Exporter;
import org.deft.gui.SelectionHelper;
import org.deft.repository.datamodel.Chapter;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.ContributionItem;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IWorkbenchWindow;

/**
 * This ContributionItem shows a menu with possible Exporters for a chapter.
 * 
 * This ContributionItem is provided via the ExportDropDownAction.
 */
public class ExportMenu extends ContributionItem {

	private IWorkbenchWindow window;
	private ExtensionServiceExtension extService;
	private SelectionHelper selection;

	/**
	 * Creates a menu containing ExportActions
	 */
	public ExportMenu(IWorkbenchWindow window, String id) {
		super(id);
		this.window = window;
		this.selection = new SelectionHelper(window);
		this.extService = RepositoryFacade.instance().getExtensionService();
	}

	@Override
	public void fill(Menu menu, int index) {
		Chapter chapter = selection.getSelectedChapter();
		for (String id : extService.getExporterIds(chapter.getTypeId())) {
			Exporter exporter = extService.getExporter(id);
			List<WizardPage> wizardPages = extService.getWizardPages(id);
			addMenuForExporter(menu, exporter, wizardPages);
		}
	}

	private void addMenuForExporter(Menu menu, Exporter exporter, List<WizardPage> wizardPages) {
		ExportAction action = new ExportAction(window, exporter.getName(), exporter, wizardPages);
		IContributionItem item = new ActionContributionItem(action);
		item.fill(menu, -1);
	}

}
