package org.deft.actions.contributionitem;

import java.util.List;

import org.deft.ExtensionServiceExtension;
import org.deft.RepositoryFacade;
import org.deft.actions.InsertRepresentationAction;
import org.deft.gui.EditorHelper;
import org.deft.gui.SelectionHelper;
import org.deft.integrator.EditorIntegrator;
import org.deft.operation.OperationChain;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.Chapter;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.ContributionItem;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchWindow;


/**
 * This ContributionItem shows a menu with all Representations that can be added
 * to the chapter.
 * 
 * This ContributionItem is provided via the InsertDropDownAction.
 */
public class InsertMenu extends ContributionItem {

	private IWorkbenchWindow window;
	private ExtensionServiceExtension extService;
	private SelectionHelper selection;
	private EditorHelper editorHelper;
	
	/**
	 * Creates a new ImportMenu containing ImportArtifactActions
	 */
	public InsertMenu(IWorkbenchWindow window, String id) {
		super(id);
		this.window = window;
		this.selection = new SelectionHelper(window);
		this.editorHelper = new EditorHelper(window);
		this.extService = RepositoryFacade.instance().getExtensionService();
	}

	@Override
	public void fill(Menu menu, int index) {
		if (selection.isOnlySingleArtifactNodeSelected()) {
			String artifactTypeId = getTypeIdOfSelectedArtifact();
			String chapterTypeId = getTypeIdOfActiveChapterEditor();
			List<OperationChain> chains 
					= extService.getCompatibleOperationChainsForEditorIntegration(artifactTypeId, chapterTypeId);
			addMenuForEachChain(menu, chains, chapterTypeId);
		}
	}


	private String getTypeIdOfSelectedArtifact() {
		Artifact selectedArtifact = selection.getSelectedArtifact();
		return selectedArtifact.getArtifactTypeId();
	}
	
	private String getTypeIdOfActiveChapterEditor() {
		IEditorPart activeEditor = editorHelper.getActiveChapterEditor();
		if (activeEditor == null) {
			return null;
		}
		Chapter chapter = editorHelper.getChapterOfActiveEditor();
		String typeId = chapter.getTypeId();
		return typeId;
	}
	
	
	private void addMenuForEachChain(Menu menu, List<OperationChain> operationChains, String chapterTypeId) {
		for (OperationChain chain : operationChains) {
			List<EditorIntegrator> integrators = extService.getEditorIntegratorsForOperationChain(chain.getId());
			for (EditorIntegrator integrator : integrators) {
				if (integrator.getChapterTypeIds().contains(chapterTypeId)) {
					addMenuForChainAndIntegrator(menu, chain, integrator);
				}
			}			
		}
	}
	
	private void addMenuForChainAndIntegrator(Menu menu, OperationChain chain, EditorIntegrator integrator) {
		InsertRepresentationAction action = new InsertRepresentationAction(window, chain, integrator);
		IContributionItem item = new ActionContributionItem(action);
		item.fill(menu, -1);
	}


}
