package org.deft.actions.contributionitem;

import org.eclipse.jface.action.ContributionItem;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ContributionItemFactory;
import org.eclipse.ui.internal.ShowViewAction;
import org.eclipse.ui.views.IViewCategory;
import org.eclipse.ui.views.IViewDescriptor;

/**
 * A menu which lets the user select a DEFT view to add to the perspective. The
 * actions in this menu are special to DEFT, we don't use the eclipse menu
 * here, as it would show inappropriate views to the user. However at the end of
 * the menu it the 'Other...' entry taken from eclipse which will show all other
 * available views (e.g. from other plugins).
 * 
 * This is not optimal as we are using internal stuff here. However the standard
 * view menu of eclipse does not fit us very well.
 */
public class ViewMenu extends ContributionItem {

	private IWorkbenchWindow window;

	public ViewMenu(IWorkbenchWindow window, String id) {
		super(id);
		this.window = window;
	}

	@Override
	public void fill(Menu menu, int index) {

		MenuManager mm = new MenuManager("Open View");
		IViewCategory[] cats = PlatformUI.getWorkbench().getViewRegistry()
				.getCategories();
		for (IViewCategory cat : cats) {

			if (cat.getId().equals("org.deft")) {

				IViewDescriptor[] views = cat.getViews();
				for (IViewDescriptor view : views) {
					//using internals here...see class comment
					// We must subclass here because of a protected constructor
					IAction action = new ShowViewAction(window, view, false) {
						@Override
						public void run() {
							super.run();
						}
					};

					mm.add(action);
					System.out.println("ADDING");
				}
			}
		}
		// Add the "Other..." menu
		//commented out because it adds double menus (shows additional menu for open views)
//		mm.add(ContributionItemFactory.VIEWS_SHORTLIST.create(window));

		// Fill menu
		mm.fill(menu, -1);
	}

}
