package org.deft.artifacttypesupport.codefile;

import java.io.BufferedReader;
import java.io.CharArrayReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringReader;
import java.nio.CharBuffer;

import org.deft.repository.ast.TreeNodeRoot;
import org.deft.repository.exception.DeftParseException;

public abstract class AbstractParser implements Parser {

	@Override
	public TreeNodeRoot getAst(String code) throws DeftParseException {
		CharBuffer buffer = getContent(code);
		TreeNodeRoot root = getAst(buffer);
		return root;
	}
	
	
	@Override
	public TreeNodeRoot getAst(InputStream is) throws DeftParseException {
		CharBuffer buffer = getContent(is);
		TreeNodeRoot root = getAst(buffer);
		return root;
	}


	public TreeNodeRoot getAst(File file) throws DeftParseException {
		CharBuffer content = getContent(file);
		return getAst(content);
	}
	
	
	
	public CharBuffer getContent(InputStream is) {
		BufferedReader reader = new BufferedReader(new InputStreamReader(is));
		try {
			CharBuffer buffer = getCharBuffer(reader);
			return buffer;
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	
	
	public CharBuffer getContent(File file) {
		try
		{
			BufferedReader reader = new BufferedReader(new FileReader(file));
			return getCharBuffer(reader);
		}
		catch (IOException e) {
			e.printStackTrace();
		}
		
		return null;
	}
	
	public CharBuffer getContent(String string) {
		try {
			BufferedReader reader = new BufferedReader(new StringReader(string));
			return getCharBuffer(reader);
		}
		catch (IOException e) {
			e.printStackTrace();
		}
		
		return null;
		
	}
	
	
	private CharBuffer getCharBuffer(BufferedReader reader) throws IOException {
		CharBuffer cb = CharBuffer.allocate(1024*8);
		char[] buf = new char[1024];
		int length;
		int offset = 0;
		while ((length = reader.read(buf)) != -1) {
			cb.put(buf, 0, length);
			offset += length;
			if (offset == cb.capacity()) {
				CharBuffer tmp = CharBuffer.allocate(cb.capacity() * 2);
				cb.position(0);
				tmp.put(cb);
				cb = tmp;
			}
		}
		return CharBuffer.allocate(offset).put(cb.array(), 0, offset);
	}
	
	
	protected PositionConverter getPositionConverter(CharBuffer content) {
		PositionConverter pConv = null;
		BufferedReader reader = new BufferedReader(new CharArrayReader(content.array()));
		try {
			pConv = new PositionConverter(reader);
		} catch (IOException e) {
			throw new RuntimeException("Error while trying to read input stream.", e);
		}
		return pConv;
	}


}
