package org.deft.artifacttypesupport.codefile;

import org.deft.repository.ast.decoration.tokentype.TokenTypeDecorator;

/**
 * This interface is intended for code file artifacts which can be represented as TreeNodes. 
 * 
 * It offers a single access point to language specific details, details which cannot be 
 * encoded in the generic TreeNode / AST packages.
 * 
 * This includes:
 * - the specific TokenTypeDecorator which types the TokenNodes of the AST
 * - the XPathGenerator which computes for a given TreeNode a stable (resistant to AST changes) XPath
 * - a registry which in turn contains Transformers that transform the AST. Mainly used for the simplified tree representation
 *   in the OperationGui
 */
public interface LanguageSpecifics {
	
	public String getLanguageId();
	
	public TokenTypeDecorator getTokenTypeDecorator();
	
	public XPathGenerator getXPathGenerator();
	
	public AstTransformerRegistry getTransformerRegistry();
	
}
