package org.deft.artifacttypesupport.codefile.java.parser.antlradapter;

import java.util.StringTokenizer;

import javax.xml.parsers.DocumentBuilderFactory;

import org.antlr.runtime.Token;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class DOMTreeAdaptor  {
	
	private Document doc;
	private Mapping mapping;
	
	public DOMTreeAdaptor(Mapping mapping) {
		this.doc = makeDocument();
		this.mapping = mapping;
	}
	
	private Document makeDocument() {
		try {
			return DocumentBuilderFactory.newInstance().newDocumentBuilder()
					.newDocument();
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}	
	
	private void setTokenAttributes(Element element, Token startToken, Token endToken) {
		//do not use startToken here because we need to find out where the last token
		//ends to compute endcol and endline
		
		//the textContent should only be set for token nodes. A token node
		//has only 1 token (therefore test for startToken == endToken). Furthermore
		//a token node has no children (therefore test if there are no children yet)
		//
		//Example: Node NamespaceName has child node Identifier, which has a token "System".
		//When this method is called with the NamespaceName node and both tokens "System", the
		//NamespaceName node would be assigned the token System. As in DOM text content is a
		//child node, the "System" text content would replace the Identifier child.
		if (startToken == endToken && element.getChildNodes().getLength() == 0) {
			element.setTextContent(endToken.getText());
			element.setAttribute("token", "true");
		}
		
		int startLine = startToken.getLine();
		int startCol = startToken.getCharPositionInLine() + 1;		
		//count lines of last token
		String sLastLine = endToken.getText();
		StringTokenizer tokenizer = new StringTokenizer(sLastLine, "\n");
		
		 //init with -1 because the next loop is executed at least once
		//and endLine is increased again
		int endLine = endToken.getLine() - 1;
		while (tokenizer.hasMoreElements()) {
			sLastLine = tokenizer.nextToken();
			endLine++;
		}
		int endCol = 0;
		if (endLine == endToken.getLine()) {//no linebreaks in last token
			//endCol is startCol + text length
			endCol = endToken.getCharPositionInLine() + 1 + sLastLine.length();
		} else {//there were linebreaks in last token
			//endCol is length of last string segment (because after linebreak col
			//counting starts from pos 1)
			endCol = sLastLine.length() + 1;
		}
		int offset = mapping.convertPosition(startLine - 1, startCol - 1);
		int length = mapping.convertPosition(endLine - 1, endCol - 1) - offset;
		
		element.setAttribute("startline", Integer.toString(startLine));
		element.setAttribute("startcol", Integer.toString(startCol));
		element.setAttribute("endline", String.valueOf(endLine));
		element.setAttribute("endcol", String.valueOf(endCol));
		element.setAttribute("offset", String.valueOf(offset));
		element.setAttribute("length", String.valueOf(length));			
	}
	

	public void addChild(Object t, Object child) {
		Element eParent = (Element)t;
		Element eChild = (Element)child;
		
		if (eParent != null && eChild != null) {
			doAddChild(eParent, eChild);
		}		
	}
	
	@SuppressWarnings(value="unused")
	private String getTreeString(Element e) {
		StringBuilder sb = new StringBuilder();
		sb.append(e.getNodeName());
		NodeList nl = e.getChildNodes();
		if (nl.getLength() > 0) {
			sb.append(" ( ");
			for (int i = 0; i < nl.getLength(); i++) {
				Node n = nl.item(i);
				if (n.getNodeType() == Node.ELEMENT_NODE) {
					sb.append(getTreeString((Element)nl.item(i)));
				} else {
					sb.append("\"").append(n.getTextContent()).append("\"");
				}
				
			}
			sb.append(" ) ");
		}
		return sb.toString();
	}

	public Object becomeRoot(Object newRoot, Object oldRoot) {
		if (oldRoot == null) {
			return newRoot;
		}
		Element eOldRoot = (Element)oldRoot;
		Element eNewRoot = (Element)newRoot;
		if (isNil(eNewRoot)) {
			if (eNewRoot.getChildNodes().getLength() > 1 ) {
				throw new RuntimeException("more than one node as root");
			}
			eNewRoot = (Element)eNewRoot.getChildNodes().item(0);
		}		
		
//		if (isNil(eOldRoot)) {
//			/*NodeList nl = eOldRoot.getChildNodes();
//			for (int i = 0; i < nl.getLength(); i++) {
//				eNewRoot.appendChild(nl.item(i));
//			}*/
//			NodeList nl = eOldRoot.getChildNodes();
//			//elements are automatically removed from eOldRoot and nl 
//			//when added to new eNewRoot
//			while (nl.getLength() > 0) { 
//				eNewRoot.appendChild(nl.item(0));
//			}			
//		} else {
//			eNewRoot.appendChild(eOldRoot);
//		}
		doAddChild(eNewRoot, eOldRoot);
		return eNewRoot;
	}

	private void doAddChild(Element eParent, Element eChild) {
		if (isNil(eChild)) {
			NodeList nl = eChild.getChildNodes();
			while (nl.getLength() > 0) { 
				eParent.appendChild(nl.item(0));
			}				
		} else {
			eParent.appendChild(eChild);
		}
		
	}
	
	
	public Object becomeRoot(Token newRoot, Object oldRoot) {
		//Auto-generated method stub
		return null;
	}

	public Object create(Token payload) {
		if (payload == null) {
			Element eNil = doc.createElement("NIL");
			eNil.setAttribute("isnil", "true");
			return eNil;
		}
		String elemName = mapping.getTokenName(payload);
		Element e = doc.createElement(elemName);
		if (payload.getText() != null) {
			setTokenAttributes(e, payload, payload);
		}
		return e;
	}

	public Object create(int tokenType, Token fromToken) {
		//Auto-generated method stub
		return null;
	}

	public Object create(int tokenType, String text) {
		String elemName = mapping.getTokenName(text, tokenType);
		Element e = doc.createElement(elemName);
		return e;
	}

	public Object create(int tokenType, Token fromToken, String text) {
		//Auto-generated method stub
		return null;
	}

	public Object dupNode(Object treeNode) {
		//Auto-generated method stub
		return null;
	}

	public Object dupTree(Object tree) {
		//Auto-generated method stub
		return null;
	}

	public Object getChild(Object t, int i) {
		//Auto-generated method stub
		return null;
	}

	public int getChildCount(Object t) {
		//Auto-generated method stub
		return 0;
	}

	public String getText(Object t) {
		//Auto-generated method stub
		return null;
	}

	public Token getToken(Object t) {
		//Auto-generated method stub
		return null;
	}

	public int getTokenStartIndex(Object t) {
		//Auto-generated method stub
		return 0;
	}

	public int getTokenStopIndex(Object t) {
		//Auto-generated method stub
		return 0;
	}

	public int getType(Object t) {
		//Auto-generated method stub
		return 0;
	}

	public int getUniqueID(Object node) {
		//Auto-generated method stub
		return 0;
	}

	public boolean isNil(Object tree) {
		Element e = (Element)tree;
		return e.getAttribute("isnil").equals("true");
	}

	public Object nil() {
		return create(null);
	}

	public Object rulePostProcessing(Object root) {
		Element e = (Element)root;
		if ( e != null && isNil(e) && e.getChildNodes().getLength() == 1 ) {
			e = (Element) e.getChildNodes().item(0);
		}
		return e;
	}

	public void setText(Object t, String text) {
		//Auto-generated method stub
		
	}

	public void setTokenBoundaries(Object t, Token startToken, Token stopToken) {
		if (t == null) {
			return;
		}
		Element eTree = (Element)t;

		setTokenAttributes(eTree, startToken, stopToken);
	}
	

	public void setType(Object t, int type) {
	}

}
