package org.deft.artifacttypesupport.codefile.java.transformer;

import java.util.ArrayList;
import java.util.List;

import org.deft.artifacttypesupport.codefile.transform.IOutlineNodeFactory;
import org.deft.artifacttypesupport.codefile.transform.TransformerRule;
import org.deft.artifacttypesupport.codefile.transform.TransformerRuleSet;
import org.deft.repository.ast.jxpath.XPath;


public class JavaDetailedOutlineTransformerRuleSet implements TransformerRuleSet {


	private List<TransformerRule> includeRules;
	private IOutlineNodeFactory outlineNodeFactory;
	private String pluginID;
	private TransformerRule defaultRule;

	public JavaDetailedOutlineTransformerRuleSet() {
		this.outlineNodeFactory = new JavaDefaultNodeFactory();
		this.includeRules = new ArrayList<TransformerRule>();

		this.defaultRule = new TransformerRule("default", "createOutlineDefault", "icons/default_icon.gif");
		addIncludeRule("PackageDeclaration", "icons/package.gif");
		addIncludeRule("ImportDeclarations", "icons/importdeclarations.gif");
		addIncludeRule("ImportDeclaration", "icons/importdeclaration.gif");
		TransformerRule cd = addIncludeRule("ClassDeclaration", "icons/class_default.gif");
		cd.addIconChangeDirective(new XPath("Modifiers/PUBLIC"), "icons/class_public.gif");
		cd.addIconChangeDirective(new XPath("Modifiers/PROTECTED"), "icons/class_protected.gif");
		cd.addIconChangeDirective(new XPath("Modifiers/PRIVATE"), "icons/class_private.gif");
		TransformerRule id = addIncludeRule("InterfaceDeclaration", "icons/interface_default.gif");
		id.addIconChangeDirective(new XPath("Modifiers/PUBLIC"), "icons/interface_public.gif");
		id.addIconChangeDirective(new XPath("Modifiers/PROTECTED"), "icons/interface_protected.gif");
		id.addIconChangeDirective(new XPath("Modifiers/PRIVATE"), "icons/interface_private.gif");
		TransformerRule ed = addIncludeRule("EnumDeclaration", "icons/enum_default.gif");
		ed.addIconChangeDirective(new XPath("Modifiers/PUBLIC"), "icons/enum_public.gif");
		ed.addIconChangeDirective(new XPath("Modifiers/PROTECTED"), "icons/enum_protected.gif");
		ed.addIconChangeDirective(new XPath("Modifiers/PRIVATE"), "icons/enum_private.gif");
		TransformerRule ad = addIncludeRule("AnnotationTypeDeclaration", "icons/annotation_default.gif");
		ad.addIconChangeDirective(new XPath("Modifiers/PUBLIC"), "icons/annotation_public.gif");
		ad.addIconChangeDirective(new XPath("Modifiers/PROTECTED"), "icons/annotation_protected.gif");
		ad.addIconChangeDirective(new XPath("Modifiers/PRIVATE"), "icons/annotation_private.gif");
		addIncludeRule("AnnotationTypeMemberDeclaration", "");
		TransformerRule fd = addIncludeRule("FieldDeclaration", "icons/field_default.gif");
		fd.addIconChangeDirective(new XPath("Modifiers/PUBLIC"), "icons/field_public.gif");
		fd.addIconChangeDirective(new XPath("Modifiers/PROTECTED"), "icons/field_protected.gif");
		fd.addIconChangeDirective(new XPath("Modifiers/PRIVATE"), "icons/field_private.gif");
		addIncludeRule("EnumConstantDeclaration", "icons/field_public.gif");
		TransformerRule ctrd = addIncludeRule("ConstructorDeclaration", "icons/constructor_default.gif");
		ctrd.addIconChangeDirective(new XPath("Modifiers/PUBLIC"), "icons/constructor_public.gif");
		ctrd.addIconChangeDirective(new XPath("Modifiers/PROTECTED"), "icons/constructor_protected.gif");
		ctrd.addIconChangeDirective(new XPath("Modifiers/PRIVATE"), "icons/constructor_private.gif");
		TransformerRule md = addIncludeRule("MethodDeclaration", "icons/method_default.gif");
		md.addIconChangeDirective(new XPath("Modifiers/PUBLIC"), "icons/method_public.gif");
		md.addIconChangeDirective(new XPath("Modifiers/PROTECTED"), "icons/method_protected.gif");
		md.addIconChangeDirective(new XPath("Modifiers/PRIVATE"), "icons/method_private.gif");
		addIncludeRule("AssertStatement", "createStatement", "");
		addIncludeRule("BreakStatement", "createStatement", "");
		addIncludeRule("CaseStatement", "createStatement", "");
		addIncludeRule("ContinueStatement", "createStatement", "");
		addIncludeRule("DoStatement", "createStatement", "");
		addIncludeRule("EmptyStatement", "createStatement", "");
		addIncludeRule("EnhancedForStatement", "createStatement", "");
		addIncludeRule("ExpressionStatement", "createStatement", "");
		addIncludeRule("ForStatement", "createStatement", "");
		addIncludeRule("IfStatement", "createStatement", "");
		addIncludeRule("IfSection", "createIfSection", "");
		addIncludeRule("ElseSection", "createElseSection", "");
		addIncludeRule("LabeledStatement", "createStatement", "");
		addIncludeRule("ReturnStatement", "createStatement", "");
		addIncludeRule("SwitchStatement", "createSwitchStatement", "");
		addIncludeRule("SwitchSection", "createSwitchSection", "");
		addIncludeRule("SynchronizedStatement", "createSynchronizedStatement", "");
		addIncludeRule("ThrowStatement", "createStatement", "");
		addIncludeRule("TryStatement", "createTryStatement", "");
		addIncludeRule("TryBlock", "createTryBlock", "");
		addIncludeRule("CatchClause", "createCatchClause", "");
		addIncludeRule("FinallyClause", "createFinallyClause", "");
		addIncludeRule("ThrowStatement", "createStatement", "");
		addIncludeRule("TypeDeclarationStatement", "createStatement", "");
		addIncludeRule("VariableDeclarationStatement", "createStatement", "");
		addIncludeRule("WhileStatement", "createWhileStatement", "");
		addIncludeRule("SuperConstructorInvocation", "createStatement", "");
		addIncludeRule("Block", "createBlock", "");
	}

	
	private TransformerRule addIncludeRule(String name, String icon) {
		return addIncludeRule(name, "createNode", icon);
	}
	
	
	private TransformerRule addIncludeRule(String name, String callback, String icon) {
		if (icon == null || icon.isEmpty()) {
			TransformerRule rule = new TransformerRule(name, callback, "icons/default_icon.gif");
			includeRules.add(rule);
			return rule;
		} else {
			TransformerRule rule = new TransformerRule(name, callback, icon);
			includeRules.add(rule);
			return rule;
		}
	}


	
	public TransformerRule getRule(String elementName) {
		for (TransformerRule rule : includeRules) {
			if (elementName.matches(rule.getElementName())){
				return rule;
			}
		}
		return null;
	}
	
	public TransformerRule getDefaultRule() {
		return defaultRule;
	}
	
	/**
	 * Returns true if there is a matching rule in the rule set or false if not.
	 * 
	 * @param elementName
	 * @return
	 */
	public boolean containsRule(String elementName) {
		return getRule(elementName) != null;
	}

	/**
	 * Returns the id of the plugin that uses the transformer extension point.
	 * 
	 * @return
	 */
	public String getPluginID() {
		return pluginID;
	}

	/**
	 * Retunrs the transformer that is connected to the sset of rules.
	 * 
	 * @return
	 */
	public IOutlineNodeFactory getOutlineNodeFactory() {
		return outlineNodeFactory;
	}

}
