package org.deft.artifacttypesupport.codefile.java.xpath;

import java.util.LinkedList;
import java.util.List;

import org.deft.artifacttypesupport.codefile.XPathGenerator;
import org.deft.repository.ast.TreeNode;
import org.deft.repository.ast.TreeNodeRoot;
import org.deft.repository.ast.annotation.astname.AstNameInformation;
import org.deft.repository.ast.decoration.Templates;
import org.deft.repository.ast.jxpath.XPath;

public class JavaXPathGenerator implements XPathGenerator {

	@Override
	public XPath generateXPath(TreeNode node) {
		List<TreeNode> lTreeNode = getTreeNodeList(node);
		XPath xPath = createXPathFromTreeNodes(lTreeNode);
		return xPath;
	}
	
	
	
	private List<TreeNode> getTreeNodeList(TreeNode node) {
		List<TreeNode> lTreeNode = new LinkedList<TreeNode>();
		TreeNode parent = node;
		while (parent != null) {
			lTreeNode.add(0, parent);
			parent = (TreeNode)parent.getParent();
		}
		return lTreeNode;
	}
	
	
	private XPath createXPathFromTreeNodes(List<TreeNode> lTreeNode) {
		StringBuilder sb = new StringBuilder();
		for (TreeNode tn : lTreeNode) {
			sb.append("/").append(tn.getName());

			if (tn.hasInformation(Templates.ASTNAME)) {
				AstNameInformation ani = (AstNameInformation)tn.getInformation(Templates.ASTNAME);
				String sAstName = ani.getAstName();
				sb.append("[@astname=\'").append(sAstName).append("\']");
			} else {
				if (tn.getParent() == null || !(tn.getParent() instanceof TreeNodeRoot)) {
					//gets all siblings of the same name as tn (including tn)
					int position = getPosition(tn);
					sb.append("[position()=").append(position).append("]");
				}
			}

		}
		return new XPath(sb.toString());
	}

	
	private int getPosition(TreeNode node){
		TreeNode parent = node.getParent();
		if (parent == null) {
			return 1;		
		}
		List<TreeNode> list = parent.executeXPathQuery(node.getName());
		int pos = list.indexOf(node);
		return pos + 1;
	}	
}
