package org.deft.artifacttypesupport.codefile.transform;

import java.util.List;

import org.deft.repository.ast.Position;
import org.deft.repository.ast.TokenNode;
import org.deft.repository.ast.TreeNode;
import org.deft.repository.ast.annotation.astname.AstNameInformation;
import org.deft.repository.ast.decoration.Templates;


public class AbstractOutlineNodeFactory implements IOutlineNodeFactory {
	
	
	/*
	 * (non-Javadoc)
	 * @see org.deft.transform.IElementTransformer#getDefaultNode(org.w3c.dom.Element)
	 */
	public OutlineNode createNode(TreeNode node){	
		String name = node.getName();
		if (node.hasInformation(Templates.ASTNAME)) {
			name = ((AstNameInformation)node.getInformation(Templates.ASTNAME))
			.getAstName();
		}
		OutlineNode oNode = new OutlineNode(name, getPosition(node),node);
		return oNode;
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.deft.transform.IElementTransformer#getRootNode()
	 */
	public OutlineNode createRootNode() {
		return new OutlineNode();
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.deft.transform.IElementTransformer#getGroupingNode(java.lang.String, org.w3c.dom.Element)
	 */
	public OutlineNode createGroupingNode(String groupName, TreeNode node) {
		return new OutlineNode(groupName, getPosition(node));
	}
	
	
	/**
	 * Helper method to get a copy of the position of the element
	 * @param element
	 * @return
	 */
	public Position getPosition(TreeNode node) {
		Position p = node.getPosition();
		return new Position(p.getStartLine(), p.getStartCol(),
				p.getEndLine(), p.getEndCol(), p.getOffset(),
				p.getLength());
	}
	

	
	public String getTrimmedLabel(TreeNode node) {
		StringBuilder sb = new StringBuilder();
		List<TokenNode> list = node.serialize();
		for (int i = 0; i < list.size(); i++) {
			if (i != 0) {
				int offset = list.get(i).getOffset();
				int prevOffset = list.get(i - 1).getOffset();
				int prevLength = list.get(i - 1).getLength();
				int nrWs = offset - (prevOffset + prevLength);
				if (nrWs > 0) {
					sb.append(" ");
				}
			}
			sb.append(list.get(i).getToken().getText());
		}
		return sb.toString();
	}
	
	public boolean isTokenNode(TreeNode node) {
		return node instanceof TokenNode;
	}



	
}
