package org.deft.artifacttypesupport.codefile.transform;

import org.deft.repository.ast.Position;
import org.deft.repository.ast.TreeNode;

public class OutlineNode extends TreeNode {

	private String label;
	private Position position;
	private transient TreeNode treeNode;
	
	private String imagePath;
	private String pluginId;
	
	
	/**
	 * Creates a node that can be displayed in the outline view.
	 * @param label
	 * @param startLine - the line where the corresponding content starts
	 * @param startCol - the column where the corresponding content starts
	 * @param endLine - the line where the corresponding content ends
	 * @param endCol - the column where the corresponding content ends
	 */
	public OutlineNode(String label, int startLine, int startCol, 
			int endLine, int endCol, int offset, int length , TreeNode node){
		this(label,new Position(startLine,startCol,endLine,endCol,
				offset,length),node);
	}
	
	/**
	 * Creates a node that can be displayed in the outline view.
	 * @param label 
	 * @param position - the position of the corresponding content
	 */
	public OutlineNode(String label, Position position){
		this(label,position,null);
	}
	
	
	/**
	 * Creates a node that can be displayed in the outline view.
	 * @param label
	 * @param position - the position of the corresponding content
	 * @param treeNode - the xml node this outline node is created of
	 */
	public OutlineNode(String label, Position position, TreeNode treeNode){
		super(label);
		this.label = label;
		this.position = position;
		this.treeNode = treeNode;
	}
	
	/**
	 * Creates a node that can be displayed in the outline view. The position of the corresponding
	 * content is set to a default value. This is expected to use as a root node.
	 */
	public OutlineNode() {
		this("outline",new Position(),null);
	}
	
	public String getLabel() {
		return label;
	}
	
	public String getImagePath(){
		return this.imagePath;
	}
	public void setImagePath(String imagePath){
		this.imagePath = imagePath;
	}
	
	public String getPluginId(){
		return this.pluginId;
	}
	
	
	public void setPluginId(String pluginId){
		this.pluginId = pluginId;
	}
	
	
	/**
	 * Returns the position of the corresponding content.
	 */
	public Position getPosition(){
		return this.position;
	}
	
	/**
	 * Returns the tree node this outline node is connected to.
	 */
	public TreeNode getTreeNode(){
		return this.treeNode;
	}
	

	
	public boolean equals(OutlineNode node) {
		return this.label.equals(node.getLabel());
		
	}



}
