package org.deft.artifacttypesupport.codefile.transform;

import java.lang.reflect.Method;
import java.util.List;

import org.deft.repository.ast.TreeNode;
import org.deft.repository.ast.jxpath.XPath;


public class OutlineNodeCreator implements NodeCreator {

	private TransformerRuleSet ruleSet;
	private IOutlineNodeFactory factory;
	private String pluginIdOfOutlineNodeIcons;

	public OutlineNodeCreator(TransformerRuleSet ruleSet, String pluginIdOfOutlineNodeIcons) {
		this.ruleSet = ruleSet;
		this.factory = ruleSet.getOutlineNodeFactory();
		this.pluginIdOfOutlineNodeIcons = pluginIdOfOutlineNodeIcons;
	}



	public NodeList<TreeNode> createNode(TreeNode node) {
		NodeList<OutlineNode> result = null;
		 
		String elementName = node.getName();
		if (ruleSet.containsRule(elementName)) {
			TransformerRule rule = ruleSet.getRule(elementName);
			try {

				// get element node
				String callBack = rule.getCallBack();
				Object object = null;
				if (callBack == null || callBack.equals("getDefaultNode")) {
					// invoke default element callback
					object = factory.createNode(node);
				} else {
					// invoke user defined callback
					Method m = factory.getClass().getMethod(callBack,
							new Class[] { TreeNode.class });
					object = m.invoke(factory, new Object[] { node });
				}

				// if obj is null no new object node will be added
				if (object != null) {
					// it is not just possible to return one single
					// TreeNode, but also to return
					// a List of TreeNodes (That means there is a 1:m
					// relationship between xml node and tree node)
					if (object instanceof NodeList) {
						result = (NodeList) object;
					} else {
						result = new NodeList<OutlineNode>(
								(OutlineNode) object);
					}
				}
			} catch (Exception e1) {
				e1.printStackTrace();
			}
		}

		addAdditionalInfo(result, node);
		return (NodeList) result;
	}

	public TreeNode createRootNode() {
		return factory.createRootNode();
	}

	public TreeNode createGroupingNode(String groupName, TreeNode node) {

		//there was previously a guard here:
		//if (node instanceof Element) ...
		//cannot remember why
		OutlineNode on = factory.createGroupingNode(groupName,
				node);
		on.setPluginId(pluginIdOfOutlineNodeIcons);
		on.setImagePath(getGrouperIconPath(node));
		return on;

	}

	private void addAdditionalInfo(NodeList<OutlineNode> list, TreeNode node) {
		if (list != null) {
			for (TreeNode treeNode : list.getElements()) {
				OutlineNode on = (OutlineNode)treeNode;
				on.setPluginId(pluginIdOfOutlineNodeIcons);
				on.setImagePath(getIconPath(node));
			}
		}
	}

	private String getIconPath(TreeNode node) {
		String nodeName = node.getName();
		TransformerRule rule = ruleSet.getRule(nodeName);
		String iconPath = null;
		// if there is a rule for this node, get icon path
		if (rule != null) {
			// first get default icon for this node
			iconPath = rule.getDefaultIconPath();
			// but do also try the "modifier icons"
			for (XPath pathToModifier : rule.getIconChangeDirectivePaths()) {
				List<TreeNode> queryResult = node.executeXPathQuery(pathToModifier);
				if (!queryResult.isEmpty()) {
					iconPath = rule.getIconPathForChangeDirectivePath(pathToModifier);
					break;
				}
			}
		}
		// if there was no rule for this node, try to get icon
		// from default rule
		if (iconPath == null) {
			TransformerRule defaultRule = ruleSet.getDefaultRule();
			// if we have a default rule, use its icon
			if (defaultRule != null) {
				iconPath = defaultRule.getDefaultIconPath();
			}
			// otherwise we just have no icon
		}
		return iconPath;
	}

	// private String getIconPath(Node node) {
	// String nodeName = node.getNodeName();
	// String iconPath = null;
	// TransformerRule rule = ruleSet.getRule(nodeName);
	// if (rule != null){
	// iconPath = rule.getDefaultIconPath();
	// org.w3c.dom.NodeList list = node.getChildNodes();
	// for (int i = 0; i < list.getLength(); i++) {
	// Node n = list.item(i);
	// if (rule.containsModifier(n.getNodeName())) {
	// iconPath = rule.getModifierIconPath(n.getNodeName());
	// break;
	// }
	// }
	// }
	// if (iconPath == null) {
	// iconPath = ruleSet.getDefaultRule().getDefaultIconPath();
	// }
	// return iconPath;
	// }

	private String getGrouperIconPath(TreeNode node) {
		String nodeName = node.getName();
		String iconPath = null;
		TransformerRule rule = ruleSet.getRule(nodeName);
		if (rule != null) {
			iconPath = rule.getGroupingElementIconPath();
			if (iconPath == null) {
				iconPath = getIconPath(node);
			}
		}
		return iconPath;
	}


}
