package org.deft.artifacttypesupport.codefile.transform;

import java.util.LinkedList;
import java.util.List;

import org.deft.repository.ast.TreeNode;


public class RuleBasedOutlineNodeTransformer implements OutlineNodeTransformer {

	private TransformerRuleSet ruleSet;
	private String pluginIdOfOutlineNodeIcons;
	
	public RuleBasedOutlineNodeTransformer(TransformerRuleSet ruleSet, String pluginIdOfOutlineNodeIcons) {
		this.ruleSet = ruleSet;
		this.pluginIdOfOutlineNodeIcons = pluginIdOfOutlineNodeIcons;
	}
	
	public OutlineNode transform(TreeNode node) {
		return transform(node,false);
	}

	public OutlineNode transform(TreeNode node, boolean removeRootElement) {
		

		OutlineNodeFilter filter = new OutlineNodeFilter(ruleSet);
		OutlineNodeGrouper grouper = new OutlineNodeGrouper(ruleSet);
		OutlineNodeCreator creator = new OutlineNodeCreator(ruleSet, pluginIdOfOutlineNodeIcons);
		
		TreeConverter treeConverter = new TreeConverter();
		
		OutlineNode root = (OutlineNode) creator.createRootNode();
		
		//collect all nodes that should be processed
		List<TreeNode> lNodes = new LinkedList<TreeNode>();
		if (removeRootElement) {
			for (TreeNode n : node.getChildren()) {
				lNodes.add(n);
			}

		} else {
			lNodes.add(node);
		}
		
		//convert all nodes and group them under the root element
		for (TreeNode n : lNodes) {
			NodeList<TreeNode> treeNodes = treeConverter.convert(n, creator, filter, grouper);
			if (treeNodes != null){
				for (TreeNode treeNode : treeNodes.getElements()) {
					root.addChild(treeNode);
				}
			}
		}
		
		//return the root element
		return root;
	}

}
