package org.deft.artifacttypesupport.codefile.transform;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.deft.repository.ast.jxpath.XPath;

/**
 * Represents a rule for a rulebased transformer. 
 *
 */
public class TransformerRule {
	
	private String elementName;
	private String callback;
	private String groupingElement;
	private String groupingElementIconPath;
	private String defaultIconPath;
	private Map<XPath,String> iconChangeMap = new HashMap<XPath,String>();
	
	public TransformerRule(String callback, String iconPath){
		this.callback = callback;
		this.defaultIconPath = iconPath;
	}
	
	public TransformerRule(String name, String callback, String icon) {
		this.elementName = name;
		this.callback =  callback;
		this.defaultIconPath = icon;
	}
	

	/**
	 * Adds information that, if the evaluation of the (relative) XPath yields a result
	 * (i.e. the addressed element exists) the default icon should be modified and replaced
	 * by the new icon instead. 
	 */
	public void addIconChangeDirective(XPath xpath, String icon) {
		iconChangeMap.put(xpath, icon);
	}


	
	public Set<XPath> getIconChangeDirectivePaths() {
		return new HashSet<XPath>(iconChangeMap.keySet());
	}
		
	
	public String getIconPathForChangeDirectivePath(XPath changeDirectivePath){
		return iconChangeMap.get(changeDirectivePath);
	}
	
	public String getGroupingElementIconPath(){
		return groupingElementIconPath;
	}
	
	public String getCallBack() {
		return callback;
	}
	public String getDefaultIconPath() {
		return defaultIconPath;
	}
	public String getElementName() {
		return elementName;
	}
	public String getGroupingElement() {
		return groupingElement;
	}
	
	public String toString() {
		return "TransformerRule (" + elementName + ") callback: " + callback +
				"; defaultIcon: " + defaultIconPath;
			
	}
	
}
