package org.deft.artifactviewer;

import java.io.InputStream;

import org.deft.RepositoryFacade;
import org.deft.operation.OperationChain;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.repository.datamodel.Fragment;
import org.deft.representationrenderer.RepresentationRenderer;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IPersistableElement;


/**
 * An IEditorInput for ArtifactRepresentations.
 */
public class ArtifactViewerInputImpl implements ArtifactViewerInput {

	public static final String KEY = "ArtifactViewerInput";
	
	private Artifact artifact;
	private OperationChain chain;
	private RepresentationRenderer renderer;

	public ArtifactViewerInputImpl(Artifact artifact, OperationChain chain, RepresentationRenderer renderer) {
		this.artifact = artifact;
		this.chain = chain;
		this.renderer = renderer;
	}
	
	
	
	@Override
	public Artifact getArtifact() {
		return artifact;
	}
	
	@Override
	public RepresentationRenderer getRenderer() {
		return renderer;
	}

	@Override
	public OperationChain getChain() {
		return chain;
	}

	@Override
	public InputStream getInputStream() {
		ArtifactRepresentation rep = chain.executeOperations(artifact);
		InputStream is = renderer.getInputStream(rep);
		return is;
	}
	
	@Override
	public Object getContent() {
		ArtifactRepresentation rep = chain.executeOperations(artifact);
		Object renderedObject = renderer.render(rep);
		return renderedObject;
	}

	
	@Override
	public boolean exists() {
		String artifactId = artifact.getId();
		Fragment fragmentFromRepository = RepositoryFacade.instance().getFragment(artifactId);
		boolean isSameArtifact = artifact.equals(fragmentFromRepository);
		return isSameArtifact;
	}


	@Override
	public ImageDescriptor getImageDescriptor() {
		return ImageDescriptor.getMissingImageDescriptor();
	}

	public String getName() {
		return artifact.getName();
	}


	public String getToolTipText() {
		return artifact.getName();
	}


	/**
	 * Two FragmentEditorInputs are equal if the two containing Fragments are
	 * equal. The two Fragments are equal if they have the same UUID.
	 */
	@Override
	public boolean equals(Object o) {
		if (o instanceof ArtifactViewerInputImpl) {
			ArtifactViewerInputImpl other = (ArtifactViewerInputImpl) o;
			return artifact.equals(other.getArtifact());
		}
		return false;
	}

	@Override
	public void saveState(IMemento memento) {
		String mementoString = computeMementoString();
		memento.putString(KEY, mementoString);
	}
	
	private String computeMementoString() {
		String mementoString = artifact.getId() + "::" + chain.getId() + "::" + renderer.getId();
		return mementoString;
	}

	@Override
	public IPersistableElement getPersistable() {
		return this;
	}

	@Override
	public String getFactoryId() {
		return ArtifactViewerInputFactory.ID;
	}

	
	@SuppressWarnings("rawtypes")
	@Override
	public Object getAdapter(Class adapter) {
		return null;
	}


}
