package org.deft.artifactviewer;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;

public class EclipseArtifactViewerConnectorRegistry implements ArtifactViewerConnectorRegistry {
	
	private static final String EXTENSION_POINT_NAME = "org.deft.artifactviewer";
		
	private Map<String, ConnectorConfig> registeredConnectors = new HashMap<String, ConnectorConfig>(); 

	public EclipseArtifactViewerConnectorRegistry() {
		getAndCacheRegisteredViewers();
	}
	
	private void getAndCacheRegisteredViewers() {
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint point = registry.getExtensionPoint(EXTENSION_POINT_NAME);
		for (IConfigurationElement ce : point.getConfigurationElements()) {
			ConnectorConfig config = new ConnectorConfig(ce);
			registeredConnectors.put(config.getId(), config);
		}
	}
	
	@Override
	public ArtifactViewerConnector getConnector(String artifactViewerId) {
		ConnectorConfig config = registeredConnectors.get(artifactViewerId);
		if (config != null) {
			ArtifactViewerConnector connector = config.getConnector();
			return connector;
		}
		return null;
	}
	
	@Override
	public List<String> getEditorIds() {
		List<String> ids = new LinkedList<String>(registeredConnectors.keySet());
		return ids;
	}
	
	
	
	private class ConnectorConfig {
	
		private IConfigurationElement ce;
		private ArtifactViewerConnector connector;
		
		public ConnectorConfig(IConfigurationElement ce) {
			this.ce = ce;
		}
		
		public String getId() {
			String id = ce.getAttribute("id");
			return id;
		}
		
		public ArtifactViewerConnector getConnector() {
			if (connector == null) {
				try {
					connector = (ArtifactViewerConnector)ce.createExecutableExtension("class");
				} catch (CoreException e) {
					e.printStackTrace();
				}
			}
			return connector;
		}	
	}


}