package org.deft.artifactviewer;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.ui.IEditorPart;

public class EclipseEditorRegistry {
	
	private static final String EXTENSION_POINT_NAME = "org.eclipse.ui.editors";
		
	private Map<String, EditorConfig> registeredEditors = new HashMap<String, EditorConfig>(); 

	public EclipseEditorRegistry() {
		getAndCacheRegisteredEditors();
	}
	
	private void getAndCacheRegisteredEditors() {
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint point = registry.getExtensionPoint(EXTENSION_POINT_NAME);
		for (IConfigurationElement ce : point.getConfigurationElements()) {
			EditorConfig config = new EditorConfig(ce);
			registeredEditors.put(config.getId(), config);
		}
	}
	
	public IEditorPart getEditor(String editorId) {
		EditorConfig config = registeredEditors.get(editorId);
		if (config != null) {
			IEditorPart editor = config.getEditor();
			return editor;
		}
		return null;
	}
	
	public List<String> getEditorIds() {
		List<String> ids = new LinkedList<String>(registeredEditors.keySet());
		return ids;
	}
	
	
	
	private class EditorConfig {
	
		private IConfigurationElement ce;
		private IEditorPart editor;
		
		public EditorConfig(IConfigurationElement ce) {
			this.ce = ce;
		}
		
		public String getId() {
			String id = ce.getAttribute("id");
			return id;
		}
		
		public IEditorPart getEditor() {
			if (editor == null) {
				try {
					editor = (IEditorPart)ce.createExecutableExtension("class");
				} catch (CoreException e) {
					e.printStackTrace();
				}
			}
			return editor;
		}	
	}


}