package org.deft.chaptereditor;

import org.deft.RepositoryFacade;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Fragment;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.ui.IElementFactory;
import org.eclipse.ui.IMemento;

/**
 * Factory creates a ChapterEditorInput, so editors that have been open when DEFT was closed
 * can be restored when DEFT is started the next time.
 * 
 * This class must be registered at the org.eclipse.ui.elementFactories extension point.
 */
public class ChapterEditorInputFactory implements IElementFactory {

	//the id used in the extension point
	public static final String ID = "org.deft.chaptereditor.ChapterEditorInputFactory";

	
	/**
	 * This method tries to return a valid ChapterEditorInput. It looks into
	 * the given memento for a previously stored value. This value should be the
	 * id of a Chapter. If it finds one, it tries to retrieve this Chapter
	 * from the Repository. If the Chapter is still in the Repository a new
	 * ChapterEditorInput will be created out of it and returned. Otherwise null is returned.
	 */
	public IAdaptable createElement(IMemento memento) {

		String idString = memento.getString(ChapterEditorInputImpl.KEY);
		// Try to retrieve the fragment with this id
		if (isValidIdString(idString)) {
			Fragment fragment = RepositoryFacade.instance().getFragment(idString);
			if (isExistingChapter(fragment)) {
				Chapter chapter = (Chapter)fragment;
				return new ChapterEditorInputImpl(chapter);
			}
		}

		return null;
	}
	
	
	private boolean isValidIdString(String idString) {
		return idString != null && !idString.equals("");
	}
	
	private boolean isExistingChapter(Fragment fragment) {
		return fragment != null && fragment instanceof Chapter;
	}
	
}
