package org.deft.chaptereditor;

import org.deft.repository.datamodel.Chapter;
import org.deft.repository.workspaceresources.WorkspaceManager;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.part.FileEditorInput;

/**
 * An IEditorInput specialized on Chapters.
 */
public class ChapterEditorInputImpl extends FileEditorInput implements ChapterEditorInput {

	/**
	 * The key is used to save and restore the ChapterEditorInput to/from a
	 * memento, when DEFT is closed/started with open editors.
	 */
	public static final String KEY = "ChapterEditorInput";

	private Chapter chapter;
	private static WorkspaceManager workspaceManager = WorkspaceManager.instance();

	public ChapterEditorInputImpl(Chapter chapter) {
		super(workspaceManager.getEclipseChapterInWorkspace(chapter));
		this.chapter = chapter;
	}


	public String getName() {
		return chapter.getName();
	}


	public String getToolTipText() {
		return chapter.getName();
	}


	@Override
	public Chapter getChapter() {
		return chapter;
	}


	@Override
	public void saveState(IMemento memento) {
		memento.putString(KEY, chapter.getId());
	}


	@Override
	public String getFactoryId() {
		return ChapterEditorInputFactory.ID;
	}

}
