package org.deft.configuration;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;

import org.deft.configuration.OperationChainGui.ChainableGuiConfiguration;
import org.deft.integrator.EditorIntegrator;
import org.eclipse.jface.dialogs.TrayDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;

public class ConfigurationContainer extends TrayDialog {

	private TabFolder tabFolder;
	private List<TabItem> tabItems;
	private LinkedHashMap<String, ChainableGuiConfiguration> elements;
	private EditorIntegrator integrator;
	private SelectionListener tabSelectionListener;
	
	public ConfigurationContainer(Shell parentShell, 
			LinkedHashMap<String, ChainableGuiConfiguration> plugins, EditorIntegrator integrator) {
		super(parentShell);
		
		elements = plugins;
		tabItems = new ArrayList<TabItem>();
		this.integrator = integrator;
	}
	
	/**
	 * creates and configures all gui widgets 
	 */
	@Override
	public void create() {
		// create display and shell
		setShellStyle(getShellStyle() | SWT.RESIZE);
		super.create();
		super.getShell().setSize(500,500);
	}
	
	@Override
	protected Control createDialogArea(Composite parent) {
		// create tab folder
		tabFolder = new TabFolder(parent, SWT.BOTTOM);
		tabFolder.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		tabFolder.setLayout(new FillLayout());
		
		loadTabItems();
		
		if (tabSelectionListener != null) {
			tabFolder.addSelectionListener(tabSelectionListener);
		}
		return tabFolder;
	}
	
	public void addTabSelectionListener(SelectionListener listener) {
		this.tabSelectionListener = listener;
	}
	
	/**
	 * create a tab item for each registered transformation plugin
	 */
	private void loadTabItems() {
	
		for (String title : elements.keySet()) {
			TabItem ti = new TabItem(tabFolder, SWT.NONE);
			ti.setText(title);
			tabItems.add(ti);
			
			ChainableGuiConfiguration e = elements.get(title);
			ChainableGui plugin = e.getGui();
			Composite c = createGui(plugin, e);
			ti.setControl(c);
		}
	}

	
	private Composite createGui(ChainableGui plugin, ChainableGuiConfiguration config) {
		Composite c = plugin.createGuiComposite(tabFolder);
		if (plugin instanceof OperationGui) {
			((OperationGui)plugin).initializeGui(config.getOpConf(), config.getArtifactReference().getArtifact(), config.getArtifactRepr());
		} else if (plugin instanceof IntegratorGui) {
			((IntegratorGui)plugin).initializeGui(config.getOpConf(), integrator, config.getArtifactReference(), config.getArtifactRepr());
			return c;
		}
		return c;
	}
}
