package org.deft.configuration;

import java.util.HashMap;
import java.util.Map;

import org.deft.operation.Chainable;
import org.deft.operation.Operation;
import org.deft.repository.integrator.Integrator;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;

public class EclipseChainableGuiRegistry implements ChainableGuiRegistry {
	private static final String OPERATIONGUI_EXTENSION_POINT_NAME = "org.deft.operationgui";
	private Map<String, ChainableGuiExtension> extensions = null;

	public EclipseChainableGuiRegistry() {
		extensions = new HashMap<String, ChainableGuiExtension>();
		getAndCacheRegisteredSources();
	}
	
	private void getAndCacheRegisteredSources() {
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint point = registry.getExtensionPoint(OPERATIONGUI_EXTENSION_POINT_NAME);
		
		IExtension[] extensions = point.getExtensions();
		for (int i = 0; i < extensions.length; ++i) {
			IConfigurationElement[] configElements = extensions[i].getConfigurationElements();
			
			String chainableClassName = null;
			
			String chainableGuiName = null;
			IConfigurationElement chainableGui = null;
			
			for (IConfigurationElement ce : configElements) {
				if (ce.getName().equals("operation")) {
					chainableClassName = (String)ce.getAttribute("className");
				}
				
				if (ce.getName().equals("gui")) {
					chainableGuiName = ce.getAttribute("name");
					chainableGui = ce;
				}
			}
			
			this.extensions.put(chainableClassName,
					new ChainableGuiExtension(chainableGuiName, chainableGui));
		}
	}
	
	@Override
	public OperationGui getOperationGui(Operation operation) {
		ChainableGui gui = getChainableGui(operation);
		return (OperationGui)gui;
	}

	@Override
	public String getOperationGuiName(Operation operation) {
		String name = getChainableGuiName(operation);
		return name;
	}

	@Override
	public IntegratorGui getIntegratorGui(Integrator integrator) {
		ChainableGui gui = getChainableGui(integrator);
		return (IntegratorGui)gui;
	}

	@Override
	public String getIntegratorGuiName(Integrator integrator) {
		String name = getChainableGuiName(integrator);
		return name;
	}

	private ChainableGui getChainableGui(Chainable chainable) {
		ChainableGuiExtension guiExt = this.extensions.get(chainable.getClass().getSimpleName());
		if (guiExt != null) {
			return guiExt.getChainableGui();
		}
		return null;
	}
	
	
	private String getChainableGuiName(Chainable chainable) {
		ChainableGuiExtension opGuiExt = this.extensions.get(chainable.getClass().getSimpleName());
		if (opGuiExt != null) {
			return opGuiExt.getOperationGuiName();
		}
		return null;
	}
	
	private class ChainableGuiExtension {
		private String guiName = null;
		private IConfigurationElement chainableGui = null;
		
		public ChainableGuiExtension(String guiName, IConfigurationElement chainableGui) {
			this.guiName = guiName;
			this.chainableGui = chainableGui;
		}
		
		public String getOperationGuiName() {
			return guiName;
		}
		
		public ChainableGui getChainableGui() {
			ChainableGui cg = null;
			try {
				cg = (ChainableGui) chainableGui.createExecutableExtension("class");
			} catch (CoreException e) {
				e.printStackTrace();
			}
			return cg;
		}
	}


}
