package org.deft.configuration;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;

public class EclipseProjectTypeGuiRegistry implements ProjectTypeGuiRegistry {

	private static final String EXTENSION_POINT_NAME = "org.deft.projecttypegui";
	private Map<String, ProjectTypeGuiConfig> registeredConfigs = new HashMap<String, ProjectTypeGuiConfig>(); 

		
	public EclipseProjectTypeGuiRegistry() {
		loadProjectTypeGuiData();
	}
	
	private void loadProjectTypeGuiData() {
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint point = registry.getExtensionPoint(EXTENSION_POINT_NAME);
		if (point == null)
			return;
		
		for (IConfigurationElement ce : point.getConfigurationElements()) {
			ProjectTypeGuiConfig config = new ProjectTypeGuiConfig(ce);
			registeredConfigs.put(config.getId(), config);
		}
	}
	
	



	@Override
	public String getPerspectiveId(String projectTypeId) {
		ProjectTypeGuiConfig config = registeredConfigs.get(projectTypeId);
		String perspectiveId = config.getPerspectiveId();
		return perspectiveId;
	}

	@Override
	public String getChapterEditorId(String projectTypeId, String chapterTypeId) {
		ProjectTypeGuiConfig config = registeredConfigs.get(projectTypeId);
		String chapterEditorId = config.getChapterEditorId(chapterTypeId);
		return chapterEditorId;
	}

	
	private class ProjectTypeGuiConfig {
		
		private IConfigurationElement ce;
		private String perspectiveId;
		
		public ProjectTypeGuiConfig(IConfigurationElement ce) {
			this.ce = ce;
		}
		
		public String getId() {
			String id = ce.getAttribute("projecttype_id");
			return id;
		}
		
		
		public String getPerspectiveId() {
			if (perspectiveId == null && hasPerspective()) {
				perspectiveId = ce.getAttribute("perspective_id");
			}
			return perspectiveId;
		}	
		
		private boolean hasPerspective() {
			String perspectiveAttrib = ce.getAttribute("perspective_id");
			return perspectiveAttrib != null && !perspectiveAttrib.equals("");
		}
		
		
		public String getChapterEditorId(String chapterTypeId) {
			for (IConfigurationElement ceChild : ce.getChildren("chapterconfig")) {
				if (chapterTypeId.equals(ceChild.getAttribute("chaptertype_id"))) {
					String chapterEditorId = ceChild.getAttribute("chaptereditor_id");
					return chapterEditorId;
				}
			}
			return null;
		}
	}



}
