package org.deft.configuration;

import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;

import org.deft.ExtensionServiceExtension;
import org.deft.RepositoryFacade;
import org.deft.integrator.EditorIntegrator;
import org.deft.operation.Operation;
import org.deft.operation.OperationChain;
import org.deft.operation.OperationConfiguration;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.TabFolder;

public class OperationChainGui {

	private OperationChain operationChain = null;
	private ArtifactReference reference = null;
	private EditorIntegrator integrator = null;
	private ExtensionServiceExtension extensionService = RepositoryFacade.instance().getExtensionService();
	private List<ChainableGui> guis = new LinkedList<ChainableGui>();
	
	public OperationChainGui(OperationChain operationChain, EditorIntegrator integrator, ArtifactReference reference) {
		this.operationChain = operationChain;
		this.integrator = integrator;
		this.reference = reference;
	} 
	
	
	public boolean hasConfigurationTabs() {
		if (hasIntegratorConfigTab()) {
			return true;
		} else {
			int count = operationChain.getOperationCount();
			for (int i = 0; i < count; ++i) {
				Operation op = operationChain.getOperation(i);
				OperationGui operationGui = extensionService.getOperationGui(op);
				
				if (operationGui != null) {
					return true;
				}
			}
			return false;
		}
	}
	
	/**
	 * Returns for a certain GUI tab (i.e. operation gui) the index of the corresponding
	 * operation from the operation chain. Not all operations have guis, so there is no 1:1 mapping.
	 * 
	 * guiIndex counting starts at 0.
	 */
	private int getOperationIndexForGuiIndex(int guiIndex) {
		int count = operationChain.getOperationCount();
		int currentGuiCount = 0;
		for (int i = 0; i < count; ++i) {
			Operation op = operationChain.getOperation(i);
			OperationGui operationGui = extensionService.getOperationGui(op);
			if (operationGui != null) {//found a gui
				currentGuiCount++;
			}
			//is this the gui for which we search the operation?
			if (currentGuiCount - 1 == guiIndex) {
				return i;
			}
		}
		return -1;
	}
	
	private boolean hasIntegratorConfigTab() {
		if (integrator == null) {
			return false;
		} else {
			IntegratorGui gui = extensionService.getIntegratorGui(integrator);
			boolean hasConfigTab = gui != null;
			return hasConfigTab;
		}
	}
	
	
	/**
	 * Adds all operation gui classes for this operation chain
	 * to the configuration dialog 
	 */
	public int openConfigurationGui() {
		Artifact artifact = reference.getArtifact();
		//Use linked hash map because we have to preserve the order in which the content is added
		LinkedHashMap<String, ChainableGuiConfiguration> plugins = new LinkedHashMap<String, ChainableGuiConfiguration>();
		int count = operationChain.getOperationCount();
		for (int i = 0; i < count; ++i) {
			Operation op = operationChain.getOperation(i);
			OperationConfiguration opConf = operationChain.getChainConfigurations().getConfiguration(i);
			OperationGui operationGui = extensionService.getOperationGui(op);
			
			if (operationGui != null) {
				String name = extensionService.getOperationGuiName(op);
				ChainableGuiConfiguration opConfGui = new ChainableGuiConfiguration(operationGui, opConf, reference, operationChain.executeOperations(artifact, i));
				plugins.put(name, opConfGui);
				guis.add(operationGui);
			}
		}
		if (hasIntegratorConfigTab()) {
			IntegratorGui integratorGui = extensionService.getIntegratorGui(integrator);
			String name = extensionService.getIntegratorGuiName(integrator);
			OperationConfiguration integratorConf = integrator.getIntegratorConfiguration().getConfiguration();
			ChainableGuiConfiguration opConfGui = new ChainableGuiConfiguration(integratorGui, integratorConf, reference, operationChain.executeOperations(artifact));
			plugins.put(name, opConfGui);
		}
		
		ConfigurationContainer gui = new ConfigurationContainer(Display.getDefault().getActiveShell(), plugins, integrator);
		gui.addTabSelectionListener(new TabSelectionListener());
		int result = gui.open();
		return result;
	}
	
	public class ChainableGuiConfiguration {
		private ChainableGui gui = null;
		private OperationConfiguration opConf = null;
		private ArtifactRepresentation artifactRepr = null;
		private ArtifactReference reference = null;
		
		public ChainableGuiConfiguration(ChainableGui gui, OperationConfiguration opConf,
				ArtifactReference reference, ArtifactRepresentation artifactRepr) {
			this.gui = gui;
			this.opConf = opConf;
			this.reference = reference;
			this.artifactRepr = artifactRepr;
		}
		
		public ChainableGui getGui() {
			return gui;
		}

		public OperationConfiguration getOpConf() {
			return opConf;
		}

		public ArtifactReference getArtifactReference() {
			return reference;
		}
		
		public void setGui(ChainableGui gui) {
			this.gui = gui;
		}

		public void setOpConf(OperationConfiguration opConf) {
			this.opConf = opConf;
		}

		public void setArtifactReference(ArtifactReference reference) {
			this.reference = reference;
		}

		public ArtifactRepresentation getArtifactRepr() {
			return artifactRepr;
		}

		public void setArtifactRepr(ArtifactRepresentation artifactRepr) {
			this.artifactRepr = artifactRepr;
		}

	}
	
	private class TabSelectionListener implements SelectionListener {

		@Override
		public void widgetSelected(SelectionEvent e) {
			TabFolder tabFolder = (TabFolder)e.getSource();
			int index = tabFolder.getSelectionIndex();
			int operationIndex = getOperationIndexForGuiIndex(index);
			ArtifactRepresentation rep = operationChain.executeOperations(reference.getArtifact(), operationIndex);
			OperationConfiguration opConf = operationChain.getChainConfigurations().getConfiguration(operationIndex);
			OperationGui operationGui = (OperationGui)guis.get(index);
			operationGui.initializeGui(opConf, reference.getArtifact(), rep);
		}

		@Override
		public void widgetDefaultSelected(SelectionEvent e) {
		}
		
	}
}
