package org.deft.dependencyactivator;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import org.deft.dependencyactivator.activation.IActivationStartup;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.BundleEvent;

public class BundleListener implements org.osgi.framework.BundleListener {

	private Map<String, List<IConfigurationElement>> activationMap = new HashMap<String, List<IConfigurationElement>>();
	private List<String> alreadyStartedBundles = new ArrayList<String>();

	private static final Logger logger = Logger.getLogger(Activator.PLUGIN_ID);

	public BundleListener(BundleContext context) {
		super();
		loadExtensions();

		for (Bundle bundle : context.getBundles()) {
			if (bundle.getState() == Bundle.ACTIVE)
				handleStartedBundle(bundle.getSymbolicName());
		}
	}

	@Override
	public void bundleChanged(BundleEvent event) {
		if (event.getType() != BundleEvent.STARTED)
			return;
		String startedBundle = event.getBundle().getSymbolicName();
		handleStartedBundle(startedBundle);
	}

	private void handleStartedBundle(String bundle) {
		List<IConfigurationElement> bundlesToStart = activationMap.get(bundle);
		startBundles(bundlesToStart, bundle);
	}

	private void startBundles(List<IConfigurationElement> bundlesToStart,
			String dependencyBundle) {
		if (bundlesToStart == null)
			return;
		for (IConfigurationElement element : bundlesToStart) {
			try {
				String clazz = element.getAttribute("startup"); //$NON-NLS-1$
				if (alreadyStartedBundles.contains(clazz))
					continue;
				Object plugin = element.createExecutableExtension("startup"); //$NON-NLS-1$
				IActivationStartup startup = (IActivationStartup) plugin;
				alreadyStartedBundles.add(clazz);
				logger.info("activate bundle by '" + clazz //$NON-NLS-1$
						+ "' because of the activation of '" + dependencyBundle //$NON-NLS-1$
						+ "'"); //$NON-NLS-1$
				startup.startup();
			} catch (CoreException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	private void loadExtensions() {
		IExtensionRegistry extensionRegistry = Platform.getExtensionRegistry();
		IConfigurationElement[] configurationElements = extensionRegistry
				.getConfigurationElementsFor(IActivationStartup.POINT_ID);
		for (IConfigurationElement element : configurationElements) {
			String bundleID = element.getAttribute("bundle_id"); //$NON-NLS-1$
			List<IConfigurationElement> bundleStartups = activationMap
					.get(bundleID);
			if (bundleStartups == null) {
				bundleStartups = new ArrayList<IConfigurationElement>();
				activationMap.put(bundleID, bundleStartups);
			}
			bundleStartups.add(element);
		}
	}

}
