package org.deft.editorintegrator.latex;

import java.util.LinkedList;
import java.util.List;

import org.deft.integrator.AbstractEditorIntegrator;
import org.deft.integrator.EditorChapterDocument;
import org.deft.projecttype.texlipse.LatexEditorChapterDocument;
import org.deft.projecttype.texlipse.TexlipseArtifactReferencePosition;
import org.deft.repository.integrator.ArtifactReferencePosition;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.Position;

public abstract class AbstractLatexIntegrator extends AbstractEditorIntegrator {

	@Override
	public List<String> getChapterTypeIds() {
		List<String> ids = new LinkedList<String>();
		ids.add("latex");
		return ids;
	}
	
	
	protected void integrateDirective(LatexEditorChapterDocument latexChapterDoc, 
			Position position, String plainDirective, String artifactRefId) {
		String integratableDirective = makeIntegratableDirective(latexChapterDoc, plainDirective, artifactRefId);
		try {
			IDocument latexDoc = latexChapterDoc.getDocument();
			latexDoc.replace(position.getOffset(), 0, integratableDirective);
		} catch (BadLocationException ble) {
			ble.printStackTrace();
		}
	}

	protected String makeIntegratableDirective(LatexEditorChapterDocument latexChapterDoc, 
			String plainDirective, String artifactRefId) {
		String start = latexChapterDoc.getIntegrationStartComment(artifactRefId);
		String end = latexChapterDoc.getIntegrationEndComment(artifactRefId);
		String result = start + "\n" + plainDirective + "\n" + end + "\n";
		return result;
	}

	
	@Override
	public ArtifactReferencePosition undoIntegration(EditorChapterDocument document, String referenceId) {
		LatexEditorChapterDocument latexDoc = (LatexEditorChapterDocument)document;
		TexlipseArtifactReferencePosition artifactRefPos = getPositionForRepresentation(document, referenceId);
		if (artifactRefPos != null) {
			try {
				Position pos = artifactRefPos.getPosition();
				latexDoc.getDocument().replace(pos.getOffset(), pos.getLength(), "");
			} catch (BadLocationException e) {
				e.printStackTrace();
			}
			return artifactRefPos;
		}
		return null;
	}
	

	
	@Override
	public TexlipseArtifactReferencePosition getPositionForRepresentation(
			EditorChapterDocument chapterDocument, String artifactRefId) {
		LatexEditorChapterDocument latexChapterDoc = (LatexEditorChapterDocument)chapterDocument;
		String text = latexChapterDoc.getDocument().get();
		String start = latexChapterDoc.getIntegrationStartComment(artifactRefId);
		String end = latexChapterDoc.getIntegrationEndComment(artifactRefId);
		int startOffset = text.indexOf(start);	//where the start string begins
		int beginningOfEndOffset = text.indexOf(end); //where the end string begins
		if (startOffset != -1 && beginningOfEndOffset != -1) {
			int endOffset = beginningOfEndOffset + end.length(); //where the end string ends
			int integratedTextLength = endOffset - startOffset + 1;
			Position pos = new Position(startOffset, integratedTextLength);
			TexlipseArtifactReferencePosition artRefPos = new TexlipseArtifactReferencePosition(latexChapterDoc, pos);
			return artRefPos;
		} else {
			return null;
		}
	}

	
}
