package org.deft.editorintegrator.latex;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;

import org.deft.RepositoryFacade;
import org.deft.editorintegrator.latex.directivebuilder.IncludeGraphicsDirectiveBuilder;
import org.deft.operation.OperationConfiguration;
import org.deft.projecttype.texlipse.LatexEditorChapterDocument;
import org.deft.projecttype.texlipse.TexlipseArtifactReferencePosition;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Project;
import org.deft.repository.integrator.ArtifactReferencePosition;
import org.deft.representation.image.BufferedImageRepresentation;
import org.eclipse.jface.text.Position;



public class ImageIntoLatexIntegrator extends AbstractLatexIntegrator {

	public static final String ID = "org.deft.editorintegrator.image4latex";
	private RepositoryFacade repository = RepositoryFacade.instance();
	
	
	
	@Override
	public void integrate(String referenceId, ArtifactRepresentation representation, 
			ArtifactType artifactType, ArtifactReferencePosition position) {
		BufferedImageRepresentation imgRep = (BufferedImageRepresentation)representation;
		TexlipseArtifactReferencePosition texPosition = (TexlipseArtifactReferencePosition)position;
		LatexEditorChapterDocument doc = texPosition.getChapterDocument();
		insertLatexImageDirective(doc, texPosition.getPosition(), imgRep, referenceId);	
		saveImageInTempFile(imgRep, referenceId);
	}
	
	

	private void insertLatexImageDirective(LatexEditorChapterDocument latexChapterDoc, 
			Position position, BufferedImageRepresentation imgRep, String artifactRefId) {
		ArtifactReference reference = repository.getArtifactReference(artifactRefId);
		String latexImageDirective = getLatexImageDirective(reference);
		integrateDirective(latexChapterDoc, position, latexImageDirective, artifactRefId);
	}
	
	public String getLatexImageDirective(ArtifactReference reference) {
		Artifact artifact = reference.getArtifact();
		Project project = artifact.getProject();
		File tempDir = project.getTempDirectory();
		ImageIntoLatexIntegratorConfiguration config = ((ImageIntoLatexIntegratorConfiguration)this.getIntegratorConfiguration().getConfiguration());
		IncludeGraphicsDirectiveBuilder builder = config.getDirectiveBuilder();
		String latexImageDirective = builder.buildDirective(reference, tempDir);
		return latexImageDirective;
	}

	
	private void saveImageInTempFile(BufferedImageRepresentation imgRep, String artifactRefId) {
		BufferedImage image = imgRep.getImage();
		File imgFile = computeImageForTempFile(artifactRefId);
		try {
			ImageIO.write(image, "png", imgFile);
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
	}
	
	
	private File computeImageForTempFile(String artifactRefId) {
		ArtifactReference ref = repository.getArtifactReference(artifactRefId);
		Chapter chapter = ref.getChapter();
		Project project = chapter.getProject();
		File tempDir = RepositoryFacade.instance().getTempDirectory(project);
		File imgFile = new File(tempDir, artifactRefId + ".png");
		return imgFile;
	}



	@Override
	public String getId() {
		return ID;
	}

	@Override
	public String getInputRepresentationType() {
		return BufferedImageRepresentation.TYPE;
	}

	@Override
	public OperationConfiguration createConfiguration() {
		return new ImageIntoLatexIntegratorConfiguration(); 
	}


	@Override
	public String getDescription() {
		return "Image";
	}

	
	@Override
	public void cleanup(String referenceId) {
		File imgFile = computeImageForTempFile(referenceId);
		if (imgFile.exists()) {
			boolean success = imgFile.delete();
			if (!success) {
				imgFile.deleteOnExit();
			}
		}
	}
	
}
