package org.deft.editorintegrator.latex;

import org.deft.editorintegrator.latex.directivebuilder.IncludeGraphicsDirectiveBuilder;
import org.deft.operation.AbstractOperationConfiguration;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class ImageIntoLatexIntegratorConfiguration extends AbstractOperationConfiguration {

	public static String ID = "org.deft.editorintegrator.image4latex";
	
	private IncludeGraphicsDirectiveBuilder builder = new IncludeGraphicsDirectiveBuilder();
	
	public ImageIntoLatexIntegratorConfiguration() {
		super("org.deft.editorintegrator.latex");
	}
	
	
	@Override
	public String getId() {
		return ID;
	}


	@Override
	public void loadFromXml(Element element) {
		if (isValid(element)) {
			loadFieldsFromXml(element);
		} else {
			System.err.println("ImageIntoLatexIntegratorConfiguration not valid");
		}
	}
	
	private void loadFieldsFromXml(Element element) {
		String options = getTextContent(element, "options");
		boolean usefigure = getBooleanValue(element, "usefigure");
		boolean force = getBooleanValue(element, "force");
		boolean here = getBooleanValue(element, "here");
		boolean top = getBooleanValue(element, "top");
		boolean bottom = getBooleanValue(element, "bottom");
		boolean page = getBooleanValue(element, "page");
		boolean centering = getBooleanValue(element, "centering");
		String caption = getTextContent(element, "caption");
		String label = getTextContent(element, "label");
		
		builder.setOptions(options);
		builder.setUsefigure(usefigure);
		builder.setForce(force);
		builder.setHere(here);
		builder.setTop(top);
		builder.setBottom(bottom);
		builder.setPage(page);
		builder.setCentering(centering);
		builder.setCaption(caption);
		builder.setLabel(label);
	}
	
	private String getTextContent(Element parent, String elementName) {
		Element e = (Element)parent.getElementsByTagName(elementName).item(0);
		return e.getTextContent();
	}
	
	private boolean getBooleanValue(Element parent, String elementName) {
		Element e = (Element)parent.getElementsByTagName(elementName).item(0);
		String sValue = e.getAttribute("value");
		boolean value = Boolean.parseBoolean(sValue);
		return value;
	}



	
	
	@Override
	public void addToXml(Element root) {
		addFieldsToXml(root);
	}
	
	private void addFieldsToXml(Element root) {
		Document doc = root.getOwnerDocument();
		Element eOptions = doc.createElement("options");
		eOptions.setTextContent(builder.getOptions());
		root.appendChild(eOptions);
		
		Element eUseFigure = doc.createElement("usefigure");
		eUseFigure.setAttribute("value", Boolean.toString(builder.isUsefigure()));
		root.appendChild(eUseFigure);
		Element eForce = doc.createElement("force");
		eForce.setAttribute("value", Boolean.toString(builder.isForce()));
		root.appendChild(eForce);
		Element eHere = doc.createElement("here");
		eHere.setAttribute("value", Boolean.toString(builder.isHere()));
		root.appendChild(eHere);
		Element eTop = doc.createElement("top");
		eTop.setAttribute("value", Boolean.toString(builder.isTop()));
		root.appendChild(eTop);
		Element eBottom = doc.createElement("bottom");
		eBottom.setAttribute("value", Boolean.toString(builder.isBottom()));
		root.appendChild(eBottom);
		Element ePage = doc.createElement("page");
		ePage.setAttribute("value", Boolean.toString(builder.isPage()));
		root.appendChild(ePage);
		
		Element eCentering = doc.createElement("centering");
		eCentering.setAttribute("value", Boolean.toString(builder.isCentering()));
		root.appendChild(eCentering);
		Element eCaption = doc.createElement("caption");
		eCaption.setTextContent(builder.getCaption());
		root.appendChild(eCaption);
		Element eLabel = doc.createElement("label");
		eLabel.setTextContent(builder.getLabel());
		root.appendChild(eLabel);
	}
	
	
	@Override
	protected String getLocalSchemaFileLocation() {
		return "resources/schema.xsd";
	}

	
	public IncludeGraphicsDirectiveBuilder getDirectiveBuilder() {
		return builder;
	}

}
