package org.deft.editorintegrator.latex;

import java.util.LinkedList;
import java.util.List;

import org.deft.integrator.EditorChapterDocument;
import org.deft.operation.EmptyOperationConfiguration;
import org.deft.operation.OperationConfiguration;
import org.deft.projecttype.texlipse.LatexEditorChapterDocument;
import org.deft.projecttype.texlipse.TexlipseArtifactReferencePosition;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.repository.integrator.ArtifactReferencePosition;
import org.deft.representation.styleddocument.StyledCodeRepresentation;
import org.deft.representation.styleddocument.data.TokenizedStyledDocument;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.Position;

public class StyledCodeIntoLatexIntegrator extends AbstractLatexIntegrator {

	public static final String ID = "org.deft.editorintegrator.styledcode4latex";
	
	@Override
	public void integrate(String artifactRefId, ArtifactRepresentation representation,
			ArtifactType artifactType, ArtifactReferencePosition position) {
		StyledCodeRepresentation codeRep = (StyledCodeRepresentation)representation;
		TexlipseArtifactReferencePosition texPosition = (TexlipseArtifactReferencePosition)position;
		LatexEditorChapterDocument doc = texPosition.getChapterDocument();
		replaceReferenceWithCodeListing(doc, texPosition.getPosition(), codeRep, artifactRefId);
	}
	
	
	private void replaceReferenceWithCodeListing(LatexEditorChapterDocument doc, 
			Position position, StyledCodeRepresentation codeRep, String artifactRefId) {
		IDocument codeDoc = doc.getDocument();
		TokenizedStyledDocument text = codeRep.getStyledDocument();
		try {
			String code = text.getText(0, text.getLength());
			String codeAsLatex = makeIntegratableDirective(doc, code, artifactRefId);
			codeDoc.replace(position.getOffset(), position.getLength(), codeAsLatex);
		} catch (javax.swing.text.BadLocationException ble) {
			ble.printStackTrace();
		} catch (org.eclipse.jface.text.BadLocationException ble) {
			ble.printStackTrace();
		}
	}


	
	@Override
	public List<String> getChapterTypeIds() {
		List<String> ids = new LinkedList<String>();
		ids.add("latex");
		return ids;
	}

	@Override
	public String getId() {
		return ID;
	}

	@Override
	public String getInputRepresentationType() {
		return StyledCodeRepresentation.TYPE;
	}

	@Override
	public OperationConfiguration createConfiguration() {
		return new EmptyOperationConfiguration();
	}


//	@Override
//	public ArtifactReferencePosition undoIntegration(EditorChapterDocument document, String referenceId) {
////		LatexEditorChapterDocument latexDoc = (LatexEditorChapterDocument)document;
////		Position pos = findIntegratedRepresentation(latexDoc, referenceId);
////		if (pos != null) {
////			String refTag = (String)document.createArtifactReferenceTag(referenceId);
////			try {
////				latexDoc.getDocument().replace(pos.getOffset(), pos.getLength(), refTag);
////			} catch (BadLocationException e) {
////				e.printStackTrace();
////			}
////			return 1;
////		} else {
////			return 0;
////		}
//		return null;
//	}
	
	
//	private Position findIntegratedRepresentation(LatexEditorChapterDocument latexDoc, String artifactRefId) {
//		String text = latexDoc.getDocument().get();
//		String start = latexDoc.getIntegrationStartComment(artifactRefId);
//		String end = latexDoc.getIntegrationEndComment(artifactRefId);
//		int startOffset = text.indexOf(start);	//where the start string begins
//		int beginningOfEndOffset = text.indexOf(end); //where the end string begins
//		if (startOffset != -1 && beginningOfEndOffset != -1) {
//			int endOffset = beginningOfEndOffset + end.length(); //where the end string ends
//			int integratedTextLength = endOffset - startOffset + 1;
//			Position pos = new Position(startOffset, integratedTextLength);
//			return pos;
//		} else {
//			return null;
//		}
//	}
	

	@Override
	public String getDescription() {
		return "Code";
	}

	@Override
	public TexlipseArtifactReferencePosition getPositionForRepresentation(
			EditorChapterDocument chapterDocument, String referenceId) {
		return null;
	}


}
