package org.deft.editorintegrator.latex.directivebuilder;

import java.io.File;

import org.deft.repository.datamodel.ArtifactReference;

public class IncludeGraphicsDirectiveBuilder {



	private boolean usefigure;
	private boolean force, here, top, bottom, page;
	private boolean centering;
	private String caption = "", label = "";
	private String options = "";
	
	
	public void setUsefigure(boolean usefigure) {
		this.usefigure = usefigure;
	}

	public void setForce(boolean force) {
		this.force = force;
	}

	public void setHere(boolean here) {
		this.here = here;
	}

	public void setTop(boolean top) {
		this.top = top;
	}

	public void setBottom(boolean bottom) {
		this.bottom = bottom;
	}

	public void setPage(boolean page) {
		this.page = page;
	}

	public void setCentering(boolean centering) {
		this.centering = centering;
	}

	public void setCaption(String caption) {
		this.caption = caption;
	}
	
	public void setLabel(String label) {
		this.label = label;
	}
	
	public void setOptions(String options) {
		this.options = options;
	}
	
	public boolean isUsefigure() {
		return usefigure;
	}

	public boolean isForce() {
		return force;
	}

	public boolean isHere() {
		return here;
	}

	public boolean isTop() {
		return top;
	}

	public boolean isBottom() {
		return bottom;
	}

	public boolean isPage() {
		return page;
	}

	public boolean isCentering() {
		return centering;
	}

	public String getCaption() {
		return caption;
	}

	public String getLabel() {
		return label;
	}

	public String getOptions() {
		return options;
	}



	public String buildDirective(ArtifactReference reference, File tempDir) {
		StringBuilder sb = new StringBuilder();
		if (usefigure) {
			sb.append("\\begin{figure}");
			sb.append(getPositionString());
			sb.append("\n");
			sb.append(getCenteringDirectiveWithLinebreak());
			sb.append(getIncludeGraphicsDirective(reference, tempDir));
			sb.append("\n");
			sb.append(getCaptionDirectiveWithLinebreak());
			sb.append(getLabelDirectiveWithLinebreak());
			sb.append("\\end{figure}");
		} else {
			sb.append(getIncludeGraphicsDirective(reference, tempDir));
		}
		
		return sb.toString();
	}
	
	
	private String getPositionString() {
		if (hasPositionInformation()) {
			StringBuilder sb = new StringBuilder();
			sb.append("[");
			sb.append(force ? "!" : "");
			sb.append(here ? "h" : "");
			sb.append(top ? "t" : "");
			sb.append(bottom ? "b" : "");
			sb.append(page ? "p" : "");
			sb.append("]");
			return sb.toString();
		} 
		return "";
	}
	
	private boolean hasPositionInformation() {
		return here || top || bottom || page;
	}
	
	
	
	private String getIncludeGraphicsDirective(ArtifactReference reference, File tempDir) {
		String tempDirName = tempDir.getName(); //only the name, not the whole path
		String pathToImageFile = tempDirName + "/" + reference.getId();
		String directive = "\\includegraphics" + getIncludeGraphicsOptionsDirective() + "{" + pathToImageFile + "}";
		return directive;
	}
	
	private String getIncludeGraphicsOptionsDirective() {
		if (options.isEmpty()) {
			return "";
		} else {
			return "[" + options + "]";
		}
	}
	
	private String getCaptionDirectiveWithLinebreak() {
		if (caption.isEmpty()) {
			return "";
		} else {
			return "\\caption{" + caption + "}\n";
		}
	}
	
	
	private String getLabelDirectiveWithLinebreak() {
		if (label.isEmpty()) {
			return "";
		} else {
			return "\\label{" + label + "}\n";
		}
	}
	
	private String getCenteringDirectiveWithLinebreak() {
		return centering ? "\\centering\n" : "";
	}
	
	
	
	
}
