package org.deft.editorintegrator.latex.gui;


import java.io.File;

import org.deft.configuration.IntegratorGui;
import org.deft.editorintegrator.latex.ImageIntoLatexIntegratorConfiguration;
import org.deft.operation.OperationConfiguration;
import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.repository.integrator.Integrator;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.TextViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.TraverseEvent;
import org.eclipse.swt.events.TraverseListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;

/**
 * GUI to control how and image is inserted into a LaTeX document. 
 */
public class ImageIntoLatexIntegratorGui implements IntegratorGui {

	ITextViewer textViewer;
	Button bUseFigure, bTop, bBottom, bHere, bPage, bForce, bCentering;
	ITextViewer tCaption, tLabel;
	ITextViewer tOptions;
	

	@Override
	public Composite createGuiComposite(Composite parent) {
		Composite gui = makeMainGui(parent);
		return gui;
	}
	
	@Override
	public void initializeGui(OperationConfiguration config,
			Integrator integrator, ArtifactReference reference, ArtifactRepresentation artifactRepr) {
		ImageIntoLatexIntegratorConfiguration image4LatexConfig = (ImageIntoLatexIntegratorConfiguration)config;
		File tempDir = reference.getArtifact().getProject().getTempDirectory();
		ImageIntoLatexIntegratorGuiController controller = new ImageIntoLatexIntegratorGuiController(this, reference, tempDir, 
				image4LatexConfig.getDirectiveBuilder());
		controller.initializeListeners();
	}

	
	
	
	
	private Composite makeMainGui(Composite parent) {
		Composite gui = new Composite(parent, SWT.NONE);
		gui.setLayout(new GridLayout());
		
		makeTextViewer(gui);
		GridData textGd = new GridData();
		textGd.horizontalAlignment = SWT.FILL;
		textGd.verticalAlignment = SWT.FILL;
		textGd.grabExcessHorizontalSpace = true;
		textGd.grabExcessVerticalSpace = false;
		textGd.heightHint = 100;
		textViewer.getTextWidget().setLayoutData(textGd);
		
		ScrolledComposite scroll = makeScrolledComposite(gui);
		GridData scrollGd = new GridData();
		scrollGd.horizontalAlignment = SWT.FILL;
		scrollGd.verticalAlignment = SWT.FILL;
		scrollGd.grabExcessHorizontalSpace = true;
		scrollGd.grabExcessVerticalSpace = true;
		scroll.setLayoutData(scrollGd);
		
		return gui;
	}
	
	
	
	
	private void makeTextViewer(Composite parent) {
		textViewer = new TextViewer(parent, SWT.BORDER | SWT.HORIZONTAL | SWT.VERTICAL);
		StyledText text = textViewer.getTextWidget();
		text.setLayout(new GridLayout());
		Font font = new Font(text.getDisplay(), new FontData("Courier", 10, SWT.NORMAL));
		textViewer.getTextWidget().setFont(font);
		textViewer.setEditable(false);
	}
	
	
	
	
	private ScrolledComposite makeScrolledComposite(Composite parent) {
		ScrolledComposite scroll = new ScrolledComposite(parent, SWT.V_SCROLL);
		scroll.setLayout(new GridLayout());
		scroll.setExpandVertical(true);
		scroll.setExpandHorizontal(true);
		
		Composite scrollContentComposite = makeScrollContentComposite(scroll);
		scroll.setContent(scrollContentComposite);

		Point scrollContentSize = scrollContentComposite.computeSize(SWT.DEFAULT, SWT.DEFAULT, false);
		int height = scrollContentSize.y;
		scroll.setMinHeight(height);
		return scroll;
	}
	
	
	
	
	private Composite makeScrollContentComposite(ScrolledComposite scroll) {
		Composite scrollContentComposite = new Composite(scroll, SWT.NONE);
		scrollContentComposite.setLayout(new GridLayout(1, false));

		Composite optionsComposite = makeOptionsComposite(scrollContentComposite);
		GridData optionsGd = new GridData();
		optionsGd.horizontalAlignment = SWT.FILL;
		optionsGd.grabExcessHorizontalSpace = true;
		optionsComposite.setLayoutData(optionsGd);

		
		bUseFigure = new Button(scrollContentComposite, SWT.CHECK);
		bUseFigure.setText("Use Figure environment");
		
		Group figureConfigGroup = makeFigureConfigGroup(scrollContentComposite);
		GridData figureConfigGd = new GridData();
		figureConfigGd.horizontalAlignment = GridData.FILL;
		figureConfigGd.grabExcessHorizontalSpace = true;
		figureConfigGroup.setLayoutData(figureConfigGd);
		
		scrollContentComposite.setTabList(new Control[] {optionsComposite, bUseFigure, figureConfigGroup});
		
		return scrollContentComposite;
	}
	
	
	
	private Composite makeOptionsComposite(Composite parent) {
		Composite comp = new Composite(parent, SWT.NONE);
		comp.setLayout(new GridLayout(2, false));
		Label label = new Label(comp, SWT.NONE);
		label.setText("Includegraphics options");
		tOptions = new TextViewer(comp, SWT.BORDER);
		tOptions.setDocument(new Document());
		fixTextViewerTraversal(tOptions);
		
		GridData optionsData = new GridData();
		optionsData.horizontalAlignment = SWT.FILL;
		optionsData.grabExcessHorizontalSpace = true;
		tOptions.getTextWidget().setLayoutData(optionsData);
		
		comp.setTabList(new Control[] {tOptions.getTextWidget()});
		
		return comp;
	}
	
	
	private Group makeFigureConfigGroup(Composite parent) {
		Group figureConfigGroup = new Group(parent, SWT.NONE);
		GridLayout layout = new GridLayout(2, false);
		layout.horizontalSpacing = 50;
		figureConfigGroup.setLayout(layout);
				
		makePositionComposite(figureConfigGroup);
		makeRemainingComposite(figureConfigGroup);
		
		return figureConfigGroup;
	}
	
	
	
	
	private Composite makePositionComposite(Composite parent) {
		Composite positionComposite = new Composite(parent, SWT.NONE);
		positionComposite.setLayout(new GridLayout(1, true));

		new Label(positionComposite, SWT.NONE).setText("Position");
		bForce = new Button(positionComposite, SWT.CHECK);
		bForce.setText("Force");
		bHere = new Button(positionComposite, SWT.CHECK);
		bHere.setText("Here");
		bTop = new Button(positionComposite, SWT.CHECK);
		bTop.setText("Top");
		bBottom = new Button(positionComposite, SWT.CHECK);
		bBottom.setText("Bottom");
		bPage = new Button(positionComposite, SWT.CHECK);
		bPage.setText("Page");
		return positionComposite;
	}
	
	
	
	
	private Composite makeRemainingComposite(Composite parent) {
		Composite remainingComposite = new Composite(parent, SWT.NONE);
		remainingComposite.setLayout(new GridLayout(2, false));
		
		Label centeringLabel = new Label(remainingComposite, SWT.NONE);
		centeringLabel.setText("Centering");
		bCentering = new Button(remainingComposite, SWT.CHECK);

		
		Label captionLabel = new Label(remainingComposite, SWT.NONE);
		captionLabel.setText("Caption");
		tCaption = new TextViewer(remainingComposite, SWT.BORDER);
		tCaption.setDocument(new Document());
		fixTextViewerTraversal(tCaption);
		
		Label labelLabel = new Label(remainingComposite, SWT.NONE);
		labelLabel.setText("Label");
		tLabel = new TextViewer(remainingComposite, SWT.BORDER);
		tLabel.setDocument(new Document());
		fixTextViewerTraversal(tLabel);
		
		return remainingComposite;
	}
	
	
	public void enableFigure() {
		Color cEnabled = Display.getDefault().getSystemColor(SWT.COLOR_LIST_BACKGROUND);
		bTop.setEnabled(true);
		bBottom.setEnabled(true);
		bHere.setEnabled(true);
		bPage.setEnabled(true);
		bForce.setEnabled(true);
		bCentering.setEnabled(true);

		tCaption.getTextWidget().setEnabled(true);
		tCaption.getTextWidget().setBackground(cEnabled);
		tLabel.getTextWidget().setEditable(true);
		tLabel.getTextWidget().setBackground(cEnabled);
	}
	
	public void disableFigure() {
		Color cDisabled = Display.getDefault().getSystemColor(SWT.COLOR_WIDGET_BACKGROUND);
		bTop.setEnabled(false);
		bBottom.setEnabled(false);
		bHere.setEnabled(false);
		bPage.setEnabled(false);
		bForce.setEnabled(false);
		bCentering.setEnabled(false);
		
		tCaption.getTextWidget().setEnabled(false);
		tCaption.getTextWidget().setBackground(cDisabled);
		tLabel.getTextWidget().setEditable(false);
		tLabel.getTextWidget().setBackground(cDisabled);
	}

	
	private void fixTextViewerTraversal(ITextViewer viewer) {
		StyledText text = viewer.getTextWidget();
		text.addTraverseListener(new TraverseListener() {
			public void keyTraversed(TraverseEvent e) {
		        if (e.detail == SWT.TRAVERSE_TAB_NEXT || e.detail == SWT.TRAVERSE_TAB_PREVIOUS) {
		        	e.doit = true;
		        }
			}
		});
	}
}
