package org.deft.editorintegrator.latex.gui;

import java.io.File;

import org.deft.editorintegrator.latex.directivebuilder.IncludeGraphicsDirectiveBuilder;
import org.deft.repository.datamodel.ArtifactReference;
import org.eclipse.jface.text.DocumentEvent;
import org.eclipse.jface.text.IDocumentListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;

public class ImageIntoLatexIntegratorGuiController {

	private ImageIntoLatexIntegratorGui gui;
	private IncludeGraphicsDirectiveBuilder builder;
	private ArtifactReference reference;
	private File tempDir;
	
	public ImageIntoLatexIntegratorGuiController(ImageIntoLatexIntegratorGui gui, 
			ArtifactReference reference, File tempDir, IncludeGraphicsDirectiveBuilder builder) {
		this.gui = gui;
		this.reference = reference;
		this.tempDir = tempDir;
		this.builder = builder;
		gui.tLabel.getDocument().set(reference.getId()); //set initial value for label
		updatePreview();
		if (gui.bUseFigure.getSelection()) {
			gui.enableFigure();
		} else {
			gui.disableFigure();
		}
	}
	
	public void initializeListeners() {
		TextListener tl = new TextListener();
		gui.tCaption.getDocument().addDocumentListener(tl);
		gui.tLabel.getDocument().addDocumentListener(tl);
		gui.tOptions.getDocument().addDocumentListener(tl);
		
		gui.bUseFigure.addSelectionListener(new DisableFigureListener());
		
		gui.bUseFigure.addSelectionListener(new CheckListener());
		gui.bForce.addSelectionListener(new CheckListener());
		gui.bTop.addSelectionListener(new CheckListener());
		gui.bBottom.addSelectionListener(new CheckListener());
		gui.bHere.addSelectionListener(new CheckListener());
		gui.bPage.addSelectionListener(new CheckListener());
		gui.bCentering.addSelectionListener(new CheckListener());
	}
	
	
	private void updatePreview() {
		String previewText = builder.buildDirective(reference, tempDir);
		gui.textViewer.getTextWidget().setText(previewText);
	}
	
	
	private void setBuilderValues() {
		builder.setCaption(gui.tCaption.getTextWidget().getText());
		builder.setLabel(gui.tLabel.getTextWidget().getText());
		builder.setOptions(gui.tOptions.getTextWidget().getText());
		builder.setUsefigure(gui.bUseFigure.getSelection());
		builder.setForce(gui.bForce.getSelection());
		builder.setTop(gui.bTop.getSelection());
		builder.setBottom(gui.bBottom.getSelection());
		builder.setHere(gui.bHere.getSelection());
		builder.setPage(gui.bPage.getSelection());
		builder.setCentering(gui.bCentering.getSelection());
	}
	
	private class TextListener implements IDocumentListener {

		@Override
		public void documentAboutToBeChanged(DocumentEvent event) {
		}

		@Override
		public void documentChanged(DocumentEvent event) {
			setBuilderValues();
			updatePreview();		
		}
	}
	
	private class CheckListener implements SelectionListener {

		@Override
		public void widgetSelected(SelectionEvent e) {
			setBuilderValues();
			updatePreview();
		}

		@Override
		public void widgetDefaultSelected(SelectionEvent e) {
		}
		
	}
	
	private class DisableFigureListener implements SelectionListener {

		@Override
		public void widgetSelected(SelectionEvent e) {
			if (gui.bUseFigure.getSelection()) {
				gui.enableFigure();
			} else {
				gui.disableFigure();
			}
		}

		@Override
		public void widgetDefaultSelected(SelectionEvent e) {
		}
		
	}
	
	
	
	
	
}
