package org.deft.emftext;

import java.io.InputStream;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.jface.text.rules.ITokenScanner;
import org.emftext.access.EMFTextAccessPlugin;
import org.emftext.access.EMFTextAccessProxy;
import org.emftext.access.resource.IColorManager;
import org.emftext.access.resource.IParser;
import org.emftext.access.resource.IResource;
import org.emftext.access.resource.IUIMetaInformation;

public class EmfTextManager {
	
	public List<String> getRegisteredLanguageIds() {
		return EMFTextAccessPlugin.getConcreteSyntaxNamesList();
	}

	
	public IUIMetaInformation getMetaInformation(String languageId) {
    	List<IUIMetaInformation> metaInfoList = EMFTextAccessPlugin.getUIPluginRegistry();
    	for (IUIMetaInformation metaInfo : metaInfoList) {
    		if (metaInfo.getSyntaxName().equals(languageId)) {
    			return metaInfo;
    		}
    	}
    	return null;
	}
	
	
	public ITokenScanner getStyledTokenScanner(String languageId) {
		IUIMetaInformation metaInfo = getMetaInformation(languageId);
		if (metaInfo != null) {
			IColorManager colorManager = metaInfo.createColorManager();
			ITokenScanner scanner = metaInfo.createTokenScanner(null, colorManager);
			return scanner;
		}
		return null;
	}
	
	public IParser getParser(InputStream is, String languageId) {
		IUIMetaInformation metaInfo = getMetaInformation(languageId);
		IParser parser = metaInfo.createParser(is, "UTF-8");
		return parser;
	}
	
	public IResource getEmfTextResource(EObject eObject) {
		Resource resource = eObject.eResource();
		IResource res = (IResource)EMFTextAccessProxy.get(resource, IResource.class);
		return res;
	}

}
