package org.deft.emftext;

import java.io.InputStream;
import java.util.Collections;
import java.util.List;

import org.deft.repository.artifacttype.MultiArtifactDependencies;
import org.deft.repository.artifacttype.Validator;
import org.deft.repository.exception.DeftNullArgumentException;
import org.deft.repository.exception.DeftParseException;
import org.deft.repository.fragmentsource.FragmentSource;
import org.deft.repository.fragmentsource.OriginalFragmentLocation;
import org.emftext.access.resource.IParseResult;
import org.emftext.access.resource.IParser;

public class EmfTextValidator implements Validator {

	private EmfTextManager etm = new EmfTextManager();
	private String emfTextLanguageId;
	
	public EmfTextValidator(String emfTextLanguageId) {
		if (emfTextLanguageId == null)  {
			throw new DeftNullArgumentException("emfTextLanguageId");
		}
		this.emfTextLanguageId = emfTextLanguageId;
			
	}
	
	@Override
	public void validate(InputStream is) {
		IParser parser = etm.getParser(is, emfTextLanguageId);
		IParseResult result = parser.parse();
		if (result.getRoot() == null) {
			throw new DeftParseException("Could not parse file");
		}

	}

	@Override
	public List<OriginalFragmentLocation> getDependencyLocations(FragmentSource source, OriginalFragmentLocation location) {
		return Collections.emptyList();
	}

	@Override
	public MultiArtifactDependencies getMultiArtifactDependencies(
			FragmentSource source, OriginalFragmentLocation location) {
		return new MultiArtifactDependencies(location);
	}

}
