package org.deft.exporter;

import java.lang.reflect.InvocationTargetException;
import java.util.Map;

import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.Tutorial;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.widgets.Display;

public abstract class AbstractExporter implements IRunnableWithProgress, Exporter {

	private Fragment fragment;
	private Map<String, Object> options;
	private IProgressMonitor monitor;
	
	@Override
	public void exportWithProgress(Fragment fragment, Map<String, Object> options) {

		if (!(fragment instanceof Chapter) && !(fragment instanceof Tutorial)) {
			throw new IllegalArgumentException("Must be Chapter or Tutorial.");
		}

		if (fragment == null || options == null) {
			throw new IllegalArgumentException("No null values are allowed.");
		}

		this.fragment = fragment;
		this.options = options;

		ProgressMonitorDialog dia = new ProgressMonitorDialog(Display.getDefault().getActiveShell());

		try {
			// Now run the progress dialog with this object
			dia.run(true, false, this);
			
		} catch (InvocationTargetException e) {
			e.printStackTrace();
			
		} catch (InterruptedException e) {
			e.printStackTrace();
		} finally {
			if (monitor != null) {
				monitor.done();
			}
		}	
	}

	@Override
	public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
		this.monitor = monitor;
		if (fragment instanceof Chapter) {
			export((Chapter)fragment, options, monitor);
		} else if (fragment instanceof Tutorial) {
			export((Tutorial)fragment, options, monitor);
		}
	}
	
	
	protected abstract void export(Chapter chapter, Map<String, Object> options, IProgressMonitor monitor);
	
	protected abstract void export(Tutorial tutorial, Map<String, Object> options, IProgressMonitor monitor);

}
