package org.deft.exporter.latex;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import net.sourceforge.texlipse.builder.BuilderRegistry;
import net.sourceforge.texlipse.builder.ProgramRunner;

import org.deft.exporter.ExportWizardPage;
import org.deft.util.ImageHelper;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

public class LatexExportWizardPage extends ExportWizardPage {


	private Composite mainComposite;
	private Text textOutputDir, textAdditionalFilesDir, textPdfLatex, textBibtex;
	private Button checkBoxCompileLatex;
	private Button buttonPdfLatex, buttonBibtex; 
	private DirectoryInputValidator outputDirValidator = new DirectoryInputValidator(false, "Output destination");
	private DirectoryInputValidator additionalFilesDirValidator = new DirectoryInputValidator(true, "Additional files directory");
	private LatexExecutableInputValidator pdfLatexValidator = new LatexExecutableInputValidator(false, "pdflatex");
	private LatexExecutableInputValidator bibtexValidator = new LatexExecutableInputValidator(true, "bibtex");
	
	
	public LatexExportWizardPage() {
		super("Latex Export");
		setTitle("Select destination to export to");
		setDescription("");
		setPageComplete(false);
	}

	@Override
	public void createControl(Composite parent) {

		mainComposite = new Composite(parent, SWT.NONE);
		createOutputDirGroup();
		createAdditionalFilesDirectoryGroup();
		createPdfGroup();
		mainComposite.setLayout(new GridLayout());
		
		setDefaultOptions();
		
		setControl(mainComposite);
	}
	


	private void createOutputDirGroup() {
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		
		GridData gridDataForGroup = new GridData();
		gridDataForGroup.horizontalAlignment = GridData.FILL;
		gridDataForGroup.grabExcessHorizontalSpace = true;
		gridDataForGroup.verticalAlignment = GridData.CENTER;
		Group groupOutputDir = new Group(mainComposite, SWT.NONE);
		groupOutputDir.setText("Choose destination");
		groupOutputDir.setLayout(gridLayout);
		groupOutputDir.setLayoutData(gridDataForGroup);
		textOutputDir = new Text(groupOutputDir, SWT.BORDER);
		
		GridData gridDataForText = new GridData();
		gridDataForText.horizontalAlignment = GridData.FILL;
		gridDataForText.grabExcessHorizontalSpace = true;
		gridDataForText.verticalAlignment = GridData.CENTER;
		textOutputDir.setLayoutData(gridDataForText);
		textOutputDir.addModifyListener(new ModifyListener() {
			@Override
			public void modifyText(ModifyEvent e) {
				setPageCompleteOrErrorState();
			}
		});

		Button buttonDirectoryDialog = new Button(groupOutputDir, SWT.NONE);
		buttonDirectoryDialog.setImage(ImageHelper.getDeftIcon("enabled/file-open.png"));
		buttonDirectoryDialog.addSelectionListener(
				new OpenDirectoryDialogSelectionListener(textOutputDir, "Select output directory"));
	}


	private void createAdditionalFilesDirectoryGroup() {
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		
		GridData gridDataForGroup = new GridData();
		gridDataForGroup.horizontalAlignment = GridData.FILL;
		gridDataForGroup.grabExcessHorizontalSpace = true;
		gridDataForGroup.verticalAlignment = GridData.CENTER;
		Group groupAdditionalFilesDir = new Group(mainComposite, SWT.NONE);
		groupAdditionalFilesDir.setText("Select directory with additional files");
		groupAdditionalFilesDir.setLayout(gridLayout);
		groupAdditionalFilesDir.setLayoutData(gridDataForGroup);
		textAdditionalFilesDir = new Text(groupAdditionalFilesDir, SWT.BORDER);
		
		GridData gridDataForText = new GridData();
		gridDataForText.horizontalAlignment = GridData.FILL;
		gridDataForText.grabExcessHorizontalSpace = true;
		gridDataForText.verticalAlignment = GridData.CENTER;
		textAdditionalFilesDir.setLayoutData(gridDataForText);
		textAdditionalFilesDir.addModifyListener(new ModifyListener() {
			@Override
			public void modifyText(ModifyEvent e) {
				setPageCompleteOrErrorState();
			}
		});

		Button buttonDirectoryDialog = new Button(groupAdditionalFilesDir, SWT.NONE);
		buttonDirectoryDialog.setImage(ImageHelper.getDeftIcon("enabled/file-open.png"));
		buttonDirectoryDialog.addSelectionListener(
				new OpenDirectoryDialogSelectionListener(textAdditionalFilesDir, "Select directory with additional files"));
	}
	
	
	private void createPdfGroup() {
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 3;
		
		GridData gridDataForGroup = new GridData();
		gridDataForGroup.horizontalAlignment = GridData.FILL;
		gridDataForGroup.grabExcessHorizontalSpace = true;
		gridDataForGroup.verticalAlignment = GridData.CENTER;

		Group groupPdf = new Group(mainComposite, SWT.NONE);
		groupPdf.setText("PDF options");
		groupPdf.setLayout(gridLayout);
		groupPdf.setLayoutData(gridDataForGroup);

		
		GridData gridData4 = new GridData();
		gridData4.horizontalSpan = 3;
		checkBoxCompileLatex = new Button(groupPdf, SWT.CHECK);
		checkBoxCompileLatex.setLayoutData(gridData4);
		checkBoxCompileLatex.setText("Compile to PDF");
		checkBoxCompileLatex.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				setExecutableWidgetsEnableState();
				setPageCompleteOrErrorState();
			}
		});

		
		

		Label labelPdfLatex = new Label(groupPdf, SWT.NONE);
		labelPdfLatex.setText("pdflatex: ");
		textPdfLatex = new Text(groupPdf, SWT.BORDER);
		textPdfLatex.addModifyListener(new ModifyListener() {
			@Override
			public void modifyText(ModifyEvent e) {
				setPageCompleteOrErrorState();
			}
		});
		buttonPdfLatex = new Button(groupPdf, SWT.NONE);
		addGroupContent(labelPdfLatex, textPdfLatex, buttonPdfLatex, "pdflatex");
		
		Label labelBibtex = new Label(groupPdf, SWT.NONE);
		labelBibtex.setText("bibtex: ");
		textBibtex = new Text(groupPdf, SWT.BORDER);
		textBibtex.addModifyListener(new ModifyListener() {
			@Override
			public void modifyText(ModifyEvent e) {
				setPageCompleteOrErrorState();
			}
		});
		buttonBibtex = new Button(groupPdf, SWT.NONE);
		addGroupContent(labelBibtex, textBibtex, buttonBibtex, "bibtex");
		

	}
	
	
	private void addGroupContent(Label label, Text text, Button button, String executable) {
		
		GridData gridDataForText = new GridData();
		gridDataForText.horizontalAlignment = GridData.FILL;
		gridDataForText.grabExcessHorizontalSpace = true;
		gridDataForText.verticalAlignment = GridData.CENTER;
		text.setLayoutData(gridDataForText);
		text.addModifyListener(new ModifyListener() {

			@Override
			public void modifyText(ModifyEvent e) {
				setPageCompleteOrErrorState();
			}
			
		});
		button.setImage(ImageHelper.getDeftIcon("enabled/file-open.png"));
		button.addSelectionListener(
				new OpenFileDialogSelectionListener(text, "Select " + executable + " executable"));
	}
	
	/**
	 * Sets the default options saved for example in the IDialogSettings.
	 */
	private void setDefaultOptions() {
		IDialogSettings settings = Activator.getDefault().getDialogSettings();
		
		String outputDir = settings.get(LatexExporterConstants.OUPUT_DIRECTORY);
		this.textOutputDir.setText(outputDir == null ? "" : outputDir);
		
		String additionalFilesDir = settings.get(LatexExporterConstants.ADDITIONAL_FILES_DIR);
		this.textAdditionalFilesDir.setText(additionalFilesDir == null ? "" : additionalFilesDir);

		boolean compile = settings.getBoolean(LatexExporterConstants.COMPILE_LATEX_FILES);
		this.checkBoxCompileLatex.setSelection(compile);

		String pdfLatexPath = settings.get(LatexExporterConstants.LATEX_EXECUTABLE);
		if (pdfLatexPath == null || pdfLatexPath.isEmpty()) {
			pdfLatexPath = getPathForExecutableFromTexlipsePreferences("pdflatex.exe");
		}
		this.textPdfLatex.setText(pdfLatexPath == null ? "" : pdfLatexPath);

		String bibtexPath = settings.get(LatexExporterConstants.BIBTEX_EXECUTABLE);
		if (bibtexPath == null || bibtexPath.isEmpty()) {
			bibtexPath = getPathForExecutableFromTexlipsePreferences("bibtex.exe");
		}
		this.textBibtex.setText(bibtexPath == null ? "" : bibtexPath);
		
		setExecutableWidgetsEnableState();
	}
	
	
	
	private String getPathForExecutableFromTexlipsePreferences(String executableName) {
        int size = BuilderRegistry.getNumberOfRunners();
        for (int i = 0; i < size; i++) {
            ProgramRunner runner = BuilderRegistry.getRunner(i);
            if (runner.getProgramName().equals(executableName)) {
            	String path = runner.getProgramPath();
            	return path;
            }
        }
        return "";
	}


	private void setExecutableWidgetsEnableState() {
		boolean shouldBeEnabled = checkBoxCompileLatex.getSelection();
		textBibtex.setEnabled(shouldBeEnabled);
		textPdfLatex.setEnabled(shouldBeEnabled);
		buttonBibtex.setEnabled(shouldBeEnabled);
		buttonPdfLatex.setEnabled(shouldBeEnabled);
	}
	
	private void setPageCompleteOrErrorState() {
		String outputError = outputDirValidator.isValid(textOutputDir.getText());
		if (!isEmpty(outputError)) {
			setErrorMessage(outputError);
			setPageComplete(false);
			return;
		}
		
		String additonalFilesDirError = additionalFilesDirValidator.isValid(textAdditionalFilesDir.getText());
		if (!isEmpty(additonalFilesDirError)) {
			setErrorMessage(additonalFilesDirError);
			setPageComplete(false);
			return;
		}
		
		//ok, we've got an output directory, and (maybe) a dir with additional files
		//now check if the pdf export options are valid
		if (checkBoxCompileLatex.getSelection()) {

			String pdfLatexError = pdfLatexValidator.isValid(textPdfLatex.getText());
			if (!isEmpty(pdfLatexError)) {
				setErrorMessage(pdfLatexError);
				setPageComplete(false);
				return;
			}
			
			String bibtexError = bibtexValidator.isValid(textBibtex.getText());
			if (!isEmpty(bibtexError)) {
				setErrorMessage(bibtexError);
				setPageComplete(false);
				return;
			}
		}
		
		//no errors found
		setErrorMessage(null);
		setPageComplete(true);

	}
	
	private boolean isEmpty(String string) {
		return string == null || string.isEmpty();
	}
	

	@Override
	public Map<String, Object> getOptions() {		
		Map<String, Object> map = new HashMap<String, Object>();
		map.put(LatexExporterConstants.OUPUT_DIRECTORY, textOutputDir.getText());
		map.put(LatexExporterConstants.ADDITIONAL_FILES_DIR, textAdditionalFilesDir.getText());		
		map.put(LatexExporterConstants.COMPILE_LATEX_FILES, checkBoxCompileLatex.getSelection());
		map.put(LatexExporterConstants.BIBTEX_EXECUTABLE, textBibtex.getText());
		map.put(LatexExporterConstants.LATEX_EXECUTABLE, textPdfLatex.getText());		
		return map;
	}
	
	
	@Override
	public void saveDialogSettings() {
		IDialogSettings settings = Activator.getDefault().getDialogSettings();
		settings.put(LatexExporterConstants.OUPUT_DIRECTORY, textOutputDir.getText());
		settings.put(LatexExporterConstants.ADDITIONAL_FILES_DIR, textAdditionalFilesDir.getText());
		settings.put(LatexExporterConstants.COMPILE_LATEX_FILES, checkBoxCompileLatex.getSelection());
		settings.put(LatexExporterConstants.BIBTEX_EXECUTABLE, textBibtex.getText());
		settings.put(LatexExporterConstants.LATEX_EXECUTABLE, textPdfLatex.getText());		
	}


	
	/**
	 * Selection adapter that is executed when the Open File Dialog button is
	 * clicked. It opens, surprise, a file dialog.
	 */
	private class OpenFileDialogSelectionListener extends SelectionAdapter {

		private Text text;
		private String caption;
		
		public OpenFileDialogSelectionListener(Text text, String caption) {
			this.text = text;
			this.caption = caption;
		}
		
		@Override
		public void widgetSelected(SelectionEvent event) {

			FileDialog dia = new FileDialog(Display.getDefault().getActiveShell(), SWT.SAVE);
			dia.setText(caption);
			String dir = getDirectory(text.getText());
			dia.setFilterPath(dir);
			String filename = dia.open();
			if (filename != null) {
				text.setText(filename);
			}
		}
		
		private String getDirectory(String path) {
			try {
				File f = new File(path);
				if (f.isDirectory()) {
					return path;
				} else {
					String parent = f.getParent();
					return parent;
				}
			}
			catch (Exception e) {
				return null;
			}
		}
	}
	
	
	
	/**
	 * Selection adapter that is executed when the Open Directory Dialog button is
	 * clicked.
	 */
	private class OpenDirectoryDialogSelectionListener extends SelectionAdapter {

		private Text text;
		private String caption;
		
		public OpenDirectoryDialogSelectionListener(Text text, String caption) {
			this.text = text;
			this.caption = caption;
		}
		
		@Override
		public void widgetSelected(SelectionEvent event) {

			DirectoryDialog dia = new DirectoryDialog(Display.getDefault().getActiveShell(), SWT.SAVE);
			dia.setText(caption);
			dia.setFilterPath(text.getText());
			String filename = dia.open();
			if (filename != null) {
				text.setText(filename);
			} 
		}
	}
	
}
