package org.deft.exporter.odftext;

import java.util.HashMap;
import java.util.Map;

import org.deft.exporter.ExportWizardPage;
import org.deft.util.ImageHelper;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Text;

public class OdfTextExportWizardPage extends ExportWizardPage {


	private Composite mainComposite;
	private Text textOutputDir;
	private DirectoryInputValidator outputDirValidator = new DirectoryInputValidator(false, "Output destination");
	
	
	public OdfTextExportWizardPage() {
		super("ODF Text Export");
		setTitle("Select destination to export to");
		setDescription("");
		setPageComplete(false);
	}

	@Override
	public void createControl(Composite parent) {

		mainComposite = new Composite(parent, SWT.NONE);
		createOutputDirGroup();
		mainComposite.setLayout(new GridLayout());
		
		setDefaultOptions();
		
		setControl(mainComposite);
	}
	


	private void createOutputDirGroup() {
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		
		GridData gridDataForGroup = new GridData();
		gridDataForGroup.horizontalAlignment = GridData.FILL;
		gridDataForGroup.grabExcessHorizontalSpace = true;
		gridDataForGroup.verticalAlignment = GridData.CENTER;
		Group groupOutputDir = new Group(mainComposite, SWT.NONE);
		groupOutputDir.setText("Choose destination");
		groupOutputDir.setLayout(gridLayout);
		groupOutputDir.setLayoutData(gridDataForGroup);
		textOutputDir = new Text(groupOutputDir, SWT.BORDER);
		
		GridData gridDataForText = new GridData();
		gridDataForText.horizontalAlignment = GridData.FILL;
		gridDataForText.grabExcessHorizontalSpace = true;
		gridDataForText.verticalAlignment = GridData.CENTER;
		textOutputDir.setLayoutData(gridDataForText);
		textOutputDir.addModifyListener(new ModifyListener() {
			@Override
			public void modifyText(ModifyEvent e) {
				setPageCompleteOrErrorState();
			}
		});

		Button buttonDirectoryDialog = new Button(groupOutputDir, SWT.NONE);
		buttonDirectoryDialog.setImage(ImageHelper.getDeftIcon("enabled/file-open.png"));
		buttonDirectoryDialog.addSelectionListener(
				new OpenDirectoryDialogSelectionListener(textOutputDir, "Select output directory"));
	}



	
	/**
	 * Sets the default options saved for example in the IDialogSettings.
	 */
	private void setDefaultOptions() {
		IDialogSettings settings = Activator.getDefault().getDialogSettings();
		
		String outputDir = settings.get(OdfTextExporterConstants.OUPUT_DIRECTORY);
		this.textOutputDir.setText(outputDir == null ? "" : outputDir);
		
	}
	
	
	

	private void setPageCompleteOrErrorState() {
		String outputError = outputDirValidator.isValid(textOutputDir.getText());
		if (!isEmpty(outputError)) {
			setErrorMessage(outputError);
			setPageComplete(false);
			return;
		}
		
		//no errors found
		setErrorMessage(null);
		setPageComplete(true);

	}
	
	private boolean isEmpty(String string) {
		return string == null || string.isEmpty();
	}
	

	@Override
	public Map<String, Object> getOptions() {		
		Map<String, Object> map = new HashMap<String, Object>();
		map.put(OdfTextExporterConstants.OUPUT_DIRECTORY, textOutputDir.getText());
		return map;
	}
	
	
	@Override
	public void saveDialogSettings() {
		IDialogSettings settings = Activator.getDefault().getDialogSettings();
		settings.put(OdfTextExporterConstants.OUPUT_DIRECTORY, textOutputDir.getText());
	}



	
	
	/**
	 * Selection adapter that is executed when the Open Directory Dialog button is
	 * clicked.
	 */
	private class OpenDirectoryDialogSelectionListener extends SelectionAdapter {

		private Text text;
		private String caption;
		
		public OpenDirectoryDialogSelectionListener(Text text, String caption) {
			this.text = text;
			this.caption = caption;
		}
		
		@Override
		public void widgetSelected(SelectionEvent event) {

			DirectoryDialog dia = new DirectoryDialog(Display.getDefault().getActiveShell(), SWT.SAVE);
			dia.setText(caption);
			dia.setFilterPath(text.getText());
			String filename = dia.open();
			if (filename != null) {
				text.setText(filename);
			} 
		}
	}
	
}
