package org.deft.fragmentsourcegui;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;

public class EclipseFragmentSourceGuiRegistry implements FragmentSourceGuiRegistry {
	
	private static final String FRAGMENTSOURCEGUI_EXTENSION_POINT_NAME = "org.deft.fragmentsourcegui";
		
	private Map<String, FragmentSourceGuiConfig> registeredConfigs = new HashMap<String, FragmentSourceGuiConfig>(); 

	public EclipseFragmentSourceGuiRegistry() {
		getAndCacheRegisteredEditors();
	}
	
	private void getAndCacheRegisteredEditors() {
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint point = registry.getExtensionPoint(FRAGMENTSOURCEGUI_EXTENSION_POINT_NAME);
		for (IConfigurationElement ce : point.getConfigurationElements()) {
			FragmentSourceGuiConfig config = new FragmentSourceGuiConfig(ce);
			registeredConfigs.put(config.getId(), config);
		}
	}
	
	@Override
	public FragmentSourceGui getFragmentSourceGui(String fragmentSourceGuiId) {
		FragmentSourceGuiConfig config = registeredConfigs.get(fragmentSourceGuiId);
		if (config != null) {
			FragmentSourceGui gui = config.getFragmentSourceGui();
			return gui;
		}
		return null;
	}
	
	@Override
	public List<String> getFragmentSourceGuiIds() {
		List<String> guiIds = new LinkedList<String>(registeredConfigs.keySet());
		return guiIds;
	}
	
	
	
	private class FragmentSourceGuiConfig {
	
		private IConfigurationElement ce;
		private FragmentSourceGui fragmentSourceGui;
		
		public FragmentSourceGuiConfig(IConfigurationElement ce) {
			this.ce = ce;
		}
		
		public String getId() {
			String id = ce.getAttribute("id");
			return id;
		}
		
		public FragmentSourceGui getFragmentSourceGui() {
			if (fragmentSourceGui == null) {
				try {
					fragmentSourceGui = (FragmentSourceGui)ce.createExecutableExtension("class");
				} catch (CoreException e) {
					e.printStackTrace();
				}
			}
			return fragmentSourceGui;
		}	
	}


}