package org.deft.gui;

import java.util.LinkedList;
import java.util.List;

import org.deft.gui.dialog.SelectArtifactTypeDialog;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.fragmentsource.FragmentSource;
import org.deft.repository.fragmentsource.OriginalFragmentLocation;
import org.deft.repository.repositorymanager.data.ArtifactTypeResolver;
import org.deft.repository.service.ExtensionService;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.ui.IWorkbenchWindow;

public class UIArtifactTypeResolver implements ArtifactTypeResolver {

	private ExtensionService extensionService;
	private IWorkbenchWindow window;
	
	public UIArtifactTypeResolver(IWorkbenchWindow window, ExtensionService es) {
		this.extensionService = es;
		this.window = window;
	}
	
	@Override
	public ArtifactType getArtifactType(FragmentSource fragmentSource, OriginalFragmentLocation location) {
		List<String> typeIds = fragmentSource.getArtifactTypeIds(location);
		if (typeIds.isEmpty()) {
			ArtifactType type = askUserToChooseAmongAllRegisteredTypes(location);
			return type;
		} else if (typeIds.size() == 1) {
			String typeId = typeIds.get(0);
			ArtifactType type = getTypeFromId(typeId);
			return type;
		} else {
			ArtifactType type = askUserToChooseAmongPossibleTypes(typeIds, location);
			return type;
		}
	}
	

	
	private ArtifactType askUserToChooseAmongAllRegisteredTypes(OriginalFragmentLocation location) {
		List<ArtifactType> types = extensionService.getArtifactTypes();
		ArtifactType type = getUserDecision(types, location);
		return type;
	}
	
	
	private ArtifactType getTypeFromId(String artifactTypeId) {
		ArtifactType type = extensionService.getArtifactType(artifactTypeId);
		return type;
	}
	
	private ArtifactType askUserToChooseAmongPossibleTypes(List<String> artifactTypeIds, OriginalFragmentLocation location) {
		List<ArtifactType> types = new LinkedList<ArtifactType>();
		for (String artifactTypeId : artifactTypeIds) {
			ArtifactType type = extensionService.getArtifactType(artifactTypeId);
			types.add(type);
		}
		ArtifactType type = getUserDecision(types, location);
		return type;
	}
	
	
	
	private ArtifactType getUserDecision(List<ArtifactType> types, OriginalFragmentLocation location) {
		SelectArtifactTypeDialog dialog = new SelectArtifactTypeDialog(window.getShell(), types, location);
		if (dialog.open() != Dialog.OK)
			return null;

		ArtifactType type = dialog.getSelectedArtifactType();
		return type;
	}


}
