package org.deft.gui.artifactselect;

import java.util.List;

import org.deft.repository.datamodel.Folder;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.FragmentType;
import org.deft.repository.datamodel.Project;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

/**
 * ContentProvider for the TreeView of the artifact selection dialog.
 * This ContentProvider displays all projects as root and all Artifacts and Artifact Folders
 * as children and descendants. 
 */
public class ArtifactContentProvider implements ITreeContentProvider {

	private static final Object[] NO_CHILDREN = new Object[0];
	private Project project;
	
	
	@Override
	public void dispose() {
		this.project = null;
	}

	@Override
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		if (newInput instanceof Project[]) {
			Project[] array = (Project[])newInput;
			this.project = array[0];
		}
	}

	@Override
	public Object[] getElements(Object inputElement) {
		Object[] elements = project.getChildren(FragmentType.ARTIFACT).toArray();
		return elements;
	}
	

	@Override
	public Object[] getChildren(Object parentElement) {
		if (parentElement instanceof Folder) {
			Folder f = (Folder)parentElement;
			List<Fragment> children = f.getChildren();
			return children.toArray();
		}
		return NO_CHILDREN;
	}

	@Override
	public Object getParent(Object element) {
		Fragment f = (Fragment)element;
		Fragment parent = f.getParentFolder();
		return parent;
	}

	@Override
	public boolean hasChildren(Object element) {
		Object[] children = getChildren(element);
		return (children != null && children.length > 0);
	}


}
