package org.deft.gui.artifactselect;

import java.util.LinkedList;
import java.util.List;

import org.deft.gui.widgets.TreeAndAutoCompleteTextfieldComposite;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.FragmentFilter;
import org.deft.repository.datamodel.Project;
import org.eclipse.jface.viewers.IContentProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.swt.widgets.Composite;

/**
 * Tree+Autocomplete Textfield composite which is used to select an artifact for the
 * insertion into a document. 
 */
public class ArtifactSelectionComposite extends TreeAndAutoCompleteTextfieldComposite {

	private Project project;
	
	public ArtifactSelectionComposite(Composite parent, int style, Project project) {
		super(parent, style);
		this.project = project;	
		super.createContents();
	}


	@Override
	protected String[] getAllProposals() {
		if (project != null) {
			List<String> artifactStrings = new LinkedList<String>();
			List<Fragment> fragments = project.getFragments(FragmentFilter.ARTIFACTFILTER);
			for (Fragment fragment : fragments) {
				String artifactString = fragment.getProjectRelativePath(); 
				artifactStrings.add(artifactString);
			}
			return artifactStrings.toArray(new String[]{});
		} else {
			return new String[]{};
		}
	}

	
	protected Character getDelimiter() {
		return '/';
	}

	@Override
	public SelectedArtifactContainer createSelectedItemContainer() {
		return new SelectedArtifactContainer(project);
	}

	public SelectedArtifactContainer getSelectedItemContainer() {
		return (SelectedArtifactContainer)super.getSelectedItemContainer();
	}
	

	@Override
	protected IContentProvider createContentProvider() {
		return new ArtifactContentProvider();
	}

	@Override
	protected ILabelProvider createLabelProvider() {
		return new ArtifactLabelProvider();
	}

	@Override
	protected Object getInput() {
		Project[] input = new Project[]{project};
		return input;
	}

}
