package org.deft.gui.dialog;

import org.deft.gui.artifactselect.ArtifactSelectionComposite;
import org.deft.gui.artifactselect.SelectedArtifactContainer;
import org.deft.gui.widgets.SelectedItemContainer;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.Project;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

public class ArtifactSelectionDialog extends Dialog {

	private ArtifactSelectionComposite asComposite;
	private Project project;
	private Artifact externallySetArtifact;
	
	public ArtifactSelectionDialog(Shell parentShell, Project project) {
		super(parentShell);
		this.project = project;
		this.setShellStyle(SWT.RESIZE | SWT.APPLICATION_MODAL | SWT.CLOSE);
	}
	
	/**
	 * Creates all the UI controls
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		
		Composite composite = new Composite(parent, SWT.NONE);
		
		// Define layout of this composite 
		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.grabExcessHorizontalSpace = true;
		gd.grabExcessVerticalSpace = true;
		composite.setLayoutData(gd);

		// Define layout of the children of this composite
		composite.setLayout(new FillLayout());
		asComposite = new ArtifactSelectionComposite(composite, SWT.FILL, project);
		asComposite.setGuiData();
		if (externallySetArtifact != null) {
			Project project = externallySetArtifact.getProject();
			SelectedArtifactContainer sac = new SelectedArtifactContainer(project);
			sac.setSelectedItem(externallySetArtifact);
			asComposite.setSelectedItemContainer(sac);
			externallySetArtifact = null;
		}
		asComposite.getText().setFocus();
		return composite;
	}
	
	@Override
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setSize(300, 300);
	}
	
	public Artifact getSelectedArtifact() {
		SelectedItemContainer container = asComposite.getSelectedItemContainer();
		Artifact artifact = (Artifact)container.getSelectedItem();
		return artifact;
	}
	
	public void setSelectedArtifact(Artifact artifact) {
		if (artifact != null) {
			//asComposite is initialized when the dialog is opened, but
			//the setSelectedArtifact method is usually called before the dialog
			//is opened, so check if asComposite is null. If so, save the artifact
			//so it can be set later (cf. createDialogArea())
			externallySetArtifact = artifact;
		}
	}
	
	@Override
	protected void okPressed() {
		Artifact selectedArtifact = getSelectedArtifact();
		if (selectedArtifact != null) {
			super.okPressed();
		}
	}
	

}
