package org.deft.gui.dialog;

import java.util.List;

import org.deft.gui.view.project.ProjectViewLabelProvider;
import org.deft.gui.view.project.nodes.ProjectViewNode;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;


public class DeletionDialog extends TitleAreaDialog {


	private List<ProjectViewNode> nodesToDelete;
	
	public DeletionDialog(Shell parentShell, List<ProjectViewNode> nodesToDelete) {
		super(parentShell);
		this.setShellStyle(SWT.RESIZE | SWT.APPLICATION_MODAL | SWT.TITLE | SWT.CLOSE);
		this.nodesToDelete = nodesToDelete;
		this.create();
	}

	/**
	 * Creates all the UI controls
	 */
	@Override
	protected Control createDialogArea(Composite parent) {

		Composite composite = new Composite(parent, SWT.NONE);

		// Define layout of this composite 
		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.grabExcessHorizontalSpace = true;
		gd.grabExcessVerticalSpace = true;
		composite.setLayoutData(gd);

		// Define layout of the children of this composite
		GridLayout layout = new GridLayout(1, false);
		composite.setLayout(layout);

		//add stuff to composite
		GridData listGd = new GridData(SWT.FILL, SWT.FILL, true, true);
		ListViewer listViewer = new ListViewer(composite, SWT.DEFAULT);
		listViewer.setContentProvider(new ListContentProvider());
		listViewer.setLabelProvider(new ProjectViewLabelProvider());
		listViewer.getList().setLayoutData(listGd);
		listViewer.setInput(nodesToDelete);
		
		// Set default text on dialog
		this.getShell().setText("Delete");
		this.setTitle("Delete content");
		this.setMessage("Should the following content be deleted?");

		return composite;
	}


	private class ListContentProvider implements IStructuredContentProvider {

		@SuppressWarnings("unchecked")
		@Override
		public Object[] getElements(Object inputElement) {
			List<ProjectViewNode> nodes = (List<ProjectViewNode>)inputElement;
			return nodes.toArray();
		}

		@Override
		public void dispose() {
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}
	}
	


}
