package org.deft.gui.dialog;

import org.deft.RepositoryFacade;
import org.deft.gui.input.ProjectNameInputValidator;
import org.deft.repository.projecttype.ProjectType;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

public class NewProjectDialog extends TitleAreaDialog {
	private String title;
	private Text projectNameText;
	private Combo projectTypesCombo;
	
	private String projectName;
	private ProjectType projectType;
	
	public NewProjectDialog(String title, Shell parentShell) {
		super(parentShell);	
		this.title = title;
	}

	@Override
	public void create() {
		// Create dialog
		super.create();
		selectFirstProjectType();
		// Then disable ok button
		Button okButton = getButton(IDialogConstants.OK_ID);
		okButton.setEnabled(false);
	}

	@Override
	protected Control createDialogArea(Composite parent) {
		Composite composite = createAndSetupDialogComposite(parent);

		addProjectNameTextField(composite);
		
		addProjectTypesCombo(composite);
		fillProjectTypesCombo();	
		
		return composite;
	}
	
	private Composite createAndSetupDialogComposite(Composite parent) {
		this.setTitle(title);
		this.setMessage("Enter project properties");
		
		// The root composite of our controls
		Composite composite = new Composite(parent, SWT.NONE);
		
		// Define how this composite should be layouted
		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.grabExcessHorizontalSpace = true;
		gd.grabExcessVerticalSpace = true;
		composite.setLayoutData(gd);
		
		// Define how the children of this composite should be layouted
		GridLayout layout = new GridLayout(2, false);
		composite.setLayout(layout);
		return composite;
	}
	
	private void addProjectNameTextField(Composite composite) {
		// Label
		GridData labelGd = new GridData();
		Label label = new Label(composite, SWT.NONE);
		label.setText("Project name: ");
		label.setLayoutData(labelGd);
		
		// Text
		GridData textGd = new GridData(GridData.FILL_HORIZONTAL);
		textGd.grabExcessHorizontalSpace = true;
		projectNameText = new Text(composite, SWT.BORDER);		
		projectNameText.setLayoutData(textGd);
		projectNameText.addModifyListener(new ModifyListener() {
		
			public void modifyText(ModifyEvent e) {				
				checkProjectNameInput();
				projectName = getInternalProjectName();
			}
		});
	}
	
	
	
	private void addProjectTypesCombo(Composite composite) {
		// Label
		GridData labelGd = new GridData();
		Label label = new Label(composite, SWT.NONE);
		label.setText("Project type: ");
		label.setLayoutData(labelGd);
		// Combo box
		GridData textGd = new GridData(GridData.FILL_HORIZONTAL);
		textGd.grabExcessHorizontalSpace = true;
		projectTypesCombo = new Combo(composite, SWT.DROP_DOWN);
		projectTypesCombo.setLayoutData(textGd);
		projectTypesCombo.addModifyListener(new ModifyListener() {
		
			public void modifyText(ModifyEvent e) {				
				checkProjectNameInput();
				projectType = getSelectedProjectType();
			}
		});
	}
	
	
	private void selectFirstProjectType() {
		if (projectTypesCombo.getItemCount() > 0)
			projectTypesCombo.select(0);
	}

	private void fillProjectTypesCombo() {
		for (ProjectType type : RepositoryFacade.instance().getProjectTypes())
			projectTypesCombo.add(type.getDescription());
	}
	
	/**
	 * Returns the type of the project as selected by the user.
	 */
	private ProjectType getSelectedProjectType() {
		
		if (projectTypesCombo.getSelectionIndex() == -1) {
			return null;
		}
		
		int index = 0;
		for (ProjectType type : RepositoryFacade.instance().getProjectTypes()) {
			if (index == projectTypesCombo.getSelectionIndex()) {
				return type;
			}
			index++;
		}
		
		return null;
	}
	
	private String getInternalProjectName() {
		return projectNameText.getText();
	}
	
	
	/**
	 * Checks if the user-specified project name is valid. If not, an error message
	 * is displayed and the OK button is disabled. 
	 */
	private void checkProjectNameInput() {
		Button okButton = getButton(IDialogConstants.OK_ID);
		okButton.setEnabled(false);
		
		if (getSelectedProjectType() == null)
			return;
		
		IInputValidator projectNameValidator = new ProjectNameInputValidator();
		String errorMsg = projectNameValidator.isValid(getInternalProjectName()); 
		if ((errorMsg != null) || (getInternalProjectName() == null) || ("".equals(getInternalProjectName()))) {
			setErrorMessage(errorMsg);
			return;
		}
						
		setErrorMessage(null);		
		okButton.setEnabled(true);
	}

	/**
	 * Returns the user-selected type for the project.
	 */
	public ProjectType getProjectType() {
		return projectType;
	}

	/**
	 * Returns the name given to the project by the user.
	 */
	public String getProjectName() {
		return projectName;
	}
	
}
