package org.deft.gui.dialog;

import java.util.List;

import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.fragmentsource.OriginalFragmentLocation;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;


public class SelectArtifactTypeDialog extends Dialog {
	
	private List<ArtifactType> artifactTypes;
	private OriginalFragmentLocation location;
	
	private ComboViewer comboViewer;
	private ArtifactType selectedArtifactType;
	

	public SelectArtifactTypeDialog(Shell parentShell, List<ArtifactType> artifactTypes,
			OriginalFragmentLocation location) {
		super(parentShell);
		this.artifactTypes = artifactTypes;
		this.location = location;
	}


	@Override
	protected Control createDialogArea(Composite parent) {
		
		// The root composite of our controls
		Composite composite = new Composite(parent, SWT.NONE);
		
		// Define how this composite should be layouted
		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.grabExcessHorizontalSpace = true;
		gd.grabExcessVerticalSpace = true;
		composite.setLayoutData(gd);
		
		// Define how the children of this composite should be layouted
		GridLayout layout = new GridLayout(3, false);
		composite.setLayout(layout);
		
		// Label
		GridData labelGd = new GridData();
		Label label = new Label(composite, SWT.NONE);
		label.setText("Import " + location.getName() + " as: ");
		label.setLayoutData(labelGd);
		
		// Combo box
		GridData textGd = new GridData(GridData.FILL_HORIZONTAL);
		textGd.grabExcessHorizontalSpace = true;
		comboViewer = new ComboViewer(composite, SWT.READ_ONLY | SWT.DROP_DOWN | SWT.BORDER);
		// Fill combo box with data
		setupCombo();

		comboViewer.addSelectionChangedListener(new ISelectionChangedListener() {
			
			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				IStructuredSelection selection = (IStructuredSelection)event.getSelection();
				selectedArtifactType = (ArtifactType)selection.getFirstElement();
			}
		});
		return composite;
	}
	
	
	private void setupCombo() {
		comboViewer.setContentProvider(new ArrayContentProvider());
		comboViewer.setLabelProvider(new ComboLabelProvider());
		comboViewer.setInput(artifactTypes);
		comboViewer.getCombo().select(0);
		selectedArtifactType = (ArtifactType)comboViewer.getElementAt(0);
	}

	private class ComboLabelProvider extends LabelProvider {

		@Override
		public String getText(Object element) {
			ArtifactType type = (ArtifactType)element;
			String name = type.getArtifactTypeName();
			return name;
		}

	}
	
	
	public ArtifactType getSelectedArtifactType() {
		return selectedArtifactType;
	}
	
	

}
