package org.deft.gui.input;

import org.eclipse.jface.dialogs.IInputValidator;

/**
 * This input validator is the basis of all validators that should be used when 
 * the user is asked to enter the
 * name of a Project, Fragment or any other repository content, which will
 * then be created. The validator will prohibit the user to enter the empty string
 * or characters like '*, ~, /, \, etc...' which might cause problems with the file system or
 * the XML representation of this name.
 */
public abstract class AbstractNameInputValidator implements IInputValidator {

	//using Strings instead of chars because of the contains() method below
	private String[] forbiddenCharacters = new String[] { "*", "~", "{", "}", "|", ";", "!",
			"[", "]", "/", ":", "\\", "<", ">", "?" };
	
	public String isValid(String newText) {

		if (isEmpty(newText)) {
			return "An empty name is not allowed.";
		}

		if (containsForbiddenCharacter(newText)) {
			String forbidden = getForbiddenCharacter(newText);
			return "The character '" + forbidden + "' is not allowed here.";
		}
		
		String s = getSpecialErrorMessage(newText);
		
		return s;
	}
	
	
	public abstract String getSpecialErrorMessage(String newText);
	
	
	private boolean isEmpty(String text) {
		return text == null || text.equals("");
	}
	
	private boolean containsForbiddenCharacter(String newText) {
		for (String str : forbiddenCharacters) {
			if (newText.contains(str)) {
				return true;
			}
		}
		return false;
	}
	
	private String getForbiddenCharacter(String newText) {
		for (String str : forbiddenCharacters) {
			if (newText.contains(str)) {
				return str;
			}
		}
		return null;
	}
}
