package org.deft.gui.input;

import org.deft.RepositoryFacade;
import org.deft.repository.datamodel.Folder;
import org.deft.repository.datamodel.FragmentType;
import org.deft.repository.datamodel.Project;

/**
 * This input validator should be used whenever the user is allowed to enter the
 * name of a Fragment. This validator will prohibit the user to enter characters
 * like '*, ~, /, \, etc...' which might cause problems with the file system or
 * the XML representation of this name.
 * 
 */
public class FragmentNameInputValidator extends AbstractNameInputValidator {

	private Project project;
	private FragmentType fragType;
	private Folder parentFolder;
	private RepositoryFacade repository = RepositoryFacade.instance();
	
	public FragmentNameInputValidator(Project project, FragmentType fragType) {
		this.project = project;
		this.fragType = fragType;
	}
	
	
	public FragmentNameInputValidator(Folder parentFolder) {
		this.parentFolder = parentFolder;
	}
	
	
	@Override
	public String getSpecialErrorMessage(String newText) {
		if (isParentRoot()) {
			if (repository.containsFragmentNameAtRoot(project, fragType, newText)) {
				return "The name \"" + newText + "\" is already used.";
			}
		} else {
			if (repository.containsFragmentName(parentFolder, newText)) {
				return "The name \"" + newText + "\" is already used.";
			}
		}
		return null;
	}
	
	/**
	 * Returns true if the fragment for which the name is validated will
	 * be added directly to a category. Returns false, if the fragment will
	 * be added to a folder.
	 */
	private boolean isParentRoot() {
		return project != null;
	}
	
	
}
