package org.deft.gui.input;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.dialogs.IInputValidator;

/**
 * This class checks if a string represents a valid location for a
 * repository. If not, an appropriate error message is returned.
 * 
 * @author abartho
 */
public class RepositoryLocationInputValidator implements IInputValidator {

	
	public String isValid(String newText) {

		if (isEmpty(newText)) {
			return "No directory selected.";
		}
		
		if (doesNotExist(newText)) {
			return "Directory does not exist.";
		}
		
		if (isNotDirectory(newText)) {
			return "This is not a directory.";
		}
		
		return null;
	}
	
	
	private boolean isEmpty(String text) {
		return text == null || text.equals("");
	}
	
	private boolean doesNotExist(String text) {
		IPath path = new Path(text);
		return !path.toFile().exists();
	}
	
	private boolean isNotDirectory(String text) {
		IPath path = new Path(text);
		return !path.toFile().isDirectory();
	}
	
}