package org.deft.gui.view.project;

import org.deft.gui.view.project.nodes.FolderNode;
import org.deft.gui.view.project.nodes.FragmentNode;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;

/**
 * Sorter for fragments (chapters, artifacts, folders, ...) in the Project Explorer. Folders
 * are always displayed before other fragments.
 * 
 * @author abartho
 */
public class ProjectViewFragmentSorter extends ViewerSorter {

	@Override
	public int compare(Viewer viewer, Object e1, Object e2) {
	
		FragmentNode f1 = (FragmentNode)e1;
		FragmentNode f2 = (FragmentNode)e2;
		
		
		if (firstFragmentIsFolderAndSecondFragmentIsNot(f1, f2)) {
			return -1;
		}
		
		if (firstFragmentIsNotFolderButSecondFragmentIs(f1, f2)) {
			return 1;
		}
		
		return super.compare(viewer, e1, e2);
	}
	
	
	private boolean firstFragmentIsFolderAndSecondFragmentIsNot(FragmentNode f1, FragmentNode f2) {
		return (f1 instanceof FolderNode) && !(f2 instanceof FolderNode);
	}
	
	private boolean firstFragmentIsNotFolderButSecondFragmentIs(FragmentNode f1, FragmentNode f2) {
		return !(f1 instanceof FolderNode) && (f2 instanceof FolderNode);
	}
	
}
