package org.deft.gui.view.project;

import org.deft.gui.view.project.nodes.CategoryNode;
import org.deft.gui.view.project.nodes.FragmentNode;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;

/**
 * Sorter for the tree elements in the Project Explorer. If the elements to be
 * sorted are categories, the sorting is delegated to the ProjectViewCategorySorter.
 * If normal fragments are to be sorted, the sorting is delegated to the ProjectViewFragmentSorter.
 * 
 * @author abartho
 */
public class ProjectViewSorter extends ViewerSorter {

	private ViewerSorter categorySorter = new ProjectViewCategorySorter();
	private ViewerSorter fragmentSorter = new ProjectViewFragmentSorter();
	
	@Override
	public int compare(Viewer viewer, Object e1, Object e2) {
		
		if (categoriesToBeSorted(e1, e2)) {
			int result = categorySorter.compare(viewer, e1, e2);
			return result;
		}
		
		if (fragmentsToBeSorted(e1, e2)) {
			int result = fragmentSorter.compare(viewer, e1, e2);
			return result;
		}
		
		return super.compare(viewer, e1, e2);
		
	}
	
	
	private boolean categoriesToBeSorted(Object e1, Object e2) {
		return e1 instanceof CategoryNode && e2 instanceof CategoryNode;
	}
	
	
	private boolean fragmentsToBeSorted(Object e1, Object e2) {
		return e1 instanceof FragmentNode && e2 instanceof FragmentNode;
	}
	
}
