package org.deft.gui.view.project.dnd;

import org.deft.gui.view.project.nodes.CategoryNode;
import org.deft.gui.view.project.nodes.FragmentNode;
import org.deft.gui.view.project.nodes.ProjectNode;
import org.deft.gui.view.project.nodes.ProjectViewNode;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.FragmentType;
import org.deft.repository.datamodel.Project;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TransferData;
import org.eclipse.ui.navigator.CommonDropAdapter;

/**
 * The CategoryDropAssistant checks if a DnD operation in the Project View is valid,
 * when the target is a CategoryNode. Fragments can be moved to the CategoryNode of
 * the same project, within special consistency rules. These rules are defined by the repository. 
 */
public class CategoryDropAssistant extends AbstractDeftDropAssistant {

	@Override
	public IStatus validateDrop(Object target, int operation, TransferData transferType) {
		CategoryNode categoryNode = (CategoryNode)target;
		FragmentType type = categoryNode.getType();
		ProjectNode projectNode = (ProjectNode)categoryNode.getParent();
		Project project = projectNode.getProject();
		boolean isValidDrop = true;
		for (ProjectViewNode node : getSelectionHelper().getSelectedProjectViewNodes()) {
			//if one selected node is not a valid drop, isValidDrop will become false
			isValidDrop = isValidDrop && doValidateDrop(project, type, node);
		}
		return isValidDrop ? Status.OK_STATUS : Status.CANCEL_STATUS;
	}
	
	private boolean doValidateDrop(Project project, FragmentType type, ProjectViewNode node) {
		if (node instanceof FragmentNode) {
			Fragment fragment = ((FragmentNode)node).getFragment();
			boolean canContain = repository.canContainFragmentAtRoot(project, type, fragment);
			return canContain;
		} else {
			return false;
		}
	}

	@Override
	public IStatus handleDrop(CommonDropAdapter aDropAdapter,
			DropTargetEvent aDropTargetEvent, Object aTarget) {
		//all nodes in the selection are fragment nodes. This has been verified in the
		//validateDrop method
		for (ProjectViewNode node : getSelectionHelper().getSelectedProjectViewNodes()) {
			Fragment fragment = ((FragmentNode)node).getFragment();
			repository.moveToFolder(fragment, null);
		}
		return Status.OK_STATUS;
	}

}
