package org.deft.gui.view.project.dnd;

import org.deft.gui.view.project.nodes.FolderNode;
import org.deft.gui.view.project.nodes.FragmentNode;
import org.deft.gui.view.project.nodes.ProjectViewNode;
import org.deft.repository.datamodel.Folder;
import org.deft.repository.datamodel.Fragment;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TransferData;
import org.eclipse.ui.navigator.CommonDropAdapter;

/**
 * The FolderDropAssistant checks if a DnD operation in the Project View is valid,
 * when the target is a FolderNode. Fragments can be moved to a FolderNode
 * the same project and fragment type, within special consistency rules. 
 * These rules are defined by the repository. 
 */
public class FolderDropAssistant extends AbstractDeftDropAssistant {

	@Override
	public IStatus validateDrop(Object target, int operation, TransferData transferType) {
		FolderNode folderNode = (FolderNode)target;
		Folder folder = folderNode.getFolder();
		boolean isValidDrop = true;
		for (ProjectViewNode node : getSelectionHelper().getSelectedProjectViewNodes()) {
			//if one selected node is not a valid drop, isValidDrop will become false
			isValidDrop = isValidDrop && doValidateDrop(folder, node);
		}
		return isValidDrop ? Status.OK_STATUS : Status.CANCEL_STATUS;
	}
	
	private boolean doValidateDrop(Folder folder, ProjectViewNode node) {
		if (node instanceof FragmentNode) {
			Fragment fragment = ((FragmentNode)node).getFragment();
			boolean canContain = repository.canContainFragment(folder, fragment);
			return canContain;
		} else {
			return false;
		}
	}

	@Override
	public IStatus handleDrop(CommonDropAdapter aDropAdapter,
			DropTargetEvent aDropTargetEvent, Object aTarget) {
		FolderNode folderNode = (FolderNode)aTarget;
		Folder folder = folderNode.getFolder();
		//all nodes in the selection are fragment nodes. This has been verified in the
		//validateDrop method
		for (ProjectViewNode node : getSelectionHelper().getSelectedProjectViewNodes()) {
			Fragment fragment = ((FragmentNode)node).getFragment();
			repository.moveToFolder(fragment, folder);
		}
		return Status.OK_STATUS;
	}

}