package org.deft.gui.view.project.nodes;

import java.util.List;

import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.FragmentType;
import org.deft.repository.datamodel.Project;
import org.deft.util.ImageHelper;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;

/**
 * This class is used to display the categories (Chapters, Artifacts, ...) in the
 * project explorer.
 */
public class CategoryNode extends ProjectViewNode {
	
	private FragmentType type;
	
	public CategoryNode(FragmentType type) {
		this.type = type;
	}
	
	public FragmentType getType() {
		return type;
	}
	
	@Override
	public org.eclipse.swt.graphics.Image getIcon() {
		switch (type) {
			case CHAPTER:
				return ImageHelper.getDeftIcon("enabled/fragment-chapter.png");
			case ARTIFACT:
				return ImageHelper.getDeftIcon("enabled/fragment-codefile.png");
			case TUTORIAL:
				return ImageHelper.getDeftIcon("enabled/fragment-tutorial.png");
			default:
				return PlatformUI.getWorkbench().getSharedImages().getImage(ISharedImages.IMG_OBJ_FILE);
		}
	}
	
	
	@Override
	public String getText() {
		switch (type) {
		case CHAPTER:
			return "Chapter";
		case ARTIFACT:
			return "Artifact";
		case TUTORIAL:
			return "Tutorial";
		default:
			return "<unknown category>";
		}
	}
	
	
	@Override
	public boolean deletionAllowed() {
		return false;
	}
	
	
		
	@Override
	public ProjectNode getParent() {
		return (ProjectNode)super.getParent();
	}

	@Override
	public String getId() {
		String id = getParent().getId() + ":cat_" + getText();
		return id;
	}


	@Override
	protected ProjectViewNode createNodeFor(Object childContent) {
		FragmentNode node = ProjectViewNodeFactory.createNode((Fragment)childContent);
		return node;
	}

	@Override
	protected Object getContent() {
		return type;
	}

	@Override
	public List<? extends Object> getListOfChildContentToDisplay() {
		ProjectNode projectNode = getParent();
		Project project = projectNode.getProject();
		List<Fragment> content = project.getChildren(getType());
		return content;
	}
		
	
}
