package org.deft.gui.view.project.nodes;

import java.util.LinkedList;
import java.util.List;

import org.deft.repository.datamodel.FragmentType;
import org.deft.repository.datamodel.Project;
import org.deft.util.ImageHelper;
import org.eclipse.swt.graphics.Image;

/**
 * Node in the Project Explorer which represents a whole project.
 * A ProjectNode has CategoryNodes as children - Chapter, Artifact
 * and Tutorial.
 */
public class ProjectNode extends ProjectViewNode {

	private Project project;
	
	public ProjectNode(Project project) {
		this.project = project;
	}
	
	public Project getProject() {
		return project;
	}
	
	@Override
	public String getId() {
		return "proj_" + project.getId();
	}

	@Override
	public Image getIcon() {
		Image icon = ImageHelper.getDeftIcon("enabled/fragment-project.png");
		return icon;
	}

	@Override
	public String getText() {
		return project.getName();
	}

	@Override
	public boolean deletionAllowed() {
		return true;
	}
	

	@Override
	protected ProjectViewNode createNodeFor(Object content) {
		FragmentType fragType = (FragmentType)content;
		CategoryNode node = new CategoryNode(fragType);
		return node;
	}

	@Override
	protected Object getContent() {
		return getProject();
	}

	@Override
	protected List<? extends Object> getListOfChildContentToDisplay() {
		List<FragmentType> content = new LinkedList<FragmentType>();
		content.add(FragmentType.CHAPTER);
		content.add(FragmentType.ARTIFACT);
//		content.add(FragmentType.TUTORIAL);
		return content;
	}

}
