package org.deft.gui.view.project.nodes;

import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Folder;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.Tutorial;

public abstract class ProjectViewNodeFactory {

	public static FragmentNode createNode(Fragment fragment) {
		if (fragment instanceof Chapter) {
			return createNode((Chapter)fragment);
		} else if (fragment instanceof Artifact) {
			return createNode((Artifact)fragment);
		} else if (fragment instanceof Tutorial) {
			return createNode((Tutorial)fragment);
		} else if (fragment instanceof Folder) {
			return createNode((Folder)fragment);
		}
		return null;
	}
	
	
	public static ChapterNode createNode(Chapter chapter) {
		ChapterNode chapterNode = new ChapterNode(chapter);
		return chapterNode;
	}
	
	public static ArtifactNode createNode(Artifact artifact) {
		ArtifactNode artifactNode = new ArtifactNode(artifact);
		return artifactNode;
	}
	
	public static TutorialNode createNode(Tutorial tutorial) {
		TutorialNode tutorialNode = new TutorialNode(tutorial);
		return tutorialNode;
	}
	
	public static FolderNode createNode(Folder folder) {
		FolderNode folderNode = new FolderNode(folder);
		return folderNode;
	}
	
	public static ArtifactReferenceNodeForChapterNode createReferenceNodeForChapter(ArtifactReference reference) {
		ArtifactReferenceNodeForChapterNode referenceNode = new ArtifactReferenceNodeForChapterNode(reference);
		return referenceNode;
	}
	
	public static ArtifactReferenceNodeForArtifactNode createReferenceNodeForArtifact(ArtifactReference reference) {
		ArtifactReferenceNodeForArtifactNode referenceNode = new ArtifactReferenceNodeForArtifactNode(reference);
		return referenceNode;
	}
}
