package org.deft.gui.widgets;

import org.eclipse.jface.fieldassist.ContentProposalAdapter;
import org.eclipse.jface.fieldassist.IContentProposalListener;
import org.eclipse.jface.fieldassist.TextContentAdapter;
import org.eclipse.swt.widgets.Control;

/**
 * An Autocomplete field for hierarchically structured data.
 * 
 * This field can be configured with an array of proposals and a delimiter.
 * The proposals are the options that will be presented during autocomplete.
 * The delimiter denotes which character is used to delimit the individual elements
 * of the data hierarchy in their string representation. For example, a tree of
 * artifacts would have its elements displayed like project/folder1/folder2/artifact, 
 * where '/' is the delimiter.  
 */
public class HierarchyDataAutoCompleteField {
		
	private RegexContentProposalProvider proposalProvider;
	private ContentProposalAdapter adapter;

	public HierarchyDataAutoCompleteField(Control control) {
		proposalProvider = new RegexContentProposalProvider();
		adapter = new ContentProposalAdapter(control, new TextContentAdapter(), proposalProvider, null, null);
		adapter.setPropagateKeys(true);
		adapter.setProposalAcceptanceStyle(ContentProposalAdapter.PROPOSAL_REPLACE);
		//workaround: we need to specify characters so that the popup does not open when the textfield
		//text is set programmatically. Therefore we make the popup open on all characters starting from ASCII 32
		//(0 - 31 are control characters that we don't want in our text field)
		adapter.setAutoActivationCharacters(makeAllChars());
	}
	
	
	private char[] makeAllChars() {
		char[] c = new char[256];
		for (int i = 0; i <= 255; i++) {
			if (i != 27) {//ignore ESC key
				c[i] = (char)(i);
			}
		}
		return c;
	}
	
	public void addContentProposalListener(IContentProposalListener listener) {
		adapter.addContentProposalListener(listener);
	}
	

	public void setProposals(String[] proposals) {
		proposalProvider.setProposals(proposals);
	}
	
	public void setDelimiter(char delimiter) {
		proposalProvider.setDelimiter(delimiter);
	}
	
}