package org.deft.gui.widgets;

import java.util.LinkedList;
import java.util.List;

/**
 * Container that keeps track of the data item which has recently been selected
 * in the TreeAndAutoCompleteTextFieldComposite.
 */
public abstract class SelectedItemContainer {

	private Object selectedItem;
	private List<SelectedItemContainerListener> listeners = new LinkedList<SelectedItemContainerListener>();
	private boolean eventsActive = false;
	
	public void setSelectedItem(Object item) {
		Object oldItem = selectedItem;
		this.selectedItem = item;
		fireSelectionChanged(oldItem);
	}
	
	
	public void addSelectedItemContainerListener(SelectedItemContainerListener listener) {
		listeners.add(listener);
	}
	
	public void clearListeners() {
		listeners.clear();
	}
 
	private void fireSelectionChanged(Object oldSelection) {
		if (!eventsActive) {
			eventsActive = true;
			SelectedItemContainerEvent event = new SelectedItemContainerEvent(oldSelection);
			for (SelectedItemContainerListener listener : listeners) {
				listener.selectionChanged(event);
			}
			eventsActive = false;
		}
	}
	
	public Object getSelectedItem() {
		return this.selectedItem;
	}
	
	/**
	 * Returns the selected item in its string representation.
	 * 
	 * This is going to be displayed in the text field of the
	 * TreeAndAutoCompleteTextFieldComposite.
	 */
	public abstract String getSelectedItemAsString();
	
	/**
	 * Transforms a string representation to a concrete item.
	 * 
	 * The text field of the TreeAndAutoCompleteTextFieldComposite contains a string
	 * representation of the item to be selected. Subclasses must implement this method
	 * to get a concrete object for a given string representation, or null if there is
	 * no matching object.  
	 */
	public abstract Object getRepresentedItem(String text);
	
}
