package org.deft.guidance.tasks;

import org.deft.actions.SetArtifactReferenceCheckedAction;
import org.deft.integrator.EditorIntegrator;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Project;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

import uk.ac.abdn.csd.REL.CQA.model.Solution;


public class SetReferenceCheckedTask extends ReferenceSpecificTask {
	
	public SetReferenceCheckedTask(Solution solution, GuidanceOntologyAccessor accessor) {
		super(solution, accessor);
	}

	public String getReferenceId() {
		String referenceId = getShortValueForVariable("?x");
		return referenceId;
	}

	@Override
	public String getTaskMessage() {
		ArtifactReference reference = getDeftArtifactReference();
		Artifact artifact = reference.getArtifact();
		Chapter chapter = reference.getChapter();
		String integratorId = reference.getIntegratorId();
		EditorIntegrator integrator = getRepository().getExtensionService().getEditorIntegrator(integratorId);
		if (integrator != null) {
			return "Accept changes in " + integrator.getDescription() + " representation of artifact \"" + artifact.getName() + "\" in chapter \"" + chapter.getName() + "\"";
		} else {
			return "Accept changes in representation of artifact \"" + artifact.getName() + "\" in chapter \"" + chapter.getName() + "\"";
		}
	}
	
	@Override
	public ArtifactReference getDeftArtifactReference() {
		String referenceId = getReferenceId();
		ArtifactReference reference = getRepository().getArtifactReference(referenceId);
		return reference;
	}

	
	

	@Override
	public void execute(Project p) {
		IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
		SetArtifactReferenceCheckedAction action = new SetArtifactReferenceCheckedAction(window);
		ArtifactReference reference = getDeftArtifactReference();
		action.acceptChanges(reference);
	}
	

	@Override
	public String getProjectName() {
		return null;
	}
	
	

}
