package org.deft.guidance.tasks;

import org.deft.RepositoryFacade;
import org.deft.repository.datamodel.Project;

import uk.ac.abdn.csd.REL.CQA.model.BinaryRole;
import uk.ac.abdn.csd.REL.CQA.model.Solution;

/**
 * Tasks correspond to the tasks identified in the guidance ontology. For each
 * task in the ontology there should be a matching Java class.
 */
public abstract class Task {

	private Solution solution;
	private GuidanceOntologyAccessor accessor;

	public Task(Solution solution, GuidanceOntologyAccessor accessor) {
		this.solution = solution;
		this.accessor = accessor;
	}

	public String getName() {
		return getClass().getSimpleName();
	}

	public abstract String getTaskMessage();

	public abstract void execute(Project p);

	public abstract String getProjectName();

	protected String getValueForVariable(String variable) {
		for (BinaryRole binding : solution.bounding) {
			String boundVariable = binding.alpha;
			if (boundVariable.equals(variable)) {
				return binding.beta;
			}
		}
		return null;
	}

	protected String getShortValueForVariable(String variable) {
		String value = getValueForVariable(variable);
		if (value != null) {
			String shortValue = accessor.getSimpleName(value);
			return shortValue;
		}
		return "";
	}

	public String toString() {
		return getTaskMessage();
	}

	protected RepositoryFacade getRepository() {

		return RepositoryFacade.instance();

	}

	/**
	 * This method removes the URL-suffix. This is necessary to identify the
	 * tasks in the repository.
	 * 
	 * @param input task name with prefix
	 * @return normalized task name
	 */
	protected String normalizeName(String input) {

		return input.replaceAll("http://www.most-project.eu/guidanceSystem/",
				"");

	}

	@Override
	public boolean equals(Object o) {
		if (o instanceof Task) {
			Task t2 = (Task) o;
			return this.getTaskMessage().equals(t2.getTaskMessage());
		}
		return false;
	}

}
