package org.deft.guidance.tasks;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * The TaskList contains all tasks that can be performed at the moment.
 * 
 * It is possible to retrieve for both compulsory and optional tasks: - all
 * tasks, no matter if they have been fulfilled or are still open - only the
 * fulfilled tasks - only the open tasks
 */
public class TaskList {

	private Map<String, List<Task>> compulsoryTasks = new HashMap<String, List<Task>>();
	private Map<String, List<Task>> optionalTasks = new HashMap<String, List<Task>>();
	private Map<String, List<Task>> fulfilledCompulsoryTasks = new HashMap<String, List<Task>>();
	private Map<String, List<Task>> fulfilledOptionalTasks = new HashMap<String, List<Task>>();

	public void addCompulsoryTasks(List<Task> tasks) {
		for (Task task : tasks) {
			addCompulsoryTask(task);
		}
	}

	public void addCompulsoryTask(Task task) {
		List<Task> tasks = getExistingTaskListForTaskName(task.getName(),
				compulsoryTasks);
		tasks.add(task);
	}

	public void addOptionalTasks(List<Task> tasks) {
		for (Task task : tasks) {
			addOptionalTask(task);
		}
	}

	public void addOptionalTask(Task task) {
		List<Task> tasks = getExistingTaskListForTaskName(task.getName(),
				optionalTasks);
		tasks.add(task);
	}

	public void addFulfilledCompulsoryTasks(List<Task> tasks) {
		for (Task task : tasks) {
			addFulfilledCompulsoryTask(task);
		}
	}

	public void addFulfilledCompulsoryTask(Task task) {
		List<Task> tasks = getExistingTaskListForTaskName(task.getName(),
				fulfilledCompulsoryTasks);
		tasks.add(task);
	}

	public void addFulfilledOptionalTasks(List<Task> tasks) {
		for (Task task : tasks) {
			addFulfilledOptionalTask(task);
		}
	}

	public void addFulfilledOptionalTask(Task task) {
		List<Task> tasks = getExistingTaskListForTaskName(task.getName(),
				fulfilledOptionalTasks);
		tasks.add(task);
	}

	// / fulfilled and open ///

	public List<String> getNamesOfFulfilledAndOpenCompulsoryTasks() {
		List<String> names = getNamesOfTasks(compulsoryTasks);
		return names;
	}

	public List<Task> getFulfilledAndOpenCompulsoryTasks(String taskName) {
		List<Task> tasks = getExistingTaskListForTaskName(taskName,
				compulsoryTasks);
		return new LinkedList<Task>(tasks); // make copy
	}

	public List<String> getNamesOfFulfilledAndOpenOptionalTasks() {
		List<String> names = getNamesOfTasks(optionalTasks);
		return names;
	}

	public List<Task> getFulfilledAndOpenOptionalTasks(String taskName) {
		List<Task> tasks = getExistingTaskListForTaskName(taskName,
				optionalTasks);
		return new LinkedList<Task>(tasks); // make copy
	}

	// / fulfilled only ///

	public List<String> getNamesOfFulfilledCompulsoryTasks() {
		List<String> names = getNamesOfTasks(fulfilledCompulsoryTasks);
		return names;
	}

	public List<Task> getFulfilledCompulsoryTasks(String taskName) {
		List<Task> tasks = getExistingTaskListForTaskName(taskName,
				fulfilledCompulsoryTasks);
		return new LinkedList<Task>(tasks); // make copy
	}

	public List<String> getNamesOfFulfilledOptionalTasks() {
		List<String> names = getNamesOfTasks(fulfilledOptionalTasks);
		return names;
	}

	public List<Task> getFulfilledOptionalTasks(String taskName) {
		List<Task> tasks = getExistingTaskListForTaskName(taskName,
				fulfilledOptionalTasks);
		return new LinkedList<Task>(tasks); // make copy
	}

	private List<String> getNamesOfTasks(Map<String, List<Task>> taskMap) {
		List<String> names = new LinkedList<String>();
		for (String name : taskMap.keySet()) {
			List<Task> tasks = getExistingTaskListForTaskName(name, taskMap);
			if (!tasks.isEmpty()) {
				names.add(name);
			}
		}
		return names;
	}

	// / open tasks///

	public List<String> getNamesOfOpenCompulsoryTasks() {
		List<String> openNames = new LinkedList<String>();
		List<String> allNames = getNamesOfFulfilledAndOpenCompulsoryTasks();
		for (String name : allNames) {
			if (someCompulsoryTasksOfThisNameAreStillOpen(name)) {
				openNames.add(name);
			}
		}
		return openNames;
	}

	private boolean someCompulsoryTasksOfThisNameAreStillOpen(String taskName) {
		List<Task> openTasks = getOpenCompulsoryTasks(taskName);
		boolean someTasksStillOpen = !openTasks.isEmpty();
		return someTasksStillOpen;
	}

	public List<Task> getOpenCompulsoryTasks(String taskName) {
		List<Task> tasks = getFulfilledAndOpenCompulsoryTasks(taskName);
		List<Task> fulfilledTasks = getFulfilledCompulsoryTasks(taskName);
		tasks.removeAll(fulfilledTasks);
		return tasks;
	}

	public List<String> getNamesOfOpenOptionalTasks() {
		List<String> openNames = new LinkedList<String>();
		List<String> allNames = getNamesOfFulfilledAndOpenOptionalTasks();
		for (String name : allNames) {
			if (someOptionalTasksOfThisNameAreStillOpen(name)) {
				openNames.add(name);
			}
		}
		return openNames;
	}

	private boolean someOptionalTasksOfThisNameAreStillOpen(String taskName) {
		List<Task> openTasks = getOpenCompulsoryTasks(taskName);
		boolean someTasksStillOpen = !openTasks.isEmpty();
		return someTasksStillOpen;
	}

	public List<Task> getOpenOptionalTasks(String taskName) {
		List<Task> tasks = getFulfilledAndOpenOptionalTasks(taskName);
		List<Task> fulfilledTasks = getFulfilledOptionalTasks(taskName);
		tasks.removeAll(fulfilledTasks);
		return tasks;
	}

	private List<Task> getExistingTaskListForTaskName(String taskName,
			Map<String, List<Task>> taskMap) {
		if (!taskMap.keySet().contains(taskName)) {
			taskMap.put(taskName, new LinkedList<Task>());
		}
		return taskMap.get(taskName);
	}

	public String toString() {
		StringBuilder sb = new StringBuilder();
		if (doCompulsoryTasksExist()) {
			setupCompulsoryTaskOutputString(sb);
			if (doOptionalTasksExist()) {
				sb.append("---\n");
			}
		}
		if (doOptionalTasksExist()) {
			setupOptionalTaskOutputString(sb);
		}

		sb.append("\n=======================\n");

		if (doFulfilledCompulsoryTasksExist()) {
			setupFulfilledCompulsoryTaskOutputString(sb);
			if (doFulfilledOptionalTasksExist()) {
				sb.append("---\n");
			}
		}
		if (doFulfilledOptionalTasksExist()) {
			setupFulfilledOptionalTaskOutputString(sb);
		}

		sb.append("\n=======================\n");

		if (doOpenCompulsoryTasksExist()) {
			setupOpenCompulsoryTaskOutputString(sb);
			if (doOpenOptionalTasksExist()) {
				sb.append("---\n");
			}
		}
		if (doOpenOptionalTasksExist()) {
			setupOpenOptionalTaskOutputString(sb);
		}
		return sb.toString();
	}

	private boolean doCompulsoryTasksExist() {
		return !getNamesOfFulfilledAndOpenCompulsoryTasks().isEmpty();
	}

	private void setupCompulsoryTaskOutputString(StringBuilder sb) {
		sb.append("All Compulsory Tasks:\n");
		for (String taskName : getNamesOfFulfilledAndOpenCompulsoryTasks()) {
			List<Task> tasks = getFulfilledAndOpenCompulsoryTasks(taskName);
			if (!tasks.isEmpty()) {
				sb.append(taskName + ": " + tasks + "\n");
			}
		}
	}

	private boolean doFulfilledCompulsoryTasksExist() {
		return !getNamesOfFulfilledCompulsoryTasks().isEmpty();
	}

	private void setupFulfilledCompulsoryTaskOutputString(StringBuilder sb) {
		sb.append("Fulfilled Compulsory Tasks:\n");
		for (String taskName : getNamesOfFulfilledCompulsoryTasks()) {
			List<Task> tasks = getFulfilledCompulsoryTasks(taskName);
			if (!tasks.isEmpty()) {
				sb.append(taskName + ": " + tasks + "\n");
			}
		}
	}

	private boolean doOpenCompulsoryTasksExist() {
		return !getNamesOfOpenCompulsoryTasks().isEmpty();
	}

	private void setupOpenCompulsoryTaskOutputString(StringBuilder sb) {
		sb.append("Open Compulsory Tasks:\n");
		for (String taskName : getNamesOfOpenCompulsoryTasks()) {
			List<Task> tasks = getOpenCompulsoryTasks(taskName);
			if (!tasks.isEmpty()) {
				sb.append(taskName + ": " + tasks + "\n");
			}
		}
	}

	private boolean doOptionalTasksExist() {
		return !getNamesOfFulfilledAndOpenOptionalTasks().isEmpty();
	}

	private void setupOptionalTaskOutputString(StringBuilder sb) {
		sb.append("All Optional Tasks:\n");
		for (String taskName : getNamesOfFulfilledAndOpenOptionalTasks()) {
			List<Task> tasks = getFulfilledAndOpenOptionalTasks(taskName);
			if (!tasks.isEmpty()) {
				sb.append(taskName + ": " + tasks + "\n");
			}
		}
	}

	private boolean doFulfilledOptionalTasksExist() {
		return !getNamesOfFulfilledOptionalTasks().isEmpty();
	}

	private void setupFulfilledOptionalTaskOutputString(StringBuilder sb) {
		sb.append("Fulfilled Optional Tasks:\n");
		for (String taskName : getNamesOfFulfilledOptionalTasks()) {
			List<Task> tasks = getFulfilledOptionalTasks(taskName);
			if (!tasks.isEmpty()) {
				sb.append(taskName + ": " + tasks + "\n");
			}
		}
	}

	private boolean doOpenOptionalTasksExist() {
		return !getNamesOfOpenOptionalTasks().isEmpty();
	}

	private void setupOpenOptionalTaskOutputString(StringBuilder sb) {
		sb.append("Open Optional Tasks:\n");
		for (String taskName : getNamesOfOpenOptionalTasks()) {
			List<Task> tasks = getOpenOptionalTasks(taskName);
			if (!tasks.isEmpty()) {
				sb.append(taskName + ": " + tasks + "\n");
			}
		}
	}
}
