package org.deft.guidance.tasks;

import java.util.List;
import java.util.Set;

import org.semanticweb.owl.model.OWLClass;
import org.semanticweb.owl.model.OWLOntology;

import uk.ac.abdn.csd.REL.CQA.model.Solution;

public class TaskListProvider {

	private GuidanceOntologyAccessor accessor;

	public TaskListProvider(OWLOntology ontology) {
		try {
			// OWLOntologyManager manager =
			// OWLManager.createOWLOntologyManager();
			// OWLOntology ontology =
			// manager.loadOntologyFromPhysicalURI(ontologyUri);
			this.accessor = new GuidanceOntologyAccessor(ontology);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * This method provides all open tasks from an ontology
	 * 
	 * @return TaskList
	 */
	public TaskList getTaskList() {
		DocumentationTaskFactory factory = new DocumentationTaskFactory(
				accessor);
		TaskQueryExecutor executor = new TaskQueryExecutor(accessor);
		TaskList taskList = new TaskList();

		addAllCompulsoryTasks(taskList, executor, factory);
		removeAllFulfilledCompulsoryTasks(taskList, executor, factory);
		addAllOptionalTasks(taskList, executor, factory);
		removeAllFulfilledOptionalTasks(taskList, executor, factory);

		return taskList;

	}

	private void addAllCompulsoryTasks(TaskList taskList,
			TaskQueryExecutor executor, TaskFactory factory) {
		OWLClass taskTypeClass = accessor.getOwlClass("TaskType");
		List<OWLClass> taskTypes = accessor
				.getDirectlySubsumedClasses(taskTypeClass);
		for (OWLClass taskType : taskTypes) {
			Set<Solution> solutions = executor
					.queryForCompulsoryTasks(taskType);
			List<Task> tasks = factory.createTasks(taskType, solutions);
			taskList.addCompulsoryTasks(tasks);
		}
	}

	private void addAllOptionalTasks(TaskList taskList,
			TaskQueryExecutor executor, TaskFactory factory) {
		OWLClass taskTypeClass = accessor.getOwlClass("TaskType");
		List<OWLClass> taskTypes = accessor
				.getDirectlySubsumedClasses(taskTypeClass);
		for (OWLClass taskType : taskTypes) {
			Set<Solution> solutions = executor.queryForOptionalTasks(taskType);
			List<Task> tasks = factory.createTasks(taskType, solutions);
			taskList.addOptionalTasks(tasks);
		}
	}

	private void removeAllFulfilledCompulsoryTasks(TaskList taskList,
			TaskQueryExecutor executor, TaskFactory factory) {
		OWLClass taskTypeClass = accessor.getOwlClass("TaskType");
		List<OWLClass> taskTypes = accessor
				.getDirectlySubsumedClasses(taskTypeClass);
		for (OWLClass taskType : taskTypes) {
			Set<Solution> solutions = executor
					.queryForFulfilledCompulsoryTasks(taskType);
			List<Task> tasks = factory.createTasks(taskType, solutions);
			taskList.addFulfilledCompulsoryTasks(tasks);
		}
	}

	private void removeAllFulfilledOptionalTasks(TaskList taskList,
			TaskQueryExecutor executor, TaskFactory factory) {
		OWLClass taskTypeClass = accessor.getOwlClass("TaskType");
		List<OWLClass> taskTypes = accessor
				.getDirectlySubsumedClasses(taskTypeClass);
		for (OWLClass taskType : taskTypes) {
			Set<Solution> solutions = executor
					.queryForFulfilledOptionalTasks(taskType);
			List<Task> tasks = factory.createTasks(taskType, solutions);
			taskList.addFulfilledOptionalTasks(tasks);
		}
	}
}
