package org.deft.guidance.tasks;

import java.util.Set;

import org.semanticweb.owl.model.OWLClass;

import uk.ac.abdn.csd.REL.CQA.model.Query;
import uk.ac.abdn.csd.REL.CQA.model.Solution;

/**
 * The TaskQueryExecutor can execute queries that are needed to create
 * the list of possible refinement tasks and put the results in a TaskList.
 */
public class TaskQueryExecutor {

    private GuidanceOntologyAccessor accessor;
    
    public TaskQueryExecutor(GuidanceOntologyAccessor accessor) {
        this.accessor = accessor;
    }
    
     
    public Set<Solution> queryForCompulsoryTasks(OWLClass taskType) {
		Set<Solution> solutions = queryForTasks(taskType, "hasCompulsoryTask");
		return solutions;
    }
    
    
    public Set<Solution> queryForCompulsoryTasks(OWLClass taskType, String role) {
    	if (isEmptyString(role)) {
    		Set<Solution> solutions = queryForTasks(taskType, "hasCompulsoryTask");
    		return solutions;
    	} else {
    		Set<Solution> solutions = queryForTasks(taskType, "hasCompulsoryTask", role);
    		return solutions;
    	}
    }
    
    private boolean isEmptyString(String s) {
    	return s == null || s.equals("");
    }
    
    
    public Set<Solution> queryForOptionalTasks(OWLClass taskType, String role) {
    	if (isEmptyString(role)) {
    		Set<Solution> solutions = queryForTasks(taskType, "hasOptionalTask");
    		return solutions;
    	} else {
    		Set<Solution> solutions = queryForTasks(taskType, "hasOptionalTask", role);
    		return solutions;
    	}
    }
    
    public Set<Solution> queryForOptionalTasks(OWLClass taskType) {
		Set<Solution> solutions = queryForTasks(taskType, "hasOptionalTask");
		return solutions;
    }
    
    
    private Set<Solution> queryForTasks(OWLClass taskType, String preconditionProperty, String role) {
    	Query query = createStandardTaskQuery(taskType, preconditionProperty);
        
    	String sRole = accessor.getUriString(role);
    	String sPerformedByProperty = accessor.getUriString("performedBy");
        query.addConceptQuery(sRole, "?r");
        query.addRoleQuery(sPerformedByProperty, "?y", "?r");

		Set<Solution> solutions = getSolutionsForQuery(query);
		return solutions;
    }
    
    
    private Set<Solution> queryForTasks(OWLClass taskType, String preconditionProperty) {
    	Query query = createStandardTaskQuery(taskType, preconditionProperty);
    	
		Set<Solution> solutions = getSolutionsForQuery(query);
		return solutions;
    }
    
    
    private Query createStandardTaskQuery(OWLClass taskType, String preconditionProperty) {
    	String sState = accessor.getUriString("Artefact");
    	String sPreconditionProperty = accessor.getUriString(preconditionProperty);
    	String sTaskType = taskType.getURI().toString();
        Query query = new Query();
        query.addVariable("?x");
        query.addConceptQuery(sState, "?x");
        query.addConceptQuery(sTaskType, "?y");
        query.addRoleQuery(sPreconditionProperty, "?x", "?y");
        return query;
    }
    
    
    
    public Set<Solution> queryForFulfilledCompulsoryTasks(OWLClass taskType) {
    	Set<Solution> solutions = queryForFulfilledTasks(taskType, "hasCompulsoryTask");
        return solutions;
    }
    
    
    public Set<Solution> queryForFulfilledOptionalTasks(OWLClass taskType) {
    	Set<Solution> solutions = queryForFulfilledTasks(taskType, "hasOptionalTask");
        return solutions;
    }
    
    
    public Set<Solution> queryForFulfilledTasks(OWLClass taskType, String preconditionProperty) {
    	Query query = createStandardTaskQuery(taskType, preconditionProperty);
		query.addVariable("?y");
		Set<Solution> solutions = getSolutionsForQuery(query);
		return solutions;
	}
    
    private Set<Solution> getSolutionsForQuery(Query query) {
		Set<Solution> solutions = null;
		try {
			solutions = accessor.getReasoner().CQA(query);
		} catch (CloneNotSupportedException e) {
			e.printStackTrace();
		}
		return solutions;
    }
    

//    /**
//     * Get all possible tasks. Tasks that have precondition but do not have
//     * postconditions satisfied yet.
//     * 
//     * @param sTaskType
//     * @param bAtom
//     * @return
//     */
//    private Set <QueryResult> getPossibleCombinations (final String sTaskType, final String sRole, final boolean bAtom)
//    {
//      final Set <QueryResult> ret = new HashSet <QueryResult> ();
//      final Query aQuery;
//      {
//        final String tbox = m_aMergedOntologyURI + "#";
//        final String abox = IOntologyConverter.URI_GUIDANCE_ARTEFACTS + "#";
//
//        aQuery = new Query ();
//        if (bAtom)
//        {
//          s_aLogger.info ("ATOM Query for task: " + tbox + sTaskType);
//          aQuery.addVariable ("?i");
//          aQuery.addVariable ("?m");
//
//          // Do not add a variable for y, since this would need concrete
//          // individuals for it (no OWA)!
//          // aQuery.addVariable ("?y");
//          aQuery.addConceptQuery (abox + "__MODELLING_OBJECT__", "?i");
//          aQuery.addConceptQuery (abox + "__MODEL__", "?m");
//          aQuery.addRoleQuery (tbox + "preconditionOf", "?i", "?y");
//          aQuery.addConceptQuery (tbox + sTaskType, "?y");
//          aQuery.addRoleQuery (abox + "__contains__", "?m", "?i");
//        }
//        else
//        {
//          s_aLogger.info ("CONTAINER Query for task: " + tbox + sTaskType);
//          aQuery.addVariable ("?m");
//          aQuery.addConceptQuery (abox + "__MODEL__", "?m");
//          aQuery.addRoleQuery (tbox + "preconditionOf", "?m", "?y");
//          aQuery.addConceptQuery (tbox + sTaskType, "?y");
//        }
//
//        // adde for 'roles'
//        if (!sRole.isEmpty ())
//        {
//          aQuery.addConceptQuery (tbox + sRole, "?r");
//          aQuery.addRoleQuery (tbox + "performedBy", "?y", "?r");
//        }
//
//        HashSet <Solution> aSolutions;
//        try
//        {
//          aSolutions = m_aReasoner.CQA (aQuery);
//          if (CGuidanceEngine.DEBUG_ENABLED)
//          {
//            s_aLogger.debug ("POSSIBLE RESULTS");
//            OWLUtils.printResult (aSolutions);
//          }
//
//          for (final Solution solution : aSolutions)
//          {
//            final Map <String, String> aResultSet = new HashMap <String, String> ();
//            for (final BinaryRole binding : solution.bounding)
//            {
//              aResultSet.put (binding.alpha, binding.beta);
//            }
//            if (bAtom)
//              ret.add (new QueryResult (sTaskType,
//                                        stripNamespace (aResultSet.get ("?i")),
//                                        stripNamespace (aResultSet.get ("?m"))));
//            else
//              ret.add (new QueryResult (sTaskType, null, stripNamespace (aResultSet.get ("?m"))));
//          }
//        }
//        catch (final CloneNotSupportedException ex)
//        {
//          s_aLogger.error (ex);
//        }
//        catch (final Exception ex)
//        {
//          s_aLogger.error (ex);
//        }
//      }
//      return ret;
//    }
//
//  /**
//     * Get all done tasks. Tasks that have postconditions.
//     * 
//     * @param sTaskType
//     * @param bAtom
//     * @return
//     */
//    private Set <QueryResult> getClosedCombinations (final String sTaskType, final String sRole, final boolean bAtom)
//    {
//      final Set <QueryResult> ret = new HashSet <QueryResult> ();
//      final Query aQuery;
//      {
//        final String tbox = m_aMergedOntologyURI + "#";
//        final String abox = IOntologyConverter.URI_GUIDANCE_ARTEFACTS + "#";
//
//        aQuery = new Query ();
//        if (bAtom)
//        {
//          aQuery.addVariable ("?i");
//          aQuery.addVariable ("?m");
//          aQuery.addVariable ("?y");
//          aQuery.addVariable ("?t");
//
//          aQuery.addConceptQuery (abox + "__MODELLING_OBJECT__", "?i");
//          aQuery.addConceptQuery (abox + "__MODEL__", "?m");
//          aQuery.addRoleQuery (tbox + "preconditionOf", "?i", "?y");
//          aQuery.addConceptQuery (tbox + "Artefact", "?t");
//          aQuery.addRoleQuery (abox + "__contains__", "?m", "?i");
//        }
//        else
//        {
//          aQuery.addVariable ("?m");
//          aQuery.addVariable ("?y");
//          aQuery.addVariable ("?t");
//          aQuery.addConceptQuery (abox + "__MODEL__", "?m");
//          aQuery.addRoleQuery (tbox + "preconditionOf", "?m", "?y");
//          aQuery.addConceptQuery (tbox + "Artefact", "?t");
//        }
//        aQuery.addRoleQuery (tbox + "hasPostcondition", "?y", "?t");
//        aQuery.addConceptQuery (tbox + sTaskType, "?y");
//
//        // added for 'roles'
//        if (!sRole.isEmpty ())
//        {
//          aQuery.addConceptQuery (tbox + sRole, "?r");
//          aQuery.addRoleQuery (tbox + "performedBy", "?y", "?r");
//        }
//
//        HashSet <Solution> aSolutions;
//        try
//        {
//          aSolutions = m_aReasoner.CQA (aQuery);
//          if (CGuidanceEngine.DEBUG_ENABLED)
//          {
//            s_aLogger.debug ("EXISTING RESULTS");
//            OWLUtils.printResult (aSolutions);
//          }
//          for (final Solution solution : aSolutions)
//          {
//            final Map <String, String> aResultSet = new HashMap <String, String> ();
//            if (CGuidanceEngine.DEBUG_ENABLED)
//              OWLUtils.printResult (aSolutions);
//            for (final BinaryRole binding : solution.bounding)
//            {
//              aResultSet.put (binding.alpha, binding.beta);
//            }
//            if (bAtom)
//              ret.add (new QueryResult (stripNamespace (sTaskType),
//                                        stripNamespace (aResultSet.get ("?i")),
//                                        stripNamespace (aResultSet.get ("?m"))));
//            else
//              ret.add (new QueryResult (stripNamespace (sTaskType), null, stripNamespace (aResultSet.get ("?m"))));
//          }
//        }
//        catch (final CloneNotSupportedException ex)
//        {
//          s_aLogger.error (ex);
//        }
//      }
//      return ret;
//    }
//
//    private Set <QueryResult> getDifference (final Set <QueryResult> aPossible, final Set <QueryResult> aExisting)
//    {
//      final Set <QueryResult> ret = new HashSet <QueryResult> (aPossible);
//      ret.removeAll (aExisting);
//      return ret;
//    }

    
}
