package org.deft.guidance.views;

import java.util.List;

import org.deft.guidance.GuidanceTaskListManager;
import org.deft.guidance.TaskListUpdateListener;
import org.deft.guidance.action.RefreshTaskListAction;
import org.deft.guidance.tasks.Task;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.resource.FontRegistry;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ITableFontProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.TableLayout;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.ui.part.ViewPart;

/**
 * The TaskView shows all open tasks in a list. Tasks can be executed via double click.
 */
public class TaskView extends ViewPart {

	/**
	 * The ID of the view as specified by the extension.
	 */
	public static final String ID = "org.deft.guidance.views.TaskView";
	private TreeColumn taskNameColumn;
	private TreeViewer viewer;
	private GuidanceTaskListManager manager = GuidanceTaskListManager.instance();
	private TaskListUpdateListener taskListUpdateListener = new TaskViewTaskUpdateListener();


	/**
	 * Creates visible components
	 */
	public void createPartControl(Composite parent) {
		parent.setLayout(new FillLayout());
		final Tree tree = new Tree(parent, SWT.H_SCROLL | SWT.V_SCROLL
				| SWT.MULTI | SWT.FULL_SELECTION);
		tree.setLinesVisible(true);

		TableLayout layout = new TableLayout();
		tree.setLayout(layout);
		tree.setHeaderVisible(true);

		taskNameColumn = new TreeColumn(tree, SWT.NONE);
		taskNameColumn.setText("Task");
		taskNameColumn.setWidth(600);

		viewer = new TreeViewer(tree);
		viewer.setContentProvider(new ContentProvider());

		viewer.setLabelProvider(new LabelProvider());
		manager.updateTaskList();
		viewer.setInput(manager.getTasks());
		viewer.addDoubleClickListener(new TaskViewDoubleClickListener());
		ViewerComparator comp = new TaskListComparator();
		viewer.setComparator(comp);
		
		IToolBarManager tbm = getViewSite().getActionBars().getToolBarManager();
		tbm.add(new RefreshTaskListAction());
		
		manager.addTaskListUpdateListener(taskListUpdateListener);
	}



	@Override
	public void setFocus() {
	}

	
	private class ContentProvider implements ITreeContentProvider {

		public Object[] getChildren(Object parentElement) {
			return new Object[0];
		}

		public Object getParent(Object element) {
			return null;
		}

		public boolean hasChildren(Object element) {
			if (element instanceof String) {
				return true;
			}
			return false;
		}

		@SuppressWarnings("unchecked")
		public Object[] getElements(Object inputElement) {

			List<Task> al = (List<Task>) inputElement;

			return al.toArray();
		}

		public void dispose() {
			viewer = null;
		}

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {

		}

	};

	private class LabelProvider implements ITableLabelProvider,
			ITableFontProvider {

		Font boldFont;

		public LabelProvider() {
			FontRegistry reg = new FontRegistry();
			boldFont = reg.getBold("");
		}

		public Image getColumnImage(Object element, int columnIndex) {
			return null;
		}

		public String getColumnText(Object element, int columnIndex) {

			Task task = (Task) element;
			return getDescription(task);

		}

		public void addListener(ILabelProviderListener listener) {
		}

		public void dispose() {
		}

		public boolean isLabelProperty(Object element, String property) {
			return false;
		}

		public void removeListener(ILabelProviderListener listener) {
		}

		public Font getFont(Object element, int columnIndex) {
			if (element instanceof String && columnIndex == 0) {
				return boldFont;
			}
			return null;
		}

		private String getDescription(Task task) {
			return task.getTaskMessage();
		}

	}
	
	
	
	
	public void dispose() {
		manager.removeTaskListUpdateListener(taskListUpdateListener);
	}
	
	
	
	private class TaskViewTaskUpdateListener implements TaskListUpdateListener {

		@Override
		public void updated(List<Task> tasks) {
			viewer.setInput(tasks);
		}
		
	}

}