package org.deft.integrator;

import java.util.LinkedList;
import java.util.List;

import org.deft.RepositoryFacade;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.ArtifactReferenceUpdateType;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.event.ArtifactReferenceUpdateEvent;
import org.deft.repository.event.FragmentUpdateEvent;
import org.deft.repository.event.SimpleRepositoryUpdateListener;
import org.deft.repository.integrator.ArtifactReferencePosition;

public class ChapterDocRepositoryListener extends SimpleRepositoryUpdateListener {

	private EditorChapterDocument chapterDocument;
	private RepositoryFacade repository = RepositoryFacade.instance();
	
	public ChapterDocRepositoryListener(EditorChapterDocument chapterDocument) {
		this.chapterDocument = chapterDocument;
	}
	
	@Override
	public void repositoryUpdated() {
	}

	@Override
	public void artifactUpdated(FragmentUpdateEvent event) {
		Artifact artifact = (Artifact)event.getFragment();
		Chapter chapter = chapterDocument.getChapter();
		List<ArtifactReference> referencesToUpdate = getReferencesToUpdate(artifact, chapter);
		for (ArtifactReference ref : referencesToUpdate) {
			ArtifactReferencePosition pos = chapterDocument.removeRepresentation(ref);
			//if representation has been removed by user, but document has not been saved,
			//pos will be null (because the representation to be removed has not been found)
			if (pos != null) {
				chapterDocument.insertRepresentation(ref, pos);
			}
		}
	}
	

	private List<ArtifactReference> getReferencesToUpdate(Artifact artifact, Chapter chapter) {
		List<ArtifactReference> references = new LinkedList<ArtifactReference>();
		List<ArtifactReference> artRefs = repository.getArtifactReferences(artifact);
		List<ArtifactReference> chapRefs = repository.getArtifactReferences(chapter);
		for (ArtifactReference ref : chapRefs) {
			if (ref.getUpdateType() != ArtifactReferenceUpdateType.NOUPDATE  && artRefs.contains(ref)) {
				references.add(ref);
			}
		}
		return references;
	}
	
	
	@Override
	public void fragmentRemoved(FragmentUpdateEvent event) {
		Chapter chapter = chapterDocument.getChapter();
		List<ArtifactReference> referencesInChapter = chapter.getArtifactReferences();
		for (ArtifactReference ref : referencesInChapter) {
			if (ref.getUpdateType() == ArtifactReferenceUpdateType.MISSINGARTIFACT) {
				ArtifactReferencePosition pos = chapterDocument.removeRepresentation(ref);
				//pos can be null if there is already an error message and the original
				//integrator cannot find/remove the content it expects to find. 
				if (pos != null) {
					chapterDocument.insertRepresentation(ref, pos);
				}
			}
		}
	}
	
	@Override
	public void artifactReferenceUnchecked(ArtifactReferenceUpdateEvent event) {
		ArtifactReference ref = event.getReference();
		ArtifactReferencePosition pos = chapterDocument.removeRepresentation(ref);
		//if representation has been removed by user, but document has not been saved,
		//pos will be null (because the representation to be removed has not been found)
		if (pos != null) {
			chapterDocument.insertRepresentation(ref, pos);
		}
	}
}
