package org.deft.integrator;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;

public class EclipseEditorIntegratorRegistry implements EditorIntegratorRegistry {
	
	private static final String EXTENSION_POINT_NAME = "org.deft.editorintegrator";
		
	private Map<String, IntegratorConfig> registeredIntegrators = new HashMap<String, IntegratorConfig>(); 

	public EclipseEditorIntegratorRegistry() {
		getAndCacheRegisteredIntegrators();
	}
	
	private void getAndCacheRegisteredIntegrators() {
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint point = registry.getExtensionPoint(EXTENSION_POINT_NAME);
		for (IConfigurationElement ce : point.getConfigurationElements()) {
			IntegratorConfig config = new IntegratorConfig(ce);
			registeredIntegrators.put(config.getId(), config);
		}
	}
	
	@Override
	public EditorIntegrator getIntegrator(String integratorId) {
		IntegratorConfig config = registeredIntegrators.get(integratorId);
		if (config == null) {
			return null;
		}
		EditorIntegrator integrator = config.getIntegrator();
		integrator.resetIntegratorConfiguration();
		return integrator;
	}
	
	@Override
	public List<String> getIntegratorIds() {
		List<String> integratorIds = new LinkedList<String>(registeredIntegrators.keySet());
		return integratorIds;
	}
	

	@Override
	public List<EditorIntegrator> getIntegrators() {
		List<EditorIntegrator> integrators = new LinkedList<EditorIntegrator>();
		for (String integratorId : getIntegratorIds()) {
			integrators.add(getIntegrator(integratorId));
		}
		return integrators;
	}
	
	
	private class IntegratorConfig {
	
		private IConfigurationElement ce;
		private EditorIntegrator integrator;
		
		public IntegratorConfig(IConfigurationElement ce) {
			this.ce = ce;
		}
		
		public String getId() {
			String id = ce.getAttribute("id");
			return id;
		}
		
		public EditorIntegrator getIntegrator() {
			if (integrator == null) {
				try {
					String integratorPropertyInExtensionPoint = "class";
					integrator = (EditorIntegrator)ce.createExecutableExtension(
							integratorPropertyInExtensionPoint);
				} catch (CoreException e) {
					e.printStackTrace();
				}
			}
			return integrator;
		}	
	}



	
}
