package org.deft.operation;

import java.io.ByteArrayInputStream;

import org.deft.operation.exception.DeftInvalidOperationChainException;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.OwlRepresentation;
import org.deft.representation.bytearray.ByteArrayRepresentation;
import org.deft.requirementsontology.OntologyAccessor;
import org.semanticweb.owl.model.OWLOntology;

public class ByteArray2OntologyOperation implements Operation {

	@Override
	public OwlRepresentation executeOperation(ArtifactRepresentation representation, 
			ArtifactType type, OperationConfiguration config) {
		byte[] byteContent = checkAndUnwrap(representation);
		OntologyAccessor oa = new OntologyAccessor(new ByteArrayInputStream(byteContent));
		OWLOntology ontology = oa.getOwlOntology();
		OwlRepresentation or = new OwlRepresentation(ontology);
		return or;
	}
	
	
	private byte[] checkAndUnwrap(ArtifactRepresentation rep) {
		if (rep.getType().equals(ByteArrayRepresentation.TYPE)) {
			ByteArrayRepresentation bar = (ByteArrayRepresentation)rep;
			return bar.getByteArray();
		} else {
			throw new DeftInvalidOperationChainException();
		}
	}
	
	

	@Override
	public OperationConfiguration createConfiguration() {
		return new EmptyOperationConfiguration();
	}


	@Override
	public String getInputRepresentationType() {
		return ByteArrayRepresentation.TYPE;
	}


	@Override
	public String getResultingRepresentationType() {
		return OwlRepresentation.TYPE;
	}

}
