package org.deft.operation;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;

public class EclipseSingleOperationRegistry implements SingleOperationRegistry {
	
	private static final String SINGLEOPERATION_EXTENSION_POINT_NAME = "org.deft.operation.singleoperation";
		
	private Map<String, OperationConfig> registeredOperations = new HashMap<String, OperationConfig>(); 

	public EclipseSingleOperationRegistry() {
		getAndCacheRegisteredSources();
	}
	
	private void getAndCacheRegisteredSources() {
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint point = registry.getExtensionPoint(SINGLEOPERATION_EXTENSION_POINT_NAME);
		for (IConfigurationElement ce : point.getConfigurationElements()) {
			OperationConfig config = new OperationConfig(ce);
			registeredOperations.put(config.getId(), config);
		}
	}
	
	@Override
	public Operation getOperation(String operationId) {
		OperationConfig config = registeredOperations.get(operationId);
		if (config == null) {
			return null;
		}
		Operation operation = config.getOperation();
		return operation;
	}
	
	public List<String> getOperationIds() {
		List<String> operationIds = new LinkedList<String>(registeredOperations.keySet());
		return operationIds;
	}
	
	
	
	private class OperationConfig {
	
		private IConfigurationElement ce;
		private Operation operation;
		
		public OperationConfig(IConfigurationElement ce) {
			this.ce = ce;
		}
		
		public String getId() {
			String id = ce.getAttribute("id");
			return id;
		}
		
		public Operation getOperation() {
			if (operation == null) {
				try {
					String operationPropertyInExtensionPoint = "class";
					operation = (Operation)ce.createExecutableExtension(
							operationPropertyInExtensionPoint);
				} catch (CoreException e) {
					e.printStackTrace();
				}
			}
			return operation;
		}	
	}
	
}
