package org.deft.operation;

import org.deft.operation.exception.DeftInvalidOperationChainException;
import org.deft.operation.exception.DeftInvalidOperationConfigurationException;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.OwlRepresentation;
import org.deft.representation.table.TableRepresentation;
import org.deft.representation.table.data.Table;
import org.deft.requirementsontology.IndividualDataVisualizer;
import org.deft.requirementsontology.OntologyAccessor;
import org.deft.requirementsontology.ROModel;
import org.semanticweb.owl.model.OWLOntology;

public class Ontology2IndividualDataOperation implements Operation {

	@Override
	public TableRepresentation executeOperation(ArtifactRepresentation representation, 
			ArtifactType type, OperationConfiguration config) {
		OWLOntology ontology = checkAndUnwrap(representation);
		Ontology2IndividualDataConfiguration o2dConfig = checkAndUnwrap(config);
		Table table = transformToTable(ontology, o2dConfig);
		TableRepresentation tr = new TableRepresentation(table);
		return tr;
	}
	
	
	private OWLOntology checkAndUnwrap(ArtifactRepresentation rep) {
		if (rep.getType().equals(OwlRepresentation.TYPE)) {
			OwlRepresentation or = (OwlRepresentation)rep;
			return or.getOntology();
		} else {
			throw new DeftInvalidOperationChainException();
		}
	}
	
	private Ontology2IndividualDataConfiguration checkAndUnwrap(OperationConfiguration config) {
		if (config instanceof Ontology2IndividualDataConfiguration) {
			Ontology2IndividualDataConfiguration o2dConfig = (Ontology2IndividualDataConfiguration)config;
			return o2dConfig;
		} else {
			throw new DeftInvalidOperationConfigurationException();
		}
	}
	
	
	private Table transformToTable(OWLOntology ontology, Ontology2IndividualDataConfiguration config) {
		OntologyAccessor oa = new OntologyAccessor(ontology);
		ROModel roModel = new ROModel(oa);
		String individual = config.getIndividual();
		if (isIndividualSet(individual)) {
			IndividualDataVisualizer visualizer = new IndividualDataVisualizer(roModel);
			Table table = visualizer.getIndividualDataSurvey(individual);
			if (table != null) {
				return table;
			}
		}
		
		Table emptyTable = createEmptyTable();
		return emptyTable;
	}
	
	private Table createEmptyTable() {
		Table emptyTable = new Table(new String[][]{{"Empty"}});
		return emptyTable;
	}
	
	private boolean isIndividualSet(String individualName) {
		return individualName != null && !individualName.equals("");
	}
	
	

	

	@Override
	public OperationConfiguration createConfiguration() {
		return new Ontology2IndividualDataConfiguration();
	}


	@Override
	public String getInputRepresentationType() {
		return OwlRepresentation.TYPE;
	}


	@Override
	public String getResultingRepresentationType() {
		return TableRepresentation.TYPE;
	}
}
