package org.deft.operation;

import java.awt.Dimension;
import java.awt.image.BufferedImage;
import java.util.List;
import java.util.Set;

import org.deft.operation.exception.DeftInvalidOperationChainException;
import org.deft.operation.exception.DeftInvalidOperationConfigurationException;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.OwlRepresentation;
import org.deft.representation.image.BufferedImageRepresentation;
import org.deft.requirementsontology.DirectedROModel;
import org.deft.requirementsontology.GoalModelVisualizer;
import org.deft.requirementsontology.OntologyAccessor;
import org.deft.requirementsontology.VisualizationLayoutType;
import org.semanticweb.owl.model.OWLOntology;

public class Ontology2RequirementsGraphOperation implements Operation {

	@Override
	public BufferedImageRepresentation executeOperation(ArtifactRepresentation representation, 
			ArtifactType type, OperationConfiguration config) {
		OWLOntology ontology = checkAndUnwrap(representation);
		Ontology2RequirementsGraphConfiguration o2gConfig = checkAndUnwrap(config);
		o2gConfig.initIfIsUnmodified(ontology);
		
		BufferedImage image = transformToImage(ontology, o2gConfig);
		BufferedImageRepresentation br = new BufferedImageRepresentation(image);
		return br;
	}
	
	
	private OWLOntology checkAndUnwrap(ArtifactRepresentation rep) {
		if (rep.getType().equals(OwlRepresentation.TYPE)) {
			OwlRepresentation or = (OwlRepresentation)rep;
			return or.getOntology();
		} else {
			throw new DeftInvalidOperationChainException();
		}
	}
	
	private Ontology2RequirementsGraphConfiguration checkAndUnwrap(OperationConfiguration config) {
		if (config instanceof Ontology2RequirementsGraphConfiguration) {
			Ontology2RequirementsGraphConfiguration o2gConfig = (Ontology2RequirementsGraphConfiguration)config;
			return o2gConfig;
		} else {
			throw new DeftInvalidOperationConfigurationException();
		}
	}
	
	
	private BufferedImage transformToImage(OWLOntology ontology, Ontology2RequirementsGraphConfiguration config) {
		DirectedROModel roModel = setupRoModelFromConfig(ontology, config);
		BufferedImage bi = paintRoModel(roModel);
		return bi;
	}
	
	
	private DirectedROModel setupRoModelFromConfig(OWLOntology ontology, Ontology2RequirementsGraphConfiguration config) {
		OntologyAccessor oa = new OntologyAccessor(ontology);
		List<String> indNamesFromConfig = config.getIndividuals();
		DirectedROModel roModel = new DirectedROModel(oa);
		roModel.removeAll();
		Set<String> availableIndividualNames = roModel.getOntologyIndividualNames();
		for (String individualName : indNamesFromConfig) {
			//if we have an outdated configuration, we must make sure not to add
			//individuals which don't exist anymore in the ontology
			if (availableIndividualNames.contains(individualName)) {
				roModel.insertIndividual(individualName);
			}
		}
		roModel.insertObjectProperty("refinesTo");
		roModel.removeUnconnectedIndividuals();
		return roModel;
	}
	
	private BufferedImage paintRoModel(DirectedROModel roModel) {
		GoalModelVisualizer gmVisualizer = new GoalModelVisualizer(roModel);

		Dimension graphicSize = new Dimension(1000,1000);
		
		BufferedImage bi = gmVisualizer.getImage(graphicSize, VisualizationLayoutType.TREE);
		return bi;
	}
	
	

	@Override
	public OperationConfiguration createConfiguration() {
		return new Ontology2RequirementsGraphConfiguration();
	}


	@Override
	public String getInputRepresentationType() {
		return OwlRepresentation.TYPE;
	}


	@Override
	public String getResultingRepresentationType() {
		return BufferedImageRepresentation.TYPE;
	}
}
