package org.deft.operation.astfilter;

import java.util.LinkedList;
import java.util.List;

import org.deft.operation.AbstractOperationConfiguration;
import org.deft.repository.ast.jxpath.XPath;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

public class AstFilterConfiguration extends AbstractOperationConfiguration {

	public static String ID = "org.deft.operation.astfilter";
	
	private List<XPath> xpaths = new LinkedList<XPath>();
	private String refId;
	
	
	public AstFilterConfiguration() {
		setXPath(new XPath("/"));
	}
	
	@Override
	public String getId() {
		return ID;
	}

	
	public void setXPaths(List<XPath> xpaths) {
		this.xpaths.clear();
		this.xpaths.addAll(xpaths);
	}
	
	public void setXPath(XPath xpath) {
		this.xpaths.clear();
		this.xpaths.add(xpath);
	}
	
	public List<XPath> getXPaths() {
		return new LinkedList<XPath>(xpaths);
	}
	
	public void setRefId(String refId) {
		this.refId = refId;
	}
	
	public String getRefId() {
		return refId;
	}
	
	
	@Override
	public void loadFromXml(Element element) {
		if (isValid(element)) {
			loadXPaths(element);
			loadRefId(element);
		}
	}
	
	private void loadXPaths(Element element) {
		xpaths.clear();
		Element eXPaths = (Element)element.getElementsByTagName("xpaths").item(0);
		NodeList nl = eXPaths.getElementsByTagName("xpath");
		for (int i = 0; i < nl.getLength(); i++) {
			Element e = (Element)nl.item(i);
			loadSingleXPathElement(e);
		}
	}
	
	private void loadSingleXPathElement(Element e) {
		String xpathString = e.getTextContent();
		XPath xpath = new XPath(xpathString);
		xpaths.add(xpath);
	}
	
	private void loadRefId(Element element) {
		Element eRefId = (Element)element.getElementsByTagName("refid").item(0);
		String refId = eRefId.getTextContent();
		this.refId = refId;
	}

	@Override
	public void addToXml(Element root) {
		addXPathsToXml(root);
		addRefIdToXml(root);
	}
	
	private void addXPathsToXml(Element root) {
		Document doc = root.getOwnerDocument();
		Element eXPaths = doc.createElement("xpaths");
		for (XPath xpath : xpaths) {
			Element e = createXPathElement(xpath, doc);
			eXPaths.appendChild(e);
		}
		root.appendChild(eXPaths);
	}
	
	private Element createXPathElement(XPath xpath, Document doc) {
		Element e = doc.createElement("xpath");
		e.setTextContent(xpath.getXPathString());
		return e;
	}
	
	private void addRefIdToXml(Element root) {
		Document doc = root.getOwnerDocument();
		Element eRefId = doc.createElement("refid");
		eRefId.setTextContent(refId);
		root.appendChild(eRefId);
	}

	
	@Override
	protected String getLocalSchemaFileLocation() {
		return "resources/schema.xsd";
	}
	
	
}
