package org.deft.operation.astmodifier.gui;

import java.text.Format;

import org.deft.artifacttypesupport.codefile.AstTransformerRegistry;
import org.deft.artifacttypesupport.codefile.LanguageSpecifics;
import org.deft.artifacttypesupport.codefile.PlainCodeFileArtifactType;
import org.deft.artifacttypesupport.codefile.XPathGenerator;
import org.deft.artifacttypesupport.codefile.transform.OutlineNode;
import org.deft.artifacttypesupport.codefile.transform.OutlineNodeTransformer;
import org.deft.configuration.OperationGui;
import org.deft.operation.OperationConfiguration;
import org.deft.operation.astmodifier.AstModifierConfiguration;
import org.deft.repository.ast.TreeNode;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.treenode.TreeNodeRepresentation;
import org.eclipse.jface.viewers.CheckboxTreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TreeEditor;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;



public class AstModifierOperationGui implements OperationGui {

	private Format oldFormat;
	private CheckboxTreeViewer treeViewer;
	private TreeEditor editor;
	private Text name;
	private Button inline;
	private Button block;
	
	//private int EXPANDED_REPLACE_SECTION_HEIGHT = 100;
	private int NOT_EXPANDED_REPLACE_SECTION_SIZE = 30;
	private int SHELL_WIDTH = 800;
	private int SHELL_HEIGHT = 600;

	private OutlineTreeWithReplacements tableData;
	private AstModifierConfiguration modifierConfig;
	
	@Override
	public Composite createGuiComposite(Composite parent) {
		Composite treeBody = new Composite(parent, SWT.NONE);
		treeBody.setLayout(new GridLayout(1, true));

		Tree tree = new Tree(treeBody, SWT.CHECK | SWT.BORDER);
		tree.setLayoutData(new GridData(GridData.FILL_BOTH));
		tree.setLinesVisible(true);
		tree.setHeaderVisible(true);
		TreeColumn col1 = new TreeColumn(tree, SWT.LEFT);
		TreeColumn col2 = new TreeColumn(tree, SWT.LEFT);
		col1.setResizable(true);
		col2.setResizable(true);
		
		int w = (int) ((SHELL_WIDTH - 52) * 0.75);
		col1.setWidth(w);
		col2.setWidth(SHELL_WIDTH - 52 - w);

		editor = new TreeEditor(tree);
		editor.horizontalAlignment = SWT.LEFT;
		editor.grabHorizontal = true;
		editor.minimumWidth = 50;

//		tableData = new OutlineTreeWithReplacements(root);
		
		treeViewer = new CheckboxTreeViewer(tree);

//		treeViewer.addCheckStateListener(new ICheckStateListener(){
//			public void checkStateChanged(CheckStateChangedEvent event) {
//				OutlineNode node = (OutlineNode)event.getElement();
//				handleStateChanged(node,event.getChecked());
//			}
//		});

		
//		treeViewer.addSelectionChangedListener(new ISelectionChangedListener(){
//			public void selectionChanged(SelectionChangedEvent event) {
//				TreeItem item = treeViewer.getTree().getSelection()[0];
//				handleSelectionChanged(item);
//			}	
//		});
//		
//		treeViewer.addDoubleClickListener(new IDoubleClickListener() {
//			public void doubleClick(DoubleClickEvent event) {
//				TreeItem item = treeViewer.getTree().getSelection()[0];
//				if (!replacementTextList.containsKey(item.getData()) && !item.getChecked()){
//					replacementTextList.put((OutlineNode) item.getData(), "");
//					item.setImage(1, ResourceHelper.getImage("rename.gif"));
//					handleSelectionChanged(item);
//				}
//			}
//		});

		// edited by abartho
		treeViewer.expandAll();
		//works only for expanded nodes, therefore we did an expandAll before
		treeViewer.setAllChecked(true);
		//all nodes checked, collapse again
		treeViewer.collapseAll();
		//and open only the first level:
		Tree t = treeViewer.getTree();
		//if there is only 1 root node, expand it
		if (t.getItemCount() == 1) {
			treeViewer.getTree().getItem(0).setExpanded(true);
		}
		//if there are multiple root nodes (due to multiple query xpaths)
		//we treat those as root nodes and do not expand them
		                                   
		
		FormData fd = new FormData();
		fd.top = new FormAttachment(0, 70);
		fd.left = new FormAttachment(0, 5);
		fd.right = new FormAttachment(100, -5);
		fd.bottom = new FormAttachment(100, -NOT_EXPANDED_REPLACE_SECTION_SIZE);
//		treeSection.setLayoutData(fd);

//		toolkit.paintBordersFor(treeBody);
//		treeSection.setClient(treeBody);
		
		//handle edit case
//		if (oldFormat != null){
//			Map<TreeItem,String> map = TreeViewerHelper.applyFormat(treeViewer, oldFormat);
//			for (TreeItem item : map.keySet()) {
//				replacementTextList.put((OutlineNode) item.getData(), map.get(item));
//				item.setImage(1, ResourceHelper.getImage("rename.gif"));
//				item.setText(1, map.get(item));
//			}
//		}
		return treeBody;
	}
	
	
	@Override
	public void initializeGui(OperationConfiguration config, Artifact artifact,
			ArtifactRepresentation artifactRepr) {
		modifierConfig = (AstModifierConfiguration)config;
		TreeNode ast = ((TreeNodeRepresentation)artifactRepr).getRoot();
		PlainCodeFileArtifactType type = (PlainCodeFileArtifactType)artifact.getArtifactType();		
		LanguageSpecifics ls = type.getLanguageSpecifics();
		AstTransformerRegistry transReg = ls.getTransformerRegistry();
		OutlineNodeTransformer transformer = transReg.getDetailedOutlineTransformer();
		OutlineNode newRoot = transformer.transform(ast);
		tableData = new OutlineTreeWithReplacements(newRoot);
		
		treeViewer.setContentProvider(new AstModifierTreeContentProvider());
		treeViewer.setLabelProvider(new AstModifierTreeLabelProvider(tableData));
		treeViewer.setInput(tableData);
		


//		XPathGenerator xpathGenerator = ls.getXPathGenerator();
//		
//		treeView.addSelectionChangedListener(new PrintXPathsListener(xpathGenerator));
		
	}

}
