package org.deft.operation.bytearray2charbuffer;

import java.nio.CharBuffer;

import org.deft.operation.EmptyOperationConfiguration;
import org.deft.operation.Operation;
import org.deft.operation.OperationConfiguration;
import org.deft.operation.exception.DeftInvalidOperationChainException;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.bytearray.ByteArrayRepresentation;
import org.deft.representation.charbuffer.CharBufferRepresentation;


public class ByteArray2CharBufferOperation implements Operation {


	
	@Override
	public ArtifactRepresentation executeOperation(ArtifactRepresentation representation, 
			ArtifactType type, OperationConfiguration config) {
		byte[] byteContent = checkAndUnwrap(representation);
		CharBuffer buffer = createBuffer(byteContent);
		CharBufferRepresentation cbr = new CharBufferRepresentation(buffer);
		return cbr;
	}
	
	
	private byte[] checkAndUnwrap(ArtifactRepresentation rep) {
		if (rep.getType().equals(ByteArrayRepresentation.TYPE)) {
			ByteArrayRepresentation bar = (ByteArrayRepresentation)rep;
			return bar.getByteArray();
		} else {
			throw new DeftInvalidOperationChainException();
		}
	}
	
	
	private CharBuffer createBuffer(byte[] byteContent) {
		CharBuffer cb = CharBuffer.allocate(byteContent.length);
		for (int i = 0; i < byteContent.length; i++) {
			byte b = byteContent[i];
			char c = (char)b;
			cb.append(c);
		}
		return cb;
	}
	


	@Override
	public OperationConfiguration createConfiguration() {
		return new EmptyOperationConfiguration();
	}


	@Override
	public String getInputRepresentationType() {
		return ByteArrayRepresentation.TYPE;
	}


	@Override
	public String getResultingRepresentationType() {
		return CharBufferRepresentation.TYPE;
	}

}



