package org.deft.operation.bytearray2ecoreinstance;

import java.io.ByteArrayInputStream;
import java.io.InputStream;

import org.deft.emftext.EmfTextArtifactType;
import org.deft.operation.EmptyOperationConfiguration;
import org.deft.operation.OperationConfiguration;
import org.deft.util.EcoreUtil;
import org.eclipse.emf.ecore.EObject;


/**
 * An operation that takes the content of an EMFText-file as byte array 
 * and returns an EObject, representing an Ecore instance.
 */
public class EmfTextAsByteArray2EcoreInstanceOperation extends ByteArray2EcoreInstanceOperation {

	@Override
	protected EObject getEcoreInstance(byte[] byteContent) {
		InputStream is = new ByteArrayInputStream(byteContent);
		String emfTextLanguageId = getEmfTextLanguageId();
		EObject root = EcoreUtil.loadRegisteredEcoreInstance(emfTextLanguageId, is);
		return root;
	}
	
	
	private String getEmfTextLanguageId() {
		if (artifactType instanceof EmfTextArtifactType) {
			EmfTextArtifactType emfType = (EmfTextArtifactType)artifactType;
			return emfType.getEmfTextLanguageId();
		}
		return null;
	}
	

	@Override
	public OperationConfiguration createConfiguration() {
		return new EmptyOperationConfiguration();
	}


}



