package org.deft.operation.bytearray2image;

import java.awt.image.BufferedImage;

import org.deft.operation.EmptyOperationConfiguration;
import org.deft.operation.Operation;
import org.deft.operation.OperationConfiguration;
import org.deft.operation.exception.DeftInvalidOperationChainException;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.bytearray.ByteArrayRepresentation;
import org.deft.representation.image.BufferedImageRepresentation;
import org.deft.util.ImageUtil;


public class ByteArray2ImageOperation implements Operation {


	@Override
	public ArtifactRepresentation executeOperation(ArtifactRepresentation representation, 
			ArtifactType type, OperationConfiguration config) {
		byte[] byteContent = checkAndUnwrap(representation);
		BufferedImage image = ImageUtil.createImage(byteContent);
		BufferedImageRepresentation bir = new BufferedImageRepresentation(image);
		return bir;
	}
	
	
	private byte[] checkAndUnwrap(ArtifactRepresentation rep) {
		if (rep.getType().equals(ByteArrayRepresentation.TYPE)) {
			ByteArrayRepresentation bar = (ByteArrayRepresentation)rep;
			return bar.getByteArray();
		} else {
			throw new DeftInvalidOperationChainException();
		}
	}
	
	

	@Override
	public OperationConfiguration createConfiguration() {
		return new EmptyOperationConfiguration();
	}


	@Override
	public String getInputRepresentationType() {
		return ByteArrayRepresentation.TYPE;
	}


	@Override
	public String getResultingRepresentationType() {
		return BufferedImageRepresentation.TYPE;
	}

}



