package org.deft.operation.charbuffer2text;

import java.nio.CharBuffer;

import org.deft.operation.EmptyOperationConfiguration;
import org.deft.operation.Operation;
import org.deft.operation.OperationConfiguration;
import org.deft.operation.exception.DeftInvalidOperationChainException;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.charbuffer.CharBufferRepresentation;
import org.deft.representation.text.TextRepresentation;


public class CharBuffer2TextOperation implements Operation {


	
	@Override
	public ArtifactRepresentation executeOperation(ArtifactRepresentation representation, 
			ArtifactType type, OperationConfiguration config) {
		CharBuffer buffer = checkAndUnwrap(representation);
		String text = createString(buffer);
		TextRepresentation tr = new TextRepresentation(text);
		return tr;
	}
	
	
	private CharBuffer checkAndUnwrap(ArtifactRepresentation rep) {
		if (rep.getType().equals(CharBufferRepresentation.TYPE)) {
			CharBufferRepresentation cbr = (CharBufferRepresentation)rep;
			return cbr.getCharBuffer();
		} else {
			throw new DeftInvalidOperationChainException();
		}
	}
	
	
	private String createString(CharBuffer buffer) {
		return new String(buffer.array());
	}
	


	@Override
	public OperationConfiguration createConfiguration() {
		return new EmptyOperationConfiguration();
	}


	@Override
	public String getInputRepresentationType() {
		return CharBufferRepresentation.TYPE;
	}


	@Override
	public String getResultingRepresentationType() {
		return TextRepresentation.TYPE;
	}

}



