package org.deft.operation.coolsoftware;



import java.io.File;
import java.io.IOException;

import org.deft.operation.AbstractOperationChain;
import org.deft.operation.tempfile2ecorediag.TempFile2MultiEcoreOperation;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.artifacttype.ecore.EcoreArtifactType;
import org.deft.repository.artifacttype.ecore.EcoreTempFileRepresentation;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.repository.datamodel.Project;
import org.deft.util.FileUtil;

/**
 * Operation chain that just takes an Ecore metamodel and produces a listing
 * of styled text that explains its attributes.
 */
public class Ecore2CoolSoftwareTextOperationChain extends AbstractOperationChain {
	
	public static String ID = "org.deft.operationchain.ecore2coolsoftwaretext";
	
	public Ecore2CoolSoftwareTextOperationChain() {
		setupChain();
	}
	
	
	private void setupChain() {
		addOperation(new TempFile2MultiEcoreOperation());
		addOperation(new Ecore2CoolSoftwareTextOperation());
	}
	
	
	@Override
	public String getId() {
		return ID;
	}
	

	@Override
	public ArtifactRepresentation getInitialArtifactRepresentation(Artifact artifact, int revision) {
		try {
			//always use different folder to store ecore diagram and ecore file, otherwise some Eclipse/GMF internal
			//caching mechanism sees same diagram location and will not compute a new image
			Project project = artifact.getProject();
			File projectTempDir = project.getTempDirectory();
			File tempDir = FileUtil.getUniqueSubDirIn(projectTempDir);
			tempDir.deleteOnExit();
			EcoreTempFileRepresentation etfr = new EcoreTempFileRepresentation(tempDir);
			etfr.writeArtifactAndDependentFiles(artifact, revision);
			return etfr;
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}



	@Override
	public boolean isValidInputArtifactType(ArtifactType artifactType) {
		boolean isEcoreType = artifactType instanceof EcoreArtifactType;
		return isEcoreType;
	}
	

	@Override
	public String getDescription() {
		return "Content as Text";
	}

}
