package org.deft.operation.coolsoftware.gui;

import java.util.List;

import org.deft.configuration.OperationGui;
import org.deft.operation.OperationConfiguration;
import org.deft.operation.coolsoftware.Ecore2CoolSoftwareTextConfiguration;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.ecore.EcoreRepresentation;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

public class Ecore2CoolSoftwareTextOperationGui implements OperationGui {

	private Ecore2CoolSoftwareTextConfiguration dataConfig;
	private TreeViewer treeViewer;
	private Tree tree;
	
	@Override
	public Composite createGuiComposite(Composite parent) {
		Composite gui = new Composite(parent, SWT.NONE);
		gui.setLayout(new FillLayout());
		treeViewer = new TreeViewer(gui, SWT.BORDER | SWT.SINGLE);
		tree = treeViewer.getTree();
		return gui;
	}
	
	
	@Override
	public void initializeGui(OperationConfiguration config,
			Artifact artifact, ArtifactRepresentation artifactRepr) {
		this.dataConfig = (Ecore2CoolSoftwareTextConfiguration)config;
		EcoreRepresentation ecoreRep = (EcoreRepresentation)artifactRepr;
		EObject modelRoot = ecoreRep.getModelRoot();
		List<Resource> allResources = modelRoot.eResource().getResourceSet().getResources();
		treeViewer.setContentProvider(new EcoreClassAndPackageContentProvider());
		treeViewer.setLabelProvider(new EcoreClassAndPackageLabelProvider());
		treeViewer.setInput(allResources);
		treeViewer.expandToLevel(2);
		treeViewer.addSelectionChangedListener(new IndividualSelectionListener(tree));
		selectFirstLineIfPossibleAndSetConfig();
	}

	
	
	private void selectFirstLineIfPossibleAndSetConfig() {
		int itemCount = tree.getItemCount();
		if (itemCount > 0) {
			Object data = treeViewer.getTree().getTopItem().getData();
			ISelection selection = new StructuredSelection(data);
			treeViewer.setSelection(selection);
		}
	}
	

	
	/**
	 * Updates the OperationConfiguration whenever an individual is checked or unchecked
	 */
	private class IndividualSelectionListener implements ISelectionChangedListener {
		
		private Tree tree;

		public IndividualSelectionListener(Tree tree) {
			this.tree = tree;
		}
		

		@Override
		public void selectionChanged(SelectionChangedEvent event) {
			int selectionCount = tree.getSelectionCount();
			if (selectionCount >= 1) {
				TreeItem ti = tree.getSelection()[0];
				Object data = ti.getData();
				if (data instanceof EClass) {
					EClass ec = (EClass)data;
					String selectedItemName = getQualifiedName(ec);
					dataConfig.setClassName(selectedItemName);
				}
			}
		}
		
		private String getQualifiedName(EClass ec) {
			String qualifiedName = ec.getName();
			EObject container = ec.eContainer();
			while (container instanceof EPackage) {
				EPackage ePackage = (EPackage)container;
				qualifiedName = ePackage.getName() + "." + qualifiedName;
				container = ePackage.eContainer();
			}
			return qualifiedName;
		}
	}


}
