package org.deft.operation.coolsoftware.gui;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

public class EcoreClassAndPackageContentProvider implements ITreeContentProvider {

	private static final Object[] NO_CHILDREN = new Object[0];
	
	private List<Resource> resources = new LinkedList<Resource>();
	
	@Override
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		resources.clear();
		if (newInput instanceof List<?>) {
			for (Object o : (List<?>)newInput) {
				if (o instanceof Resource) {
					resources.add((Resource)o);
				}
			}
		}
		
	}

	@Override
	public Object[] getElements(Object inputElement) {
		return resources.toArray();
	}

	@Override
	public Object[] getChildren(Object parentElement) {
		if (parentElement instanceof Resource) {
			Resource resource = (Resource)parentElement;
			Object[] childrenArray = resource.getContents().toArray();
			return childrenArray;
		}
		if (parentElement instanceof EPackage) {
			EPackage ep = (EPackage)parentElement;
			List<EObject> packageAndClassChildren = new LinkedList<EObject>();
			for (EObject eo : ep.eContents()) {
				if (eo instanceof EPackage || eo instanceof EClass) {
					packageAndClassChildren.add(eo);
				}
			}
			Object[] childrenArray = packageAndClassChildren.toArray();
			return childrenArray;
		}
		return NO_CHILDREN;
	}

	@Override
	public Object getParent(Object element) {
		if (element instanceof EClass) {
			EClass ec = (EClass)element;
			EPackage parent = ec.getEPackage();
			return parent;
		} else if (element instanceof EPackage) {
			EPackage ep = (EPackage)element;
			EPackage parent = ep.getESuperPackage();
			if (parent != null) {
				return parent;
			} else {
				Resource resource = ep.eResource();
				return resource;
			}
		}
		return null;
	}

	@Override
	public boolean hasChildren(Object element) {
		return getChildren(element).length != 0;
	}
	
	@Override
	public void dispose() {
		this.resources = null;
	}

}
