package org.deft.operation.coolsoftware.textcompute;

import java.util.List;

import javax.swing.text.StyleContext;

import org.deft.representation.styleddocument.data.ContentSequence;
import org.deft.representation.styleddocument.data.DocumentContentContainer;
import org.deft.representation.styleddocument.data.SimpleListing;
import org.deft.representation.styleddocument.data.StyleEnum;
import org.deft.representation.styleddocument.data.TokenizedStyledDocument;
import org.deft.util.StyledDocumentUtil;
import org.eclipse.emf.common.util.EMap;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EReference;

public class AssociationsBuilder extends LatexTextBuilder {
	
	public void addReferences(EClass eClass, DocumentContentContainer container) {
		ContentSequence contentSequence =  new ContentSequence();
		container.addDocumentContent(contentSequence);
		addHeading(contentSequence, "Associations:");
		if (hasReferences(eClass)) {
			addLineBreak(contentSequence);
			addReferences(eClass, contentSequence);
		} else {
			addNoReferencesText(contentSequence);
			addLineBreak(contentSequence);
		}
	}
	
	
	private boolean hasReferences(EClass eClass) {
		List<EReference> references = eClass.getEReferences();
		boolean hasReferences = !references.isEmpty();
		return hasReferences;
	}
	
	private void addReferences(EClass eClass, ContentSequence contentSequence) {
		List<EReference> references = eClass.getEReferences();
		SimpleListing listing = new SimpleListing();
		for (EReference reference : references) {
			addReferenceToListing(reference, listing);
		}
		contentSequence.addDocumentContent(listing);
	}
	
	private void addNoReferencesText(ContentSequence contentSequence) {
		TokenizedStyledDocument tsd = new TokenizedStyledDocument();
		StyledDocumentUtil.add(tsd, " none");
		contentSequence.addDocumentContent(tsd);
	}
	
	private void addReferenceToListing(EReference reference, SimpleListing listing) {
		StyleContext context = new LocalStyle().getStyleContext();
		StringBuilder sb = new StringBuilder();
		String refName = reference.getName();
		sb.append(refName).append(" : ");
		
		String refTypeName = "no type";
		EClass ec = reference.getEReferenceType();
		if (ec != null) {
			refTypeName = ec.getName();
		}
		sb.append(refTypeName).append(" ");
		
		int lowerBound = reference.getLowerBound();
		int upperBound = reference.getUpperBound();
		String boundString = getBoundString(lowerBound, upperBound);
		sb.append(boundString);
		
		TokenizedStyledDocument tsdReference = new TokenizedStyledDocument(context);
		StyledDocumentUtil.add(tsdReference, sb.toString(), StyleEnum.CODE.getName());
		
		String containmentOrNotString = getContainmentOrNotString(reference);
		StyledDocumentUtil.add(tsdReference, containmentOrNotString);
	
		
		if (hasComment(reference)) {
			String commentString = getCommentString(reference);
			String linebreakAndComment = "\\\\\n" + commentString;
			StyledDocumentUtil.add(tsdReference, linebreakAndComment);
		}
		
		listing.addListItem(tsdReference);
	}
	
	
	protected String getContainmentOrNotString(EReference reference) {
		if (reference.isContainment()) {
			return " (containment reference)";
		} else {
			return "";
		}
	}
	

	private boolean hasComment(EReference reference) {
		EAnnotation annotation = reference.getEAnnotation("http://www.eclipse.org/emf/2002/GenModel");
		if (annotation == null) {
			return false;
		}
		EMap<String, String> details = annotation.getDetails();
		return details != null && details.get("documentation") != null;
	}
	
	
	private String getCommentString(EReference reference) {
		EAnnotation annotation = reference.getEAnnotation("http://www.eclipse.org/emf/2002/GenModel");
		String commentString = annotation.getDetails().get("documentation");
		return commentString;
	}
	
	
}
