package org.deft.operation.coolsoftware.textcompute;

import java.util.List;

import javax.swing.text.StyleContext;

import org.deft.representation.styleddocument.data.ContentSequence;
import org.deft.representation.styleddocument.data.DocumentContentContainer;
import org.deft.representation.styleddocument.data.SimpleListing;
import org.deft.representation.styleddocument.data.StyleEnum;
import org.deft.representation.styleddocument.data.TokenizedStyledDocument;
import org.deft.util.StyledDocumentUtil;
import org.eclipse.emf.common.util.EMap;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EDataType;

public class AttributesBuilder extends LatexTextBuilder {
	
	public void addAttributes(EClass eClass, DocumentContentContainer container) {
		ContentSequence contentSequence =  new ContentSequence();
		container.addDocumentContent(contentSequence);
		addHeading(contentSequence, "Attributes:");
		if (hasAttributes(eClass)) {
			addLineBreak(contentSequence);
			addAttributes(eClass, contentSequence);
		} else {
			addNoAttributesText(contentSequence);
			addLineBreak(contentSequence);
		}
	}
	
	
	private boolean hasAttributes(EClass eClass) {
		List<EAttribute> attributes = eClass.getEAttributes();
		boolean hasAttributes = !attributes.isEmpty();
		return hasAttributes;
	}
	
	private void addAttributes(EClass eClass, ContentSequence contentSequence) {
		List<EAttribute> attributes = eClass.getEAttributes();
		SimpleListing listing = new SimpleListing();
		for (EAttribute attribute : attributes) {
			addAttributeToListing(attribute, listing);
		}
		contentSequence.addDocumentContent(listing);
	}
	
	private void addNoAttributesText(ContentSequence contentSequence) {
		TokenizedStyledDocument tsd = new TokenizedStyledDocument();
		StyledDocumentUtil.add(tsd, " none");
		contentSequence.addDocumentContent(tsd);
	}
	
	private void addAttributeToListing(EAttribute attribute, SimpleListing listing) {
		StyleContext context = new LocalStyle().getStyleContext();
		StringBuilder sb = new StringBuilder();
		String attName = attribute.getName();
		sb.append(attName).append(" : ");
		
		String attTypeName = "no type";
		EDataType ed = attribute.getEAttributeType();
		if (ed != null) {
			attTypeName = ed.getName();
		}
		sb.append(attTypeName).append(" ");
		
		int lowerBound = attribute.getLowerBound();
		int upperBound = attribute.getUpperBound();
		String boundString = getBoundString(lowerBound, upperBound);
		sb.append(boundString);
		
		TokenizedStyledDocument tsdAttribute = new TokenizedStyledDocument(context);
		StyledDocumentUtil.add(tsdAttribute, sb.toString(), StyleEnum.CODE.getName());
		
		if (hasComment(attribute)) {
			String commentString = getCommentString(attribute);
			String linebreakAndComment = "\\\\\n" + commentString;
			StyledDocumentUtil.add(tsdAttribute, linebreakAndComment);
		}
		
		listing.addListItem(tsdAttribute);
	}
	

	private boolean hasComment(EAttribute attribute) {
		EAnnotation annotation = attribute.getEAnnotation("http://www.eclipse.org/emf/2002/GenModel");
		if (annotation == null) {
			return false;
		}
		EMap<String, String> details = annotation.getDetails();
		return details != null && details.get("documentation") != null;
	}
	
	
	private String getCommentString(EAttribute attribute) {
		EAnnotation annotation = attribute.getEAnnotation("http://www.eclipse.org/emf/2002/GenModel");
		String commentString = annotation.getDetails().get("documentation");
		return commentString;
	}
	
	
}
