package org.deft.operation.decoratedtreetooutputxml;

import org.deft.operation.Operation;
import org.deft.operation.OperationConfiguration;
import org.deft.operation.exception.DeftInvalidOperationChainException;
import org.deft.operation.exception.DeftInvalidOperationConfigurationException;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.ast.TreeNodeRoot;
import org.deft.repository.ast.decoration.DecoratedTreeToOutputXmlTransformer;
import org.deft.repository.ast.decoration.DecoratorSelection;
import org.deft.repository.ast.decoration.TreeNodeDecorationApplier;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.treenode.TreeNodeRepresentation;
import org.deft.representation.xml.XmlRepresentation;
import org.w3c.dom.Document;



public class DecoratedTreeToOutputXmlOperation implements Operation {

    
	@Override
	public ArtifactRepresentation executeOperation(ArtifactRepresentation representation,
			ArtifactType type, OperationConfiguration config) {
		TreeNodeRoot root = checkAndUnwrap(representation);
		DecoratedTreeToOutputXmlConfiguration treeToXmlConfig = checkAndUnwrap(config);
		DecoratorSelection ds = treeToXmlConfig.getDecorators();
		Document doc = createOutputXml(root, ds);
		XmlRepresentation xmlRep = new XmlRepresentation(doc);
		return xmlRep;
	}
	
	
	private TreeNodeRoot checkAndUnwrap(ArtifactRepresentation rep) {
		if (rep.getType().equals(TreeNodeRepresentation.TYPE)) {
			TreeNodeRepresentation tnr = (TreeNodeRepresentation)rep;
			return tnr.getRoot();
		} else {
			throw new DeftInvalidOperationChainException();
		}
	}
	
	private DecoratedTreeToOutputXmlConfiguration checkAndUnwrap(OperationConfiguration config) {
		if (config instanceof DecoratedTreeToOutputXmlConfiguration) {
			DecoratedTreeToOutputXmlConfiguration afConfig = (DecoratedTreeToOutputXmlConfiguration)config;
			return afConfig;
		} else {
			throw new DeftInvalidOperationConfigurationException();
		}
	}
	

	
	protected TreeNodeRoot checkAndCast(Object input) {
		if (input instanceof TreeNodeRoot) {
			return (TreeNodeRoot)input;
		} else {
			throw new DeftInvalidOperationChainException();
		}
	}
	
	private Document createOutputXml(TreeNodeRoot root, DecoratorSelection decorators) {
		addDecorationsToTreeNode(root, decorators);
		Document outputXml = transformDecoratedTreeNodeToOutputXml(root, decorators);
		return outputXml;
	}
	
	
	private void addDecorationsToTreeNode(TreeNodeRoot root, DecoratorSelection decorators) {
		TreeNodeDecorationApplier applier = new TreeNodeDecorationApplier(root);
		applier.applyAstDecorators(decorators);
	}
	
	private Document transformDecoratedTreeNodeToOutputXml(TreeNodeRoot root, DecoratorSelection decorators) {
		DecoratedTreeToOutputXmlTransformer transformer 
				= new DecoratedTreeToOutputXmlTransformer(root, decorators);
		Document outputXml = transformer.createOutputXml();
		return outputXml;
	}

	
	@Override
	public OperationConfiguration createConfiguration() {
		return new DecoratedTreeToOutputXmlConfiguration();
	}


	@Override
	public String getInputRepresentationType() {
		// TODO Auto-generated method stub
		return null;
	}


	@Override
	public String getResultingRepresentationType() {
		return XmlRepresentation.TYPE;
	}

}