package org.deft.operation.ecore2idents;

import org.deft.operation.EmptyOperationConfiguration;
import org.deft.operation.Operation;
import org.deft.operation.OperationConfiguration;
import org.deft.operation.exception.DeftInvalidOperationChainException;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.ecore.EcoreInstanceRepresentation;
import org.deft.representation.identifiers.IdentifiersRepresentation;
import org.deft.representation.identifiers.ecore.EcoreModelToIdentTransformer;
import org.deft.representation.identifiers.ecore.EcorePackageIdentifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;

public class Ecore2IdentsOperation implements Operation {

	public static String ID = "org.deft.operation.ecore2idents";
	
	
	@Override
	public IdentifiersRepresentation executeOperation(ArtifactRepresentation representation, 
			ArtifactType type, OperationConfiguration config) {
		EObject modelRoot = checkAndUnwrap(representation);
		EcoreModelToIdentTransformer trans = new EcoreModelToIdentTransformer();
		EcorePackageIdentifier id = trans.transform((EPackage)modelRoot);
		IdentifiersRepresentation idRep = new IdentifiersRepresentation(id);
        return idRep;
	}
	
	private EObject checkAndUnwrap(ArtifactRepresentation rep) {
		if (rep.getType().equals(EcoreInstanceRepresentation.TYPE)) {
			EcoreInstanceRepresentation eir = (EcoreInstanceRepresentation)rep;
			return eir.getModelRoot();
		} else {
			throw new DeftInvalidOperationChainException();
		}
	}
	
	
	
	@Override
	public OperationConfiguration createConfiguration() {
		return new EmptyOperationConfiguration();
	}


	@Override
	public String getInputRepresentationType() {
		return EcoreInstanceRepresentation.TYPE;
	}


	@Override
	public String getResultingRepresentationType() {
		return IdentifiersRepresentation.TYPE;
	}

}
