package org.deft.operation.ecore2idents;

import java.util.List;

import org.deft.operation.EmptyOperationConfiguration;
import org.deft.operation.Operation;
import org.deft.operation.OperationConfiguration;
import org.deft.operation.exception.DeftInvalidOperationChainException;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.ecore.EcoreInstanceRepresentation;
import org.deft.representation.ecorediag.EcoreDiagRepresentation;
import org.deft.representation.identifiers.IdentifiersRepresentation;
import org.deft.representation.identifiers.data.Scope;
import org.deft.representation.identifiers.ecore.EcoreDiagramToIdentTransformer;
import org.eclipse.gmf.runtime.notation.Diagram;

public class EcoreDiag2IdentsOperation implements Operation {

	public static String ID = "org.deft.operation.ecorediag2idents";
	
	
	@Override
	public IdentifiersRepresentation executeOperation(ArtifactRepresentation representation, 
			ArtifactType type, OperationConfiguration config) {
		List<Diagram> diagrams = checkAndUnwrap(representation);
		EcoreDiagramToIdentTransformer trans = new EcoreDiagramToIdentTransformer();
		Scope id = trans.transform(diagrams);
		IdentifiersRepresentation idRep = new IdentifiersRepresentation(id);
        return idRep;
	}
	
	private List<Diagram> checkAndUnwrap(ArtifactRepresentation rep) {
		if (rep.getType().equals(EcoreDiagRepresentation.TYPE)) {
			EcoreDiagRepresentation eir = (EcoreDiagRepresentation)rep;
			return eir.getDiagrams();
		} else {
			throw new DeftInvalidOperationChainException();
		}
	}
	
	
	
	@Override
	public OperationConfiguration createConfiguration() {
		return new EmptyOperationConfiguration();
	}


	@Override
	public String getInputRepresentationType() {
		return EcoreInstanceRepresentation.TYPE;
	}


	@Override
	public String getResultingRepresentationType() {
		return IdentifiersRepresentation.TYPE;
	}

}
