package org.deft.operation.ecorediag2image.gui;

import java.util.List;

import org.deft.configuration.OperationGui;
import org.deft.operation.OperationConfiguration;
import org.deft.operation.ecorediag2image.EcoreDiag2ImageConfiguration;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.ecorediag.EcoreDiagRepresentation;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.gmf.runtime.notation.Diagram;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;

public class EcoreDiag2ImageOperationGui implements OperationGui {

	private ListViewer listViewer;
	private EcoreDiag2ImageConfiguration diag2ImageConfig;
	
	@Override
	public Composite createGuiComposite(Composite parent) {
		Composite container = new Composite(parent, SWT.NONE);
		container.setLayout(new FillLayout(SWT.HORIZONTAL));
		listViewer = new ListViewer(container, SWT.BORDER | SWT.V_SCROLL | SWT.SINGLE);
		listViewer.addSelectionChangedListener(new SelectionChangedListener());
		listViewer.setLabelProvider(new ListLabelProvider());
		return container;
	}

	@Override
	public void initializeGui(OperationConfiguration config, Artifact artifact,
			ArtifactRepresentation artifactRepr) {
		EcoreDiagRepresentation edr = (EcoreDiagRepresentation)artifactRepr;
		List<Diagram> diagrams = edr.getDiagrams();
		this.diag2ImageConfig = (EcoreDiag2ImageConfiguration)config;
		listViewer.getList().removeAll();
		listViewer.add(diagrams.toArray());
		listViewer.setSelection(new StructuredSelection(diagrams.get(0)));
	}
	
	private class SelectionChangedListener implements ISelectionChangedListener {

		@Override
		public void selectionChanged(SelectionChangedEvent event) {
			IStructuredSelection selection = (IStructuredSelection)event.getSelection();
			Diagram diagram = (Diagram)selection.getFirstElement();
			XMLResource resource = (XMLResource)diagram.eResource();
			String id = resource.getID(diagram);
			diag2ImageConfig.setDiagramId(id);
		}
	}
	
	private class ListLabelProvider extends LabelProvider {

		@Override
		public String getText(Object element) {
			Diagram diagram = (Diagram)element;
			String name = diagram.getName();
			return name;
		}
	}

}
