package org.deft.operation.ecoreinstance2image.gui;

import java.util.LinkedList;
import java.util.List;

import org.deft.configuration.OperationGui;
import org.deft.operation.OperationConfiguration;
import org.deft.operation.ecoreinstance2image.EcoreInstance2ImageConfiguration;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.ecore.EcoreInstanceRepresentation;
import org.deft.util.EcoreUtil;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.viewers.ITreeViewerListener;
import org.eclipse.jface.viewers.TreeExpansionEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;


/**
 * Gui for the operation that converts an Ecore instance model to an image.
 * This gui allows to expand and collapse nodes of the Ecore model tree viewer.
 * 
 * Expanded nodes will be saved in the EcoreInstance2ImageConfiguration, so that the
 * EcoreInstance2ImageOperation can produce and image with the correct expanded and
 * collapsed nodes.  
 */
public class EcoreInstance2ImageOperationGui implements OperationGui {

	private EcoreInstance2ImageConfiguration instance2ImageConfig;
	private TreeViewer treeViewer;
	
	
	@Override
	public Composite createGuiComposite(Composite parent) {
		Composite gui = new Composite(parent, SWT.NONE);
		gui.setLayout(new FillLayout());
		treeViewer = new TreeViewer(parent, SWT.NONE);
		return gui;
	}

	
	@Override
	public void initializeGui(OperationConfiguration config,	
			Artifact artifact, ArtifactRepresentation artifactRepr) {
		this.instance2ImageConfig = (EcoreInstance2ImageConfiguration)config;
		EObject modelRoot = ((EcoreInstanceRepresentation)artifactRepr).getModelRoot();
		
		EcoreUtil.setupEcoreTreeViewer(modelRoot, treeViewer);
		treeViewer.expandToLevel(3);
		treeViewer.addTreeListener(new RecordTreeListener(treeViewer));
	}
	
	
	private class RecordTreeListener implements ITreeViewerListener {

		public RecordTreeListener(TreeViewer tv) {
			saveExpandedNodes(tv);
		}
		
		
		private void saveExpandedNodes(TreeViewer tv) {
			List<EObject> expandedNodes = new LinkedList<EObject>();
			for (Object o : tv.getExpandedElements()) {
				EObject eo = (EObject)o;
				expandedNodes.add(eo);
			}
			instance2ImageConfig.setEObjects(expandedNodes);
		}
		
		@Override
		public void treeExpanded(TreeExpansionEvent event) {
			EObject eo = (EObject)event.getElement();
			List<EObject> expandedNodes = instance2ImageConfig.getEObjects(eo.eResource());
			expandedNodes.add(eo);
			instance2ImageConfig.setEObjects(expandedNodes);
		}

		
		@Override
		public void treeCollapsed(TreeExpansionEvent event) {
			EObject eo = (EObject)event.getElement();
			List<EObject> expandedNodes = instance2ImageConfig.getEObjects(eo.eResource());
			expandedNodes.remove(eo);
			instance2ImageConfig.setEObjects(expandedNodes);
		}

	}


	
}