package org.deft.operation.emftextfilter;

import java.util.LinkedList;
import java.util.List;

import org.deft.operation.AbstractOperationConfiguration;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

public class EmfTextFilterConfiguration extends AbstractOperationConfiguration {

	public static String ID = "org.deft.operation.emftextfilter";
	
	private List<String> uris = new LinkedList<String>();
	
	@Override
	public String getId() {
		return ID;
	}

	public void setEObjects(List<EObject> eObjects) {
		this.uris.clear();
		for (EObject eo : eObjects) {
			String uri = eo.eResource().getURIFragment(eo);
			this.uris.add(uri);
		}
	}
	
	public void setEObject(EObject eObject) {
		this.uris.clear();
		this.uris.add(eObject.eResource().getURIFragment(eObject));
	}
	
	public List<EObject> getEObjects(Resource resource) {
		List<EObject> eObjects = new LinkedList<EObject>();
		for (String uri : uris) {
			EObject eo = resource.getEObject(uri);
			eObjects.add(eo);
		}
		return eObjects;
	}
	
	public List<String> getUris() {
		return uris;
	}

	@Override
	public void loadFromXml(Element element) {
		if (isValid(element)) {
			loadUris(element);
		}
	}
	
	private void loadUris(Element element) {
		uris.clear();
		Element eUris = (Element)element.getElementsByTagName("uris").item(0);
		NodeList nl = eUris.getElementsByTagName("uri");
		for (int i = 0; i < nl.getLength(); i++) {
			Element e = (Element)nl.item(i);
			loadSingleUriElement(e);
		}
	}
	
	private void loadSingleUriElement(Element e) {
		String uri = e.getTextContent();
		uris.add(uri);
	}
	
	@Override
	public void addToXml(Element root) {
		addUrisToXml(root);
	}
	
	private void addUrisToXml(Element root) {
		Document doc = root.getOwnerDocument();
		Element eUris = doc.createElement("uris");
		for (String uri : uris) {
			Element e = createUriElement(uri, doc);
			eUris.appendChild(e);
		}
		root.appendChild(eUris);
	}
	
	private Element createUriElement(String uri, Document doc) {
		Element e = doc.createElement("uri");
		e.setTextContent(uri);
		return e;
	}
	
	@Override
	protected String getLocalSchemaFileLocation() {
		return "resources/schema.xsd";
	}


}
