package org.deft.operation.emftextfilter.gui;

import java.util.LinkedList;
import java.util.List;

import org.deft.configuration.OperationGui;
import org.deft.emftext.EmfTextArtifactType;
import org.deft.operation.OperationConfiguration;
import org.deft.operation.emftextfilter.EmfTextFilterConfiguration;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.text.TextRepresentation;
import org.deft.util.EcoreUtil;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;

public class EmfTextFilterOperationGui implements OperationGui {

	private EmfTextFilterConfiguration emfTextConfig;
	private TreeViewer treeViewer;
	
	@Override
	public Composite createGuiComposite(Composite parent) {
		Composite gui = new Composite(parent, SWT.NONE);
		gui.setLayout(new FillLayout());
		treeViewer = new TreeViewer(gui, SWT.NONE);
		return gui;
	}
	
	
	@Override
	public void initializeGui(OperationConfiguration config,	
			Artifact artifact, ArtifactRepresentation artifactRepr) {
		this.emfTextConfig = (EmfTextFilterConfiguration)config;
		String text = ((TextRepresentation)artifactRepr).getText();
		String emfTextLanguageId = ((EmfTextArtifactType)artifact.getArtifactType()).getEmfTextLanguageId();
		EObject modelRoot = EcoreUtil.loadRegisteredEcoreInstance(emfTextLanguageId, text);
		EcoreUtil.setupEcoreTreeViewer(modelRoot, treeViewer);	
		treeViewer.expandToLevel(3);
		treeViewer.addSelectionChangedListener(new RecordEObjectListener());
	}
	
	
	private class RecordEObjectListener implements ISelectionChangedListener {

		@Override
		public void selectionChanged(SelectionChangedEvent event) {
			
			IStructuredSelection selection = (IStructuredSelection)event.getSelection();
			List<?> selectedObjects = selection.toList();
			List<EObject> eObjects = new LinkedList<EObject>();
			for (Object o : selectedObjects) {
				EObject eo = (EObject)o;
				eObjects.add(eo);
			}
			emfTextConfig.setEObjects(eObjects);
		}
	}

	
}