package org.deft.operation.gui;

import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import org.deft.configuration.OperationGui;
import org.deft.operation.Ontology2IndividualDataConfiguration;
import org.deft.operation.OperationConfiguration;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.OwlRepresentation;
import org.deft.requirementsontology.IndividualVertex;
import org.deft.requirementsontology.OntologyAccessor;
import org.deft.requirementsontology.ROModel;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.semanticweb.owl.model.OWLOntology;

public class Ontology2IndividualDataOperationGui implements OperationGui {

	private Ontology2IndividualDataConfiguration dataConfig;
	private TableViewer tableViewer;
	private Combo comboDropDown;
	

	@Override
	public Composite createGuiComposite(Composite parent) {
		Composite gui = new Composite(parent, SWT.NONE);

		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 1;
		gui.setLayout(gridLayout);
		
		tableViewer = insertTableViewer(gui);
		comboDropDown = new Combo(gui, SWT.DROP_DOWN | SWT.BORDER);

		return gui;
	}

	
	
	
	private TableViewer insertTableViewer(Composite parent) {
		Table table = createTable(parent, "Individuals");
		GridData gd = new GridData(GridData.FILL_BOTH);
		table.setLayoutData(gd);
		TableViewer tableViewer = new TableViewer(table);
		tableViewer.setUseHashlookup(true);
		tableViewer.setContentProvider(new ArrayContentProvider());
		return tableViewer;
	}

	
	
	
	private Table createTable(Composite parent, String header) {
		
		int style = SWT.BORDER | SWT.SINGLE;
		Table table = new Table(parent, style);

		GridData gridData = new GridData(GridData.FILL_BOTH);
		table.setLayoutData(gridData);

		table.setLinesVisible(true);
		table.setHeaderVisible(true);

		TableColumn column = new TableColumn(table, SWT.CENTER, 0);
		column.setWidth(500);
		column.setText(header);
		
		return table;
	}
	
	
	
	
	
	
	
	@Override
	public void initializeGui(OperationConfiguration config,
			Artifact artifact, ArtifactRepresentation artifactRepr) {
		
		this.dataConfig = (Ontology2IndividualDataConfiguration)config;
		OWLOntology ontology = ((OwlRepresentation)artifactRepr).getOntology();
		OntologyAccessor oa = new OntologyAccessor(ontology);
		ROModel roModel = new ROModel(oa);
		
		roModel.getOntologyIndividualNames();
		List<String> ontologyIndividuals = new LinkedList<String>(roModel.getOntologyIndividualNames());
		Collections.sort(ontologyIndividuals);
		

		fillTableViewer(ontologyIndividuals);
		fillDropdownFilter(roModel);
	}

	

	
	private void fillTableViewer(List<String> allOntologyIndividualNames) {
		tableViewer.setInput(allOntologyIndividualNames.toArray());
		tableViewer.addSelectionChangedListener(new IndividualSelectionListener(tableViewer.getTable()));
		selectFirstLineIfPossibleAndSetConfig(tableViewer);
	}
	


	private void selectFirstLineIfPossibleAndSetConfig(TableViewer tableViewer) {
		Table table = tableViewer.getTable();
		int itemCount = table.getItemCount();
		if (itemCount > 0) {
			table.select(0);
			String text = table.getItem(0).getText();
			dataConfig.setIndividual(text);
		} else {
			dataConfig.setIndividual(null);
		}
	}
	
	
	
	
	
	private void fillDropdownFilter(ROModel roModel) {
		List<String> categoryNames = new LinkedList<String>(roModel.getOntologyMainTypeNames());
		Collections.sort(categoryNames);

		comboDropDown.add("All Entries");
		for (String category : categoryNames) {
			comboDropDown.add(category);
		}
		comboDropDown.select(0);
		comboDropDown.addSelectionListener(new ComboBoxListener(tableViewer, roModel));
	}
	

	
	/**
	 * Updates the OperationConfiguration whenever an individual is checked or unchecked
	 */
	private class IndividualSelectionListener implements ISelectionChangedListener {
		
		private Table table;

		public IndividualSelectionListener(Table table) {
			this.table = table;
		}
		

		@Override
		public void selectionChanged(SelectionChangedEvent event) {
			int idx = table.getSelectionIndex();
			if (idx != -1) {
				TableItem ti = table.getItem(idx);
				String selectedIndividualName = ti.getText();
				dataConfig.setIndividual(selectedIndividualName);
			}
		}
	}

	
	private class ComboBoxListener implements SelectionListener {

		private TableViewer tableViewer;
		private ROModel roModel;
		
		public ComboBoxListener(TableViewer tableViewer, ROModel roModel) {
			this.tableViewer = tableViewer;
			this.roModel = roModel;
		}
		
		@Override
		public void widgetSelected(SelectionEvent e) {
			Combo combo = (Combo)e.getSource();
			int selectedIndex = combo.getSelectionIndex();
			if (selectedIndex == 0) {
				setAllIndividuals();
			} else {
				String category = combo.getItem(selectedIndex);
				setIndividuals(category);
			}
		}
		
		private void setAllIndividuals() {
			List<String> ontologyIndividuals = new LinkedList<String>(roModel.getOntologyIndividualNames());
			Collections.sort(ontologyIndividuals);
			tableViewer.setInput(ontologyIndividuals.toArray());
			selectFirstLineIfPossibleAndSetConfig(tableViewer);
		}
		
		private void setIndividuals(String category) {
			Collection<IndividualVertex> vertexes = roModel.getIndividuals(category);
			List<String> individualNames = new LinkedList<String>();
			for (IndividualVertex vertex : vertexes) {
				individualNames.add(vertex.getSimpleName());
			}
			Collections.sort(individualNames);
			tableViewer.setInput(individualNames.toArray());
			selectFirstLineIfPossibleAndSetConfig(tableViewer);
		}

		@Override
		public void widgetDefaultSelected(SelectionEvent e) {
		}
		
	}


	
}
