package org.deft.operation.gui;

import java.util.LinkedList;
import java.util.List;

import org.deft.configuration.OperationGui;
import org.deft.operation.Ontology2RequirementsGraphConfiguration;
import org.deft.operation.OperationConfiguration;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.OwlRepresentation;
import org.deft.requirementsontology.DirectedROModel;
import org.deft.requirementsontology.IndividualVertex;
import org.deft.requirementsontology.ObjectPropertyEdge;
import org.deft.requirementsontology.OntologyAccessor;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTreeViewer;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Tree;
import org.semanticweb.owl.model.OWLOntology;

import edu.uci.ics.jung.graph.DelegateForest;

public class Ontology2RequirementsGraphOperationGui implements OperationGui {

	private Ontology2RequirementsGraphConfiguration graphConfig;
	private CheckboxTreeViewer treeViewer;
	
	
	@Override
	public Composite createGuiComposite(Composite parent) {
		Composite gui = new Composite(parent, SWT.NONE);

		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 1;
		gui.setLayout(gridLayout);

		insertTreeViewer(gui);
		return null;
	}

	
	private CheckboxTreeViewer insertTreeViewer(Composite parent) {
		Tree tree = new Tree(parent, SWT.BORDER | SWT.CHECK);
		tree.setLayoutData(new GridData(GridData.FILL_BOTH));
		ITreeContentProvider contentProvider = new RequirementsGraphTableContentProvider();
		treeViewer = new CheckboxTreeViewer(tree);
		treeViewer.setContentProvider(contentProvider);
		return treeViewer;
	}

	
	
	
	
	@Override
	public void initializeGui(OperationConfiguration config,
			Artifact artifact, ArtifactRepresentation artifactRepr) {
		
		this.graphConfig = (Ontology2RequirementsGraphConfiguration)config;
		OWLOntology ontology = ((OwlRepresentation)artifactRepr).getOntology();
		graphConfig.initIfIsUnmodified(ontology);
		OntologyAccessor oa = new OntologyAccessor(ontology);
		DirectedROModel roModel = new DirectedROModel(oa);
		roModel.createRefinementForest();

		DelegateForest<IndividualVertex,ObjectPropertyEdge> forest = roModel.getGoalModelForest();
		
		initializeTreeViewer(forest);
		treeViewer.addCheckStateListener(new CheckStateListener(treeViewer));

		saveCheckedIndividualsInConfig(treeViewer);		
	}
	
	
	


	

	
	@SuppressWarnings("deprecation")
	private void initializeTreeViewer(DelegateForest<IndividualVertex,ObjectPropertyEdge> forest) {
		treeViewer.setInput(forest);
		
		treeViewer.expandAll();
		//works only for expanded nodes, therefore we did an expandAll before
		treeViewer.setAllChecked(true);
		//all nodes checked, collapse again
		treeViewer.collapseAll();
		treeViewer.expandToLevel(3);
	}
	

	private void saveCheckedIndividualsInConfig(CheckboxTreeViewer treeViewer) {
		Object[] checkedElements = treeViewer.getCheckedElements();
		List<String> checkedElementNames = new LinkedList<String>();
		for (Object o : checkedElements) {
			if (o instanceof IndividualVertex) {
				IndividualVertex iv = (IndividualVertex)o;
				checkedElementNames.add(iv.getSimpleName());
			}
		}
		graphConfig.setIndividuals(checkedElementNames);
	}
	
	

	
	private class CheckStateListener implements ICheckStateListener {

		private CheckboxTreeViewer treeViewer;
		
		public CheckStateListener(CheckboxTreeViewer treeViewer) {
			this.treeViewer = treeViewer;
		}
		
		@Override
		public void checkStateChanged(CheckStateChangedEvent event) {
			checkOrUncheckChildren(event.getChecked(), event.getElement());
			saveCheckedIndividualsInConfig(treeViewer);
		}
		
		
		private void checkOrUncheckChildren(boolean checked, Object element) {
			treeViewer.setSubtreeChecked(element, checked);
		}
		
	}

}
