package org.deft.operation.idents2styledrunningtext;

import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;

import org.deft.operation.Operation;
import org.deft.operation.OperationConfiguration;
import org.deft.operation.exception.DeftInvalidOperationChainException;
import org.deft.operation.exception.DeftInvalidOperationConfigurationException;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.identifiers.IdentifiersRepresentation;
import org.deft.representation.identifiers.data.Scope;
import org.deft.representation.styleddocument.StyledTextRepresentation;
import org.deft.representation.styleddocument.data.TokenizedStyledDocument;

public class Idents2StyledRunningTextOperation implements Operation {

	@Override
	public ArtifactRepresentation executeOperation(ArtifactRepresentation representation, 
			ArtifactType type, OperationConfiguration config) {
		Scope rootScope = checkAndUnwrap(representation);
		Idents2StyledRunningTextConfiguration i2tConfig = checkAndUnwrap(config);
		String scopeId = i2tConfig.getScopeId();
		Scope selectedScope = rootScope.getDescendantScopeById(scopeId);
		String text = createString(selectedScope);
		TokenizedStyledDocument doc = createStyledDocument(text);
		StyledTextRepresentation srtr = new StyledTextRepresentation(doc);
		return srtr;
	}
	
	
	private String font;
	public void setFont(String font) {
		this.font = font;
	}
	
	private Scope checkAndUnwrap(ArtifactRepresentation rep) {
		if (rep.getType().equals(IdentifiersRepresentation.TYPE)) {
			IdentifiersRepresentation ir = (IdentifiersRepresentation)rep;
			return ir.getRoot();
		} else {
			throw new DeftInvalidOperationChainException();
		}
	}
	
	private Idents2StyledRunningTextConfiguration checkAndUnwrap(OperationConfiguration config) {
		if (config instanceof Idents2StyledRunningTextConfiguration) {
			Idents2StyledRunningTextConfiguration i2tConfig = (Idents2StyledRunningTextConfiguration)config;
			return i2tConfig;
		} else {
			throw new DeftInvalidOperationConfigurationException();
		}
	}
	
	
	private String createString(Scope scope) {
		String text = scope.getFullLocalName();
		return text;
	}
	
	private TokenizedStyledDocument createStyledDocument(String text) {
		StyleContext context = setupBasicStyleContext();
		TokenizedStyledDocument document = new TokenizedStyledDocument(context);
		try {
			document.insertString(document.getLength(), text, context.getStyle("normal"));
		}
		catch (BadLocationException ble) {
			ble.printStackTrace();
		}
		return document;
	}
	
	private StyleContext setupBasicStyleContext() {
		StyleContext context = new StyleContext();
		Style defaultStyle = context.getStyle(StyleContext.DEFAULT_STYLE);

		Style normal = context.addStyle("normal", defaultStyle);
		StyleConstants.setFontFamily(normal, font);
		
		return context;
	}
	


	@Override
	public OperationConfiguration createConfiguration() {
		return new Idents2StyledRunningTextConfiguration();
	}


	@Override
	public String getInputRepresentationType() {
		return IdentifiersRepresentation.TYPE;
	}


	@Override
	public String getResultingRepresentationType() {
		return StyledTextRepresentation.TYPE;
	}

}



