package org.deft.operation.idents2styledrunningtext.gui.data;

import java.util.LinkedList;
import java.util.List;

import org.deft.gui.widgets.TreeAndAutoCompleteTextfieldComposite;
import org.deft.representation.identifiers.data.Scope;
import org.eclipse.jface.viewers.IContentProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.swt.widgets.Composite;

/**
 * Tree+Autocomplete Textfield composite which is used to select a scope for the
 * insertion into a document. 
 */
public class ScopeSelectionComposite extends TreeAndAutoCompleteTextfieldComposite {

	private Scope rootScope;
	
	public ScopeSelectionComposite(Composite parent, int style) {
		super(parent, style);
		super.createContents();
	}

	public void setRootScope(Scope rootScope) {
		this.rootScope = rootScope;
	}
	

	@Override
	protected String[] getAllProposals() {
		if (rootScope != null) {
			List<String> scopeStrings = new LinkedList<String>();
			List<Scope> thisAndDescendants = rootScope.getThisAndDescendentScopesAsList();
			for (Scope scope : thisAndDescendants) {
				String string = scope.getFullyQualifiedName();
				scopeStrings.add(string);
			}
			return scopeStrings.toArray(new String[]{});
		} else {
			return new String[]{};
		}
	}

	
	protected Character getDelimiter() {
		if (rootScope != null) {
			return rootScope.getDelimiter();
		} else {
			return null;
		}
	}

	@Override
	public SelectedScopeContainer createSelectedItemContainer() {
		if (rootScope != null) {
			return new SelectedScopeContainer(rootScope);
		}
		return null;
	}

	public SelectedScopeContainer getSelectedItemContainer() {
		return (SelectedScopeContainer)super.getSelectedItemContainer();
	}
	

	@Override
	protected IContentProvider createContentProvider() {
		return new ScopesContentProvider();
	}

	@Override
	protected ILabelProvider createLabelProvider() {
		return new ScopesLabelProvider();
	}

	@Override
	protected Object getInput() {
		if (rootScope != null) {
			Scope[] input = new Scope[]{rootScope};
			return input;
		}
		return null;
	}

}
