package org.deft.operation.linebreak.wrapper.binding;

import org.deft.repository.ast.TokenNode;

/**
 * Identifier for a rule specifying the concrete instances
 * of both token nodes.
 * 
 * @author Christoph Seidl
 */
public class TokenBindingKey
{
	private TokenNode firstTokenNode;
	private TokenNode secondTokenNode;
	
	/**
	 * Creates a new <code>TokenBindingKey</code>.
	 * 
	 * @param firstTokenNode The concrete instance of the first token node to match.
	 * @param secondTokenNode The concrete instance of the second token node to match.
	 */
	public TokenBindingKey(TokenNode firstTokenNode, TokenNode secondTokenNode)
	{
		assert(firstTokenNode != null);
		assert(secondTokenNode != null);
		
		this.firstTokenNode = firstTokenNode;
		this.secondTokenNode = secondTokenNode;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public boolean equals(Object that)
	{
		if (that instanceof TokenBindingKey)
		{
			TokenBindingKey thatKey = (TokenBindingKey) that;
			
			if (firstTokenNode.equals(thatKey.firstTokenNode) && secondTokenNode.equals(thatKey.secondTokenNode))
			{
				return true;
			}
		}
		
		return false;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public int hashCode()
	{
		return firstTokenNode.hashCode() + secondTokenNode.hashCode();
	}
	
	/**
	 * {@inheritDoc}
	 */
	public String toString()
	{
		return "<" + firstTokenNode + ", " + secondTokenNode + ">";
	}
	
	/**
	 * For testing only!
	 * 
	 * @return The first token node of this key.
	 */
	public TokenNode getFirstTokenNode()
	{
		return firstTokenNode;
	}

	/**
	 * For testing only!
	 * 
	 * @return The second token node of this key.
	 */
	public TokenNode getSecondTokenNode()
	{
		return secondTokenNode;
	}
}
