package org.deft.operation.linebreak.wrapper.preprocessing;

import java.util.Iterator;

import org.deft.operation.linebreak.wrapper.binding.TokenBindingContainer;
import org.deft.operation.linebreak.wrapper.binding.TokenBindingKey;
import org.deft.operation.linebreak.wrapper.options.LineWrapperOptions;
import org.deft.operation.linebreak.wrapper.rules.alignment.AlignmentRule;
import org.deft.operation.linebreak.wrapper.rules.wrapping.WrappingRule;
import org.deft.operation.linebreak.wrapper.util.Caret;
import org.deft.operation.linebreak.wrapper.util.CodeLine;
import org.deft.repository.ast.TokenNode;

/**
 * A pre-processing step applying the calculated values of wrapping and
 * alignment rules.
 * 
 * @author Christoph Seidl
 */
public class WrappingRulesPreprocessingStep extends PreprocessingStep
{
	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void doPreprocess(LineWrapperOptions lineWrapperOptions, CodeLine physicalLine, CodeLine nextPhysicalLine, TokenBindingContainer tbc, Caret caret)
	{
		int colOfOriginalLine = caret.convertCol(physicalLine.get(0).getStartCol());
		int colOfNextLine = -1;
		
		if (nextPhysicalLine != null)
		{
			colOfNextLine = caret.convertCol(nextPhysicalLine.get(0).getStartCol());
		}
		
		Iterator<TokenNode> iterator = physicalLine.iterator();
		TokenNode oldTokenNode = null;
		
		while(iterator.hasNext())
		{
			TokenNode currentTokenNode = iterator.next();

			if (oldTokenNode!= null)
			{
				//get matching wrapping rule if line would be wrapped before this token
				WrappingRule wr = lineWrapperOptions.findMatchingWrappingRule(oldTokenNode, currentTokenNode);
				AlignmentRule ar = lineWrapperOptions.findMatchingAlignmentRule(oldTokenNode, currentTokenNode);
				TokenBindingKey key = new TokenBindingKey(oldTokenNode, currentTokenNode);
								
				if (wr != null)
				{
					tbc.setBindingStrength(key, wr.getBindingStrength());
				}
				
				if (ar != null)
				{
					int alignmentCol = ar.determineColAfterWrap(colOfOriginalLine, colOfNextLine);
					tbc.setAlignmentCol(key, alignmentCol);				
				}
			}
			
			oldTokenNode = currentTokenNode;
		}
	}

}
