package org.deft.operation.linebreak.wrapper.rules;

/**
 * Identifier for a rule specifying the types of both tokens.
 */
public class RuleIdentifier
{
	private String firstTokenName;
	private String secondTokenName;
	
	/**
	 * Creates a new <code>RuleIdentifier</code>.
	 * 
	 * @param firstTokenName The name of the first token type.
	 * @param secondTokenName The name of the second token type.
	 */
	public RuleIdentifier(String firstTokenName, String secondTokenName)
	{
		assert(firstTokenName != null);
		assert(secondTokenName != null);
		
		this.firstTokenName = firstTokenName;
		this.secondTokenName = secondTokenName;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public boolean equals(Object other)
	{
		if (other instanceof RuleIdentifier)
		{
			RuleIdentifier identifier = (RuleIdentifier) other;
			
			if (firstTokenName.equals(identifier.firstTokenName) && secondTokenName.equals(identifier.secondTokenName))
			{
				return true;
			}
		}

		return false;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public int hashCode()
	{
		return firstTokenName.hashCode() + secondTokenName.hashCode();
	}

	/**
	 * Returns the name of the first token type.
	 * 
	 * @return The name of the first token type.
	 */
	public String getFirstTokenName()
	{
		return firstTokenName;
	}

	/**
	 * Returns the name of the second token type.
	 * 
	 * @return The name of the second token type.
	 */
	public String getSecondTokenName()
	{
		return secondTokenName;
	}
}
