package org.deft.operation.linebreak.wrapper.rules.alignment;

import org.deft.operation.linebreak.wrapper.options.LineWrapperOptionsDefaults;
import org.deft.operation.linebreak.wrapper.rules.RuleIdentifier;
import org.deft.operation.linebreak.wrapper.rules.RulesContainer;

/**
 * Container for a number of alignment rules.
 * 
 * @author Christoph Seidl
 */
public class AlignmentRulesContainer extends RulesContainer<AlignmentRule>
{
	/**
	 * Creates a new <code>AlignmentRulesContainer</code>.
	 */
	public AlignmentRulesContainer()
	{
		//Take care that there is always a default wrapping rule in place.
		setDefaultRule(createDefaultAlignmentRule());
	}
	
	/**
	 * Creates a default alignment rule if it is not specified explicitly.
	 * 
	 * @return The default alignment rule.
	 */
	private static AlignmentRule createDefaultAlignmentRule()
	{
		RuleIdentifier ri = new RuleIdentifier(LineWrapperOptionsDefaults.TOKEN, LineWrapperOptionsDefaults.TOKEN);
		
		String firstContext = LineWrapperOptionsDefaults.CONTEXT;
		String secondContext = LineWrapperOptionsDefaults.CONTEXT;

		int anchor = Alignment.createAnchor(LineWrapperOptionsDefaults.ANCHOR_NAME);
		int indentation = LineWrapperOptionsDefaults.INDENTATION;
		
		AlignmentRule ar = new AlignmentRule(ri, firstContext, secondContext);
		
		ar.addAlignment(new Alignment(anchor, indentation));
		
		return ar;
	}
	
	/**
	 * Set the default alignment rule of this container.
	 * 
	 * @param defaultAlignmentRule The default alignment rule.
	 */
	public void setDefaultRule(AlignmentRule defaultAlignmentRule)
	{
		//Ensure a valid default wrapping rule at all cost.
		if (defaultAlignmentRule != null)
		{
			super.setDefaultRule(defaultAlignmentRule);
		}
	}
}
