package org.deft.operation.linebreak.wrapper.rules.wrapping;

import org.deft.operation.linebreak.wrapper.rules.Rule;
import org.deft.operation.linebreak.wrapper.rules.RuleIdentifier;

/**
 * Class for rules containing the binding strength of two types of tokens.
 * 
 * @author Christoph Seidl
 */
public class WrappingRule extends Rule
{
	private int bindingStrength;
		
	/**
	 * Creates a new <code>WrappingRule</code>.
	 * 
	 * @param ruleIdentifier The combination of types of token nodes.
	 * @param firstContext The context of the first token node.
	 * @param secondContext The context of the second token node.
	 * @param bindingStrength The binding strength of two types of tokens. The greater the value the less likely a line wrap will appear at this position.
	 */
	public WrappingRule(RuleIdentifier ruleIdentifier, String firstContext, String secondContext, int bindingStrength)
	{
		super(ruleIdentifier, firstContext, secondContext);
		
		this.bindingStrength = bindingStrength;
	}

	/**
	 * Returns the binding strength of this rule.
	 * 
	 * @return The binding strength of this rule.
	 */
	public int getBindingStrength()
	{
		return bindingStrength;
	}
}
