package org.deft.operation.oclfilter;

import java.util.List;

import org.deft.emftext.EmfTextManager;
import org.deft.operation.Operation;
import org.deft.operation.OperationConfiguration;
import org.deft.operation.emftextfilter.EmfTextFilterConfiguration;
import org.deft.operation.exception.DeftInvalidOperationChainException;
import org.deft.operation.exception.DeftInvalidOperationConfigurationException;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.styleddocument.DocumentContentContainerRepresentation;
import org.deft.representation.text.TextRepresentation;
import org.deft.util.EcoreUtil;
import org.eclipse.emf.ecore.EObject;
import org.emftext.access.resource.ILocationMap;
import org.emftext.access.resource.IResource;

public class OclFilterOperation implements Operation {

	private EmfTextManager mgr = new EmfTextManager();
	
	@Override
	public TextRepresentation executeOperation(ArtifactRepresentation representation,
			ArtifactType type, OperationConfiguration config) {
		String text = checkAndUnwrap(representation);
		EObject modelRoot = EcoreUtil.loadRegisteredEcoreInstance("ocl", text);
		IResource resource = mgr.getEmfTextResource(modelRoot);
		EmfTextFilterConfiguration emfTextConfig = checkAndUnwrap(config);
		List<EObject> eObjects = emfTextConfig.getEObjects(modelRoot.eResource());
		if (eObjects.isEmpty()) {
			int start = 0;
			int end = text.length();
			TextRepresentation tr = new TextRepresentation(text, start, end);
			return tr;
		} else {
			String uri = emfTextConfig.getUris().get(0);
			EObject eo = resource.getEObject(uri);
			ILocationMap locationMap = resource.getLocationMap();
			int start = locationMap.getCharStart(eo);
			int end = locationMap.getCharEnd(eo);
			TextRepresentation tr = new TextRepresentation(text, start, end);
			return tr;
		}
	}

	
	private String checkAndUnwrap(ArtifactRepresentation rep) {
		if (rep.getType().equals(getInputRepresentationType())) {
			TextRepresentation codeRep = (TextRepresentation)rep;
			return codeRep.getText();
		} else {
			throw new DeftInvalidOperationChainException();
		}
	}
	

	private EmfTextFilterConfiguration checkAndUnwrap(OperationConfiguration config) {
		if (config instanceof EmfTextFilterConfiguration) {
			EmfTextFilterConfiguration emfTextConfig = (EmfTextFilterConfiguration)config;
			return emfTextConfig;
		} else {
			throw new DeftInvalidOperationConfigurationException();
		}
	}
	
	
	@Override
	public OperationConfiguration createConfiguration() {
		return new EmfTextFilterConfiguration();
	}


	
	@Override
	public String getInputRepresentationType() {
		return TextRepresentation.TYPE;
	}


	@Override
	public String getResultingRepresentationType() {
		return DocumentContentContainerRepresentation.TYPE;
	}

}
