package org.deft.operation.oclfilter.gui;

import java.util.LinkedList;
import java.util.List;

import org.deft.emftext.EmfTextManager;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.navigator.ICommonContentExtensionSite;
import org.eclipse.ui.navigator.ICommonContentProvider;
import org.emftext.access.resource.ILocationMap;
import org.emftext.access.resource.IResource;

import tudresden.ocl20.pivot.language.ocl.AttributeContextDeclarationCS;
import tudresden.ocl20.pivot.language.ocl.ClassifierContextDeclarationCS;
import tudresden.ocl20.pivot.language.ocl.ContextDeclarationCS;
import tudresden.ocl20.pivot.language.ocl.InitOrDeriveValueCS;
import tudresden.ocl20.pivot.language.ocl.OperationContextDeclarationCS;
import tudresden.ocl20.pivot.language.ocl.OperationDefinitionInContextCS;
import tudresden.ocl20.pivot.language.ocl.PackageDeclarationCS;
import tudresden.ocl20.pivot.language.ocl.TypePathNameCS;

public class OclTreeProvider implements ICommonContentProvider, ILabelProvider {

	private String completeOclFileText;	
	private EmfTextManager mgr = new EmfTextManager();

	
	public OclTreeProvider(String completeOclFileText) {
		this.completeOclFileText = completeOclFileText;
	}
	
	@Override
	public Object[] getElements(Object inputElement) {
		PackageDeclarationCS root = (PackageDeclarationCS)inputElement;
		List<EObject> children = root.eContents();
		List<ContextDeclarationCS> cd = new LinkedList<ContextDeclarationCS>(); 
		for (EObject eo : children) {
			if (eo instanceof ContextDeclarationCS) {
				cd.add((ContextDeclarationCS)eo);
			}
		}
		return cd.toArray();
	}

	@Override
	public Object[] getChildren(Object parentElement) {
		return null;
	}

	@Override
	public Object getParent(Object element) {
		return null;
	}

	@Override
	public boolean hasChildren(Object element) {
		return false;
	}

	@Override
	public void dispose() {
	}

	@Override
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
	}

	@Override
	public void restoreState(IMemento aMemento) {
	}

	@Override
	public void saveState(IMemento aMemento) {
	}

	@Override
	public void init(ICommonContentExtensionSite aConfig) {
	}

	@Override
	public void addListener(ILabelProviderListener listener) {
	}

	@Override
	public boolean isLabelProperty(Object element, String property) {
		return false;
	}

	@Override
	public void removeListener(ILabelProviderListener listener) {
	}

	@Override
	public Image getImage(Object element) {
		return null;
	}
	

	/**
	 * This method returns the labels for all 3 kinds of ContextDeclarationCS objects.
	 * If the parameter element is not one of these ContextDeclarationCS, its
	 * toString() is called and returned.
	 */
	@Override
	public String getText(Object element) {
		if (element instanceof OperationContextDeclarationCS) {
			String name = getName((OperationContextDeclarationCS)element);
			return name;
		} else if (element instanceof ClassifierContextDeclarationCS) {
			String name = getName((ClassifierContextDeclarationCS)element);
			return name;
		} else if (element instanceof AttributeContextDeclarationCS) {
			String name = getName((AttributeContextDeclarationCS)element);
			return name;
		}
		return element.toString();
	}
	
	private String getName(OperationContextDeclarationCS eOpContDecl) {
		OperationDefinitionInContextCS eOpDef = eOpContDecl.getOperation();
		String name = getStringFor(eOpDef);
		return name;
	}
	
	private String getName(ClassifierContextDeclarationCS eClassifierContDecl) {
		EObject eOpDef = eClassifierContDecl.getTypeName();
		String name = getStringFor(eOpDef);
		return name;
	}

	/**
	 * Computing the name for AttributeContextDeclarations is a bit more difficult.
	 * There seems to be no EObject representing the declaration name. Therefore we
	 * first get the TypeName (with which the name starts) and then the InitOrDeriveValues
	 * (which come directly after the name and some whitespace). The name can then be computed
	 * as the substring between the TypeName start and the values start. Finally the
	 * whitespace is trimmed. 
	 */
	private String getName(AttributeContextDeclarationCS eAttributeContDecl) {
		TypePathNameCS tpn = eAttributeContDecl.getTypeName();
		EList<InitOrDeriveValueCS> values = eAttributeContDecl.getInitOrDeriveValue();
		IResource resource = mgr.getEmfTextResource(eAttributeContDecl);
		ILocationMap locationMap = resource.getLocationMap();
		int start = locationMap.getCharStart(tpn);
		int startValues = locationMap.getCharStart(values.get(0));
		String name = completeOclFileText.substring(start, startValues);
		name = name.trim();
		return name;
	}

	/**
	 * This method takes EObject which corresponds to the context name
	 * specified in the EMFText OCL file. The actual name is NOT stored in the Ecore
	 * model we are currently working with, so we explicitly have to use the 
	 * textual representation. 
	 * The method will get the start and end position and
	 * and extract the return value (the name) from the OCL file content
	 * that was passed in the constructor.
 	 */
	private String getStringFor(EObject eo) {
		IResource resource = mgr.getEmfTextResource(eo);
		ILocationMap locationMap = resource.getLocationMap();
		int start = locationMap.getCharStart(eo);
		int end = locationMap.getCharEnd(eo);
		String name = completeOclFileText.substring(start, end + 1);
		return name;
	}
}