package org.deft.operation.replacetabs;

import org.deft.operation.AbstractOperationConfiguration;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class ReplaceTabsConfiguration extends AbstractOperationConfiguration {

	public static String ID = "org.deft.operation.replacetabs";
	
	private int tabWidth;
	
	
	public ReplaceTabsConfiguration() {
		setTabWidth(4);
	}
	
	
	public void setTabWidth(int tabWidth) {
		this.tabWidth = tabWidth;
	}
	
	public int getTabWidth() {
		return tabWidth;
	}
	
	
	@Override
	public String getId() {
		return ID;
	}
	
	@Override
	public void loadFromXml(Element element) {
		if (isValid(element)) {
			loadTabWidth(element);
		}
	}
	
	private void loadTabWidth(Element element) {
		Element eTabWidth = (Element)element.getElementsByTagName("tabwidth").item(0);
		String sTabWidth = eTabWidth.getTextContent();
		if (isIntegerString(sTabWidth)) {
			this.tabWidth = Integer.parseInt(sTabWidth);
		}
	}
	
	private boolean isIntegerString(String s) {
		try {
			Integer.parseInt(s);
			return true;
		}
		catch (NumberFormatException nfe) {
			return false;
		}
	}
	


	@Override
	public void addToXml(Element root) {
		addTabWidthToXml(root);
	}
	
	private void addTabWidthToXml(Element root) {
		Document doc = root.getOwnerDocument();
		Element eTabWidth = doc.createElement("tabwidth");
		eTabWidth.setTextContent(Integer.toString(this.tabWidth));
		root.appendChild(eTabWidth);
	}

	@Override
	protected String getLocalSchemaFileLocation() {
		return "resources/schema.xsd";
	}

}
