package org.deft.operation.tempfile2ecorediag;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.gmf.runtime.notation.Diagram;
import org.eclipse.swt.widgets.Display;

public abstract class DiagramLoader {

	/**
	 * There can be multiple diagrams in a diagram file. Return the first one (or the only one)
	 */
	public static Diagram getDiagram(URI diagramURI) {
		EditorRunnable editorRunnable = new EditorRunnable(diagramURI);

		boolean runningInGUI = false;
		if (Display.getCurrent() != null)
			runningInGUI = Thread.currentThread().equals(
					Display.getCurrent().getThread());

		if (runningInGUI) {
			editorRunnable.run();
		} else {
			Display display = Display.getDefault();
			display.syncExec(editorRunnable);
		}

		return editorRunnable.getDiagram();
	}
	
	/**
	 * There can be multiple diagrams in a diagram file. Return them all.
	 */
	public static List<Diagram> getDiagrams(URI diagramURI) {
		List<Diagram> diagrams = new LinkedList<Diagram>();
		Diagram firstDiag = getDiagram(diagramURI);
		Resource resource = firstDiag.eResource();
		for (Iterator<EObject> it = resource.getAllContents(); it.hasNext();) {
			EObject eo = it.next();
			if (eo instanceof Diagram) {
				Diagram d = (Diagram)eo;
				diagrams.add(d);
			}
		}
		return diagrams;
	}

}
