package org.deft.operation.tempfile2ecorediag;

import java.util.List;

import org.deft.operation.EmptyOperationConfiguration;
import org.deft.operation.Operation;
import org.deft.operation.OperationConfiguration;
import org.deft.operation.exception.DeftInvalidOperationChainException;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.artifacttype.ecore.EcoreTempFileRepresentation;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.ecorediag.EcoreDiagRepresentation;
import org.eclipse.emf.common.util.URI;
import org.eclipse.gmf.runtime.notation.Diagram;

public class TempFile2EcoreDiagOperation implements Operation {


	@Override
	public ArtifactRepresentation executeOperation(ArtifactRepresentation representation, 
			ArtifactType type, OperationConfiguration config) {
		String tempFile = checkAndUnwrap(representation);
		List<Diagram> diagrams = createEcoreDiagrams(tempFile);
		EcoreDiagRepresentation edr = new EcoreDiagRepresentation(diagrams);
		return edr;
	}
	
	
	private String checkAndUnwrap(ArtifactRepresentation rep) {
		if (rep.getType().equals(EcoreTempFileRepresentation.TYPE)) {
			EcoreTempFileRepresentation tfr = (EcoreTempFileRepresentation)rep;
			return tfr.getPath();
		} else {
			throw new DeftInvalidOperationChainException();
		}
	}
	

	
	private List<Diagram> createEcoreDiagrams(String tempFile) {
		URI uri = URI.createFileURI(tempFile);
		List<Diagram> diagrams = DiagramLoader.getDiagrams(uri);
		return diagrams;
	}

	
	@Override
	public OperationConfiguration createConfiguration() {
		return new EmptyOperationConfiguration();
	}


	@Override
	public String getInputRepresentationType() {
		return EcoreTempFileRepresentation.TYPE;
	}


	@Override
	public String getResultingRepresentationType() {
		return EcoreDiagRepresentation.TYPE;
	}

}



