package org.deft.operation.tempfile2ecorediag;

import java.io.File;
import java.util.LinkedList;
import java.util.List;

import org.deft.operation.EmptyOperationConfiguration;
import org.deft.operation.Operation;
import org.deft.operation.OperationConfiguration;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.artifacttype.ecore.EcoreTempFileRepresentation;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.ecore.EcoreRepresentation;
import org.deft.util.EcoreUtil;
import org.eclipse.emf.ecore.EObject;

/**
 * Same as TempFile2EcoreOperation. The only difference is that if the artifact
 * has dependent ecore artifacts, their contents are also added to the representation. 
 */
public class TempFile2MultiEcoreOperation implements Operation {


	@Override
	public ArtifactRepresentation executeOperation(ArtifactRepresentation representation, 
			ArtifactType type, OperationConfiguration config) {
		EcoreTempFileRepresentation tfr = (EcoreTempFileRepresentation)representation;
		File tempFile = new File(tfr.getPath());
		EObject root = EcoreUtil.loadEcoreModel(tempFile);
		List<EObject> roots = new LinkedList<EObject>();
		roots.add(root);
		for (String basicArtifactPath : tfr.getPathsForBasicArtifacts()) {
			File f = new File(basicArtifactPath);
			EObject basicArtifactRoot = EcoreUtil.loadEcoreModel(f);
			roots.add(basicArtifactRoot);
		}
		EcoreRepresentation er = new EcoreRepresentation(roots);
		return er;
	}


	
	@Override
	public OperationConfiguration createConfiguration() {
		return new EmptyOperationConfiguration();
	}


	@Override
	public String getInputRepresentationType() {
		return EcoreTempFileRepresentation.TYPE;
	}


	@Override
	public String getResultingRepresentationType() {
		return EcoreRepresentation.TYPE;
	}

}



