package org.deft.operation.text2ast;


import org.deft.artifacttypesupport.codefile.Parser;
import org.deft.operation.EmptyOperationConfiguration;
import org.deft.operation.Operation;
import org.deft.operation.OperationConfiguration;
import org.deft.operation.exception.DeftInvalidOperationChainException;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.artifacttype.Validator;
import org.deft.repository.ast.TreeNodeRoot;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.repository.exception.DeftInvalidStateException;
import org.deft.representation.text.TextRepresentation;
import org.deft.representation.treenode.TreeNodeRepresentation;


public class TextToTreeNodeOperation implements Operation {

	private ArtifactType artifactType;
	
	@Override
	public TreeNodeRepresentation executeOperation(ArtifactRepresentation representation,
			ArtifactType type, OperationConfiguration config) {
		this.artifactType = type;
		String code = checkAndUnwrap(representation);
		TreeNodeRoot ast = createAst(code);
		TreeNodeRepresentation tnr = new TreeNodeRepresentation(ast);
		return tnr;
	}
	
	
	private String checkAndUnwrap(ArtifactRepresentation rep) {
		if (rep.getType().equals(getInputRepresentationType())) {
			TextRepresentation codeRep = (TextRepresentation)rep;
			return codeRep.getText();
		} else {
			throw new DeftInvalidOperationChainException();
		}
	}
	
	
	private TreeNodeRoot createAst(String code) {
		Parser parser = getParser();
		if (parser == null) {
			throw new DeftInvalidStateException("No parser has been found for TextToTreeNodeOperation.");
		}
		TreeNodeRoot root = parser.getAst(code);
		return root;
	}
	
	
	public Parser getParser() {
		Validator validator = artifactType.getValidator();
		if (validator instanceof Parser) {
			return (Parser)validator;
		}
		return null;
	}
	

	@Override
	public OperationConfiguration createConfiguration() {
		return new EmptyOperationConfiguration();
	}


	@Override
	public String getInputRepresentationType() {
		return TextRepresentation.TYPE;
	}


	@Override
	public String getResultingRepresentationType() {
		return TreeNodeRepresentation.TYPE;
	}

}



