package org.deft.operationchain.code2styledcode;

import org.deft.artifacttypesupport.codefile.PlainCodeFileArtifactType;
import org.deft.operation.AbstractOperationChain;
import org.deft.operation.ast2styledtext.TreeNodeToStyledCodeOperation;
import org.deft.operation.astdecoration.AstDecorationOperation;
import org.deft.operation.astfilter.AstFilterOperation;
import org.deft.operation.astmodifier.AstModifierOperation;
import org.deft.operation.bytearray2charbuffer.ByteArray2CharBufferOperation;
import org.deft.operation.charbuffer2text.CharBuffer2TextOperation;
import org.deft.operation.linebreak.LineBreakOperation;
import org.deft.operation.replacetabs.ReplaceTabsOperation;
import org.deft.operation.text2ast.TextToTreeNodeOperation;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.bytearray.ByteArrayRepresentation;

/**
 * Operation chain that transforms source code into styled text.
 */
public class CodeToStyledCodeOperationChain extends AbstractOperationChain {
	
	public static String ID = "org.deft.operationchain.code2styledcode";
	
	public CodeToStyledCodeOperationChain() {
		setupChain();
	}
	
	private void setupChain() {
		addOperation(new ByteArray2CharBufferOperation());
		addOperation(new CharBuffer2TextOperation());
		addOperation(new ReplaceTabsOperation());
		addOperation(new TextToTreeNodeOperation());
		addOperation(new AstFilterOperation());
		addOperation(new AstModifierOperation());
		addOperation(new AstDecorationOperation());
		addOperation(new LineBreakOperation());
		addOperation(new TreeNodeToStyledCodeOperation());
	}
	
	
	
	@Override
	public String getId() {
		return ID;
	}
	

	@Override
	public ArtifactRepresentation getInitialArtifactRepresentation(Artifact artifact, int revision) {
		byte[] content = artifact.getArtifactContent(revision);
		ByteArrayRepresentation rep = new ByteArrayRepresentation(content);
		return rep;
	}
	
	

	@Override
	public boolean isValidInputArtifactType(ArtifactType artifactType) {
		boolean isCodeFile = artifactType instanceof PlainCodeFileArtifactType;
		return isCodeFile;
	}
	
	@Override
	public String getDescription() {
		return "Styled Code";
	}

}
