package org.deft.operationchain.ecore2identtext;

import org.deft.operation.AbstractOperationChain;
import org.deft.operation.IdentifierOperationChain;
import org.deft.operation.bytearray2ecoreinstance.XmiAsByteArray2EcoreInstanceOperation;
import org.deft.operation.ecore2idents.Ecore2IdentsOperation;
import org.deft.operation.idents2styledrunningtext.Idents2StyledRunningTextOperation;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.artifacttype.ecore.EcoreArtifactType;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.bytearray.ByteArrayRepresentation;

/**
 * Operation chain that transforms identifiers from Ecore models into text.
 */
public class Ecore2IdentifierOperationChain extends AbstractOperationChain implements IdentifierOperationChain {
	
	public static String ID = "org.deft.operationchain.ecore2identtext";
	
	public Ecore2IdentifierOperationChain() {
		setupChain();
	}
	
	private void setupChain() {
		addOperation(new XmiAsByteArray2EcoreInstanceOperation());
		addOperation(new Ecore2IdentsOperation());
		Idents2StyledRunningTextOperation i2so = new Idents2StyledRunningTextOperation();
		i2so.setFont("Courier New");
		addOperation(i2so);
	}
	

	
	@Override
	public String getId() {
		return ID;
	}
	

	@Override
	public ArtifactRepresentation getInitialArtifactRepresentation(Artifact artifact, int revision) {
		byte[] content = artifact.getArtifactContent(revision);
		ByteArrayRepresentation rep = new ByteArrayRepresentation(content);
		return rep;
	}

	@Override
	public boolean isValidInputArtifactType(ArtifactType artifactType) {
		boolean isEcoreArtifactType = artifactType.getTypeId().equals(EcoreArtifactType.ID);
		return isEcoreArtifactType;
	}

	@Override
	public String getDescription() {
		return "Identifier";
	}


	
}
