package org.deft.operationchain.ecore2identtext;

import java.io.File;
import java.io.IOException;

import org.deft.operation.AbstractOperationChain;
import org.deft.operation.IdentifierOperationChain;
import org.deft.operation.ecore2idents.EcoreDiag2IdentsOperation;
import org.deft.operation.idents2styledrunningtext.Idents2StyledRunningTextOperation;
import org.deft.operation.tempfile2ecorediag.TempFile2EcoreDiagOperation;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.artifacttype.ecore.EcoreTempFileRepresentation;
import org.deft.repository.artifacttype.ecorediag.EcoreDiagramArtifactType;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.repository.datamodel.Project;
import org.deft.util.FileUtil;

/**
 * Operation chain that transforms identifiers from Ecore models into text.
 */
public class EcoreDiag2IdentifierOperationChain extends AbstractOperationChain implements IdentifierOperationChain {
	
	public static String ID = "org.deft.operationchain.ecorediag2identtext";
	
	public EcoreDiag2IdentifierOperationChain() {
		setupChain();
	}
	
	private void setupChain() {
		addOperation(new TempFile2EcoreDiagOperation());
		addOperation(new EcoreDiag2IdentsOperation());
		Idents2StyledRunningTextOperation i2so = new Idents2StyledRunningTextOperation();
		i2so.setFont("Courier New");
		addOperation(i2so);
	}
	

	
	@Override
	public String getId() {
		return ID;
	}
	

	@Override
	public ArtifactRepresentation getInitialArtifactRepresentation(Artifact artifact, int revision) {
		try {
			//always use different folder to store ecore diagram and ecore file, otherwise some Eclipse/GMF internal
			//caching mechanism sees same diagram location and will not compute a new image
			Project project = artifact.getProject();
			File projectTempDir = project.getTempDirectory();
			File tempDir = FileUtil.getUniqueSubDirIn(projectTempDir);
			tempDir.deleteOnExit();
			EcoreTempFileRepresentation etfr = new EcoreTempFileRepresentation(tempDir.getAbsolutePath());
			etfr.writeArtifactAndDependentFiles(artifact, revision);
			return etfr;
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public boolean isValidInputArtifactType(ArtifactType artifactType) {
		boolean isEcoreDiag = artifactType.getTypeId().equals(EcoreDiagramArtifactType.ID);
		return isEcoreDiag;
	}

	@Override
	public String getDescription() {
		return "Identifier";
	}


	
}
