package org.deft.operationchain.ecorediag2image;

import java.io.File;
import java.io.IOException;

import org.deft.operation.AbstractOperationChain;
import org.deft.operation.ecorediag2image.EcoreDiag2ImageOperation;
import org.deft.operation.tempfile2ecorediag.TempFile2EcoreDiagOperation;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.artifacttype.ecore.EcoreTempFileRepresentation;
import org.deft.repository.artifacttype.ecorediag.EcoreDiagramArtifactType;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.repository.datamodel.Project;
import org.deft.util.FileUtil;

/**
 * Operation chain that transforms an Ecore diagram (contents of a .ecorediag file)
 * into an image.
 */
public class EcoreDiagram2ImageOperationChain extends AbstractOperationChain {
	
	public static String ID = "org.deft.operationchain.ecorediag2image";
	private File tempDir;
	
	public EcoreDiagram2ImageOperationChain() {
		setupChain();
	}
	
	private void setupChain() {
		addOperation(new TempFile2EcoreDiagOperation());
		addOperation(new EcoreDiag2ImageOperation());
	}
	
	@Override
	public String getId() {
		return ID;
	}
	

	@Override
	public ArtifactRepresentation getInitialArtifactRepresentation(Artifact artifact, int revision) {
		try {
			//always use different folder to store ecore diagram and ecore file, otherwise some Eclipse/GMF internal
			//caching mechanism sees same diagram location and will not compute a new image
			Project project = artifact.getProject();
			File projectTempDir = project.getTempDirectory();
			tempDir = FileUtil.getUniqueSubDirIn(projectTempDir);
			tempDir.deleteOnExit();
			EcoreTempFileRepresentation etfr = new EcoreTempFileRepresentation(tempDir);
			etfr.writeArtifactAndDependentFiles(artifact, revision);
			return etfr;
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	
	@Override
	public void cleanUpAfterExecution() {
		if (tempDir != null) {
			FileUtil.deleteDirectory(tempDir);
		}
	}


	

	@Override
	public boolean isValidInputArtifactType(ArtifactType artifactType) {
		boolean isEcoreDiag = artifactType.getTypeId().equals(EcoreDiagramArtifactType.ID);
		return isEcoreDiag;
	}

	@Override
	public String getDescription() {
		return "Ecore Diagram";
	}

}
