package org.deft.operationchain.ecoreinstance2ecoreinstance;

import org.deft.operation.AbstractOperationChain;
import org.deft.operation.bytearray2ecoreinstance.XmiAsByteArray2EcoreInstanceOperation;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.artifacttype.ecore.EcoreInstanceArtifactType;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.bytearray.ByteArrayRepresentation;

/**
 * Operation chain that just takes an EcoreInstanceArtifactType and produces an
 * EcoreInstanceRepresentation from it.
 */
public class EcoreInstance2EcoreInstanceOperationChain extends AbstractOperationChain {
	
	public static String ID = "org.deft.operationchain.ecoreinstance2ecoreinstance";
	
	public EcoreInstance2EcoreInstanceOperationChain() {
		setupChain();
	}
	
	
	private void setupChain() {
		addOperation(new XmiAsByteArray2EcoreInstanceOperation());
	}
	
	
	@Override
	public String getId() {
		return ID;
	}
	

	@Override
	public ArtifactRepresentation getInitialArtifactRepresentation(Artifact artifact, int revision) {
		byte[] content = artifact.getArtifactContent(revision);
		ByteArrayRepresentation rep = new ByteArrayRepresentation(content);
		return rep;
	}



	@Override
	public boolean isValidInputArtifactType(ArtifactType artifactType) {
		boolean isEcoreImageType = artifactType.getTypeId().equals(EcoreInstanceArtifactType.ID);
		return isEcoreImageType;
	}

	
	@Override
	public String getDescription() {
		return "Ecore Instance";
	}

}
