package org.deft.operationchain.ecoreuml2associationslisting;

import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;

import javax.swing.text.StyleContext;

import org.deft.operation.Operation;
import org.deft.operation.OperationConfiguration;
import org.deft.operation.exception.DeftInvalidOperationChainException;
import org.deft.operation.exception.DeftInvalidOperationConfigurationException;
import org.deft.operationchain.ecoreuml2associationslisting.style.LocalStyle;
import org.deft.operationchain.ecoreuml2associationslisting.textcompute.AssociationsExplanationBuilder;
import org.deft.operationchain.ecoreuml2associationslisting.textcompute.AssociationsTermBuilder;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.ecore.EcoreInstanceRepresentation;
import org.deft.representation.ecore.EcoreRepresentation;
import org.deft.representation.styleddocument.StyledDefinitionListingRepresentation;
import org.deft.representation.styleddocument.data.DefinitionListing;
import org.deft.representation.styleddocument.data.TokenizedStyledDocument;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;

public class EcoreUml2AssociationsListingOperation implements Operation {

	@Override
	public ArtifactRepresentation executeOperation(ArtifactRepresentation representation, 
			ArtifactType type, OperationConfiguration config) {
		EObject modelRoot = checkAndUnwrap(representation);
		EcoreUml2AssociationsListingConfiguration listingConfig = checkAndUnwrap(config);
		String className = listingConfig.getClassName();
		EClass eClassToDocument = getEClass(modelRoot, className);

		DefinitionListing listing = new DefinitionListing("No additional associations");
		createStyledListingElements(eClassToDocument, listing);
		StyledDefinitionListingRepresentation rep = new StyledDefinitionListingRepresentation(listing);
		return rep;
	}

	
	
	private void createStyledListingElements(EClass eClass, DefinitionListing listing) {
		StyleContext context = new LocalStyle().getStyleContext();

		List<EReference> references = new LinkedList<EReference>(eClass.getEReferences());
		AssociationsTermBuilder tb = new AssociationsTermBuilder();
		AssociationsExplanationBuilder eb = new AssociationsExplanationBuilder();
		Collections.sort(references, new NameComparator());
		
		for (EReference reference : references) {
			TokenizedStyledDocument termDoc = new TokenizedStyledDocument(context);
			tb.addAssociationsTerm(reference, termDoc);

			TokenizedStyledDocument explDoc = new TokenizedStyledDocument(context);
			eb.addAssociationsExplanation(reference, explDoc);
			listing.addDefinition(termDoc, explDoc);
		}
	}
	
	
	private class NameComparator implements Comparator<EReference> {

		@Override
		public int compare(EReference r1, EReference r2) {
			String name1 = r1.getName();
			String name2 = r2.getName();
			return name1.compareTo(name2);
		}
		
	}
	
	private EClass getEClass(EObject modelRoot, String className) {
		for (EObject eo : modelRoot.eContents()) {
			if (eo instanceof EClass) {
				EClass eClass = (EClass)eo;
				String name = eClass.getName();
				if (name.equals(className)) {
					return eClass;
				}
			}
		}
		return null;
	}
	
	
	

	
	
	private EObject checkAndUnwrap(ArtifactRepresentation rep) {
		if (rep.getType().equals(EcoreInstanceRepresentation.TYPE)) {
			EcoreInstanceRepresentation eir = (EcoreInstanceRepresentation)rep;
			return eir.getModelRoot();
		} else {
			throw new DeftInvalidOperationChainException();
		}
	}
	
	private EcoreUml2AssociationsListingConfiguration checkAndUnwrap(OperationConfiguration config) {
		if (config instanceof EcoreUml2AssociationsListingConfiguration) {
			EcoreUml2AssociationsListingConfiguration u2aConfig = (EcoreUml2AssociationsListingConfiguration)config;
			return u2aConfig;
		} else {
			throw new DeftInvalidOperationConfigurationException();
		}
	}


	@Override
	public OperationConfiguration createConfiguration() {
		return new EcoreUml2AssociationsListingConfiguration();
	}


	@Override
	public String getInputRepresentationType() {
		return EcoreRepresentation.TYPE;
	}


	@Override
	public String getResultingRepresentationType() {
		return StyledDefinitionListingRepresentation.TYPE;
	}
}
