package org.deft.operationchain.ecoreuml2associationslisting.textcompute;

import java.util.LinkedList;
import java.util.List;

import javax.swing.text.StyledDocument;

import org.deft.operationchain.ecoreuml2associationslisting.style.StyleEnum;
import org.deft.util.StyledDocumentUtil;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;

public class AssociationsExplanationBuilder {
	
	public void addAssociationsExplanation(EReference reference, StyledDocument doc) {
		boolean textAdded = addDocumentationText(reference, doc);
		if (textAdded) StyledDocumentUtil.add(doc, " ");
		addSubsetsText(reference, doc);
		if (textAdded) StyledDocumentUtil.add(doc, " ");
		addDerivesText(reference, doc);
	}

	
	private boolean addDocumentationText(EReference reference, StyledDocument doc) {
		EAnnotation genmodel = reference.getEAnnotation("http://www.eclipse.org/emf/2002/GenModel");
		String documentation = genmodel.getDetails().get("documentation");
		String linebreak = System.getProperty("line.separator");
		documentation = documentation.replaceAll(linebreak, " ");
		if (documentation != null && !documentation.isEmpty()) {
			StyledDocumentUtil.add(doc, documentation);
			return true;
		}
		return false;
	}
	
	
	private boolean addSubsetsText(EReference reference, StyledDocument doc) {
		EAnnotation subsets = reference.getEAnnotation("subsets");
		if (subsets != null) {
			List<EReference> subsetsRefs = castToRef(subsets.getReferences());
			List<String> strings = getReferenceTargets(subsetsRefs);
			if (!strings.isEmpty()) {
				StyledDocumentUtil.add(doc, "Subsets ");
				StyledDocumentUtil.add(doc, strings.get(0), StyleEnum.SUBSETS.getName());
				for (int i = 1; i < strings.size(); i++) {
					StyledDocumentUtil.add(doc, ", ");
					StyledDocumentUtil.add(doc, strings.get(i), StyleEnum.SUBSETS.getName());
				}
				StyledDocumentUtil.add(doc, ".");
				return true;
			}
		}
		return false;
	}


	private List<EReference> castToRef(List<EObject> eObjects) {
		List<EReference> refs = new LinkedList<EReference>();
		for (EObject eo : eObjects) {
			if (eo instanceof EReference) {
				refs.add((EReference)eo);
			}
		}
		return refs;
	}

	private List<String> getReferenceTargets(List<EReference> references) {
		List<String> targetStrings = new LinkedList<String>();
		for (EReference ref : references) {
			String refName = ref.getName();
			String refType = ref.getEType().getName();
			targetStrings.add(refType + "::" + refName);
		}
		return targetStrings;
	}
	
	
	private boolean addDerivesText(EReference reference, StyledDocument doc) {
		boolean isDerived = reference.isDerived();
		if (isDerived) {
			EAnnotation union = reference.getEAnnotation("union");
			if (union == null) {
				StyledDocumentUtil.add(doc, "This is derived.");
			} else {
				StyledDocumentUtil.add(doc, "This is a derived union.");
			}
			return true;
		}
		return false;
	}
	
}
