package org.deft.operationchain.ecoreuml2associationslisting.textcompute;

import javax.swing.text.StyledDocument;

import org.deft.util.StyledDocumentUtil;
import org.eclipse.emf.ecore.EReference;

public class AssociationsTermBuilder {
	
	public void addAssociationsTerm(EReference reference, StyledDocument doc) {
		String derivedTag = reference.isDerived() ? "/" : "";
		String refName = reference.getName();
		String refType = reference.getEType().getName();
		String multiplicity = computeMultiplicityString(reference);
				
		StyledDocumentUtil.add(doc, derivedTag);
		StyledDocumentUtil.add(doc, refName);
		StyledDocumentUtil.add(doc, ": ");
		StyledDocumentUtil.add(doc, refType);
		StyledDocumentUtil.add(doc, " [" + multiplicity + "]");
	}
	
	
	private String computeMultiplicityString(EReference reference) {
		int lower = reference.getLowerBound();
		int upper = reference.getUpperBound();
		if (lower == upper) {
			return computeSingleMultiplicityString(lower);
		} else if (lower == 0 && upper == -1) {
			return computeSingleMultiplicityString(-1);
		} else { 
			return computeFromToMultiplicityString(lower, upper);
		}
	}
	
	
	private String computeSingleMultiplicityString(int mult) {
		if (mult < 0) {
			return "*";
		} else {
			return String.valueOf(mult);
		}
	}
	
	private String computeFromToMultiplicityString(int lower, int upper) {
		String sLower = computeSingleMultiplicityString(lower);
		String sUpper = computeSingleMultiplicityString(upper);
		return sLower + ".." + sUpper;
	}

}
