package org.deft.operationchain.ecoreuml2attributeslisting.textcompute;

import javax.swing.text.StyledDocument;

import org.deft.util.StyledDocumentUtil;
import org.eclipse.emf.ecore.EAttribute;

public class AttributesTermBuilder {
	
	public void addAttributesTerm(EAttribute attribute, StyledDocument doc) {
		String derivedTag = attribute.isDerived() ? "/" : "";
		String refName = attribute.getName();
		String refType = attribute.getEType().getName();
		String multiplicity = computeMultiplicityString(attribute);
				
		StyledDocumentUtil.add(doc, derivedTag);
		StyledDocumentUtil.add(doc, refName);
		StyledDocumentUtil.add(doc, ": ");
		StyledDocumentUtil.add(doc, refType);
		StyledDocumentUtil.add(doc, multiplicity.isEmpty() ? "" : " " + multiplicity);
	}
	
	
	private String computeMultiplicityString(EAttribute attribute) {
		int lower = attribute.getLowerBound();
		int upper = attribute.getUpperBound();
		if (lower == upper) {
			if (lower == 1) {
				return "";
			}
			return "[" + computeSingleMultiplicityString(lower) + "]";
		} else if (lower == 0 && upper == -1) {
			return "[" + computeSingleMultiplicityString(-1) + "]";
		} else { 
			return "[" + computeFromToMultiplicityString(lower, upper) + "]";
		}
	}
	
	
	private String computeSingleMultiplicityString(int mult) {
		if (mult < 0) {
			return "*";
		} else {
			return String.valueOf(mult);
		}
	}
	
	private String computeFromToMultiplicityString(int lower, int upper) {
		String sLower = computeSingleMultiplicityString(lower);
		String sUpper = computeSingleMultiplicityString(upper);
		return sLower + ".." + sUpper;
	}

}
