package org.deft.operationchain.ecoreuml2generalizationslisting;

import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;

import javax.swing.text.StyleContext;

import org.deft.operation.Operation;
import org.deft.operation.OperationConfiguration;
import org.deft.operation.exception.DeftInvalidOperationChainException;
import org.deft.operation.exception.DeftInvalidOperationConfigurationException;
import org.deft.operationchain.ecoreuml2generalizationslisting.style.LocalStyle;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.ecore.EcoreInstanceRepresentation;
import org.deft.representation.ecore.EcoreRepresentation;
import org.deft.representation.styleddocument.StyledSimpleListingRepresentation;
import org.deft.representation.styleddocument.data.SimpleListing;
import org.deft.representation.styleddocument.data.TokenizedStyledDocument;
import org.deft.util.StyledDocumentUtil;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;

public class EcoreUml2GeneralizationsListingOperation implements Operation {

	@Override
	public ArtifactRepresentation executeOperation(ArtifactRepresentation representation, 
			ArtifactType type, OperationConfiguration config) {
		EObject modelRoot = checkAndUnwrap(representation);
		EcoreUml2GeneralizationsListingConfiguration listingConfig = checkAndUnwrap(config);
		String className = listingConfig.getClassName();
		EClass eClassToDocument = getEClass(modelRoot, className);

		SimpleListing listing = new SimpleListing("None");
		createStyledListingElements(eClassToDocument, listing);
		StyledSimpleListingRepresentation rep = new StyledSimpleListingRepresentation(listing);
		return rep;
	}

	
	
	private void createStyledListingElements(EClass eClass, SimpleListing listing) {
		StyleContext context = new LocalStyle().getStyleContext();

		List<EClass> superTypes = new LinkedList<EClass>(eClass.getESuperTypes());
		Collections.sort(superTypes, new NameComparator());
		for (EClass superType : superTypes) {
			TokenizedStyledDocument listElementDoc = new TokenizedStyledDocument(context);
			addListElement(superType, listElementDoc);
			listing.addListItem(listElementDoc);
		}
	}
	
	
	private void addListElement(EClass eClass, TokenizedStyledDocument doc) {
		String name = eClass.getName();
		StyledDocumentUtil.add(doc, name);
	}
	
	private class NameComparator implements Comparator<EClass> {

		@Override
		public int compare(EClass c1, EClass c2) {
			String name1 = c1.getName();
			String name2 = c2.getName();
			return name1.compareTo(name2);
		}
		
	}
	
	private EClass getEClass(EObject modelRoot, String className) {
		for (EObject eo : modelRoot.eContents()) {
			if (eo instanceof EClass) {
				EClass eClass = (EClass)eo;
				String name = eClass.getName();
				if (name.equals(className)) {
					return eClass;
				}
			}
		}
		return null;
	}
	
	
	

	
	
	private EObject checkAndUnwrap(ArtifactRepresentation rep) {
		if (rep.getType().equals(EcoreInstanceRepresentation.TYPE)) {
			EcoreInstanceRepresentation eir = (EcoreInstanceRepresentation)rep;
			return eir.getModelRoot();
		} else {
			throw new DeftInvalidOperationChainException();
		}
	}
	
	private EcoreUml2GeneralizationsListingConfiguration checkAndUnwrap(OperationConfiguration config) {
		if (config instanceof EcoreUml2GeneralizationsListingConfiguration) {
			EcoreUml2GeneralizationsListingConfiguration u2aConfig = (EcoreUml2GeneralizationsListingConfiguration)config;
			return u2aConfig;
		} else {
			throw new DeftInvalidOperationConfigurationException();
		}
	}


	@Override
	public OperationConfiguration createConfiguration() {
		return new EcoreUml2GeneralizationsListingConfiguration();
	}


	@Override
	public String getInputRepresentationType() {
		return EcoreRepresentation.TYPE;
	}


	@Override
	public String getResultingRepresentationType() {
		return StyledSimpleListingRepresentation.TYPE;
	}
}
