package org.deft.operationchain.ecoreuml2generalizationslisting;



import org.deft.operation.AbstractOperationChain;
import org.deft.operation.bytearray2ecoreinstance.XmiAsByteArray2EcoreInstanceOperation;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.artifacttype.ecore.uml.EcoreUmlArtifactType;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.bytearray.ByteArrayRepresentation;

/**
 * Operation chain that just takes a UML metamodel (modelled in Ecore) and produces a listing
 * of styled text with the superclasses (generalizations) a selected element.
 */
public class EcoreUml2GeneralizationsListingOperationChain extends AbstractOperationChain {
	
	public static String ID = "org.deft.operationchain.ecoreuml2generalizationslisting";
	
	public EcoreUml2GeneralizationsListingOperationChain() {
		setupChain();
	}
	
	
	private void setupChain() {
		addOperation(new XmiAsByteArray2EcoreInstanceOperation());
		addOperation(new EcoreUml2GeneralizationsListingOperation());
	}
	
	
	@Override
	public String getId() {
		return ID;
	}
	

	@Override
	public ArtifactRepresentation getInitialArtifactRepresentation(Artifact artifact, int revision) {
		byte[] content = artifact.getArtifactContent(revision);
		ByteArrayRepresentation rep = new ByteArrayRepresentation(content);
		return rep;
	}



	@Override
	public boolean isValidInputArtifactType(ArtifactType artifactType) {
		boolean isEcoreUmlType = artifactType instanceof EcoreUmlArtifactType;
		return isEcoreUmlType;
	}
	

	@Override
	public String getDescription() {
		return "Generalizations Listing";
	}

}
