package org.deft.operationchain.ecoreuml2generalizationslisting.gui;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import org.deft.configuration.OperationGui;
import org.deft.operation.OperationConfiguration;
import org.deft.operationchain.ecoreuml2generalizationslisting.EcoreUml2GeneralizationsListingConfiguration;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.ecore.EcoreInstanceRepresentation;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

public class EcoreUml2GeneralizationsListingOperationGui implements OperationGui {

	private EcoreUml2GeneralizationsListingConfiguration dataConfig;
	private TableViewer tableViewer;
	
	@Override
	public Composite createGuiComposite(Composite parent) {
		Composite gui = new Composite(parent, SWT.NONE);

		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 1;
		gui.setLayout(gridLayout);
		insertTableViewer(gui);
		return gui;
	}
	
	
	
	
	
	private TableViewer insertTableViewer(Composite parent) {
		Table table = createTable(parent, "Classes");
		GridData gd = new GridData(GridData.FILL_BOTH);
		table.setLayoutData(gd);

		tableViewer = new TableViewer(table);
		tableViewer.setUseHashlookup(true);

		tableViewer.setContentProvider(new ArrayContentProvider());
		return tableViewer;
	}
	
	
	
	
	private Table createTable(Composite parent, String header) {
		
		int style = SWT.BORDER | SWT.SINGLE;
		Table table = new Table(parent, style);

		GridData gridData = new GridData(GridData.FILL_BOTH);
		table.setLayoutData(gridData);

		table.setLinesVisible(true);
		table.setHeaderVisible(true);

		TableColumn column = new TableColumn(table, SWT.CENTER, 0);
		column.setWidth(500);
		column.setText(header);
		
		return table;
	}
	
	
	
	
	
	
	@Override
	public void initializeGui(OperationConfiguration config,
			Artifact artifact, ArtifactRepresentation artifactRepr) {
		
		this.dataConfig = (EcoreUml2GeneralizationsListingConfiguration)config;
		EcoreInstanceRepresentation ecoreRep = (EcoreInstanceRepresentation)artifactRepr;
		EObject modelRoot = ecoreRep.getModelRoot();
		List<String> classNames = getClassNames(modelRoot);
		Collections.sort(classNames);
		
		tableViewer.setInput(classNames.toArray());
		tableViewer.addSelectionChangedListener(new IndividualSelectionListener(tableViewer.getTable()));
		selectFirstLineIfPossibleAndSetConfig(tableViewer);
	}

	
	private List<String> getClassNames(EObject modelRoot) {
		List<String> names = new LinkedList<String>();
		for (EObject eo : modelRoot.eContents()) {
			if (eo instanceof EClass) {
				EClass eClass = (EClass)eo;
				String name = eClass.getName();
				names.add(name);
			}
		}
		return names;
	}
	
	
	private void selectFirstLineIfPossibleAndSetConfig(TableViewer tableViewer) {
		Table table = tableViewer.getTable();
		int itemCount = table.getItemCount();
		if (itemCount > 0) {
			table.select(0);
			String text = table.getItem(0).getText();
			dataConfig.setClassName(text);
		} else {
			dataConfig.setClassName(null);
		}
	}
	

	
	/**
	 * Updates the OperationConfiguration whenever an individual is checked or unchecked
	 */
	private class IndividualSelectionListener implements ISelectionChangedListener {
		
		private Table table;

		public IndividualSelectionListener(Table table) {
			this.table = table;
		}
		

		@Override
		public void selectionChanged(SelectionChangedEvent event) {
			int idx = table.getSelectionIndex();
			if (idx != -1) {
				TableItem ti = table.getItem(idx);
				String selectedIndividualName = ti.getText();
				dataConfig.setClassName(selectedIndividualName);
			}
		}
	}


}
