package org.deft.operationchain.emftext2ecoreinstance;

import org.deft.emftext.EmfTextArtifactType;
import org.deft.operation.AbstractOperationChain;
import org.deft.operation.bytearray2ecoreinstance.EmfTextAsByteArray2EcoreInstanceOperation;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.bytearray.ByteArrayRepresentation;

/**
 * Operation chain that just takes an EmfTextArtifactType and produces an
 * EcoreInstanceRepresentation from it.
 */
public class EmfText2EcoreInstanceOperationChain extends AbstractOperationChain {
	
	public static String ID = "org.deft.operationchain.emftext2ecoreinstance";
	
	
	public EmfText2EcoreInstanceOperationChain() {
		setupChain();
	}
	
	
	private void setupChain() {
		addOperation(new EmfTextAsByteArray2EcoreInstanceOperation());
	}
	
	
	@Override
	public String getId() {
		return ID;
	}
	

	@Override
	public ArtifactRepresentation getInitialArtifactRepresentation(Artifact artifact, int revision) {
		byte[] content = artifact.getArtifactContent(revision);
		ByteArrayRepresentation rep = new ByteArrayRepresentation(content);
		return rep;
	}



	@Override
	public boolean isValidInputArtifactType(ArtifactType artifactType) {
		boolean isEmfTextType = artifactType instanceof EmfTextArtifactType;
		return isEmfTextType;
	}


	@Override
	public String getDescription() {
		return "Ecore Instance";
	}
}
