package org.deft.operationchain.image2image;

import org.deft.operation.AbstractOperationChain;
import org.deft.operation.bytearray2image.ByteArray2ImageOperation;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.artifacttype.image.ImageArtifactType;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.bytearray.ByteArrayRepresentation;

/**
 * Operation chain that just takes an ImageArtifactType and produces a
 * BufferedImageRepresentation from it.
 */
public class Image2ImageOperationChain extends AbstractOperationChain {
	
	public static String ID = "org.deft.operationchain.image2image";
	
	public Image2ImageOperationChain() {
		setupChain();
	}
	
	
	private void setupChain() {
		addOperation(new ByteArray2ImageOperation());
	}
	
	@Override
	public String getId() {
		return ID;
	}
	

	@Override
	public ArtifactRepresentation getInitialArtifactRepresentation(Artifact artifact, int revision) {
		byte[] content = artifact.getArtifactContent(revision);
		ByteArrayRepresentation rep = new ByteArrayRepresentation(content);
		return rep;
	}



	@Override
	public boolean isValidInputArtifactType(ArtifactType artifactType) {
		boolean isImageType = artifactType.getTypeId().equals(ImageArtifactType.ID);
		return isImageType;
	}

	@Override
	public String getDescription() {
		return "Image";
	}

}
