package org.deft.operationchain.ocl2styledcode;

import org.deft.operation.AbstractOperationChain;
import org.deft.operation.bytearray2charbuffer.ByteArray2CharBufferOperation;
import org.deft.operation.charbuffer2text.CharBuffer2TextOperation;
import org.deft.operation.ocl2styledcode.Ocl2StyledCodeOperation;
import org.deft.operation.oclfilter.OclFilterOperation;
import org.deft.operation.replacetabs.ReplaceTabsOperation;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.artifacttype.emftext.dresdenocl.DresdenOclArtifactType;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.bytearray.ByteArrayRepresentation;

/**
 * Operation chain that transforms EMFText DSLs into styled text.
 */
public class Ocl2StyledCodeOperationChain extends AbstractOperationChain {
	
	public static String ID = "org.deft.operationchain.ocl2styledcode";
	
	public Ocl2StyledCodeOperationChain() {
		setupChain();
	}
	
	private void setupChain() {
		addOperation(new ByteArray2CharBufferOperation());
		addOperation(new CharBuffer2TextOperation());
		addOperation(new ReplaceTabsOperation());
		addOperation(new OclFilterOperation());
		addOperation(new Ocl2StyledCodeOperation());
	}
	

	
	@Override
	public String getId() {
		return ID;
	}
	

	@Override
	public ArtifactRepresentation getInitialArtifactRepresentation(Artifact artifact, int revision) {
		byte[] content = artifact.getArtifactContent(revision);
		ByteArrayRepresentation rep = new ByteArrayRepresentation(content);
		return rep;
	}


	@Override
	public boolean isValidInputArtifactType(ArtifactType artifactType) {
		boolean isOclType = artifactType instanceof DresdenOclArtifactType;
		return isOclType;
	}

	@Override
	public String getDescription() {
		return "OCL Code";
	}
	
}
