package org.deft.projecttype.openoffice;

import org.deft.repository.integrator.ArtifactReferencePosition;

import ag.ion.bion.officelayer.text.ITextCursor;
import ag.ion.bion.officelayer.text.ITextDocument;
import ag.ion.bion.officelayer.text.ITextRange;

public class OdfTextArtifactReferencePosition implements ArtifactReferencePosition {
	
	private OdfTextChapterDocument document;
	private ITextRange textRange;

	public OdfTextArtifactReferencePosition(OdfTextChapterDocument document, ITextRange textRange) {
		super();
		this.document = document;
		this.textRange = textRange;
	}

	@Override
	public OdfTextChapterDocument getChapterDocument() {
		return document;
	}
	
	
	/**
	 * Tries to read the reference tag ( <artifactreference id="some_id"> )
	 * If the reference tag cannot be read for some reason, an empty string is returned.
	 * 
	 * A reason might be the occurrence of a string that looks like a reference tag
	 * but is contained in a code box, where it can't be read with a TextCursor.
	 */
	@Override
	public String extractReferenceTag() {
		ITextDocument textDocument = (ITextDocument)document.getDocument();
		ITextCursor textCursor = textDocument.getTextService().getCursorService().getTextCursor();
		String referenceTag = tryToReadReferenceTag(textCursor);
		return referenceTag;
	}
	
	private String tryToReadReferenceTag(ITextCursor textCursor) {
		try {
			textCursor.gotoRange(textRange, false);	
			return textCursor.getString();
		} catch (com.sun.star.uno.RuntimeException e) {
			//we are trying to reference a tag somewhere where we don't have a cursor
			//This happens, for example, if a code box contains the String <ArtifactReference> or 
			//something similar.
			System.err.println("Could not use textCursor");
			return "";
		}
	}

	
	public ITextRange getTextRange() {
		return textRange;
	}
	

}
