package org.deft.projecttype.texlipse;

import java.util.LinkedList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.deft.integrator.EditorChapterDocument;
import org.deft.integrator.ErrorMessageIntegrator;
import org.deft.operation.EmptyOperationConfiguration;
import org.deft.operation.OperationConfiguration;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.repository.integrator.ArtifactReferencePosition;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.Position;



public class ErrorMessageIntoLatexIntegrator extends ErrorMessageIntegrator {

	public static final String ID = "org.deft.editorintegrator.uncomputable4latex";
	private String errorMessage;
	
	@Override
	public void setErrorMessage(String errorMessage) {
		this.errorMessage = errorMessage;
	}
	
	@Override
	public void integrate(String referenceId, ArtifactRepresentation representation, 
			ArtifactType artifactType, ArtifactReferencePosition position) {
		TexlipseArtifactReferencePosition latexPosition = (TexlipseArtifactReferencePosition)position;
		LatexEditorChapterDocument doc = latexPosition.getChapterDocument();
		insertErrorMessage(doc, latexPosition.getPosition(), referenceId);
	}
	
	
	private void insertErrorMessage(LatexEditorChapterDocument doc, Position position, String referenceId) {
		String errorDirective = buildDeftErrorReferenceDirective(referenceId, errorMessage);
		try {
			IDocument latexDoc = doc.getDocument();
			latexDoc.replace(position.getOffset(), 0, errorDirective);
		} catch (BadLocationException ble) {
			ble.printStackTrace();
		}
	}
	
	private String buildDeftErrorReferenceDirective(String referenceId, String text) {
		String directive = "\\deftreferenceerror{" + referenceId + "}{" + text + "}";
		return directive;
	}
	

	@Override
	public String getInputRepresentationType() {
		return "uncomputable";
	}

	@Override
	public List<String> getChapterTypeIds() {
		List<String> ids = new LinkedList<String>();
		ids.add("latex");
		return ids;
	}

	@Override
	public String getId() {
		return ID;
	}

	@Override
	public String getDescription() {
		return "Error Message";
	}


	@Override
	public TexlipseArtifactReferencePosition getPositionForRepresentation(EditorChapterDocument chapterDocument, String referenceId) {
		LatexEditorChapterDocument latexChapterDoc = (LatexEditorChapterDocument)chapterDocument;
		String text = latexChapterDoc.getDocument().get();
		String completeDirective = getCompleteDirective(text, referenceId);
		if (completeDirective != null) {
			int startOffset = text.indexOf(completeDirective);	//where the directive string begins
			int integratedTextLength = completeDirective.length();
			Position pos = new Position(startOffset, integratedTextLength);
			TexlipseArtifactReferencePosition artRefPos = new TexlipseArtifactReferencePosition(latexChapterDoc, pos);
			return artRefPos;
		} else {
			return null;
		}
	}

	private String getCompleteDirective(String text, String referenceId) {
		String regex = "\\\\deftreferenceerror\\{" + referenceId + "\\}\\{(.*?)\\}";
		Pattern pattern = Pattern.compile(regex);
		Matcher matcher = pattern.matcher(text);
		while (matcher.find()) {
			String writtenText = matcher.group(1);
			return "\\deftreferenceerror{" + referenceId + "}{" + writtenText +"}";
		}
		return null;
	}



	@Override
	public ArtifactReferencePosition undoIntegration(EditorChapterDocument document, String referenceId) {
		LatexEditorChapterDocument latexDoc = (LatexEditorChapterDocument)document;
		TexlipseArtifactReferencePosition artifactRefPos = getPositionForRepresentation(document, referenceId);
		if (artifactRefPos != null) {
			try {
				Position pos = artifactRefPos.getPosition();
				latexDoc.getDocument().replace(pos.getOffset(), pos.getLength(), "");
			} catch (BadLocationException e) {
				e.printStackTrace();
			}
			return artifactRefPos;
		}
		return null;
	}

	
	
	@Override
	public OperationConfiguration createConfiguration() {
		return new EmptyOperationConfiguration();
	}

	
}
