package org.deft.projecttype.texlipse;

import org.deft.integrator.AbstractEditorChapterDocument;
import org.deft.integrator.ChapterDocRepositoryListener;
import org.deft.integrator.EditorIntegrator;
import org.deft.integrator.ErrorMessageIntegrator;
import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.integrator.ArtifactReferencePosition;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.Position;
import org.eclipse.jface.text.TextSelection;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionProvider;

/**
 * Represents a LaTeX document in an open Texlipse editor. 
 */
public class LatexEditorChapterDocument extends AbstractEditorChapterDocument {

	
	private IDocument document;
	private ISelectionProvider selectionProvider;
	private ErrorMessageIntegrator uncompIntegrator = new ErrorMessageIntoLatexIntegrator();
	private ChapterDocRepositoryListener listener = new ChapterDocRepositoryListener(this);
	
	
	public LatexEditorChapterDocument(Chapter chapter, IDocument document, 
			TexlipseChapterEditorPart editor) {
		super(chapter);
		this.document = document;
		this.selectionProvider = editor.getSelectionProvider();
	}
	
	
	public IDocument getDocument() {
		return (IDocument)document;
	}

	
	
	public ArtifactReferencePosition getCursorPosition() {
		ISelection selection = selectionProvider.getSelection();
		if (selection instanceof TextSelection) {
			TextSelection ts = (TextSelection)selection;
			int offset = ts.getOffset();
			Position texlipsePosition = new Position(offset, 0);
			TexlipseArtifactReferencePosition pos = new TexlipseArtifactReferencePosition(this, texlipsePosition);
			return pos;
		} else {
			throw new IllegalStateException("Selection was not TextSelection, but: " + selection);
		}
	}
	
	public void setPosition(int offset) {
		TextSelection selection = new TextSelection(offset, 0);
		selectionProvider.setSelection(selection);
	}

	

	@Override
	public void scrollToRepresentation(ArtifactReference reference) {
		EditorIntegrator integrator = getIntegrator(reference);
		ArtifactReferencePosition arPos = integrator.getPositionForRepresentation(this, reference.getId());
		TexlipseArtifactReferencePosition tarPos = (TexlipseArtifactReferencePosition)arPos;
		Position pos = tarPos.getPosition();
		int offset = pos.getOffset();
		TextSelection selection = new TextSelection(offset, 0);
		selectionProvider.setSelection(selection);	
	}

	
	public String getIntegrationStartComment(String integrationId) {
		return "%%%start deft artifact integration " + integrationId;
	}
	
	public String getIntegrationEndComment(String integrationId) {
		return "%%%end deft artifact integration " + integrationId;
	}


	@Override
	public void addChapterDocRepositoryListener() {
		repository.addRepositoryUpdateListener(listener);
	}
	
	@Override
	public void removeChapterDocRepositoryListener() {
		repository.removeRepositoryUpdateListener(listener);
	}


	@Override
	public ErrorMessageIntegrator getErrorMessageIntegrator() {
		return uncompIntegrator;
	}


	
	
	
	
//	public void addArtifactUpdateListener() {
//		repository.addRepositoryUpdateListener(listener);
//	}
//	
//
//	public void removeArtifactUpdateListener() {
//		repository.removeRepositoryUpdateListener(listener);
//	}
}
