package org.deft.projecttype.texlipse;

import org.deft.gui.EditorHelper;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Project;
import org.deft.repository.workspaceresources.WorkspaceManager;
import org.deft.texlipse.UpdateOutlineAction;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.progress.UIJob;

/**
 * Listener that listens for opening editors. If a chapter editor is opened
 * the Latex outline will be updated. If the chapter is part of a latex project,
 * the corresponding LaTeX outline will be displayed, otherwise the outline
 * will be made empty. See UpdateOutlineAction for details. 
 */
public class LatexOutlineUpdater implements IPartListener {
	
	private EditorHelper editorHelper = new EditorHelper();
	private UpdateOutlineAction updateOutlineAction = new UpdateOutlineAction();

	public void partActivated(IWorkbenchPart part) {
		updateOutline();
	}

	public void partBroughtToTop(IWorkbenchPart part) {
	}

	public void partClosed(IWorkbenchPart part) {
	}

	public void partDeactivated(IWorkbenchPart part) {
	}

	public void partOpened(IWorkbenchPart part) {
	}

	
	public void updateOutline() {
		final Chapter chapter = editorHelper.getChapterOfActiveEditor();
		if (chapter != null) {
			new UIJob("Update LaTeX Outline") {
				public IStatus runInUIThread(IProgressMonitor monitor) {
					Project deftProject = chapter.getProject();
					IProject project = WorkspaceManager.instance().getEclipseProjectInWorkspace(deftProject);
					updateOutlineAction.updateOutline(project);
					return Status.OK_STATUS;
				}
			}.schedule();
		}
	}

	
}
