package org.deft.projecttype.texlipse;

import net.sourceforge.texlipse.editor.TexEditor;

import org.deft.chaptereditor.ChapterEditorInput;
import org.deft.chaptereditor.ChapterEditorPart;
import org.deft.integrator.EditorChapterDocument;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.workspaceresources.WorkspaceManager;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.ITextListener;
import org.eclipse.jface.text.TextEvent;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IPropertyListener;



public class TexlipseChapterEditorPart extends TexEditor implements ChapterEditorPart {

	public static final String ID = "org.deft.chaptereditor.texlipse"; //$NON-NLS-1$
	
	private boolean externallySetDirtyFlag;
	private boolean forceUsageOfExternallySetDirtyFlag;
	private boolean returnDirty;
	
	private WorkspaceManager workspaceManager = WorkspaceManager.instance();
	private LatexEditorChapterDocument chapterDocument;
	private TextListener textListener;
	private static LatexOutlineUpdater listener;
	
	public TexlipseChapterEditorPart() {
		this.addPropertyListener(new IPropertyListener() {

			@Override
			public void propertyChanged(Object source, int propId) {
				if (propId == PROP_DIRTY) {
					if (forceUsageOfExternallySetDirtyFlag) {
						returnDirty = externallySetDirtyFlag;
					} else {
						returnDirty = TexlipseChapterEditorPart.super.isDirty();
					}
				}
			}

		});
	}
	
	@Override
	public void createPartControl(Composite parent) {
		super.createPartControl(parent);
		if (listener == null) {
			 listener = new LatexOutlineUpdater();
			 this.getSite().getPage().addPartListener(listener);
		}
	}
	
	
	@Override
	public Chapter getChapter() {
		ChapterEditorInput input = (ChapterEditorInput)getEditorInput();
		Chapter chapter = input.getChapter();
		return chapter;
	}
	

	@Override
	public LatexEditorChapterDocument getChapterDocument() {
		if (chapterDocumentDoesNotYetExist()) {
			instantiateChapterDocument();
		}
		return chapterDocument;
	}
	
	private boolean chapterDocumentDoesNotYetExist() {
		return chapterDocument == null;
	}
	
	private void instantiateChapterDocument() {
		Chapter chapter = getChapter();
		IDocument document = getTexDocument();
		chapterDocument = new LatexEditorChapterDocument(chapter, document, this);
	}
	

	
	public void setDirty(boolean dirty) {
		externallySetDirtyFlag = dirty;
		forceUsageOfExternallySetDirtyFlag = true;
		firePropertyChange(PROP_DIRTY);
	}
	
	public void addTextListener() {
		if (!isTextListenerAdded()) {
			ISourceViewer sv = getSourceViewer();
			if (sv != null) {
				textListener = new TextListener();
				sv.addTextListener(textListener);
			}
		}
	}
	
	private boolean isTextListenerAdded() {
		return textListener != null;
	}
	
	
	@Override
	public boolean isDirty() {
		return returnDirty;
	}
	
	
	@Override
	public void doSave(IProgressMonitor monitor) {
		EditorChapterDocument doc = getChapterDocument();
		doc.removeNonexistingReferencesFromRepository();
		performActualSave(monitor);
	}
	
	private void performActualSave(IProgressMonitor monitor) {
		super.doSave(monitor);
		Chapter chapter = getChapter();
		workspaceManager.saveWorkspaceCopyToRepository(chapter);
	}

	
	/**
	 * Listener that listens for changes in the editor text and fires a 
	 * DIRTY property change event if necessary.  
	 */
	private class TextListener implements ITextListener {
		
		private String textInEditor;
		
		@Override
		public void textChanged(TextEvent event) {
			String textInDocument = getChapterDocument().getDocument().get();
			//When the editor opens up there will be some events AFTER the integration
			// took place (e.g. code folding stuff). These events are sent from the bowels
			// of Texlipse and are thrown from Jobs, i.e. there are synchronisation issues.
			// 
			// So the first event is ignored because textInEditor is still null. Without
			// that check the editor would be marked dirty upon opening.
			if (textInEditor == null) {
				textInEditor = textInDocument;	
				return;
			}
			
			//Text change events only cause a DIRTY property change if 
			//there was REALLY a text change
			if (!textInDocument.equals(textInEditor)) {
				forceUsageOfExternallySetDirtyFlag = false;
				firePropertyChange(PROP_DIRTY);
			}
			
		}
	};



}
