package org.deft.repository;

import org.deft.operation.EclipseOperationChainRegistry;
import org.deft.operation.OperationChainRegistry;
import org.deft.repository.artifacttype.ArtifactTypeRegistry;
import org.deft.repository.artifacttype.EclipseArtifactTypeRegistry;
import org.deft.repository.artifacttype.EclipseFileExtensionToArtifactTypeMappingRegistry;
import org.deft.repository.artifacttype.FileExtensionToArtifactTypeMappingRegistry;
import org.deft.repository.chaptertype.ChapterAwareResourceHandlerFactory;
import org.deft.repository.chaptertype.ChapterAwareResourceHandlerProvider;
import org.deft.repository.chaptertype.ChapterType;
import org.deft.repository.chaptertype.ChapterType2IntegratorIdMappingRegistry;
import org.deft.repository.chaptertype.ChapterTypeRegistry;
import org.deft.repository.chaptertype.ChapterTypeResourceHandler;
import org.deft.repository.chaptertype.EclipseChapterType2IntegratorIdMappingRegistry;
import org.deft.repository.chaptertype.EclipseChapterTypeRegistry;
import org.deft.repository.datamodel.adapter.ecore.EcoreWrapper;
import org.deft.repository.fragmentsource.EclipseFragmentSourceRegistry;
import org.deft.repository.fragmentsource.FragmentSourceRegistry;
import org.deft.repository.integrator.EclipseFileIntegratorRegistry;
import org.deft.repository.integrator.FileIntegratorRegistry;
import org.deft.repository.projecttype.EclipseProjectTypeRegistry;
import org.deft.repository.projecttype.ProjectType;
import org.deft.repository.projecttype.ProjectTypeRegistry;
import org.deft.repository.repositorymanager.ConstraintsManager;
import org.deft.repository.repositorymanager.ContentManagementService;
import org.deft.repository.repositorymanager.ContentManager;
import org.deft.repository.repositorymanager.DataStorageManager;
import org.deft.repository.repositorymanager.OptionsService;
import org.deft.repository.repositorymanager.PersistenceManager;
import org.deft.repository.repositorymanager.impl.ConstraintsManagerTuPrologImpl;
import org.deft.repository.repositorymanager.impl.ContentManagementServiceImpl;
import org.deft.repository.repositorymanager.impl.ContentManagerEcoreDataModelImpl;
import org.deft.repository.repositorymanager.impl.DataStorageManagerFileSystemImpl;
import org.deft.repository.repositorymanager.impl.EcoreDataModelImplCore;
import org.deft.repository.repositorymanager.impl.OptionsServiceImpl;
import org.deft.repository.repositorymanager.impl.PersistenceManagerEcoreDataModelImpl;
import org.deft.repository.service.ExtensionService;
import org.deft.repository.service.InternalStateService;
import org.deft.repository.service.impl.ExtensionServiceImpl;
import org.deft.repository.service.impl.InternalStateServiceImpl;
import org.deft.resource.ResourceHandler;
import org.deft.resource.ResourceHandlerFactory;

public class RepositoryImpl implements Repository {

	private ContentManagementService cms;
	private OptionsService optionsService;
	private ExtensionService extensionService;
	private InternalStateService internalStateService;
	
	private static Repository instance;
	
	
	private ContentManager contentManager;
	private ConstraintsManager constraintsManager;
	private PersistenceManager persistenceManager;
	private DataStorageManager dataStorageManager;
	private EcoreDataModelImplCore core;
	private ChapterAwareResourceHandlerProvider resourceHandlerProvider;
	
	private RepositoryImpl() {
		initAllManagers();
	}
	
	private void initAllManagers() {
		core = new EcoreDataModelImplCore();
		resourceHandlerProvider = new ChapterAwareResourceHandlerProvider();
		//create and register org.deft.repository resource handler
		ResourceHandler handler = ResourceHandlerFactory.createSimpleResourceHandler(
				"org.deft.repository", ResourceHandlerFactory.ECLIPSE);
		resourceHandlerProvider.registerResourceHandler(handler);
		//create and register project type resource handlers
		
		EclipseChapterTypeRegistry chapterTypeRegistry = new EclipseChapterTypeRegistry();
		ProjectTypeRegistry projectTypeRegistry = new EclipseProjectTypeRegistry(chapterTypeRegistry);
		for (ProjectType projectType : projectTypeRegistry.getProjectTypes()) {
			for (ChapterType chapterType : projectType.getAllowedChapterTypes()) {
				ChapterTypeResourceHandler chapterHandler 
						= ChapterAwareResourceHandlerFactory.createChapterTypeResourceHandler(
						chapterType, ResourceHandlerFactory.ECLIPSE);
				resourceHandlerProvider.registerResourceHandler(chapterHandler);
				
			}
		}

		EcoreWrapper wrapper = new EcoreWrapper(this);
		contentManager = new ContentManagerEcoreDataModelImpl(core, wrapper);
		constraintsManager = new ConstraintsManagerTuPrologImpl(contentManager, 
				resourceHandlerProvider);
		persistenceManager = new PersistenceManagerEcoreDataModelImpl(core, getOptionsService());
		dataStorageManager = new DataStorageManagerFileSystemImpl(contentManager, 
				getOptionsService(), resourceHandlerProvider, getExtensionService());

	}
	
	public static Repository instance() {
		if (instance == null) {
			instance = new RepositoryImpl();
		}
		return instance;
	}
	
	@Override
	public ContentManagementService getContentManagementService() {
		if (cms == null) {
			cms = createCms();
		}
		return cms;
	}
	
	private ContentManagementService createCms() {
		ContentManagementService cms = new ContentManagementServiceImpl(
				contentManager, 
				constraintsManager, persistenceManager, 
				dataStorageManager, getOptionsService(), getExtensionService(),
				getInternalStateService());
		return cms;
	}

	@Override
	public OptionsService getOptionsService() {
		if (optionsService == null) {
			optionsService = createOptionsService();
		}
		return optionsService;
	}
	
	private OptionsService createOptionsService() {
		OptionsService os = new OptionsServiceImpl();
		return os;
	}

	@Override
	public ExtensionService getExtensionService() {
		if (extensionService == null) {
			extensionService = createExtensionService();
		}
		return extensionService;
	}
	
	private ExtensionService createExtensionService() {
		FragmentSourceRegistry fragmentSourceReg = new EclipseFragmentSourceRegistry();
		ArtifactTypeRegistry artifactTypeReg = new EclipseArtifactTypeRegistry();
		FileExtensionToArtifactTypeMappingRegistry fileExtensionMappingReg = new EclipseFileExtensionToArtifactTypeMappingRegistry();
		ChapterTypeRegistry chapterTypeReg = new EclipseChapterTypeRegistry();
		ProjectTypeRegistry projectTypeReg = new EclipseProjectTypeRegistry(chapterTypeReg);
		FileIntegratorRegistry integratorReg = new EclipseFileIntegratorRegistry();
		ChapterType2IntegratorIdMappingRegistry chapterType2IntegratorIdMappingReg
				= new EclipseChapterType2IntegratorIdMappingRegistry();
		OperationChainRegistry operationChainReg = new EclipseOperationChainRegistry();
		
		ExtensionService service = new ExtensionServiceImpl(fragmentSourceReg, 
				artifactTypeReg, fileExtensionMappingReg, projectTypeReg,
				integratorReg, chapterType2IntegratorIdMappingReg, operationChainReg);
		return service;
	}
	
	@Override
	public InternalStateService getInternalStateService() {
		if (internalStateService == null) {
			internalStateService = new InternalStateServiceImpl();
		}
		return internalStateService;
	}
	
}
