package org.deft.repository.artifacttype;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;

public class EclipseArtifactTypeRegistry implements ArtifactTypeRegistry {
	
	private static final String ARTIFACTTYPE_EXTENSION_POINT_NAME = "org.deft.repository.artifacttype.artifacttype";
		
	private Map<String, ArtifactTypeConfig> registeredTypes = new HashMap<String, ArtifactTypeConfig>(); 

	public EclipseArtifactTypeRegistry() {
		getAndCacheRegisteredTypes();
	}
	
	private void getAndCacheRegisteredTypes() {
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint point = registry.getExtensionPoint(ARTIFACTTYPE_EXTENSION_POINT_NAME);
		for (IConfigurationElement ce : point.getConfigurationElements()) {
			ArtifactTypeConfig config = new ArtifactTypeConfig(ce);
			registeredTypes.put(config.getId(), config);
		}
	}
	
	@Override
	public ArtifactType getArtifactType(String artifactTypeId) {
		ArtifactTypeConfig config = registeredTypes.get(artifactTypeId);
		if (config == null) {
			return null;
		}
		ArtifactType type = config.getArtifactType();
		return type;
	}
	
	@Override
	public List<String> getArtifactTypeIds() {
		List<String> typeIds = new LinkedList<String>(registeredTypes.keySet());
		return typeIds;
	}
	

	@Override
	public List<ArtifactType> getArtifactTypes() {
		List<ArtifactType> types = new LinkedList<ArtifactType>();
		for (String typeId : getArtifactTypeIds()) {
			types.add(getArtifactType(typeId));
		}
		return types;
	}
	
	
	private class ArtifactTypeConfig {
	
		private IConfigurationElement ce;
		private ArtifactType artifactType;
		
		public ArtifactTypeConfig(IConfigurationElement ce) {
			this.ce = ce;
		}
		
		public String getId() {
			String id = ce.getAttribute("id");
			return id;
		}
		
		public ArtifactType getArtifactType() {
			if (artifactType == null) {
				try {
					String artifactTypePropertyInExtensionPoint = "class";
					artifactType = (ArtifactType)ce.createExecutableExtension(
							artifactTypePropertyInExtensionPoint);
				} catch (CoreException e) {
					e.printStackTrace();
				}
			}
			return artifactType;
		}	
	}



	
}
