package org.deft.repository.artifacttype.ecorediag;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;

import org.deft.repository.artifacttype.MultiArtifactDependencies;
import org.deft.repository.artifacttype.Validator;
import org.deft.repository.artifacttype.ecore.EcoreCrossReferenceResolver;
import org.deft.repository.fragmentsource.FileBasedFragmentLocation;
import org.deft.repository.fragmentsource.FragmentSource;
import org.deft.repository.fragmentsource.OriginalFragmentLocation;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EcorePackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.emf.ecore.xmi.impl.EcoreResourceFactoryImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceImpl;
import org.eclipse.gmf.runtime.emf.core.resources.GMFResource;
import org.eclipse.gmf.runtime.notation.NotationPackage;

public class EcoreDiagramValidator implements Validator {

	private EcoreCrossReferenceResolver resolver = new EcoreCrossReferenceResolver();
	
	@Override
	public void validate(InputStream is) {
	}

	@Override
	public List<OriginalFragmentLocation> getDependencyLocations(FragmentSource source, 
			OriginalFragmentLocation location) {
		String path = location.getFullLocation();
		URI uri = URI.createFileURI(path);
		Resource resource = new GMFResource(uri);
		if (!resource.isLoaded()) {
			try {
				//without registering resource loading will fail
				registerEmfResourceFactories();
				resource.load(null);
			} catch (IOException e) {
				String msg = "The given path '" + path
						+ "' does not lead to a valid EMF/Ecore resource.";
				throw new IllegalArgumentException(msg, e);
			}
		}
		
		List<OriginalFragmentLocation> locations = resolver.getReferencedLocations(resource, 
				(FileBasedFragmentLocation)location);
		return locations;
	}
	
	/**
	 * Registers EMF resource factories at EMF.
	 * 
	 * @throws IOException
	 */
	protected void registerEmfResourceFactories() throws IOException {

		if (!Platform.isRunning()) {
			/* Probably register the Ecore resource for EMF. */
			if (Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap()
					.get(EcorePackage.eNS_PREFIX) == null) {

				Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap()
						.put(EcorePackage.eNS_PREFIX,
								new EcoreResourceFactoryImpl() {
									public Resource createResource(URI uri) {
										XMIResource xmiResource = new XMIResourceImpl(
												uri);
										return xmiResource;
									}
								});
			}
			// no else.

			if (EPackage.Registry.INSTANCE.getEPackage(EcorePackage.eNS_URI) == null) {
				EPackage.Registry.INSTANCE.put(EcorePackage.eNS_PREFIX,
						EcorePackage.eINSTANCE);
			}
			// no else.

			/* Probably register the notation resource for EMF. */
			if (Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap()
					.get(NotationPackage.eNS_PREFIX) == null) {

				Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap()
						.put(NotationPackage.eNS_PREFIX,
								new XMIResourceFactoryImpl() {
									public Resource createResource(URI uri) {
										XMIResource xmiResource = new XMIResourceImpl(
												uri);
										return xmiResource;
									}
								});
			}
			// no else.

			if (EPackage.Registry.INSTANCE.getEPackage(NotationPackage.eNS_URI) == null) {
				EPackage.Registry.INSTANCE.put(NotationPackage.eNS_PREFIX,
						NotationPackage.eINSTANCE);
			}
			// no else.
		}
		// no else.
	}
	
	
	
	public MultiArtifactDependencies getMultiArtifactDependencies(FragmentSource source, OriginalFragmentLocation location) {
		MultiArtifactDependencies dependencies = new MultiArtifactDependencies(location);
		addDependencies(dependencies, source, location);
		return dependencies;
	}
	
	private void addDependencies(MultiArtifactDependencies dependencies,
			FragmentSource source, OriginalFragmentLocation location) {
		List<OriginalFragmentLocation> dependencyLocations = getDependencyLocations(source, location);
		dependencies.addDependentLocations(location, dependencyLocations);
		for (OriginalFragmentLocation dl : dependencyLocations) {
			addDependencies(dependencies, source, dl);
		}
	}
}
