package org.deft.repository.artifacttype.emftext.dresdenocl;

import org.deft.emftext.EmfTextValidator;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.artifacttype.Validator;

/**
 * Represents an EMFText OCL artifact. This class does deliberately not
 * inherit from EmfTextArtifactType, because the Insert menu in the Project view
 * would then also display the standard EMFText insert actions. This is not desired.
 * 
 * This is actually the only reason why this class does not inherit from EmfTextArtifactType.
 * If the insert menu will be populated differently in a later version, this class can
 * inherit from ArtifactType and make some classes such as Ocl2StyledCodeOperation obsolete. 
 */
public class DresdenOclArtifactType implements ArtifactType {

	//must be same as in plugin.xml
	public static final String ID = "org.deft.repository.artifacttype.emftext.dresdenocl";
	
	@Override
	public String getTypeId() {
		return ID;
	}
	
	private Validator validator;
	
	public Validator getValidator() {
		if (validator == null) {
			validator = new EmfTextValidator("ocl");
		}
		return validator;
	}
	
	public String getEmfTextLanguageId() {
		return "ocl";
	}
	
	@Override
	public String getArtifactTypeName() {
		return "EMFText \"" + getEmfTextLanguageId() + "\"";
	}

}