package org.deft.repository.ast;

import java.util.List;

import org.apache.commons.jxpath.JXPathContext;
import org.apache.commons.jxpath.ri.JXPathContextReferenceImpl;
import org.deft.repository.ast.jxpath.TreeNodePointer;
import org.deft.repository.ast.jxpath.TreeNodePointerFactory;

/**
 * This is not a real tree node. It's just that JXPath can climb through the
 * object tree. 
 * 
 * @author Frank Herrlich
 * @author abartho
 */
public class TreeNodeRoot extends TreeNode {
	private JXPathContext context;
	
	static {
		JXPathContextReferenceImpl.addNodePointerFactory(new TreeNodePointerFactory());
	}
	
	public TreeNodeRoot(TreeNode child) {
		super("root");
		//do not use the normal addChild() method,
		//the child should not know its TreeNodeRoot
		//as parent
		children.add(child);
		child.setTnr(this);
	}	
	
	public TreeNodeRoot(List<TreeNode> newChildren) {
		super("root");
		//TreeNodeRoot can only have one child, so we introduce an artificial one
		//to which in turn the children from the parameter are added
		TreeNode tnSource = new TreeNode("source");
		children.add(tnSource);
		tnSource.setTnr(this);
		
		for (TreeNode child : newChildren) {
			tnSource.addChild(child);			
		}
	}
	
	

	public void addChild(TreeNode child) {
		children.add(child);
		child.setTnr(this);
	}
	
	public JXPathContext getContext() {
		if (context == null)		
			context = JXPathContext.newContext(this);
		return context;
	}	
	
	public TreeNodePointer getPointer()	{
		return new TreeNodePointer(null, this);
	}
	
	public TreeNodeRoot getTreeNodeRoot() {
    	return this;
    }
	
	/**
	 * TreeNodeRoot needs special attention for serialization. The original serialize 
	 * method uses the TreeNodeRoot to determine when it is finished. It is assumed that
	 * the TreeNodeRoot has only one child, which is true for unaltered ASTs. However,
	 * if there are additional children added, serialize will only consider the first child.
	 * Therefore this method is overridden and all children of TreeNodeRoot are explicitly
	 * serialized.
	 */
	@Override
    public void serialize(List<TokenNode> list) {
		for (TreeNode child : getChildren()) {
			child.serialize(list);
		}
    }
	
}
