package org.deft.repository.ast.decoration;

import java.util.LinkedList;
import java.util.List;

import org.deft.repository.ast.jxpath.XPath;

public class ReplaceGroup {

	private String id;
	private String replaceString;
	private List<XPath> xpathsHide = new LinkedList<XPath>();
	private List<XPath> xpathsHideExcept = new LinkedList<XPath>();

	public ReplaceGroup(String id, List<XPath> xpathsHide, List<XPath> xpathsHideExcept, String replaceString) {
		this.id = id;
		if (xpathsHide != null) {
			add(this.xpathsHide, xpathsHide);
		}
		if (xpathsHideExcept != null) {
			add(this.xpathsHideExcept, xpathsHideExcept);
		}
		this.replaceString = replaceString;
	}
	
	private void add(List<XPath> local, List<XPath> extParams) {
		for (XPath p : extParams) {
			local.add(addLeadingSlash(p));
		}
	}
	
	private XPath addLeadingSlash(XPath xPath) {
		if (!xPath.getXPathString().startsWith("/")) {
			return new XPath("/" + xPath);
		} else {
			return xPath;
		}
	}
	
	public String getId() {
		return id;
	}
	
	public String getReplaceString() {
		return replaceString;
	}
	
	public List<XPath> getXPathsHide() {
		return xpathsHide;
	}
	
	public List<XPath> getXPathsHideExcept() {
		return xpathsHideExcept;
	}
	
	/**
	 * Returns whether a node identified by an XPath is hidden
	 * by this ReplaceGroup. A node is hidden, if the hide list
	 * contains the XPath or if the hideExcept list does NOT 
	 * contain the XPath (If the hideExcept list is empty, though,
	 * we ignore it. Otherwise everything would be considered 
	 * hidden.)
	 * 
	 * @param xPath
	 * @return
	 */
	public boolean isHidden(XPath xPath) {
		XPath p = addLeadingSlash(xPath);
		if (xpathsHide.contains(p)) {
			return true;
		}
		if (!xpathsHideExcept.isEmpty() && !xpathsHideExcept.contains(p)) {
			return true;
		}
		return false;
	}
	
	public String toString() {
		String sHide = xpathsHide.isEmpty() ? "" : " hide: " + xpathsHide.toString();
		String sHideExcept = xpathsHideExcept.isEmpty() ? "" : " hideExcept: " + xpathsHideExcept.toString();
		String sReplace = replaceString == null ? "" : " replace: " + replaceString;
		return "ReplaceGroup:"  + sHide + sHideExcept + sReplace;
	}

}
