package org.deft.repository.ast.decoration.selected;

import java.util.LinkedList;
import java.util.List;

import org.deft.repository.ast.TokenNode;
import org.deft.repository.ast.decoration.Group;
import org.deft.repository.ast.decoration.Templates;
import org.deft.repository.ast.jxpath.XPath;
import org.w3c.dom.Document;
import org.w3c.dom.Element;


public class SelectedGroup extends Group {
    
    private XPath xpath;
    private String artifactRefId;

    public SelectedGroup(XPath xpath, String artifactRefId) {
    	this.xpath = xpath;
    	this.artifactRefId = artifactRefId;
    }
    
    
    public XPath getXPath() {
    	return xpath;
    }
    
    public String getArtifactReferenceId() {
    	return artifactRefId;
    }
    
    public boolean isSelected() {
        return xpath != null;
    }
    
    //returns whether an artifact reference is saved in the group
    public boolean hasReference() {
    	return artifactRefId != null;
    }
    
    public boolean isDummy() {
    	return !isSelected();
    }
    

    protected Element createXmlTag(Document doc) {
    	List<String> l = new LinkedList<String>();
    	if (isSelected()) {
    		for (TokenNode tn : getTokenNodeList()) {
    			l.add(tn.getToken().getText());
    		}
    	}
    	
        Element element = doc.createElement(Templates.SELECTED.getIdString());
        element.setAttribute("value", String.valueOf(isSelected()));
        if (isSelected()) {
            if (hasReference()) {
            	element.setAttribute("refid", artifactRefId);
            }
        }
        return element;
    }      
    
    public String toString() {
        return "SelectedGroup " + (isSelected() ? "selected" : "unselected");
    }


}
