package org.deft.repository.chaptertype;

/**
 * This is a simple data holder class, which holds all the data that defines a chapter type. 
 * 
 */
public class ChapterType {
	
	private final String namespaceId;
	private final String description;
	private final String chapterTemplateLocation;
	private final String resourceLocation;
	private final String id;
	
	public ChapterType(String namespaceId, String description, String resourceLocation, 
			String chapterTemplateLocation, String id) {
		this.namespaceId = namespaceId;
		this.description = description;
		this.chapterTemplateLocation = chapterTemplateLocation;
		this.resourceLocation = resourceLocation;
		this.id = id;
	}
	
	/**
	 * The name of the project in which the project type is defined, 
	 * e.g. org.deft.repository.chaptertype.xhtml
	 */
	public String getNamespaceId() {
		return namespaceId;
	}
	
	/**
	 * The description of this type which is displayed within the GUI.
	 */
	public String getDescription() {
		return description;
	}
	
	/**
	 * The location of the chapter template used if a new chapter is created. The
	 * location is relative to the resource location.
	 */
	public String getChapterTemplateLocation() {
		return chapterTemplateLocation;
	}

	/**
	 * A system wide unique id to identify this type.
	 */
	public String getId() {
		return id;
	}

	/**
	 * The main folder for resources, such as the initial template.
	 */
	public String getResourceLocation() {
		return resourceLocation;
	}
	
	public String toString() {
		return "ChapterType: " + id;
	}

}
