package org.deft.repository.chaptertype;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;

public class EclipseChapterType2IntegratorIdMappingRegistry implements ChapterType2IntegratorIdMappingRegistry {
	
	private static final String MAPPING_EXTENSION_POINT_NAME = "org.deft.repository.chaptertype2integratoridmapping";
		
	private Map<String, List<ChapterType2IntegratorIdMappingConfig>> registeredMappings 
			= new HashMap<String, List<ChapterType2IntegratorIdMappingConfig>>();
	
	
	public EclipseChapterType2IntegratorIdMappingRegistry() {
		getAndCacheRegisteredMappings();
	}
	
	private void getAndCacheRegisteredMappings() {
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint point = registry.getExtensionPoint(MAPPING_EXTENSION_POINT_NAME);
		for (IConfigurationElement ce : point.getConfigurationElements()) {
			ChapterType2IntegratorIdMappingConfig config = new ChapterType2IntegratorIdMappingConfig(ce);
			String chapterTypeId = config.getChapterTypeId();
			List<ChapterType2IntegratorIdMappingConfig> configs = registeredMappings.get(chapterTypeId);
			if (configs == null) {
				configs = new LinkedList<ChapterType2IntegratorIdMappingConfig>();
				registeredMappings.put(chapterTypeId, configs);
			}
			configs.add(config);
		}
	}
	
	@Override
	public List<String> getIntegratorIds(String chapterTypeId) {
		List<String> integratorIds = new LinkedList<String>();
		List<ChapterType2IntegratorIdMappingConfig> configs = registeredMappings.get(chapterTypeId);
		if (configs != null) {
			for (ChapterType2IntegratorIdMappingConfig config : configs) {
				integratorIds.add(config.getIntegratorId());
			}
		}
		return integratorIds;
	}
	
	

	@Override
	public List<String> getChapterTypeIds(String integratorId) {
		List<String> chapterTypeIds = new LinkedList<String>();
		for (List<ChapterType2IntegratorIdMappingConfig> configs : registeredMappings.values()) {
			for (ChapterType2IntegratorIdMappingConfig config : configs) {
				String configIntegratorId = config.getIntegratorId();
				if (configIntegratorId.equals(integratorId)) {
					chapterTypeIds.add(config.getChapterTypeId());
				}
			}
		}
		return chapterTypeIds;
	}



	
	
	/**
	 * This class offers convenient access to the mappings by
	 * encapsulating the Eclipse internal extension mechanism. There is
	 * one ChapterType2IntegratorIdMappingConfig for each concrete chaptertype2integratoridmapping.
	 * 
	 * The config contains the id of the chapter type, and a list of 
	 * ids of integrators that are applicable on the respective chapter type. 
	 */
	private class ChapterType2IntegratorIdMappingConfig {
	
		private IConfigurationElement ce;
		
		public ChapterType2IntegratorIdMappingConfig(IConfigurationElement ce) {
			this.ce = ce;
		}
		
		/**
		 * Returns the id of the chapter type
		 */
		public String getChapterTypeId() {
			String id = ce.getAttribute("chaptertype_id");
			return id;
		}
		
		/**
		 * Returns the list of integrator id for the chapter type
		 */
		public String getIntegratorId() {
			String integratorId = ce.getAttribute("integrator_id");
			return integratorId;
		}

	}



}