package org.deft.repository.chaptertype;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;


/**
 * All chapter types (XHTML, Docbook, OOText) are registered within this registry.
 */
public class EclipseChapterTypeRegistry implements ChapterTypeRegistry {
	
	private static final String EXTENSION_POINT_NAME = "org.deft.repository.chaptertype";
		
	private final Map<String, ChapterType> registeredChapterTypes = new HashMap<String, ChapterType>(); 
		
	public EclipseChapterTypeRegistry() {
		loadChapterTypes();
	}
	
	private void loadChapterTypes() {
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint point = registry.getExtensionPoint(EXTENSION_POINT_NAME);
		if (point == null)
			return;
		
		for (IConfigurationElement ce : point.getConfigurationElements()) {
			if (ce.getName().equals("chaptertype")) {
				String namespaceId = ce.getNamespaceIdentifier();
				String desc = ce.getAttribute("description");
				String resourceslocation = ce.getAttribute("resourceslocation");
				String template = ce.getAttribute("template");
				String id = ce.getAttribute("id");
				ChapterType chapterType = new ChapterType(
						namespaceId, desc, resourceslocation, template, id);
 				registeredChapterTypes.put(chapterType.getId(), chapterType);
			}
		}
	}
	

	@Override
	public ChapterType getChapterType(String chapterTypeId) {
		return registeredChapterTypes.get(chapterTypeId);
	}

	@Override
	public List<String> getChapterTypeIds() {
		List<String> ids = new LinkedList<String>(registeredChapterTypes.keySet());
		return ids;
	}

	@Override
	public List<ChapterType> getChapterTypes() {
		List<ChapterType> types = new LinkedList<ChapterType>(registeredChapterTypes.values());
		return types;
	}

}
