package org.deft.repository.chaptertype;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.deft.repository.integrator.Integrator;

/**
 * Stores and returns temporary information about artifact integration.
 * 
 * When a document is saved, all integrated representations 
 * in that document must be replaced by the original representations.
 * Otherwise the representation links will be lost. A ChapterDocument
 * can (and should) use the IntegrationRecorder to store all performed
 * integrations.
 * 
 * The recorded information are:
 * - integration id: an ID representing the integration process
 * - integrator: the integrator that has performed the integration
 * - reference id: the ID of the artifact reference that has been replaced
 * 
 * Integration id and reference id can be the same.
 * 
 * When the ChapterDocument is discarded (closed), the IntegrationRecorder can be
 * discarded along with it.
 */
public class IntegrationRecorder {

	private Map<String, Integrator> integratorMapping = new HashMap<String, Integrator>();
	private Map<String, String> refIdMapping = new HashMap<String, String>();
	private Map<String, String> refIdMapping2 = new HashMap<String, String>();
	
	public void recordIntegration(String integrationId, String artifactRefId, 
			Integrator integrator) {
		integratorMapping.put(integrationId, integrator);
		refIdMapping.put(integrationId, artifactRefId);
		refIdMapping2.put(artifactRefId, integrationId);
	}
	
	
	public Integrator getIntegrator(String integrationId) {
		Integrator integrator = integratorMapping.get(integrationId);
		return integrator;
	}
	
	
	public String getArtifactRefId(String integrationId) {
		String artifactRefId = refIdMapping.get(integrationId);
		return artifactRefId;
	}
	
	
	public String getIntegrationId(String artifactRefId) {
		String integrationId = refIdMapping2.get(artifactRefId);
		return integrationId;
	}
	
	
	public List<String> getIntegrationIds() {
		return new LinkedList<String>(integratorMapping.keySet());
	}
	
}
