package org.deft.repository.check;

import java.util.List;

import org.deft.repository.fragmentsource.FragmentSource;
import org.deft.repository.fragmentsource.OriginalFragmentLocation;

/**
 * Checks whether a specific ArtifactType is known to the repository.
 * If not, an UnknownArtifactTypeError is created which can be used
 * for further error reporting and handling.
 * 
 * @author abartho
 */
public class ArtifactTypeKnownCheck extends DeftCheck {

	private OriginalFragmentLocation location;
	private FragmentSource fragmentSource;
	
	public ArtifactTypeKnownCheck(FragmentSource fragmentSource, OriginalFragmentLocation location) {
		this.fragmentSource = fragmentSource;
		this.location = location;
	}
	
	@Override
	public void perform() {
		clearErrors();
		if (isFileExtensionUnknown()) {
			UnknownArtifactTypeError error = new UnknownArtifactTypeError(location);
			addError(error);
		}
	}
	

	private boolean isFileExtensionUnknown() {
		List<String> artifactTypeIds = fragmentSource.getArtifactTypeIds(location);
		return artifactTypeIds.isEmpty();
	}

	
}
