package org.deft.repository.datamodel;

/**
* Filter that returns the fragment specified by the given path and type. 
* A path looks like a file system path, i.e. it consists of slash ( '/' ) separated identifiers,
* which denote the fragments.
* 
* Example: /some/folder/MainChapter
*/
public class PathFragmentFilter implements FragmentFilter {

	private String path;
	private FragmentType fragType;
	
	public PathFragmentFilter(String path, FragmentType fragType) {
		this.path = path;
		this.fragType = fragType;
		if (!this.path.startsWith("/")) {
			this.path = "/" + this.path;
		}
	}
	
	@Override
	public boolean accept(Fragment fragment) {
		if (fragment.getFragmentType().equals(fragType)) {
			String fragPath = fragment.getProjectRelativePath();
			if (!fragPath.startsWith("/")) {
				fragPath = "/" + fragPath;
			}
			return fragPath.equals(path);
		}

		return false;
	}

}
