package org.deft.repository.datamodel;

import java.io.File;
import java.util.List;

import org.deft.repository.projecttype.ProjectType;

public interface Project extends Nameable, Identifiable {
	
	/**
	 * Returns the type id of this project, e.g. xhtml or latex. The type can
	 * be used to determine what types of chapters are allowed. 
	 */
	public String getProjectTypeId();
	
	public ProjectType getProjectType();

	/**
	 * Returns all fragments of a specific type from the project root
	 * (i.e., for fragments which have no parent folder). If there are no
	 * fragments of the wanted type, an empty list is returned.
	 */
	public List<Fragment> getChildren(FragmentType fragmentType);
	
	/**
	 * Returns a list of fragments from the project. Only fragments
	 * matching the filter are contained in the list. If the filter is 
	 * null, all fragments of the project are returned. If no fragments match,
	 * an empty list is returned.
	 * 
	 * Please note: in contrast to {@link #getChildren(FragmentType)}
	 * ALL fragments are considered, not only the fragments from the project
	 * root.
	 */
	public List<Fragment> getFragments(FragmentFilter filter);
	
	public List<ArtifactReference> getArtifactReferences();
	
	public List<ArtifactReference> getUncheckedArtifactReferences();
	
	public int getLatestRevision();
	
	public File getTempDirectory();
	
}
