package org.deft.repository.datamodel.adapter.ecore;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.ecoredatamodel.Artifact;
import org.deft.repository.datamodel.ecoredatamodel.Revision;
import org.deft.repository.repositorymanager.ContentManagementService;
import org.deft.repository.service.ExtensionService;

public class DeftArtifactImpl extends DeftFragmentImpl implements org.deft.repository.datamodel.Artifact {
	
	private ExtensionService es;
	
	public DeftArtifactImpl(Artifact ecoreArtifact, ContentManagementService cms, ExtensionService es) {
		super(ecoreArtifact, cms);
		this.es = es;
	}
	
	public org.deft.repository.datamodel.FragmentType getFragmentType() {
		return org.deft.repository.datamodel.FragmentType.ARTIFACT;
	}

	@Override
	public String getFragmentSourceId() {
		Artifact ecoreArtifact = (Artifact)getEcoreFragment();
		String source = ecoreArtifact.getSourceId();
		return source;
	}

	@Override
	public ArtifactType getArtifactType() {
		String artifactTypeId = getArtifactTypeId();
		ArtifactType type = es.getArtifactType(artifactTypeId);
		return type;
	}
	
	@Override
	public String getArtifactTypeId() {
		Artifact ecoreArtifact = (Artifact)getEcoreFragment();
		String type = ecoreArtifact.getArtifactType();
		return type;
	}

	@Override
	public String getOriginalLocation() {
		ContentManagementService cms = getContentManagementService();
		String location = cms.getOriginalLocation(this);
		return location;
	}
	
	@Override
	public String getOriginalLocation(int revision) {
		ContentManagementService cms = getContentManagementService();
		String location = cms.getOriginalLocation(this, revision);
		return location;
	}
	
	@Override
	public String getOriginalLocation(String dependentFileId) {
		ContentManagementService cms = getContentManagementService();
		String location = cms.getOriginalLocation(this, dependentFileId);
		return location;	
	}
	
	@Override
	public String getOriginalLocation(String dependentFileId, int revision) {
		ContentManagementService cms = getContentManagementService();
		String location = cms.getOriginalLocation(this, dependentFileId, revision);
		return location;	
	}

	@Override
	public byte[] getArtifactContent() {
		ContentManagementService cms = getContentManagementService();
		byte[] content = cms.getArtifactContent(this);
		return content;
	}

	@Override
	public byte[] getArtifactContent(int revision) {
		ContentManagementService cms = getContentManagementService();
		byte[] content = cms.getArtifactContent(this, revision);
		return content;
	}
	
	@Override
	public byte[] getContent(String dependentFileId) {
		ContentManagementService cms = getContentManagementService();
		byte[] content = cms.getContent(this, dependentFileId);
		return content;
	}

	@Override
	public byte[] getContent(String dependentFileId, int revision) {
		ContentManagementService cms = getContentManagementService();
		byte[] content = cms.getContent(this, dependentFileId, revision);
		return content;
	}
	
	@Override
	public List<String> getDependentFileIds() {
		ContentManagementService cms = getContentManagementService();
		List<String> ids = cms.getDependentFileIds(this);
		return ids;
	}

	@Override
	public List<String> getDependentFileIds(int revision) {
		ContentManagementService cms = getContentManagementService();
		List<String> ids = cms.getDependentFileIds(this, revision);
		return ids;
	}


	@Override
	public List<ArtifactReference> getArtifactReferences() {
		ContentManagementService cms = getContentManagementService();
		List<ArtifactReference> references = cms.getArtifactReferences(this);
		return references;
	}

	@Override
	public List<Integer> getRevisions() {
		Artifact ecoreArtifact = (Artifact)getEcoreFragment();
		List<Integer> revisions = new LinkedList<Integer>();
		for (Revision rev : ecoreArtifact.getRevisions()) {
			int revisionNumber = rev.getRevisionNumber();
			revisions.add(revisionNumber);
		}
		Collections.sort(revisions);
		return revisions;
	}
	
	@Override
	public int getLatestRevision() {
		List<Integer> revisions = getRevisions();
		int latestRev = revisions.get(revisions.size() - 1);
		return latestRev;
	}
	

	
	@Override
	public String toString() {
		return "Artifact " + getName() + " " + getId();
	}

}
