package org.deft.repository.datamodel.adapter.ecore;

import org.deft.repository.datamodel.Folder;
import org.deft.repository.datamodel.ecoredatamodel.Fragment;
import org.deft.repository.repositorymanager.ContentManagementService;

public abstract class DeftFragmentImpl implements org.deft.repository.datamodel.Fragment {
	
	private Fragment ecoreFragment;
	private ContentManagementService cms;
	
	public DeftFragmentImpl(Fragment ecoreFragment, ContentManagementService cms) {
		if (ecoreFragment == null) {
			throw new RuntimeException("Null fragment");
		}
		this.ecoreFragment = ecoreFragment;
		this.cms = cms;
	}
	
	protected ContentManagementService getContentManagementService() {
		return cms;
	}

	@Override
	public String getName() {
		return ecoreFragment.getName();
	}

	@Override
	public String getId() {
		return ecoreFragment.getId();
	}
	
	@Override
	public org.deft.repository.datamodel.Folder getParentFolder() {
		org.deft.repository.datamodel.Folder folder = cms.getParentFolder(this);
		return folder;
	}
	
	@Override
	public org.deft.repository.datamodel.Project getProject() {
		org.deft.repository.datamodel.Project project = cms.getProjectOf(this);
		return project;
	}
	
	public Fragment getEcoreFragment() {
		return ecoreFragment;
	}

	@Override
	public boolean equals(Object o) {
		if (o instanceof DeftFragmentImpl) {
			DeftFragmentImpl dfi = (DeftFragmentImpl)o;
			return dfi.getId().equals(this.getId());
		}
		return false;
	}
	
	@Override
	public int hashCode() {
		return getId().hashCode();
	}
	
	@Override
	public String getProjectRelativePath() {
		Folder parent = getParentFolder();
		if (parent == null) {
			return getName();
		} else {
			String parentPath = parent.getProjectRelativePath();
			String path = parentPath + "/" + getName();
			return path;
		}
	}
	
}
