package org.deft.repository.datamodel.adapter.ecore;

import java.io.File;
import java.util.List;

import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.FragmentFilter;
import org.deft.repository.datamodel.ecoredatamodel.Project;
import org.deft.repository.projecttype.ProjectType;
import org.deft.repository.repositorymanager.ContentManagementService;
import org.deft.repository.service.ExtensionService;

public class DeftProjectImpl implements org.deft.repository.datamodel.Project {
	
	private Project ecoreProject;
	private ContentManagementService cms;
	private ExtensionService es;
	
	public DeftProjectImpl(Project ecoreProject, ContentManagementService cms, ExtensionService es) {
		this.ecoreProject = ecoreProject;
		this.cms = cms;
		this.es = es;
	}

	@Override
	public String getName() {
		return ecoreProject.getName();
	}

	@Override
	public String getId() {
		return ecoreProject.getId();
	}
	
	@Override
	public String getProjectTypeId() {
		return ecoreProject.getProjectType();
	}
	
	
	@Override
	public ProjectType getProjectType() {
		String projectTypeId = getProjectTypeId();
		ProjectType type = es.getProjectType(projectTypeId);
		return type;
	}

	@Override
	public List<org.deft.repository.datamodel.Fragment> getChildren(org.deft.repository.datamodel.FragmentType fragmentType) {
		List<org.deft.repository.datamodel.Fragment> children = cms.getChildren(this, fragmentType);
		return children;
	}
	
	@Override
	public List<Fragment> getFragments(FragmentFilter filter) {
		List<Fragment> fragments = cms.getFragments(this, filter);
		return fragments;
	}
	
	@Override
	public List<ArtifactReference> getArtifactReferences() {
		List<ArtifactReference> references = cms.getArtifactReferences(this);
		return references;
	}
	
	@Override
	public List<ArtifactReference> getUncheckedArtifactReferences() {
		List<ArtifactReference> references = cms.getUncheckedReferences(this);
		return references;
	}

	
	public Project getEcoreProject() {
		return ecoreProject;
	}
	
	@Override
	public boolean equals(Object o) {
		if (o instanceof DeftProjectImpl) {
			DeftProjectImpl dpi = (DeftProjectImpl)o;
			return dpi.getId().equals(this.getId());
		}
		return false;
	}
	
	@Override
	public int hashCode() {
		return ecoreProject.hashCode();
	}

	@Override
	public int getLatestRevision() {
		return cms.getLatestRevision(this);
	}
	
	@Override
	public File getTempDirectory() {
		return cms.getTempDirectory(this);
	}


}
