package org.deft.repository.datamodel.adapter.ecore;

import java.util.LinkedList;
import java.util.List;

import org.deft.repository.Repository;
import org.deft.repository.datamodel.ecoredatamodel.ArtifactReferenceUpdateType;
import org.deft.repository.datamodel.ecoredatamodel.Artifact;
import org.deft.repository.datamodel.ecoredatamodel.ArtifactReference;
import org.deft.repository.datamodel.ecoredatamodel.Chapter;
import org.deft.repository.datamodel.ecoredatamodel.Folder;
import org.deft.repository.datamodel.ecoredatamodel.Fragment;
import org.deft.repository.datamodel.ecoredatamodel.FragmentType;
import org.deft.repository.datamodel.ecoredatamodel.Project;
import org.deft.repository.datamodel.ecoredatamodel.Referable;
import org.deft.repository.datamodel.ecoredatamodel.Tutorial;
import org.deft.repository.repositorymanager.ContentManagementService;
import org.deft.repository.service.ExtensionService;

public class EcoreWrapper {
	
	private Repository repository;
	
	public EcoreWrapper(Repository repository) {
		this.repository = repository;
	}
	
	private ContentManagementService getCms() {
		return repository.getContentManagementService();
	}
	
	private ExtensionService getEs() {
		return repository.getExtensionService();
	}
	
	
	public org.deft.repository.datamodel.Chapter wrap(Chapter eChapter) {
		return eChapter == null ? null : new DeftChapterImpl(eChapter, getCms());
	}
	
	public Chapter unwrap(org.deft.repository.datamodel.Chapter chapter) {
		return chapter == null ? null : (Chapter)((DeftChapterImpl)chapter).getEcoreFragment();
	}
	
	
	
	public org.deft.repository.datamodel.Tutorial wrap(Tutorial eTutorial) {
		return eTutorial == null ? null : new DeftTutorialImpl(eTutorial, getCms());
	}
	
	public Tutorial unwrap(org.deft.repository.datamodel.Tutorial tutorial) {
		return tutorial == null ? null : (Tutorial)((DeftTutorialImpl)tutorial).getEcoreFragment();
	}
	
	
	
	public org.deft.repository.datamodel.Artifact wrap(Artifact eArtifact) {
		return eArtifact == null ? null : new DeftArtifactImpl(eArtifact, getCms(), getEs());
	}

	

	public Artifact unwrap(org.deft.repository.datamodel.Artifact artifact) {
		return artifact == null ? null : (Artifact)((DeftArtifactImpl)artifact).getEcoreFragment();
	}
	
	
	
	public org.deft.repository.datamodel.Folder wrap(Folder eFolder) {
		return eFolder == null ? null : new DeftFolderImpl(eFolder, getCms());
	}
	
	public Folder unwrap(org.deft.repository.datamodel.Folder folder) {
		return folder == null ? null : (Folder)((DeftFolderImpl)folder).getEcoreFragment();
	}
	
	
	
	public org.deft.repository.datamodel.Fragment wrap(Fragment eFragment) {
		if (eFragment instanceof Chapter) {
			return wrap((Chapter)eFragment);
		} else if (eFragment instanceof Artifact) {
			return wrap((Artifact)eFragment);
		} else if (eFragment instanceof Tutorial) {
			return wrap((Tutorial)eFragment);
		} else if (eFragment instanceof Folder) {
			return wrap((Folder)eFragment);
		}  
		return null;
	}
	
	public Fragment unwrap(org.deft.repository.datamodel.Fragment fragment) {
		return fragment == null ? null : (Fragment)((DeftFragmentImpl)fragment).getEcoreFragment();
	}
	
	public Referable unwrap(org.deft.repository.datamodel.Referable referable) {
		return referable == null ? null : (Referable)((DeftFragmentImpl)referable).getEcoreFragment();
	}
	
	
	public org.deft.repository.datamodel.Project wrap(Project eProject) {
		return eProject == null ? null : new DeftProjectImpl(eProject, getCms(), getEs());
	}
	
	public Project unwrap(org.deft.repository.datamodel.Project project) {
		return project == null ? null : ((DeftProjectImpl)project).getEcoreProject();
	}
	
	
	public org.deft.repository.datamodel.FragmentType wrap(FragmentType eFragType) {
		return eFragType == null ? null : org.deft.repository.datamodel.FragmentType.get(eFragType.getValue());
	}
	
	public FragmentType unwrap(org.deft.repository.datamodel.FragmentType fragType) {
		return fragType == null ? null : FragmentType.get(fragType.getValue());
	}
	
	
	public org.deft.repository.datamodel.ArtifactReferenceUpdateType wrap(ArtifactReferenceUpdateType eUpdateType) {
		return eUpdateType == null ? null : org.deft.repository.datamodel.ArtifactReferenceUpdateType.get(eUpdateType.getValue());
	}
	
	public ArtifactReferenceUpdateType unwrap(org.deft.repository.datamodel.ArtifactReferenceUpdateType updateType) {
		return updateType == null ? null : ArtifactReferenceUpdateType.get(updateType.getValue());
	}
	
	
	public org.deft.repository.datamodel.ArtifactReference wrap(ArtifactReference eArtifactReference) {
		return eArtifactReference == null ? null : new DeftArtifactReferenceImpl(eArtifactReference, getCms());
	}
			
	
	public ArtifactReference unwrap(org.deft.repository.datamodel.ArtifactReference artifactReference) {
		return artifactReference == null ? null : 
				(ArtifactReference)((DeftArtifactReferenceImpl)artifactReference).getEcoreArtifactReference();
	}
	

	
	@SuppressWarnings("unchecked")
	public <T extends Fragment, C> List<C> convertFragmentList(List<T> eList, Class<C> clazz) {
		List<C> list = new LinkedList<C>();
		for (T t : eList) {
			list.add((C)wrap(t));
		}
		return list;
	}
	
	public List<org.deft.repository.datamodel.Project> convertProjectList(List<Project> eList) {
		List<org.deft.repository.datamodel.Project> list = new LinkedList<org.deft.repository.datamodel.Project>();
		for (Project project : eList) {
			list.add(wrap(project));
		}
		return list;
	}
	
	@SuppressWarnings("unchecked")
	public <T extends ArtifactReference, C> List<C> convertArtifactReferenceList(List<T> eList, Class<C> clazz) {
		List<C> list = new LinkedList<C>();
		for (T t : eList) {
			list.add((C)wrap(t));
		}
		return list;
	}
	
	

}
