package org.deft.repository.event;

import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.Project;

public class FragmentUpdateEvent extends RepositoryUpdateEvent {

	private Project project;
	private Fragment fragment;
	private Fragment oldParent;
	private String oldName;
	

	public FragmentUpdateEvent(Fragment fragment, UpdateType updateType, boolean direct) {
		super(updateType, direct);
		this.fragment = fragment;
	}
	

	/**
	 * Constructor for a remove or move event 
	 * @param fragment
	 * @param oldParent
	 * @param project the project of the fragment. If this event is a remove, the project parameter is the only way to determine
	 * from which project the fragment came, because fragment.getProject() will only return null.
	 * @param updateType
	 * @param direct
	 */
	public FragmentUpdateEvent(Fragment fragment, Fragment oldParent, Project project, UpdateType updateType, boolean direct) {
		super(updateType, direct);
		this.fragment = fragment;
		this.oldParent = oldParent;
		this.project = project;
	}
	

	/**
	 * Constructor for a rename event
	 */
	public FragmentUpdateEvent(Fragment fragment, String oldName) {
		super(UpdateType.RENAMED, true);
		this.fragment = fragment;
		this.oldName = oldName;
	}

	public Project getProject() {
		return project;
	}
	
	public Fragment getFragment() {
		return fragment;
	}
	
	public Fragment getOldParent() {
		return oldParent;
	}
	
	public String getOldName() {
		return oldName;
	}
	
	public String toString() {
		return getUpdateType() + " Fragment \"" + fragment.getName() + "\" " + (isDirectlyAffected() ? "directly" : "indirectly");

	}
	
}
