package org.deft.repository.event;

import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.Tutorial;

public class RepositoryRelationUpdateEvent extends RepositoryUpdateEvent {
	
	public static enum RelationType {
		CHAPTER_TO_TUTORIAL;
	}

		
	private RelationType relationType;	
	private Fragment primaryFrag, secondaryFrag;
	private int oldIndex = -1;


	
	public RepositoryRelationUpdateEvent(Tutorial tutorial, Chapter chapter, int oldIndex, UpdateType updateType, boolean direct) {
		super(updateType, direct);
		this.primaryFrag = tutorial;
		this.secondaryFrag = chapter;
		this.relationType = RelationType.CHAPTER_TO_TUTORIAL;
		this.oldIndex = oldIndex;
		
	}
	
	public RepositoryRelationUpdateEvent(Tutorial tutorial, Chapter chapter, UpdateType updateType, boolean direct) {
		this(tutorial, chapter, -1, updateType, direct);
	}
	

	public boolean isChapterToTutorialUpdate() {
		return relationType == RelationType.CHAPTER_TO_TUTORIAL;
	}
	
	public Fragment getPrimaryFragment() {
		return primaryFrag;
	}
	
	public Fragment getSecondaryFragment() {
		return secondaryFrag;
	}
	
	public RelationType getRelationType() {
		return relationType;
	}

	
	public int getOldIndex() {
		return oldIndex;
	}
	
	public String toString() {
		switch (getUpdateType()) {
		case CREATED: return createdUpdateToString();
		case REMOVED: return removedUpdateToString();
		case MOVED: return movedUpdateToString();
		}
		return "Unknown Relation Event Type";
	}
	
	private String createdUpdateToString() {
		return "CREATED relation \"" + getRelationType() + "\" " + (isDirectlyAffected() ? "directly" : "indirectly");
	}
	
	private String removedUpdateToString() {
		return "REMOVED relation \"" + getRelationType() + "\" " + (isDirectlyAffected() ? "directly" : "indirectly");
	}
	
	private String movedUpdateToString() {
		return "MOVED secondary fragment \"" + secondaryFrag.getName() + "\" in \"" + primaryFrag.getName() + "\"" 
		+ " from index " + oldIndex + " to a new index.";
	}
	
	
	
	
}



