package org.deft.repository.event;

public abstract class SimpleRepositoryUpdateListener implements RepositoryUpdateListener {

	private boolean ignoreIndirectEvents;
	
	public SimpleRepositoryUpdateListener(boolean ignoreIndirectEvents) {
		this.ignoreIndirectEvents = ignoreIndirectEvents;
	}
	
	public SimpleRepositoryUpdateListener() {
		this(true);
	}
	
	
	public abstract void repositoryUpdated();
	
	
	protected void repositoryUpdated(RepositoryUpdateEvent event) {
		if (!ignoreIndirectEvents || event.isDirectlyAffected()) {
			repositoryUpdated();
		}
	}
	
	@Override
	public void contentLoaded() {
		repositoryUpdated();
	}
	
	@Override
	public void chapterAddedToTutorial(RepositoryRelationUpdateEvent event) {
		repositoryUpdated(event);
	}

	@Override
	public void chapterPositionChanged(RepositoryRelationUpdateEvent event) {
		repositoryUpdated(event);
	}

	@Override
	public void chapterRemovedFromTutorial(RepositoryRelationUpdateEvent event) {
		repositoryUpdated(event);
	}

	@Override
	public void fragmentCreated(FragmentUpdateEvent event) {
		repositoryUpdated(event);
	}

	@Override
	public void fragmentMoved(FragmentUpdateEvent event) {
		repositoryUpdated(event);
	}
	
	@Override
	public void artifactUpdated(FragmentUpdateEvent event) {
		repositoryUpdated(event);
	}

	@Override
	public void fragmentRemoved(FragmentUpdateEvent event) {
		repositoryUpdated(event);
	}

	@Override
	public void fragmentRenamed(FragmentUpdateEvent event) {
		repositoryUpdated(event);
	}

	@Override
	public void projectCreated(ProjectUpdateEvent event) {
		repositoryUpdated(event);
	}

	@Override
	public void projectRemoved(ProjectUpdateEvent event) {
		repositoryUpdated(event);
	}

	@Override
	public void projectRenamed(ProjectUpdateEvent event) {
		repositoryUpdated(event);
	}

	@Override
	public void artifactReferenceCreated(ArtifactReferenceUpdateEvent event) {
		repositoryUpdated(event);
	}

	@Override
	public void artifactReferenceRemoved(ArtifactReferenceUpdateEvent event) {
		repositoryUpdated(event);
	}
	
	@Override
	public void artifactReferenceUpdated(ArtifactReferenceUpdateEvent event) {
		repositoryUpdated(event);
	}

	@Override
	public void artifactReferenceChecked(ArtifactReferenceUpdateEvent event) {
		repositoryUpdated(event);
	}
	
	@Override
	public void artifactReferenceUnchecked(ArtifactReferenceUpdateEvent event) {
		repositoryUpdated(event);
	}
	
}
