package org.deft.repository.fragmentsource;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;

public class EclipseFragmentSourceRegistry implements FragmentSourceRegistry {
	
	private static final String FRAGMENTSOURCE_EXTENSION_POINT_NAME = "org.deft.repository.fragmentsource";
		
	private Map<String, FragmentSourceConfig> registeredSources = new HashMap<String, FragmentSourceConfig>(); 

	public EclipseFragmentSourceRegistry() {
		getAndCacheRegisteredSources();
	}
	
	private void getAndCacheRegisteredSources() {
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint point = registry.getExtensionPoint(FRAGMENTSOURCE_EXTENSION_POINT_NAME);
		for (IConfigurationElement ce : point.getConfigurationElements()) {
			FragmentSourceConfig config = new FragmentSourceConfig(ce);
			registeredSources.put(config.getId(), config);
		}
	}
	
	@Override
	public FragmentSource getFragmentSource(String fragmentSourceId) {
		FragmentSourceConfig config = registeredSources.get(fragmentSourceId);
		if (config == null) {
			return null;
		}
		FragmentSource source = config.getFragmentSource();
		return source;
	}
	
	public List<String> getFragmentSourceIds() {
		List<String> sourceIds = new LinkedList<String>(registeredSources.keySet());
		return sourceIds;
	}
	
	
	
	private class FragmentSourceConfig {
	
		private IConfigurationElement ce;
		private FragmentSource fragmentSource;
		
		public FragmentSourceConfig(IConfigurationElement ce) {
			this.ce = ce;
		}
		
		public String getId() {
			String id = ce.getAttribute("id");
			return id;
		}
		
		public FragmentSource getFragmentSource() {
			if (fragmentSource == null) {
				try {
					String fragmentSourcePropertyInExtensionPoint = "class";
					fragmentSource = (FragmentSource)ce.createExecutableExtension(
							fragmentSourcePropertyInExtensionPoint);
				} catch (CoreException e) {
					e.printStackTrace();
				}
			}
			return fragmentSource;
		}	
	}
	
}
