package org.deft.repository.fragmentsource;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Abstract superclass of all FragmentSources that work with files. Implements the
 * mapping from file extensions to ArtifactType IDs.
 */
public abstract class FileBasedFragmentSource implements FragmentSource {

	private Map<String, List<String>> extensionsMap;
	
	public FileBasedFragmentSource(Map<String, List<String>> extensionsMap) {
		this.extensionsMap = extensionsMap;
	}
	
	@Override
	public List<String> getArtifactTypeIds(OriginalFragmentLocation location) {
		String fileExtension = getFileExtensionFromLocation(location);
		List<String> artifactTypeIds = new LinkedList<String>();
		for (String artifactTypeId : extensionsMap.keySet()) {
			List<String> extensions = extensionsMap.get(artifactTypeId);
			if (extensions.contains(fileExtension)) {
				artifactTypeIds.add(artifactTypeId);
			}
		}
		return artifactTypeIds;
	}
	
	private String getFileExtensionFromLocation(OriginalFragmentLocation location) {
		String name = location.getName();
		if (hasFileExtension(name)) {
			String extension = doGetExtension(name);
			return extension;
		}
		return null;
	}
	
	private boolean hasFileExtension(String name) {
		int posOfDot = name.lastIndexOf(".");
		return posOfDot != -1;
	}
	
	
	private String doGetExtension(String name) {
		int posOfDot = name.lastIndexOf(".");
		String extension = name.substring(posOfDot + 1);
		return extension;
	}
	
	
	public Map<String, List<String>> getRegisteredFileExtensions() {
		Map<String, List<String>> copy = new HashMap<String, List<String>>(extensionsMap);
		Set<String> artifactTypeIds = extensionsMap.keySet();
		for (String artifactTypeId : artifactTypeIds) {
			List<String> fileExtensions = new LinkedList<String>();
			fileExtensions.addAll(extensionsMap.get(artifactTypeId));
			copy.put(artifactTypeId, fileExtensions);
		}
		return copy;
	}




}
