package org.deft.repository.fragmentsource;

import java.io.InputStream;
import java.util.List;


/**
 * A FragmentSource describes an external repository in which fragments
 * such as source code or models (artifacts) or images reside. Examples 
 * for FragmentSources are the file system, an Eclipse workspace or an SVN repository.
 */
public interface FragmentSource {

	public String getId();
	
	public InputStream getInputStream(OriginalFragmentLocation location);
	
	public boolean doesLocationExist(OriginalFragmentLocation location);
	
	public OriginalFragmentLocation createLocationObject(String stringRepresentation);
	
	/**
	 * Returns the ArtifactTypes (or rather their ids) which a location could represent.
	 * For example, if the location is a file, its ArtifactType could be deduced from the
	 * file type. It is possible that multiple ArtifactTypes are associated with a location
	 * (e.g. file extension), but this should be the exception. 
	 */
	public List<String> getArtifactTypeIds(OriginalFragmentLocation location);
	
	/**
	 * Sometimes a fragment depends on other fragments. For example, an Ecore diagram
	 * depends on an ecore model. This method returns the locations of all fragments
	 * on which some fragment depends. Note that this does NOT work recursively.
	 */
	public List<OriginalFragmentLocation> getLocationsOfDependentFragments(OriginalFragmentLocation location);
}
