package org.deft.repository.fragmentsource;

/**
 * An OriginalFragmentLocation points to a specific piece of data (e.g. file)
 * in a FragmentSource. It is the location from where a fragment has been
 * imported into the repository. 
 * 
 * The location is used to automate fragment updates (especially artifact updates).
 * If the corresponding piece of data (file) still exists at the time of an update,
 * it will be used.
 * 
 * Examples of types of fragment locations:
 * If the FragmentSource is the file system, the location would be a path.
 * If the FragmentSource is an SVN repository, the location would be a URL.
 */
public interface OriginalFragmentLocation {

	public String getFullLocation();
	
	public String getName(); 
	
	/**
	 * OriginalFragmentLocations are used as keys in maps,
	 * so override hashCode() 
	 */
	public int hashCode();
}
