package org.deft.repository.fragmentsource.filesystem;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.deft.repository.Repository;
import org.deft.repository.service.ExtensionService;

/**
 * A special FileSystemFragmentSource which uses the default extension service that the 
 * repository provides. 
 * 
 * This class has been factored out to make FileSystemFragmentSource independent from the Repository
 * interface and increase testability. 
 */
public class DefaultFileSystemFragmentSource extends FileSystemFragmentSource {
	
	private static ExtensionService extensionService = Repository.instance.getExtensionService();
	
	public DefaultFileSystemFragmentSource() {
		super(getExtensionsMap());
	}
	
	private static Map<String, List<String>> getExtensionsMap() {
		Map<String, List<String>> extensionsMap = new HashMap<String, List<String>>();
		List<String> artifactTypeIds = extensionService.getArtifactTypeIds();
		for (String artifactTypeId : artifactTypeIds) {
			List<String> extensions = extensionService.getFileExtensionsForArtifactTypeId(artifactTypeId);
			extensionsMap.put(artifactTypeId, extensions);
		}
		return extensionsMap;
	}
	
}
