package org.deft.repository.fragmentsource.filesystem;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.List;
import java.util.Map;

import org.deft.repository.fragmentsource.FileBasedFragmentSource;
import org.deft.repository.fragmentsource.OriginalFragmentLocation;

public class FileSystemFragmentSource extends FileBasedFragmentSource {

	//should be in sync with id defined in plugin.xml
	private static final String id = "org.deft.repository.fragmentsource.filesystem";
	
	public FileSystemFragmentSource(Map<String, List<String>> extensionsMap) {
		super(extensionsMap);
	}
	
	
	
	@Override
	public InputStream getInputStream(OriginalFragmentLocation location) {
		if (doesLocationExist(location)) {
			FileSystemFragmentLocation fsfl = (FileSystemFragmentLocation)location;
			File file = fsfl.getFile();
			if (file.exists()) {
				FileInputStream fis;
				try {
					fis = new FileInputStream(file);
					BufferedInputStream bis = new BufferedInputStream(fis);
					return bis;
				} catch (FileNotFoundException fnfe) {					
					fnfe.printStackTrace();
				}
			}
		}
		return null;
	}
	
	@Override
	public boolean doesLocationExist(OriginalFragmentLocation location) {
		if (location instanceof FileSystemFragmentLocation) {
			FileSystemFragmentLocation fsfl = (FileSystemFragmentLocation)location;
			File file = fsfl.getFile();
			boolean exists = file.exists();
			return exists;
		} 
		return false;
	}
	
	@Override
	public String getId() {
		return id;
	}


	@Override
	public FileSystemFragmentLocation createLocationObject(String stringRepresentation) {
		File file = new File(stringRepresentation);
		FileSystemFragmentLocation location = new FileSystemFragmentLocation(file);
		return location;
	}


	@Override
	public List<OriginalFragmentLocation> getLocationsOfDependentFragments(OriginalFragmentLocation location) {
		return null;
	}

}
