package org.deft.repository.fragmentsource.filesystem.gui;

import java.io.File;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.deft.fragmentsourcegui.FragmentSourceGui;
import org.deft.repository.fragmentsource.FragmentSource;
import org.deft.repository.fragmentsource.OriginalFragmentLocation;
import org.deft.repository.fragmentsource.filesystem.DefaultFileSystemFragmentSource;
import org.deft.repository.fragmentsource.filesystem.FileSystemFragmentSource;
import org.deft.util.FileUtil;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Display;


public class DirectoryFragmentSourceGui implements FragmentSourceGui {

	private FileSystemFragmentSource fragmentSource = new DefaultFileSystemFragmentSource();
	private OriginalFragmentLocation rootLocation;
	
	@Override
	public String getName() {
		return "Directory";
	}
	
	@Override
	public FragmentSource getFragmentSource() {
		return fragmentSource;
	}
	
	
	@Override
	public OriginalFragmentLocation getCommonRootLocation() {
		return rootLocation;
	}

	@Override
	public List<OriginalFragmentLocation> getLocationsForImport() {
		DirectoryDialog dia = createDirectoryDialog(); 
		dia.open();
		String path = dia.getFilterPath();
		rootLocation = fragmentSource.createLocationObject(path);

		List<OriginalFragmentLocation> result = getLocationsFromDirectory(path);
		return result;
	}
	
	private DirectoryDialog createDirectoryDialog() {
		DirectoryDialog dia = new DirectoryDialog(Display.getDefault().getActiveShell(), SWT.MULTI);
		return dia;
	}
	


	private List<OriginalFragmentLocation> getLocationsFromDirectory(String path) {
		List<OriginalFragmentLocation> locations = new LinkedList<OriginalFragmentLocation>();
		File dir = new File(path);
		List<File> files = getFileNamesRecursively(dir);
		for (File file : files) {
			if (hasRegisteredFileExtension(file)) {
				OriginalFragmentLocation location = fragmentSource.createLocationObject(file.getAbsolutePath());
				locations.add(location);
			}
		}
		return locations;
	}
	
	
	private boolean hasRegisteredFileExtension(File file) {
		if (file.isFile()) {
			Map<String, List<String>> extensions = fragmentSource.getRegisteredFileExtensions();
			String fileName = file.getName();
			int dotIdx = fileName.lastIndexOf('.');
			if (dotIdx > 0) {
				String ext = fileName.substring(dotIdx + 1);
				for (List<String> registeredExts : extensions.values()) {
					if (registeredExts.contains(ext)) {
						return true;
					}
				}
			}
		}
		return false;
	}
	
	
	private List<File> getFileNamesRecursively(File dir) {
		List<File> files = new LinkedList<File>();
		List<File> filesFromDir = Arrays.asList(FileUtil.getFilesFromDirectory(dir));
		files.addAll(filesFromDir);
		File[] subDirs = FileUtil.getSubDirectories(dir);
		for (File subDir : subDirs) {
			List<File> subDirFiles = getFileNamesRecursively(subDir);
			files.addAll(subDirFiles);
		}
		return files;
	}
	
}
