package org.deft.repository.fragmentsource.filesystem.gui;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.deft.fragmentsourcegui.FragmentSourceGui;
import org.deft.repository.fragmentsource.FragmentSource;
import org.deft.repository.fragmentsource.OriginalFragmentLocation;
import org.deft.repository.fragmentsource.filesystem.DefaultFileSystemFragmentSource;
import org.deft.repository.fragmentsource.filesystem.FileSystemFragmentLocation;
import org.deft.repository.fragmentsource.filesystem.FileSystemFragmentSource;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;


public class FileFragmentSourceGui implements FragmentSourceGui {

	private FileSystemFragmentSource fragmentSource = new DefaultFileSystemFragmentSource();
	
	@Override
	public String getName() {
		return "File";
	}
	
	@Override
	public FragmentSource getFragmentSource() {
		return fragmentSource;
	}
	
	@Override
	public OriginalFragmentLocation getCommonRootLocation() {
		return null;
	}

	@Override
	public List<OriginalFragmentLocation> getLocationsForImport() {
		FileDialog dia = createFileDialog(); 
		dia.open();

		String pathname = dia.getFilterPath();
		String[] filenames = dia.getFileNames();

		List<OriginalFragmentLocation> result = new ArrayList<OriginalFragmentLocation>();
		
		for (String filename : filenames) {
			if (representsExistingFile(pathname, filename)) {
				String absoluteFilename = computeAbsolutePath(pathname, filename);
				File file = new File(absoluteFilename);
				FileSystemFragmentLocation location = new FileSystemFragmentLocation(file);
				result.add(location);
			}
		}
		
		return result;
	}
	
	private FileDialog createFileDialog() {
		FileDialog dia = new FileDialog(Display.getDefault().getActiveShell(), SWT.MULTI);
		Map<String, List<String>> registeredFileExtensions = fragmentSource.getRegisteredFileExtensions();
		String[] filters = createFileFilters(registeredFileExtensions);
		dia.setFilterExtensions(filters);
		return dia;
	}
	
	private String[] createFileFilters(Map<String, List<String>> registeredFileExtensions) {
		String[] extensionFilters = convertRegisteredFileExtensionIntoFilterFormat(registeredFileExtensions);
		String[] filters = new String[extensionFilters.length + 1];
		filters[0] = "*.*";
		for (int i = 0; i < extensionFilters.length; i++) {
			filters[i + 1] = extensionFilters[i];
		}
		return filters;
	}
	
	private String[] convertRegisteredFileExtensionIntoFilterFormat(Map<String, List<String>> registeredFileExtensions) {
		
		Set<String> extensionsWithoutDoublings = new HashSet<String>();
		for (List<String> extensionsForSingleArtifactType : registeredFileExtensions.values()) {
			String extensionsString = convertExtensionsForSingleArtifactTypeIntoString(extensionsForSingleArtifactType);
			extensionsWithoutDoublings.add(extensionsString);
		}
		List<String> extensionStrings = new LinkedList<String>(extensionsWithoutDoublings);
		Collections.sort(extensionStrings);
		return extensionStrings.toArray(new String[]{});
	}
	
	private String convertExtensionsForSingleArtifactTypeIntoString(List<String> extensions) {
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < extensions.size(); i++) {
			String extension = extensions.get(i);
			sb.append("*.").append(extension);
			if (areMoreExtensionsToCome(extensions, i)) {
				sb.append(";");
			}
		}
		return sb.toString();
	}
		
	private boolean areMoreExtensionsToCome(List<String> extensions, int i) {
		return i < extensions.size() - 1;
	}
	
	
	
	private boolean representsExistingFile(String pathname, String filename) {
		if (filename != null && !filename.equals("")) {
			String absoluteFilename = computeAbsolutePath(pathname, filename);
			File file = new File(absoluteFilename);
			boolean exists = file.exists();
			return exists;
		}
		return false;
	}
	
	private String computeAbsolutePath(String pathname, String filename) {
		IPath path = new Path(pathname);
		IPath file = path.append(filename);
		return file.toOSString();
	}



}
