package org.deft.repository.integrator;

import java.util.List;

import org.deft.operation.Chainable;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.datamodel.ArtifactRepresentation;

public interface Integrator extends Chainable {

	/**
	 * Integrates an ArtifactRepresentation at a specific position. The position
	 * includes the actual document and the artifact reference in that document.
	 * 
	 * @param referenceId the ID of the reference for which to integrate the representation
	 * @param representation the representation to add to the document
	 * @param artifactType the type of the original artifact from which the representation was created
	 * @param position the actual document and the artifact reference in that document
	 */
	public void integrate(String referenceId, ArtifactRepresentation representation, 
			ArtifactType artifactType, ArtifactReferencePosition position);
	
	
	/**
	 * Undoes integrator-specific work that was done during the integration.
	 * For example, images that were stored in a temporary directory (see Latex)
	 * can be deleted here. 
	 */
	public void cleanup(String referenceId);
	
	
	/**
	 * Returns the IDs of all ChapterTypes on which this Integrator can operate. Usually there
	 * is only one ChapterType per Integrator, but if several ChapterTypes are similar enough
	 * the Integrator might support multiple (e.g. several OpenDocument formats (text, presentation)
	 * or multiple XML formats (XHTML, DocBook)). 
	 */
	public List<String> getChapterTypeIds();
	
	/**
	 * Returns the current configuration of the integrator.
	 */
	public IntegratorConfiguration getIntegratorConfiguration();
	
	/**
	 * Resets the current configuration of the integrator to the default configuration
	 */
	public void resetIntegratorConfiguration();
	
	/**
	 * Returns the ID of this Integrator. Must be the same ID with which 
	 * the Integrator is registered in the IntegratorRegistry 
	 * (see also extension point org.deft.integrator)  
	 */
	public String getId();
	
}
