package org.deft.repository.projecttype;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.deft.repository.chaptertype.ChapterType;
import org.deft.repository.chaptertype.ChapterTypeRegistry;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;

/**
* All project types are registered within this registry.
*/
public class EclipseProjectTypeRegistry implements ProjectTypeRegistry {
	
	private static final String EXTENSION_POINT_NAME = "org.deft.repository.projecttype";
	private ChapterTypeRegistry chapterTypeReg;
		
	private final Map<String, ProjectType> registeredProjectTypes = new HashMap<String, ProjectType>(); 
		
	public EclipseProjectTypeRegistry(ChapterTypeRegistry chapterTypeReg) {
		this.chapterTypeReg = chapterTypeReg;
		loadProjectTypes();
	}
	
	private void loadProjectTypes() {
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint point = registry.getExtensionPoint(EXTENSION_POINT_NAME);
		if (point == null)
			return;
		
		for (IConfigurationElement ce : point.getConfigurationElements()) {
			if (ce.getName().equals("projecttype")) {
				String desc = ce.getAttribute("description");
				String id = ce.getAttribute("id");
				IConfigurationElement[] chapConfigs = ce.getChildren();
				List<ChapterType> chapterTypes = getChapterTypes(chapConfigs);
				ProjectType projectType = new ProjectType(id, desc, chapterTypes);
				registeredProjectTypes.put(id, projectType);
			}
		}
	}
	
	
	private List<ChapterType> getChapterTypes(IConfigurationElement[] ces) {
		List<ChapterType> chapterTypes = new LinkedList<ChapterType>();
		for (IConfigurationElement ce : ces) {
			String id = ce.getAttribute("id");
			ChapterType chapterType = chapterTypeReg.getChapterType(id);
			chapterTypes.add(chapterType);
		}
		return chapterTypes;
	}

	
	public List<ProjectType> getRegisteredProjectTypes() {
		return new LinkedList<ProjectType>(registeredProjectTypes.values());
	}

	@Override
	public ProjectType getProjectType(String projectTypeId) {
		return registeredProjectTypes.get(projectTypeId);
	}

	@Override
	public List<String> getProjectTypeIds() {
		return new LinkedList<String>(registeredProjectTypes.keySet());
	}

	@Override
	public List<ProjectType> getProjectTypes() {
		return new LinkedList<ProjectType>(registeredProjectTypes.values());
	}

}

