package org.deft.repository.repositorymanager;

import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Folder;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.FragmentType;
import org.deft.repository.datamodel.FragmentFilter;
import org.deft.repository.datamodel.Project;
import org.deft.repository.datamodel.Tutorial;
import org.deft.repository.exception.DeftCrossProjectRelationException;
import org.deft.repository.exception.DeftIncompatibleNameException;
import org.deft.repository.exception.DeftIncompatibleTypeException;
import org.deft.repository.exception.DeftInvalidArgumentException;
import org.deft.repository.exception.DeftNullArgumentException;
import org.deft.repository.exception.DeftProjectAlreadyExistsException;
import org.deft.repository.fragmentsource.FragmentSource;
import org.deft.repository.fragmentsource.OriginalFragmentLocation;

public interface ValidationManagerContentManagementService {

	public void createProject(String name) throws DeftNullArgumentException,
			DeftProjectAlreadyExistsException;

	public void getChapterType(Project project)
			throws DeftNullArgumentException, DeftInvalidArgumentException;

	public void createChapter(Project project, String name)
			throws DeftNullArgumentException, DeftInvalidArgumentException,
			DeftIncompatibleNameException;

	public void createChapter(Folder parent, String name)
			throws DeftNullArgumentException, DeftInvalidArgumentException,
			DeftIncompatibleTypeException, DeftIncompatibleNameException;

	public void getChapterType(Chapter chapter)
			throws DeftInvalidArgumentException;

	public void createTutorial(Project project, String name)
			throws DeftNullArgumentException, DeftInvalidArgumentException,
			DeftIncompatibleNameException;

	public void createTutorial(Folder parent, String name)
			throws DeftNullArgumentException, DeftInvalidArgumentException,
			DeftIncompatibleTypeException, DeftIncompatibleNameException;

	public void importArtifact(Project project, String artifactId,
			FragmentSource fragmentSource, OriginalFragmentLocation location,
			String name) throws DeftNullArgumentException,
			DeftInvalidArgumentException, DeftIncompatibleNameException;

	public void importArtifact(Folder parent, String artifactTypeId,
			FragmentSource fragmentSource, OriginalFragmentLocation location,
			String name) throws DeftNullArgumentException,
			DeftInvalidArgumentException, DeftIncompatibleTypeException,
			DeftIncompatibleNameException;

	public void createImage(Project project, String name)
			throws DeftNullArgumentException, DeftInvalidArgumentException,
			DeftIncompatibleNameException;

	public void createImage(Folder parent, String name)
			throws DeftNullArgumentException, DeftInvalidArgumentException,
			DeftIncompatibleTypeException, DeftIncompatibleNameException;

	public void createFolder(Project project, FragmentType fragType, String name)
			throws DeftNullArgumentException, DeftInvalidArgumentException,
			DeftIncompatibleNameException;

	public void createFolder(Folder parent, String name)
			throws DeftNullArgumentException, DeftInvalidArgumentException,
			DeftIncompatibleNameException;

	public void moveToFolder(Folder target, Fragment toMove)
			throws DeftNullArgumentException, DeftInvalidArgumentException,
			DeftCrossProjectRelationException, DeftIncompatibleTypeException;

	public void getParentFolder(Fragment fragment)
			throws DeftNullArgumentException, DeftInvalidArgumentException;

	public void getChildren(Folder folder) throws DeftNullArgumentException,
			DeftInvalidArgumentException;

	public void getChildren(Project project, FragmentType fragmentType)
			throws DeftNullArgumentException, DeftInvalidArgumentException;

	public void getProjectOf(Fragment fragment)
			throws DeftNullArgumentException, DeftInvalidArgumentException;

	public void getProjects();

	public void getProject(String name) throws DeftNullArgumentException;

	public void addChapterToTutorial(Tutorial tutorial, Chapter chapter)
			throws DeftNullArgumentException, DeftInvalidArgumentException,
			DeftCrossProjectRelationException;

	public void removeChapterFromTutorial(Tutorial tutorial, Chapter chapter)
			throws DeftNullArgumentException, DeftInvalidArgumentException;

	public void getChaptersForTutorial(Tutorial tutorial)
			throws DeftNullArgumentException, DeftInvalidArgumentException;

	public void getTutorialsForChapter(Chapter chapter)
			throws DeftNullArgumentException, DeftInvalidArgumentException;

	public void getFragment(String id) throws DeftNullArgumentException;

	public void getFragments(Project project, FragmentFilter filter)
			throws DeftNullArgumentException, DeftInvalidArgumentException;

	public void getFragmentTypeAffiliation(Folder folder)
			throws DeftNullArgumentException, DeftInvalidArgumentException;

	public void setChapterPosition(Tutorial tutorial, Chapter chapter,
			int position) throws DeftNullArgumentException,
			DeftInvalidArgumentException, DeftCrossProjectRelationException;

	public void rename(Project project, String newName)
			throws DeftNullArgumentException, DeftInvalidArgumentException,
			DeftProjectAlreadyExistsException;

	public void rename(Fragment fragment, String newName)
			throws DeftNullArgumentException, DeftInvalidArgumentException,
			DeftIncompatibleNameException;

	public void removeProject(Project project)
			throws DeftNullArgumentException, DeftInvalidArgumentException;

	public void removeFragment(Fragment fragment)
			throws DeftNullArgumentException, DeftInvalidArgumentException;

}
