package org.deft.repository.repositorymanager.actions;

import org.deft.operation.OperationChain;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.event.ArtifactReferenceUpdateEvent;
import org.deft.repository.event.RepositoryUpdateEvent.UpdateType;
import org.deft.repository.integrator.Integrator;
import org.deft.repository.repositorymanager.ContentManager;
import org.deft.repository.repositorymanager.DataStorageManager;

public class CreateArtifactReferenceAction extends AbstractRepositoryAction {

	private ContentManager contentManager;
	private DataStorageManager dataStorageManager;
	private Chapter chapter;
	private Artifact artifact;
	private OperationChain chain;
	private Integrator integrator;
	
	public CreateArtifactReferenceAction(ContentManager cm, DataStorageManager dsm, 
			Chapter chapter, Artifact artifact, OperationChain chain, Integrator integrator) {
		this.contentManager = cm;
		this.dataStorageManager = dsm;
		this.chapter = chapter;
		this.artifact = artifact;
		this.chain = chain;
		this.integrator = integrator;
	}
	
	@Override
	public void execute() {
		ArtifactReference ref = contentManager.createArtifactReference(chapter, artifact,
				chain.getId(), integrator.getId());
		executeSuccessful = dataStorageManager.createArtifactReferenceContent(ref, chain, integrator);
		if (executeSuccessful) {
			createdObject = ref;
		} else {
			contentManager.removeArtifactReference(ref);
		}
	}

	@Override
	public ArtifactReferenceUpdateEvent createEvent(boolean direct) {
		ArtifactReference ref = (ArtifactReference)createdObject;
		Chapter chapter = contentManager.getChapter(ref);
		Artifact artifact = contentManager.getArtifact(ref);
		ArtifactReferenceUpdateEvent event = new ArtifactReferenceUpdateEvent(ref, chapter, artifact, UpdateType.CREATED, direct);
		return event;
	}
	
}
