package org.deft.repository.repositorymanager.actions;

import org.deft.repository.datamodel.Folder;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.FragmentType;
import org.deft.repository.datamodel.Project;
import org.deft.repository.event.FragmentUpdateEvent;
import org.deft.repository.event.RepositoryUpdateEvent.UpdateType;
import org.deft.repository.repositorymanager.ContentManager;

public class CreateFolderAction extends AbstractRepositoryAction {

	private ContentManager contentManager;
	private Project project;
	private FragmentType fragType;
	private Folder parentFolder;
	private String name;
	private boolean createAtRoot; 

	public CreateFolderAction(ContentManager cm, Project project, 
			FragmentType fragType, String name) {
		this.contentManager = cm;
		this.project = project;
		this.fragType = fragType;
		this.name = name;
		this.createAtRoot = true;
	}
	
	public CreateFolderAction(ContentManager cm, Folder parent, String name) {
		this.contentManager = cm;
		this.parentFolder = parent;
		this.name = name;
		this.createAtRoot = false;
	}
	
	
	@Override
	public void execute() {
		if (createAtRoot) {
			createFolderAtRoot();
		} else {
			createFolderAsSubfolder();
		}
	}
	
	private void createFolderAtRoot() {
		Folder folder = contentManager.createFolder(project, fragType, name);
		executeSuccessful = true;
		createdObject = folder;
	}
	
	private void createFolderAsSubfolder() {
		Folder folder = contentManager.createFolder(parentFolder, name);
		executeSuccessful = true;
		createdObject = folder;
	}

	@Override
	public FragmentUpdateEvent createEvent(boolean direct) {
		Fragment fragment = (Fragment)createdObject;
		FragmentUpdateEvent event = new FragmentUpdateEvent(fragment, UpdateType.CREATED, direct);
		return event;
	}


}
