package org.deft.repository.repositorymanager.actions;

import java.io.IOException;

import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.artifacttype.MultiArtifactDependencies;
import org.deft.repository.artifacttype.Validator;
import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.Folder;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.datamodel.Project;
import org.deft.repository.event.FragmentUpdateEvent;
import org.deft.repository.event.RepositoryUpdateEvent.UpdateType;
import org.deft.repository.fragmentsource.FragmentSource;
import org.deft.repository.fragmentsource.OriginalFragmentLocation;
import org.deft.repository.repositorymanager.ContentManager;
import org.deft.repository.repositorymanager.DataStorageManager;

public class ImportArtifactAction extends AbstractRepositoryAction {

	private ContentManager contentManager;
	private DataStorageManager dataStorageManager;
	private Project project;
	private Folder parentFolder;
	private ArtifactType artifactType;
	private FragmentSource fragmentSource;
	private OriginalFragmentLocation location;
	private String name;
	private int revision;
	
	public ImportArtifactAction(ContentManager cm, DataStorageManager dsm,
			Project project, ArtifactType artifactType, FragmentSource fragmentSource, 
			OriginalFragmentLocation location, String name, int revision) {
		this.contentManager = cm;
		this.dataStorageManager = dsm;
		this.project = project;
		this.artifactType = artifactType;
		this.fragmentSource = fragmentSource;
		this.location = location;
		this.name = name;
		this.revision = revision;
	}
	
	public ImportArtifactAction(ContentManager cm, DataStorageManager dsm,
			Folder parent, ArtifactType artifactType, FragmentSource fragmentSource, 
			OriginalFragmentLocation location, String name, int revision) {
		this.contentManager = cm;
		this.dataStorageManager = dsm;
		this.parentFolder = parent;
		this.project = cm.getProjectOf(parent);
		this.artifactType = artifactType;
		this.fragmentSource = fragmentSource;
		this.location = location;
		this.name = name;
		this.revision = revision;
	}
	
	
	@Override
	public void execute() {
		MultiArtifactDependencies dependencies = setupDependencies();

		Artifact artifact = contentManager.createArtifact(project, fragmentSource.getId(),
				location.getFullLocation(), artifactType.getTypeId(), revision, dependencies, name);

		if (parentFolder != null) {
			contentManager.setParentFolder(artifact, parentFolder);
		}
		try {
			executeSuccessful = dataStorageManager.createArtifactContent(artifact, revision, 
					location.getFullLocation(), dependencies);
		}
		catch (IOException ioe) {
			ioe.printStackTrace();
			executeSuccessful = false;
		}
		if (executeSuccessful) {
			createdObject = artifact;
		} else {
			dataStorageManager.removeArtifactContent(artifact);
			contentManager.remove(artifact);
		}
	}
	
	
	private MultiArtifactDependencies setupDependencies() {
		Validator validator = artifactType.getValidator();
		MultiArtifactDependencies dependencies = validator.getMultiArtifactDependencies(fragmentSource, location);
		return dependencies;
	}
	

	@Override
	public FragmentUpdateEvent createEvent(boolean direct) {
		Fragment fragment = (Fragment)createdObject;
		FragmentUpdateEvent event = new FragmentUpdateEvent(fragment, UpdateType.CREATED, direct);
		return event;
	}
}
