package org.deft.repository.repositorymanager.actions;

import java.util.List;

import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.ArtifactReferenceUpdateType;
import org.deft.repository.datamodel.Folder;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.event.FragmentUpdateEvent;
import org.deft.repository.event.RepositoryUpdateEvent.UpdateType;
import org.deft.repository.repositorymanager.ContentManager;
import org.deft.repository.repositorymanager.DataStorageManager;

public class RemoveArtifactAction extends RemoveAction {
	
	private DataStorageManager dataStorageManager;
	private Artifact artifact;
	private Folder oldParent;
	
	public RemoveArtifactAction(ContentManager cm, DataStorageManager dsm, Artifact artifact) {
		super(cm, cm.getProjectOf(artifact));
		this.dataStorageManager = dsm;
		this.artifact = artifact;
		this.oldParent = contentManager.getParentFolder(artifact);
	}

	
	@Override
	public void execute() {
		Fragment fragment = contentManager.getFragment(artifact.getId());
		//maybe the artifact has already been deleted by a prior remove action.
		//this can happen if multiple artifacts are to be deleted at once, 
		//and they have interdependencies.
		if (fragment == null) {
			executeSuccessful = true;
		} else {
			boolean success = dataStorageManager.removeArtifactContent(artifact);
			if (success) {
				List<ArtifactReference> references = contentManager.getArtifactReferences(artifact);
				for (ArtifactReference reference : references) {
					contentManager.clearArtifactOfArtifactReference(reference);
				}
				contentManager.remove(artifact);
				setUpdateStatusForAllReferences();
				executeSuccessful = true;
			} else {
				executeSuccessful = false;;
			}
		}
	}
	
	
	@Override
	public FragmentUpdateEvent createEvent(boolean direct) {
		FragmentUpdateEvent event = new FragmentUpdateEvent(artifact, 
				oldParent, getProject(), UpdateType.REMOVED, direct);
		return event;
	}

	
	private void setUpdateStatusForAllReferences() {
		List<ArtifactReference> references = contentManager.getArtifactReferences(artifact);
		for (ArtifactReference reference : references) {
			contentManager.setUpdateType(reference, ArtifactReferenceUpdateType.MISSINGARTIFACT);
		}
	}
	


	@Override
	public boolean equals(Object o) {
		if (o instanceof RemoveArtifactAction) {
			RemoveArtifactAction raa = (RemoveArtifactAction)o;
			return raa.artifact.equals(this.artifact);
		}
		return false;
	}
	
}
