package org.deft.repository.repositorymanager.actions;

import java.util.LinkedList;
import java.util.List;

import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.event.ArtifactReferenceUpdateEvent;
import org.deft.repository.event.RepositoryUpdateEvent.UpdateType;
import org.deft.repository.repositorymanager.ContentManager;
import org.deft.repository.repositorymanager.DataStorageManager;

public class RemoveArtifactReferenceAction extends RemoveAction {
	
	private DataStorageManager dataStorageManager;
	private ArtifactReference reference;
	private Chapter chapter;
	private Artifact artifact;
	
	public RemoveArtifactReferenceAction(ContentManager cm, DataStorageManager dsm, ArtifactReference reference) {
		super(cm, cm.getProjectOf(cm.getChapter(reference)));
		this.dataStorageManager = dsm;
		this.reference = reference;
		this.chapter = contentManager.getChapter(reference);
		this.artifact = contentManager.getArtifact(reference);
	}

	
	@Override
	public void execute() {
		boolean success = dataStorageManager.removeArtifactReferenceContent(reference);
		if (success) {
			contentManager.removeArtifactReference(reference);
			executeSuccessful = true;
		} else {
			executeSuccessful = false;;
		}
	}
	
	@Override
	public ArtifactReferenceUpdateEvent createEvent(boolean direct) {
		ArtifactReferenceUpdateEvent event = new ArtifactReferenceUpdateEvent(reference,
				chapter, artifact, UpdateType.REMOVED, direct);
		return event;
	}
	
	
	@Override
	public List<RemoveAction> getPriorRemoveActions() {
		List<RemoveAction> actions = new LinkedList<RemoveAction>();
		return actions;
	}

	
	@Override
	public boolean equals(Object o) {
		if (o instanceof RemoveArtifactReferenceAction) {
			RemoveArtifactReferenceAction rara = (RemoveArtifactReferenceAction)o;
			return rara.reference.equals(this.reference);
		}
		return false;
	}
	
	
}
