package org.deft.repository.repositorymanager.actions;

import java.util.LinkedList;
import java.util.List;

import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Folder;
import org.deft.repository.datamodel.Tutorial;
import org.deft.repository.event.FragmentUpdateEvent;
import org.deft.repository.event.RepositoryUpdateEvent.UpdateType;
import org.deft.repository.repositorymanager.ContentManager;
import org.deft.repository.repositorymanager.DataStorageManager;

public class RemoveChapterAction extends RemoveAction {
	
	private DataStorageManager dataStorageManager;
	private Chapter chapter;
	private Folder oldParent;
	
	public RemoveChapterAction(ContentManager cm, DataStorageManager dsm, Chapter chapter) {
		super(cm, cm.getProjectOf(chapter));
		this.dataStorageManager = dsm;
		this.chapter = chapter;
		this.oldParent = contentManager.getParentFolder(chapter);
	}

	
	@Override
	public void execute() {
		boolean success = dataStorageManager.removeChapterContent(chapter);
		if (success) {
			contentManager.remove(chapter);
			executeSuccessful = true;
		} else {
			executeSuccessful = false;;
		}
	}
	
	@Override
	public FragmentUpdateEvent createEvent(boolean direct) {
		FragmentUpdateEvent event = new FragmentUpdateEvent(chapter, oldParent,
				getProject(), UpdateType.REMOVED, direct);
		return event;
	}
	
	
	@Override
	public List<RemoveAction> getPriorRemoveActions() {
		List<RemoveAction> actions = new LinkedList<RemoveAction>();
		addActionsToRemoveChapterFromTutorials(actions);
		addActionsToRemoveArtifactReferences(actions);
		List<RemoveAction> copy = getCopyWithoutDuplicateActions(actions); 
		return copy;
	}
	
	private void addActionsToRemoveChapterFromTutorials(List<RemoveAction> actions) {
		List<Tutorial> tutorials = contentManager.getTutorialsForChapter(chapter);
		for (Tutorial tutorial : tutorials) {
			RemoveChapterFromTutorialAction action = new RemoveChapterFromTutorialAction(
					contentManager, tutorial, chapter);
			List<RemoveAction> actionsPrior = action.getPriorRemoveActions();
			actions.addAll(actionsPrior);
			actions.add(action);
		}
	}
	
	private void addActionsToRemoveArtifactReferences(List<RemoveAction> actions) {
		List<ArtifactReference> artifactReferences = contentManager.getArtifactReferences(chapter);
		for (ArtifactReference reference : artifactReferences) {
			RemoveArtifactReferenceAction action = new RemoveArtifactReferenceAction(
					contentManager, dataStorageManager, reference);
			List<RemoveAction> actionsPrior = action.getPriorRemoveActions();
			actions.addAll(actionsPrior);
			actions.add(action);
		}
	}

	
	@Override
	public boolean equals(Object o) {
		if (o instanceof RemoveChapterAction) {
			RemoveChapterAction rca = (RemoveChapterAction)o;
			return rca.chapter == this.chapter;
		}
		return false;
	}
	
	
}
