package org.deft.repository.repositorymanager.actions;

import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Tutorial;
import org.deft.repository.event.RepositoryRelationUpdateEvent;
import org.deft.repository.event.RepositoryUpdateEvent.UpdateType;
import org.deft.repository.repositorymanager.ContentManager;


public class RemoveChapterFromTutorialAction extends RemoveAction {
	
	private Tutorial tutorial;
	private Chapter chapter;
	private int oldPos;

	public RemoveChapterFromTutorialAction(ContentManager cm, Tutorial tutorial, Chapter chapter) {
		super(cm, cm.getProjectOf(tutorial));
		this.tutorial = tutorial;
		this.chapter = chapter;
		this.oldPos = contentManager.getChapterPosition(tutorial, chapter);
	}
	
	
	@Override
	public void execute() {
		contentManager.removeChapterFromTutorial(tutorial, chapter);
		executeSuccessful = true;
	}


	
	@Override
	public RepositoryRelationUpdateEvent createEvent(boolean direct) {
		RepositoryRelationUpdateEvent event = new RepositoryRelationUpdateEvent(tutorial,
				chapter, oldPos, UpdateType.REMOVED, direct);
		return event;
	}

	@Override
	public boolean equals(Object o) {
		if (o instanceof RemoveChapterFromTutorialAction) {
			RemoveChapterFromTutorialAction rcfta = (RemoveChapterFromTutorialAction)o;
			boolean chapterEquals = chapter.equals(rcfta.chapter);
			boolean tutorialEquals = tutorial.equals(rcfta.tutorial);
			boolean oldPosEquals = oldPos == rcfta.oldPos; 
			return  chapterEquals && tutorialEquals && oldPosEquals;
		}
		return false;
	}
}
