package org.deft.repository.repositorymanager.actions;

import java.util.LinkedList;
import java.util.List;

import org.deft.repository.datamodel.Folder;
import org.deft.repository.datamodel.Fragment;
import org.deft.repository.event.FragmentUpdateEvent;
import org.deft.repository.event.RepositoryUpdateEvent.UpdateType;
import org.deft.repository.repositorymanager.ContentManager;
import org.deft.repository.repositorymanager.DataStorageManager;

public class RemoveFolderAction extends RemoveAction {
	
	private DataStorageManager dataStorageManager;
	private Folder folder;
	private Folder oldParent;
	
	public RemoveFolderAction(ContentManager cm, DataStorageManager dsm, Folder folder) {
		super(cm, cm.getProjectOf(folder));
		this.dataStorageManager = dsm;
		this.folder = folder;
		this.oldParent = contentManager.getParentFolder(folder);
	}

	
	@Override
	public void execute() {
		contentManager.remove(folder);
		executeSuccessful = true;
	}
	
	
	@Override
	public FragmentUpdateEvent createEvent(boolean direct) {
		FragmentUpdateEvent event = new FragmentUpdateEvent(folder, 
				oldParent, getProject(), UpdateType.REMOVED, direct);
		return event;
	}

	

	@Override
	public List<RemoveAction> getPriorRemoveActions() {
		List<RemoveAction> actions = new LinkedList<RemoveAction>();
		List<Fragment> fragments = contentManager.getChildren(folder);
		for (Fragment fragment : fragments) {
			RemoveAction action = RemoveAction.createFragmentRemoveAction(fragment, contentManager, dataStorageManager);
			List<RemoveAction> actionsPrior = action.getPriorRemoveActions();
			actions.addAll(actionsPrior);
			actions.add(action);
		}
		List<RemoveAction> copy = getCopyWithoutDuplicateActions(actions); 
		return copy;
	}
	
	
	@Override
	public boolean equals(Object o) {
		if (o instanceof RemoveFolderAction) {
			RemoveFolderAction rfa = (RemoveFolderAction)o;
			return rfa.folder == this.folder;
		}
		return false;
	}
}
