package org.deft.repository.repositorymanager.actions;

import java.util.LinkedList;
import java.util.List;

import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Folder;
import org.deft.repository.datamodel.Tutorial;
import org.deft.repository.event.FragmentUpdateEvent;
import org.deft.repository.event.RepositoryUpdateEvent.UpdateType;
import org.deft.repository.repositorymanager.ContentManager;

public class RemoveTutorialAction extends RemoveAction {
	
	private Tutorial tutorial;
	private Folder oldParent;
	
	public RemoveTutorialAction(ContentManager cm, Tutorial tutorial) {
		super(cm, cm.getProjectOf(tutorial));
		this.tutorial = tutorial;
		this.oldParent = contentManager.getParentFolder(tutorial);
	}

	
	@Override
	public void execute() {
		contentManager.remove(tutorial);
		executeSuccessful = true;
	}

	@Override
	public FragmentUpdateEvent createEvent(boolean direct) {
		FragmentUpdateEvent event = new FragmentUpdateEvent(tutorial, 
				oldParent, getProject(), UpdateType.REMOVED, direct);
		return event;
	}
	
	@Override
	public List<RemoveAction> getPriorRemoveActions() {
		List<RemoveAction> actions = new LinkedList<RemoveAction>();
		List<Chapter> chapters = contentManager.getChaptersForTutorial(tutorial);
		for (Chapter chapter : chapters) {
			RemoveChapterFromTutorialAction action = new RemoveChapterFromTutorialAction(
					contentManager, tutorial, chapter);
			List<RemoveAction> actionsPrior = action.getPriorRemoveActions();
			actions.addAll(actionsPrior);
			actions.add(action);
		}
		List<RemoveAction> copy = getCopyWithoutDuplicateActions(actions); 
		return copy;
	}
	
	
	@Override
	public boolean equals(Object o) {
		if (o instanceof RemoveTutorialAction) {
			RemoveTutorialAction rta = (RemoveTutorialAction)o;
			return rta.tutorial == this.tutorial;
		}
		return false;
	}

}
