package org.deft.repository.repositorymanager.actions;

import org.deft.repository.datamodel.Artifact;
import org.deft.repository.datamodel.ArtifactReference;
import org.deft.repository.datamodel.ArtifactReferenceUpdateType;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.event.ArtifactReferenceUpdateEvent;
import org.deft.repository.event.RepositoryUpdateEvent.UpdateType;
import org.deft.repository.repositorymanager.ContentManager;

public class SetArtifactReferenceCheckedAction extends AbstractRepositoryAction {

	private ContentManager contentManager;
	private ArtifactReference reference;

	
	public SetArtifactReferenceCheckedAction(ContentManager cm, ArtifactReference reference) {
		this.contentManager = cm;
		this.reference = reference;
	}
	

	@Override
	public void execute() {
		Artifact artifact = contentManager.getArtifact(reference);
		int latestRevision = contentManager.getLatestRevision(artifact);
		contentManager.setLatestCheckedRevision(reference, latestRevision);
		contentManager.setUpdateType(reference, ArtifactReferenceUpdateType.NOUPDATE);
		executeSuccessful = true;
	}
	
	
	@Override
	public ArtifactReferenceUpdateEvent createEvent(boolean direct) {
		Chapter chapter = contentManager.getChapter(reference);
		Artifact artifact = contentManager.getArtifact(reference);
		ArtifactReferenceUpdateEvent event = new ArtifactReferenceUpdateEvent(reference, chapter, artifact, UpdateType.UPDATED, direct);
		return event;
	}
}
