package org.deft.repository.repositorymanager.impl;

import java.io.File;
import java.io.IOException;
import java.util.Collections;

import org.deft.repository.datamodel.ecoredatamodel.ProjectsContainer;
import org.deft.repository.repositorymanager.OptionsService;
import org.deft.repository.repositorymanager.PersistenceManager;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

public class PersistenceManagerEcoreDataModelImpl implements PersistenceManager {

	private EcoreDataModelImplCore core;
	private OptionsService options;
	private String XMI_FILE_NAME = "repositorycontent.ecore";
	
	public PersistenceManagerEcoreDataModelImpl(EcoreDataModelImplCore core, OptionsService options) {
		this.core = core;
		this.options = options;
	}

	@Override
	public void loadRepositoryContents() {
		ResourceSet resourceSet = new ResourceSetImpl();
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(
		    Resource.Factory.Registry.DEFAULT_EXTENSION, new XMIResourceFactoryImpl());

		URI fileURI = getFileURI();
		Resource resource = null;
		try {
			resource = resourceSet.getResource(fileURI, true);
		}
		catch (Exception e) {
			//resource could not be found, there is probably no content file
		}
		if (resource != null) {
			ProjectsContainer projects = (ProjectsContainer)resource.getContents().get(0);
			core.setRootContainer(projects);
		}
	}
	
	@Override
	public void saveRepositoryContents() {
		ResourceSet resourceSet = new ResourceSetImpl();
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(
					Resource.Factory.Registry.DEFAULT_EXTENSION, new XMIResourceFactoryImpl());
		URI fileURI = getFileURI();
		Resource resource = resourceSet.createResource(fileURI);
		resource.getContents().add(core.getRootContainer());
		try {
		    resource.save(Collections.EMPTY_MAP);
		}
		catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	private URI getFileURI() {
		IPath directory = getRepositoryDirectoryAndCreateIfItNotExists();
		IPath saveFile = directory.append(XMI_FILE_NAME);
		return URI.createFileURI(saveFile.toOSString());
	}
	
	private IPath getRepositoryDirectoryAndCreateIfItNotExists() {
		String accessData = options.getRepositoryLocation();
		File directory = new File(accessData);
		if (!directory.isDirectory()) {
			directory.mkdirs();
		}
		IPath path = new Path(directory.getAbsolutePath());
		return path;
	}
	
}
