package org.deft.repository.service.impl;

import java.util.LinkedList;
import java.util.List;

import org.deft.operation.OperationChain;
import org.deft.operation.OperationChainRegistry;
import org.deft.repository.artifacttype.ArtifactType;
import org.deft.repository.artifacttype.ArtifactTypeRegistry;
import org.deft.repository.artifacttype.FileExtensionToArtifactTypeMappingRegistry;
import org.deft.repository.chaptertype.ChapterType2IntegratorIdMappingRegistry;
import org.deft.repository.fragmentsource.FragmentSource;
import org.deft.repository.fragmentsource.FragmentSourceRegistry;
import org.deft.repository.integrator.FileIntegrator;
import org.deft.repository.integrator.FileIntegratorRegistry;
import org.deft.repository.integrator.Integrator;
import org.deft.repository.projecttype.ProjectType;
import org.deft.repository.projecttype.ProjectTypeRegistry;
import org.deft.repository.service.ExtensionService;

public class ExtensionServiceImpl implements ExtensionService {

	private FragmentSourceRegistry fragmentSourceReg;
	private ArtifactTypeRegistry artifactTypeReg;
	private FileExtensionToArtifactTypeMappingRegistry fileExtensionMappingReg;
	private ProjectTypeRegistry projectTypeReg;
	private FileIntegratorRegistry fileIntegratorReg;
	private ChapterType2IntegratorIdMappingRegistry chapterType2IntegratorMappingReg;
	private OperationChainRegistry operationChainReg;
	
	
	public ExtensionServiceImpl(FragmentSourceRegistry fragmentSourceRegistry,
			ArtifactTypeRegistry artifactTypeRegistry, 
			FileExtensionToArtifactTypeMappingRegistry fileExtensionMappingRegistry,
			ProjectTypeRegistry projectTypeReg, FileIntegratorRegistry integratorReg,
			ChapterType2IntegratorIdMappingRegistry chapterType2IntegratorMappingReg,
			OperationChainRegistry operationChainReg) {
		this.fragmentSourceReg = fragmentSourceRegistry;
		this.artifactTypeReg = artifactTypeRegistry;
		this.fileExtensionMappingReg = fileExtensionMappingRegistry;
		this.projectTypeReg = projectTypeReg;
		this.fileIntegratorReg = integratorReg;
		this.chapterType2IntegratorMappingReg = chapterType2IntegratorMappingReg;
		this.operationChainReg = operationChainReg;
	}
	
	@Override
	public FragmentSource getFragmentSource(String fragmentSourceId) {
		FragmentSource source = fragmentSourceReg.getFragmentSource(fragmentSourceId);
		return source;
	}

	@Override
	public List<ArtifactType> getArtifactTypes() {
		return artifactTypeReg.getArtifactTypes();
	}
	
	@Override
	public List<String> getArtifactTypeIds() {
		return artifactTypeReg.getArtifactTypeIds();
	}
	
	@Override
	public ArtifactType getArtifactType(String artifactTypeId) {
		ArtifactType type = artifactTypeReg.getArtifactType(artifactTypeId);
		return type;
	}

	
	@Override
	public List<String> getFileExtensionsForArtifactTypeId(String artifactTypeId) {
		List<String> fileExtensions = fileExtensionMappingReg.getFileExtensions(artifactTypeId);
		return fileExtensions;
	}
	
	@Override
	public List<String> getArtifactTypeIdsForFileExtension(String fileExtension) {
		List<String> artifactTypeIds = fileExtensionMappingReg.getArtifactTypeIds(fileExtension);
		return artifactTypeIds;
	}
	

	@Override
	public ProjectType getProjectType(String projectTypeId) {
		ProjectType projectType = projectTypeReg.getProjectType(projectTypeId);
		return projectType;
	}

	@Override
	public List<ProjectType> getProjectTypes() {
		return projectTypeReg.getProjectTypes();
	}

	@Override
	public List<String> getProjectTypeIds() {
		return projectTypeReg.getProjectTypeIds();
	}
	
	@Override
	public List<String> getChapterTypeIdsForFileIntegrator(String integratorId) {
		return chapterType2IntegratorMappingReg.getChapterTypeIds(integratorId);
	}

	@Override
	public FileIntegrator getFileIntegrator(String id) {
		return fileIntegratorReg.getIntegrator(id);
	}

	@Override
	public List<String> getFileIntegratorIds() {
		return fileIntegratorReg.getIntegratorIds();
	}

	@Override
	public List<FileIntegrator> getFileIntegrators() {
		return fileIntegratorReg.getIntegrators();
	}

	@Override
	public OperationChain getOperationChain(String id) {
		return operationChainReg.getOperationChain(id);
	}

	@Override
	public List<String> getOperationChainIds() {
		return operationChainReg.getOperationChainIds();
	}

	@Override
	public List<OperationChain> getOperationChains() {
		return operationChainReg.getOperationChains();
	}

	@Override
	public List<OperationChain> getCompatibleOperationChainsForFileIntegration(
			String artifactTypeId, String chapterTypeId) {
		List<OperationChain> chains = new LinkedList<OperationChain>();
		for (OperationChain chain : operationChainReg.getOperationChains()) {
			if (areCompatible(artifactTypeId, chain, chapterTypeId)) {
				chains.add(chain);
			}
		}
		return chains;
	}
	
	
	private boolean areCompatible(String artifactTypeId, OperationChain chain,
			String chapterTypeId) {
		ArtifactType artifactType = getArtifactType(artifactTypeId);
		boolean compatible = chain.isValidInputArtifactType(artifactType)
				&& doCompatibleFileIntegratorsExist(chain, chapterTypeId);
		return compatible;
	}
	
	
	
	private boolean doCompatibleFileIntegratorsExist(OperationChain chain, String chapterTypeId) {
		List<FileIntegrator> integratorsForChapterType = getFileIntegratorsForChapterType(chapterTypeId);
		for (FileIntegrator integrator : integratorsForChapterType) {
			String integratorInputType = integrator.getInputRepresentationType();
			String chainOutputType = chain.getFinalRepresentationType();
			if (integratorInputType.equals(chainOutputType)) {
				return true;
			}
		}
		return false;	
	}
	
	@Override
	public List<FileIntegrator> getFileIntegratorsForChapterType(String chapterTypeId) {
		List<FileIntegrator> integrators = new LinkedList<FileIntegrator>();
		List<String> ids = chapterType2IntegratorMappingReg.getIntegratorIds(chapterTypeId);
		for (String integratorId : ids) {
			Integrator integrator = getFileIntegrator(integratorId);
			if (integrator != null) {
				integrators.add((FileIntegrator)integrator);
			}
		}
		return integrators;
	}



	@Override
	public List<String> getOperationChainIdsForArtifactType(String artifactTypeId) {
		List<String> chainIds = new LinkedList<String>();
		ArtifactType artifactType = getArtifactType(artifactTypeId);
		for (OperationChain chain : getOperationChains()) {
			if (chain.isValidInputArtifactType(artifactType)) {
				chainIds.add(chain.getId());
			}
		}
		return chainIds;
	}
	
	
	@Override
	public List<String> getFileIntegratorIdsForOperationChain(String operationChainId) {
		List<String> integratorIds = new LinkedList<String>();
		OperationChain chain = getOperationChain(operationChainId);
		String chainRepTypeId = chain.getFinalRepresentationType();
		for (FileIntegrator integrator : getFileIntegrators()) {
			String integratorRepTypeId = integrator.getInputRepresentationType();
			if (integratorRepTypeId.equals(chainRepTypeId)) {
				integratorIds.add(integrator.getId());
			}
		}
		return integratorIds;
	}
	
}
