package org.deft.representation.identifiers.ecore;

import java.util.List;

import org.deft.representation.identifiers.data.Scope;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EEnum;
import org.eclipse.emf.ecore.EEnumLiteral;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EOperation;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EParameter;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.xmi.XMIResource;

public class EcoreModelToIdentTransformer {
	
	public EcorePackageIdentifier transform(EPackage modelPackage) {
		EcorePackageIdentifier rootIdent = createPackageIdentifier(modelPackage, null);
		return rootIdent;
	}
	
	private void fill(Scope scope, EObject eObject) {
		List<EObject> contents = eObject.eContents();
		for (EObject eo : contents) {
			if (eo instanceof EPackage) {
				try {
					EcorePackageIdentifier epi = createPackageIdentifier((EPackage)eo, scope);
					scope.addChildScope(epi);
				} 
				catch (Exception e) {e.printStackTrace();}
			} else if (eo instanceof EClass) {
				try {
					EcoreClassIdentifier eci = createClassIdentifier((EClass)eo, scope);
					scope.addChildScope(eci);
				} 
				catch (Exception e) {e.printStackTrace();}
			} else if (eo instanceof EEnum) {
				try {
					EcoreEnumIdentifier eei = createEnumIdentifier((EEnum)eo, scope);
					scope.addChildScope(eei);
				} 
				catch (Exception e) {e.printStackTrace();}
			} else if (eo instanceof EEnumLiteral) {
				try {
					EcoreEnumLiteralIdentifier eeli = createEnumLiteralIdentifier((EEnumLiteral)eo, scope);
					scope.addChildScope(eeli);
				} 
				catch (Exception e) {e.printStackTrace();}
			} else if (eo instanceof EAttribute) {
				try {
					EcoreAttributeIdentifier eai = createAttributeIdentifier((EAttribute)eo, scope);
					scope.addChildScope(eai);
				} 
				catch (Exception e) {e.printStackTrace();}
			} else if (eo instanceof EReference) {
				try {
					EcoreReferenceIdentifier eri = createReferenceIdentifier((EReference)eo, scope);
					scope.addChildScope(eri);
				} 
				catch (Exception e) {e.printStackTrace();}
			} else if (eo instanceof EAnnotation) {
				try {
					EcoreAnnotationIdentifier eai = createAnnotationIdentifier((EAnnotation)eo, scope);
					scope.addChildScope(eai);
				} 
				catch (Exception e) {e.printStackTrace();}
			} else if (eo instanceof EOperation) {
				try {
					EOperation eOp = (EOperation)eo;
					EcoreOperationIdentifier eoi = createOperationIdentifier(eOp, scope);
					scope.addChildScope(eoi);
				} 
				catch (Exception e) {e.printStackTrace();}
			} 
		}
	}
	
	private String getId(EObject eo) {
		Resource r = eo.eResource();
		if (r instanceof XMIResource) {
			XMIResource xmir = (XMIResource)r;
			String id = xmir.getID(eo);
			return id;
		}
		return null;
	}
	
	private EcorePackageIdentifier createPackageIdentifier(EPackage ep, Scope parent) {
		String name = ep.getName();
		String id = getId(ep);
		EcorePackageIdentifier ident = new EcorePackageIdentifier(name, id);
		fill(ident, ep);
		return ident;
	}
	
	private EcoreClassIdentifier createClassIdentifier(EClass ec, Scope parent) {
		String name = ec.getName();
		String id = getId(ec);
		EcoreClassIdentifier ident = new EcoreClassIdentifier(name, id);
		fill(ident, ec);
		return ident;
	}
	
	private EcoreEnumIdentifier createEnumIdentifier(EEnum ee, Scope parent) {
		String name = ee.getName();
		String id = getId(ee);
		EcoreEnumIdentifier ident = new EcoreEnumIdentifier(name, id);
		fill(ident, ee);
		return ident;
	}
	
	private EcoreEnumLiteralIdentifier createEnumLiteralIdentifier(EEnumLiteral eel, Scope parent) {
		String name = eel.getName();
		String id = getId(eel);
		EcoreEnumLiteralIdentifier ident = new EcoreEnumLiteralIdentifier(name, id);
		fill(ident, eel);
		return ident;
	}	
	
	private EcoreAttributeIdentifier createAttributeIdentifier(EAttribute ea, Scope parent) {
		String name = ea.getName();
		String id = getId(ea);
		EClassifier eType = ea.getEType();
		String type = null;
		if (eType != null) {
			type = eType.getName();
		}
		int lowerBound = ea.getLowerBound();
		int upperBound = ea.getUpperBound();
		EcoreAttributeIdentifier ident = new EcoreAttributeIdentifier(name, id, type, lowerBound, upperBound);
		fill(ident, ea);
		return ident;
	}
	
	private EcoreReferenceIdentifier createReferenceIdentifier(EReference er, Scope parent) {
		String name = er.getName();
		String id = getId(er);
		EClassifier eType = er.getEType();
		String type = eType.getName();
		int lowerBound = er.getLowerBound();
		int upperBound = er.getUpperBound();
		EcoreReferenceIdentifier ident = new EcoreReferenceIdentifier(name, id, type, lowerBound, upperBound);
		return ident;
	}
	
	private EcoreOperationIdentifier createOperationIdentifier(EOperation eo, Scope parent) {
		String name = eo.getName();
		String id = getId(eo);
		EClassifier eType = eo.getEType();
		String type = "";
		if (eType != null) {
			type = eType.getName();
		}
		int lowerBound = eo.getLowerBound();
		int upperBound = eo.getUpperBound();
		EcoreOperationIdentifier ident = new EcoreOperationIdentifier(name, id, type, lowerBound, upperBound);
		List<EParameter> params = eo.getEParameters();
		for (EParameter eParam : params) {
			EcoreParameterIdentifier epi = createParameterIdentifier(eParam);
			ident.addParameter(epi);
		}
		fill(ident, eo);
		return ident;
	}
	
	
	private EcoreParameterIdentifier createParameterIdentifier(EParameter ep) {
		String name = ep.getName();
		String id = getId(ep);
		EClassifier eType = ep.getEType();
		String type = eType.getName();
		EcoreParameterIdentifier ident = new EcoreParameterIdentifier(name, id, type);
		fill(ident, ep);
		return ident;
	}
	
	
	private EcoreAnnotationIdentifier createAnnotationIdentifier(EAnnotation ea, Scope parent) {
		String name = ea.getSource();
		String id = getId(ea);
		EcoreAnnotationIdentifier ident = new EcoreAnnotationIdentifier(name, id);
		return ident;
	}

}
