package org.deft.representation.identifiers.ecore;

import java.util.LinkedList;
import java.util.List;

public class EcoreOperationIdentifier extends AbstractEcoreIdentifier {

	private String type;
	private int lowerBound, upperBound;
	private List<EcoreParameterIdentifier> parameters = new LinkedList<EcoreParameterIdentifier>();
	
	
	public EcoreOperationIdentifier(String name, String id, String type, int lowerBound, int upperBound) {
		super(name, id);
		this.type = type;
		this.lowerBound = lowerBound;
		this.upperBound = upperBound;
	}
	
	public void addParameter(EcoreParameterIdentifier parameter) {
		parameters.add(parameter);
		addChildScope(parameter);
	}
	
	public String getType() {
		return type;
	}
	
	public int getLowerBound() {
		return lowerBound;
	}
	
	public int getUpperBound() {
		return upperBound;
	}
	
	public List<EcoreParameterIdentifier> getParameters() {
		return new LinkedList<EcoreParameterIdentifier>(parameters);
	}
	
	@Override
	public String getIconPath() {
		return "icons/eoperation.gif";
	}
	
	@Override
	public String getFullLocalName() {
		StringBuilder sb = new StringBuilder();
		sb.append(getName());
		sb.append("(");
		List<EcoreParameterIdentifier> params = getParameters();
		for (int i = 0; i < params.size(); i++) {
			EcoreParameterIdentifier param = params.get(i); 
			sb.append(param.getType());
			if (i < params.size() - 1) {
				sb.append(",");
			}
		}
		sb.append(")");
		return sb.toString();
	}
	


}
	
