package org.deft.representation.identifiers.java;

import java.util.List;

import org.deft.repository.ast.TreeNode;
import org.deft.repository.ast.TreeNodeRoot;
import org.deft.repository.ast.annotation.TemplatesBasic;
import org.deft.repository.ast.annotation.astname.AstNameInformation;
import org.deft.representation.identifiers.data.Scope;

public class JavaToIdentTransformer {
	
	public JavaCompilationUnitScope transform(TreeNodeRoot root) {
//		XmlTreeNodeSerializer serializer = new XmlTreeNodeSerializer();
//		Document d = serializer.convertToXml(root);
//		XmlUtil.outputPrettyXml(d, System.out);
		TreeNode cu = root.getFirstChild();
		JavaCompilationUnitScope cus = createCompilationUnitScope(cu, null);
		return cus;
	}
	
	private void fill(Scope scope, TreeNode node) {
		List<TreeNode> children = node.getChildren();
		for (TreeNode child : children) {
//			System.out.println(child.getName());
			if (child.getName().equals("PackageDeclaration")) {
				try {
					JavaPackageIdentifier ident = createPackageIdentifier(child, scope);
					scope.addChildScope(ident);
				} 
				catch (Exception e) {e.printStackTrace();}
			} else if (child.getName().equals("ImportDeclarations")) {
				List<TreeNode> importDecls = child.getChildren();
				for (TreeNode importDecl : importDecls) {
					try {
						JavaImportIdentifier ident = createImportIdentifier(importDecl, scope);
						scope.addChildScope(ident);
					} 
					catch (Exception e) {e.printStackTrace();}
				}
			} else if (child.getName().equals("ClassDeclaration")) {
				try {
					JavaClassIdentifier ident = createClassIdentifier(child, scope);
					scope.addChildScope(ident);
				} 
				catch (Exception e) {e.printStackTrace();}
			} else if (child.getName().equals("FieldDeclaration")) {
				try {
					JavaFieldIdentifier ident = createFieldIdentifier(child, scope);
					scope.addChildScope(ident);
				} 
				catch (Exception e) {e.printStackTrace();}
			} else if (child.getName().equals("ConstructorDeclaration")) {
				try {
					JavaConstructorIdentifier ident = createConstructorIdentifier(child, scope);
					scope.addChildScope(ident);
				} 
				catch (Exception e) {e.printStackTrace();}
			} else if (child.getName().equals("MethodDeclaration")) {
				try {
					JavaMethodIdentifier ident = createMethodIdentifier(child, scope);
					scope.addChildScope(ident);
				} 
				catch (Exception e) {e.printStackTrace();}
			} else if (child.getName().equals("SingleVariableDeclaration")) {
				try {
					JavaVariableIdentifier jvi = createSingleVariableIdentifier(child, scope);
					scope.addChildScope(jvi);
				} 
				catch (Exception e) {e.printStackTrace();}
			} 
			
			else {
				//recursion if the current tree node has no identifiers
				fill(scope, child);
			}
		}
	}
	
//	private String getId(EObject eo) {
//		Resource r = eo.eResource();
//		if (r instanceof XMIResource) {
//			XMIResource xmir = (XMIResource)r;
//			String id = xmir.getID(eo);
//			return id;
//		}
//		return null;
//	}
	
	private JavaCompilationUnitScope createCompilationUnitScope(TreeNode node, Scope parent) {
		String name = node.getName();
		JavaCompilationUnitScope scope = new JavaCompilationUnitScope(name);
		fill(scope, node);
		return scope;
	}
	
	private JavaPackageIdentifier createPackageIdentifier(TreeNode node, Scope parent) {
		TreeNode qualifiedName = node.getChild(1);
		String name = buildIdentifierName(qualifiedName);
		JavaPackageIdentifier ident = new JavaPackageIdentifier(name);
		fill(ident, node);
		return ident;
	}
	
	private JavaImportIdentifier createImportIdentifier(TreeNode node, Scope parent) {
		TreeNode qualifiedName = node.getChild(1);
		String name = buildIdentifierName(qualifiedName);
		if (hasPath(node, "DOTSTAR")) {
			name += ".*";
		}
		JavaImportIdentifier ident = new JavaImportIdentifier(name);
		fill(ident, node);
		return ident;
	}
	
	private JavaClassIdentifier createClassIdentifier(TreeNode node, Scope parent) {
		String name = getAstName(node);
		JavaClassIdentifier ident = new JavaClassIdentifier(name);
		fill(ident, node);
		return ident;
	}
	
	private JavaFieldIdentifier createFieldIdentifier(TreeNode node, Scope parent) {
		String name = getAstName(node);
		JavaFieldIdentifier ident = new JavaFieldIdentifier(name);
		fill(ident, node);
		return ident;
	}
	
	private JavaConstructorIdentifier createConstructorIdentifier(TreeNode node, Scope parent) {
		String name = getAstName(node);
		JavaConstructorIdentifier ident = new JavaConstructorIdentifier(name);
		fill(ident, node);
		return ident;
	}
	
	private JavaMethodIdentifier createMethodIdentifier(TreeNode node, Scope parent) {
		String name = getAstName(node);
		JavaMethodIdentifier ident = new JavaMethodIdentifier(name);
		fill(ident, node);
		return ident;
	}
	
	private JavaVariableIdentifier createSingleVariableIdentifier(TreeNode node, Scope parent) {
////		TreeNode nameNode = node.
//		String name = getAstName(node);
//		JavaMethodIdentifier ident = new JavaMethodIdentifier(name);
//		fill(ident, node);
//		return ident;
		return null;
	}
	
	
	
	private String getAstName(TreeNode node) {
		if (node.hasInformation(TemplatesBasic.ASTNAME)) {
			AstNameInformation ani = (AstNameInformation)node.getInformation(TemplatesBasic.ASTNAME);
			String name = ani.getAstName();
			return name;
		} else {
			return null;
		}
	}
	
	
	private String buildIdentifierName(TreeNode node) {
		StringBuilder sb = new StringBuilder();
		if (node.getName().equals("QualifiedName")) {
			List<TreeNode> children = node.getChildren();
			for (TreeNode child : children) {
				String namePart = buildIdentifierName(child);
				sb.append(namePart);
			}
		} else {
			String namePart = node.getTextContent();
			sb.append(namePart);
		}
		String name = sb.toString();
		return name;
	}
	
	private boolean hasPath(TreeNode node, String xPathString) {
		List<TreeNode> result = node.executeXPathQuery(xPathString);
		return !result.isEmpty();
	}
}
