package org.deft.representation.image;

import java.awt.image.BufferedImage;
import java.io.File;

import org.deft.repository.datamodel.ArtifactRepresentation;
import org.eclipse.swt.graphics.Image;


public class BufferedImageRepresentation implements ArtifactRepresentation {

	private BufferedImage image;
	private File file;
	
	public static final String TYPE = "org.deft.representation.bufferedimage";
	private static final String STANDARD_EXTENSION = "image"; //$NON-NLS-1$
	private static final String IMAGE_PREFIX = "tutorial-image_"; //$NON-NLS-1$
	private static final String DOT = "."; //$NON-NLS-1$

	
	public BufferedImageRepresentation(BufferedImage image) {
		this.image = image;
	}
	
	
	public BufferedImageRepresentation(File file) {
		this.file = file;
	}
	
	public BufferedImageRepresentation(Image swtImage) {
		this.image = convertToBufferedImage(swtImage);
	}
	
	private BufferedImage convertToBufferedImage(Image swtImage) {
		BufferedImage bi = ImageConverter.convertToAWT(swtImage.getImageData());
		return bi;
	}
	
	public File getImageFile() {
		return file;
	}
	
	public static File allocateTempImage(String extension) {
		if (extension == null || extension.length() == 0)
			extension = STANDARD_EXTENSION;

		extension = extension.toLowerCase();

		String tmpDir = System.getProperty("java.io.tmpdir"); //$NON-NLS-1$
		if (!tmpDir.endsWith(File.separator))
			tmpDir += File.separator;
		String fileName = IMAGE_PREFIX;
		fileName += Long.toString(System.currentTimeMillis());
		if (!extension.startsWith(DOT))
			fileName += DOT;
		fileName += extension;

		File tempImage = new File(tmpDir + fileName);
		tempImage.deleteOnExit(); //temp image, delete after DEFT closes
		return tempImage;
	}
	
	
	
	public BufferedImage getImage() {
		return image;
	}
	
	@Override
	public String getType() {
		return TYPE;
	}
	
	public boolean equals(Object o) {
		if (o instanceof BufferedImageRepresentation) {
			BufferedImageRepresentation bir2 = (BufferedImageRepresentation)o;
			BufferedImage bi1 = getImage();
			BufferedImage bi2 = bir2.getImage();
			boolean equals = compare(bi1, bi2);
			return equals;
		} else {
			return false;
		}
	}
	
	private boolean compare(BufferedImage bi1, BufferedImage bi2){
		int w = bi1.getWidth();
		int h = bi1.getHeight();
		if (w == bi2.getWidth() && h == bi2.getHeight()) {
			for(int i = 0; i < w; ++i) {
				for (int j = 0; j < h; ++j) {
					int c1 = bi1.getRGB(i, j);
					int c2 = bi2.getRGB(i, j);
					if (c1 != c2){
						return false;
					}
				}
			}
		} else {
			return false;
		}
		return true;
	}

}

