package org.deft.representation.styleddocument;

import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleContext;

import org.deft.representation.styleddocument.data.SimpleListing;
import org.deft.representation.styleddocument.data.DocumentContentContainer;
import org.deft.representation.styleddocument.data.TokenizedStyledDocument;

public class StyledSimpleListingRepresentation extends DocumentContentContainerRepresentation {
	
	public static final String TYPE = "org.deft.representation.styleddocumentsimplelisting";

	private TokenizedStyledDocument emptyListingMessage;
	
	
	public StyledSimpleListingRepresentation(SimpleListing listing) {
		this("", listing);
	}
	
	
	public StyledSimpleListingRepresentation(String emptyListingMessage, SimpleListing listing) {
		super(createStyledDoc(listing));
		this.emptyListingMessage = createMessage(emptyListingMessage);
	}
	
	private static DocumentContentContainer createStyledDoc(SimpleListing listing) {
		DocumentContentContainer doc = new DocumentContentContainer();
		if (listing == null) {
			doc.addDocumentContent(new SimpleListing());
		} else {
			doc.addDocumentContent(listing);
		}
		return doc;
	}
	
	private static TokenizedStyledDocument createMessage(String string) {
		TokenizedStyledDocument doc = new TokenizedStyledDocument();
		Style style = doc.getStyle(StyleContext.DEFAULT_STYLE);
		try {
			doc.insertString(doc.getLength(), string, style);
			return doc;
		} catch (BadLocationException e) {
			e.printStackTrace();
			return null;
		}
	}
	
	public StyledSimpleListingRepresentation(TokenizedStyledDocument emptyListingMessage) {
		super(createStyledDoc(null));
		this.emptyListingMessage = emptyListingMessage;
	}
	
	public TokenizedStyledDocument getEmptyListingMessage() {
		return emptyListingMessage;
	}
	
	public SimpleListing getListing() {
		return (SimpleListing)super.getDocumentContentContainer().getContents().get(0);
	}
	
	
	@Override
	public String getType() {
		return TYPE;
	}


}
