package org.deft.representation.styleddocument.conversion;

import java.awt.Color;

import org.deft.representation.styleddocument.data.StyledToken;

import com.lowagie.text.html.HtmlEncoder;

public class HtmlTextMapping implements PlainText2StyledTextMapping {


	protected String htmlText = "";
	private String currentString = "";
	private String styleString = "";
	private boolean useNamedStyles;
	
	
	public HtmlTextMapping(boolean useNamedStyles) {
		this.useNamedStyles = useNamedStyles;
	}
	
	
	@Override
	public void init() {
		htmlText = "";
		currentString = "";
		styleString = "";
	}
	
	@Override
	public void finish() {
	}


	@Override
	public String getStyledText() {
		return htmlText;
	}
	
	
	@Override
	public void nextToken(StyledToken token) {
		if (useNamedStyles) {
			nextTokenWithNamedStyles(token);
		} else {
			nextTokenWithExplicitStyles(token);
		}
	}
	
	private void nextTokenWithNamedStyles(StyledToken token) {
		boolean isWs = token.isWhitespace();
		nextString(token.getText(), !isWs);
		if (!isNormalStyle(token)) {
			String styleName = token.getStyle().getName();
			setNamedStyle(styleName);
		}
	}
	
	private void setNamedStyle(String name) {
		currentString = "<span class=\"" + name + "\">" + currentString + "</span>";
	}

	

	
	
	
	private void nextTokenWithExplicitStyles(StyledToken token) {
		boolean isWs = token.isWhitespace();
		nextString(token.getText(), !isWs);
		if (!isNormalStyle(token)) {
			if (token.isBold()) setBold();
			if (token.isItalic()) setItalic();
			if (token.isUnderlined()) setUnderline();
			if (token.isStrikeThrough()) setStrikeThrough();
			Color c = token.getColor();
			setForeground(c.getRed(), c.getGreen(), c.getBlue());
			setExplicitStyle();
		}
	}
	
	private void setExplicitStyle() {
		currentString = "<span style=\"" + styleString  + "\">" + currentString + "</span>";
		styleString = "";
	}
	
	
	private boolean isNormalStyle(StyledToken token) {
		String styleName = token.getStyle().getName();
		return styleName.equals("normal");
	}


	@Override
	public void nextString(String nextString) {
		nextString(nextString, true);
	}
	
	private void nextString(String nextString, boolean doHtmlEncode) {
		if (currentString != null) {
			htmlText += currentString;
		}
		
		String stringToAppend = nextString;
		if (doHtmlEncode) {
			stringToAppend = HtmlEncoder.encode(nextString);
		}

		currentString = stringToAppend;

	}
	

	@Override
	public void setBold() {
		styleString += "font-weight:bold;";
	}
	
	@Override
	public void setItalic() {
		styleString += "font-style:italic;";
	}

	@Override
	public void setForeground(int r, int g, int b) {
		styleString += "color:rgb(" + r +"," + g + "," + b + ");";
	}


	@Override
	public void setStrikeThrough() {
		// empty
	}

	@Override
	public void setUnderline() {
		// empty
	}


	@Override
	public void setMonospace() {
		styleString += "font-family: 'Courier New';";
	}
	
}