package org.deft.representation.styleddocument.conversion;

import org.deft.representation.styleddocument.data.StyledToken;


/**
 * A mapping provides the necessary actions to convert plain text and (external or connected) 
 * formatting information into a styled text, i.e. text with formatting. 
 */ 
public interface PlainText2StyledTextMapping {

	/**
	 * Will be called before the construction process. Intended to do some initializations here.
	 */
	public void init();
	
	/**
	 * Will be called after the construction process. Intended to do some clean up here.
	 */
	public void finish();
	
	/**
	 * Can be called after the construction process. This method returns the String of the styled text.
	 * @return the styled text
	 */
	public String getStyledText();
	
	
	public void nextToken(StyledToken token);

	
	/**
	 * Sets the next string which will be styled and added to the final styled text.
	 * 
	 * @param nextString the next string to style
	 */
	public void nextString(String nextString);
	
	/**
	 * Sets the RGB value of the foreground color for the actual String.
	 * 
	 * @param r
	 * @param g
	 * @param b
	 */
	public void setForeground(int r, int g, int b);
	
	/**
	 * Sets the actual String to bold.
	 */
	public void setBold();
	
	/**
	 * Sets the actual String to italic.
	 */
	public void setItalic();
		
	/**
	 * Sets the actual String underlined.
	 */
	public void setUnderline();
	
	/**
	 * Sets the actual String stroke through.
	 */
	public void setStrikeThrough();
	
	public void setMonospace();
	
}
