package org.deft.representation.styleddocument.data;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleContext;

public class DefinitionListing implements Listing {
	
	//contains the definition terms and their order
	private List<TokenizedStyledDocument> terms = new LinkedList<TokenizedStyledDocument>();
	
	//contains the mapping from terms to explanations
	private Map<TokenizedStyledDocument, DocumentContent> mapping 
			= new HashMap<TokenizedStyledDocument, DocumentContent>();
	
	
	private TokenizedStyledDocument emptyListingMessage;
	
	
	public DefinitionListing() {
		this("");
	}
	
	
	public DefinitionListing(String emptyListingMessage) {
		this(createMessage(emptyListingMessage));
	}
	
	private static TokenizedStyledDocument createMessage(String string) {
		TokenizedStyledDocument doc = new TokenizedStyledDocument();
		Style style = doc.getStyle(StyleContext.DEFAULT_STYLE);
		try {
			doc.insertString(doc.getLength(), string, style);
			return doc;
		} catch (BadLocationException e) {
			e.printStackTrace();
			return null;
		}
	}
	
	public DefinitionListing(TokenizedStyledDocument emptyListingMessage) {
		this.emptyListingMessage = emptyListingMessage;
	}
	

	public void addDefinition(TokenizedStyledDocument term, DocumentContent explanation) {
		terms.add(term);
		mapping.put(term, explanation);
	}
	
	
	public List<TokenizedStyledDocument> getTerms() {
		return new LinkedList<TokenizedStyledDocument>(terms);
	}
	
	public Map<TokenizedStyledDocument, DocumentContent> getMapping() {
		return new HashMap<TokenizedStyledDocument, DocumentContent>(mapping);
	}
	
	public DocumentContent getExplanation(TokenizedStyledDocument term) {
		DocumentContent explanation = mapping.get(term);
		return explanation;
	}
	
	public boolean isEmpty() {
		return terms.isEmpty();
	}
	
	public TokenizedStyledDocument getEmptyListingMessage() {
		return emptyListingMessage;
	}
	
	@Override
	public boolean equals(Object o) {
		if (o instanceof DefinitionListing) {
			DefinitionListing dl = (DefinitionListing)o;
			boolean equals = terms.equals(dl.terms)
					&& mapping.equals(dl.mapping)
					&& emptyListingMessage.equals(dl.emptyListingMessage);
			return equals;
		} else {
			return false;
		}
	}

	
}
