package org.deft.representation.styleddocument.data;

import java.util.LinkedList;
import java.util.List;

import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleContext;

public class SimpleListing implements Listing {

	private List<DocumentContent> listItems = new LinkedList<DocumentContent>();
	private TokenizedStyledDocument emptyListingMessage;

	
	public SimpleListing() {
		this("");
	}
	
	public SimpleListing(String emptyListingMessage) {
		this(createMessage(emptyListingMessage));
	}
	
	private static TokenizedStyledDocument createMessage(String string) {
		TokenizedStyledDocument doc = new TokenizedStyledDocument();
		Style style = doc.getStyle(StyleContext.DEFAULT_STYLE);
		try {
			doc.insertString(doc.getLength(), string, style);
			return doc;
		} catch (BadLocationException e) {
			e.printStackTrace();
			return null;
		}
	}

	public SimpleListing(TokenizedStyledDocument emptyListingMessage) {
		this.emptyListingMessage = emptyListingMessage;
	}

	
	public boolean isEmpty() {
		return listItems.isEmpty();
	}
	
	public TokenizedStyledDocument getEmptyListingMessage() {
		return emptyListingMessage;
	}

	
	public SimpleListing(List<DocumentContent> listItems) {
		this.listItems = listItems;
	}
	
	public List<DocumentContent> getListItems() {
		return new LinkedList<DocumentContent>(listItems);
	}
	
	public void addListItem(DocumentContent content) {
		this.listItems.add(content);
	}
	
	@Override
	public boolean equals(Object o) {
		if (o instanceof SimpleListing) {
			SimpleListing sl = (SimpleListing)o;
			boolean equals = listItems.equals(sl.listItems)
					&& emptyListingMessage.equals(sl.emptyListingMessage);
			return equals;
		} else {
			return false;
		}
	}
	
}
