package org.deft.representation.styleddocument.data;

import java.awt.Color;

import javax.swing.text.Style;
import javax.swing.text.StyleConstants;

/**
 * A StyledToken is a set of adjacent characters of a StyledDocument (or rather its
 * subclass TokenizedStyledDocument) which have the same Style. 
 */
public class StyledToken {

	private int offset;
	private String text;
	private Style style;
	
	public StyledToken(String text, int offset, Style style) {
		this.text = text.replaceAll("\r\n", "\n");	//remove the \r of Windows linebreaks
		this.offset = offset;
		this.style = style;
	}

	public int getOffset() {
		return offset;
	}
	
	public int getEndOffset() {
		return offset + text.length();
	}

	public String getText() {
		return text;
	}
	
	public boolean isWhitespace() {
		return text.matches("\\s*");
	}

	public Style getStyle() {
		return style;
	}
	

	public boolean isBold() {
		Object isBold = style.getAttribute(StyleConstants.Bold);
		return !(isBold == null || isBold.equals(false));
	}
	
	public boolean isItalic() {
		Object isItalic = style.getAttribute(StyleConstants.Italic);
		return !(isItalic == null || isItalic.equals(false));
	}
	
	public boolean isUnderlined() {
		Object isUnderlined = style.getAttribute(StyleConstants.Underline);
		return !(isUnderlined == null || isUnderlined.equals(false));
	}
	
	
	public boolean isStrikeThrough() {
		Object isStrikeThrough = style.getAttribute(StyleConstants.StrikeThrough);
		return !(isStrikeThrough == null || isStrikeThrough.equals(false));
	}
	

	public String getFontFamily() {
		Object ff = style.getAttribute(StyleConstants.FontFamily);
		return (String)ff;
	}
	
	public Color getColor() {
		Object color = style.getAttribute(StyleConstants.Foreground);
		if (color == null) {
			return Color.BLACK;
		} else {
			return (Color)color;
		}
	}
	

	private String getShortColorRepresentation() {
		Color color = getColor();
		return "<" + color.getRed() + "," + color.getGreen() + "," + color.getBlue() + ">";
	}
	
	
	public String toString() {
		if (isWhitespace()) {
			return "";
		} else {
			return text + ": \"" + getFontFamily() + "\" color" + getShortColorRepresentation() + " " 
					+ (isBold() ? "[bold]" : "") + (isItalic() ? "[italic]" : "") + (isUnderlined() ? "[underlined]" : "");
		}
	}
	
	
	public boolean equals(Object o) {
		if (o instanceof StyledToken) {
			StyledToken t2 = (StyledToken)o;
			boolean equals = getOffset() == t2.getOffset() && getEndOffset() == t2.getEndOffset()
					&& getText().equals(t2.getText()) && isBold() == t2.isBold()
					&& isItalic() == t2.isItalic() && isStrikeThrough() == t2.isStrikeThrough()
					&& isUnderlined() == t2.isUnderlined();
			return equals;
		} else {
			return false;
		}
	}
	
}
