package org.deft.representation.text;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;

import org.deft.artifacttypesupport.codefile.PositionConverter;
import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.util.StringUtil;

public class TextRepresentation implements ArtifactRepresentation {

	public static final String TYPE = "org.deft.representation.text";
	
	private String text;
	private int startOffset, endOffset;
	private PositionConverter positionConverter;
	
	public TextRepresentation(String text) {
		this.text = text;
		try {
			BufferedReader br = new BufferedReader(new StringReader(text));
			this.positionConverter = new PositionConverter(br);
		} catch (IOException e) {
			e.printStackTrace();
		}
		this.startOffset = 0;
		this.endOffset = text.length();
	}
	
	public TextRepresentation(String text, int startOffset, int endOffset) {
		this(text);
		this.startOffset = startOffset;
		this.endOffset = endOffset;
	}
	
	
	public String getText() {
		return text;
	}
	
	public int getStartOffset() {
		return startOffset;
	}
	
	public int getEndOffset() {
		return endOffset;
	}
	
	
	public String getTextFromOffsets(boolean multilineTrim) {
    	//if the whole file is to be processed, endOffset-startOffset+1
    	//exceeds the file size which causes an error in the scanner.
    	//Thus, limit the length to the actual document length;
        int length = Math.min(getEndOffset() + 1, getText().length());

		String substring = text.substring(getStartOffset(), length);
		if (multilineTrim) {
			int colOfStartChar = positionConverter.getCol(getStartOffset());
			String trimmedString = StringUtil.multilineTrim(colOfStartChar, substring);
			return trimmedString;
		} else {
			return substring;
		}
	}


	
	
	@Override
	public String getType() {
		return TYPE;
	}

}
