package org.deft.representationrenderer;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;

public class EclipseRepresentationRendererId2ArtifactViewerConnectorIdMappingRegistry 
		implements RepresentationRendererId2ArtifactViewerConnectorIdMappingRegistry {
	
	private static final String EXTENSION_POINT_NAME = "org.deft.rendererid2artifactviewerconnectoridmapping";
		
	private Map<String, MappingConfig> registeredMappings = new HashMap<String, MappingConfig>(); 

	public EclipseRepresentationRendererId2ArtifactViewerConnectorIdMappingRegistry() {
		getAndCacheRegisteredViewers();
	}
	
	private void getAndCacheRegisteredViewers() {
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint point = registry.getExtensionPoint(EXTENSION_POINT_NAME);
		for (IConfigurationElement ce : point.getConfigurationElements()) {
			MappingConfig config = new MappingConfig(ce);
			registeredMappings.put(config.getRendererId(), config);
		}
	}
	
	@Override
	public String getArtifactViewerId(String rendererId) {
		MappingConfig config = registeredMappings.get(rendererId);
		if (config != null) {
			String viewerId = config.getViewerId();
			return viewerId;
		}
		return null;
	}

	
	
	/**
	 * This class offers convenient access to ArtifactViewerMappings by
	 * encapsulating the Eclipse internal extension mechanism. There is
	 * one MappingConfig for each concrete mapping.
	 * 
	 * The config contains the id of the renderer and the id of
	 * the viewer that can display representations rendered with this renderer.
	 */
	private class MappingConfig {
	
		private IConfigurationElement ce;
		
		public MappingConfig(IConfigurationElement ce) {
			this.ce = ce;
		}
		
		/**
		 * Returns the renderer id
		 */
		public String getRendererId() {
			String id = ce.getAttribute("renderer_id");
			return id;
		}
		
		/**
		 * Returns the viewer id for the renderer
		 */
		public String getViewerId() {
			IConfigurationElement[] artifactViewerIdConfigElements = ce.getChildren();
			for (IConfigurationElement artifactViewerConfigElement : artifactViewerIdConfigElements) {
				String id = artifactViewerConfigElement.getAttribute("artifactviewerconnector_id");
				return id;
			}
			return null;
		}

	}


}