package org.deft.representationrenderer;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;


public class EclipseRepresentationRendererRegistry implements RepresentationRendererRegistry {
	
	private static final String EXTENSION_POINT_NAME = "org.deft.representationrenderer";
		
	private Map<String, RendererConfig> registeredRenderers = new HashMap<String, RendererConfig>(); 

	public EclipseRepresentationRendererRegistry() {
		getAndCacheRegisteredRenderers();
	}
	
	private void getAndCacheRegisteredRenderers() {
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint point = registry.getExtensionPoint(EXTENSION_POINT_NAME);
		for (IConfigurationElement ce : point.getConfigurationElements()) {
			RendererConfig config = new RendererConfig(ce);
			registeredRenderers.put(config.getId(), config);
		}
	}
	
	@Override
	public RepresentationRenderer getRenderer(String rendererId) {
		RendererConfig config = registeredRenderers.get(rendererId);
		if (config == null) {
			return null;
		}
		RepresentationRenderer renderer = config.getRenderer();
		return renderer;
	}
	
	@Override
	public List<String> getRendererIds() {
		List<String> rendererIds = new LinkedList<String>(registeredRenderers.keySet());
		return rendererIds;
	}
	

	@Override
	public List<RepresentationRenderer> getRenderers() {
		List<RepresentationRenderer> renderers = new LinkedList<RepresentationRenderer>();
		for (String rendererId : getRendererIds()) {
			renderers.add(getRenderer(rendererId));
		}
		return renderers;
	}
	
	
	@Override
	public String getArtifactViewerConnectorIdForRendererId(String rendererId) {
		RendererConfig config = registeredRenderers.get(rendererId);
		if (config == null) {
			return null;
		}
		String id = config.getArtifactViewerId();
		return id;
	}


	
	private class RendererConfig {
	
		private IConfigurationElement ce;
		private RepresentationRenderer renderer;
		
		public RendererConfig(IConfigurationElement ce) {
			this.ce = ce;
		}
		
		public String getId() {
			String id = ce.getAttribute("id");
			return id;
		}
		
		public String getArtifactViewerId() {
			String id = ce.getAttribute("artifactviewer_id");
			return id;
		}

		
		public RepresentationRenderer getRenderer() {
			if (renderer == null) {
				try {
					String rendererPropertyInExtensionPoint = "class";
					renderer = (RepresentationRenderer)ce.createExecutableExtension(
							rendererPropertyInExtensionPoint);
				} catch (CoreException e) {
					e.printStackTrace();
				}
			}
			return renderer;
		}	
	}






	
}
