package org.deft.representationrenderer.ecore;


import java.io.InputStream;

import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.ecore.EcoreInstanceRepresentation;
import org.deft.representationrenderer.RepresentationRenderer;
import org.eclipse.emf.ecore.EObject;

/**
 * A RepresentationRenderer that converts an EcoreInstanceRepresentation into
 * an EObject (root element of an Ecore instance model). 
 * 
 * Actually, there is no real conversion going on here, the EObject is just
 * retrieved from the EcoreInstanceRepresentation.
 */
public class EcoreRepresentationRenderer implements RepresentationRenderer {

	public static final String ID = "org.deft.representationrenderer.ecore";
	
	@Override
	public String getId() {
		return ID;
	}
	
	public String getDescription() {
		return "Ecore Tree";
	}

	@Override
	public boolean canRender(String representationType) {
		boolean canRender = EcoreInstanceRepresentation.TYPE.equals(representationType);
		return canRender;
	}

	@Override
	public EObject render(ArtifactRepresentation representation) {
		EObject root = ((EcoreInstanceRepresentation)representation).getModelRoot();
		return root;
	}

	/**
	 * No input stream is provided here, because there are many possible ways to
	 * serialize an Ecore (instance) model. Take for example XMI, or a DSL syntax as
	 * provided by EMFText. Consequently it is hard to deserialize the stream again,
	 * because it would be necessary to analyze the stream and then decide which
	 * deserializer to choose.
	 * So until the day we really need a stream, this method is not going to be implemented.
	 */
	@Override
	public InputStream getInputStream(ArtifactRepresentation representation) {
		throw new UnsupportedOperationException("No input stream available");
	}

}
