package org.deft.representationrenderer.standard;

import java.awt.image.BufferedImage;
import java.io.InputStream;

import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.image.BufferedImageRepresentation;
import org.deft.representationrenderer.RepresentationRenderer;
import org.deft.util.ImageUtil;

/**
 * A RepresentationRenderer that converts a BufferedImageRepresentation into
 * a BufferedImage. 
 * 
 * Actually, there is no real conversion going on here, the BufferedImage is just
 * retrieved from the BufferedImageRepresentation.
 */
public class ImageRepresentationRenderer implements RepresentationRenderer {

	public static final String ID = "org.deft.representationrenderer.image";
	
	@Override
	public String getId() {
		return ID;
	}
	
	public String getDescription() {
		return "Image";
	}

	@Override
	public boolean canRender(String representationType) {
		boolean canRender = BufferedImageRepresentation.TYPE.equals(representationType);
		return canRender;
	}

	@Override
	public BufferedImage render(ArtifactRepresentation representation) {
		BufferedImage image = ((BufferedImageRepresentation)representation).getImage();
		return image;
	}

	@Override
	public InputStream getInputStream(ArtifactRepresentation representation) {
		BufferedImage image = render(representation);
		InputStream is = ImageUtil.getInputStream(image);
		return is;
	}

}
