package org.deft.representationrenderer.standard;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.List;
import java.util.Map;

import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.styleddocument.StyledDefinitionListingRepresentation;
import org.deft.representation.styleddocument.conversion.HtmlTextMapping;
import org.deft.representation.styleddocument.data.DefinitionListing;
import org.deft.representation.styleddocument.data.DocumentContent;
import org.deft.representation.styleddocument.data.TokenizedStyledDocument;
import org.deft.representationrenderer.RepresentationRenderer;


public class StyledTextDefinitionListingRepresenationRenderer implements RepresentationRenderer {

	public static final String ID = "org.deft.representationrenderer.definitionlisting.styledtext";
	private HtmlTextMapping mapping = new HtmlTextMapping(false);
	
	@Override
	public String getId() {
		return ID;
	}
	
	public String getDescription() {
		return "Definition Listing";
	}

	@Override
	public boolean canRender(String representationType) {
		boolean canRender = StyledDefinitionListingRepresentation.TYPE.equals(representationType);
		return canRender;
	}

	@Override
	public String render(ArtifactRepresentation representation) {
		StyledDefinitionListingRepresentation listingRep = (StyledDefinitionListingRepresentation)representation;
		DefinitionListing listing = listingRep.getListing();
		List<TokenizedStyledDocument> terms = listing.getTerms();
		Map<TokenizedStyledDocument, DocumentContent> termExplanationMap = listing.getMapping();

		String htmlString = "<dl>";
		for (TokenizedStyledDocument term : terms) {
			DocumentContent dc = termExplanationMap.get(term);
			if (dc instanceof TokenizedStyledDocument) {
				TokenizedStyledDocument expl = (TokenizedStyledDocument)dc;
				htmlString += "<dt>";
				htmlString += term.getMarkedUpText(mapping);
				htmlString += "</dt>";
				htmlString += "<dd>";
				htmlString += expl.getMarkedUpText(mapping);
				htmlString += "</dd>";
			}
		}
		htmlString += "</dl>";
		return htmlString;
	}

	@Override
	public InputStream getInputStream(ArtifactRepresentation representation) {
		String text = render(representation);
		byte[] bytes = text.getBytes();
		InputStream is = new ByteArrayInputStream(bytes);
		BufferedInputStream bis = new BufferedInputStream(is);
		return bis;
	}
}

