package org.deft.representationrenderer.standard;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.InputStream;

import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.styleddocument.StyledTextRepresentation;
import org.deft.representation.styleddocument.conversion.HtmlTextMapping;
import org.deft.representation.styleddocument.data.TokenizedStyledDocument;
import org.deft.representationrenderer.RepresentationRenderer;

/**
 * A RepresentationRenderer that converts a StyledDocumentRepresentation into
 * a plain string with markup. 
 * 
 * Currently HTML is used as markup.  
 */
public class StyledTextRepresentationRenderer implements RepresentationRenderer {

	public static final String ID = "org.deft.representationrenderer.styledtext";
	protected HtmlTextMapping mapping;
	
	public StyledTextRepresentationRenderer() {
		mapping = new HtmlTextMapping(false);
	}
	
	@Override
	public String getId() {
		return ID;
	}
	
	public String getDescription() {
		return "Styled Text";
	}

	@Override
	public boolean canRender(String representationType) {
		boolean canRender = StyledTextRepresentation.TYPE.equals(representationType);
		return canRender;
	}

	@Override
	public String render(ArtifactRepresentation representation) {
		TokenizedStyledDocument styledDoc = ((StyledTextRepresentation)representation).getStyledDocument();
		String htmlString = styledDoc.getMarkedUpText(mapping);
		return htmlString;
	}

	@Override
	public InputStream getInputStream(ArtifactRepresentation representation) {
		String text = render(representation);
		byte[] bytes = text.getBytes();
		InputStream is = new ByteArrayInputStream(bytes);
		BufferedInputStream bis = new BufferedInputStream(is);
		return bis;
	}
	
}
