package org.deft.representationrenderer.standard;


import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.InputStream;

import org.deft.repository.datamodel.ArtifactRepresentation;
import org.deft.representation.text.TextRepresentation;
import org.deft.representationrenderer.RepresentationRenderer;

/**
 * A RepresentationRenderer that converts a TextRepresentation into a String.
 * 
 * Actually, there is no real conversion going on here, the String is just
 * retrieved from the TextRepresentation.
 */
public class TextRepresentationRenderer implements RepresentationRenderer {

	public static final String ID = "org.deft.representationrenderer.text";
	
	@Override
	public String getId() {
		return ID;
	}
	
	public String getDescription() {
		return "Simple Text";
	}

	@Override
	public boolean canRender(String representationType) {
		boolean canRender = TextRepresentation.TYPE.equals(representationType);
		return canRender;
	}

	@Override
	public String render(ArtifactRepresentation representation) {
		String text = ((TextRepresentation)representation).getText();
		return text;
	}

	@Override
	public InputStream getInputStream(ArtifactRepresentation representation) {
		String text = render(representation);
		byte[] bytes = text.getBytes();
		InputStream is = new ByteArrayInputStream(bytes);
		BufferedInputStream bis = new BufferedInputStream(is);
		return bis;
	}

}
