package org.deft.requirementsontology;

import java.util.List;

import javax.swing.JTable;

public abstract class AbstractTraceabilityMatrix
{
	/**
	 * contains all source individual names
	 */
	private String[] sourceIndividuals;
	
	/**
	 * contains all target individual names
	 */
	private String[] targetIndividuals;

	/**
	 * contains all relations between source individuals and target individuals
	 */
	private String[][] matrixCells;

////////// constructors

	/**
	 * abstract constructor of a traceability matrix object
	 */
	public AbstractTraceabilityMatrix(List<String> sourceIndividuals, List<String> targetIndividuals, ROModel roModel)
	{
		for (String individual : sourceIndividuals)
		{
			if (roModel.containsIndividual(individual) == false)
			{
				throw new IllegalArgumentException("source individual <" + individual + "> does not exists in the goal model graph");
			}
			// end if
		}
		// end for
		
		for (String individual : targetIndividuals)
		{
			if (roModel.containsIndividual(individual) == false)
			{
				throw new IllegalArgumentException("target individual <" + individual + "> does not exists in the goal model graph");
			}
			// end if
		}
		// end for
		
		this.sourceIndividuals = new String[sourceIndividuals.size()];
		this.targetIndividuals = new String[targetIndividuals.size()];
		this.matrixCells = new String[sourceIndividuals.size()][targetIndividuals.size()];
		this.initialize(sourceIndividuals, targetIndividuals);
	}
	// end constructor

////////// end constructors
	
////////// initializing the traceability matrix

	/**
	 * initializes the traceability matrix
	 */
	private void initialize(List<String> sourceIndividuals, List<String> targetIndividuals)
	{
		for (int index = 0; index < sourceIndividuals.size(); index++)
		{
			this.sourceIndividuals[index] = sourceIndividuals.get(index);
		}
		// end for
		
		for (int index = 0; index < targetIndividuals.size(); index++)
		{
			this.targetIndividuals[index] = targetIndividuals.get(index);
		}
		// end for
		
		for (int sourceIndex = 0; sourceIndex < this.sourceIndividuals.length; sourceIndex++)
		{
			for (int targetIndex = 0; targetIndex < this.targetIndividuals.length; targetIndex++)
			{
				matrixCells[sourceIndex][targetIndex] = "";
			}
			// end for
		}
		// end for
	}
	// end initialize
	
////////// end initializing the traceability matrix	

////////// accessing the tracebility matrix

	public String[] getSourceIndividuals()
	{
		return this.sourceIndividuals;
	}
	// end getSourceIndividuals
	
	public String[] getTargetIndividuals()
	{
		return this.targetIndividuals;
	}
	// end getTargetIndividuals
	
	public void setCellContent(int sourceIndex, int targetIndex, String cellContent)
	{
		this.matrixCells[sourceIndex][targetIndex] = cellContent;
	}
	// end setCellContent
	
	public String[][] getMatrixCells()
	{
		return this.matrixCells;
	}
	// end getMatrixCells

	protected abstract JTable getJTableMatrix();
	
////////// end accessing the traceability matrix
}
// end class AbstractTraceabilityMatrix