package org.deft.requirementsontology;

import java.util.Set;

import org.deft.representation.table.data.Table;
import org.deft.representation.table.data.TableCell;
import org.deft.representation.table.data.TableRow;

public class IndividualDataVisualizer
{
	/**
	 * contains the requirements ontology model
	 */
	private AbstractROModel	roModel;
	
////////// constructors
	
	/**
	 * creates a visualizer for individuals of the requirements ontology model (to a given requirements ontology model)
	 */
	public IndividualDataVisualizer(AbstractROModel roModel)
	{
		this.roModel = roModel;
	}
	// end constructor
	
////////// end constructors
	
////////// visualizing the individual data as a data survey formular
	
	/**
	 * returns the data survey of an individual as a table (to a given simple individual name)
	 */
	public Table getIndividualDataSurvey(String simpleIndividualName)
	{
		if (roModel.containsIndividual(simpleIndividualName))
		{
			IndividualVertex individual = roModel.getIndividual(simpleIndividualName);
			Table individualDataSurvey = new Table();
			
			this.createsMandatoryRows(individualDataSurvey,individual);	
			individualDataSurvey.addRow(this.createsEmptyRow());
			this.createsCommentRows(individualDataSurvey,individual);
			individualDataSurvey.addRow(this.createsEmptyRow());
			this.createsAttributeRows(individualDataSurvey,individual);		
			individualDataSurvey.addRow(this.createsEmptyRow());
			this.createsFormalAttributeRows(individualDataSurvey,individual);		
			individualDataSurvey.addRow(this.createsEmptyRow());
			this.createsAdditionalObjectPropertyRows(individualDataSurvey,individual);			
			individualDataSurvey.addRow(this.createsEmptyRow());
			this.createsAdditionalDataPropertyRows(individualDataSurvey,individual);
			individualDataSurvey.addRow(this.createsEmptyRow());
			this.createsIncomingObjectPropertyRows(individualDataSurvey,individual);			
			return individualDataSurvey;
		}
		else
		{
			return null;
		}
		// end if else
	}
	// end getIndividualDataSurvey

	/**
	 * creates the rows that contains the name, essential type and main type of the individual
	 */	
	private void createsMandatoryRows(Table individualDataSurvey, IndividualVertex individual)
	{
		TableRow row1 = new TableRow();
		row1.addCell(new TableCell("Name:"));
		row1.addCell(new TableCell(individual.getSimpleName(),1,2));
		individualDataSurvey.addRow(row1);
		
		TableRow row2 = new TableRow();
		row2.addCell(new TableCell("Essential Type:"));
		row2.addCell(new TableCell(individual.getEssentialType(),1,2));
		individualDataSurvey.addRow(row2);
		
		TableRow row3 = new TableRow();
		row3.addCell(new TableCell("Main Type:"));
		row3.addCell(new TableCell(individual.getMainType(),1,2));
		individualDataSurvey.addRow(row3);
	}
	// end createsMandatoryRows

	/**
	 * creates the rows that contains the comments of the individual
	 */
	private void createsCommentRows(Table individualDataSurvey, IndividualVertex individual)
	{
		TableRow row1 = new TableRow();
		row1.addCell(new TableCell("Comments:",1,3));
		individualDataSurvey.addRow(row1);
		
		if (individual.getComments().size() > 0)
		{
			for (String comment : individual.getComments())
			{
				TableRow row2 = new TableRow();
				row2.addCell(new TableCell(comment,1,3));
				individualDataSurvey.addRow(row2);
			}
			// end for
		}
		else
		{
			individualDataSurvey.addRow(this.createsRowWithStroke());
		}
		// end if else
	}
	// end createsCommentRows

	/**
	 * creates the rows that contains the attributes of the individual
	 */
	private void createsAttributeRows(Table individualDataSurvey, IndividualVertex individual)
	{
		TableRow row1 = new TableRow();
		row1.addCell(new TableCell("Attributes:",1,3));
		individualDataSurvey.addRow(row1);
		
		if (individual.getAttributes().size() > 0)
		{
			for (String attribute : individual.getAttributes())
			{
				TableRow row2 = new TableRow();
				row2.addCell(new TableCell(""));
				row2.addCell(new TableCell(attribute,1,2));
				individualDataSurvey.addRow(row2);
			}
			// end for
		}
		else
		{
			individualDataSurvey.addRow(this.createsRowWithStroke());				
		}
		// end if else
	}
	// end createsAttributeRows

	/**
	 * creates the rows that contains the formal attributes of the individual
	 */
	private void createsFormalAttributeRows(Table individualDataSurvey, IndividualVertex individual)
	{
		TableRow row1 = new TableRow();
		row1.addCell(new TableCell("Formal Attributes (Mandatory Object/Datatype Properties):",1,3));
		individualDataSurvey.addRow(row1);
		
		if (individual.getFormalAttributeNames().size() > 0)
		{
			for (String formalAttributeName : individual.getFormalAttributeNames())
			{
				Set<String> formalAttributeTypes = individual.getFormalAttributeTypes(formalAttributeName);
				
				for (String formalAttributeType : formalAttributeTypes)
				{
					Set<String> formalAttributeIndividualSet = individual.getFormalAttributeIndividuals(formalAttributeName,formalAttributeType);
					
					TableRow row2 = new TableRow();
					row2.addCell(new TableCell(formalAttributeName + " (" + formalAttributeType + "):"));
					StringBuffer formalAttributeIndividualNames = new StringBuffer();
					
					for (String formalAttributeIndividual : formalAttributeIndividualSet)
					{
						formalAttributeIndividualNames.append(formalAttributeIndividual + ", ");
					}
					// end for
					
					formalAttributeIndividualNames.delete(formalAttributeIndividualNames.lastIndexOf(","),formalAttributeIndividualNames.lastIndexOf(" ") + 1);
					
					if (formalAttributeIndividualNames.toString().equals(""))
					{
						formalAttributeIndividualNames.append("-");
					}
					// end if
						
					row2.addCell(new TableCell(formalAttributeIndividualNames.toString(),1,2));
					individualDataSurvey.addRow(row2);
				}
				// end for
			}
			// end for
		}
		else
		{
			individualDataSurvey.addRow(this.createsRowWithStroke());	
		}
		// end if else
	}
	// end createsFormalAttributeRows

	/**
	 * creates the rows that contains the additional object properties of the individual
	 */
	private void createsAdditionalObjectPropertyRows(Table individualDataSurvey, IndividualVertex individual)
	{
		TableRow row1 = new TableRow();
		row1.addCell(new TableCell("Additional outgoing Object Properties:",1,3));
		individualDataSurvey.addRow(row1);
		
		if (individual.getAdditionalObjectPropertyNames().size() > 0)
		{
			for (String additionalObjectPropertyName : individual.getAdditionalObjectPropertyNames())
			{
				String additionalObjectPropertyType = individual.getAdditionalObjectPropertyType(additionalObjectPropertyName);
				Set<String> additionalObjectPropertyIndividualSet = individual.getAdditionalObjectPropertyIndividuals(additionalObjectPropertyName,additionalObjectPropertyType);
				
				TableRow row2 = new TableRow();
				row2.addCell(new TableCell(additionalObjectPropertyName + " (" + additionalObjectPropertyType + "):"));
				StringBuffer additionalObjectPropertyIndividualNames = new StringBuffer();
				
				for (String additionalObjectPropertyIndividual : additionalObjectPropertyIndividualSet)
				{
					additionalObjectPropertyIndividualNames.append(additionalObjectPropertyIndividual + ", ");
				}
				// end for
				
				additionalObjectPropertyIndividualNames.delete(additionalObjectPropertyIndividualNames.lastIndexOf(","),additionalObjectPropertyIndividualNames.lastIndexOf(" ") + 1);
				
				if (additionalObjectPropertyIndividualNames.toString().equals(""))
				{
					additionalObjectPropertyIndividualNames.append("-");
				}
				// end if
					
				row2.addCell(new TableCell(additionalObjectPropertyIndividualNames.toString(),1,2));
				individualDataSurvey.addRow(row2);
			}
			// end for
		}
		else
		{
			individualDataSurvey.addRow(this.createsRowWithStroke());
		}
		// end if else
	}
	// end createsAdditionalObjectPropertyRows
	
	/**
	 * creates the rows that contains the additional data properties of the individual
	 */
	private void createsAdditionalDataPropertyRows(Table individualDataSurvey, IndividualVertex individual)
	{
		TableRow row1 = new TableRow();
		row1.addCell(new TableCell("Additional Datatype Properties:",1,3));
		individualDataSurvey.addRow(row1);
		
		if (individual.getAdditionalDataPropertyNames().size() > 0)
		{
			for (String additionalDataPropertyName : individual.getAdditionalDataPropertyNames())
			{
				String additionalDataPropertyType = individual.getAdditionalDataPropertyType(additionalDataPropertyName);
				Set<String> additionalDataPropertyIndividualSet = individual.getAdditionalDataPropertyValues(additionalDataPropertyName,additionalDataPropertyType);
				
				TableRow row2 = new TableRow();
				row2.addCell(new TableCell(additionalDataPropertyName + " (" + additionalDataPropertyType + "):"));
				StringBuffer additionalDataPropertyValues = new StringBuffer();
				
				for (String additionalDataPropertyValue : additionalDataPropertyIndividualSet)
				{
					additionalDataPropertyValues.append(additionalDataPropertyValue + ", ");
				}
				// end for
				
				additionalDataPropertyValues.delete(additionalDataPropertyValues.lastIndexOf(","),additionalDataPropertyValues.lastIndexOf(" ") + 1);
				
				if (additionalDataPropertyValues.toString().equals(""))
				{
					additionalDataPropertyValues.append("-");
				}
				// end if
					
				row2.addCell(new TableCell(additionalDataPropertyValues.toString(),1,2));
				individualDataSurvey.addRow(row2);
			}
			// end for
		}
		else
		{
			individualDataSurvey.addRow(this.createsRowWithStroke());
		}
		// end if else
	}
	// end createAdditionalDataPropertyRows
	
	/**
	 * creates the rows that contains the incoming object properties of the individual
	 */
	private void createsIncomingObjectPropertyRows(Table individualDataSurvey, IndividualVertex individual)
	{
		TableRow row1 = new TableRow();
		row1.addCell(new TableCell("Incoming Object Properties:",1,3));
		individualDataSurvey.addRow(row1);
		
		if (individual.getIncomingObjectPropertyNames().size() > 0)
		{
			for (String incomingObjectPropertyName : individual.getIncomingObjectPropertyNames())
			{
				Set<String> incomingObjectPropertySourceIndividualTypes = individual.getIncomingObjectPropertySourceIndividualTypes(incomingObjectPropertyName);
				
				for (String incomingObjectPropertySourceIndividualType : incomingObjectPropertySourceIndividualTypes)
				{
					Set<String> incomingObjectPropertySourceIndividualSet = individual.getIncomingObjectPropertySourceIndividuals(incomingObjectPropertyName,incomingObjectPropertySourceIndividualType);
					
					for (String incomingObjectPropertySourceIndividual : incomingObjectPropertySourceIndividualSet)
					{
						TableRow row2 = new TableRow();
						row2.addCell(new TableCell(incomingObjectPropertySourceIndividual + " (" + incomingObjectPropertySourceIndividualType + "):"));
						row2.addCell(new TableCell(incomingObjectPropertyName));
						row2.addCell(new TableCell(individual.getSimpleName()));
						individualDataSurvey.addRow(row2);
					}
					// end for
				}
				// end for
			}
			// end for
		}
		else
		{
			individualDataSurvey.addRow(this.createsRowWithStroke());
		}
		// end if else
	}
	// end createsIncomingObjectPropertyRows
	
	/**
	 * returns a row with a stroke in it
	 */
	private TableRow createsRowWithStroke()
	{
		TableRow row = new TableRow();
		row.addCell(new TableCell("-",1,3));
		return row;
	}
	// end createsRowWithStroke
	
	/**
	 * returns an empty row
	 */
	private TableRow createsEmptyRow()
	{
		TableRow row = new TableRow();
		row.addCell(new TableCell("",1,3));
		return row;
	}
	// end createsEmptyRow
		
////////// end visualizing the individual data as a data survey formular
}
// end class IndividualDataVisualizer