package org.deft.requirementsontology;

import java.util.Collection;
import java.util.List;

import javax.swing.JTable;

public class PropertyTraceabilityMatrix extends AbstractTraceabilityMatrix
{
	/**
	 * contains the simple object property name
	 */
	private String objectPropertyName;

////////// constructors
	
	/**
	 * creates a traceability matrix that holds the information if source individuals are relate to target individuals over an object property 
	 * (
	 */
	public PropertyTraceabilityMatrix(String simpleObjectPropertyName, List<String> sourceIndividuals, List<String> targetIndividuals, ROModel roModel)
	{
		super(sourceIndividuals,targetIndividuals,roModel);
		this.objectPropertyName = simpleObjectPropertyName;
		this.setContent(sourceIndividuals, targetIndividuals, roModel);
	}
	// end constructor

////////// end constructors	
	
////////// accessing the traceability matrix	
	
	/**
	 * inserts an "x" if the specific object property exsits between the source individuals and the target individuals (to the given lists of simple source individual names and simple target individual names) 
	 */
	private void setContent(List<String> sourceIndividuals, List<String> targetIndividuals, ROModel roModel)
	{
		for (int sourceIndex = 0; sourceIndex < sourceIndividuals.size(); sourceIndex++)
		{
			for (int targetIndex = 0; targetIndex < targetIndividuals.size(); targetIndex++)
			{
				Collection <ObjectPropertyEdge> edgeCollection = roModel.getObjectProperties(sourceIndividuals.get(sourceIndex), targetIndividuals.get(targetIndex));

				if (edgeCollection.isEmpty() == false)
				{
					for (ObjectPropertyEdge edge : edgeCollection)
					{
						if (edge.getSimpleName().equals(this.objectPropertyName) == true)
						{
							this.setCellContent(sourceIndex,targetIndex,"x");
						}
						// end if
					}
					// end for
				}
				// end if
			}
			// end for
		}
		// end for
	}
	// end setContent
	
	public String getObjectPropertyName()
	{
		return this.objectPropertyName;
	}
	// end getObjectPropertyName
	
	/**
	 * just for testing
	 */
	@Override
	protected JTable getJTableMatrix()
	{
		JTable table = new JTable();
		
		if ((this.getSourceIndividuals().length > 0) && (this.getTargetIndividuals().length > 0))
		{
			String[] columnNames = new String[this.getTargetIndividuals().length + 1];
			columnNames[0] = this.objectPropertyName;
			
			for (int targetIndex = 0; targetIndex < this.getTargetIndividuals().length; targetIndex++)
			{
				columnNames[targetIndex + 1] = this.getTargetIndividuals()[targetIndex];
			}
			// end for
			
			String[][] rowData = new String[this.getSourceIndividuals().length][this.getTargetIndividuals().length + 1];
			
			for (int sourceIndex = 0; sourceIndex < this.getSourceIndividuals().length; sourceIndex++)
			{
				rowData[sourceIndex][0] = this.getSourceIndividuals()[sourceIndex];
			}
			// end for
			
			for (int matrixRowIndex = 0; matrixRowIndex < this.getMatrixCells().length; matrixRowIndex++)
			{
				for (int matrixColumnIndex = 0; matrixColumnIndex < this.getMatrixCells()[matrixRowIndex].length; matrixColumnIndex++)
				{
					rowData[matrixRowIndex][matrixColumnIndex + 1] = this.getMatrixCells()[matrixRowIndex][matrixColumnIndex];
				}
				// end for
			}
			// end for

			table = new JTable(rowData,columnNames);
		}
		// end if

		return table;
	}
	// end getJTableMatrix
	
////////// end accessing the traceability matrix	
}
// end class PropertyTraceabilityMatrix