package org.deft.resource.impl;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;

import org.deft.resource.ResourceHandler;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Platform;
import org.osgi.framework.Bundle;



public class EclipsePluginResourceHandler implements ResourceHandler {
	
	private String resourceHandlerId;
	private Bundle bundle;
	
	/**
	 * 
	 * @param resourceHandlerId name of the bundle where the resource resides
	 */
	public EclipsePluginResourceHandler(String resourceHandlerId) {
		this.resourceHandlerId = resourceHandlerId;
		bundle = Platform.getBundle(resourceHandlerId);
	}
	

	public URL getResourceUrl(String resource) {
    	if (bundle == null)
    		return null;
    	URL bundleUrl = bundle.getEntry(resource);
        if (bundleUrl == null)
        	return null;
        try {
            URL fileUrl = FileLocator.resolve(bundleUrl);

            //the fileUrl must be escaped. This is easy with a URL starting
            //with file:/ but takes some extra measures for jar:file URLs 
            boolean isJar = false;
            String fileUrlString = fileUrl.toString();
            
            //if we have a jar:file URL, memorize it (isJar flag) and
            //remove the jar: prefix, so that it can be correctly escaped 
            //in the next step
            if (fileUrlString.startsWith("jar:file:/")) {
            	isJar = true;
            	fileUrlString = fileUrlString.replaceFirst("jar:file:/", "file:/");
            }
            //escape the fileUri
            fileUrl = new URL(fileUrlString);
			URI escapedFileUri = new URI(fileUrl.getProtocol(), fileUrl.getHost(), 
					fileUrl.getPath(), fileUrl.getQuery(), null);

			//if we had a jar-URL originally, add the jar prefix again  
			if (isJar) {
				String newUriString = escapedFileUri.toString();
				newUriString = "jar:" + newUriString;
				URL newUrl = new URL(newUriString);
				return newUrl;
			} else {
				//otherwise just return the escaped URL
				URL escapedFileUrl = escapedFileUri.toURL();
				return escapedFileUrl;
			}
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } catch (URISyntaxException use) {
			use.printStackTrace();
		}
        return null;
    }
	
	

	public String getResourceHandlerId() {
		return resourceHandlerId;
	}

    
	

}
