package org.deft.texlipse;

import net.sourceforge.texlipse.properties.TexlipseProperties;

import org.deft.actions.AbstractDeftAction;
import org.deft.gui.SelectionHelper;
import org.deft.repository.datamodel.Chapter;
import org.deft.repository.workspaceresources.WorkspaceManager;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.ui.IWorkbenchWindow;

public class ProjectActionProvider extends org.deft.gui.view.project.ProjectActionProvider {

	private UpdateOutlineAction updateOutlineAction = new UpdateOutlineAction();
	
	@Override
	public void fillContextMenu(IMenuManager menu) {
		super.fillContextMenu(menu);
		menu.add(new Separator("group.latex"));
		menu.appendToGroup("group.latex", new MainFileAction(getWindow()));
	}
	
	
	private class MainFileAction extends AbstractDeftAction {

		public MainFileAction(IWorkbenchWindow window) {
			super(window);
			setText("Set as LaTeX main file");
		}
		
		@Override
		protected void setEnabledState() {
			SelectionHelper selection = getSelectionHelper();
			if (selection.isOnlySingleChapterNodeSelected()) {
				Chapter chapter = selection.getSelectedChapter();
				if (chapter.getTypeId().equals("latex")) {
					setEnabled(true);
					return;
				}
			}
			setEnabled(false);
		}
		
		
		@Override
		public void run() {
			Chapter chapter = getSelectionHelper().getSelectedChapter();
			setProperties(chapter);
			updateOutlineAction.updateOutline(chapter);
		}
		
		private void setProperties(Chapter deftChapter) {
			IFile file = WorkspaceManager.instance().getEclipseChapterInWorkspace(deftChapter);
		    IProject project = file.getProject();
		    
	        //load settings, if changed on disk
	        if (TexlipseProperties.isProjectPropertiesFileChanged(project)) {
	            TexlipseProperties.loadProjectProperties(project);
	        }
	        
		    String name = file.getName();
	        IPath path = file.getFullPath();
	        
	        IResource currentMainFile = TexlipseProperties.getProjectSourceFile(project);
	        // check if this is already the main file
	        if (currentMainFile != null && path.equals(currentMainFile.getFullPath())) {
	            return;
	        }
	        
	        // set main file
		    TexlipseProperties.setProjectProperty(project, TexlipseProperties.MAINFILE_PROPERTY, name);
		    
	        // set source directory
		    String dir = path.removeFirstSegments(1).removeLastSegments(1).toString();
	        TexlipseProperties.setProjectProperty(project, TexlipseProperties.SOURCE_DIR_PROPERTY, dir);
	        
	        // make sure there is output directory setting
	        String oldOut = TexlipseProperties.getProjectProperty(project, TexlipseProperties.OUTPUT_DIR_PROPERTY);
	        if (oldOut == null) {
	            TexlipseProperties.setProjectProperty(project, TexlipseProperties.OUTPUT_DIR_PROPERTY, dir);
	        }
	        
	        //save settings to file
	        TexlipseProperties.saveProjectProperties(project);
		}
	}

}
