package org.deft.texlipse;

import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

import net.sourceforge.texlipse.model.OutlineNode;
import net.sourceforge.texlipse.model.TexOutlineInput;
import net.sourceforge.texlipse.outline.TexProjectOutline;
import net.sourceforge.texlipse.properties.TexlipseProperties;
import net.sourceforge.texlipse.texparser.TexParser;
import net.sourceforge.texlipse.treeview.views.TexOutlineTreeView;

import org.deft.repository.datamodel.Chapter;
import org.deft.repository.datamodel.Project;
import org.deft.repository.workspaceresources.WorkspaceManager;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.PlatformUI;

public class UpdateOutlineAction {
	
	
	public void updateOutline(Chapter deftChapter) {
		Project deftProject = deftChapter.getProject();
		IProject project = WorkspaceManager.instance().getEclipseProjectInWorkspace(deftProject);
		updateOutline(project);
	}
	
	public void updateOutline(IProject project) {
		
		try {
			TexOutlineTreeView view = getTreeView();
			if (view != null) {
				WorkspaceManager wm = WorkspaceManager.instance();
				IFile mainFile = null;
				try {
					mainFile = TexlipseProperties.getProjectSourceFile(project);
				}
				catch (Exception e) {
					setEmptyOutline();
				}
				if (mainFile != null && mainFile.exists()) {
					TexParser parser = new TexParser(null);
					Chapter deftChapter = wm.getDeftChapter(mainFile); 
					parser.parseDocument(new String(deftChapter.getChapterContent()), false);
					TexOutlineTreeView treeView = (TexOutlineTreeView)view;
					TexProjectOutline ol = new TexProjectOutline(project);
					List<OutlineNode> outlineNodes = ol.getFullOutline();
					TexOutlineInput input = new TexOutlineInput(outlineNodes);
					treeView.update(input);
				} else {
					setEmptyOutline();
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	private void setEmptyOutline() {
		TexOutlineTreeView treeView = getTreeView();
		List<OutlineNode> emptyList = new LinkedList<OutlineNode>();
		TexOutlineInput input = new TexOutlineInput(emptyList);
		treeView.update(input);
	}
	
	private TexOutlineTreeView getTreeView() {
		IViewPart view = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().findView("net.sourceforge.texlipse.FullOutline");
		TexOutlineTreeView treeView = (TexOutlineTreeView)view;
		return treeView;
	}

}
