package org.deft.util;

import org.deft.util.screenshot.ControlConfiguration;
import org.deft.util.screenshot.InvisibleShell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

public class ScreenshotUtil {
	
	/**
	 * Creates a Composite on an invisible shell. Can be used to 
	 * create widgets for which a screenshot should be made, without
	 * actually displaying them.
	 * 
	 * Example:
	 * 	Composite parent = ScreenshotUtil.createInvisibleComposite();
	 *	TreeViewer treeViewer = new TreeViewer(parent, SWT.NONE);
	 */
	public static Composite createInvisibleComposite() {
		Display display = Display.getCurrent();
		Shell shell = new InvisibleShell(display);
		Composite composite = new Composite(shell, SWT.NONE);
		return composite;
	}
	
	
	

	public static Image makeScreenshot(Control control) {

		ControlConfiguration config = new ControlConfiguration(control);

		setControlToInvisibleShell(control);
		Image image = makeActualScreenshot(control);
		
		undoChangesOnControl(control, config);
		
        return image;
	}
	
	private static void setControlToInvisibleShell(Control control) {
		Point p = control.computeSize(SWT.DEFAULT, SWT.DEFAULT);
		Display display = control.getDisplay();
	    Shell invisShell = new InvisibleShell(display);
	    invisShell.open();
	    control.setParent(invisShell);
	    control.setSize(p.x, p.y);
	}
	
	private static Image makeActualScreenshot(Control control) {
		Point size = control.getSize();
        Image image = new Image(control.getDisplay(), size.x, size.y);
        GC gc = new GC(image);
        control.print(gc);
//        int hWnd = control.handle;
//        int wParam = gc.handle;
//        int lParam = OS.PRF_NONCLIENT | OS.PRF_CLIENT | OS.PRF_ERASEBKGND | OS.PRF_CHILDREN;
//        OS.SendMessage(hWnd, OS.WM_PRINT, wParam, lParam);
        gc.dispose();
        return image;
	}
	
	private static void undoChangesOnControl(Control control, ControlConfiguration config) {
        control.setSize(config.getSize());
        control.setParent(config.getParent());
	}
	

	
	


}
