/**
 * <copyright>
 * </copyright>
 *
 * 
 */
package org.emftext.language.office.resource.office.util;

/**
 * This class provides basic infrastructure to interpret models. To implement
 * concrete interpreters, subclass this abstract interpreter and override the
 * interprete_* methods. The interpretation can be customized by binding the two
 * type parameters (ResultType, ContextType). The former is returned by all
 * interprete_* methods, while the latter is passed from method to method while
 * traversing the model. The concrete traversal strategy can also be exchanged.
 * One can use a static traversal strategy by pushing all objects to interpret on
 * the interpretation stack (using addObjectToInterprete()) before calling
 * interprete(). Alternatively, the traversal strategy can be dynamic by pushing
 * objects on the interpretation stack during interpretation.
 */
public class AbstractOfficeInterpreter<ResultType, ContextType> {
	
	private java.util.Stack<org.eclipse.emf.ecore.EObject> interpretationStack = new java.util.Stack<org.eclipse.emf.ecore.EObject>();
	
	public ResultType interprete(ContextType context) {
		ResultType result = null;
		while (!interpretationStack.empty()) {
			org.eclipse.emf.ecore.EObject next = interpretationStack.pop();
			result = interprete(next, context);
			if (!continueInterpretation(result)) {
				break;
			}
		}
		return result;
	}
	
	/**
	 * Override this method to stop the overall interpretation depending on the result
	 * of the interpretation of a single model elements.
	 */
	public boolean continueInterpretation(ResultType result) {
		return true;
	}
	
	public ResultType interprete(org.eclipse.emf.ecore.EObject object, ContextType context) {
		ResultType result = null;
		if (object instanceof org.emftext.language.office.Office) {
			result = interprete_org_emftext_language_office_Office((org.emftext.language.office.Office) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.office.Employee) {
			result = interprete_org_emftext_language_office_Employee((org.emftext.language.office.Employee) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.office.OfficeElement) {
			result = interprete_org_emftext_language_office_OfficeElement((org.emftext.language.office.OfficeElement) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.office.OfficeModel) {
			result = interprete_org_emftext_language_office_OfficeModel((org.emftext.language.office.OfficeModel) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.office.NamedElement) {
			result = interprete_org_emftext_language_office_NamedElement((org.emftext.language.office.NamedElement) object, context);
		}
		if (result != null) {
			return result;
		}
		return result;
	}
	
	public ResultType interprete_org_emftext_language_office_OfficeModel(org.emftext.language.office.OfficeModel object, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_office_Employee(org.emftext.language.office.Employee object, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_office_Office(org.emftext.language.office.Office object, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_office_NamedElement(org.emftext.language.office.NamedElement object, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_office_OfficeElement(org.emftext.language.office.OfficeElement object, ContextType context) {
		return null;
	}
	
	public void addObjectToInterprete(org.eclipse.emf.ecore.EObject object) {
		interpretationStack.push(object);
	}
	
}
