/**
 * <copyright>
 * </copyright>
 *
 * 
 */
package org.emftext.language.pl0.resource.pl0.mopp;

/**
 * The Pl0TokenResolverFactory class provides access to all generated token
 * resolvers. By giving the name of a defined token, the corresponding resolve can
 * be obtained. Despite the fact that this class is called TokenResolverFactory is
 * does NOT create new token resolvers whenever a client calls methods to obtain a
 * resolver. Rather, this class maintains a map of all resolvers and creates each
 * resolver at most once.
 */
public class Pl0TokenResolverFactory implements org.emftext.language.pl0.resource.pl0.IPl0TokenResolverFactory {
	
	private java.util.Map<String, org.emftext.language.pl0.resource.pl0.IPl0TokenResolver> tokenName2TokenResolver;
	private java.util.Map<String, org.emftext.language.pl0.resource.pl0.IPl0TokenResolver> featureName2CollectInTokenResolver;
	private static org.emftext.language.pl0.resource.pl0.IPl0TokenResolver defaultResolver = new org.emftext.language.pl0.resource.pl0.analysis.Pl0DefaultTokenResolver();
	
	public Pl0TokenResolverFactory() {
		tokenName2TokenResolver = new java.util.LinkedHashMap<String, org.emftext.language.pl0.resource.pl0.IPl0TokenResolver>();
		featureName2CollectInTokenResolver = new java.util.LinkedHashMap<String, org.emftext.language.pl0.resource.pl0.IPl0TokenResolver>();
		registerTokenResolver("PL0ID", new org.emftext.language.pl0.resource.pl0.analysis.Pl0PL0IDTokenResolver());
		registerTokenResolver("NUMBER", new org.emftext.language.pl0.resource.pl0.analysis.Pl0NUMBERTokenResolver());
		registerTokenResolver("REL_OP", new org.emftext.language.pl0.resource.pl0.analysis.Pl0REL_OPTokenResolver());
		registerTokenResolver("PLUS_MINUS", new org.emftext.language.pl0.resource.pl0.analysis.Pl0PLUS_MINUSTokenResolver());
		registerTokenResolver("MUL_DIV", new org.emftext.language.pl0.resource.pl0.analysis.Pl0MUL_DIVTokenResolver());
	}
	
	public org.emftext.language.pl0.resource.pl0.IPl0TokenResolver createTokenResolver(String tokenName) {
		return internalCreateResolver(tokenName2TokenResolver, tokenName);
	}
	
	public org.emftext.language.pl0.resource.pl0.IPl0TokenResolver createCollectInTokenResolver(String featureName) {
		return internalCreateResolver(featureName2CollectInTokenResolver, featureName);
	}
	
	protected boolean registerTokenResolver(String tokenName, org.emftext.language.pl0.resource.pl0.IPl0TokenResolver resolver){
		return internalRegisterTokenResolver(tokenName2TokenResolver, tokenName, resolver);
	}
	
	protected boolean registerCollectInTokenResolver(String featureName, org.emftext.language.pl0.resource.pl0.IPl0TokenResolver resolver){
		return internalRegisterTokenResolver(featureName2CollectInTokenResolver, featureName, resolver);
	}
	
	protected org.emftext.language.pl0.resource.pl0.IPl0TokenResolver deRegisterTokenResolver(String tokenName){
		return tokenName2TokenResolver.remove(tokenName);
	}
	
	private org.emftext.language.pl0.resource.pl0.IPl0TokenResolver internalCreateResolver(java.util.Map<String, org.emftext.language.pl0.resource.pl0.IPl0TokenResolver> resolverMap, String key) {
		if (resolverMap.containsKey(key)){
			return resolverMap.get(key);
		} else {
			return defaultResolver;
		}
	}
	
	private boolean internalRegisterTokenResolver(java.util.Map<String, org.emftext.language.pl0.resource.pl0.IPl0TokenResolver> resolverMap, String key, org.emftext.language.pl0.resource.pl0.IPl0TokenResolver resolver) {
		if (!resolverMap.containsKey(key)) {
			resolverMap.put(key,resolver);
			return true;
		}
		return false;
	}
	
}
