/**
 * <copyright>
 * </copyright>
 *
 * 
 */
package org.emftext.language.pl0.resource.pl0.util;

/**
 * This class provides basic infrastructure to interpret models. To implement
 * concrete interpreters, subclass this abstract interpreter and override the
 * interprete_* methods. The interpretation can be customized by binding the two
 * type parameters (ResultType, ContextType). The former is returned by all
 * interprete_* methods, while the latter is passed from method to method while
 * traversing the model. The concrete traversal strategy can also be exchanged.
 * One can use a static traversal strategy by pushing all objects to interpret on
 * the interpretation stack (using addObjectToInterprete()) before calling
 * interprete(). Alternatively, the traversal strategy can be dynamic by pushing
 * objects on the interpretation stack during interpretation.
 */
public class AbstractPl0Interpreter<ResultType, ContextType> {
	
	private java.util.Stack<org.eclipse.emf.ecore.EObject> interpretationStack = new java.util.Stack<org.eclipse.emf.ecore.EObject>();
	
	public ResultType interprete(ContextType context) {
		ResultType result = null;
		while (!interpretationStack.empty()) {
			org.eclipse.emf.ecore.EObject next = interpretationStack.pop();
			result = interprete(next, context);
			if (!continueInterpretation(result)) {
				break;
			}
		}
		return result;
	}
	
	/**
	 * Override this method to stop the overall interpretation depending on the result
	 * of the interpretation of a single model elements.
	 */
	public boolean continueInterpretation(ResultType result) {
		return true;
	}
	
	public ResultType interprete(org.eclipse.emf.ecore.EObject object, ContextType context) {
		ResultType result = null;
		if (object instanceof org.emftext.language.pl0.Assignment) {
			result = interprete_org_emftext_language_pl0_Assignment((org.emftext.language.pl0.Assignment) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.pl0.WhileStatement) {
			result = interprete_org_emftext_language_pl0_WhileStatement((org.emftext.language.pl0.WhileStatement) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.pl0.Condition) {
			result = interprete_org_emftext_language_pl0_Condition((org.emftext.language.pl0.Condition) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.pl0.IfStatement) {
			result = interprete_org_emftext_language_pl0_IfStatement((org.emftext.language.pl0.IfStatement) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.pl0.CallStatement) {
			result = interprete_org_emftext_language_pl0_CallStatement((org.emftext.language.pl0.CallStatement) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.pl0.Statement) {
			result = interprete_org_emftext_language_pl0_Statement((org.emftext.language.pl0.Statement) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.pl0.ProcedureDeclaration) {
			result = interprete_org_emftext_language_pl0_ProcedureDeclaration((org.emftext.language.pl0.ProcedureDeclaration) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.pl0.ConstDeclaration) {
			result = interprete_org_emftext_language_pl0_ConstDeclaration((org.emftext.language.pl0.ConstDeclaration) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.pl0.VarDeclaration) {
			result = interprete_org_emftext_language_pl0_VarDeclaration((org.emftext.language.pl0.VarDeclaration) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.pl0.Declaration) {
			result = interprete_org_emftext_language_pl0_Declaration((org.emftext.language.pl0.Declaration) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.pl0.Block) {
			result = interprete_org_emftext_language_pl0_Block((org.emftext.language.pl0.Block) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.pl0.Program) {
			result = interprete_org_emftext_language_pl0_Program((org.emftext.language.pl0.Program) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.pl0.BlockOwner) {
			result = interprete_org_emftext_language_pl0_BlockOwner((org.emftext.language.pl0.BlockOwner) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.pl0.Body) {
			result = interprete_org_emftext_language_pl0_Body((org.emftext.language.pl0.Body) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.pl0.ExpressionFactor) {
			result = interprete_org_emftext_language_pl0_ExpressionFactor((org.emftext.language.pl0.ExpressionFactor) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.pl0.Number) {
			result = interprete_org_emftext_language_pl0_Number((org.emftext.language.pl0.Number) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.pl0.IdentReference) {
			result = interprete_org_emftext_language_pl0_IdentReference((org.emftext.language.pl0.IdentReference) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.pl0.OptionalFactor) {
			result = interprete_org_emftext_language_pl0_OptionalFactor((org.emftext.language.pl0.OptionalFactor) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.pl0.Factor) {
			result = interprete_org_emftext_language_pl0_Factor((org.emftext.language.pl0.Factor) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.pl0.OptionalTerm) {
			result = interprete_org_emftext_language_pl0_OptionalTerm((org.emftext.language.pl0.OptionalTerm) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.pl0.Term) {
			result = interprete_org_emftext_language_pl0_Term((org.emftext.language.pl0.Term) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.pl0.TermExpression) {
			result = interprete_org_emftext_language_pl0_TermExpression((org.emftext.language.pl0.TermExpression) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.pl0.RelationalCondition) {
			result = interprete_org_emftext_language_pl0_RelationalCondition((org.emftext.language.pl0.RelationalCondition) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.pl0.OddCondition) {
			result = interprete_org_emftext_language_pl0_OddCondition((org.emftext.language.pl0.OddCondition) object, context);
		}
		if (result != null) {
			return result;
		}
		if (object instanceof org.emftext.language.pl0.Expression) {
			result = interprete_org_emftext_language_pl0_Expression((org.emftext.language.pl0.Expression) object, context);
		}
		if (result != null) {
			return result;
		}
		return result;
	}
	
	public ResultType interprete_org_emftext_language_pl0_Program(org.emftext.language.pl0.Program object, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_pl0_Block(org.emftext.language.pl0.Block object, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_pl0_VarDeclaration(org.emftext.language.pl0.VarDeclaration object, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_pl0_ConstDeclaration(org.emftext.language.pl0.ConstDeclaration object, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_pl0_ProcedureDeclaration(org.emftext.language.pl0.ProcedureDeclaration object, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_pl0_Declaration(org.emftext.language.pl0.Declaration object, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_pl0_Statement(org.emftext.language.pl0.Statement object, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_pl0_CallStatement(org.emftext.language.pl0.CallStatement object, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_pl0_IfStatement(org.emftext.language.pl0.IfStatement object, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_pl0_Condition(org.emftext.language.pl0.Condition object, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_pl0_WhileStatement(org.emftext.language.pl0.WhileStatement object, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_pl0_Assignment(org.emftext.language.pl0.Assignment object, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_pl0_Expression(org.emftext.language.pl0.Expression object, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_pl0_OddCondition(org.emftext.language.pl0.OddCondition object, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_pl0_RelationalCondition(org.emftext.language.pl0.RelationalCondition object, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_pl0_TermExpression(org.emftext.language.pl0.TermExpression object, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_pl0_Term(org.emftext.language.pl0.Term object, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_pl0_OptionalTerm(org.emftext.language.pl0.OptionalTerm object, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_pl0_Factor(org.emftext.language.pl0.Factor object, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_pl0_OptionalFactor(org.emftext.language.pl0.OptionalFactor object, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_pl0_IdentReference(org.emftext.language.pl0.IdentReference object, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_pl0_Number(org.emftext.language.pl0.Number object, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_pl0_ExpressionFactor(org.emftext.language.pl0.ExpressionFactor object, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_pl0_Body(org.emftext.language.pl0.Body object, ContextType context) {
		return null;
	}
	
	public ResultType interprete_org_emftext_language_pl0_BlockOwner(org.emftext.language.pl0.BlockOwner object, ContextType context) {
		return null;
	}
	
	public void addObjectToInterprete(org.eclipse.emf.ecore.EObject object) {
		interpretationStack.push(object);
	}
	
}
